/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Parameters supplied to update Identity Provider
 *
 */
class IdentityProviderUpdateParameters {
  /**
   * Create a IdentityProviderUpdateParameters.
   * @member {string} [type] Identity Provider Type identifier. Possible values
   * include: 'facebook', 'google', 'microsoft', 'twitter', 'aad', 'aadB2C'
   * @member {array} [allowedTenants] List of Allowed Tenants when configuring
   * Azure Active Directory login.
   * @member {string} [signupPolicyName] Signup Policy Name. Only applies to
   * AAD B2C Identity Provider.
   * @member {string} [signinPolicyName] Signin Policy Name. Only applies to
   * AAD B2C Identity Provider.
   * @member {string} [profileEditingPolicyName] Profile Editing Policy Name.
   * Only applies to AAD B2C Identity Provider.
   * @member {string} [passwordResetPolicyName] Password Reset Policy Name.
   * Only applies to AAD B2C Identity Provider.
   * @member {string} [clientId] Client Id of the Application in the external
   * Identity Provider. It is App ID for Facebook login, Client ID for Google
   * login, App ID for Microsoft.
   * @member {string} [clientSecret] Client secret of the Application in
   * external Identity Provider, used to authenticate login request. For
   * example, it is App Secret for Facebook login, API Key for Google login,
   * Public Key for Microsoft.
   */
  constructor() {
  }

  /**
   * Defines the metadata of IdentityProviderUpdateParameters
   *
   * @returns {object} metadata of IdentityProviderUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'IdentityProviderUpdateParameters',
      type: {
        name: 'Composite',
        className: 'IdentityProviderUpdateParameters',
        modelProperties: {
          type: {
            required: false,
            serializedName: 'properties.type',
            type: {
              name: 'Enum',
              allowedValues: [ 'facebook', 'google', 'microsoft', 'twitter', 'aad', 'aadB2C' ]
            }
          },
          allowedTenants: {
            required: false,
            serializedName: 'properties.allowedTenants',
            constraints: {
              MaxItems: 32
            },
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          signupPolicyName: {
            required: false,
            serializedName: 'properties.signupPolicyName',
            constraints: {
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          signinPolicyName: {
            required: false,
            serializedName: 'properties.signinPolicyName',
            constraints: {
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          profileEditingPolicyName: {
            required: false,
            serializedName: 'properties.profileEditingPolicyName',
            constraints: {
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          passwordResetPolicyName: {
            required: false,
            serializedName: 'properties.passwordResetPolicyName',
            constraints: {
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          clientId: {
            required: false,
            serializedName: 'properties.clientId',
            constraints: {
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          clientSecret: {
            required: false,
            serializedName: 'properties.clientSecret',
            constraints: {
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = IdentityProviderUpdateParameters;
