/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * @summary A pool in the Azure Batch service.
  *
 */
class CloudPool {
  /**
   * Create a CloudPool.
   * @member {string} [id] A string that uniquely identifies the pool within
   * the account. The ID can contain any combination of alphanumeric characters
   * including hyphens and underscores, and cannot contain more than 64
   * characters. The ID is case-preserving and case-insensitive (that is, you
   * may not have two IDs within an account that differ only by case).
   * @member {string} [displayName] The display name for the pool. The display
   * name need not be unique and can contain any Unicode characters up to a
   * maximum length of 1024.
   * @member {string} [url] The URL of the pool.
   * @member {string} [eTag] The ETag of the pool. This is an opaque string.
   * You can use it to detect whether the pool has changed between requests. In
   * particular, you can be pass the ETag when updating a pool to specify that
   * your changes should take effect only if nobody else has modified the pool
   * in the meantime.
   * @member {date} [lastModified] The last modified time of the pool. This is
   * the last time at which the pool level data, such as the
   * targetDedicatedNodes or enableAutoscale settings, changed. It does not
   * factor in node-level changes such as a compute node changing state.
   * @member {date} [creationTime] The creation time of the pool.
   * @member {string} [state] The current state of the pool. Values are:
   *
   * active - The pool is available to run tasks subject to the availability of
   * compute nodes.
   * deleting - The user has requested that the pool be deleted, but the delete
   * operation has not yet completed.
   * upgrading - The user has requested that the operating system of the pool's
   * nodes be upgraded, but the upgrade operation has not yet completed (that
   * is, some nodes in the pool have not yet been upgraded). While upgrading,
   * the pool may be able to run tasks (with reduced capacity) but this is not
   * guaranteed. Possible values include: 'active', 'deleting', 'upgrading'
   * @member {date} [stateTransitionTime] The time at which the pool entered
   * its current state.
   * @member {string} [allocationState] Whether the pool is resizing. Values
   * are:
   *
   * steady - The pool is not resizing. There are no changes to the number of
   * nodes in the pool in progress. A pool enters this state when it is created
   * and when no operations are being performed on the pool to change the
   * number of dedicated nodes.
   * resizing - The pool is resizing; that is, compute nodes are being added to
   * or removed from the pool.
   * stopping - The pool was resizing, but the user has requested that the
   * resize be stopped, but the stop request has not yet been completed.
   * Possible values include: 'steady', 'resizing', 'stopping'
   * @member {date} [allocationStateTransitionTime] The time at which the pool
   * entered its current allocation state.
   * @member {string} [vmSize] The size of virtual machines in the pool. All
   * virtual machines in a pool are the same size. For information about
   * available sizes of virtual machines for Cloud Services pools (pools
   * created with cloudServiceConfiguration), see Sizes for Cloud Services
   * (http://azure.microsoft.com/documentation/articles/cloud-services-sizes-specs/).
   * Batch supports all Cloud Services VM sizes except ExtraSmall, A1V2 and
   * A2V2. For information about available VM sizes for pools using images from
   * the Virtual Machines Marketplace (pools created with
   * virtualMachineConfiguration) see Sizes for Virtual Machines (Linux)
   * (https://azure.microsoft.com/documentation/articles/virtual-machines-linux-sizes/)
   * or Sizes for Virtual Machines (Windows)
   * (https://azure.microsoft.com/documentation/articles/virtual-machines-windows-sizes/).
   * Batch supports all Azure VM sizes except STANDARD_A0 and those with
   * premium storage (STANDARD_GS, STANDARD_DS, and STANDARD_DSV2 series).
   * @member {object} [cloudServiceConfiguration] The cloud service
   * configuration for the pool. This property and virtualMachineConfiguration
   * are mutually exclusive and one of the properties must be specified. This
   * property cannot be specified if the Batch account was created with its
   * poolAllocationMode property set to 'UserSubscription'.
   * @member {string} [cloudServiceConfiguration.osFamily] Possible values are:
   * 2 - OS Family 2, equivalent to Windows Server 2008 R2 SP1. 3 - OS Family
   * 3, equivalent to Windows Server 2012. 4 - OS Family 4, equivalent to
   * Windows Server 2012 R2. 5 - OS Family 5, equivalent to Windows Server
   * 2016. For more information, see Azure Guest OS Releases
   * (https://azure.microsoft.com/documentation/articles/cloud-services-guestos-update-matrix/#releases).
   * @member {string} [cloudServiceConfiguration.targetOSVersion] The default
   * value is * which specifies the latest operating system version for the
   * specified OS family.
   * @member {string} [cloudServiceConfiguration.currentOSVersion] This may
   * differ from targetOSVersion if the pool state is Upgrading. In this case
   * some virtual machines may be on the targetOSVersion and some may be on the
   * currentOSVersion during the upgrade process. Once all virtual machines
   * have upgraded, currentOSVersion is updated to be the same as
   * targetOSVersion.
   * @member {object} [virtualMachineConfiguration] The virtual machine
   * configuration for the pool. This property and cloudServiceConfiguration
   * are mutually exclusive and one of the properties must be specified.
   * @member {object} [virtualMachineConfiguration.imageReference]
   * @member {string} [virtualMachineConfiguration.imageReference.publisher]
   * For example, Canonical or MicrosoftWindowsServer.
   * @member {string} [virtualMachineConfiguration.imageReference.offer] For
   * example, UbuntuServer or WindowsServer.
   * @member {string} [virtualMachineConfiguration.imageReference.sku] For
   * example, 14.04.0-LTS or 2012-R2-Datacenter.
   * @member {string} [virtualMachineConfiguration.imageReference.version] A
   * value of 'latest' can be specified to select the latest version of an
   * image. If omitted, the default is 'latest'.
   * @member {string}
   * [virtualMachineConfiguration.imageReference.virtualMachineImageId] This
   * property is mutually exclusive with other ImageReference properties. The
   * virtual machine image must be in the same region and subscription as the
   * Azure Batch account. For information about the firewall settings for the
   * Batch node agent to communicate with the Batch service see
   * https://docs.microsoft.com/en-us/azure/batch/batch-api-basics#virtual-network-vnet-and-firewall-configuration.
   * @member {object} [virtualMachineConfiguration.osDisk]
   * @member {string} [virtualMachineConfiguration.osDisk.caching] Values are:
   *
   * none - The caching mode for the disk is not enabled.
   * readOnly - The caching mode for the disk is read only.
   * readWrite - The caching mode for the disk is read and write.
   *
   * The default value for caching is none. For information about the caching
   * options see:
   * https://blogs.msdn.microsoft.com/windowsazurestorage/2012/06/27/exploring-windows-azure-drives-disks-and-images/.
   * Possible values include: 'none', 'readOnly', 'readWrite'
   * @member {string} [virtualMachineConfiguration.nodeAgentSKUId] The Batch
   * node agent is a program that runs on each node in the pool, and provides
   * the command-and-control interface between the node and the Batch service.
   * There are different implementations of the node agent, known as SKUs, for
   * different operating systems. You must specify a node agent SKU which
   * matches the selected image reference. To get the list of supported node
   * agent SKUs along with their list of verified image references, see the
   * 'List supported node agent SKUs' operation.
   * @member {object} [virtualMachineConfiguration.windowsConfiguration] This
   * property must not be specified if the imageReference or osDisk property
   * specifies a Linux OS image.
   * @member {boolean}
   * [virtualMachineConfiguration.windowsConfiguration.enableAutomaticUpdates]
   * If omitted, the default value is true.
   * @member {array} [virtualMachineConfiguration.dataDisks] This property must
   * be specified if the compute nodes in the pool need to have empty data
   * disks attached to them. This cannot be updated.
   * @member {string} [virtualMachineConfiguration.licenseType] This only
   * applies to images that contain the Windows operating system, and should
   * only be used when you hold valid on-premises licenses for the nodes which
   * will be deployed. If omitted, no on-premises licensing discount is
   * applied. Values are:
   *
   * Windows_Server - The on-premises license is for Windows Server.
   * Windows_Client - The on-premises license is for Windows Client.
   * @member {object} [virtualMachineConfiguration.containerConfiguration] If
   * specified, setup is performed on each node in the pool to allow tasks to
   * run in containers. All regular tasks and job manager tasks run on this
   * pool must specify the containerSettings property, and all other tasks may
   * specify it.
   * @member {array}
   * [virtualMachineConfiguration.containerConfiguration.containerImageNames]
   * This is the full image reference, as would be specified to "docker pull".
   * An image will be sourced from the default Docker registry unless the image
   * is fully qualified with an alternative registry.
   * @member {array}
   * [virtualMachineConfiguration.containerConfiguration.containerRegistries]
   * If any images must be downloaded from a private registry which requires
   * credentials, then those credentials must be provided here.
   * @member {moment.duration} [resizeTimeout] The timeout for allocation of
   * compute nodes to the pool. This is the timeout for the most recent resize
   * operation. (The initial sizing when the pool is created counts as a
   * resize.) The default value is 15 minutes.
   * @member {array} [resizeErrors] A list of errors encountered while
   * performing the last resize on the pool. This property is set only if one
   * or more errors occurred during the last pool resize, and only when the
   * pool allocationState is Steady.
   * @member {number} [currentDedicatedNodes] The number of dedicated compute
   * nodes currently in the pool.
   * @member {number} [currentLowPriorityNodes] The number of low-priority
   * compute nodes currently in the pool. Low-priority compute nodes which have
   * been preempted are included in this count.
   * @member {number} [targetDedicatedNodes] The desired number of dedicated
   * compute nodes in the pool.
   * @member {number} [targetLowPriorityNodes] The desired number of
   * low-priority compute nodes in the pool.
   * @member {boolean} [enableAutoScale] Whether the pool size should
   * automatically adjust over time. If false, at least one of
   * targetDedicateNodes and targetLowPriorityNodes must be specified. If true,
   * the autoScaleFormula property is required and the pool automatically
   * resizes according to the formula. The default value is false.
   * @member {string} [autoScaleFormula] A formula for the desired number of
   * compute nodes in the pool. This property is set only if the pool
   * automatically scales, i.e. enableAutoScale is true.
   * @member {moment.duration} [autoScaleEvaluationInterval] The time interval
   * at which to automatically adjust the pool size according to the autoscale
   * formula. This property is set only if the pool automatically scales, i.e.
   * enableAutoScale is true.
   * @member {object} [autoScaleRun] The results and errors from the last
   * execution of the autoscale formula. This property is set only if the pool
   * automatically scales, i.e. enableAutoScale is true.
   * @member {date} [autoScaleRun.timestamp]
   * @member {string} [autoScaleRun.results] Each variable value is returned in
   * the form $variable=value, and variables are separated by semicolons.
   * @member {object} [autoScaleRun.error]
   * @member {string} [autoScaleRun.error.code]
   * @member {string} [autoScaleRun.error.message]
   * @member {array} [autoScaleRun.error.values]
   * @member {boolean} [enableInterNodeCommunication] Whether the pool permits
   * direct communication between nodes. This imposes restrictions on which
   * nodes can be assigned to the pool. Specifying this value can reduce the
   * chance of the requested number of nodes to be allocated in the pool.
   * @member {object} [networkConfiguration] The network configuration for the
   * pool.
   * @member {string} [networkConfiguration.subnetId] The virtual network must
   * be in the same region and subscription as the Azure Batch account. The
   * specified subnet should have enough free IP addresses to accommodate the
   * number of nodes in the pool. If the subnet doesn't have enough free IP
   * addresses, the pool will partially allocate compute nodes, and a resize
   * error will occur. The 'MicrosoftAzureBatch' service principal must have
   * the 'Classic Virtual Machine Contributor' Role-Based Access Control (RBAC)
   * role for the specified VNet. The specified subnet must allow communication
   * from the Azure Batch service to be able to schedule tasks on the compute
   * nodes. This can be verified by checking if the specified VNet has any
   * associated Network Security Groups (NSG). If communication to the compute
   * nodes in the specified subnet is denied by an NSG, then the Batch service
   * will set the state of the compute nodes to unusable. For pools created
   * with virtualMachineConfiguration only ARM virtual networks
   * ('Microsoft.Network/virtualNetworks') are supported, but for pools created
   * with cloudServiceConfiguration both ARM and classic virtual networks are
   * supported. If the specified VNet has any associated Network Security
   * Groups (NSG), then a few reserved system ports must be enabled for inbound
   * communication. For pools created with a virtual machine configuration,
   * enable ports 29876 and 29877, as well as port 22 for Linux and port 3389
   * for Windows. For pools created with a cloud service configuration, enable
   * ports 10100, 20100, and 30100. Also enable outbound connections to Azure
   * Storage on port 443. For more details see:
   * https://docs.microsoft.com/en-us/azure/batch/batch-api-basics#virtual-network-vnet-and-firewall-configuration
   * @member {object} [networkConfiguration.endpointConfiguration] Pool
   * endpoint configuration is only supported on pools with the
   * virtualMachineConfiguration property.
   * @member {array}
   * [networkConfiguration.endpointConfiguration.inboundNATPools] The maximum
   * number of inbound NAT pools per Batch pool is 5. If the maximum number of
   * inbound NAT pools is exceeded the request fails with HTTP status code 400.
   * @member {object} [startTask] A task specified to run on each compute node
   * as it joins the pool.
   * @member {string} [startTask.commandLine] The command line does not run
   * under a shell, and therefore cannot take advantage of shell features such
   * as environment variable expansion. If you want to take advantage of such
   * features, you should invoke the shell in the command line, for example
   * using "cmd /c MyCommand" in Windows or "/bin/sh -c MyCommand" in Linux.
   * @member {object} [startTask.containerSettings] When this is specified, all
   * directories recursively below the AZ_BATCH_NODE_ROOT_DIR (the root of
   * Azure Batch directories on the node) are mapped into the container, all
   * task environment variables are mapped into the container, and the task
   * command line is executed in the container.
   * @member {string} [startTask.containerSettings.containerRunOptions] These
   * additional options are supplied as arguments to the "docker create"
   * command, in addition to those controlled by the Batch Service.
   * @member {string} [startTask.containerSettings.imageName] This is the full
   * image reference, as would be specified to "docker pull". If no tag is
   * provided as part of the image name, the tag ":latest" is used as a
   * default.
   * @member {object} [startTask.containerSettings.registry] This setting can
   * be omitted if was already provided at pool creation.
   * @member {string} [startTask.containerSettings.registry.registryServer] If
   * omitted, the default is "docker.io".
   * @member {string} [startTask.containerSettings.registry.userName]
   * @member {string} [startTask.containerSettings.registry.password]
   * @member {array} [startTask.resourceFiles] Files listed under this element
   * are located in the task's working directory.
   * @member {array} [startTask.environmentSettings]
   * @member {object} [startTask.userIdentity] If omitted, the task runs as a
   * non-administrative user unique to the task.
   * @member {string} [startTask.userIdentity.userName] The userName and
   * autoUser properties are mutually exclusive; you must specify one but not
   * both.
   * @member {object} [startTask.userIdentity.autoUser] The userName and
   * autoUser properties are mutually exclusive; you must specify one but not
   * both.
   * @member {string} [startTask.userIdentity.autoUser.scope] Values are:
   *
   * pool - specifies that the task runs as the common auto user account which
   * is created on every node in a pool.
   * task - specifies that the service should create a new user for the task.
   * The default value is task. Possible values include: 'task', 'pool'
   * @member {string} [startTask.userIdentity.autoUser.elevationLevel] nonAdmin
   * - The auto user is a standard user without elevated access. admin - The
   * auto user is a user with elevated access and operates with full
   * Administrator permissions. The default value is nonAdmin. Possible values
   * include: 'nonAdmin', 'admin'
   * @member {number} [startTask.maxTaskRetryCount] The Batch service retries a
   * task if its exit code is nonzero. Note that this value specifically
   * controls the number of retries. The Batch service will try the task once,
   * and may then retry up to this limit. For example, if the maximum retry
   * count is 3, Batch tries the task up to 4 times (one initial try and 3
   * retries). If the maximum retry count is 0, the Batch service does not
   * retry the task. If the maximum retry count is -1, the Batch service
   * retries the task without limit.
   * @member {boolean} [startTask.waitForSuccess] If true and the start task
   * fails on a compute node, the Batch service retries the start task up to
   * its maximum retry count (maxTaskRetryCount). If the task has still not
   * completed successfully after all retries, then the Batch service marks the
   * compute node unusable, and will not schedule tasks to it. This condition
   * can be detected via the node state and failure info details. If false, the
   * Batch service will not wait for the start task to complete. In this case,
   * other tasks can start executing on the compute node while the start task
   * is still running; and even if the start task fails, new tasks will
   * continue to be scheduled on the node. The default is false.
   * @member {array} [certificateReferences] The list of certificates to be
   * installed on each compute node in the pool. For Windows compute nodes, the
   * Batch service installs the certificates to the specified certificate store
   * and location. For Linux compute nodes, the certificates are stored in a
   * directory inside the task working directory and an environment variable
   * AZ_BATCH_CERTIFICATES_DIR is supplied to the task to query for this
   * location. For certificates with visibility of 'remoteUser', a 'certs'
   * directory is created in the user's home directory (e.g.,
   * /home/{user-name}/certs) and certificates are placed in that directory.
   * @member {array} [applicationPackageReferences] The list of application
   * packages to be installed on each compute node in the pool.
   * @member {array} [applicationLicenses] The list of application licenses the
   * Batch service will make available on each compute node in the pool. The
   * list of application licenses must be a subset of available Batch service
   * application licenses. If a license is requested which is not supported,
   * pool creation will fail.
   * @member {number} [maxTasksPerNode] The maximum number of tasks that can
   * run concurrently on a single compute node in the pool.
   * @member {object} [taskSchedulingPolicy] How tasks are distributed across
   * compute nodes in a pool.
   * @member {string} [taskSchedulingPolicy.nodeFillType] Possible values
   * include: 'spread', 'pack'
   * @member {array} [userAccounts] The list of user accounts to be created on
   * each node in the pool.
   * @member {array} [metadata] A list of name-value pairs associated with the
   * pool as metadata.
   * @member {object} [stats] Utilization and resource usage statistics for the
   * entire lifetime of the pool.
   * @member {string} [stats.url]
   * @member {date} [stats.startTime]
   * @member {date} [stats.lastUpdateTime]
   * @member {object} [stats.usageStats]
   * @member {date} [stats.usageStats.startTime]
   * @member {date} [stats.usageStats.lastUpdateTime]
   * @member {moment.duration} [stats.usageStats.dedicatedCoreTime]
   * @member {object} [stats.resourceStats]
   * @member {date} [stats.resourceStats.startTime]
   * @member {date} [stats.resourceStats.lastUpdateTime]
   * @member {number} [stats.resourceStats.avgCPUPercentage]
   * @member {number} [stats.resourceStats.avgMemoryGiB]
   * @member {number} [stats.resourceStats.peakMemoryGiB]
   * @member {number} [stats.resourceStats.avgDiskGiB]
   * @member {number} [stats.resourceStats.peakDiskGiB]
   * @member {number} [stats.resourceStats.diskReadIOps]
   * @member {number} [stats.resourceStats.diskWriteIOps]
   * @member {number} [stats.resourceStats.diskReadGiB]
   * @member {number} [stats.resourceStats.diskWriteGiB]
   * @member {number} [stats.resourceStats.networkReadGiB]
   * @member {number} [stats.resourceStats.networkWriteGiB]
   */
  constructor() {
  }

  /**
   * Defines the metadata of CloudPool
   *
   * @returns {object} metadata of CloudPool
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CloudPool',
      type: {
        name: 'Composite',
        className: 'CloudPool',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          displayName: {
            required: false,
            serializedName: 'displayName',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          eTag: {
            required: false,
            serializedName: 'eTag',
            type: {
              name: 'String'
            }
          },
          lastModified: {
            required: false,
            serializedName: 'lastModified',
            type: {
              name: 'DateTime'
            }
          },
          creationTime: {
            required: false,
            serializedName: 'creationTime',
            type: {
              name: 'DateTime'
            }
          },
          state: {
            required: false,
            serializedName: 'state',
            type: {
              name: 'Enum',
              allowedValues: [ 'active', 'deleting', 'upgrading' ]
            }
          },
          stateTransitionTime: {
            required: false,
            serializedName: 'stateTransitionTime',
            type: {
              name: 'DateTime'
            }
          },
          allocationState: {
            required: false,
            serializedName: 'allocationState',
            type: {
              name: 'Enum',
              allowedValues: [ 'steady', 'resizing', 'stopping' ]
            }
          },
          allocationStateTransitionTime: {
            required: false,
            serializedName: 'allocationStateTransitionTime',
            type: {
              name: 'DateTime'
            }
          },
          vmSize: {
            required: false,
            serializedName: 'vmSize',
            type: {
              name: 'String'
            }
          },
          cloudServiceConfiguration: {
            required: false,
            serializedName: 'cloudServiceConfiguration',
            type: {
              name: 'Composite',
              className: 'CloudServiceConfiguration'
            }
          },
          virtualMachineConfiguration: {
            required: false,
            serializedName: 'virtualMachineConfiguration',
            type: {
              name: 'Composite',
              className: 'VirtualMachineConfiguration'
            }
          },
          resizeTimeout: {
            required: false,
            serializedName: 'resizeTimeout',
            type: {
              name: 'TimeSpan'
            }
          },
          resizeErrors: {
            required: false,
            serializedName: 'resizeErrors',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ResizeErrorElementType',
                  type: {
                    name: 'Composite',
                    className: 'ResizeError'
                  }
              }
            }
          },
          currentDedicatedNodes: {
            required: false,
            serializedName: 'currentDedicatedNodes',
            type: {
              name: 'Number'
            }
          },
          currentLowPriorityNodes: {
            required: false,
            serializedName: 'currentLowPriorityNodes',
            type: {
              name: 'Number'
            }
          },
          targetDedicatedNodes: {
            required: false,
            serializedName: 'targetDedicatedNodes',
            type: {
              name: 'Number'
            }
          },
          targetLowPriorityNodes: {
            required: false,
            serializedName: 'targetLowPriorityNodes',
            type: {
              name: 'Number'
            }
          },
          enableAutoScale: {
            required: false,
            serializedName: 'enableAutoScale',
            type: {
              name: 'Boolean'
            }
          },
          autoScaleFormula: {
            required: false,
            serializedName: 'autoScaleFormula',
            type: {
              name: 'String'
            }
          },
          autoScaleEvaluationInterval: {
            required: false,
            serializedName: 'autoScaleEvaluationInterval',
            type: {
              name: 'TimeSpan'
            }
          },
          autoScaleRun: {
            required: false,
            serializedName: 'autoScaleRun',
            type: {
              name: 'Composite',
              className: 'AutoScaleRun'
            }
          },
          enableInterNodeCommunication: {
            required: false,
            serializedName: 'enableInterNodeCommunication',
            type: {
              name: 'Boolean'
            }
          },
          networkConfiguration: {
            required: false,
            serializedName: 'networkConfiguration',
            type: {
              name: 'Composite',
              className: 'NetworkConfiguration'
            }
          },
          startTask: {
            required: false,
            serializedName: 'startTask',
            type: {
              name: 'Composite',
              className: 'StartTask'
            }
          },
          certificateReferences: {
            required: false,
            serializedName: 'certificateReferences',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'CertificateReferenceElementType',
                  type: {
                    name: 'Composite',
                    className: 'CertificateReference'
                  }
              }
            }
          },
          applicationPackageReferences: {
            required: false,
            serializedName: 'applicationPackageReferences',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationPackageReferenceElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationPackageReference'
                  }
              }
            }
          },
          applicationLicenses: {
            required: false,
            serializedName: 'applicationLicenses',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          maxTasksPerNode: {
            required: false,
            serializedName: 'maxTasksPerNode',
            type: {
              name: 'Number'
            }
          },
          taskSchedulingPolicy: {
            required: false,
            serializedName: 'taskSchedulingPolicy',
            type: {
              name: 'Composite',
              className: 'TaskSchedulingPolicy'
            }
          },
          userAccounts: {
            required: false,
            serializedName: 'userAccounts',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'UserAccountElementType',
                  type: {
                    name: 'Composite',
                    className: 'UserAccount'
                  }
              }
            }
          },
          metadata: {
            required: false,
            serializedName: 'metadata',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'MetadataItemElementType',
                  type: {
                    name: 'Composite',
                    className: 'MetadataItem'
                  }
              }
            }
          },
          stats: {
            required: false,
            serializedName: 'stats',
            type: {
              name: 'Composite',
              className: 'PoolStatistics'
            }
          }
        }
      }
    };
  }
}

module.exports = CloudPool;
