/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * @summary How the Batch service should respond if a task exits with a
 * particular exit code.
  *
 */
class ExitCodeMapping {
  /**
   * Create a ExitCodeMapping.
   * @member {number} code A process exit code.
   * @member {object} exitOptions How the Batch service should respond if the
   * task exits with this exit code.
   * @member {string} [exitOptions.jobAction] Values are:
   *
   * none - Take no action.
   * disable - Disable the job. This is equivalent to calling the disable job
   * API, with a disableTasks value of requeue.
   * terminate - Terminate the job. The terminateReason in the job's
   * executionInfo is set to "TaskFailed". The default is none for exit code 0
   * and terminate for all other exit conditions.
   *
   * If the job's onTaskFailed property is noAction, then specifying this
   * property returns an error and the add task request fails with an invalid
   * property value error; if you are calling the REST API directly, the HTTP
   * status code is 400 (Bad Request). Possible values include: 'none',
   * 'disable', 'terminate'
   * @member {string} [exitOptions.dependencyAction] Values are:
   *
   * satisfy - Satisfy the task's dependencies.
   * block - Block the task's dependencies.
   *
   * The default is 'satisfy' for exit code 0, and 'block' for all other exit
   * conditions. If the job's usesTaskDependencies property is set to false,
   * then specifying the dependencyAction property returns an erro and the add
   * task request fails with an invalid property value error; if you are
   * calling the REST API directly, the HTTP status code is 400  (Bad Request).
   * Possible values include: 'satisfy', 'block'
   */
  constructor() {
  }

  /**
   * Defines the metadata of ExitCodeMapping
   *
   * @returns {object} metadata of ExitCodeMapping
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ExitCodeMapping',
      type: {
        name: 'Composite',
        className: 'ExitCodeMapping',
        modelProperties: {
          code: {
            required: true,
            serializedName: 'code',
            type: {
              name: 'Number'
            }
          },
          exitOptions: {
            required: true,
            serializedName: 'exitOptions',
            type: {
              name: 'Composite',
              className: 'ExitOptions'
            }
          }
        }
      }
    };
  }
}

module.exports = ExitCodeMapping;
