/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * @summary Specifies how the Batch service should respond when the task
 * completes.
  *
 */
class ExitConditions {
  /**
   * Create a ExitConditions.
   * @member {array} [exitCodes] A list of individual task exit codes and how
   * the Batch service should respond to them.
   * @member {array} [exitCodeRanges] A list of task exit code ranges and how
   * the Batch service should respond to them.
   * @member {object} [preProcessingError] How the Batch service should respond
   * if the task fails to start due to an error.
   * @member {string} [preProcessingError.jobAction] Values are:
   *
   * none - Take no action.
   * disable - Disable the job. This is equivalent to calling the disable job
   * API, with a disableTasks value of requeue.
   * terminate - Terminate the job. The terminateReason in the job's
   * executionInfo is set to "TaskFailed". The default is none for exit code 0
   * and terminate for all other exit conditions.
   *
   * If the job's onTaskFailed property is noAction, then specifying this
   * property returns an error and the add task request fails with an invalid
   * property value error; if you are calling the REST API directly, the HTTP
   * status code is 400 (Bad Request). Possible values include: 'none',
   * 'disable', 'terminate'
   * @member {string} [preProcessingError.dependencyAction] Values are:
   *
   * satisfy - Satisfy the task's dependencies.
   * block - Block the task's dependencies.
   *
   * The default is 'satisfy' for exit code 0, and 'block' for all other exit
   * conditions. If the job's usesTaskDependencies property is set to false,
   * then specifying the dependencyAction property returns an erro and the add
   * task request fails with an invalid property value error; if you are
   * calling the REST API directly, the HTTP status code is 400  (Bad Request).
   * Possible values include: 'satisfy', 'block'
   * @member {object} [fileUploadError] How the Batch service should respond if
   * a file upload error occurs. If the task exited with an exit code that was
   * specified via exitCodes or exitCodeRanges, and then encountered a file
   * upload error, then the action specified by the exit code takes precedence.
   * @member {string} [fileUploadError.jobAction] Values are:
   *
   * none - Take no action.
   * disable - Disable the job. This is equivalent to calling the disable job
   * API, with a disableTasks value of requeue.
   * terminate - Terminate the job. The terminateReason in the job's
   * executionInfo is set to "TaskFailed". The default is none for exit code 0
   * and terminate for all other exit conditions.
   *
   * If the job's onTaskFailed property is noAction, then specifying this
   * property returns an error and the add task request fails with an invalid
   * property value error; if you are calling the REST API directly, the HTTP
   * status code is 400 (Bad Request). Possible values include: 'none',
   * 'disable', 'terminate'
   * @member {string} [fileUploadError.dependencyAction] Values are:
   *
   * satisfy - Satisfy the task's dependencies.
   * block - Block the task's dependencies.
   *
   * The default is 'satisfy' for exit code 0, and 'block' for all other exit
   * conditions. If the job's usesTaskDependencies property is set to false,
   * then specifying the dependencyAction property returns an erro and the add
   * task request fails with an invalid property value error; if you are
   * calling the REST API directly, the HTTP status code is 400  (Bad Request).
   * Possible values include: 'satisfy', 'block'
   * @member {object} [default] How the Batch service should respond if the
   * task fails with an exit condition not covered by any of the other
   * properties. This value is used if the task exits with any nonzero exit
   * code not listed in the exitCodes or exitCodeRanges collection, with a
   * pre-processing error if the preProcessingError property is not present, or
   * with a file upload error if the fileUploadError property is not present.
   * If you want non-default behaviour on exit code 0, you must list it
   * explicitly using the exitCodes or exitCodeRanges collection.
   * @member {string} [default.jobAction] Values are:
   *
   * none - Take no action.
   * disable - Disable the job. This is equivalent to calling the disable job
   * API, with a disableTasks value of requeue.
   * terminate - Terminate the job. The terminateReason in the job's
   * executionInfo is set to "TaskFailed". The default is none for exit code 0
   * and terminate for all other exit conditions.
   *
   * If the job's onTaskFailed property is noAction, then specifying this
   * property returns an error and the add task request fails with an invalid
   * property value error; if you are calling the REST API directly, the HTTP
   * status code is 400 (Bad Request). Possible values include: 'none',
   * 'disable', 'terminate'
   * @member {string} [default.dependencyAction] Values are:
   *
   * satisfy - Satisfy the task's dependencies.
   * block - Block the task's dependencies.
   *
   * The default is 'satisfy' for exit code 0, and 'block' for all other exit
   * conditions. If the job's usesTaskDependencies property is set to false,
   * then specifying the dependencyAction property returns an erro and the add
   * task request fails with an invalid property value error; if you are
   * calling the REST API directly, the HTTP status code is 400  (Bad Request).
   * Possible values include: 'satisfy', 'block'
   */
  constructor() {
  }

  /**
   * Defines the metadata of ExitConditions
   *
   * @returns {object} metadata of ExitConditions
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ExitConditions',
      type: {
        name: 'Composite',
        className: 'ExitConditions',
        modelProperties: {
          exitCodes: {
            required: false,
            serializedName: 'exitCodes',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ExitCodeMappingElementType',
                  type: {
                    name: 'Composite',
                    className: 'ExitCodeMapping'
                  }
              }
            }
          },
          exitCodeRanges: {
            required: false,
            serializedName: 'exitCodeRanges',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ExitCodeRangeMappingElementType',
                  type: {
                    name: 'Composite',
                    className: 'ExitCodeRangeMapping'
                  }
              }
            }
          },
          preProcessingError: {
            required: false,
            serializedName: 'preProcessingError',
            type: {
              name: 'Composite',
              className: 'ExitOptions'
            }
          },
          fileUploadError: {
            required: false,
            serializedName: 'fileUploadError',
            type: {
              name: 'Composite',
              className: 'ExitOptions'
            }
          },
          default: {
            required: false,
            serializedName: 'default',
            type: {
              name: 'Composite',
              className: 'ExitOptions'
            }
          }
        }
      }
    };
  }
}

module.exports = ExitConditions;
