/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary A reference to an Azure Virtual Machines Marketplace image or a
 * custom Azure Virtual Machine image. To get the list of all Azure Marketplace
 * image references verified by Azure Batch, see the 'List node agent SKUs'
 * operation.
  *
 */
class ImageReference {
  /**
   * Create a ImageReference.
   * @member {string} [publisher] The publisher of the Azure Virtual Machines
   * Marketplace image. For example, Canonical or MicrosoftWindowsServer.
   * @member {string} [offer] The offer type of the Azure Virtual Machines
   * Marketplace image. For example, UbuntuServer or WindowsServer.
   * @member {string} [sku] The SKU of the Azure Virtual Machines Marketplace
   * image. For example, 14.04.0-LTS or 2012-R2-Datacenter.
   * @member {string} [version] The version of the Azure Virtual Machines
   * Marketplace image. A value of 'latest' can be specified to select the
   * latest version of an image. If omitted, the default is 'latest'.
   * @member {string} [virtualMachineImageId] The ARM resource identifier of
   * the virtual machine image. Computes nodes of the pool will be created
   * using this custom image. This is of the form
   * /subscriptions/{subscriptionId}/resourceGroups/{resourceGroup}/providers/Microsoft.Compute/images/{imageName}.
   * This property is mutually exclusive with other ImageReference properties.
   * The virtual machine image must be in the same region and subscription as
   * the Azure Batch account. For information about the firewall settings for
   * the Batch node agent to communicate with the Batch service see
   * https://docs.microsoft.com/en-us/azure/batch/batch-api-basics#virtual-network-vnet-and-firewall-configuration.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ImageReference
   *
   * @returns {object} metadata of ImageReference
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ImageReference',
      type: {
        name: 'Composite',
        className: 'ImageReference',
        modelProperties: {
          publisher: {
            required: false,
            serializedName: 'publisher',
            type: {
              name: 'String'
            }
          },
          offer: {
            required: false,
            serializedName: 'offer',
            type: {
              name: 'String'
            }
          },
          sku: {
            required: false,
            serializedName: 'sku',
            type: {
              name: 'String'
            }
          },
          version: {
            required: false,
            serializedName: 'version',
            type: {
              name: 'String'
            }
          },
          virtualMachineImageId: {
            required: false,
            serializedName: 'virtualMachineImageId',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ImageReference;
