/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * @summary The status of the Job Preparation and Job Release tasks on a
 * compute node.
  *
 */
class JobPreparationAndReleaseTaskExecutionInformation {
  /**
   * Create a JobPreparationAndReleaseTaskExecutionInformation.
   * @member {string} [poolId] The ID of the pool containing the compute node
   * to which this entry refers.
   * @member {string} [nodeId] The ID of the compute node to which this entry
   * refers.
   * @member {string} [nodeUrl] The URL of the compute node to which this entry
   * refers.
   * @member {object} [jobPreparationTaskExecutionInfo] Information about the
   * execution status of the Job Preparation task on this compute node.
   * @member {date} [jobPreparationTaskExecutionInfo.startTime] If the task has
   * been restarted or retried, this is the most recent time at which the task
   * started running.
   * @member {date} [jobPreparationTaskExecutionInfo.endTime] This property is
   * set only if the task is in the Completed state.
   * @member {string} [jobPreparationTaskExecutionInfo.state] Values are:
   *
   * running - the task is currently running (including retrying).
   * completed - the task has exited with exit code 0, or the task has
   * exhausted its retry limit, or the Batch service was unable to start the
   * task due to task preparation errors (such as resource file download
   * failures). Possible values include: 'running', 'completed'
   * @member {string} [jobPreparationTaskExecutionInfo.taskRootDirectory]
   * @member {string} [jobPreparationTaskExecutionInfo.taskRootDirectoryUrl]
   * @member {number} [jobPreparationTaskExecutionInfo.exitCode] This parameter
   * is returned only if the task is in the completed state. The exit code for
   * a process reflects the specific convention implemented by the application
   * developer for that process. If you use the exit code value to make
   * decisions in your code, be sure that you know the exit code convention
   * used by the application process. Note that the exit code may also be
   * generated by the compute node operating system, such as when a process is
   * forcibly terminated.
   * @member {object} [jobPreparationTaskExecutionInfo.containerInfo] This
   * property is set only if the task runs in a container context.
   * @member {string}
   * [jobPreparationTaskExecutionInfo.containerInfo.containerId]
   * @member {string} [jobPreparationTaskExecutionInfo.containerInfo.state]
   * This is the state of the container according to the Docker service. It is
   * equivalent to the status field returned by "docker inspect".
   * @member {string} [jobPreparationTaskExecutionInfo.containerInfo.error]
   * This is the detailed error string from the Docker service, if available.
   * It is equivalent to the error field returned by "docker inspect".
   * @member {object} [jobPreparationTaskExecutionInfo.failureInfo] This
   * property is set only if the task is in the completed state and encountered
   * a failure.
   * @member {string} [jobPreparationTaskExecutionInfo.failureInfo.category]
   * Possible values include: 'userError', 'serverError'
   * @member {string} [jobPreparationTaskExecutionInfo.failureInfo.code]
   * @member {string} [jobPreparationTaskExecutionInfo.failureInfo.message]
   * @member {array} [jobPreparationTaskExecutionInfo.failureInfo.details]
   * @member {number} [jobPreparationTaskExecutionInfo.retryCount] Task
   * application failures (non-zero exit code) are retried, pre-processing
   * errors (the task could not be run) and file upload errors are not retried.
   * The Batch service will retry the task up to the limit specified by the
   * constraints.
   * @member {date} [jobPreparationTaskExecutionInfo.lastRetryTime] This
   * property is set only if the task was retried (i.e. retryCount is nonzero).
   * If present, this is typically the same as startTime, but may be different
   * if the task has been restarted for reasons other than retry; for example,
   * if the compute node was rebooted during a retry, then the startTime is
   * updated but the lastRetryTime is not.
   * @member {string} [jobPreparationTaskExecutionInfo.result] If the value is
   * 'failed', then the details of the failure can be found in the failureInfo
   * property. Possible values include: 'success', 'failure'
   * @member {object} [jobReleaseTaskExecutionInfo] Information about the
   * execution status of the Job Release task on this compute node. This
   * property is set only if the Job Release task has run on the node.
   * @member {date} [jobReleaseTaskExecutionInfo.startTime] If the task has
   * been restarted or retried, this is the most recent time at which the task
   * started running.
   * @member {date} [jobReleaseTaskExecutionInfo.endTime] This property is set
   * only if the task is in the Completed state.
   * @member {string} [jobReleaseTaskExecutionInfo.state] Values are:
   *
   * running - the task is currently running (including retrying).
   * completed - the task has exited, or the Batch service was unable to start
   * the task due to task preparation errors (such as resource file download
   * failures). Possible values include: 'running', 'completed'
   * @member {string} [jobReleaseTaskExecutionInfo.taskRootDirectory]
   * @member {string} [jobReleaseTaskExecutionInfo.taskRootDirectoryUrl]
   * @member {number} [jobReleaseTaskExecutionInfo.exitCode] This parameter is
   * returned only if the task is in the completed state. The exit code for a
   * process reflects the specific convention implemented by the application
   * developer for that process. If you use the exit code value to make
   * decisions in your code, be sure that you know the exit code convention
   * used by the application process. Note that the exit code may also be
   * generated by the compute node operating system, such as when a process is
   * forcibly terminated.
   * @member {object} [jobReleaseTaskExecutionInfo.containerInfo] This property
   * is set only if the task runs in a container context.
   * @member {string} [jobReleaseTaskExecutionInfo.containerInfo.containerId]
   * @member {string} [jobReleaseTaskExecutionInfo.containerInfo.state] This is
   * the state of the container according to the Docker service. It is
   * equivalent to the status field returned by "docker inspect".
   * @member {string} [jobReleaseTaskExecutionInfo.containerInfo.error] This is
   * the detailed error string from the Docker service, if available. It is
   * equivalent to the error field returned by "docker inspect".
   * @member {object} [jobReleaseTaskExecutionInfo.failureInfo] This property
   * is set only if the task is in the completed state and encountered a
   * failure.
   * @member {string} [jobReleaseTaskExecutionInfo.failureInfo.category]
   * Possible values include: 'userError', 'serverError'
   * @member {string} [jobReleaseTaskExecutionInfo.failureInfo.code]
   * @member {string} [jobReleaseTaskExecutionInfo.failureInfo.message]
   * @member {array} [jobReleaseTaskExecutionInfo.failureInfo.details]
   * @member {string} [jobReleaseTaskExecutionInfo.result] If the value is
   * 'failed', then the details of the failure can be found in the failureInfo
   * property. Possible values include: 'success', 'failure'
   */
  constructor() {
  }

  /**
   * Defines the metadata of JobPreparationAndReleaseTaskExecutionInformation
   *
   * @returns {object} metadata of JobPreparationAndReleaseTaskExecutionInformation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'JobPreparationAndReleaseTaskExecutionInformation',
      type: {
        name: 'Composite',
        className: 'JobPreparationAndReleaseTaskExecutionInformation',
        modelProperties: {
          poolId: {
            required: false,
            serializedName: 'poolId',
            type: {
              name: 'String'
            }
          },
          nodeId: {
            required: false,
            serializedName: 'nodeId',
            type: {
              name: 'String'
            }
          },
          nodeUrl: {
            required: false,
            serializedName: 'nodeUrl',
            type: {
              name: 'String'
            }
          },
          jobPreparationTaskExecutionInfo: {
            required: false,
            serializedName: 'jobPreparationTaskExecutionInfo',
            type: {
              name: 'Composite',
              className: 'JobPreparationTaskExecutionInformation'
            }
          },
          jobReleaseTaskExecutionInfo: {
            required: false,
            serializedName: 'jobReleaseTaskExecutionInfo',
            type: {
              name: 'Composite',
              className: 'JobReleaseTaskExecutionInformation'
            }
          }
        }
      }
    };
  }
}

module.exports = JobPreparationAndReleaseTaskExecutionInformation;
