/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * @summary Result for a single task added as part of an add task collection
 * operation.
  *
 */
class TaskAddResult {
  /**
   * Create a TaskAddResult.
   * @member {string} status The status of the add task request. Values are:
   *
   * success - Task was added successfully.
   * clienterror - Task failed to add due to a client error and should not be
   * retried without modifying the request as appropriate.
   * servererror - Task failed to add due to a server error and can be retried
   * without modification. Possible values include: 'success', 'clientError',
   * 'serverError'
   * @member {string} taskId The ID of the task for which this is the result.
   * @member {string} [eTag] The ETag of the task, if the task was successfully
   * added. You can use this to detect whether the task has changed between
   * requests. In particular, you can be pass the ETag with an Update Task
   * request to specify that your changes should take effect only if nobody
   * else has modified the job in the meantime.
   * @member {date} [lastModified] The last modified time of the task.
   * @member {string} [location] The URL of the task, if the task was
   * successfully added.
   * @member {object} [error] The error encountered while attempting to add the
   * task.
   * @member {string} [error.code]
   * @member {object} [error.message]
   * @member {string} [error.message.lang]
   * @member {string} [error.message.value]
   * @member {array} [error.values]
   */
  constructor() {
  }

  /**
   * Defines the metadata of TaskAddResult
   *
   * @returns {object} metadata of TaskAddResult
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'TaskAddResult',
      type: {
        name: 'Composite',
        className: 'TaskAddResult',
        modelProperties: {
          status: {
            required: true,
            serializedName: 'status',
            type: {
              name: 'Enum',
              allowedValues: [ 'success', 'clientError', 'serverError' ]
            }
          },
          taskId: {
            required: true,
            serializedName: 'taskId',
            type: {
              name: 'String'
            }
          },
          eTag: {
            required: false,
            serializedName: 'eTag',
            type: {
              name: 'String'
            }
          },
          lastModified: {
            required: false,
            serializedName: 'lastModified',
            type: {
              name: 'DateTime'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          error: {
            required: false,
            serializedName: 'error',
            type: {
              name: 'Composite',
              className: 'BatchError'
            }
          }
        }
      }
    };
  }
}

module.exports = TaskAddResult;
