/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * @summary Execution constraints to apply to a task.
  *
 */
class TaskConstraints {
  /**
   * Create a TaskConstraints.
   * @member {moment.duration} [maxWallClockTime] The maximum elapsed time that
   * the task may run, measured from the time the task starts. If the task does
   * not complete within the time limit, the Batch service terminates it. If
   * this is not specified, there is no time limit on how long the task may
   * run.
   * @member {moment.duration} [retentionTime] The minimum time to retain the
   * task directory on the compute node where it ran, from the time it
   * completes execution. After this time, the Batch service may delete the
   * task directory and all its contents. The default is infinite, i.e. the
   * task directory will be retained until the compute node is removed or
   * reimaged.
   * @member {number} [maxTaskRetryCount] The maximum number of times the task
   * may be retried. The Batch service retries a task if its exit code is
   * nonzero. Note that this value specifically controls the number of retries.
   * The Batch service will try the task once, and may then retry up to this
   * limit. For example, if the maximum retry count is 3, Batch tries the task
   * up to 4 times (one initial try and 3 retries). If the maximum retry count
   * is 0, the Batch service does not retry the task. If the maximum retry
   * count is -1, the Batch service retries the task without limit.
   */
  constructor() {
  }

  /**
   * Defines the metadata of TaskConstraints
   *
   * @returns {object} metadata of TaskConstraints
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'TaskConstraints',
      type: {
        name: 'Composite',
        className: 'TaskConstraints',
        modelProperties: {
          maxWallClockTime: {
            required: false,
            serializedName: 'maxWallClockTime',
            type: {
              name: 'TimeSpan'
            }
          },
          retentionTime: {
            required: false,
            serializedName: 'retentionTime',
            type: {
              name: 'TimeSpan'
            }
          },
          maxTaskRetryCount: {
            required: false,
            serializedName: 'maxTaskRetryCount',
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = TaskConstraints;
