/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Registries
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Registries {


    /**
     * Checks whether the container registry name is available for use. The name
     * must contain only alphanumeric characters, be globally unique, and between 5
     * and 50 characters in length.
     *
     * @param {object} registryNameCheckRequest The object containing information
     * for the availability request.
     *
     * @param {string} registryNameCheckRequest.name The name of the container
     * registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryNameStatus>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    checkNameAvailabilityWithHttpOperationResponse(registryNameCheckRequest: models.RegistryNameCheckRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryNameStatus>>;

    /**
     * Checks whether the container registry name is available for use. The name
     * must contain only alphanumeric characters, be globally unique, and between 5
     * and 50 characters in length.
     *
     * @param {object} registryNameCheckRequest The object containing information
     * for the availability request.
     *
     * @param {string} registryNameCheckRequest.name The name of the container
     * registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryNameStatus} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryNameStatus} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryNameStatus} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    checkNameAvailability(registryNameCheckRequest: models.RegistryNameCheckRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryNameStatus>;
    checkNameAvailability(registryNameCheckRequest: models.RegistryNameCheckRequest, callback: ServiceCallback<models.RegistryNameStatus>): void;
    checkNameAvailability(registryNameCheckRequest: models.RegistryNameCheckRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryNameStatus>): void;


    /**
     * Gets the properties of the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Gets the properties of the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    get(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.Registry>): void;
    get(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic',
     * 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic',
     * 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    create(resourceGroupName: string, registryName: string, registry: models.Registry, callback: ServiceCallback<models.Registry>): void;
    create(resourceGroupName: string, registryName: string, registry: models.Registry, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    update(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, callback: ServiceCallback<models.Registry>): void;
    update(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.RegistryListResult>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;


    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    list(callback: ServiceCallback<models.RegistryListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;


    /**
     * Lists the login credentials for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListCredentialsResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listCredentialsWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListCredentialsResult>>;

    /**
     * Lists the login credentials for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListCredentialsResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListCredentialsResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListCredentialsResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listCredentials(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListCredentialsResult>;
    listCredentials(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;
    listCredentials(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;


    /**
     * Regenerates one of the login credentials for the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} regenerateCredentialParameters Specifies name of the
     * password which should be regenerated -- password or password2.
     *
     * @param {string} regenerateCredentialParameters.name Specifies name of the
     * password which should be regenerated -- password or password2. Possible
     * values include: 'password', 'password2'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListCredentialsResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    regenerateCredentialWithHttpOperationResponse(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListCredentialsResult>>;

    /**
     * Regenerates one of the login credentials for the specified container
     * registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} regenerateCredentialParameters Specifies name of the
     * password which should be regenerated -- password or password2.
     *
     * @param {string} regenerateCredentialParameters.name Specifies name of the
     * password which should be regenerated -- password or password2. Possible
     * values include: 'password', 'password2'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListCredentialsResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListCredentialsResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListCredentialsResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    regenerateCredential(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListCredentialsResult>;
    regenerateCredential(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;
    regenerateCredential(resourceGroupName: string, registryName: string, regenerateCredentialParameters: models.RegenerateCredentialParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListCredentialsResult>): void;


    /**
     * Gets the quota usages for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryUsageListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listUsagesWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryUsageListResult>>;

    /**
     * Gets the quota usages for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryUsageListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryUsageListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryUsageListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listUsages(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryUsageListResult>;
    listUsages(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.RegistryUsageListResult>): void;
    listUsages(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryUsageListResult>): void;


    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic',
     * 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Creates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registry The parameters for creating a container registry.
     *
     * @param {object} registry.sku The SKU of the container registry.
     *
     * @param {string} registry.sku.name The SKU name of the container registry.
     * Required for registry creation. Possible values include: 'Classic',
     * 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registry.adminUserEnabled] The value that indicates
     * whether the admin user is enabled.
     *
     * @param {object} [registry.storageAccount] The properties of the storage
     * account for the container registry. Only applicable to Classic SKU.
     *
     * @param {string} registry.storageAccount.id The resource ID of the storage
     * account.
     *
     * @param {string} registry.location The location of the resource. This cannot
     * be changed after the resource is created.
     *
     * @param {object} [registry.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, registry: models.Registry, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    beginCreate(resourceGroupName: string, registryName: string, registry: models.Registry, callback: ServiceCallback<models.Registry>): void;
    beginCreate(resourceGroupName: string, registryName: string, registry: models.Registry, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Registry>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Registry>>;

    /**
     * Updates a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} registryUpdateParameters The parameters for updating a
     * container registry.
     *
     * @param {object} [registryUpdateParameters.tags] The tags for the container
     * registry.
     *
     * @param {object} [registryUpdateParameters.sku] The SKU of the container
     * registry.
     *
     * @param {string} registryUpdateParameters.sku.name The SKU name of the
     * container registry. Required for registry creation. Possible values include:
     * 'Classic', 'Managed_Basic', 'Managed_Standard', 'Managed_Premium'
     *
     * @param {boolean} [registryUpdateParameters.adminUserEnabled] The value that
     * indicates whether the admin user is enabled.
     *
     * @param {object} [registryUpdateParameters.storageAccount] The parameters of
     * a storage account for the container registry. Only applicable to Classic
     * SKU. If specified, the storage account must be in the same physical location
     * as the container registry.
     *
     * @param {string} registryUpdateParameters.storageAccount.id The resource ID
     * of the storage account.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Registry} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Registry} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Registry} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Registry>;
    beginUpdate(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, callback: ServiceCallback<models.Registry>): void;
    beginUpdate(resourceGroupName: string, registryName: string, registryUpdateParameters: models.RegistryUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Registry>): void;


    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.RegistryListResult>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;


    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RegistryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RegistryListResult>>;

    /**
     * Lists all the container registries under the specified subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RegistryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RegistryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RegistryListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RegistryListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.RegistryListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RegistryListResult>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Operations {


    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;


    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available Azure Container Registry REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.OperationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}

/**
 * @class
 * Replications
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Replications {


    /**
     * Gets the properties of the specified replication.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Gets the properties of the specified replication.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    get(resourceGroupName: string, registryName: string, replicationName: string, callback: ServiceCallback<models.Replication>): void;
    get(resourceGroupName: string, registryName: string, replicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    create(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, callback: ServiceCallback<models.Replication>): void;
    create(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, replicationName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    update(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, callback: ServiceCallback<models.Replication>): void;
    update(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ReplicationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ReplicationListResult>>;

    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ReplicationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ReplicationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ReplicationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ReplicationListResult>;
    list(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.ReplicationListResult>): void;
    list(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ReplicationListResult>): void;


    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Creates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replication The parameters for creating a replication.
     *
     * @param {string} replication.location The location of the resource. This
     * cannot be changed after the resource is created.
     *
     * @param {object} [replication.tags] The tags of the resource.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    beginCreate(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, callback: ServiceCallback<models.Replication>): void;
    beginCreate(resourceGroupName: string, registryName: string, replicationName: string, replication: models.Replication, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a replication from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, replicationName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, replicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Replication>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Replication>>;

    /**
     * Updates a replication for a container registry with the specified
     * parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} replicationName The name of the replication.
     *
     * @param {object} replicationUpdateParameters The parameters for updating a
     * replication.
     *
     * @param {object} [replicationUpdateParameters.tags] The tags for the
     * replication.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Replication} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Replication} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Replication} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Replication>;
    beginUpdate(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, callback: ServiceCallback<models.Replication>): void;
    beginUpdate(resourceGroupName: string, registryName: string, replicationName: string, replicationUpdateParameters: models.ReplicationUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Replication>): void;


    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ReplicationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ReplicationListResult>>;

    /**
     * Lists all the replications for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ReplicationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ReplicationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ReplicationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ReplicationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ReplicationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ReplicationListResult>): void;
}

/**
 * @class
 * Webhooks
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the ContainerRegistryManagementClient.
 */
export interface Webhooks {


    /**
     * Gets the properties of the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Gets the properties of the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    get(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.Webhook>): void;
    get(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    create(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, callback: ServiceCallback<models.Webhook>): void;
    create(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    update(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, callback: ServiceCallback<models.Webhook>): void;
    update(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WebhookListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WebhookListResult>>;

    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WebhookListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WebhookListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WebhookListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, registryName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WebhookListResult>;
    list(resourceGroupName: string, registryName: string, callback: ServiceCallback<models.WebhookListResult>): void;
    list(resourceGroupName: string, registryName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WebhookListResult>): void;


    /**
     * Triggers a ping event to be sent to the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventInfo>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    pingWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventInfo>>;

    /**
     * Triggers a ping event to be sent to the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventInfo} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventInfo} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventInfo} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    ping(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventInfo>;
    ping(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.EventInfo>): void;
    ping(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventInfo>): void;


    /**
     * Gets the configuration of service URI and custom headers for the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CallbackConfig>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getCallbackConfigWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CallbackConfig>>;

    /**
     * Gets the configuration of service URI and custom headers for the webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CallbackConfig} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CallbackConfig} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CallbackConfig} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getCallbackConfig(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CallbackConfig>;
    getCallbackConfig(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.CallbackConfig>): void;
    getCallbackConfig(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CallbackConfig>): void;


    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listEventsWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventListResult>>;

    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listEvents(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventListResult>;
    listEvents(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<models.EventListResult>): void;
    listEvents(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventListResult>): void;


    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Creates a webhook for a container registry with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookCreateParameters The parameters for creating a
     * webhook.
     *
     * @param {object} [webhookCreateParameters.tags] The tags for the webhook.
     *
     * @param {string} webhookCreateParameters.location The location of the
     * webhook. This cannot be changed after the resource is created.
     *
     * @param {string} webhookCreateParameters.serviceUri The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookCreateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookCreateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} webhookCreateParameters.actions The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookCreateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    beginCreate(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, callback: ServiceCallback<models.Webhook>): void;
    beginCreate(resourceGroupName: string, registryName: string, webhookName: string, webhookCreateParameters: models.WebhookCreateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a webhook from a container registry.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, registryName: string, webhookName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, registryName: string, webhookName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Webhook>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateWithHttpOperationResponse(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Webhook>>;

    /**
     * Updates a webhook with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group to which
     * the container registry belongs.
     *
     * @param {string} registryName The name of the container registry.
     *
     * @param {string} webhookName The name of the webhook.
     *
     * @param {object} webhookUpdateParameters The parameters for updating a
     * webhook.
     *
     * @param {object} [webhookUpdateParameters.tags] The tags for the webhook.
     *
     * @param {string} [webhookUpdateParameters.serviceUri] The service URI for the
     * webhook to post notifications.
     *
     * @param {string} [webhookUpdateParameters.status] The status of the webhook
     * at the time the operation was called. Possible values include: 'enabled',
     * 'disabled'
     *
     * @param {string} [webhookUpdateParameters.scope] The scope of repositories
     * where the event can be triggered. For example, 'foo:*' means events for all
     * tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only.
     * 'foo' is equivalent to 'foo:latest'. Empty means all events.
     *
     * @param {array} [webhookUpdateParameters.actions] The list of actions that
     * trigger the webhook to post notifications.
     *
     * @param {object} [webhookUpdateParameters.customHeaders] Custom headers that
     * will be added to the webhook notifications.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Webhook} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Webhook} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Webhook} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdate(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Webhook>;
    beginUpdate(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, callback: ServiceCallback<models.Webhook>): void;
    beginUpdate(resourceGroupName: string, registryName: string, webhookName: string, webhookUpdateParameters: models.WebhookUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Webhook>): void;


    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WebhookListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WebhookListResult>>;

    /**
     * Lists all the webhooks for the specified container registry.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WebhookListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WebhookListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WebhookListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WebhookListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.WebhookListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WebhookListResult>): void;


    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<EventListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listEventsNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.EventListResult>>;

    /**
     * Lists recent events for the specified webhook.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {EventListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {EventListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link EventListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listEventsNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.EventListResult>;
    listEventsNext(nextPageLink: string, callback: ServiceCallback<models.EventListResult>): void;
    listEventsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.EventListResult>): void;
}
