/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Dynamics linked service.
 *
 * @extends models['LinkedService']
 */
class DynamicsLinkedService extends models['LinkedService'] {
  /**
   * Create a DynamicsLinkedService.
   * @member {object} deploymentType The deployment type of the Dynamics
   * instance. 'Online' for Dynamics Online and 'OnPremisesWithIfd' for
   * Dynamics on-premises with Ifd. Type: string (or Expression with resultType
   * string).
   * @member {object} [hostName] The host name of the on-premises Dynamics
   * server. The property is required for on-prem and not allowed for online.
   * Type: string (or Expression with resultType string).
   * @member {object} [port] The port of on-premises Dynamics server. The
   * property is required for on-prem and not allowed for online. Default is
   * 443. Type: integer (or Expression with resultType integer), minimum: 0.
   * @member {object} [organizationName] The organization name of the Dynamics
   * instance. The property is required for on-prem and required for online
   * when there are more than one Dynamics instances associated with the user.
   * Type: string (or Expression with resultType string).
   * @member {object} authenticationType The authentication type to connect to
   * Dynamics server. 'Office365' for online scenario, 'Ifd' for on-premises
   * with Ifd scenario. Type: string (or Expression with resultType string).
   * @member {object} username User name to access the Dynamics instance. Type:
   * string (or Expression with resultType string).
   * @member {object} password Password to access the Dynamics instance.
   * @member {object} [password.secretName] The name of secret in Azure Key
   * Vault. Type: string (or Expression with resultType string).
   * @member {object} [password.secretVersion] The version of the secret in
   * Azure Key Vault. The default value is the latest version of the secret.
   * Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DynamicsLinkedService
   *
   * @returns {object} metadata of DynamicsLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Dynamics',
      type: {
        name: 'Composite',
        className: 'DynamicsLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          deploymentType: {
            required: true,
            serializedName: 'typeProperties.deploymentType',
            type: {
              name: 'Object'
            }
          },
          hostName: {
            required: false,
            serializedName: 'typeProperties.hostName',
            type: {
              name: 'Object'
            }
          },
          port: {
            required: false,
            serializedName: 'typeProperties.port',
            type: {
              name: 'Object'
            }
          },
          organizationName: {
            required: false,
            serializedName: 'typeProperties.organizationName',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: true,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'Object'
            }
          },
          username: {
            required: true,
            serializedName: 'typeProperties.username',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: true,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              className: 'AzureKeyVaultSecretReference'
            }
          }
        }
      }
    };
  }
}

module.exports = DynamicsLinkedService;
