/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A FTP server Linked Service.
 *
 * @extends models['LinkedService']
 */
class FtpServerLinkedService extends models['LinkedService'] {
  /**
   * Create a FtpServerLinkedService.
   * @member {object} host Host name of the FTP server. Type: string (or
   * Expression with resultType string).
   * @member {object} [port] The TCP port number that the FTP server uses to
   * listen for client connections. Default value is 21. Type: integer (or
   * Expression with resultType integer), minimum: 0.
   * @member {string} [authenticationType] The authentication type to be used
   * to connect to the FTP server. Possible values include: 'Basic',
   * 'Anonymous'
   * @member {object} [userName] Username to logon the FTP server. Type: string
   * (or Expression with resultType string).
   * @member {object} [password] Password to logon the FTP server.
   * @member {string} [password.value] Value of secure string.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   * @member {object} [enableSsl] If true, connect to the FTP server over
   * SSL/TLS channel. Default value is true. Type: boolean (or Expression with
   * resultType boolean).
   * @member {object} [enableServerCertificateValidation] If true, validate the
   * FTP server SSL certificate when connect over SSL/TLS channel. Default
   * value is true. Type: boolean (or Expression with resultType boolean).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of FtpServerLinkedService
   *
   * @returns {object} metadata of FtpServerLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'FtpServer',
      type: {
        name: 'Composite',
        className: 'FtpServerLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          host: {
            required: true,
            serializedName: 'typeProperties.host',
            type: {
              name: 'Object'
            }
          },
          port: {
            required: false,
            serializedName: 'typeProperties.port',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: false,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'String'
            }
          },
          userName: {
            required: false,
            serializedName: 'typeProperties.userName',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              className: 'SecureString'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          },
          enableSsl: {
            required: false,
            serializedName: 'typeProperties.enableSsl',
            type: {
              name: 'Object'
            }
          },
          enableServerCertificateValidation: {
            required: false,
            serializedName: 'typeProperties.enableServerCertificateValidation',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = FtpServerLinkedService;
