/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * HDInsight streaming activity type.
 *
 * @extends models['ExecutionActivity']
 */
class HDInsightStreamingActivity extends models['ExecutionActivity'] {
  /**
   * Create a HDInsightStreamingActivity.
   * @member {array} [storageLinkedServices] Storage linked service references.
   * @member {array} [arguments] User specified arguments to HDInsightActivity.
   * @member {string} [getDebugInfo] Debug info option. Possible values
   * include: 'None', 'Always', 'Failure'
   * @member {object} mapper Mapper executable name. Type: string (or
   * Expression with resultType string).
   * @member {object} reducer Reducer executable name. Type: string (or
   * Expression with resultType string).
   * @member {object} input Input blob path. Type: string (or Expression with
   * resultType string).
   * @member {object} output Output blob path. Type: string (or Expression with
   * resultType string).
   * @member {array} filePaths Paths to streaming job files. Can be
   * directories.
   * @member {object} [fileLinkedService] Linked service reference where the
   * files are located.
   * @member {string} [fileLinkedService.referenceName] Reference LinkedService
   * name.
   * @member {object} [combiner] Combiner executable name. Type: string (or
   * Expression with resultType string).
   * @member {array} [commandEnvironment] Command line environment values.
   * @member {object} [defines] Allows user to specify defines for streaming
   * job request.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of HDInsightStreamingActivity
   *
   * @returns {object} metadata of HDInsightStreamingActivity
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'HDInsightStreaming',
      type: {
        name: 'Composite',
        className: 'HDInsightStreamingActivity',
        modelProperties: {
          name: {
            required: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          dependsOn: {
            required: false,
            serializedName: 'dependsOn',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ActivityDependencyElementType',
                  type: {
                    name: 'Composite',
                    className: 'ActivityDependency'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          linkedServiceName: {
            required: false,
            serializedName: 'linkedServiceName',
            type: {
              name: 'Composite',
              className: 'LinkedServiceReference'
            }
          },
          policy: {
            required: false,
            serializedName: 'policy',
            type: {
              name: 'Composite',
              className: 'ActivityPolicy'
            }
          },
          storageLinkedServices: {
            required: false,
            serializedName: 'typeProperties.storageLinkedServices',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'LinkedServiceReferenceElementType',
                  type: {
                    name: 'Composite',
                    className: 'LinkedServiceReference'
                  }
              }
            }
          },
          arguments: {
            required: false,
            serializedName: 'typeProperties.arguments',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          getDebugInfo: {
            required: false,
            serializedName: 'typeProperties.getDebugInfo',
            type: {
              name: 'String'
            }
          },
          mapper: {
            required: true,
            serializedName: 'typeProperties.mapper',
            type: {
              name: 'Object'
            }
          },
          reducer: {
            required: true,
            serializedName: 'typeProperties.reducer',
            type: {
              name: 'Object'
            }
          },
          input: {
            required: true,
            serializedName: 'typeProperties.input',
            type: {
              name: 'Object'
            }
          },
          output: {
            required: true,
            serializedName: 'typeProperties.output',
            type: {
              name: 'Object'
            }
          },
          filePaths: {
            required: true,
            serializedName: 'typeProperties.filePaths',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          fileLinkedService: {
            required: false,
            serializedName: 'typeProperties.fileLinkedService',
            type: {
              name: 'Composite',
              className: 'LinkedServiceReference'
            }
          },
          combiner: {
            required: false,
            serializedName: 'typeProperties.combiner',
            type: {
              name: 'Object'
            }
          },
          commandEnvironment: {
            required: false,
            serializedName: 'typeProperties.commandEnvironment',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          },
          defines: {
            required: false,
            serializedName: 'typeProperties.defines',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = HDInsightStreamingActivity;
