/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A file in an HTTP web server.
 *
 * @extends models['Dataset']
 */
class HttpDataset extends models['Dataset'] {
  /**
   * Create a HttpDataset.
   * @member {object} [relativeUrl] The relative URL based on the URL in the
   * HttpLinkedService refers to an HTTP file Type: string (or Expression with
   * resultType string).
   * @member {object} [requestMethod] The HTTP method for the HTTP request.
   * Type: string (or Expression with resultType string).
   * @member {object} [requestBody] The body for the HTTP request. Type: string
   * (or Expression with resultType string).
   * @member {object} [additionalHeaders] The headers for the HTTP Request.
   * e.g. request-header-name-1:request-header-value-1
   * ...
   * request-header-name-n:request-header-value-n Type: string (or Expression
   * with resultType string).
   * @member {object} [partitionedBy] The HTTP method for the HTTP request.
   * @member {object} [partitionedBy.name] Name of the partition. Type: string
   * (or Expression with resultType string).
   * @member {object} [partitionedBy.value] Value of the partition.
   * @member {string} [partitionedBy.value.type] Polymorphic Discriminator
   * @member {object} [format] The format of files.
   * @member {object} [format.serializer] Serializer. Type: string (or
   * Expression with resultType string).
   * @member {object} [format.deserializer] Deserializer. Type: string (or
   * Expression with resultType string).
   * @member {string} [format.type] Polymorphic Discriminator
   * @member {object} [compression] The data compression method used on files.
   * @member {string} [compression.type] Polymorphic Discriminator
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of HttpDataset
   *
   * @returns {object} metadata of HttpDataset
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'HttpFile',
      type: {
        name: 'Composite',
        className: 'HttpDataset',
        modelProperties: {
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          structure: {
            required: false,
            serializedName: 'structure',
            type: {
              name: 'Object'
            }
          },
          linkedServiceName: {
            required: true,
            serializedName: 'linkedServiceName',
            defaultValue: {},
            type: {
              name: 'Composite',
              className: 'LinkedServiceReference'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          relativeUrl: {
            required: false,
            serializedName: 'typeProperties.relativeUrl',
            type: {
              name: 'Object'
            }
          },
          requestMethod: {
            required: false,
            serializedName: 'typeProperties.requestMethod',
            type: {
              name: 'Object'
            }
          },
          requestBody: {
            required: false,
            serializedName: 'typeProperties.requestBody',
            type: {
              name: 'Object'
            }
          },
          additionalHeaders: {
            required: false,
            serializedName: 'typeProperties.additionalHeaders',
            type: {
              name: 'Object'
            }
          },
          partitionedBy: {
            required: false,
            serializedName: 'typeProperties.partitionedBy',
            type: {
              name: 'Composite',
              className: 'DatasetPartition'
            }
          },
          format: {
            required: false,
            serializedName: 'typeProperties.format',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'DatasetStorageFormat',
              className: 'DatasetStorageFormat'
            }
          },
          compression: {
            required: false,
            serializedName: 'typeProperties.compression',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'DatasetCompression',
              className: 'DatasetCompression'
            }
          }
        }
      }
    };
  }
}

module.exports = HttpDataset;
