/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Azure Data Factory top-level resource.
 *
 * @member {string} [id] The resource identifier.
 * @member {string} [name] The resource name.
 * @member {string} [type] The resource type.
 * @member {string} [location] The resource location.
 * @member {object} [tags] The resource tags.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the SubResource class.
 * @constructor
 * Azure Data Factory nested resource, which belongs to a factory.
 *
 * @member {string} [id] The resource identifier.
 * @member {string} [name] The resource name.
 * @member {string} [type] The resource type.
 * @member {string} [etag] Etag identifies change in the resource.
 */
export interface SubResource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  readonly etag?: string;
}

/**
 * @class
 * Initializes a new instance of the Expression class.
 * @constructor
 * Azure Data Factory expression definition.
 *
 * @member {string} value Expression value.
 */
export interface Expression {
  value: string;
}

/**
 * @class
 * Initializes a new instance of the SecureString class.
 * @constructor
 * Azure Data Factory secure string definition. The string value will be masked
 * with asterisks '*' during Get or List API calls.
 *
 * @member {string} value Value of secure string.
 */
export interface SecureString {
  value: string;
}

/**
 * @class
 * Initializes a new instance of the FactoryIdentity class.
 * @constructor
 * Identity properties of the factory resource.
 *
 * @member {uuid} [principalId] The principal id of the identity.
 * @member {uuid} [tenantId] The client tenant id of the identity.
 */
export interface FactoryIdentity {
  readonly principalId?: string;
  readonly tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the Factory class.
 * @constructor
 * Factory resource type.
 *
 * @member {object} [identity] Managed service identity of the factory.
 * @member {uuid} [identity.principalId] The principal id of the identity.
 * @member {uuid} [identity.tenantId] The client tenant id of the identity.
 * @member {string} [provisioningState] Factory provisioning state, example
 * Succeeded.
 * @member {date} [createTime] Time the factory was created in ISO8601 format.
 * @member {string} [version] Version of the factory.
 */
export interface Factory extends Resource {
  identity?: FactoryIdentity;
  readonly provisioningState?: string;
  readonly createTime?: Date;
  readonly version?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntime class.
 * @constructor
 * Azure Data Factory nested object which serves as a compute resource for
 * activities.
 *
 * @member {string} [description] Integration runtime description.
 * @member {string} type Polymorphic Discriminator
 */
export interface IntegrationRuntime {
  description?: string;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeResource class.
 * @constructor
 * Integration runtime resource type.
 *
 * @member {object} properties Integration runtime properties.
 * @member {string} [properties.description] Integration runtime description.
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface IntegrationRuntimeResource extends SubResource {
  properties: IntegrationRuntime;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeReference class.
 * @constructor
 * Integration runtime reference type.
 *
 * @member {string} referenceName Reference integration runtime name.
 * @member {object} [parameters] Arguments for integration runtime.
 */
export interface IntegrationRuntimeReference {
  referenceName: string;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeStatus class.
 * @constructor
 * Integration runtime status.
 *
 * @member {string} [state] The state of integration runtime. Possible values
 * include: 'Initial', 'Stopped', 'Started', 'Starting', 'Stopping',
 * 'NeedRegistration', 'Online', 'Limited', 'Offline'
 * @member {string} type Polymorphic Discriminator
 */
export interface IntegrationRuntimeStatus {
  readonly state?: string;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeStatusResponse class.
 * @constructor
 * Integration runtime status response.
 *
 * @member {string} [name] The integration runtime name.
 * @member {object} properties Integration runtime properties.
 * @member {string} [properties.state] The state of integration runtime.
 * Possible values include: 'Initial', 'Stopped', 'Started', 'Starting',
 * 'Stopping', 'NeedRegistration', 'Online', 'Limited', 'Offline'
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface IntegrationRuntimeStatusResponse {
  readonly name?: string;
  properties: IntegrationRuntimeStatus;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeStatusListResponse class.
 * @constructor
 * A list of integration runtime status.
 *
 * @member {array} value List of integration runtime status.
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface IntegrationRuntimeStatusListResponse {
  value: IntegrationRuntimeStatusResponse[];
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedService class.
 * @constructor
 * The Azure Data Factory nested object which contains the information and
 * credential which can be used to connect with related store or compute
 * resource.
 *
 * @member {object} [connectVia] The integration runtime reference.
 * @member {string} [connectVia.referenceName] Reference integration runtime
 * name.
 * @member {object} [connectVia.parameters] Arguments for integration runtime.
 * @member {string} [description] Linked service description.
 * @member {string} type Polymorphic Discriminator
 */
export interface LinkedService {
  connectVia?: IntegrationRuntimeReference;
  description?: string;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedServiceResource class.
 * @constructor
 * Linked service resource type.
 *
 * @member {object} properties Properties of linked service.
 * @member {object} [properties.connectVia] The integration runtime reference.
 * @member {string} [properties.connectVia.referenceName] Reference integration
 * runtime name.
 * @member {object} [properties.connectVia.parameters] Arguments for
 * integration runtime.
 * @member {string} [properties.description] Linked service description.
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface LinkedServiceResource extends SubResource {
  properties: LinkedService;
}

/**
 * @class
 * Initializes a new instance of the LinkedServiceReference class.
 * @constructor
 * Linked service reference type.
 *
 * @member {string} referenceName Reference LinkedService name.
 */
export interface LinkedServiceReference {
  referenceName: string;
}

/**
 * @class
 * Initializes a new instance of the ParameterSpecification class.
 * @constructor
 * Definition of a single parameter for an entity.
 *
 * @member {string} type Parameter type. Possible values include: 'Object',
 * 'String', 'Int', 'Float', 'Bool', 'Array'
 * @member {object} [defaultValue] Default value of parameter.
 */
export interface ParameterSpecification {
  type: string;
  defaultValue?: any;
}

/**
 * @class
 * Initializes a new instance of the Dataset class.
 * @constructor
 * The Azure Data Factory nested object which identifies data within different
 * data stores, such as tables, files, folders, and documents.
 *
 * @member {string} [description] Dataset description.
 * @member {object} [structure] Columns that define the structure of the
 * dataset. Type: array (or Expression with resultType array), itemType:
 * DatasetDataElement.
 * @member {object} linkedServiceName Linked service reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [parameters] Parameters for dataset.
 * @member {string} type Polymorphic Discriminator
 */
export interface Dataset {
  description?: string;
  structure?: any;
  linkedServiceName: LinkedServiceReference;
  parameters?: { [propertyName: string]: ParameterSpecification };
  type: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetResource class.
 * @constructor
 * Dataset resource type.
 *
 * @member {object} properties Dataset properties.
 * @member {string} [properties.description] Dataset description.
 * @member {object} [properties.structure] Columns that define the structure of
 * the dataset. Type: array (or Expression with resultType array), itemType:
 * DatasetDataElement.
 * @member {object} [properties.linkedServiceName] Linked service reference.
 * @member {string} [properties.linkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [properties.parameters] Parameters for dataset.
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface DatasetResource extends SubResource {
  properties: Dataset;
}

/**
 * @class
 * Initializes a new instance of the ActivityDependency class.
 * @constructor
 * Activity dependency information.
 *
 * @member {string} activity Activity name.
 * @member {array} dependencyConditions Match-Condition for the dependency.
 */
export interface ActivityDependency {
  activity: string;
  dependencyConditions: string[];
}

/**
 * @class
 * Initializes a new instance of the Activity class.
 * @constructor
 * A pipeline activity.
 *
 * @member {string} name Activity name.
 * @member {string} [description] Activity description.
 * @member {array} [dependsOn] Activity depends on condition.
 * @member {string} type Polymorphic Discriminator
 */
export interface Activity {
  name: string;
  description?: string;
  dependsOn?: ActivityDependency[];
  type: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineResource class.
 * @constructor
 * Pipeline resource type.
 *
 * @member {string} [description] The description of the pipeline.
 * @member {array} [activities] List of activities in pipeline.
 * @member {object} [parameters] List of parameters for pipeline.
 * @member {number} [concurrency] The max number of concurrent runs for the
 * pipeline.
 */
export interface PipelineResource extends SubResource {
  description?: string;
  activities?: Activity[];
  parameters?: { [propertyName: string]: ParameterSpecification };
  concurrency?: number;
}

/**
 * @class
 * Initializes a new instance of the PipelineReference class.
 * @constructor
 * Pipeline reference type.
 *
 * @member {string} referenceName Reference pipeline name.
 * @member {string} [name] Reference name.
 */
export interface PipelineReference {
  referenceName: string;
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerPipelineReference class.
 * @constructor
 * Pipeline that needs to be triggered with the given parameters.
 *
 * @member {object} [pipelineReference] Pipeline reference.
 * @member {string} [pipelineReference.referenceName] Reference pipeline name.
 * @member {string} [pipelineReference.name] Reference name.
 * @member {object} [parameters] Pipeline parameters.
 */
export interface TriggerPipelineReference {
  pipelineReference?: PipelineReference;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the Trigger class.
 * @constructor
 * Azure data factory nested object which contains information about creating
 * pipeline run
 *
 * @member {string} [description] Trigger description.
 * @member {array} [pipelines] Pipelines that need to be started.
 * @member {string} [runtimeState] Indicates if trigger is running or not.
 * Updated when Start/Stop APIs are called on the Trigger. Possible values
 * include: 'Started', 'Stopped', 'Disabled'
 * @member {string} type Polymorphic Discriminator
 */
export interface Trigger {
  description?: string;
  pipelines?: TriggerPipelineReference[];
  readonly runtimeState?: string;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerResource class.
 * @constructor
 * Trigger resource type.
 *
 * @member {object} properties Properties of the trigger.
 * @member {string} [properties.description] Trigger description.
 * @member {array} [properties.pipelines] Pipelines that need to be started.
 * @member {string} [properties.runtimeState] Indicates if trigger is running
 * or not. Updated when Start/Stop APIs are called on the Trigger. Possible
 * values include: 'Started', 'Stopped', 'Disabled'
 * @member {string} [properties.type] Polymorphic Discriminator
 */
export interface TriggerResource extends SubResource {
  properties: Trigger;
}

/**
 * @class
 * Initializes a new instance of the CreateRunResponse class.
 * @constructor
 * Response body with a run identifier.
 *
 * @member {string} runId Identifier of a run.
 */
export interface CreateRunResponse {
  runId: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * The object that defines the structure of an Azure Data Factory response.
 *
 * @member {string} code Error code.
 * @member {string} message Error message.
 * @member {string} [target] Property name/path in request associated with
 * error.
 * @member {array} [details] Array with additional error details.
 */
export interface ErrorResponse {
  code: string;
  message: string;
  target?: string;
  details?: ErrorResponse[];
}

/**
 * @class
 * Initializes a new instance of the FactoryUpdateParameters class.
 * @constructor
 * Parameters for updating a factory resource.
 *
 * @member {object} [tags] The resource tags.
 * @member {object} [identity] Managed service identity of the factory.
 * @member {uuid} [identity.principalId] The principal id of the identity.
 * @member {uuid} [identity.tenantId] The client tenant id of the identity.
 */
export interface FactoryUpdateParameters {
  tags?: { [propertyName: string]: string };
  identity?: FactoryIdentity;
}

/**
 * @class
 * Initializes a new instance of the DatasetReference class.
 * @constructor
 * Dataset reference type.
 *
 * @member {string} referenceName Reference dataset name.
 * @member {object} [parameters] Arguments for dataset.
 */
export interface DatasetReference {
  referenceName: string;
  parameters?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the PipelineRunQueryFilter class.
 * @constructor
 * Query filter option for listing pipeline runs.
 *
 * @member {string} operand Parameter name to be used for filter. Possible
 * values include: 'PipelineName', 'Status', 'RunStart', 'RunEnd'
 * @member {string} operator Operator to be used for filter. Possible values
 * include: 'Equals', 'NotEquals', 'In', 'NotIn'
 * @member {array} values List of filter values.
 */
export interface PipelineRunQueryFilter {
  operand: string;
  operator: string;
  values: string[];
}

/**
 * @class
 * Initializes a new instance of the PipelineRunQueryOrderBy class.
 * @constructor
 * An object to provide order by options for listing pipeline runs.
 *
 * @member {string} orderBy Parameter name to be used for order by. Possible
 * values include: 'RunStart', 'RunEnd'
 * @member {string} order Sorting order of the parameter. Possible values
 * include: 'ASC', 'DESC'
 */
export interface PipelineRunQueryOrderBy {
  orderBy: string;
  order: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineRunFilterParameters class.
 * @constructor
 * Query parameters for listing pipeline runs.
 *
 * @member {string} [continuationToken] The continuation token for getting the
 * next page of results. Null for first page.
 * @member {date} lastUpdatedAfter The time at or after which the pipeline run
 * event was updated in 'ISO 8601' format.
 * @member {date} lastUpdatedBefore The time at or before which the pipeline
 * run event was updated in 'ISO 8601' format.
 * @member {array} [filters] List of filters.
 * @member {array} [orderBy] List of OrderBy option.
 */
export interface PipelineRunFilterParameters {
  continuationToken?: string;
  lastUpdatedAfter: Date;
  lastUpdatedBefore: Date;
  filters?: PipelineRunQueryFilter[];
  orderBy?: PipelineRunQueryOrderBy[];
}

/**
 * @class
 * Initializes a new instance of the PipelineRunInvokedBy class.
 * @constructor
 * Provides entity name and id that started the pipeline run.
 *
 * @member {string} [name] Name of the entity that started the pipeline run.
 * @member {string} [id] The ID of the entity that started the run.
 */
export interface PipelineRunInvokedBy {
  readonly name?: string;
  readonly id?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineRun class.
 * @constructor
 * Information about a pipeline run.
 *
 * @member {string} [runId] Identifier of a run.
 * @member {string} [pipelineName] The pipeline name.
 * @member {object} [parameters] The full or partial list of parameter name,
 * value pair used in the pipeline run.
 * @member {object} [invokedBy] Entity that started the pipeline run.
 * @member {string} [invokedBy.name] Name of the entity that started the
 * pipeline run.
 * @member {string} [invokedBy.id] The ID of the entity that started the run.
 * @member {date} [lastUpdated] The last updated timestamp for the pipeline run
 * event in ISO8601 format.
 * @member {date} [runStart] The start time of a pipeline run in ISO8601
 * format.
 * @member {date} [runEnd] The end time of a pipeline run in ISO8601 format.
 * @member {number} [durationInMs] The duration of a pipeline run.
 * @member {string} [status] The status of a pipeline run.
 * @member {string} [message] The message from a pipeline run.
 */
export interface PipelineRun {
  readonly runId?: string;
  readonly pipelineName?: string;
  readonly parameters?: { [propertyName: string]: string };
  readonly invokedBy?: PipelineRunInvokedBy;
  readonly lastUpdated?: Date;
  readonly runStart?: Date;
  readonly runEnd?: Date;
  readonly durationInMs?: number;
  readonly status?: string;
  readonly message?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineRunQueryResponse class.
 * @constructor
 * A list pipeline runs.
 *
 * @member {array} value List of pipeline runs.
 * @member {string} [continuationToken] The continuation token for getting the
 * next page of results, if any remaining results exist, null otherwise.
 */
export interface PipelineRunQueryResponse {
  value: PipelineRun[];
  continuationToken?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityRun class.
 * @constructor
 * Information about an activity run in a pipeline.
 *
 * @member {string} [pipelineName] The name of the pipeline.
 * @member {string} [pipelineRunId] The id of the pipeline run.
 * @member {string} [activityName] The name of the activity.
 * @member {string} [activityType] The type of the activity.
 * @member {string} [activityRunId] The id of the activity run.
 * @member {string} [linkedServiceName] The name of the compute linked service.
 * @member {string} [status] The status of the activity run.
 * @member {date} [activityRunStart] The start time of the activity run in 'ISO
 * 8601' format.
 * @member {date} [activityRunEnd] The end time of the activity run in 'ISO
 * 8601' format.
 * @member {number} [durationInMs] The duration of the activity run.
 * @member {object} [input] The input for the activity.
 * @member {object} [output] The output for the activity.
 * @member {object} [error] The error if any from the activity run.
 */
export interface ActivityRun {
  readonly pipelineName?: string;
  readonly pipelineRunId?: string;
  readonly activityName?: string;
  readonly activityType?: string;
  readonly activityRunId?: string;
  readonly linkedServiceName?: string;
  readonly status?: string;
  readonly activityRunStart?: Date;
  readonly activityRunEnd?: Date;
  readonly durationInMs?: number;
  readonly input?: any;
  readonly output?: any;
  readonly error?: any;
}

/**
 * @class
 * Initializes a new instance of the TriggerRun class.
 * @constructor
 * Trigger runs.
 *
 * @member {string} [triggerRunId] Trigger run id.
 * @member {string} [triggerName] Trigger name.
 * @member {string} [triggerType] Trigger type.
 * @member {date} [triggerRunTimestamp] Trigger run start time.
 * @member {string} [status] Trigger run status. Possible values include:
 * 'Succeeded', 'Failed', 'Inprogress'
 * @member {string} [message] Trigger error message.
 * @member {object} [properties] List of property name and value related to
 * trigger run. Name, value pair depends on type of trigger.
 * @member {object} [triggeredPipelines] List of pipeline name and run Id
 * triggered by the trigger run.
 */
export interface TriggerRun {
  readonly triggerRunId?: string;
  readonly triggerName?: string;
  readonly triggerType?: string;
  readonly triggerRunTimestamp?: Date;
  readonly status?: string;
  readonly message?: string;
  readonly properties?: { [propertyName: string]: string };
  readonly triggeredPipelines?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Metadata associated with the operation.
 *
 * @member {string} [description] The description of the operation.
 * @member {string} [provider] The name of the provider.
 * @member {string} [resource] The name of the resource type on which the
 * operation is performed.
 * @member {string} [operation] The type of operation: get, read, delete, etc.
 */
export interface OperationDisplay {
  description?: string;
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationLogSpecification class.
 * @constructor
 * Details about an operation related to logs.
 *
 * @member {string} [name] The name of the log category.
 * @member {string} [displayName] Localized display name.
 * @member {string} [blobDuration] Blobs created in the customer storage
 * account, per hour.
 */
export interface OperationLogSpecification {
  name?: string;
  displayName?: string;
  blobDuration?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationMetricAvailability class.
 * @constructor
 * Defines how often data for a metric becomes available.
 *
 * @member {string} [timeGrain] The granularity for the metric.
 * @member {string} [blobDuration] Blob created in the customer storage
 * account, per hour.
 */
export interface OperationMetricAvailability {
  timeGrain?: string;
  blobDuration?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationMetricSpecification class.
 * @constructor
 * Details about an operation related to metrics.
 *
 * @member {string} [name] The name of the metric.
 * @member {string} [displayName] Localized display name of the metric.
 * @member {string} [displayDescription] The description of the metric.
 * @member {string} [unit] The unit that the metric is measured in.
 * @member {string} [aggregationType] The type of metric aggregation.
 * @member {string} [enableRegionalMdmAccount] Whether or not the service is
 * using regional MDM accounts.
 * @member {string} [sourceMdmAccount] The name of the MDM account.
 * @member {string} [sourceMdmNamespace] The name of the MDM namespace.
 * @member {array} [availabilities] Defines how often data for metrics becomes
 * available.
 */
export interface OperationMetricSpecification {
  name?: string;
  displayName?: string;
  displayDescription?: string;
  unit?: string;
  aggregationType?: string;
  enableRegionalMdmAccount?: string;
  sourceMdmAccount?: string;
  sourceMdmNamespace?: string;
  availabilities?: OperationMetricAvailability[];
}

/**
 * @class
 * Initializes a new instance of the OperationServiceSpecification class.
 * @constructor
 * Details about a service operation.
 *
 * @member {array} [logSpecifications] Details about operations related to
 * logs.
 * @member {array} [metricSpecifications] Details about operations related to
 * metrics.
 */
export interface OperationServiceSpecification {
  logSpecifications?: OperationLogSpecification[];
  metricSpecifications?: OperationMetricSpecification[];
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Azure Data Factory API operation definition.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {string} [origin] The intended executor of the operation.
 * @member {object} [display] Metadata associated with the operation.
 * @member {string} [display.description] The description of the operation.
 * @member {string} [display.provider] The name of the provider.
 * @member {string} [display.resource] The name of the resource type on which
 * the operation is performed.
 * @member {string} [display.operation] The type of operation: get, read,
 * delete, etc.
 * @member {object} [serviceSpecification] Details about a service operation.
 * @member {array} [serviceSpecification.logSpecifications] Details about
 * operations related to logs.
 * @member {array} [serviceSpecification.metricSpecifications] Details about
 * operations related to metrics.
 */
export interface Operation {
  name?: string;
  origin?: string;
  display?: OperationDisplay;
  serviceSpecification?: OperationServiceSpecification;
}

/**
 * @class
 * Initializes a new instance of the OperationListResponse class.
 * @constructor
 * A list of operations that can be performed by the Data Factory service.
 *
 * @member {array} [value] List of Data Factory operations supported by the
 * Data Factory resource provider.
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface OperationListResponse {
  value?: Operation[];
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeAnalyticsLinkedService class.
 * @constructor
 * Azure Data Lake Analytics linked service.
 *
 * @member {object} accountName The Azure Data Lake Analytics account name.
 * Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The ID of the application used to
 * authenticate against the Azure Data Lake Analytics account. Type: string (or
 * Expression with resultType string).
 * @member {object} [servicePrincipalKey] The Key of the application used to
 * authenticate against the Azure Data Lake Analytics account.
 * @member {string} [servicePrincipalKey.value] Value of secure string.
 * @member {object} tenant The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [subscriptionId] Data Lake Analytics account subscription
 * ID (if different from Data Factory account). Type: string (or Expression
 * with resultType string).
 * @member {object} [resourceGroupName] Data Lake Analytics account resource
 * group name (if different from Data Factory account). Type: string (or
 * Expression with resultType string).
 * @member {object} [dataLakeAnalyticsUri] Azure Data Lake Analytics URI Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureDataLakeAnalyticsLinkedService extends LinkedService {
  accountName: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecureString;
  tenant: any;
  subscriptionId?: any;
  resourceGroupName?: any;
  dataLakeAnalyticsUri?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HDInsightOnDemandLinkedService class.
 * @constructor
 * HDInsight ondemand linked service.
 *
 * @member {object} clusterSize Number of worker/data nodes in the cluster.
 * Suggestion value: 4. Type: string (or Expression with resultType string).
 * @member {object} timeToLive The allowed idle time for the on-demand
 * HDInsight cluster. Specifies how long the on-demand HDInsight cluster stays
 * alive after completion of an activity run if there are no other active jobs
 * in the cluster. The minimum value is 5 mins. Type: string (or Expression
 * with resultType string).
 * @member {object} version Version of the HDInsight cluster.  Type: string (or
 * Expression with resultType string).
 * @member {object} linkedServiceName Azure Storage linked service to be used
 * by the on-demand cluster for storing and processing data.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} hostSubscriptionId The customer’s subscription to host the
 * cluster. Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The service principal id for the
 * hostSubscriptionId. Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalKey] The key for the service principal id.
 * @member {string} [servicePrincipalKey.value] Value of secure string.
 * @member {object} tenant The Tenant id/name to which the service principal
 * belongs. Type: string (or Expression with resultType string).
 * @member {object} clusterResourceGroup The resource group where the cluster
 * belongs. Type: string (or Expression with resultType string).
 * @member {object} [clusterNamePrefix] The prefix of cluster name, postfix
 * will be distinct with timestamp. Type: string (or Expression with resultType
 * string).
 * @member {object} [clusterUserName] The username to access the cluster. Type:
 * string (or Expression with resultType string).
 * @member {object} [clusterPassword] The password to access the cluster.
 * @member {string} [clusterPassword.value] Value of secure string.
 * @member {object} [clusterSshUserName] The username to SSH remotely connect
 * to cluster’s node (for Linux). Type: string (or Expression with resultType
 * string).
 * @member {object} [clusterSshPassword] The password to SSH remotely connect
 * cluster’s node (for Linux).
 * @member {string} [clusterSshPassword.value] Value of secure string.
 * @member {array} [additionalLinkedServiceNames] Specifies additional storage
 * accounts for the HDInsight linked service so that the Data Factory service
 * can register them on your behalf.
 * @member {object} [hcatalogLinkedServiceName] The name of Azure SQL linked
 * service that point to the HCatalog database. The on-demand HDInsight cluster
 * is created by using the Azure SQL database as the metastore.
 * @member {string} [hcatalogLinkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [clusterType] The cluster type. Type: string (or Expression
 * with resultType string).
 * @member {object} [sparkVersion] The version of spark if the cluster type is
 * 'spark'. Type: string (or Expression with resultType string).
 * @member {object} [coreConfiguration] Specifies the core configuration
 * parameters (as in core-site.xml) for the HDInsight cluster to be created.
 * @member {object} [hBaseConfiguration] Specifies the HBase configuration
 * parameters (hbase-site.xml) for the HDInsight cluster.
 * @member {object} [hdfsConfiguration] Specifies the HDFS configuration
 * parameters (hdfs-site.xml) for the HDInsight cluster.
 * @member {object} [hiveConfiguration] Specifies the hive configuration
 * parameters (hive-site.xml) for the HDInsight cluster.
 * @member {object} [mapReduceConfiguration] Specifies the MapReduce
 * configuration parameters (mapred-site.xml) for the HDInsight cluster.
 * @member {object} [oozieConfiguration] Specifies the Oozie configuration
 * parameters (oozie-site.xml) for the HDInsight cluster.
 * @member {object} [stormConfiguration] Specifies the Storm configuration
 * parameters (storm-site.xml) for the HDInsight cluster.
 * @member {object} [yarnConfiguration] Specifies the Yarn configuration
 * parameters (yarn-site.xml) for the HDInsight cluster.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface HDInsightOnDemandLinkedService extends LinkedService {
  clusterSize: any;
  timeToLive: any;
  version: any;
  linkedServiceName: LinkedServiceReference;
  hostSubscriptionId: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecureString;
  tenant: any;
  clusterResourceGroup: any;
  clusterNamePrefix?: any;
  clusterUserName?: any;
  clusterPassword?: SecureString;
  clusterSshUserName?: any;
  clusterSshPassword?: SecureString;
  additionalLinkedServiceNames?: LinkedServiceReference[];
  hcatalogLinkedServiceName?: LinkedServiceReference;
  clusterType?: any;
  sparkVersion?: any;
  coreConfiguration?: any;
  hBaseConfiguration?: any;
  hdfsConfiguration?: any;
  hiveConfiguration?: any;
  mapReduceConfiguration?: any;
  oozieConfiguration?: any;
  stormConfiguration?: any;
  yarnConfiguration?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SapHanaLinkedService class.
 * @constructor
 * SAP HANA Linked Service.
 *
 * @member {object} server Host name of the SAP HANA server. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the SAP HANA server. Possible values include: 'Basic', 'Windows'
 * @member {object} [userName] Username to access the SAP HANA server. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password to access the SAP HANA server.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SapHanaLinkedService extends LinkedService {
  server: any;
  authenticationType?: string;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SapBWLinkedService class.
 * @constructor
 * SAP Business Warehouse Linked Service.
 *
 * @member {object} server Host name of the SAP BW instance. Type: string (or
 * Expression with resultType string).
 * @member {object} systemNumber System number of the BW system. (Usually a
 * two-digit decimal number represented as a string.) Type: string (or
 * Expression with resultType string).
 * @member {object} clientId Client ID of the client on the BW system. (Usually
 * a three-digit decimal number represented as a string) Type: string (or
 * Expression with resultType string).
 * @member {object} [userName] Username to access the SAP BW server. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password to access the SAP BW server.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SapBWLinkedService extends LinkedService {
  server: any;
  systemNumber: any;
  clientId: any;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SftpServerLinkedService class.
 * @constructor
 * A linked service for an SSH File Transfer Protocol (SFTP) server.
 *
 * @member {object} host The SFTP server host name. Type: string (or Expression
 * with resultType string).
 * @member {object} [port] The TCP port number that the SFTP server uses to
 * listen for client connections. Default value is 22. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the FTP server. Possible values include: 'Basic', 'SshPublicKey'
 * @member {object} [userName] The username used to log on to the SFTP server.
 * Type: string (or Expression with resultType string).
 * @member {object} [password] Password to logon the SFTP server for Basic
 * authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [privateKeyPath] The SSH private key file path for
 * SshPublicKey authentication. Only valid for on-premises copy. For
 * on-premises copy with SshPublicKey authentication, either PrivateKeyPath or
 * PrivateKeyContent should be specified. SSH private key should be OpenSSH
 * format. Type: string (or Expression with resultType string).
 * @member {object} [privateKeyContent] Base64 encoded SSH private key content
 * for SshPublicKey authentication. For on-premises copy with SshPublicKey
 * authentication, either PrivateKeyPath or PrivateKeyContent should be
 * specified. SSH private key should be OpenSSH format.
 * @member {string} [privateKeyContent.value] Value of secure string.
 * @member {object} [passPhrase] The password to decrypt the SSH private key if
 * the SSH private key is encrypted.
 * @member {string} [passPhrase.value] Value of secure string.
 * @member {object} [skipHostKeyValidation] If true, skip the SSH host key
 * validation. Default value is false. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [hostKeyFingerprint] The host key finger-print of the SFTP
 * server. When SkipHostKeyValidation is false, HostKeyFingerprint should be
 * specified. Type: string (or Expression with resultType string).
 */
export interface SftpServerLinkedService extends LinkedService {
  host: any;
  port?: any;
  authenticationType?: string;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
  privateKeyPath?: any;
  privateKeyContent?: SecureString;
  passPhrase?: SecureString;
  skipHostKeyValidation?: any;
  hostKeyFingerprint?: any;
}

/**
 * @class
 * Initializes a new instance of the FtpServerLinkedService class.
 * @constructor
 * A FTP server Linked Service.
 *
 * @member {object} host Host name of the FTP server. Type: string (or
 * Expression with resultType string).
 * @member {object} [port] The TCP port number that the FTP server uses to
 * listen for client connections. Default value is 21. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the FTP server. Possible values include: 'Basic', 'Anonymous'
 * @member {object} [userName] Username to logon the FTP server. Type: string
 * (or Expression with resultType string).
 * @member {object} [password] Password to logon the FTP server.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [enableSsl] If true, connect to the FTP server over SSL/TLS
 * channel. Default value is true. Type: boolean (or Expression with resultType
 * boolean).
 * @member {object} [enableServerCertificateValidation] If true, validate the
 * FTP server SSL certificate when connect over SSL/TLS channel. Default value
 * is true. Type: boolean (or Expression with resultType boolean).
 */
export interface FtpServerLinkedService extends LinkedService {
  host: any;
  port?: any;
  authenticationType?: string;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
  enableSsl?: any;
  enableServerCertificateValidation?: any;
}

/**
 * @class
 * Initializes a new instance of the HttpLinkedService class.
 * @constructor
 * Linked service for an HTTP source.
 *
 * @member {object} url The base URL of the HTTP endpoint, e.g.
 * http://www.microsoft.com. Type: string (or Expression with resultType
 * string).
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the HTTP server. Possible values include: 'Basic', 'Anonymous',
 * 'Digest', 'Windows', 'ClientCertificate'
 * @member {object} [userName] User name for Basic, Digest, or Windows
 * authentication. Type: string (or Expression with resultType string).
 * @member {object} [password] Password for Basic, Digest, Windows, or
 * ClientCertificate with EmbeddedCertData authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [embeddedCertData] Base64 encoded certificate data for
 * ClientCertificate authentication. For on-premises copy with
 * ClientCertificate authentication, either CertThumbprint or
 * EmbeddedCertData/Password should be specified. Type: string (or Expression
 * with resultType string).
 * @member {object} [certThumbprint] Thumbprint of certificate for
 * ClientCertificate authentication. Only valid for on-premises copy. For
 * on-premises copy with ClientCertificate authentication, either
 * CertThumbprint or EmbeddedCertData/Password should be specified. Type:
 * string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [enableServerCertificateValidation] If true, validate the
 * HTTPS server SSL certificate. Default value is true. Type: boolean (or
 * Expression with resultType boolean).
 */
export interface HttpLinkedService extends LinkedService {
  url: any;
  authenticationType?: string;
  userName?: any;
  password?: SecureString;
  embeddedCertData?: any;
  certThumbprint?: any;
  encryptedCredential?: any;
  enableServerCertificateValidation?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSearchLinkedService class.
 * @constructor
 * Linked service for Windows Azure Search Service.
 *
 * @member {object} url URL for Azure Search service. Type: string (or
 * Expression with resultType string).
 * @member {object} [key] Admin Key for Azure Search service
 * @member {string} [key.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureSearchLinkedService extends LinkedService {
  url: any;
  key?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the CustomDataSourceLinkedService class.
 * @constructor
 * Custom linked service.
 *
 * @member {object} typeProperties Custom linked service properties.
 */
export interface CustomDataSourceLinkedService extends LinkedService {
  typeProperties: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonRedshiftLinkedService class.
 * @constructor
 * Linked service for Amazon Redshift.
 *
 * @member {object} server The name of the Amazon Redshift server. Type: string
 * (or Expression with resultType string).
 * @member {object} [username] The username of the Amazon Redshift source.
 * Type: string (or Expression with resultType string).
 * @member {object} [password] The password of the Amazon Redshift source.
 * @member {string} [password.value] Value of secure string.
 * @member {object} database The database name of the Amazon Redshift source.
 * Type: string (or Expression with resultType string).
 * @member {object} [port] The TCP port number that the Amazon Redshift server
 * uses to listen for client connections. The default value is 5439. Type:
 * integer (or Expression with resultType integer).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AmazonRedshiftLinkedService extends LinkedService {
  server: any;
  username?: any;
  password?: SecureString;
  database: any;
  port?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AmazonS3LinkedService class.
 * @constructor
 * Linked service for Amazon S3.
 *
 * @member {object} [accessKeyId] The access key identifier of the Amazon S3
 * Identity and Access Management (IAM) user. Type: string (or Expression with
 * resultType string).
 * @member {object} [secretAccessKey] The secret access key of the Amazon S3
 * Identity and Access Management (IAM) user.
 * @member {string} [secretAccessKey.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AmazonS3LinkedService extends LinkedService {
  accessKeyId?: any;
  secretAccessKey?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SalesforceLinkedService class.
 * @constructor
 * Linked service for Salesforce.
 *
 * @member {object} [environmentUrl] The URL of Salesforce instance. Default is
 * 'https://login.salesforce.com'. To copy data from sandbox, specify
 * 'https://test.salesforce.com'. To copy data from custom domain, specify, for
 * example, 'https://[domain].my.salesforce.com'. Type: string (or Expression
 * with resultType string).
 * @member {object} [username] The username for Basic authentication of the
 * Salesforce source. Type: string (or Expression with resultType string).
 * @member {object} [password] The password for Basic authentication of the
 * Salesforce source.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [securityToken] The security token is required to remotely
 * access Salesforce source.
 * @member {string} [securityToken.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SalesforceLinkedService extends LinkedService {
  environmentUrl?: any;
  username?: any;
  password?: SecureString;
  securityToken?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreLinkedService class.
 * @constructor
 * Azure Data Lake Store linked service.
 *
 * @member {object} dataLakeStoreUri Data Lake Store service URI. Type: string
 * (or Expression with resultType string).
 * @member {object} [servicePrincipalId] The ID of the application used to
 * authenticate against the Azure Data Lake Store account. Type: string (or
 * Expression with resultType string).
 * @member {object} [servicePrincipalKey] The Key of the application used to
 * authenticate against the Azure Data Lake Store account.
 * @member {string} [servicePrincipalKey.value] Value of secure string.
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [accountName] Data Lake Store account name. Type: string
 * (or Expression with resultType string).
 * @member {object} [subscriptionId] Data Lake Store account subscription ID
 * (if different from Data Factory account). Type: string (or Expression with
 * resultType string).
 * @member {object} [resourceGroupName] Data Lake Store account resource group
 * name (if different from Data Factory account). Type: string (or Expression
 * with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureDataLakeStoreLinkedService extends LinkedService {
  dataLakeStoreUri: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecureString;
  tenant?: any;
  accountName?: any;
  subscriptionId?: any;
  resourceGroupName?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MongoDbLinkedService class.
 * @constructor
 * Linked service for MongoDb data source.
 *
 * @member {object} server The IP address or server name of the MongoDB server.
 * Type: string (or Expression with resultType string).
 * @member {string} [authenticationType] The authentication type to be used to
 * connect to the MongoDB database. Possible values include: 'Basic',
 * 'Anonymous'
 * @member {object} databaseName The name of the MongoDB database that you want
 * to access. Type: string (or Expression with resultType string).
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [authSource] Database to verify the username and password.
 * Type: string (or Expression with resultType string).
 * @member {object} [port] The TCP port number that the MongoDB server uses to
 * listen for client connections. The default value is 27017. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MongoDbLinkedService extends LinkedService {
  server: any;
  authenticationType?: string;
  databaseName: any;
  username?: any;
  password?: SecureString;
  authSource?: any;
  port?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the CassandraLinkedService class.
 * @constructor
 * Linked service for Cassandra data source.
 *
 * @member {object} host Host name for connection. Type: string (or Expression
 * with resultType string).
 * @member {object} [authenticationType] AuthenticationType to be used for
 * connection. Type: string (or Expression with resultType string).
 * @member {object} [port] The port for the connection. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface CassandraLinkedService extends LinkedService {
  host: any;
  authenticationType?: any;
  port?: any;
  username?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the WebLinkedServiceTypeProperties class.
 * @constructor
 * Base definition of WebLinkedServiceTypeProperties, this typeProperties is
 * polymorphic based on authenticationType, so not flattened in SDK models.
 *
 * @member {object} url The URL of the web service endpoint, e.g.
 * http://www.microsoft.com . Type: string (or Expression with resultType
 * string).
 * @member {string} authenticationType Polymorphic Discriminator
 */
export interface WebLinkedServiceTypeProperties {
  url: any;
  authenticationType: string;
}

/**
 * @class
 * Initializes a new instance of the WebClientCertificateAuthentication class.
 * @constructor
 * A WebLinkedService that uses client certificate based authentication to
 * communicate with an HTTP endpoint. This scheme follows mutual
 * authentication; the server must also provide valid credentials to the
 * client.
 *
 * @member {object} pfx Base64-encoded contents of a PFX file.
 * @member {string} [pfx.value] Value of secure string.
 * @member {object} password Password for the PFX file.
 * @member {string} [password.value] Value of secure string.
 */
export interface WebClientCertificateAuthentication extends WebLinkedServiceTypeProperties {
  pfx: SecureString;
  password: SecureString;
}

/**
 * @class
 * Initializes a new instance of the WebBasicAuthentication class.
 * @constructor
 * A WebLinkedService that uses basic authentication to communicate with an
 * HTTP endpoint.
 *
 * @member {object} username User name for Basic authentication. Type: string
 * (or Expression with resultType string).
 * @member {object} password The password for Basic authentication.
 * @member {string} [password.value] Value of secure string.
 */
export interface WebBasicAuthentication extends WebLinkedServiceTypeProperties {
  username: any;
  password: SecureString;
}

/**
 * @class
 * Initializes a new instance of the WebAnonymousAuthentication class.
 * @constructor
 * A WebLinkedService that uses anonymous authentication to communicate with an
 * HTTP endpoint.
 *
 */
export interface WebAnonymousAuthentication extends WebLinkedServiceTypeProperties {
}

/**
 * @class
 * Initializes a new instance of the WebLinkedService class.
 * @constructor
 * Web linked service.
 *
 * @member {object} typeProperties Web linked service properties.
 * @member {object} [typeProperties.url] The URL of the web service endpoint,
 * e.g. http://www.microsoft.com . Type: string (or Expression with resultType
 * string).
 * @member {string} [typeProperties.authenticationType] Polymorphic
 * Discriminator
 */
export interface WebLinkedService extends LinkedService {
  typeProperties: WebLinkedServiceTypeProperties;
}

/**
 * @class
 * Initializes a new instance of the ODataLinkedService class.
 * @constructor
 * Open Data Protocol (OData) linked service.
 *
 * @member {object} url The URL of the OData service endpoint. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] Type of authentication used to connect
 * to the OData service. Possible values include: 'Basic', 'Anonymous'
 * @member {object} [userName] User name of the OData service. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password of the OData service.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface ODataLinkedService extends LinkedService {
  url: any;
  authenticationType?: string;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HdfsLinkedService class.
 * @constructor
 * Hadoop Distributed File System (HDFS) linked service.
 *
 * @member {object} url The URL of the HDFS service endpoint, e.g.
 * http://myhostname:50070/webhdfs/v1 . Type: string (or Expression with
 * resultType string).
 * @member {object} [authenticationType] Type of authentication used to connect
 * to the HDFS. Possible values are: Anonymous and Windows. Type: string (or
 * Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 * @member {object} [userName] User name for Windows authentication. Type:
 * string (or Expression with resultType string).
 * @member {object} [password] Password for Windows authentication.
 * @member {string} [password.value] Value of secure string.
 */
export interface HdfsLinkedService extends LinkedService {
  url: any;
  authenticationType?: any;
  encryptedCredential?: any;
  userName?: any;
  password?: SecureString;
}

/**
 * @class
 * Initializes a new instance of the OdbcLinkedService class.
 * @constructor
 * Open Database Connectivity (ODBC) linked service.
 *
 * @member {object} connectionString The non-access credential portion of the
 * connection string as well as an optional encrypted credential.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [authenticationType] Type of authentication used to connect
 * to the ODBC data store. Possible values are: Anonymous and Basic. Type:
 * string (or Expression with resultType string).
 * @member {object} [credential] The access credential portion of the
 * connection string specified in driver-specific property-value format.
 * @member {string} [credential.value] Value of secure string.
 * @member {object} [userName] User name for Basic authentication. Type: string
 * (or Expression with resultType string).
 * @member {object} [password] Password for Basic authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface OdbcLinkedService extends LinkedService {
  connectionString: SecureString;
  authenticationType?: any;
  credential?: SecureString;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMLLinkedService class.
 * @constructor
 * Azure ML Web Service linked service.
 *
 * @member {object} mlEndpoint The Batch Execution REST URL for an Azure ML Web
 * Service endpoint. Type: string (or Expression with resultType string).
 * @member {object} apiKey The API key for accessing the Azure ML model
 * endpoint.
 * @member {string} [apiKey.value] Value of secure string.
 * @member {object} [updateResourceEndpoint] The Update Resource REST URL for
 * an Azure ML Web Service endpoint. Type: string (or Expression with
 * resultType string).
 * @member {object} [servicePrincipalId] The ID of the service principal used
 * to authenticate against the ARM-based updateResourceEndpoint of an Azure ML
 * web service. Type: string (or Expression with resultType string).
 * @member {object} [servicePrincipalKey] The key of the service principal used
 * to authenticate against the ARM-based updateResourceEndpoint of an Azure ML
 * web service.
 * @member {string} [servicePrincipalKey.value] Value of secure string.
 * @member {object} [tenant] The name or ID of the tenant to which the service
 * principal belongs. Type: string (or Expression with resultType string).
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureMLLinkedService extends LinkedService {
  mlEndpoint: any;
  apiKey: SecureString;
  updateResourceEndpoint?: any;
  servicePrincipalId?: any;
  servicePrincipalKey?: SecureString;
  tenant?: any;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the TeradataLinkedService class.
 * @constructor
 * Linked service for Teradata data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [schema] Schema name for connection. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] AuthenticationType to be used for
 * connection. Possible values include: 'Basic', 'Windows'
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface TeradataLinkedService extends LinkedService {
  server: any;
  schema?: any;
  authenticationType?: string;
  username?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the Db2LinkedService class.
 * @constructor
 * Linked service for DB2 data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} database Database name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [schema] Schema name for connection. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] AuthenticationType to be used for
 * connection. Possible values include: 'Basic'
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface Db2LinkedService extends LinkedService {
  server: any;
  database: any;
  schema?: any;
  authenticationType?: string;
  username?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SybaseLinkedService class.
 * @constructor
 * Linked service for Sybase data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} database Database name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [schema] Schema name for connection. Type: string (or
 * Expression with resultType string).
 * @member {string} [authenticationType] AuthenticationType to be used for
 * connection. Possible values include: 'Basic', 'Windows'
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SybaseLinkedService extends LinkedService {
  server: any;
  database: any;
  schema?: any;
  authenticationType?: string;
  username?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the PostgreSqlLinkedService class.
 * @constructor
 * Linked service for PostgreSQL data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} database Database name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [schema] Schema name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface PostgreSqlLinkedService extends LinkedService {
  server: any;
  database: any;
  schema?: any;
  username?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the MySqlLinkedService class.
 * @constructor
 * Linked service for MySQL data source.
 *
 * @member {object} server Server name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} database Database name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [schema] Schema name for connection. Type: string (or
 * Expression with resultType string).
 * @member {object} [username] Username for authentication. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password for authentication.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface MySqlLinkedService extends LinkedService {
  server: any;
  database: any;
  schema?: any;
  username?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the OracleLinkedService class.
 * @constructor
 * Oracle database.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface OracleLinkedService extends LinkedService {
  connectionString: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the FileServerLinkedService class.
 * @constructor
 * File system linked service.
 *
 * @member {object} host Host name of the server. Type: string (or Expression
 * with resultType string).
 * @member {object} [userId] User ID to logon the server. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] Password to logon the server.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface FileServerLinkedService extends LinkedService {
  host: any;
  userId?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the HDInsightLinkedService class.
 * @constructor
 * HDInsight linked service.
 *
 * @member {object} clusterUri HDInsight cluster URI. Type: string (or
 * Expression with resultType string).
 * @member {object} [userName] HDInsight cluster user name. Type: string (or
 * Expression with resultType string).
 * @member {object} [password] HDInsight cluster password.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [linkedServiceName] The Azure Storage linked service
 * reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [hcatalogLinkedServiceName] A reference to the Azure SQL
 * linked service that points to the HCatalog database.
 * @member {string} [hcatalogLinkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface HDInsightLinkedService extends LinkedService {
  clusterUri: any;
  userName?: any;
  password?: SecureString;
  linkedServiceName?: LinkedServiceReference;
  hcatalogLinkedServiceName?: LinkedServiceReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureKeyVaultReference class.
 * @constructor
 * A reference to an object in Azure Key Vault.
 *
 * @member {object} store The Azure Key Vault LinkedService.
 * @member {string} [store.referenceName] Reference LinkedService name.
 * @member {string} type Polymorphic Discriminator
 */
export interface AzureKeyVaultReference {
  store: LinkedServiceReference;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the AzureKeyVaultSecretReference class.
 * @constructor
 * Azure Key Vault Secret properties.
 *
 * @member {object} secretName The name of secret in Azure Key Vault. Type:
 * string (or Expression with resultType string).
 * @member {object} [secretVersion] The version of the secret in Azure Key
 * Vault. The default value is the latest version of the secret. Type: string
 * (or Expression with resultType string).
 */
export interface AzureKeyVaultSecretReference extends AzureKeyVaultReference {
  secretName: any;
  secretVersion?: any;
}

/**
 * @class
 * Initializes a new instance of the DynamicsLinkedService class.
 * @constructor
 * Dynamics linked service.
 *
 * @member {object} deploymentType The deployment type of the Dynamics
 * instance. 'Online' for Dynamics Online and 'OnPremisesWithIfd' for Dynamics
 * on-premises with Ifd. Type: string (or Expression with resultType string).
 * @member {object} [hostName] The host name of the on-premises Dynamics
 * server. The property is required for on-prem and not allowed for online.
 * Type: string (or Expression with resultType string).
 * @member {object} [port] The port of on-premises Dynamics server. The
 * property is required for on-prem and not allowed for online. Default is 443.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {object} [organizationName] The organization name of the Dynamics
 * instance. The property is required for on-prem and required for online when
 * there are more than one Dynamics instances associated with the user. Type:
 * string (or Expression with resultType string).
 * @member {object} authenticationType The authentication type to connect to
 * Dynamics server. 'Office365' for online scenario, 'Ifd' for on-premises with
 * Ifd scenario. Type: string (or Expression with resultType string).
 * @member {object} username User name to access the Dynamics instance. Type:
 * string (or Expression with resultType string).
 * @member {object} password Password to access the Dynamics instance.
 * @member {object} [password.secretName] The name of secret in Azure Key
 * Vault. Type: string (or Expression with resultType string).
 * @member {object} [password.secretVersion] The version of the secret in Azure
 * Key Vault. The default value is the latest version of the secret. Type:
 * string (or Expression with resultType string).
 */
export interface DynamicsLinkedService extends LinkedService {
  deploymentType: any;
  hostName?: any;
  port?: any;
  organizationName?: any;
  authenticationType: any;
  username: any;
  password: AzureKeyVaultSecretReference;
}

/**
 * @class
 * Initializes a new instance of the CosmosDbLinkedService class.
 * @constructor
 * Microsoft Azure Cosmos Database (CosmosDB) linked service.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface CosmosDbLinkedService extends LinkedService {
  connectionString: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureKeyVaultLinkedService class.
 * @constructor
 * Azure Key Vault linked service.
 *
 * @member {object} baseUrl The base URL of the Azure Key Vault. e.g.
 * https://myakv.vault.azure.net Type: string (or Expression with resultType
 * string).
 */
export interface AzureKeyVaultLinkedService extends LinkedService {
  baseUrl: any;
}

/**
 * @class
 * Initializes a new instance of the AzureBatchLinkedService class.
 * @constructor
 * Azure Batch linked service.
 *
 * @member {object} accountName The Azure Batch account name. Type: string (or
 * Expression with resultType string).
 * @member {object} [accessKey] The Azure Batch account access key.
 * @member {string} [accessKey.value] Value of secure string.
 * @member {object} batchUri The Azure Batch URI. Type: string (or Expression
 * with resultType string).
 * @member {object} poolName The Azure Batch pool name. Type: string (or
 * Expression with resultType string).
 * @member {object} linkedServiceName The Azure Storage linked service
 * reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureBatchLinkedService extends LinkedService {
  accountName: any;
  accessKey?: SecureString;
  batchUri: any;
  poolName: any;
  linkedServiceName: LinkedServiceReference;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlDatabaseLinkedService class.
 * @constructor
 * Microsoft Azure SQL Database linked service.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureSqlDatabaseLinkedService extends LinkedService {
  connectionString: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlServerLinkedService class.
 * @constructor
 * SQL Server linked service.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [userName] The on-premises Windows authentication user
 * name. Type: string (or Expression with resultType string).
 * @member {object} [password] The on-premises Windows authentication password.
 * @member {string} [password.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface SqlServerLinkedService extends LinkedService {
  connectionString: SecureString;
  userName?: any;
  password?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlDWLinkedService class.
 * @constructor
 * Azure SQL Data Warehouse linked service.
 *
 * @member {object} connectionString The connection string.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureSqlDWLinkedService extends LinkedService {
  connectionString: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureStorageLinkedService class.
 * @constructor
 * The storage account linked service.
 *
 * @member {object} [connectionString] The connection string. It is mutually
 * exclusive with sasUri property.
 * @member {string} [connectionString.value] Value of secure string.
 * @member {object} [sasUri] SAS URI of the Azure Storage resource. It is
 * mutually exclusive with connectionString property.
 * @member {string} [sasUri.value] Value of secure string.
 * @member {object} [encryptedCredential] The encrypted credential used for
 * authentication. Credentials are encrypted using the integration runtime
 * credential manager. Type: string (or Expression with resultType string).
 */
export interface AzureStorageLinkedService extends LinkedService {
  connectionString?: SecureString;
  sasUri?: SecureString;
  encryptedCredential?: any;
}

/**
 * @class
 * Initializes a new instance of the DatasetCompression class.
 * @constructor
 * The compression method used on a dataset.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface DatasetCompression {
  type: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetDeflateCompression class.
 * @constructor
 * The Deflate compression method used on a dataset.
 *
 * @member {string} [level] The Deflate compression level. Possible values
 * include: 'Optimal', 'Fastest'
 */
export interface DatasetDeflateCompression extends DatasetCompression {
  level?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetGZipCompression class.
 * @constructor
 * The GZip compression method used on a dataset.
 *
 * @member {string} [level] The GZip compression level. Possible values
 * include: 'Optimal', 'Fastest'
 */
export interface DatasetGZipCompression extends DatasetCompression {
  level?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetBZip2Compression class.
 * @constructor
 * The BZip2 compression method used on a dataset.
 *
 */
export interface DatasetBZip2Compression extends DatasetCompression {
}

/**
 * @class
 * Initializes a new instance of the DatasetStorageFormat class.
 * @constructor
 * The format definition of a storage.
 *
 * @member {object} [serializer] Serializer. Type: string (or Expression with
 * resultType string).
 * @member {object} [deserializer] Deserializer. Type: string (or Expression
 * with resultType string).
 * @member {string} type Polymorphic Discriminator
 */
export interface DatasetStorageFormat {
  serializer?: any;
  deserializer?: any;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the ParquetFormat class.
 * @constructor
 * The data stored in Parquet format.
 *
 */
export interface ParquetFormat extends DatasetStorageFormat {
}

/**
 * @class
 * Initializes a new instance of the OrcFormat class.
 * @constructor
 * The data stored in Optimized Row Columnar (ORC) format.
 *
 */
export interface OrcFormat extends DatasetStorageFormat {
}

/**
 * @class
 * Initializes a new instance of the AvroFormat class.
 * @constructor
 * The data stored in Avro format.
 *
 */
export interface AvroFormat extends DatasetStorageFormat {
}

/**
 * @class
 * Initializes a new instance of the JsonFormat class.
 * @constructor
 * The data stored in JSON format.
 *
 * @member {string} [filePattern] File pattern of JSON. To be more specific,
 * the way of separating a collection of JSON objects. The default value is
 * 'setOfObjects'. It is case-sensitive. Possible values include:
 * 'setOfObjects', 'arrayOfObjects'
 * @member {object} [nestingSeparator] The character used to separate nesting
 * levels. Default value is '.' (dot). Type: string (or Expression with
 * resultType string).
 * @member {object} [encodingName] The code page name of the preferred
 * encoding. If not provided, the default value is 'utf-8', unless the byte
 * order mark (BOM) denotes another Unicode encoding. The full list of
 * supported values can be found in the 'Name' column of the table of encodings
 * in the following reference:
 * https://msdn.microsoft.com/library/system.text.encoding.aspx#Anchor_5. Type:
 * string (or Expression with resultType string).
 */
export interface JsonFormat extends DatasetStorageFormat {
  filePattern?: string;
  nestingSeparator?: any;
  encodingName?: any;
}

/**
 * @class
 * Initializes a new instance of the TextFormat class.
 * @constructor
 * The data stored in text format.
 *
 * @member {object} [columnDelimiter] The column delimiter. Type: string (or
 * Expression with resultType string).
 * @member {object} [rowDelimiter] The row delimiter. Type: string (or
 * Expression with resultType string).
 * @member {object} [escapeChar] The escape character. Type: string (or
 * Expression with resultType string).
 * @member {object} [quoteChar] The quote character. Type: string (or
 * Expression with resultType string).
 * @member {object} [nullValue] The null value string. Type: string (or
 * Expression with resultType string).
 * @member {object} [encodingName] The code page name of the preferred
 * encoding. If miss, the default value is ΓÇ£utf-8ΓÇ¥, unless BOM denotes
 * another Unicode encoding. Refer to the ΓÇ£NameΓÇ¥ column of the table in the
 * following link to set supported values:
 * https://msdn.microsoft.com/library/system.text.encoding.aspx. Type: string
 * (or Expression with resultType string).
 * @member {object} [treatEmptyAsNull] Treat empty column values in the text
 * file as null. The default value is true. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [skipLineCount] The number of lines/rows to be skipped when
 * parsing text files. The default value is 0. Type: integer (or Expression
 * with resultType integer).
 * @member {object} [firstRowAsHeader] When used as input, treat the first row
 * of data as headers. When used as output,write the headers into the output as
 * the first row of data. The default value is false. Type: boolean (or
 * Expression with resultType boolean).
 */
export interface TextFormat extends DatasetStorageFormat {
  columnDelimiter?: any;
  rowDelimiter?: any;
  escapeChar?: any;
  quoteChar?: any;
  nullValue?: any;
  encodingName?: any;
  treatEmptyAsNull?: any;
  skipLineCount?: any;
  firstRowAsHeader?: any;
}

/**
 * @class
 * Initializes a new instance of the DatasetPartitionValue class.
 * @constructor
 * The value of a partition.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface DatasetPartitionValue {
  type: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetDateTimePartitionValue class.
 * @constructor
 * The date/time value of a partition.
 *
 * @member {object} [date] Name of variable containing date. Type: string (or
 * Expression with resultType string).
 * @member {object} [format] Format string for the Date value. Type: string (or
 * Expression with resultType string).
 */
export interface DatasetDateTimePartitionValue extends DatasetPartitionValue {
  date?: any;
  format?: any;
}

/**
 * @class
 * Initializes a new instance of the DatasetPartition class.
 * @constructor
 * The partition definition.
 *
 * @member {object} [name] Name of the partition. Type: string (or Expression
 * with resultType string).
 * @member {object} [value] Value of the partition.
 * @member {string} [value.type] Polymorphic Discriminator
 */
export interface DatasetPartition {
  name?: any;
  value?: DatasetPartitionValue;
}

/**
 * @class
 * Initializes a new instance of the HttpDataset class.
 * @constructor
 * A file in an HTTP web server.
 *
 * @member {object} [relativeUrl] The relative URL based on the URL in the
 * HttpLinkedService refers to an HTTP file Type: string (or Expression with
 * resultType string).
 * @member {object} [requestMethod] The HTTP method for the HTTP request. Type:
 * string (or Expression with resultType string).
 * @member {object} [requestBody] The body for the HTTP request. Type: string
 * (or Expression with resultType string).
 * @member {object} [additionalHeaders] The headers for the HTTP Request. e.g.
 * request-header-name-1:request-header-value-1
 * ...
 * request-header-name-n:request-header-value-n Type: string (or Expression
 * with resultType string).
 * @member {object} [partitionedBy] The HTTP method for the HTTP request.
 * @member {object} [partitionedBy.name] Name of the partition. Type: string
 * (or Expression with resultType string).
 * @member {object} [partitionedBy.value] Value of the partition.
 * @member {string} [partitionedBy.value.type] Polymorphic Discriminator
 * @member {object} [format] The format of files.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used on files.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface HttpDataset extends Dataset {
  relativeUrl?: any;
  requestMethod?: any;
  requestBody?: any;
  additionalHeaders?: any;
  partitionedBy?: DatasetPartition;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the AzureSearchIndexDataset class.
 * @constructor
 * The Azure Search Index.
 *
 * @member {object} indexName The name of the Azure Search Index. Type: string
 * (or Expression with resultType string).
 */
export interface AzureSearchIndexDataset extends Dataset {
  indexName: any;
}

/**
 * @class
 * Initializes a new instance of the WebTableDataset class.
 * @constructor
 * The dataset points to a HTML table in the web page.
 *
 * @member {object} index The zero-based index of the table in the web page.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {object} [path] The relative URL to the web page from the linked
 * service URL. Type: string (or Expression with resultType string).
 */
export interface WebTableDataset extends Dataset {
  index: any;
  path?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlServerTableDataset class.
 * @constructor
 * The on-premises SQL Server dataset.
 *
 * @member {object} tableName The table name of the SQL Server dataset. Type:
 * string (or Expression with resultType string).
 */
export interface SqlServerTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the RelationalTableDataset class.
 * @constructor
 * The relational table dataset.
 *
 * @member {object} [tableName] The relational table name. Type: string (or
 * Expression with resultType string).
 */
export interface RelationalTableDataset extends Dataset {
  tableName?: any;
}

/**
 * @class
 * Initializes a new instance of the OracleTableDataset class.
 * @constructor
 * The on-premises Oracle database dataset.
 *
 * @member {object} tableName The table name of the on-premises Oracle
 * database. Type: string (or Expression with resultType string).
 */
export interface OracleTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the ODataResourceDataset class.
 * @constructor
 * The Open Data Protocol (OData) resource dataset.
 *
 * @member {object} [path] The OData resource path. Type: string (or Expression
 * with resultType string).
 */
export interface ODataResourceDataset extends Dataset {
  path?: any;
}

/**
 * @class
 * Initializes a new instance of the MongoDbCollectionDataset class.
 * @constructor
 * The MongoDB database dataset.
 *
 * @member {object} collectionName The table name of the MongoDB database.
 * Type: string (or Expression with resultType string).
 */
export interface MongoDbCollectionDataset extends Dataset {
  collectionName: any;
}

/**
 * @class
 * Initializes a new instance of the FileShareDataset class.
 * @constructor
 * An on-premises file system dataset.
 *
 * @member {object} [folderPath] The path of the on-premises file system. Type:
 * string (or Expression with resultType string).
 * @member {object} [fileName] The name of the on-premises file system. Type:
 * string (or Expression with resultType string).
 * @member {object} [format] The format of the files.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [fileFilter] Specify a filter to be used to select a subset
 * of files in the folderPath rather than all files. Type: string (or
 * Expression with resultType string).
 * @member {object} [compression] The data compression method used for the file
 * system.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface FileShareDataset extends Dataset {
  folderPath?: any;
  fileName?: any;
  format?: DatasetStorageFormat;
  fileFilter?: any;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreDataset class.
 * @constructor
 * Azure Data Lake Store dataset.
 *
 * @member {object} folderPath Path to the folder in the Azure Data Lake Store.
 * Type: string (or Expression with resultType string).
 * @member {object} [fileName] The name of the file in the Azure Data Lake
 * Store. Type: string (or Expression with resultType string).
 * @member {object} [format] The format of the Data Lake Store.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used for the
 * item(s) in the Azure Data Lake Store.
 * @member {string} [compression.type] Polymorphic Discriminator
 * @member {array} [partitionedBy] Specify a dynamic path and filename for time
 * series data.
 */
export interface AzureDataLakeStoreDataset extends Dataset {
  folderPath: any;
  fileName?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
  partitionedBy?: DatasetPartition[];
}

/**
 * @class
 * Initializes a new instance of the DynamicsEntityDataset class.
 * @constructor
 * The Dynamics entity dataset.
 *
 * @member {object} [entityName] The logical name of the entity. Type: string
 * (or Expression with resultType string).
 */
export interface DynamicsEntityDataset extends Dataset {
  entityName?: any;
}

/**
 * @class
 * Initializes a new instance of the DocumentDbCollectionDataset class.
 * @constructor
 * Microsoft Azure Document Database Collection dataset.
 *
 * @member {object} collectionName Document Database collection name. Type:
 * string (or Expression with resultType string).
 */
export interface DocumentDbCollectionDataset extends Dataset {
  collectionName: any;
}

/**
 * @class
 * Initializes a new instance of the CustomDataset class.
 * @constructor
 * The custom dataset.
 *
 * @member {object} typeProperties Custom dataset properties.
 */
export interface CustomDataset extends Dataset {
  typeProperties: any;
}

/**
 * @class
 * Initializes a new instance of the CassandraTableDataset class.
 * @constructor
 * The Cassandra database dataset.
 *
 * @member {object} [tableName] The table name of the Cassandra database. Type:
 * string (or Expression with resultType string).
 * @member {object} [keyspace] The keyspace of the Cassandra database. Type:
 * string (or Expression with resultType string).
 */
export interface CassandraTableDataset extends Dataset {
  tableName?: any;
  keyspace?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlDWTableDataset class.
 * @constructor
 * The Azure SQL Data Warehouse dataset.
 *
 * @member {object} tableName The table name of the Azure SQL Data Warehouse.
 * Type: string (or Expression with resultType string).
 */
export interface AzureSqlDWTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlTableDataset class.
 * @constructor
 * The Azure SQL Server database dataset.
 *
 * @member {object} tableName The table name of the Azure SQL database. Type:
 * string (or Expression with resultType string).
 */
export interface AzureSqlTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the AzureTableDataset class.
 * @constructor
 * The Azure Table storage dataset.
 *
 * @member {object} tableName The table name of the Azure Table storage. Type:
 * string (or Expression with resultType string).
 */
export interface AzureTableDataset extends Dataset {
  tableName: any;
}

/**
 * @class
 * Initializes a new instance of the AzureBlobDataset class.
 * @constructor
 * The Azure Blob storage.
 *
 * @member {object} [folderPath] The path of the Azure Blob storage. Type:
 * string (or Expression with resultType string).
 * @member {object} [tableRootLocation] The root of blob path. Type: string (or
 * Expression with resultType string).
 * @member {object} [fileName] The name of the Azure Blob. Type: string (or
 * Expression with resultType string).
 * @member {object} [format] The format of the Azure Blob storage.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used for the blob
 * storage.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface AzureBlobDataset extends Dataset {
  folderPath?: any;
  tableRootLocation?: any;
  fileName?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the AmazonS3Dataset class.
 * @constructor
 * A single Amazon Simple Storage Service (S3) object or a set of S3 objects.
 *
 * @member {object} bucketName The name of the Amazon S3 bucket. Type: string
 * (or Expression with resultType string).
 * @member {object} [key] The key of the Amazon S3 object. Type: string (or
 * Expression with resultType string).
 * @member {object} [prefix] The prefix filter for the S3 object name. Type:
 * string (or Expression with resultType string).
 * @member {object} [version] The version for the S3 object. Type: string (or
 * Expression with resultType string).
 * @member {object} [format] The format of files.
 * @member {object} [format.serializer] Serializer. Type: string (or Expression
 * with resultType string).
 * @member {object} [format.deserializer] Deserializer. Type: string (or
 * Expression with resultType string).
 * @member {string} [format.type] Polymorphic Discriminator
 * @member {object} [compression] The data compression method used for the
 * Amazon S3 object.
 * @member {string} [compression.type] Polymorphic Discriminator
 */
export interface AmazonS3Dataset extends Dataset {
  bucketName: any;
  key?: any;
  prefix?: any;
  version?: any;
  format?: DatasetStorageFormat;
  compression?: DatasetCompression;
}

/**
 * @class
 * Initializes a new instance of the BlobTrigger class.
 * @constructor
 * Trigger that runs everytime the selected Blob container changes.
 *
 * @member {string} [folderPath] The path of the container/folder that will
 * trigger the pipeline.
 * @member {number} [maxConcurrency] The max number of parallel files to handle
 * when it is triggered.
 * @member {object} [linkedService] The Azure Storage linked service reference.
 * @member {string} [linkedService.referenceName] Reference LinkedService name.
 */
export interface BlobTrigger extends Trigger {
  folderPath?: string;
  maxConcurrency?: number;
  linkedService?: LinkedServiceReference;
}

/**
 * @class
 * Initializes a new instance of the RecurrenceScheduleOccurrence class.
 * @constructor
 * The recurrence schedule occurence.
 *
 * @member {string} [day] The day of the week. Possible values include:
 * 'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'
 * @member {number} [occurrence] The occurrence.
 */
export interface RecurrenceScheduleOccurrence {
  day?: string;
  occurrence?: number;
}

/**
 * @class
 * Initializes a new instance of the RecurrenceSchedule class.
 * @constructor
 * The recurrence schedule.
 *
 * @member {array} [minutes] The minutes.
 * @member {array} [hours] The hours.
 * @member {array} [weekDays] The days of the week.
 * @member {array} [monthDays] The month days.
 * @member {array} [monthlyOccurrences] The monthly occurrences.
 */
export interface RecurrenceSchedule {
  minutes?: number[];
  hours?: number[];
  weekDays?: string[];
  monthDays?: number[];
  monthlyOccurrences?: RecurrenceScheduleOccurrence[];
}

/**
 * @class
 * Initializes a new instance of the ScheduleTriggerRecurrence class.
 * @constructor
 * The workflow trigger recurrence.
 *
 * @member {string} [frequency] The frequency. Possible values include:
 * 'NotSpecified', 'Minute', 'Hour', 'Day', 'Week', 'Month', 'Year'
 * @member {number} [interval] The interval.
 * @member {date} [startTime] The start time.
 * @member {date} [endTime] The end time.
 * @member {string} [timeZone] The time zone.
 * @member {object} [schedule] The recurrence schedule.
 * @member {array} [schedule.minutes] The minutes.
 * @member {array} [schedule.hours] The hours.
 * @member {array} [schedule.weekDays] The days of the week.
 * @member {array} [schedule.monthDays] The month days.
 * @member {array} [schedule.monthlyOccurrences] The monthly occurrences.
 */
export interface ScheduleTriggerRecurrence {
  frequency?: string;
  interval?: number;
  startTime?: Date;
  endTime?: Date;
  timeZone?: string;
  schedule?: RecurrenceSchedule;
}

/**
 * @class
 * Initializes a new instance of the ScheduleTrigger class.
 * @constructor
 * Trigger that creates pipeline runs periodically, on schedule.
 *
 * @member {object} [recurrence] Recurrence schedule configuration.
 * @member {string} [recurrence.frequency] The frequency. Possible values
 * include: 'NotSpecified', 'Minute', 'Hour', 'Day', 'Week', 'Month', 'Year'
 * @member {number} [recurrence.interval] The interval.
 * @member {date} [recurrence.startTime] The start time.
 * @member {date} [recurrence.endTime] The end time.
 * @member {string} [recurrence.timeZone] The time zone.
 * @member {object} [recurrence.schedule] The recurrence schedule.
 * @member {array} [recurrence.schedule.minutes] The minutes.
 * @member {array} [recurrence.schedule.hours] The hours.
 * @member {array} [recurrence.schedule.weekDays] The days of the week.
 * @member {array} [recurrence.schedule.monthDays] The month days.
 * @member {array} [recurrence.schedule.monthlyOccurrences] The monthly
 * occurrences.
 */
export interface ScheduleTrigger extends Trigger {
  recurrence?: ScheduleTriggerRecurrence;
}

/**
 * @class
 * Initializes a new instance of the ActivityPolicy class.
 * @constructor
 * Execution policy for an activity.
 *
 * @member {object} [timeout] Specifies the timeout for the activity to run.
 * The default timeout is 7 days. Type: string (or Expression with resultType
 * string), pattern: ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [retry] Maximum ordinary retry attempts. Default is 0.
 * Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {number} [retryIntervalInSeconds] Interval between each retry
 * attempt (in seconds). The default is 30 sec.
 */
export interface ActivityPolicy {
  timeout?: any;
  retry?: any;
  retryIntervalInSeconds?: number;
}

/**
 * @class
 * Initializes a new instance of the ExecutionActivity class.
 * @constructor
 * Base class for all execution activities.
 *
 * @member {object} [linkedServiceName] Linked service reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [policy] Activity policy.
 * @member {object} [policy.timeout] Specifies the timeout for the activity to
 * run. The default timeout is 7 days. Type: string (or Expression with
 * resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [policy.retry] Maximum ordinary retry attempts. Default is
 * 0. Type: integer (or Expression with resultType integer), minimum: 0.
 * @member {number} [policy.retryIntervalInSeconds] Interval between each retry
 * attempt (in seconds). The default is 30 sec.
 */
export interface ExecutionActivity extends Activity {
  linkedServiceName?: LinkedServiceReference;
  policy?: ActivityPolicy;
}

/**
 * @class
 * Initializes a new instance of the DataLakeAnalyticsUSQLActivity class.
 * @constructor
 * Data Lake Analytics U-SQL activity.
 *
 * @member {object} scriptPath Case-sensitive path to folder that contains the
 * U-SQL script. Type: string (or Expression with resultType string).
 * @member {object} scriptLinkedService Script linked service reference.
 * @member {string} [scriptLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [degreeOfParallelism] The maximum number of nodes
 * simultaneously used to run the job. Default value is 1. Type: integer (or
 * Expression with resultType integer), minimum: 1.
 * @member {object} [priority] Determines which jobs out of all that are queued
 * should be selected to run first. The lower the number, the higher the
 * priority. Default value is 1000. Type: integer (or Expression with
 * resultType integer), minimum: 1.
 * @member {object} [parameters] Parameters for U-SQL job request.
 * @member {object} [runtimeVersion] Runtime version of the U-SQL engine to
 * use. Type: string (or Expression with resultType string).
 * @member {object} [compilationMode] Compilation mode of U-SQL. Must be one of
 * these values : Semantic, Full and SingleBox. Type: string (or Expression
 * with resultType string).
 */
export interface DataLakeAnalyticsUSQLActivity extends ExecutionActivity {
  scriptPath: any;
  scriptLinkedService: LinkedServiceReference;
  degreeOfParallelism?: any;
  priority?: any;
  parameters?: { [propertyName: string]: any };
  runtimeVersion?: any;
  compilationMode?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMLUpdateResourceActivity class.
 * @constructor
 * Azure ML Update Resource management activity.
 *
 * @member {object} trainedModelName Name of the Trained Model module in the
 * Web Service experiment to be updated. Type: string (or Expression with
 * resultType string).
 * @member {object} trainedModelLinkedServiceName Name of Azure Storage linked
 * service holding the .ilearner file that will be uploaded by the update
 * operation.
 * @member {string} [trainedModelLinkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} trainedModelFilePath The relative file path in
 * trainedModelLinkedService to represent the .ilearner file that will be
 * uploaded by the update operation.  Type: string (or Expression with
 * resultType string).
 */
export interface AzureMLUpdateResourceActivity extends ExecutionActivity {
  trainedModelName: any;
  trainedModelLinkedServiceName: LinkedServiceReference;
  trainedModelFilePath: any;
}

/**
 * @class
 * Initializes a new instance of the AzureMLWebServiceFile class.
 * @constructor
 * Azure ML WebService Input/Output file
 *
 * @member {object} filePath The relative file path, including container name,
 * in the Azure Blob Storage specified by the LinkedService. Type: string (or
 * Expression with resultType string).
 * @member {object} linkedServiceName Reference to an Azure Storage
 * LinkedService, where Azure ML WebService Input/Output file located.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 */
export interface AzureMLWebServiceFile {
  filePath: any;
  linkedServiceName: LinkedServiceReference;
}

/**
 * @class
 * Initializes a new instance of the AzureMLBatchExecutionActivity class.
 * @constructor
 * Azure ML Batch Execution activity.
 *
 * @member {object} [globalParameters] Key,Value pairs to be passed to the
 * Azure ML Batch Execution Service endpoint. Keys must match the names of web
 * service parameters defined in the published Azure ML web service. Values
 * will be passed in the GlobalParameters property of the Azure ML batch
 * execution request.
 * @member {object} [webServiceOutputs] Key,Value pairs, mapping the names of
 * Azure ML endpoint's Web Service Outputs to AzureMLWebServiceFile objects
 * specifying the output Blob locations. This information will be passed in the
 * WebServiceOutputs property of the Azure ML batch execution request.
 * @member {object} [webServiceInputs] Key,Value pairs, mapping the names of
 * Azure ML endpoint's Web Service Inputs to AzureMLWebServiceFile objects
 * specifying the input Blob locations.. This information will be passed in the
 * WebServiceInputs property of the Azure ML batch execution request.
 */
export interface AzureMLBatchExecutionActivity extends ExecutionActivity {
  globalParameters?: { [propertyName: string]: any };
  webServiceOutputs?: { [propertyName: string]: AzureMLWebServiceFile };
  webServiceInputs?: { [propertyName: string]: AzureMLWebServiceFile };
}

/**
 * @class
 * Initializes a new instance of the GetMetadataActivity class.
 * @constructor
 * Activity to get metadata of dataset
 *
 * @member {object} dataset GetMetadata activity dataset reference.
 * @member {string} [dataset.referenceName] Reference dataset name.
 * @member {object} [dataset.parameters] Arguments for dataset.
 * @member {array} [fieldList] Fields of metadata to get from dataset.
 */
export interface GetMetadataActivity extends ExecutionActivity {
  dataset: DatasetReference;
  fieldList?: any[];
}

/**
 * @class
 * Initializes a new instance of the WebActivityAuthentication class.
 * @constructor
 * Web activity authentication properties.
 *
 * @member {string} type Web activity authentication (Basic/ClientCertificate)
 * @member {object} [pfx] Base64-encoded contents of a PFX file.
 * @member {string} [pfx.value] Value of secure string.
 * @member {string} [username] Web activity authentication user name for basic
 * authentication.
 * @member {object} [password] Password for the PFX file or basic
 * authentication.
 * @member {string} [password.value] Value of secure string.
 */
export interface WebActivityAuthentication {
  type: string;
  pfx?: SecureString;
  username?: string;
  password?: SecureString;
}

/**
 * @class
 * Initializes a new instance of the WebActivity class.
 * @constructor
 * Web activity.
 *
 * @member {string} method Rest API method for target endpoint. Possible values
 * include: 'GET', 'POST', 'PUT'
 * @member {object} url Web activity target endpoint and path. Type: string (or
 * Expression with resultType string).
 * @member {object} [headers] Represents the headers that will be sent to the
 * request. For example, to set the language and type on a request: "headers" :
 * { "Accept-Language": "en-us", "Content-Type": "application/json" }. Type:
 * string (or Expression with resultType string).
 * @member {object} [body] Represents the payload that will be sent to the
 * endpoint. Required for POST/PUT method, not allowed for GET method Type:
 * string (or Expression with resultType string).
 * @member {object} [authentication] Authentication method used for calling the
 * endpoint.
 * @member {string} [authentication.type] Web activity authentication
 * (Basic/ClientCertificate)
 * @member {object} [authentication.pfx] Base64-encoded contents of a PFX file.
 * @member {string} [authentication.pfx.value] Value of secure string.
 * @member {string} [authentication.username] Web activity authentication user
 * name for basic authentication.
 * @member {object} [authentication.password] Password for the PFX file or
 * basic authentication.
 * @member {string} [authentication.password.value] Value of secure string.
 * @member {array} [datasets] List of datasets passed to web endpoint.
 * @member {array} [linkedServices] List of linked services passed to web
 * endpoint.
 */
export interface WebActivity extends ExecutionActivity {
  method: string;
  url: any;
  headers?: any;
  body?: any;
  authentication?: WebActivityAuthentication;
  datasets?: DatasetReference[];
  linkedServices?: LinkedServiceReference[];
}

/**
 * @class
 * Initializes a new instance of the RedshiftUnloadSettings class.
 * @constructor
 * The Amazon S3 settings needed for the interim Amazon S3 when copying from
 * Amazon Redshift with unload. With this, data from Amazon Redshift source
 * will be unloaded into S3 first and then copied into the targeted sink from
 * the interim S3.
 *
 * @member {object} s3LinkedServiceName The name of the Amazon S3 linked
 * service which will be used for the unload operation when copying from the
 * Amazon Redshift source.
 * @member {string} [s3LinkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} bucketName The bucket of the interim Amazon S3 which will
 * be used to store the unloaded data from Amazon Redshift source. The bucket
 * must be in the same region as the Amazon Redshift source. Type: string (or
 * Expression with resultType string).
 */
export interface RedshiftUnloadSettings {
  s3LinkedServiceName: LinkedServiceReference;
  bucketName: any;
}

/**
 * @class
 * Initializes a new instance of the CopySource class.
 * @constructor
 * A copy activity source.
 *
 * @member {object} [sourceRetryCount] Source retry count. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [sourceRetryWait] Source retry wait. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} type Polymorphic Discriminator
 */
export interface CopySource {
  sourceRetryCount?: any;
  sourceRetryWait?: any;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the AmazonRedshiftSource class.
 * @constructor
 * A copy activity source for Amazon Redshift Source.
 *
 * @member {object} [query] Database query. Type: string (or Expression with
 * resultType string).
 * @member {object} [redshiftUnloadSettings] The Amazon S3 settings needed for
 * the interim Amazon S3 when copying from Amazon Redshift with unload. With
 * this, data from Amazon Redshift source will be unloaded into S3 first and
 * then copied into the targeted sink from the interim S3.
 * @member {object} [redshiftUnloadSettings.s3LinkedServiceName] The name of
 * the Amazon S3 linked service which will be used for the unload operation
 * when copying from the Amazon Redshift source.
 * @member {string} [redshiftUnloadSettings.s3LinkedServiceName.referenceName]
 * Reference LinkedService name.
 * @member {object} [redshiftUnloadSettings.bucketName] The bucket of the
 * interim Amazon S3 which will be used to store the unloaded data from Amazon
 * Redshift source. The bucket must be in the same region as the Amazon
 * Redshift source. Type: string (or Expression with resultType string).
 */
export interface AmazonRedshiftSource extends CopySource {
  query?: any;
  redshiftUnloadSettings?: RedshiftUnloadSettings;
}

/**
 * @class
 * Initializes a new instance of the HttpSource class.
 * @constructor
 * A copy activity source for an HTTP file.
 *
 * @member {object} [httpRequestTimeout] Specifies the timeout for a HTTP
 * client to get HTTP response from HTTP server. The default value is
 * equivalent to System.Net.HttpWebRequest.Timeout. Type: string (or Expression
 * with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 */
export interface HttpSource extends CopySource {
  httpRequestTimeout?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreSource class.
 * @constructor
 * A copy activity Azure Data Lake source.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 */
export interface AzureDataLakeStoreSource extends CopySource {
  recursive?: any;
}

/**
 * @class
 * Initializes a new instance of the MongoDbSource class.
 * @constructor
 * A copy activity source for a MongoDB database.
 *
 * @member {object} [query] Database query. Should be a SQL-92 query
 * expression. Type: string (or Expression with resultType string).
 */
export interface MongoDbSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the CassandraSource class.
 * @constructor
 * A copy activity source for a Cassandra database.
 *
 * @member {object} [query] Database query. Should be a SQL-92 query expression
 * or Cassandra Query Language (CQL) command. Type: string (or Expression with
 * resultType string).
 * @member {string} [consistencyLevel] The consistency level specifies how many
 * Cassandra servers must respond to a read request before returning data to
 * the client application. Cassandra checks the specified number of Cassandra
 * servers for data to satisfy the read request. Must be one of
 * cassandraSourceReadConsistencyLevels. The default value is 'ONE'. It is
 * case-insensitive. Possible values include: 'ALL', 'EACH_QUORUM', 'QUORUM',
 * 'LOCAL_QUORUM', 'ONE', 'TWO', 'THREE', 'LOCAL_ONE', 'SERIAL', 'LOCAL_SERIAL'
 */
export interface CassandraSource extends CopySource {
  query?: any;
  consistencyLevel?: string;
}

/**
 * @class
 * Initializes a new instance of the WebSource class.
 * @constructor
 * A copy activity source for web page table.
 *
 */
export interface WebSource extends CopySource {
}

/**
 * @class
 * Initializes a new instance of the OracleSource class.
 * @constructor
 * A copy activity Oracle source.
 *
 * @member {object} [oracleReaderQuery] Oracle reader query. Type: string (or
 * Expression with resultType string).
 * @member {object} [queryTimeout] Query timeout. Type: string (or Expression
 * with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 */
export interface OracleSource extends CopySource {
  oracleReaderQuery?: any;
  queryTimeout?: any;
}

/**
 * @class
 * Initializes a new instance of the DistcpSettings class.
 * @constructor
 * Distcp settings.
 *
 * @member {object} resourceManagerEndpoint Specifies the Yarn ResourceManager
 * endpoint. Type: string (or Expression with resultType string).
 * @member {object} tempScriptPath Specifies an existing folder path which will
 * be used to store temp Distcp command script. The script file is generated by
 * ADF and will be removed after Copy job finished. Type: string (or Expression
 * with resultType string).
 * @member {object} [distcpOptions] Specifies the Distcp options. Type: string
 * (or Expression with resultType string).
 */
export interface DistcpSettings {
  resourceManagerEndpoint: any;
  tempScriptPath: any;
  distcpOptions?: any;
}

/**
 * @class
 * Initializes a new instance of the HdfsSource class.
 * @constructor
 * A copy activity source for HDFS source.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [distcpSettings] Specifies Distcp-related settings.
 * @member {object} [distcpSettings.resourceManagerEndpoint] Specifies the Yarn
 * ResourceManager endpoint. Type: string (or Expression with resultType
 * string).
 * @member {object} [distcpSettings.tempScriptPath] Specifies an existing
 * folder path which will be used to store temp Distcp command script. The
 * script file is generated by ADF and will be removed after Copy job finished.
 * Type: string (or Expression with resultType string).
 * @member {object} [distcpSettings.distcpOptions] Specifies the Distcp
 * options. Type: string (or Expression with resultType string).
 */
export interface HdfsSource extends CopySource {
  recursive?: any;
  distcpSettings?: DistcpSettings;
}

/**
 * @class
 * Initializes a new instance of the FileSystemSource class.
 * @constructor
 * A copy activity file system source.
 *
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 */
export interface FileSystemSource extends CopySource {
  recursive?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlDWSource class.
 * @constructor
 * A copy activity SQL Data Warehouse source.
 *
 * @member {object} [sqlReaderQuery] SQL Data Warehouse reader query. Type:
 * string (or Expression with resultType string).
 * @member {object} [sqlReaderStoredProcedureName] Name of the stored procedure
 * for a SQL Data Warehouse source. This cannot be used at the same time as
 * SqlReaderQuery. Type: string (or Expression with resultType string).
 * @member {object} [storedProcedureParameters] Value and type setting for
 * stored procedure parameters. Example: "{Parameter1: {value: "1", type:
 * "int"}}". Type: object (or Expression with resultType object), itemType:
 * StoredProcedureParameter.
 */
export interface SqlDWSource extends CopySource {
  sqlReaderQuery?: any;
  sqlReaderStoredProcedureName?: any;
  storedProcedureParameters?: any;
}

/**
 * @class
 * Initializes a new instance of the StoredProcedureParameter class.
 * @constructor
 * SQL stored procedure parameter.
 *
 * @member {object} value Stored procedure parameter value. Type: string (or
 * Expression with resultType string).
 * @member {string} [type] Stored procedure parameter type. Possible values
 * include: 'String', 'Int', 'Decimal', 'Guid', 'Boolean', 'Date'
 */
export interface StoredProcedureParameter {
  value: any;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the SqlSource class.
 * @constructor
 * A copy activity SQL source.
 *
 * @member {object} [sqlReaderQuery] SQL reader query. Type: string (or
 * Expression with resultType string).
 * @member {object} [sqlReaderStoredProcedureName] Name of the stored procedure
 * for a SQL Database source. This cannot be used at the same time as
 * SqlReaderQuery. Type: string (or Expression with resultType string).
 * @member {object} [storedProcedureParameters] Value and type setting for
 * stored procedure parameters. Example: "{Parameter1: {value: "1", type:
 * "int"}}".
 */
export interface SqlSource extends CopySource {
  sqlReaderQuery?: any;
  sqlReaderStoredProcedureName?: any;
  storedProcedureParameters?: { [propertyName: string]: StoredProcedureParameter };
}

/**
 * @class
 * Initializes a new instance of the RelationalSource class.
 * @constructor
 * A copy activity source for various relational databases.
 *
 * @member {object} [query] Database query. Type: string (or Expression with
 * resultType string).
 */
export interface RelationalSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the DynamicsSource class.
 * @constructor
 * A copy activity Dynamics entity source.
 *
 * @member {object} [query] FetchXML is a proprietary query language that is
 * used in Microsoft Dynamics (online & on-premises). Type: string (or
 * Expression with resultType string).
 */
export interface DynamicsSource extends CopySource {
  query?: any;
}

/**
 * @class
 * Initializes a new instance of the DocumentDbCollectionSource class.
 * @constructor
 * A copy activity Document Database Collection source.
 *
 * @member {object} [query] Documents query. Type: string (or Expression with
 * resultType string).
 * @member {object} [nestingSeparator] Nested properties separator. Type:
 * string (or Expression with resultType string).
 */
export interface DocumentDbCollectionSource extends CopySource {
  query?: any;
  nestingSeparator?: any;
}

/**
 * @class
 * Initializes a new instance of the BlobSource class.
 * @constructor
 * A copy activity Azure Blob source.
 *
 * @member {object} [treatEmptyAsNull] Treat empty as null. Type: boolean (or
 * Expression with resultType boolean).
 * @member {object} [skipHeaderLineCount] Number of header lines to skip from
 * each blob. Type: integer (or Expression with resultType integer).
 * @member {object} [recursive] If true, files under the folder path will be
 * read recursively. Default is true. Type: boolean (or Expression with
 * resultType boolean).
 */
export interface BlobSource extends CopySource {
  treatEmptyAsNull?: any;
  skipHeaderLineCount?: any;
  recursive?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureTableSource class.
 * @constructor
 * A copy activity Azure Table source.
 *
 * @member {object} [azureTableSourceQuery] Azure Table source query. Type:
 * string (or Expression with resultType string).
 * @member {object} [azureTableSourceIgnoreTableNotFound] Azure Table source
 * ignore table not found. Type: boolean (or Expression with resultType
 * boolean).
 */
export interface AzureTableSource extends CopySource {
  azureTableSourceQuery?: any;
  azureTableSourceIgnoreTableNotFound?: any;
}

/**
 * @class
 * Initializes a new instance of the LookupActivity class.
 * @constructor
 * Lookup activity.
 *
 * @member {object} source Dataset-specific source properties, same as copy
 * activity source.
 * @member {object} [source.sourceRetryCount] Source retry count. Type: integer
 * (or Expression with resultType integer).
 * @member {object} [source.sourceRetryWait] Source retry wait. Type: string
 * (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} [source.type] Polymorphic Discriminator
 * @member {object} dataset Lookup activity dataset reference.
 * @member {string} [dataset.referenceName] Reference dataset name.
 * @member {object} [dataset.parameters] Arguments for dataset.
 * @member {object} [firstRowOnly] Whether to return first row or all rows.
 * Default value is true. Type: boolean (or Expression with resultType
 * boolean).
 */
export interface LookupActivity extends ExecutionActivity {
  source: CopySource;
  dataset: DatasetReference;
  firstRowOnly?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlServerStoredProcedureActivity class.
 * @constructor
 * SQL stored procedure activity type.
 *
 * @member {object} storedProcedureName Stored procedure name. Type: string (or
 * Expression with resultType string).
 * @member {object} [storedProcedureParameters] Value and type setting for
 * stored procedure parameters. Example: "{Parameter1: {value: "1", type:
 * "int"}}".
 */
export interface SqlServerStoredProcedureActivity extends ExecutionActivity {
  storedProcedureName: any;
  storedProcedureParameters?: { [propertyName: string]: StoredProcedureParameter };
}

/**
 * @class
 * Initializes a new instance of the CustomActivityReferenceObject class.
 * @constructor
 * Reference objects for custom activity
 *
 * @member {array} [linkedServices] Linked service references.
 * @member {array} [datasets] Dataset references.
 */
export interface CustomActivityReferenceObject {
  linkedServices?: LinkedServiceReference[];
  datasets?: DatasetReference[];
}

/**
 * @class
 * Initializes a new instance of the CustomActivity class.
 * @constructor
 * Custom activity type.
 *
 * @member {object} command Command for custom activity Type: string (or
 * Expression with resultType string).
 * @member {object} [resourceLinkedService] Resource linked service reference.
 * @member {string} [resourceLinkedService.referenceName] Reference
 * LinkedService name.
 * @member {object} [folderPath] Folder path for resource files Type: string
 * (or Expression with resultType string).
 * @member {object} [referenceObjects] Reference objects
 * @member {array} [referenceObjects.linkedServices] Linked service references.
 * @member {array} [referenceObjects.datasets] Dataset references.
 * @member {object} [extendedProperties] User defined property bag. There is no
 * restriction on the keys or values that can be used. The user specified
 * custom activity has the full responsibility to consume and interpret the
 * content defined.
 */
export interface CustomActivity extends ExecutionActivity {
  command: any;
  resourceLinkedService?: LinkedServiceReference;
  folderPath?: any;
  referenceObjects?: CustomActivityReferenceObject;
  extendedProperties?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightSparkActivity class.
 * @constructor
 * HDInsight Spark activity.
 *
 * @member {object} rootPath The root path in 'sparkJobLinkedService' for all
 * the job’s files. Type: string (or Expression with resultType string).
 * @member {object} entryFilePath The relative path to the root folder of the
 * code/package to be executed. Type: string (or Expression with resultType
 * string).
 * @member {array} [arguments] The user-specified arguments to
 * HDInsightSparkActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} [sparkJobLinkedService] The storage linked service for
 * uploading the entry file and dependencies, and for receiving logs.
 * @member {string} [sparkJobLinkedService.referenceName] Reference
 * LinkedService name.
 * @member {string} [className] The application's Java/Spark main class.
 * @member {object} [proxyUser] The user to impersonate that will execute the
 * job. Type: string (or Expression with resultType string).
 * @member {object} [sparkConfig] Spark configuration property.
 */
export interface HDInsightSparkActivity extends ExecutionActivity {
  rootPath: any;
  entryFilePath: any;
  arguments?: any[];
  getDebugInfo?: string;
  sparkJobLinkedService?: LinkedServiceReference;
  className?: string;
  proxyUser?: any;
  sparkConfig?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightStreamingActivity class.
 * @constructor
 * HDInsight streaming activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [arguments] User specified arguments to HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} mapper Mapper executable name. Type: string (or Expression
 * with resultType string).
 * @member {object} reducer Reducer executable name. Type: string (or
 * Expression with resultType string).
 * @member {object} input Input blob path. Type: string (or Expression with
 * resultType string).
 * @member {object} output Output blob path. Type: string (or Expression with
 * resultType string).
 * @member {array} filePaths Paths to streaming job files. Can be directories.
 * @member {object} [fileLinkedService] Linked service reference where the
 * files are located.
 * @member {string} [fileLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [combiner] Combiner executable name. Type: string (or
 * Expression with resultType string).
 * @member {array} [commandEnvironment] Command line environment values.
 * @member {object} [defines] Allows user to specify defines for streaming job
 * request.
 */
export interface HDInsightStreamingActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  arguments?: any[];
  getDebugInfo?: string;
  mapper: any;
  reducer: any;
  input: any;
  output: any;
  filePaths: any[];
  fileLinkedService?: LinkedServiceReference;
  combiner?: any;
  commandEnvironment?: any[];
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightMapReduceActivity class.
 * @constructor
 * HDInsight MapReduce activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [arguments] User specified arguments to HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} className Class name. Type: string (or Expression with
 * resultType string).
 * @member {object} jarFilePath Jar path. Type: string (or Expression with
 * resultType string).
 * @member {object} [jarLinkedService] Jar linked service reference.
 * @member {string} [jarLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {array} [jarLibs] Jar libs.
 * @member {object} [defines] Allows user to specify defines for the MapReduce
 * job request.
 */
export interface HDInsightMapReduceActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  arguments?: any[];
  getDebugInfo?: string;
  className: any;
  jarFilePath: any;
  jarLinkedService?: LinkedServiceReference;
  jarLibs?: any[];
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightPigActivity class.
 * @constructor
 * HDInsight Pig activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [arguments] User specified arguments to HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} [scriptPath] Script path. Type: string (or Expression with
 * resultType string).
 * @member {object} [scriptLinkedService] Script linked service reference.
 * @member {string} [scriptLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [defines] Allows user to specify defines for Pig job
 * request.
 */
export interface HDInsightPigActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  arguments?: any[];
  getDebugInfo?: string;
  scriptPath?: any;
  scriptLinkedService?: LinkedServiceReference;
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the HDInsightHiveActivity class.
 * @constructor
 * HDInsight Hive activity type.
 *
 * @member {array} [storageLinkedServices] Storage linked service references.
 * @member {array} [arguments] User specified arguments to HDInsightActivity.
 * @member {string} [getDebugInfo] Debug info option. Possible values include:
 * 'None', 'Always', 'Failure'
 * @member {object} [scriptPath] Script path. Type: string (or Expression with
 * resultType string).
 * @member {object} [scriptLinkedService] Script linked service reference.
 * @member {string} [scriptLinkedService.referenceName] Reference LinkedService
 * name.
 * @member {object} [defines] Allows user to specify defines for Hive job
 * request.
 */
export interface HDInsightHiveActivity extends ExecutionActivity {
  storageLinkedServices?: LinkedServiceReference[];
  arguments?: any[];
  getDebugInfo?: string;
  scriptPath?: any;
  scriptLinkedService?: LinkedServiceReference;
  defines?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the RedirectIncompatibleRowSettings class.
 * @constructor
 * Redirect incompatible row settings
 *
 * @member {object} linkedServiceName Name of the Azure Storage or Storage SAS
 * linked service used for redirecting incompatible row. Must be specified if
 * redirectIncompatibleRowSettings is specified. Type: string (or Expression
 * with resultType string).
 * @member {object} [path] The path to storage for storing the redirect
 * incompatible row data Type: string (or Expression with resultType string).
 */
export interface RedirectIncompatibleRowSettings {
  linkedServiceName: any;
  path?: any;
}

/**
 * @class
 * Initializes a new instance of the StagingSettings class.
 * @constructor
 * Staging settings.
 *
 * @member {object} linkedServiceName Staging linked service reference.
 * @member {string} [linkedServiceName.referenceName] Reference LinkedService
 * name.
 * @member {object} [path] The path to storage for storing the interim data.
 * Type: string (or Expression with resultType string).
 * @member {object} [enableCompression] Specifies whether to use compression
 * when copying data via an interim staging. Default value is false. Type:
 * boolean (or Expression with resultType boolean).
 */
export interface StagingSettings {
  linkedServiceName: LinkedServiceReference;
  path?: any;
  enableCompression?: any;
}

/**
 * @class
 * Initializes a new instance of the CopyTranslator class.
 * @constructor
 * A copy activity translator.
 *
 * @member {string} type Polymorphic Discriminator
 */
export interface CopyTranslator {
  type: string;
}

/**
 * @class
 * Initializes a new instance of the TabularTranslator class.
 * @constructor
 * A copy activity tabular translator.
 *
 * @member {object} [columnMappings] Column mappings. Type: string (or
 * Expression with resultType string).
 */
export interface TabularTranslator extends CopyTranslator {
  columnMappings?: any;
}

/**
 * @class
 * Initializes a new instance of the CopySink class.
 * @constructor
 * A copy activity sink.
 *
 * @member {object} [writeBatchSize] Write batch size. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [writeBatchTimeout] Write batch timeout. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [sinkRetryCount] Sink retry count. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [sinkRetryWait] Sink retry wait. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} type Polymorphic Discriminator
 */
export interface CopySink {
  writeBatchSize?: any;
  writeBatchTimeout?: any;
  sinkRetryCount?: any;
  sinkRetryWait?: any;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the OdbcSink class.
 * @constructor
 * A copy activity ODBC sink.
 *
 * @member {object} [preCopyScript] A query to execute before starting the
 * copy. Type: string (or Expression with resultType string).
 */
export interface OdbcSink extends CopySink {
  preCopyScript?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureSearchIndexSink class.
 * @constructor
 * A copy activity Azure Search Index sink.
 *
 * @member {string} [writeBehavior] Specify the write behavior when upserting
 * documents into Azure Search Index. Possible values include: 'Merge',
 * 'Upload'
 */
export interface AzureSearchIndexSink extends CopySink {
  writeBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureDataLakeStoreSink class.
 * @constructor
 * A copy activity Azure Data Lake Store sink.
 *
 * @member {string} [copyBehavior] The type of copy behavior for copy sink.
 * Possible values include: 'PreserveHierarchy', 'FlattenHierarchy',
 * 'MergeFiles'
 */
export interface AzureDataLakeStoreSink extends CopySink {
  copyBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the OracleSink class.
 * @constructor
 * A copy activity Oracle sink.
 *
 * @member {object} [preCopyScript] SQL pre-copy script. Type: string (or
 * Expression with resultType string).
 */
export interface OracleSink extends CopySink {
  preCopyScript?: any;
}

/**
 * @class
 * Initializes a new instance of the PolybaseSettings class.
 * @constructor
 * PolyBase settings.
 *
 * @member {string} [rejectType] Reject type. Possible values include: 'value',
 * 'percentage'
 * @member {object} [rejectValue] Specifies the value or the percentage of rows
 * that can be rejected before the query fails. Type: number (or Expression
 * with resultType number), minimum: 0.
 * @member {object} [rejectSampleValue] Determines the number of rows to
 * attempt to retrieve before the PolyBase recalculates the percentage of
 * rejected rows. Type: integer (or Expression with resultType integer),
 * minimum: 0.
 * @member {object} [useTypeDefault] Specifies how to handle missing values in
 * delimited text files when PolyBase retrieves data from the text file. Type:
 * boolean (or Expression with resultType boolean).
 */
export interface PolybaseSettings {
  rejectType?: string;
  rejectValue?: any;
  rejectSampleValue?: any;
  useTypeDefault?: any;
}

/**
 * @class
 * Initializes a new instance of the SqlDWSink class.
 * @constructor
 * A copy activity SQL Data Warehouse sink.
 *
 * @member {object} [preCopyScript] SQL pre-copy script. Type: string (or
 * Expression with resultType string).
 * @member {object} [allowPolyBase] Indicates to use PolyBase to copy data into
 * SQL Data Warehouse when applicable. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [polyBaseSettings] Specifies PolyBase-related settings when
 * allowPolyBase is true.
 * @member {string} [polyBaseSettings.rejectType] Reject type. Possible values
 * include: 'value', 'percentage'
 * @member {object} [polyBaseSettings.rejectValue] Specifies the value or the
 * percentage of rows that can be rejected before the query fails. Type: number
 * (or Expression with resultType number), minimum: 0.
 * @member {object} [polyBaseSettings.rejectSampleValue] Determines the number
 * of rows to attempt to retrieve before the PolyBase recalculates the
 * percentage of rejected rows. Type: integer (or Expression with resultType
 * integer), minimum: 0.
 * @member {object} [polyBaseSettings.useTypeDefault] Specifies how to handle
 * missing values in delimited text files when PolyBase retrieves data from the
 * text file. Type: boolean (or Expression with resultType boolean).
 */
export interface SqlDWSink extends CopySink {
  preCopyScript?: any;
  allowPolyBase?: any;
  polyBaseSettings?: PolybaseSettings;
}

/**
 * @class
 * Initializes a new instance of the SqlSink class.
 * @constructor
 * A copy activity SQL sink.
 *
 * @member {object} [sqlWriterStoredProcedureName] SQL writer stored procedure
 * name. Type: string (or Expression with resultType string).
 * @member {object} [sqlWriterTableType] SQL writer table type. Type: string
 * (or Expression with resultType string).
 * @member {object} [preCopyScript] SQL pre-copy script. Type: string (or
 * Expression with resultType string).
 * @member {object} [storedProcedureParameters] SQL stored procedure
 * parameters.
 */
export interface SqlSink extends CopySink {
  sqlWriterStoredProcedureName?: any;
  sqlWriterTableType?: any;
  preCopyScript?: any;
  storedProcedureParameters?: { [propertyName: string]: StoredProcedureParameter };
}

/**
 * @class
 * Initializes a new instance of the DocumentDbCollectionSink class.
 * @constructor
 * A copy activity Document Database Collection sink.
 *
 * @member {object} [nestingSeparator] Nested properties separator. Default is
 * . (dot). Type: string (or Expression with resultType string).
 */
export interface DocumentDbCollectionSink extends CopySink {
  nestingSeparator?: any;
}

/**
 * @class
 * Initializes a new instance of the FileSystemSink class.
 * @constructor
 * A copy activity file system sink.
 *
 * @member {string} [copyBehavior] The type of copy behavior for copy sink.
 * Possible values include: 'PreserveHierarchy', 'FlattenHierarchy',
 * 'MergeFiles'
 */
export interface FileSystemSink extends CopySink {
  copyBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the BlobSink class.
 * @constructor
 * A copy activity Azure Blob sink.
 *
 * @member {object} [blobWriterOverwriteFiles] Blob writer overwrite files.
 * Type: boolean (or Expression with resultType boolean).
 * @member {object} [blobWriterDateTimeFormat] Blob writer date time format.
 * Type: string (or Expression with resultType string).
 * @member {object} [blobWriterAddHeader] Blob writer add header. Type: boolean
 * (or Expression with resultType boolean).
 * @member {string} [copyBehavior] The type of copy behavior for copy sink.
 * Possible values include: 'PreserveHierarchy', 'FlattenHierarchy',
 * 'MergeFiles'
 */
export interface BlobSink extends CopySink {
  blobWriterOverwriteFiles?: any;
  blobWriterDateTimeFormat?: any;
  blobWriterAddHeader?: any;
  copyBehavior?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureTableSink class.
 * @constructor
 * A copy activity Azure Table sink.
 *
 * @member {object} [azureTableDefaultPartitionKeyValue] Azure Table default
 * partition key value. Type: string (or Expression with resultType string).
 * @member {object} [azureTablePartitionKeyName] Azure Table partition key
 * name. Type: string (or Expression with resultType string).
 * @member {object} [azureTableRowKeyName] Azure Table row key name. Type:
 * string (or Expression with resultType string).
 * @member {object} [azureTableInsertType] Azure Table insert type. Type:
 * string (or Expression with resultType string).
 */
export interface AzureTableSink extends CopySink {
  azureTableDefaultPartitionKeyValue?: any;
  azureTablePartitionKeyName?: any;
  azureTableRowKeyName?: any;
  azureTableInsertType?: any;
}

/**
 * @class
 * Initializes a new instance of the AzureQueueSink class.
 * @constructor
 * A copy activity Azure Queue sink.
 *
 */
export interface AzureQueueSink extends CopySink {
}

/**
 * @class
 * Initializes a new instance of the CopyActivity class.
 * @constructor
 * Copy activity.
 *
 * @member {object} source Copy activity source.
 * @member {object} [source.sourceRetryCount] Source retry count. Type: integer
 * (or Expression with resultType integer).
 * @member {object} [source.sourceRetryWait] Source retry wait. Type: string
 * (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} [source.type] Polymorphic Discriminator
 * @member {object} sink Copy activity sink.
 * @member {object} [sink.writeBatchSize] Write batch size. Type: integer (or
 * Expression with resultType integer), minimum: 0.
 * @member {object} [sink.writeBatchTimeout] Write batch timeout. Type: string
 * (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {object} [sink.sinkRetryCount] Sink retry count. Type: integer (or
 * Expression with resultType integer).
 * @member {object} [sink.sinkRetryWait] Sink retry wait. Type: string (or
 * Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {string} [sink.type] Polymorphic Discriminator
 * @member {object} [translator] Copy activity translator. If not specificed,
 * tabular translator is used.
 * @member {string} [translator.type] Polymorphic Discriminator
 * @member {object} [enableStaging] Specifies whether to copy data via an
 * interim staging. Default value is false. Type: boolean (or Expression with
 * resultType boolean).
 * @member {object} [stagingSettings] Specifies interim staging settings when
 * EnableStaging is true.
 * @member {object} [stagingSettings.linkedServiceName] Staging linked service
 * reference.
 * @member {string} [stagingSettings.linkedServiceName.referenceName] Reference
 * LinkedService name.
 * @member {object} [stagingSettings.path] The path to storage for storing the
 * interim data. Type: string (or Expression with resultType string).
 * @member {object} [stagingSettings.enableCompression] Specifies whether to
 * use compression when copying data via an interim staging. Default value is
 * false. Type: boolean (or Expression with resultType boolean).
 * @member {object} [parallelCopies] Maximum number of concurrent sessions
 * opened on the source or sink to avoid overloading the data store. Type:
 * integer (or Expression with resultType integer), minimum: 0.
 * @member {object} [cloudDataMovementUnits] Maximum number of cloud data
 * movement units that can be used to perform this data movement. Type: integer
 * (or Expression with resultType integer), minimum: 0.
 * @member {object} [enableSkipIncompatibleRow] Whether to skip incompatible
 * row. Default value is false. Type: boolean (or Expression with resultType
 * boolean).
 * @member {object} [redirectIncompatibleRowSettings] Redirect incompatible row
 * settings when EnableSkipIncompatibleRow is true.
 * @member {object} [redirectIncompatibleRowSettings.linkedServiceName] Name of
 * the Azure Storage or Storage SAS linked service used for redirecting
 * incompatible row. Must be specified if redirectIncompatibleRowSettings is
 * specified. Type: string (or Expression with resultType string).
 * @member {object} [redirectIncompatibleRowSettings.path] The path to storage
 * for storing the redirect incompatible row data Type: string (or Expression
 * with resultType string).
 * @member {array} [inputs] List of inputs for the activity.
 * @member {array} [outputs] List of outputs for the activity.
 */
export interface CopyActivity extends ExecutionActivity {
  source: CopySource;
  sink: CopySink;
  translator?: CopyTranslator;
  enableStaging?: any;
  stagingSettings?: StagingSettings;
  parallelCopies?: any;
  cloudDataMovementUnits?: any;
  enableSkipIncompatibleRow?: any;
  redirectIncompatibleRowSettings?: RedirectIncompatibleRowSettings;
  inputs?: DatasetReference[];
  outputs?: DatasetReference[];
}

/**
 * @class
 * Initializes a new instance of the ControlActivity class.
 * @constructor
 * Base class for all control activities like IfCondition, ForEach , Until.
 *
 */
export interface ControlActivity extends Activity {
}

/**
 * @class
 * Initializes a new instance of the UntilActivity class.
 * @constructor
 * This activity executes inner activities until the specified boolean
 * expression results to true or timeout is reached, whichever is earlier.
 *
 * @member {object} expression An expression that would evaluate to Boolean.
 * The loop will continue until this expression evaluates to true
 * @member {string} [expression.value] Expression value.
 * @member {object} [timeout] Specifies the timeout for the activity to run. If
 * there is no value specified, it takes the value of TimeSpan.FromDays(7)
 * which is 1 week as default. Type: string (or Expression with resultType
 * string), pattern: ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * Type: string (or Expression with resultType string), pattern:
 * ((\d+)\.)?(\d\d):(60|([0-5][0-9])):(60|([0-5][0-9])).
 * @member {array} activities List of activities to execute.
 */
export interface UntilActivity extends ControlActivity {
  expression: Expression;
  timeout?: any;
  activities: Activity[];
}

/**
 * @class
 * Initializes a new instance of the WaitActivity class.
 * @constructor
 * This activity suspends pipeline execution for the specified interval.
 *
 * @member {number} waitTimeInSeconds Duration in seconds.
 */
export interface WaitActivity extends ControlActivity {
  waitTimeInSeconds: number;
}

/**
 * @class
 * Initializes a new instance of the ForEachActivity class.
 * @constructor
 * This activity is used for iterating over a collection and execute given
 * activities.
 *
 * @member {boolean} [isSequential] Should the loop be executed in sequence or
 * in parallel (max 20)
 * @member {object} items Collection to iterate.
 * @member {string} [items.value] Expression value.
 * @member {array} activities List of activities to execute .
 */
export interface ForEachActivity extends ControlActivity {
  isSequential?: boolean;
  items: Expression;
  activities: Activity[];
}

/**
 * @class
 * Initializes a new instance of the IfConditionActivity class.
 * @constructor
 * This activity evaluates a boolean expression and executes either the
 * activities under the ifTrueActivities property or the ifFalseActivities
 * property depending on the result of the expression.
 *
 * @member {object} expression An expression that would evaluate to Boolean.
 * This is used to determine the block of activities (ifTrueActivities or
 * ifFalseActivities) that will be executed.
 * @member {string} [expression.value] Expression value.
 * @member {array} [ifTrueActivities] List of activities to execute if
 * expression is evaluated to true. This is an optional property and if not
 * provided, the activity will exit without any action.
 * @member {array} [ifFalseActivities] List of activities to execute if
 * expression is evaluated to false. This is an optional property and if not
 * provided, the activity will exit without any action.
 */
export interface IfConditionActivity extends ControlActivity {
  expression: Expression;
  ifTrueActivities?: Activity[];
  ifFalseActivities?: Activity[];
}

/**
 * @class
 * Initializes a new instance of the ExecutePipelineActivity class.
 * @constructor
 * Execute pipeline activity.
 *
 * @member {object} pipelineProperty Pipeline reference.
 * @member {string} [pipelineProperty.referenceName] Reference pipeline name.
 * @member {string} [pipelineProperty.name] Reference name.
 * @member {object} [parameters] Pipeline parameters.
 * @member {boolean} [waitOnCompletion] Defines whether activity execution will
 * wait for the dependent pipeline execution to finish. Default is false.
 */
export interface ExecutePipelineActivity extends ControlActivity {
  pipelineProperty: PipelineReference;
  parameters?: { [propertyName: string]: any };
  waitOnCompletion?: boolean;
}

/**
 * @class
 * Initializes a new instance of the SelfHostedIntegrationRuntimeNode class.
 * @constructor
 * Properties of Self-hosted integration runtime node.
 *
 * @member {string} [nodeName] Name of the integration runtime node.
 * @member {string} [machineName] Machine name of the integration runtime node.
 * @member {string} [hostServiceUri] URI for the host machine of the
 * integration runtime.
 * @member {string} [status] Status of the integration runtime node. Possible
 * values include: 'NeedRegistration', 'Online', 'Limited', 'Offline',
 * 'Upgrading', 'Initializing', 'InitializeFailed'
 * @member {object} [capabilities] The integration runtime capabilities
 * dictionary
 * @member {string} [versionStatus] Status of the integration runtime node
 * version.
 * @member {string} [version] Version of the integration runtime node.
 * @member {date} [registerTime] The time at which the integration runtime node
 * was registered in ISO8601 format.
 * @member {date} [lastConnectTime] The most recent time at which the
 * integration runtime was connected in ISO8601 format.
 * @member {date} [expiryTime] The time at which the integration runtime will
 * expire in ISO8601 format.
 * @member {date} [lastStartTime] The time the node last started up.
 * @member {date} [lastStopTime] The integration runtime node last stop time.
 * @member {string} [lastUpdateResult] The result of the last integration
 * runtime node update. Possible values include: 'Succeed', 'Fail'
 * @member {date} [lastStartUpdateTime] The last time for the integration
 * runtime node update start.
 * @member {date} [lastEndUpdateTime] The last time for the integration runtime
 * node update end.
 * @member {boolean} [isActiveDispatcher] Indicates whether this node is the
 * active dispatcher for integration runtime requests.
 * @member {number} [concurrentJobsLimit] Maximum concurrent jobs on the
 * integration runtime node.
 * @member {number} [maxConcurrentJobs] The maximum concurrent jobs in this
 * integration runtime.
 */
export interface SelfHostedIntegrationRuntimeNode {
  readonly nodeName?: string;
  readonly machineName?: string;
  readonly hostServiceUri?: string;
  readonly status?: string;
  readonly capabilities?: { [propertyName: string]: string };
  readonly versionStatus?: string;
  readonly version?: string;
  readonly registerTime?: Date;
  readonly lastConnectTime?: Date;
  readonly expiryTime?: Date;
  readonly lastStartTime?: Date;
  readonly lastStopTime?: Date;
  readonly lastUpdateResult?: string;
  readonly lastStartUpdateTime?: Date;
  readonly lastEndUpdateTime?: Date;
  readonly isActiveDispatcher?: boolean;
  readonly concurrentJobsLimit?: number;
  readonly maxConcurrentJobs?: number;
}

/**
 * @class
 * Initializes a new instance of the SelfHostedIntegrationRuntimeStatus class.
 * @constructor
 * Self-hosted integration runtime status.
 *
 * @member {date} [createTime] The time at which the integration runtime was
 * created, in ISO8601 format.
 * @member {string} [taskQueueId] The task queue id of the integration runtime.
 * @member {string} [internalChannelEncryption] It is used to set the
 * encryption mode for node-node communication channel (when more than 2
 * self-hosted integration runtime nodes exist). Possible values include:
 * 'NotSet', 'SslEncrypted', 'NotEncrypted'
 * @member {string} [version] Version of the integration runtime.
 * @member {array} [nodes] The list of nodes for this integration runtime.
 * @member {date} [scheduledUpdateDate] The date at which the integration
 * runtime will be scheduled to update, in ISO8601 format.
 * @member {string} [updateDelayOffset] The time in the date scheduled by
 * service to update the integration runtime, e.g., PT03H is 3 hours
 * @member {string} [localTimeZoneOffset] The local time zone offset in hours.
 * @member {object} [capabilities] Object with additional information about
 * integration runtime capabilities.
 * @member {array} [serviceUrls] The URLs for the services used in integration
 * runtime backend service.
 * @member {string} [autoUpdate] Whether Self-hosted integration runtime auto
 * update has been turned on. Possible values include: 'On', 'Off'
 */
export interface SelfHostedIntegrationRuntimeStatus extends IntegrationRuntimeStatus {
  readonly createTime?: Date;
  readonly taskQueueId?: string;
  readonly internalChannelEncryption?: string;
  readonly version?: string;
  nodes?: SelfHostedIntegrationRuntimeNode[];
  readonly scheduledUpdateDate?: Date;
  readonly updateDelayOffset?: string;
  readonly localTimeZoneOffset?: string;
  readonly capabilities?: { [propertyName: string]: string };
  readonly serviceUrls?: string[];
  readonly autoUpdate?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeOperationResult class.
 * @constructor
 * Properties of managed integration runtime operation result.
 *
 * @member {string} [type] The operation type. Could be start or stop.
 * @member {date} [startTime] The start time of the operation.
 * @member {string} [result] The operation result.
 * @member {string} [errorCode] The error code.
 * @member {array} [parameters] Managed integration runtime error parameters.
 */
export interface ManagedIntegrationRuntimeOperationResult {
  readonly type?: string;
  readonly startTime?: Date;
  readonly result?: string;
  readonly errorCode?: string;
  readonly parameters?: string[];
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeError class.
 * @constructor
 * Error definition for managed integration runtime.
 *
 * @member {date} [time] The time when the error occurred.
 * @member {string} [code] Error code.
 * @member {array} [parameters] Managed integration runtime error parameters.
 * @member {string} [message] Error message.
 */
export interface ManagedIntegrationRuntimeError {
  readonly time?: Date;
  readonly code?: string;
  readonly parameters?: string[];
  readonly message?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeNode class.
 * @constructor
 * Properties of integration runtime node.
 *
 * @member {string} [nodeId] The managed integration runtime node id.
 * @member {string} [status] The managed integration runtime node status.
 * Possible values include: 'Starting', 'Available', 'Recycling', 'Unavailable'
 * @member {array} [errors] The errors that occurred on this integration
 * runtime node.
 */
export interface ManagedIntegrationRuntimeNode {
  readonly nodeId?: string;
  readonly status?: string;
  errors?: ManagedIntegrationRuntimeError[];
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntimeStatus class.
 * @constructor
 * Managed integration runtime status.
 *
 * @member {date} [createTime] The time at which the integration runtime was
 * created, in ISO8601 format.
 * @member {array} [nodes] The list of nodes for managed integration runtime.
 * @member {array} [otherErrors] The errors that occurred on this integration
 * runtime.
 * @member {object} [lastOperation] The last operation result that occurred on
 * this integration runtime.
 * @member {string} [lastOperation.type] The operation type. Could be start or
 * stop.
 * @member {date} [lastOperation.startTime] The start time of the operation.
 * @member {string} [lastOperation.result] The operation result.
 * @member {string} [lastOperation.errorCode] The error code.
 * @member {array} [lastOperation.parameters] Managed integration runtime error
 * parameters.
 */
export interface ManagedIntegrationRuntimeStatus extends IntegrationRuntimeStatus {
  readonly createTime?: Date;
  readonly nodes?: ManagedIntegrationRuntimeNode[];
  readonly otherErrors?: ManagedIntegrationRuntimeError[];
  readonly lastOperation?: ManagedIntegrationRuntimeOperationResult;
}

/**
 * @class
 * Initializes a new instance of the SelfHostedIntegrationRuntime class.
 * @constructor
 * Self-hosted integration runtime.
 *
 */
export interface SelfHostedIntegrationRuntime extends IntegrationRuntime {
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeSsisCatalogInfo class.
 * @constructor
 * Catalog information for managed dedicated integration runtime.
 *
 * @member {string} [catalogServerEndpoint] The catalog database server URL.
 * @member {string} [catalogAdminUserName] The administrator user name of
 * catalog database.
 * @member {object} [catalogAdminPassword] The password of the administrator
 * user account of the catalog database.
 * @member {string} [catalogAdminPassword.value] Value of secure string.
 * @member {string} [catalogPricingTier] The pricing tier for the catalog
 * database. The valid values could be found in
 * https://azure.microsoft.com/en-us/pricing/details/sql-database/. Possible
 * values include: 'Basic', 'Standard', 'Premium', 'PremiumRS'
 */
export interface IntegrationRuntimeSsisCatalogInfo {
  catalogServerEndpoint?: string;
  catalogAdminUserName?: string;
  catalogAdminPassword?: SecureString;
  catalogPricingTier?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeSsisProperties class.
 * @constructor
 * SSIS properties for managed integration runtime.
 *
 * @member {object} [catalogInfo] Catalog information for managed dedicated
 * integration runtime.
 * @member {string} [catalogInfo.catalogServerEndpoint] The catalog database
 * server URL.
 * @member {string} [catalogInfo.catalogAdminUserName] The administrator user
 * name of catalog database.
 * @member {object} [catalogInfo.catalogAdminPassword] The password of the
 * administrator user account of the catalog database.
 * @member {string} [catalogInfo.catalogAdminPassword.value] Value of secure
 * string.
 * @member {string} [catalogInfo.catalogPricingTier] The pricing tier for the
 * catalog database. The valid values could be found in
 * https://azure.microsoft.com/en-us/pricing/details/sql-database/. Possible
 * values include: 'Basic', 'Standard', 'Premium', 'PremiumRS'
 */
export interface IntegrationRuntimeSsisProperties {
  catalogInfo?: IntegrationRuntimeSsisCatalogInfo;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeVNetProperties class.
 * @constructor
 * VNet properties for managed integration runtime.
 *
 * @member {string} [vNetId] The ID of the VNet that this integration runtime
 * will join.
 * @member {string} [subnet] The name of the subnet this integration runtime
 * will join.
 */
export interface IntegrationRuntimeVNetProperties {
  vNetId?: string;
  subnet?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeComputeProperties class.
 * @constructor
 * The compute resource properties for managed integration runtime.
 *
 * @member {string} [location] The location for managed integration runtime.
 * The supported regions could be found on
 * https://docs.microsoft.com/en-us/azure/data-factory/data-factory-data-movement-activities
 * @member {string} [nodeSize] The node size requirement to managed integration
 * runtime.
 * @member {number} [numberOfNodes] The required number of nodes for managed
 * integration runtime.
 * @member {number} [maxParallelExecutionsPerNode] Maximum parallel executions
 * count per node for managed integration runtime.
 * @member {object} [vNetProperties] VNet properties for managed integration
 * runtime.
 * @member {string} [vNetProperties.vNetId] The ID of the VNet that this
 * integration runtime will join.
 * @member {string} [vNetProperties.subnet] The name of the subnet this
 * integration runtime will join.
 */
export interface IntegrationRuntimeComputeProperties {
  location?: string;
  nodeSize?: string;
  numberOfNodes?: number;
  maxParallelExecutionsPerNode?: number;
  vNetProperties?: IntegrationRuntimeVNetProperties;
}

/**
 * @class
 * Initializes a new instance of the ManagedIntegrationRuntime class.
 * @constructor
 * Managed integration runtime, including managed elastic and managed dedicated
 * integration runtimes.
 *
 * @member {string} [state] Integration runtime state, only valid for managed
 * dedicated integration runtime. Possible values include: 'Initial',
 * 'Stopped', 'Started', 'Starting', 'Stopping', 'NeedRegistration', 'Online',
 * 'Limited', 'Offline'
 * @member {object} [computeProperties] The compute resource for managed
 * integration runtime.
 * @member {string} [computeProperties.location] The location for managed
 * integration runtime. The supported regions could be found on
 * https://docs.microsoft.com/en-us/azure/data-factory/data-factory-data-movement-activities
 * @member {string} [computeProperties.nodeSize] The node size requirement to
 * managed integration runtime.
 * @member {number} [computeProperties.numberOfNodes] The required number of
 * nodes for managed integration runtime.
 * @member {number} [computeProperties.maxParallelExecutionsPerNode] Maximum
 * parallel executions count per node for managed integration runtime.
 * @member {object} [computeProperties.vNetProperties] VNet properties for
 * managed integration runtime.
 * @member {string} [computeProperties.vNetProperties.vNetId] The ID of the
 * VNet that this integration runtime will join.
 * @member {string} [computeProperties.vNetProperties.subnet] The name of the
 * subnet this integration runtime will join.
 * @member {object} [ssisProperties] SSIS properties for managed integration
 * runtime.
 * @member {object} [ssisProperties.catalogInfo] Catalog information for
 * managed dedicated integration runtime.
 * @member {string} [ssisProperties.catalogInfo.catalogServerEndpoint] The
 * catalog database server URL.
 * @member {string} [ssisProperties.catalogInfo.catalogAdminUserName] The
 * administrator user name of catalog database.
 * @member {object} [ssisProperties.catalogInfo.catalogAdminPassword] The
 * password of the administrator user account of the catalog database.
 * @member {string} [ssisProperties.catalogInfo.catalogAdminPassword.value]
 * Value of secure string.
 * @member {string} [ssisProperties.catalogInfo.catalogPricingTier] The pricing
 * tier for the catalog database. The valid values could be found in
 * https://azure.microsoft.com/en-us/pricing/details/sql-database/. Possible
 * values include: 'Basic', 'Standard', 'Premium', 'PremiumRS'
 */
export interface ManagedIntegrationRuntime extends IntegrationRuntime {
  readonly state?: string;
  computeProperties?: IntegrationRuntimeComputeProperties;
  ssisProperties?: IntegrationRuntimeSsisProperties;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeNodeMonitoringData class.
 * @constructor
 * Monitoring data for integration runtime node.
 *
 * @member {string} [nodeName] Name of the integration runtime node.
 * @member {number} [availableMemoryInMB] Available memory (MB) on the
 * integration runtime node.
 * @member {number} [cpuUtilization] CPU percentage on the integration runtime
 * node.
 * @member {number} [concurrentJobsLimit] Maximum concurrent jobs on the
 * integration runtime node.
 * @member {number} [concurrentJobsRunning] The number of jobs currently
 * running on the integration runtime node.
 * @member {number} [maxConcurrentJobs] The maximum concurrent jobs in this
 * integration runtime.
 * @member {number} [sentBytes] Sent bytes on the integration runtime node.
 * @member {number} [receivedBytes] Received bytes on the integration runtime
 * node.
 */
export interface IntegrationRuntimeNodeMonitoringData {
  readonly nodeName?: string;
  readonly availableMemoryInMB?: number;
  readonly cpuUtilization?: number;
  readonly concurrentJobsLimit?: number;
  readonly concurrentJobsRunning?: number;
  readonly maxConcurrentJobs?: number;
  readonly sentBytes?: number;
  readonly receivedBytes?: number;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeMonitoringData class.
 * @constructor
 * Get monitoring data response.
 *
 * @member {string} [name] Integration runtime name.
 * @member {array} [nodes] Integration runtime node monitoring data.
 */
export interface IntegrationRuntimeMonitoringData {
  name?: string;
  nodes?: IntegrationRuntimeNodeMonitoringData[];
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeRemoveNodeRequest class.
 * @constructor
 * Request to remove a node.
 *
 * @member {string} [nodeName] The name of the node to be removed.
 */
export interface IntegrationRuntimeRemoveNodeRequest {
  nodeName?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeAuthKeys class.
 * @constructor
 * The integration runtime authentication keys.
 *
 * @member {string} [authKey1] The primary integration runtime authentication
 * key.
 * @member {string} [authKey2] The secondary integration runtime authentication
 * key.
 */
export interface IntegrationRuntimeAuthKeys {
  authKey1?: string;
  authKey2?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeRegenerateKeyParameters class.
 * @constructor
 * Parameters to regenerate the authentication key.
 *
 * @member {string} [keyName] The name of the authentication key to regenerate.
 * Possible values include: 'authKey1', 'authKey2'
 */
export interface IntegrationRuntimeRegenerateKeyParameters {
  keyName?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeConnectionInfo class.
 * @constructor
 * Connection information for encrypting the on-premises data source
 * credentials.
 *
 * @member {string} [serviceToken] The token generated in service. Callers use
 * this token to authenticate to integration runtime.
 * @member {string} [identityCertThumbprint] The integration runtime SSL
 * certificate thumbprint. Click-Once application uses it to do server
 * validation.
 * @member {string} [hostServiceUri] The on-premises integration runtime host
 * URL.
 * @member {string} [version] The integration runtime version.
 * @member {string} [publicKey] The public key for encrypting a credential when
 * transferring the credential to the integration runtime.
 * @member {boolean} [isIdentityCertExprired] Whether the identity certificate
 * is expired.
 */
export interface IntegrationRuntimeConnectionInfo {
  readonly serviceToken?: string;
  readonly identityCertThumbprint?: string;
  readonly hostServiceUri?: string;
  readonly version?: string;
  readonly publicKey?: string;
  readonly isIdentityCertExprired?: boolean;
}


/**
 * @class
 * Initializes a new instance of the FactoryListResponse class.
 * @constructor
 * A list of factory resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface FactoryListResponse extends Array<Factory> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the IntegrationRuntimeListResponse class.
 * @constructor
 * A list of integration runtime resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface IntegrationRuntimeListResponse extends Array<IntegrationRuntimeResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the LinkedServiceListResponse class.
 * @constructor
 * A list of linked service resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface LinkedServiceListResponse extends Array<LinkedServiceResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DatasetListResponse class.
 * @constructor
 * A list of dataset resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface DatasetListResponse extends Array<DatasetResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the PipelineListResponse class.
 * @constructor
 * A list of pipeline resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface PipelineListResponse extends Array<PipelineResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityRunsListResponse class.
 * @constructor
 * A list activity runs.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface ActivityRunsListResponse extends Array<ActivityRun> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerListResponse class.
 * @constructor
 * A list of trigger resources.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface TriggerListResponse extends Array<TriggerResource> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the TriggerRunListResponse class.
 * @constructor
 * A list of trigger runs.
 *
 * @member {string} [nextLink] The link to the next page of results, if any
 * remaining results exist.
 */
export interface TriggerRunListResponse extends Array<TriggerRun> {
  nextLink?: string;
}
