/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes the cluster.
 *
 * @extends models['Resource']
 */
class Cluster extends models['Resource'] {
  /**
   * Create a Cluster.
   * @member {string} [etag] The ETag for the resource
   * @member {object} [properties] The properties of the cluster.
   * @member {string} [properties.clusterVersion] The version of the cluster.
   * @member {string} [properties.osType] The type of operating system.
   * Possible values include: 'Windows', 'Linux'
   * @member {string} [properties.tier] The cluster tier. Possible values
   * include: 'Standard', 'Premium'
   * @member {object} [properties.clusterDefinition] The cluster definition.
   * @member {string} [properties.clusterDefinition.blueprint] The link to the
   * blueprint.
   * @member {string} [properties.clusterDefinition.kind] The type of cluster.
   * @member {object} [properties.clusterDefinition.componentVersion] The
   * versions of different services in the cluster.
   * @member {object} [properties.clusterDefinition.configurations] The cluster
   * configurations.
   * @member {object} [properties.securityProfile] The security profile.
   * @member {string} [properties.securityProfile.directoryType] The directory
   * type. Possible values include: 'ActiveDirectory'
   * @member {string} [properties.securityProfile.domain] domain.
   * @member {string} [properties.securityProfile.organizationalUnitDN]
   * Distinguished Name for organizational unit.
   * @member {array} [properties.securityProfile.ldapsUrls] The LDAP URLs.
   * @member {string} [properties.securityProfile.domainUsername] The domain
   * admin user name.
   * @member {string} [properties.securityProfile.domainUserPassword] The
   * domain admin password.
   * @member {array} [properties.securityProfile.clusterUsersGroupDNs]
   * Optional. The Distinguished Names for cluster user groups
   * @member {object} [properties.computeProfile] The compute profile.
   * @member {array} [properties.computeProfile.roles] The list of roles in the
   * cluster.
   * @member {string} [properties.provisioningState] The provisioning state,
   * which only appears in the response. Possible values include: 'InProgress',
   * 'Failed', 'Succeeded', 'Canceled', 'Deleting'
   * @member {string} [properties.createdDate] The date on which the cluster
   * was created.
   * @member {string} [properties.clusterState] The state of the cluster.
   * @member {object} [properties.quotaInfo] The quota information.
   * @member {number} [properties.quotaInfo.coresUsed] The cores used by the
   * cluster.
   * @member {array} [properties.errors] The list of errors.
   * @member {array} [properties.connectivityEndpoints] The list of
   * connectivity endpoints.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Cluster
   *
   * @returns {object} metadata of Cluster
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Cluster',
      type: {
        name: 'Composite',
        className: 'Cluster',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          },
          properties: {
            required: false,
            serializedName: 'properties',
            type: {
              name: 'Composite',
              className: 'ClusterGetProperties'
            }
          }
        }
      }
    };
  }
}

module.exports = Cluster;
