/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The cluster create parameters.
 *
 */
class ClusterCreateProperties {
  /**
   * Create a ClusterCreateProperties.
   * @member {string} [clusterVersion] The version of the cluster.
   * @member {string} [osType] The type of operating system. Possible values
   * include: 'Windows', 'Linux'
   * @member {string} [tier] The cluster tier. Possible values include:
   * 'Standard', 'Premium'
   * @member {object} [clusterDefinition] The cluster definition.
   * @member {string} [clusterDefinition.blueprint] The link to the blueprint.
   * @member {string} [clusterDefinition.kind] The type of cluster.
   * @member {object} [clusterDefinition.componentVersion] The versions of
   * different services in the cluster.
   * @member {object} [clusterDefinition.configurations] The cluster
   * configurations.
   * @member {object} [securityProfile] The security profile.
   * @member {string} [securityProfile.directoryType] The directory type.
   * Possible values include: 'ActiveDirectory'
   * @member {string} [securityProfile.domain] domain.
   * @member {string} [securityProfile.organizationalUnitDN] Distinguished Name
   * for organizational unit.
   * @member {array} [securityProfile.ldapsUrls] The LDAP URLs.
   * @member {string} [securityProfile.domainUsername] The domain admin user
   * name.
   * @member {string} [securityProfile.domainUserPassword] The domain admin
   * password.
   * @member {array} [securityProfile.clusterUsersGroupDNs] Optional. The
   * Distinguished Names for cluster user groups
   * @member {object} [computeProfile] The compute profile.
   * @member {array} [computeProfile.roles] The list of roles in the cluster.
   * @member {object} [storageProfile] The storage profile.
   * @member {array} [storageProfile.storageaccounts] The list of storage
   * accounts in the cluster.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ClusterCreateProperties
   *
   * @returns {object} metadata of ClusterCreateProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ClusterCreateProperties',
      type: {
        name: 'Composite',
        className: 'ClusterCreateProperties',
        modelProperties: {
          clusterVersion: {
            required: false,
            serializedName: 'clusterVersion',
            type: {
              name: 'String'
            }
          },
          osType: {
            required: false,
            serializedName: 'osType',
            type: {
              name: 'Enum',
              allowedValues: [ 'Windows', 'Linux' ]
            }
          },
          tier: {
            required: false,
            serializedName: 'tier',
            type: {
              name: 'Enum',
              allowedValues: [ 'Standard', 'Premium' ]
            }
          },
          clusterDefinition: {
            required: false,
            serializedName: 'clusterDefinition',
            type: {
              name: 'Composite',
              className: 'ClusterDefinition'
            }
          },
          securityProfile: {
            required: false,
            serializedName: 'securityProfile',
            type: {
              name: 'Composite',
              className: 'SecurityProfile'
            }
          },
          computeProfile: {
            required: false,
            serializedName: 'computeProfile',
            type: {
              name: 'Composite',
              className: 'ComputeProfile'
            }
          },
          storageProfile: {
            required: false,
            serializedName: 'storageProfile',
            type: {
              name: 'Composite',
              className: 'StorageProfile'
            }
          }
        }
      }
    };
  }
}

module.exports = ClusterCreateProperties;
