/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The properties of cluster.
 *
 */
class ClusterGetProperties {
  /**
   * Create a ClusterGetProperties.
   * @member {string} [clusterVersion] The version of the cluster.
   * @member {string} [osType] The type of operating system. Possible values
   * include: 'Windows', 'Linux'
   * @member {string} [tier] The cluster tier. Possible values include:
   * 'Standard', 'Premium'
   * @member {object} clusterDefinition The cluster definition.
   * @member {string} [clusterDefinition.blueprint] The link to the blueprint.
   * @member {string} [clusterDefinition.kind] The type of cluster.
   * @member {object} [clusterDefinition.componentVersion] The versions of
   * different services in the cluster.
   * @member {object} [clusterDefinition.configurations] The cluster
   * configurations.
   * @member {object} [securityProfile] The security profile.
   * @member {string} [securityProfile.directoryType] The directory type.
   * Possible values include: 'ActiveDirectory'
   * @member {string} [securityProfile.domain] domain.
   * @member {string} [securityProfile.organizationalUnitDN] Distinguished Name
   * for organizational unit.
   * @member {array} [securityProfile.ldapsUrls] The LDAP URLs.
   * @member {string} [securityProfile.domainUsername] The domain admin user
   * name.
   * @member {string} [securityProfile.domainUserPassword] The domain admin
   * password.
   * @member {array} [securityProfile.clusterUsersGroupDNs] Optional. The
   * Distinguished Names for cluster user groups
   * @member {object} [computeProfile] The compute profile.
   * @member {array} [computeProfile.roles] The list of roles in the cluster.
   * @member {string} [provisioningState] The provisioning state, which only
   * appears in the response. Possible values include: 'InProgress', 'Failed',
   * 'Succeeded', 'Canceled', 'Deleting'
   * @member {string} [createdDate] The date on which the cluster was created.
   * @member {string} [clusterState] The state of the cluster.
   * @member {object} [quotaInfo] The quota information.
   * @member {number} [quotaInfo.coresUsed] The cores used by the cluster.
   * @member {array} [errors] The list of errors.
   * @member {array} [connectivityEndpoints] The list of connectivity
   * endpoints.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ClusterGetProperties
   *
   * @returns {object} metadata of ClusterGetProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ClusterGetProperties',
      type: {
        name: 'Composite',
        className: 'ClusterGetProperties',
        modelProperties: {
          clusterVersion: {
            required: false,
            serializedName: 'clusterVersion',
            type: {
              name: 'String'
            }
          },
          osType: {
            required: false,
            serializedName: 'osType',
            type: {
              name: 'Enum',
              allowedValues: [ 'Windows', 'Linux' ]
            }
          },
          tier: {
            required: false,
            serializedName: 'tier',
            type: {
              name: 'Enum',
              allowedValues: [ 'Standard', 'Premium' ]
            }
          },
          clusterDefinition: {
            required: true,
            serializedName: 'clusterDefinition',
            type: {
              name: 'Composite',
              className: 'ClusterDefinition'
            }
          },
          securityProfile: {
            required: false,
            serializedName: 'securityProfile',
            type: {
              name: 'Composite',
              className: 'SecurityProfile'
            }
          },
          computeProfile: {
            required: false,
            serializedName: 'computeProfile',
            type: {
              name: 'Composite',
              className: 'ComputeProfile'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'provisioningState',
            type: {
              name: 'Enum',
              allowedValues: [ 'InProgress', 'Failed', 'Succeeded', 'Canceled', 'Deleting' ]
            }
          },
          createdDate: {
            required: false,
            serializedName: 'createdDate',
            type: {
              name: 'String'
            }
          },
          clusterState: {
            required: false,
            serializedName: 'clusterState',
            type: {
              name: 'String'
            }
          },
          quotaInfo: {
            required: false,
            serializedName: 'quotaInfo',
            type: {
              name: 'Composite',
              className: 'QuotaInfo'
            }
          },
          errors: {
            required: false,
            serializedName: 'errors',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ErrorsElementType',
                  type: {
                    name: 'Composite',
                    className: 'Errors'
                  }
              }
            }
          },
          connectivityEndpoints: {
            required: false,
            serializedName: 'connectivityEndpoints',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ConnectivityEndpointElementType',
                  type: {
                    name: 'Composite',
                    className: 'ConnectivityEndpoint'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ClusterGetProperties;
