/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes a role on the cluster.
 *
 */
class Role {
  /**
   * Create a Role.
   * @member {string} [name] The name of the role.
   * @member {number} [minInstanceCount] The minimum instance count of the
   * cluster.
   * @member {number} [targetInstanceCount] The instance count of the cluster.
   * @member {object} [hardwareProfile] The hardware profile.
   * @member {string} [hardwareProfile.vmSize] The size of the VM
   * @member {object} [osProfile] The operating system profile.
   * @member {object} [osProfile.windowsOperatingSystemProfile] The Windows OS
   * profile.
   * @member {object} [osProfile.windowsOperatingSystemProfile.rdpSettings] The
   * RDP settings.
   * @member {string}
   * [osProfile.windowsOperatingSystemProfile.rdpSettings.username] The
   * username for the RDP user.
   * @member {string}
   * [osProfile.windowsOperatingSystemProfile.rdpSettings.password] The
   * password for the RDP user.
   * @member {date}
   * [osProfile.windowsOperatingSystemProfile.rdpSettings.expiryDate] The RDP
   * expiry date(YYYY-MM-DD).
   * @member {object} [osProfile.linuxOperatingSystemProfile] The Linux OS
   * profile.
   * @member {string} [osProfile.linuxOperatingSystemProfile.username] The
   * username.
   * @member {string} [osProfile.linuxOperatingSystemProfile.password] The
   * password.
   * @member {object} [osProfile.linuxOperatingSystemProfile.sshProfile] The
   * SSH profile.
   * @member {array}
   * [osProfile.linuxOperatingSystemProfile.sshProfile.publicKeys] The list of
   * SSH public keys.
   * @member {object} [virtualNetworkProfile] The virtual network profile.
   * @member {string} [virtualNetworkProfile.id] The ID of the virtual network.
   * @member {string} [virtualNetworkProfile.subnet] The name of the subnet.
   * @member {array} [scriptActions] The list of script actions on the role.
   */
  constructor() {
  }

  /**
   * Defines the metadata of Role
   *
   * @returns {object} metadata of Role
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Role',
      type: {
        name: 'Composite',
        className: 'Role',
        modelProperties: {
          name: {
            required: false,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          minInstanceCount: {
            required: false,
            serializedName: 'minInstanceCount',
            type: {
              name: 'Number'
            }
          },
          targetInstanceCount: {
            required: false,
            serializedName: 'targetInstanceCount',
            type: {
              name: 'Number'
            }
          },
          hardwareProfile: {
            required: false,
            serializedName: 'hardwareProfile',
            type: {
              name: 'Composite',
              className: 'HardwareProfile'
            }
          },
          osProfile: {
            required: false,
            serializedName: 'osProfile',
            type: {
              name: 'Composite',
              className: 'OsProfile'
            }
          },
          virtualNetworkProfile: {
            required: false,
            serializedName: 'virtualNetworkProfile',
            type: {
              name: 'Composite',
              className: 'VirtualNetworkProfile'
            }
          },
          scriptActions: {
            required: false,
            serializedName: 'scriptActions',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ScriptActionElementType',
                  type: {
                    name: 'Composite',
                    className: 'ScriptAction'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = Role;
