/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Clusters
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface Clusters {


    /**
     * Begins creating a new HDInsight cluster with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster create request.
     *
     * @param {string} [parameters.location] The location of the cluster.
     *
     * @param {object} [parameters.tags] The resource tags.
     *
     * @param {object} [parameters.properties] The cluster create parameters.
     *
     * @param {string} [parameters.properties.clusterVersion] The version of the
     * cluster.
     *
     * @param {string} [parameters.properties.osType] The type of operating system.
     * Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [parameters.properties.tier] The cluster tier. Possible
     * values include: 'Standard', 'Premium'
     *
     * @param {object} [parameters.properties.clusterDefinition] The cluster
     * definition.
     *
     * @param {string} [parameters.properties.clusterDefinition.blueprint] The link
     * to the blueprint.
     *
     * @param {string} [parameters.properties.clusterDefinition.kind] The type of
     * cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.componentVersion]
     * The versions of different services in the cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.configurations] The
     * cluster configurations.
     *
     * @param {object} [parameters.properties.securityProfile] The security
     * profile.
     *
     * @param {string} [parameters.properties.securityProfile.directoryType] The
     * directory type. Possible values include: 'ActiveDirectory'
     *
     * @param {string} [parameters.properties.securityProfile.domain] domain.
     *
     * @param {string} [parameters.properties.securityProfile.organizationalUnitDN]
     * Distinguished Name for organizational unit.
     *
     * @param {array} [parameters.properties.securityProfile.ldapsUrls] The LDAP
     * URLs.
     *
     * @param {string} [parameters.properties.securityProfile.domainUsername] The
     * domain admin user name.
     *
     * @param {string} [parameters.properties.securityProfile.domainUserPassword]
     * The domain admin password.
     *
     * @param {array} [parameters.properties.securityProfile.clusterUsersGroupDNs]
     * Optional. The Distinguished Names for cluster user groups
     *
     * @param {object} [parameters.properties.computeProfile] The compute profile.
     *
     * @param {array} [parameters.properties.computeProfile.roles] The list of
     * roles in the cluster.
     *
     * @param {object} [parameters.properties.storageProfile] The storage profile.
     *
     * @param {array} [parameters.properties.storageProfile.storageaccounts] The
     * list of storage accounts in the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Cluster>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Cluster>>;

    /**
     * Begins creating a new HDInsight cluster with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster create request.
     *
     * @param {string} [parameters.location] The location of the cluster.
     *
     * @param {object} [parameters.tags] The resource tags.
     *
     * @param {object} [parameters.properties] The cluster create parameters.
     *
     * @param {string} [parameters.properties.clusterVersion] The version of the
     * cluster.
     *
     * @param {string} [parameters.properties.osType] The type of operating system.
     * Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [parameters.properties.tier] The cluster tier. Possible
     * values include: 'Standard', 'Premium'
     *
     * @param {object} [parameters.properties.clusterDefinition] The cluster
     * definition.
     *
     * @param {string} [parameters.properties.clusterDefinition.blueprint] The link
     * to the blueprint.
     *
     * @param {string} [parameters.properties.clusterDefinition.kind] The type of
     * cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.componentVersion]
     * The versions of different services in the cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.configurations] The
     * cluster configurations.
     *
     * @param {object} [parameters.properties.securityProfile] The security
     * profile.
     *
     * @param {string} [parameters.properties.securityProfile.directoryType] The
     * directory type. Possible values include: 'ActiveDirectory'
     *
     * @param {string} [parameters.properties.securityProfile.domain] domain.
     *
     * @param {string} [parameters.properties.securityProfile.organizationalUnitDN]
     * Distinguished Name for organizational unit.
     *
     * @param {array} [parameters.properties.securityProfile.ldapsUrls] The LDAP
     * URLs.
     *
     * @param {string} [parameters.properties.securityProfile.domainUsername] The
     * domain admin user name.
     *
     * @param {string} [parameters.properties.securityProfile.domainUserPassword]
     * The domain admin password.
     *
     * @param {array} [parameters.properties.securityProfile.clusterUsersGroupDNs]
     * Optional. The Distinguished Names for cluster user groups
     *
     * @param {object} [parameters.properties.computeProfile] The compute profile.
     *
     * @param {array} [parameters.properties.computeProfile.roles] The list of
     * roles in the cluster.
     *
     * @param {object} [parameters.properties.storageProfile] The storage profile.
     *
     * @param {array} [parameters.properties.storageProfile.storageaccounts] The
     * list of storage accounts in the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Cluster} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Cluster} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Cluster} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Cluster>;
    create(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, callback: ServiceCallback<models.Cluster>): void;
    create(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Cluster>): void;


    /**
     * Patch HDInsight cluster with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster patch request.
     *
     * @param {object} [parameters.tags] The resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Cluster>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ClusterPatchParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Cluster>>;

    /**
     * Patch HDInsight cluster with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster patch request.
     *
     * @param {object} [parameters.tags] The resource tags.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Cluster} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Cluster} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Cluster} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, clusterName: string, parameters: models.ClusterPatchParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Cluster>;
    update(resourceGroupName: string, clusterName: string, parameters: models.ClusterPatchParameters, callback: ServiceCallback<models.Cluster>): void;
    update(resourceGroupName: string, clusterName: string, parameters: models.ClusterPatchParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Cluster>): void;


    /**
     * Begins deleting the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins deleting the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, clusterName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Gets the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Cluster>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Cluster>>;

    /**
     * Gets the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Cluster} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Cluster} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Cluster} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Cluster>;
    get(resourceGroupName: string, clusterName: string, callback: ServiceCallback<models.Cluster>): void;
    get(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Cluster>): void;


    /**
     * List the HDInsight clusters in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ClusterListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ClusterListResult>>;

    /**
     * List the HDInsight clusters in a resource group.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ClusterListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ClusterListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ClusterListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ClusterListResult>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.ClusterListResult>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ClusterListResult>): void;


    /**
     * Begins a resize operation on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for the resize operation.
     *
     * @param {number} [parameters.targetInstanceCount] The target instance count
     * for the operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    resizeWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins a resize operation on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for the resize operation.
     *
     * @param {number} [parameters.targetInstanceCount] The target instance count
     * for the operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    resize(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    resize(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, callback: ServiceCallback<void>): void;
    resize(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists HDInsight clusters under the subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ClusterListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ClusterListResult>>;

    /**
     * Lists HDInsight clusters under the subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ClusterListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ClusterListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ClusterListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ClusterListResult>;
    list(callback: ServiceCallback<models.ClusterListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ClusterListResult>): void;


    /**
     * Begins changing the RDP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The OS profile for RDP.
     *
     * @param {object} parameters.osProfile The definition of a data factory
     * gateway to be created or updated.
     *
     * @param {object} [parameters.osProfile.windowsOperatingSystemProfile] The
     * Windows OS profile.
     *
     * @param {object}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings] The RDP
     * settings.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.username]
     * The username for the RDP user.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.password]
     * The password for the RDP user.
     *
     * @param {date}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.expiryDate]
     * The RDP expiry date(YYYY-MM-DD).
     *
     * @param {object} [parameters.osProfile.linuxOperatingSystemProfile] The Linux
     * OS profile.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.username]
     * The username.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.password]
     * The password.
     *
     * @param {object}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile] The SSH
     * profile.
     *
     * @param {array}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile.publicKeys] The
     * list of SSH public keys.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    changeRdpSettingsWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins changing the RDP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The OS profile for RDP.
     *
     * @param {object} parameters.osProfile The definition of a data factory
     * gateway to be created or updated.
     *
     * @param {object} [parameters.osProfile.windowsOperatingSystemProfile] The
     * Windows OS profile.
     *
     * @param {object}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings] The RDP
     * settings.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.username]
     * The username for the RDP user.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.password]
     * The password for the RDP user.
     *
     * @param {date}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.expiryDate]
     * The RDP expiry date(YYYY-MM-DD).
     *
     * @param {object} [parameters.osProfile.linuxOperatingSystemProfile] The Linux
     * OS profile.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.username]
     * The username.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.password]
     * The password.
     *
     * @param {object}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile] The SSH
     * profile.
     *
     * @param {array}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile.publicKeys] The
     * list of SSH public keys.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    changeRdpSettings(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    changeRdpSettings(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, callback: ServiceCallback<void>): void;
    changeRdpSettings(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Begins executing script actions on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for executing script actions.
     *
     * @param {array} [parameters.scriptActions] The list of run time script
     * actions.
     *
     * @param {string} parameters.persistOnSuccess Gets or sets if the scripts
     * needs to be persisted.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    executeScriptActionsWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins executing script actions on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for executing script actions.
     *
     * @param {array} [parameters.scriptActions] The list of run time script
     * actions.
     *
     * @param {string} parameters.persistOnSuccess Gets or sets if the scripts
     * needs to be persisted.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    executeScriptActions(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    executeScriptActions(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, callback: ServiceCallback<void>): void;
    executeScriptActions(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Begins creating a new HDInsight cluster with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster create request.
     *
     * @param {string} [parameters.location] The location of the cluster.
     *
     * @param {object} [parameters.tags] The resource tags.
     *
     * @param {object} [parameters.properties] The cluster create parameters.
     *
     * @param {string} [parameters.properties.clusterVersion] The version of the
     * cluster.
     *
     * @param {string} [parameters.properties.osType] The type of operating system.
     * Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [parameters.properties.tier] The cluster tier. Possible
     * values include: 'Standard', 'Premium'
     *
     * @param {object} [parameters.properties.clusterDefinition] The cluster
     * definition.
     *
     * @param {string} [parameters.properties.clusterDefinition.blueprint] The link
     * to the blueprint.
     *
     * @param {string} [parameters.properties.clusterDefinition.kind] The type of
     * cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.componentVersion]
     * The versions of different services in the cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.configurations] The
     * cluster configurations.
     *
     * @param {object} [parameters.properties.securityProfile] The security
     * profile.
     *
     * @param {string} [parameters.properties.securityProfile.directoryType] The
     * directory type. Possible values include: 'ActiveDirectory'
     *
     * @param {string} [parameters.properties.securityProfile.domain] domain.
     *
     * @param {string} [parameters.properties.securityProfile.organizationalUnitDN]
     * Distinguished Name for organizational unit.
     *
     * @param {array} [parameters.properties.securityProfile.ldapsUrls] The LDAP
     * URLs.
     *
     * @param {string} [parameters.properties.securityProfile.domainUsername] The
     * domain admin user name.
     *
     * @param {string} [parameters.properties.securityProfile.domainUserPassword]
     * The domain admin password.
     *
     * @param {array} [parameters.properties.securityProfile.clusterUsersGroupDNs]
     * Optional. The Distinguished Names for cluster user groups
     *
     * @param {object} [parameters.properties.computeProfile] The compute profile.
     *
     * @param {array} [parameters.properties.computeProfile.roles] The list of
     * roles in the cluster.
     *
     * @param {object} [parameters.properties.storageProfile] The storage profile.
     *
     * @param {array} [parameters.properties.storageProfile.storageaccounts] The
     * list of storage accounts in the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Cluster>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginCreateWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Cluster>>;

    /**
     * Begins creating a new HDInsight cluster with the specified parameters.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster create request.
     *
     * @param {string} [parameters.location] The location of the cluster.
     *
     * @param {object} [parameters.tags] The resource tags.
     *
     * @param {object} [parameters.properties] The cluster create parameters.
     *
     * @param {string} [parameters.properties.clusterVersion] The version of the
     * cluster.
     *
     * @param {string} [parameters.properties.osType] The type of operating system.
     * Possible values include: 'Windows', 'Linux'
     *
     * @param {string} [parameters.properties.tier] The cluster tier. Possible
     * values include: 'Standard', 'Premium'
     *
     * @param {object} [parameters.properties.clusterDefinition] The cluster
     * definition.
     *
     * @param {string} [parameters.properties.clusterDefinition.blueprint] The link
     * to the blueprint.
     *
     * @param {string} [parameters.properties.clusterDefinition.kind] The type of
     * cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.componentVersion]
     * The versions of different services in the cluster.
     *
     * @param {object} [parameters.properties.clusterDefinition.configurations] The
     * cluster configurations.
     *
     * @param {object} [parameters.properties.securityProfile] The security
     * profile.
     *
     * @param {string} [parameters.properties.securityProfile.directoryType] The
     * directory type. Possible values include: 'ActiveDirectory'
     *
     * @param {string} [parameters.properties.securityProfile.domain] domain.
     *
     * @param {string} [parameters.properties.securityProfile.organizationalUnitDN]
     * Distinguished Name for organizational unit.
     *
     * @param {array} [parameters.properties.securityProfile.ldapsUrls] The LDAP
     * URLs.
     *
     * @param {string} [parameters.properties.securityProfile.domainUsername] The
     * domain admin user name.
     *
     * @param {string} [parameters.properties.securityProfile.domainUserPassword]
     * The domain admin password.
     *
     * @param {array} [parameters.properties.securityProfile.clusterUsersGroupDNs]
     * Optional. The Distinguished Names for cluster user groups
     *
     * @param {object} [parameters.properties.computeProfile] The compute profile.
     *
     * @param {array} [parameters.properties.computeProfile.roles] The list of
     * roles in the cluster.
     *
     * @param {object} [parameters.properties.storageProfile] The storage profile.
     *
     * @param {array} [parameters.properties.storageProfile.storageaccounts] The
     * list of storage accounts in the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Cluster} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Cluster} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Cluster} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginCreate(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Cluster>;
    beginCreate(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, callback: ServiceCallback<models.Cluster>): void;
    beginCreate(resourceGroupName: string, clusterName: string, parameters: models.ClusterCreateParametersExtended, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Cluster>): void;


    /**
     * Begins deleting the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins deleting the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, clusterName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Begins a resize operation on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for the resize operation.
     *
     * @param {number} [parameters.targetInstanceCount] The target instance count
     * for the operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginResizeWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins a resize operation on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for the resize operation.
     *
     * @param {number} [parameters.targetInstanceCount] The target instance count
     * for the operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginResize(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginResize(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, callback: ServiceCallback<void>): void;
    beginResize(resourceGroupName: string, clusterName: string, parameters: models.ClusterResizeParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Begins changing the RDP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The OS profile for RDP.
     *
     * @param {object} parameters.osProfile The definition of a data factory
     * gateway to be created or updated.
     *
     * @param {object} [parameters.osProfile.windowsOperatingSystemProfile] The
     * Windows OS profile.
     *
     * @param {object}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings] The RDP
     * settings.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.username]
     * The username for the RDP user.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.password]
     * The password for the RDP user.
     *
     * @param {date}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.expiryDate]
     * The RDP expiry date(YYYY-MM-DD).
     *
     * @param {object} [parameters.osProfile.linuxOperatingSystemProfile] The Linux
     * OS profile.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.username]
     * The username.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.password]
     * The password.
     *
     * @param {object}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile] The SSH
     * profile.
     *
     * @param {array}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile.publicKeys] The
     * list of SSH public keys.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginChangeRdpSettingsWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins changing the RDP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The OS profile for RDP.
     *
     * @param {object} parameters.osProfile The definition of a data factory
     * gateway to be created or updated.
     *
     * @param {object} [parameters.osProfile.windowsOperatingSystemProfile] The
     * Windows OS profile.
     *
     * @param {object}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings] The RDP
     * settings.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.username]
     * The username for the RDP user.
     *
     * @param {string}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.password]
     * The password for the RDP user.
     *
     * @param {date}
     * [parameters.osProfile.windowsOperatingSystemProfile.rdpSettings.expiryDate]
     * The RDP expiry date(YYYY-MM-DD).
     *
     * @param {object} [parameters.osProfile.linuxOperatingSystemProfile] The Linux
     * OS profile.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.username]
     * The username.
     *
     * @param {string} [parameters.osProfile.linuxOperatingSystemProfile.password]
     * The password.
     *
     * @param {object}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile] The SSH
     * profile.
     *
     * @param {array}
     * [parameters.osProfile.linuxOperatingSystemProfile.sshProfile.publicKeys] The
     * list of SSH public keys.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginChangeRdpSettings(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginChangeRdpSettings(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, callback: ServiceCallback<void>): void;
    beginChangeRdpSettings(resourceGroupName: string, clusterName: string, parameters: models.RDPSettingsParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Begins executing script actions on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for executing script actions.
     *
     * @param {array} [parameters.scriptActions] The list of run time script
     * actions.
     *
     * @param {string} parameters.persistOnSuccess Gets or sets if the scripts
     * needs to be persisted.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginExecuteScriptActionsWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins executing script actions on the specified HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The parameters for executing script actions.
     *
     * @param {array} [parameters.scriptActions] The list of run time script
     * actions.
     *
     * @param {string} parameters.persistOnSuccess Gets or sets if the scripts
     * needs to be persisted.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginExecuteScriptActions(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginExecuteScriptActions(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, callback: ServiceCallback<void>): void;
    beginExecuteScriptActions(resourceGroupName: string, clusterName: string, parameters: models.ExecuteScriptActionParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * List the HDInsight clusters in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ClusterListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ClusterListResult>>;

    /**
     * List the HDInsight clusters in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ClusterListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ClusterListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ClusterListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ClusterListResult>;
    listByResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.ClusterListResult>): void;
    listByResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ClusterListResult>): void;


    /**
     * Lists HDInsight clusters under the subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ClusterListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ClusterListResult>>;

    /**
     * Lists HDInsight clusters under the subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ClusterListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ClusterListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ClusterListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ClusterListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ClusterListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ClusterListResult>): void;
}

/**
 * @class
 * Applications
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface Applications {


    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicationListResult>>;

    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicationListResult>;
    list(resourceGroupName: string, clusterName: string, callback: ServiceCallback<models.ApplicationListResult>): void;
    list(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicationListResult>): void;


    /**
     * Lists properties of the application.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} applicationName The constant value for the applicationName.
     * Possible values include:
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Application>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, clusterName: string, applicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Application>>;

    /**
     * Lists properties of the application.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} applicationName The constant value for the applicationName.
     * Possible values include:
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Application} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Application} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Application} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, clusterName: string, applicationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Application>;
    get(resourceGroupName: string, clusterName: string, applicationName: string, callback: ServiceCallback<models.Application>): void;
    get(resourceGroupName: string, clusterName: string, applicationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Application>): void;


    /**
     * The operation creates applications for the HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The application create request.
     *
     * @param {object} [parameters.computeProfile] The list of roles in the
     * cluster.
     *
     * @param {array} [parameters.computeProfile.roles] The list of roles in the
     * cluster.
     *
     * @param {array} [parameters.installScriptActions] The list of install script
     * actions.
     *
     * @param {array} [parameters.uninstallScriptActions] The list of uninstall
     * script actions.
     *
     * @param {array} [parameters.httpsEndpoints] The list of application Https
     * endpoints.
     *
     * @param {array} [parameters.sshEndpoints] The list of application ssh
     * endpoints.
     *
     * @param {string} [parameters.applicationType] The Application type.
     *
     * @param {array} [parameters.errors] The list of errors.
     *
     * @param {string} [parameters.additionalProperties] The additional properties
     * for application.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Application>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.ApplicationGetProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Application>>;

    /**
     * The operation creates applications for the HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The application create request.
     *
     * @param {object} [parameters.computeProfile] The list of roles in the
     * cluster.
     *
     * @param {array} [parameters.computeProfile.roles] The list of roles in the
     * cluster.
     *
     * @param {array} [parameters.installScriptActions] The list of install script
     * actions.
     *
     * @param {array} [parameters.uninstallScriptActions] The list of uninstall
     * script actions.
     *
     * @param {array} [parameters.httpsEndpoints] The list of application Https
     * endpoints.
     *
     * @param {array} [parameters.sshEndpoints] The list of application ssh
     * endpoints.
     *
     * @param {string} [parameters.applicationType] The Application type.
     *
     * @param {array} [parameters.errors] The list of errors.
     *
     * @param {string} [parameters.additionalProperties] The additional properties
     * for application.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Application} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Application} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Application} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, clusterName: string, parameters: models.ApplicationGetProperties, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Application>;
    create(resourceGroupName: string, clusterName: string, parameters: models.ApplicationGetProperties, callback: ServiceCallback<models.Application>): void;
    create(resourceGroupName: string, clusterName: string, parameters: models.ApplicationGetProperties, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Application>): void;


    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, clusterName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, clusterName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicationListResult>>;

    /**
     * Lists all of the applications HDInsight cluster.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ApplicationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicationListResult>): void;
}

/**
 * @class
 * Location
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface Location {


    /**
     * Gets the capabilities for the specified location.
     *
     * @param {string} location The location to get capabilities for.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CapabilitiesResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getCapabilitiesWithHttpOperationResponse(location: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CapabilitiesResult>>;

    /**
     * Gets the capabilities for the specified location.
     *
     * @param {string} location The location to get capabilities for.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CapabilitiesResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CapabilitiesResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CapabilitiesResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getCapabilities(location: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CapabilitiesResult>;
    getCapabilities(location: string, callback: ServiceCallback<models.CapabilitiesResult>): void;
    getCapabilities(location: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CapabilitiesResult>): void;
}

/**
 * @class
 * Configurations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface Configurations {


    /**
     * Begins configuring the HTTP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The name of the resource group.
     *
     * @param {string} [parameters.enabledCredential] Describes whether HTTP based
     * authorization is enabled or not.
     *
     * @param {string} [parameters.username] The http username.
     *
     * @param {string} [parameters.password] The http user password.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateHTTPSettingsWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins configuring the HTTP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The name of the resource group.
     *
     * @param {string} [parameters.enabledCredential] Describes whether HTTP based
     * authorization is enabled or not.
     *
     * @param {string} [parameters.username] The http username.
     *
     * @param {string} [parameters.password] The http user password.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    updateHTTPSettings(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    updateHTTPSettings(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, callback: ServiceCallback<void>): void;
    updateHTTPSettings(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * The configuration object for the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} configurationName The constant for configuration type of
     * gateway. Possible values include: 'gateway', 'core-site'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<HttpConnectivitySettings>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, clusterName: string, configurationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.HttpConnectivitySettings>>;

    /**
     * The configuration object for the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} configurationName The constant for configuration type of
     * gateway. Possible values include: 'gateway', 'core-site'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {HttpConnectivitySettings} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {HttpConnectivitySettings} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link HttpConnectivitySettings} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, clusterName: string, configurationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.HttpConnectivitySettings>;
    get(resourceGroupName: string, clusterName: string, configurationName: string, callback: ServiceCallback<models.HttpConnectivitySettings>): void;
    get(resourceGroupName: string, clusterName: string, configurationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.HttpConnectivitySettings>): void;


    /**
     * Begins configuring the HTTP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The name of the resource group.
     *
     * @param {string} [parameters.enabledCredential] Describes whether HTTP based
     * authorization is enabled or not.
     *
     * @param {string} [parameters.username] The http username.
     *
     * @param {string} [parameters.password] The http user password.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginUpdateHTTPSettingsWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Begins configuring the HTTP settings on the specified cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The name of the resource group.
     *
     * @param {string} [parameters.enabledCredential] Describes whether HTTP based
     * authorization is enabled or not.
     *
     * @param {string} [parameters.username] The http username.
     *
     * @param {string} [parameters.password] The http user password.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginUpdateHTTPSettings(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginUpdateHTTPSettings(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, callback: ServiceCallback<void>): void;
    beginUpdateHTTPSettings(resourceGroupName: string, clusterName: string, parameters: models.HttpConnectivitySettings, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * ExtensionOperations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface ExtensionOperations {


    /**
     * Create HDInsight cluster extension.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster extensions create request.
     *
     * @param {string} [parameters.workspaceId] Workspace Id for cluster monitoring
     * extension
     *
     * @param {string} [parameters.primaryKey] Certificate for monitoring
     * extensions
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, clusterName: string, parameters: models.Extension, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Create HDInsight cluster extension.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} parameters The cluster extensions create request.
     *
     * @param {string} [parameters.workspaceId] Workspace Id for cluster monitoring
     * extension
     *
     * @param {string} [parameters.primaryKey] Certificate for monitoring
     * extensions
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, clusterName: string, parameters: models.Extension, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    create(resourceGroupName: string, clusterName: string, parameters: models.Extension, callback: ServiceCallback<void>): void;
    create(resourceGroupName: string, clusterName: string, parameters: models.Extension, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Get extension properties for HDInsight cluster extension.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Extension>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Extension>>;

    /**
     * Get extension properties for HDInsight cluster extension.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Extension} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Extension} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Extension} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Extension>;
    get(resourceGroupName: string, clusterName: string, callback: ServiceCallback<models.Extension>): void;
    get(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Extension>): void;


    /**
     * Delete extension for HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete extension for HDInsight cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, clusterName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * ScriptActions
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface ScriptActions {


    /**
     * Deletes a given persisted script action of the cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} scriptName The name of the script.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, clusterName: string, scriptName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Deletes a given persisted script action of the cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} scriptName The name of the script.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, clusterName: string, scriptName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, clusterName: string, scriptName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, clusterName: string, scriptName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists all persisted script actions for the given cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScriptActionsList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScriptActionsList>>;

    /**
     * Lists all persisted script actions for the given cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScriptActionsList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScriptActionsList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScriptActionsList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScriptActionsList>;
    list(resourceGroupName: string, clusterName: string, callback: ServiceCallback<models.ScriptActionsList>): void;
    list(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScriptActionsList>): void;


    /**
     * Lists all persisted script actions for the given cluster.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScriptActionsList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScriptActionsList>>;

    /**
     * Lists all persisted script actions for the given cluster.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScriptActionsList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScriptActionsList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScriptActionsList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScriptActionsList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ScriptActionsList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScriptActionsList>): void;
}

/**
 * @class
 * ScriptExecutionHistory
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface ScriptExecutionHistory {


    /**
     * Gets the script execution detail for the given script execution id.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} scriptExecutionId The script execution Id
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<RuntimeScriptActionDetail>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, clusterName: string, scriptExecutionId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.RuntimeScriptActionDetail>>;

    /**
     * Gets the script execution detail for the given script execution id.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {string} scriptExecutionId The script execution Id
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {RuntimeScriptActionDetail} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {RuntimeScriptActionDetail} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link RuntimeScriptActionDetail} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, clusterName: string, scriptExecutionId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.RuntimeScriptActionDetail>;
    get(resourceGroupName: string, clusterName: string, scriptExecutionId: string, callback: ServiceCallback<models.RuntimeScriptActionDetail>): void;
    get(resourceGroupName: string, clusterName: string, scriptExecutionId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.RuntimeScriptActionDetail>): void;


    /**
     * Lists all scripts execution history for the given cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScriptActionExecutionHistoryList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScriptActionExecutionHistoryList>>;

    /**
     * Lists all scripts execution history for the given cluster.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScriptActionExecutionHistoryList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScriptActionExecutionHistoryList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScriptActionExecutionHistoryList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, clusterName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScriptActionExecutionHistoryList>;
    list(resourceGroupName: string, clusterName: string, callback: ServiceCallback<models.ScriptActionExecutionHistoryList>): void;
    list(resourceGroupName: string, clusterName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScriptActionExecutionHistoryList>): void;


    /**
     * Promote ad-hoc script execution to a persisted script.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {number} scriptExecutionId The script execution Id
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    promoteWithHttpOperationResponse(resourceGroupName: string, clusterName: string, scriptExecutionId: number, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Promote ad-hoc script execution to a persisted script.
     *
     * @param {string} resourceGroupName The name of the resource group.
     *
     * @param {string} clusterName The name of the cluster.
     *
     * @param {number} scriptExecutionId The script execution Id
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    promote(resourceGroupName: string, clusterName: string, scriptExecutionId: number, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    promote(resourceGroupName: string, clusterName: string, scriptExecutionId: number, callback: ServiceCallback<void>): void;
    promote(resourceGroupName: string, clusterName: string, scriptExecutionId: number, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Lists all scripts execution history for the given cluster.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ScriptActionExecutionHistoryList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ScriptActionExecutionHistoryList>>;

    /**
     * Lists all scripts execution history for the given cluster.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ScriptActionExecutionHistoryList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ScriptActionExecutionHistoryList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ScriptActionExecutionHistoryList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ScriptActionExecutionHistoryList>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.ScriptActionExecutionHistoryList>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ScriptActionExecutionHistoryList>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the HDInsightManagementClient.
 */
export interface Operations {


    /**
     * Lists all of the available HDInsight REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available HDInsight REST API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;


    /**
     * Lists all of the available HDInsight REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available HDInsight REST API operations.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.OperationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}
