/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * @class
 * Initializes a new instance of the ServiceDiagnosticSettingsResource class.
 * @constructor
 * Description of a service diagnostic setting
 *
 * @member {string} [storageAccountId] The resource ID of the storage account
 * to which you would like to send Diagnostic Logs.
 *
 * @member {string} [serviceBusRuleId] The service bus rule ID of the service
 * bus namespace in which you would like to have Event Hubs created for
 * streaming Diagnostic Logs. The rule ID is of the format: '{service bus
 * resource ID}/authorizationrules/{key name}'.
 *
 * @member {array} [metrics] the list of metric settings.
 *
 * @member {array} [logs] the list of logs settings.
 *
 * @member {string} [workspaceId] The workspace ID (resource ID of a Log
 * Analytics workspace) for a Log Analytics workspace to which you would like
 * to send Diagnostic Logs. Example:
 * /subscriptions/4b9e8510-67ab-4e9a-95a9-e2f1e570ea9c/resourceGroups/insights-integration/providers/Microsoft.OperationalInsights/workspaces/viruela2
 *
 */
class ServiceDiagnosticSettingsResource extends models['Resource'] {
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ServiceDiagnosticSettingsResource
   *
   * @returns {object} metadata of ServiceDiagnosticSettingsResource
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ServiceDiagnosticSettingsResource',
      type: {
        name: 'Composite',
        className: 'ServiceDiagnosticSettingsResource',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          storageAccountId: {
            required: false,
            serializedName: 'properties.storageAccountId',
            type: {
              name: 'String'
            }
          },
          serviceBusRuleId: {
            required: false,
            serializedName: 'properties.serviceBusRuleId',
            type: {
              name: 'String'
            }
          },
          metrics: {
            required: false,
            serializedName: 'properties.metrics',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'MetricSettingsElementType',
                  type: {
                    name: 'Composite',
                    className: 'MetricSettings'
                  }
              }
            }
          },
          logs: {
            required: false,
            serializedName: 'properties.logs',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'LogSettingsElementType',
                  type: {
                    name: 'Composite',
                    className: 'LogSettings'
                  }
              }
            }
          },
          workspaceId: {
            required: false,
            serializedName: 'properties.workspaceId',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ServiceDiagnosticSettingsResource;
