/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as moment from 'moment';
import * as models from '../models';


/**
 * @class
 * Ios
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the IntuneResourceManagementClient.
 */
export interface Ios {


    /**
     * Returns Intune iOSPolicies.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IOSMAMPolicyCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getMAMPoliciesWithHttpOperationResponse(hostName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IOSMAMPolicyCollection>>;

    /**
     * Returns Intune iOSPolicies.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IOSMAMPolicyCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IOSMAMPolicyCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IOSMAMPolicyCollection} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getMAMPolicies(hostName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.IOSMAMPolicyCollection>;
    getMAMPolicies(hostName: string, callback: ServiceCallback<models.IOSMAMPolicyCollection>): void;
    getMAMPolicies(hostName: string, options: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IOSMAMPolicyCollection>): void;


    /**
     * Returns Intune iOS policies.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IOSMAMPolicy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getMAMPolicyByNameWithHttpOperationResponse(hostName: string, policyName: string, options?: { select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IOSMAMPolicy>>;

    /**
     * Returns Intune iOS policies.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IOSMAMPolicy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IOSMAMPolicy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IOSMAMPolicy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getMAMPolicyByName(hostName: string, policyName: string, options?: { select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.IOSMAMPolicy>;
    getMAMPolicyByName(hostName: string, policyName: string, callback: ServiceCallback<models.IOSMAMPolicy>): void;
    getMAMPolicyByName(hostName: string, policyName: string, options: { select? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IOSMAMPolicy>): void;


    /**
     * Creates or updates iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.fileEncryptionLevel] Possible values include:
     * 'deviceLocked', 'deviceLockedExceptFilesOpen', 'afterDeviceRestart',
     * 'useDeviceSettings'
     *
     * @param {string} [parameters.touchId] Possible values include: 'enable',
     * 'disable'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IOSMAMPolicy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IOSMAMPolicy>>;

    /**
     * Creates or updates iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.fileEncryptionLevel] Possible values include:
     * 'deviceLocked', 'deviceLockedExceptFilesOpen', 'afterDeviceRestart',
     * 'useDeviceSettings'
     *
     * @param {string} [parameters.touchId] Possible values include: 'enable',
     * 'disable'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IOSMAMPolicy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IOSMAMPolicy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IOSMAMPolicy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdateMAMPolicy(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IOSMAMPolicy>;
    createOrUpdateMAMPolicy(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, callback: ServiceCallback<models.IOSMAMPolicy>): void;
    createOrUpdateMAMPolicy(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IOSMAMPolicy>): void;


    /**
     * patch an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.fileEncryptionLevel] Possible values include:
     * 'deviceLocked', 'deviceLockedExceptFilesOpen', 'afterDeviceRestart',
     * 'useDeviceSettings'
     *
     * @param {string} [parameters.touchId] Possible values include: 'enable',
     * 'disable'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IOSMAMPolicy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    patchMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IOSMAMPolicy>>;

    /**
     * patch an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.fileEncryptionLevel] Possible values include:
     * 'deviceLocked', 'deviceLockedExceptFilesOpen', 'afterDeviceRestart',
     * 'useDeviceSettings'
     *
     * @param {string} [parameters.touchId] Possible values include: 'enable',
     * 'disable'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IOSMAMPolicy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IOSMAMPolicy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IOSMAMPolicy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    patchMAMPolicy(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IOSMAMPolicy>;
    patchMAMPolicy(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, callback: ServiceCallback<models.IOSMAMPolicy>): void;
    patchMAMPolicy(hostName: string, policyName: string, parameters: models.IOSMAMPolicy, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IOSMAMPolicy>): void;


    /**
     * Delete Ios Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete Ios Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMAMPolicy(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMAMPolicy(hostName: string, policyName: string, callback: ServiceCallback<void>): void;
    deleteMAMPolicy(hostName: string, policyName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Get apps for an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicationCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getAppForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicationCollection>>;

    /**
     * Get apps for an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicationCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicationCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicationCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getAppForMAMPolicy(hostName: string, policyName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicationCollection>;
    getAppForMAMPolicy(hostName: string, policyName: string, callback: ServiceCallback<models.ApplicationCollection>): void;
    getAppForMAMPolicy(hostName: string, policyName: string, options: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicationCollection>): void;


    /**
     * Add app to an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} parameters Parameters supplied to add an app to an ios
     * policy.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    addAppForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Add app to an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} parameters Parameters supplied to add an app to an ios
     * policy.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    addAppForMAMPolicy(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    addAppForMAMPolicy(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, callback: ServiceCallback<void>): void;
    addAppForMAMPolicy(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Delete App for Ios Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteAppForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, appName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete App for Ios Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteAppForMAMPolicy(hostName: string, policyName: string, appName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteAppForMAMPolicy(hostName: string, policyName: string, appName: string, callback: ServiceCallback<void>): void;
    deleteAppForMAMPolicy(hostName: string, policyName: string, appName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Returns groups for a given iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName policy name for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GroupsCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getGroupsForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GroupsCollection>>;

    /**
     * Returns groups for a given iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName policy name for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GroupsCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GroupsCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GroupsCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getGroupsForMAMPolicy(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GroupsCollection>;
    getGroupsForMAMPolicy(hostName: string, policyName: string, callback: ServiceCallback<models.GroupsCollection>): void;
    getGroupsForMAMPolicy(hostName: string, policyName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GroupsCollection>): void;


    /**
     * Add group to an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId group Id
     *
     * @param {object} parameters Parameters supplied to the Create or update app
     * to an android policy operation.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    addGroupForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Add group to an iOSMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId group Id
     *
     * @param {object} parameters Parameters supplied to the Create or update app
     * to an android policy operation.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    addGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    addGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, callback: ServiceCallback<void>): void;
    addGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Delete Group for iOS Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteGroupForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, groupId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete Group for iOS Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, callback: ServiceCallback<void>): void;
    deleteGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Returns Intune iOSPolicies.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<IOSMAMPolicyCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getMAMPoliciesNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.IOSMAMPolicyCollection>>;

    /**
     * Returns Intune iOSPolicies.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {IOSMAMPolicyCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {IOSMAMPolicyCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link IOSMAMPolicyCollection} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getMAMPoliciesNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.IOSMAMPolicyCollection>;
    getMAMPoliciesNext(nextPageLink: string, callback: ServiceCallback<models.IOSMAMPolicyCollection>): void;
    getMAMPoliciesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.IOSMAMPolicyCollection>): void;


    /**
     * Get apps for an iOSMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicationCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getAppForMAMPolicyNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicationCollection>>;

    /**
     * Get apps for an iOSMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicationCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicationCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicationCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getAppForMAMPolicyNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicationCollection>;
    getAppForMAMPolicyNext(nextPageLink: string, callback: ServiceCallback<models.ApplicationCollection>): void;
    getAppForMAMPolicyNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicationCollection>): void;


    /**
     * Returns groups for a given iOSMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GroupsCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getGroupsForMAMPolicyNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GroupsCollection>>;

    /**
     * Returns groups for a given iOSMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GroupsCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GroupsCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GroupsCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getGroupsForMAMPolicyNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GroupsCollection>;
    getGroupsForMAMPolicyNext(nextPageLink: string, callback: ServiceCallback<models.GroupsCollection>): void;
    getGroupsForMAMPolicyNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GroupsCollection>): void;
}

/**
 * @class
 * Android
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the IntuneResourceManagementClient.
 */
export interface Android {


    /**
     * Returns Intune Android policies.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AndroidMAMPolicyCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getMAMPoliciesWithHttpOperationResponse(hostName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AndroidMAMPolicyCollection>>;

    /**
     * Returns Intune Android policies.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AndroidMAMPolicyCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AndroidMAMPolicyCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AndroidMAMPolicyCollection} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getMAMPolicies(hostName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.AndroidMAMPolicyCollection>;
    getMAMPolicies(hostName: string, callback: ServiceCallback<models.AndroidMAMPolicyCollection>): void;
    getMAMPolicies(hostName: string, options: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AndroidMAMPolicyCollection>): void;


    /**
     * Returns AndroidMAMPolicy with given name.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AndroidMAMPolicy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getMAMPolicyByNameWithHttpOperationResponse(hostName: string, policyName: string, options?: { select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AndroidMAMPolicy>>;

    /**
     * Returns AndroidMAMPolicy with given name.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AndroidMAMPolicy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AndroidMAMPolicy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AndroidMAMPolicy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getMAMPolicyByName(hostName: string, policyName: string, options?: { select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.AndroidMAMPolicy>;
    getMAMPolicyByName(hostName: string, policyName: string, callback: ServiceCallback<models.AndroidMAMPolicy>): void;
    getMAMPolicyByName(hostName: string, policyName: string, options: { select? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AndroidMAMPolicy>): void;


    /**
     * Creates or updates AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.screenCapture] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileEncryption] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AndroidMAMPolicy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AndroidMAMPolicy>>;

    /**
     * Creates or updates AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.screenCapture] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileEncryption] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AndroidMAMPolicy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AndroidMAMPolicy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AndroidMAMPolicy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdateMAMPolicy(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AndroidMAMPolicy>;
    createOrUpdateMAMPolicy(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, callback: ServiceCallback<models.AndroidMAMPolicy>): void;
    createOrUpdateMAMPolicy(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AndroidMAMPolicy>): void;


    /**
     * Patch AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.screenCapture] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileEncryption] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AndroidMAMPolicy>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    patchMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AndroidMAMPolicy>>;

    /**
     * Patch AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} parameters Parameters supplied to the Create or update an
     * android policy operation.
     *
     * @param {string} parameters.friendlyName
     *
     * @param {string} [parameters.description]
     *
     * @param {string} [parameters.appSharingFromLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.appSharingToLevel] Possible values include:
     * 'none', 'policyManagedApps', 'allApps'
     *
     * @param {string} [parameters.authentication] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {string} [parameters.clipboardSharingLevel] Possible values include:
     * 'blocked', 'policyManagedApps', 'policyManagedAppsWithPasteIn', 'allApps'
     *
     * @param {string} [parameters.dataBackup] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileSharingSaveAs] Possible values include:
     * 'allow', 'block'
     *
     * @param {string} [parameters.pin] Possible values include: 'required',
     * 'notRequired'
     *
     * @param {number} [parameters.pinNumRetry]
     *
     * @param {string} [parameters.deviceCompliance] Possible values include:
     * 'enable', 'disable'
     *
     * @param {string} [parameters.managedBrowser] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {moment.duration} [parameters.accessRecheckOfflineTimeout]
     *
     * @param {moment.duration} [parameters.accessRecheckOnlineTimeout]
     *
     * @param {moment.duration} [parameters.offlineWipeTimeout]
     *
     * @param {string} [parameters.screenCapture] Possible values include: 'allow',
     * 'block'
     *
     * @param {string} [parameters.fileEncryption] Possible values include:
     * 'required', 'notRequired'
     *
     * @param {object} [parameters.tags] Resource Tags
     *
     * @param {string} [parameters.location] Resource Location
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AndroidMAMPolicy} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AndroidMAMPolicy} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AndroidMAMPolicy} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    patchMAMPolicy(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AndroidMAMPolicy>;
    patchMAMPolicy(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, callback: ServiceCallback<models.AndroidMAMPolicy>): void;
    patchMAMPolicy(hostName: string, policyName: string, parameters: models.AndroidMAMPolicy, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AndroidMAMPolicy>): void;


    /**
     * Delete Android Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete Android Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMAMPolicy(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMAMPolicy(hostName: string, policyName: string, callback: ServiceCallback<void>): void;
    deleteMAMPolicy(hostName: string, policyName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Get apps for an AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicationCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getAppForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicationCollection>>;

    /**
     * Get apps for an AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.filter] The filter to apply on the operation.
     *
     * @param {number} [options.top]
     *
     * @param {string} [options.select] select specific fields in entity.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicationCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicationCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicationCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getAppForMAMPolicy(hostName: string, policyName: string, options?: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicationCollection>;
    getAppForMAMPolicy(hostName: string, policyName: string, callback: ServiceCallback<models.ApplicationCollection>): void;
    getAppForMAMPolicy(hostName: string, policyName: string, options: { filter? : string, top? : number, select? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicationCollection>): void;


    /**
     * Add app to an AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} parameters Parameters supplied to the Create or update app
     * to an android policy operation.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    addAppForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Add app to an AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} parameters Parameters supplied to the Create or update app
     * to an android policy operation.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    addAppForMAMPolicy(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    addAppForMAMPolicy(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, callback: ServiceCallback<void>): void;
    addAppForMAMPolicy(hostName: string, policyName: string, appName: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Delete App for Android Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteAppForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, appName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete App for Android Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} appName application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteAppForMAMPolicy(hostName: string, policyName: string, appName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteAppForMAMPolicy(hostName: string, policyName: string, appName: string, callback: ServiceCallback<void>): void;
    deleteAppForMAMPolicy(hostName: string, policyName: string, appName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Returns groups for a given AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName policy name for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GroupsCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getGroupsForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GroupsCollection>>;

    /**
     * Returns groups for a given AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName policy name for the tenant
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GroupsCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GroupsCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GroupsCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getGroupsForMAMPolicy(hostName: string, policyName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GroupsCollection>;
    getGroupsForMAMPolicy(hostName: string, policyName: string, callback: ServiceCallback<models.GroupsCollection>): void;
    getGroupsForMAMPolicy(hostName: string, policyName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GroupsCollection>): void;


    /**
     * Add group to an AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId group Id
     *
     * @param {object} parameters Parameters supplied to the Create or update app
     * to an android policy operation.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    addGroupForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Add group to an AndroidMAMPolicy.
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId group Id
     *
     * @param {object} parameters Parameters supplied to the Create or update app
     * to an android policy operation.
     *
     * @param {object} [parameters.properties]
     *
     * @param {string} parameters.properties.url
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    addGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    addGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, callback: ServiceCallback<void>): void;
    addGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, parameters: models.MAMPolicyAppIdOrGroupIdPayload, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Delete Group for Android Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteGroupForMAMPolicyWithHttpOperationResponse(hostName: string, policyName: string, groupId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete Group for Android Policy
     *
     * @param {string} hostName Location hostName for the tenant
     *
     * @param {string} policyName Unique name for the policy
     *
     * @param {string} groupId application unique Name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, callback: ServiceCallback<void>): void;
    deleteGroupForMAMPolicy(hostName: string, policyName: string, groupId: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Returns Intune Android policies.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<AndroidMAMPolicyCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getMAMPoliciesNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.AndroidMAMPolicyCollection>>;

    /**
     * Returns Intune Android policies.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {AndroidMAMPolicyCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {AndroidMAMPolicyCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link AndroidMAMPolicyCollection} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getMAMPoliciesNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.AndroidMAMPolicyCollection>;
    getMAMPoliciesNext(nextPageLink: string, callback: ServiceCallback<models.AndroidMAMPolicyCollection>): void;
    getMAMPoliciesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.AndroidMAMPolicyCollection>): void;


    /**
     * Get apps for an AndroidMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<ApplicationCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getAppForMAMPolicyNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ApplicationCollection>>;

    /**
     * Get apps for an AndroidMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {ApplicationCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {ApplicationCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link ApplicationCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getAppForMAMPolicyNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ApplicationCollection>;
    getAppForMAMPolicyNext(nextPageLink: string, callback: ServiceCallback<models.ApplicationCollection>): void;
    getAppForMAMPolicyNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ApplicationCollection>): void;


    /**
     * Returns groups for a given AndroidMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<GroupsCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getGroupsForMAMPolicyNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.GroupsCollection>>;

    /**
     * Returns groups for a given AndroidMAMPolicy.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {GroupsCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {GroupsCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link GroupsCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getGroupsForMAMPolicyNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.GroupsCollection>;
    getGroupsForMAMPolicyNext(nextPageLink: string, callback: ServiceCallback<models.GroupsCollection>): void;
    getGroupsForMAMPolicyNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.GroupsCollection>): void;
}
