/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The IoT hub cloud-to-device messaging properties.
 *
 */
class CloudToDeviceProperties {
  /**
   * Create a CloudToDeviceProperties.
   * @member {number} [maxDeliveryCount] The max delivery count for
   * cloud-to-device messages in the device queue. See:
   * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
   * @member {moment.duration} [defaultTtlAsIso8601] The default time to live
   * for cloud-to-device messages in the device queue. See:
   * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
   * @member {object} [feedback]
   * @member {moment.duration} [feedback.lockDurationAsIso8601] The lock
   * duration for the feedback queue. See:
   * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
   * @member {moment.duration} [feedback.ttlAsIso8601] The period of time for
   * which a message is available to consume before it is expired by the IoT
   * hub. See:
   * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
   * @member {number} [feedback.maxDeliveryCount] The number of times the IoT
   * hub attempts to deliver a message on the feedback queue. See:
   * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging#cloud-to-device-messages.
   */
  constructor() {
  }

  /**
   * Defines the metadata of CloudToDeviceProperties
   *
   * @returns {object} metadata of CloudToDeviceProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CloudToDeviceProperties',
      type: {
        name: 'Composite',
        className: 'CloudToDeviceProperties',
        modelProperties: {
          maxDeliveryCount: {
            required: false,
            serializedName: 'maxDeliveryCount',
            constraints: {
              InclusiveMaximum: 100,
              InclusiveMinimum: 1
            },
            type: {
              name: 'Number'
            }
          },
          defaultTtlAsIso8601: {
            required: false,
            serializedName: 'defaultTtlAsIso8601',
            type: {
              name: 'TimeSpan'
            }
          },
          feedback: {
            required: false,
            serializedName: 'feedback',
            type: {
              name: 'Composite',
              className: 'FeedbackProperties'
            }
          }
        }
      }
    };
  }
}

module.exports = CloudToDeviceProperties;
