/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The routing related properties of the IoT hub. See:
 * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-messaging
 *
 */
class RoutingProperties {
  /**
   * Create a RoutingProperties.
   * @member {object} [endpoints]
   * @member {array} [endpoints.serviceBusQueues] The list of Service Bus queue
   * endpoints that IoT hub routes the messages to, based on the routing rules.
   * @member {array} [endpoints.serviceBusTopics] The list of Service Bus topic
   * endpoints that the IoT hub routes the messages to, based on the routing
   * rules.
   * @member {array} [endpoints.eventHubs] The list of Event Hubs endpoints
   * that IoT hub routes messages to, based on the routing rules. This list
   * does not include the built-in Event Hubs endpoint.
   * @member {array} [endpoints.storageContainers] The list of storage
   * container endpoints that IoT hub routes messages to, based on the routing
   * rules.
   * @member {array} [routes] The list of user-provided routing rules that the
   * IoT hub uses to route messages to built-in and custom endpoints. A maximum
   * of 100 routing rules are allowed for paid hubs and a maximum of 5 routing
   * rules are allowed for free hubs.
   * @member {object} [fallbackRoute] The properties of the route that is used
   * as a fall-back route when none of the conditions specified in the 'routes'
   * section are met. This is an optional parameter. When this property is not
   * set, the messages which do not meet any of the conditions specified in the
   * 'routes' section get routed to the built-in eventhub endpoint.
   * @member {string} [fallbackRoute.condition] The condition which is
   * evaluated in order to apply the fallback route. If the condition is not
   * provided it will evaluate to true by default. For grammar, See:
   * https://docs.microsoft.com/azure/iot-hub/iot-hub-devguide-query-language
   * @member {array} [fallbackRoute.endpointNames] The list of endpoints to
   * which the messages that satisfy the condition are routed to. Currently
   * only 1 endpoint is allowed.
   * @member {boolean} [fallbackRoute.isEnabled] Used to specify whether the
   * fallback route is enabled.
   */
  constructor() {
  }

  /**
   * Defines the metadata of RoutingProperties
   *
   * @returns {object} metadata of RoutingProperties
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'RoutingProperties',
      type: {
        name: 'Composite',
        className: 'RoutingProperties',
        modelProperties: {
          endpoints: {
            required: false,
            serializedName: 'endpoints',
            type: {
              name: 'Composite',
              className: 'RoutingEndpoints'
            }
          },
          routes: {
            required: false,
            serializedName: 'routes',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'RoutePropertiesElementType',
                  type: {
                    name: 'Composite',
                    className: 'RouteProperties'
                  }
              }
            }
          },
          fallbackRoute: {
            required: false,
            serializedName: 'fallbackRoute',
            type: {
              name: 'Composite',
              className: 'FallbackRouteProperties'
            }
          }
        }
      }
    };
  }
}

module.exports = RoutingProperties;
