/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * The X12 agreement envelope settings.
 *
 */
class X12EnvelopeSettings {
  /**
   * Create a X12EnvelopeSettings.
   * @member {number} controlStandardsId The controls standards id.
   * @member {boolean} useControlStandardsIdAsRepetitionCharacter The value
   * indicating whether to use control standards id as repetition character.
   * @member {string} senderApplicationId The sender application id.
   * @member {string} receiverApplicationId The receiver application id.
   * @member {string} controlVersionNumber The control version number.
   * @member {number} interchangeControlNumberLowerBound The interchange
   * control number lower bound.
   * @member {number} interchangeControlNumberUpperBound The interchange
   * control number upper bound.
   * @member {boolean} rolloverInterchangeControlNumber The value indicating
   * whether to rollover interchange control number.
   * @member {boolean} enableDefaultGroupHeaders The value indicating whether
   * to enable default group headers.
   * @member {string} [functionalGroupId] The functional group id.
   * @member {number} groupControlNumberLowerBound The group control number
   * lower bound.
   * @member {number} groupControlNumberUpperBound The group control number
   * upper bound.
   * @member {boolean} rolloverGroupControlNumber The value indicating whether
   * to rollover group control number.
   * @member {string} groupHeaderAgencyCode The group header agency code.
   * @member {string} groupHeaderVersion The group header version.
   * @member {number} transactionSetControlNumberLowerBound The transaction set
   * control number lower bound.
   * @member {number} transactionSetControlNumberUpperBound The transaction set
   * control number upper bound.
   * @member {boolean} rolloverTransactionSetControlNumber The value indicating
   * whether to rollover transaction set control number.
   * @member {string} [transactionSetControlNumberPrefix] The transaction set
   * control number prefix.
   * @member {string} [transactionSetControlNumberSuffix] The transaction set
   * control number suffix.
   * @member {boolean} overwriteExistingTransactionSetControlNumber The value
   * indicating whether to overwrite existing transaction set control number.
   * @member {string} groupHeaderDateFormat The group header date format.
   * Possible values include: 'NotSpecified', 'CCYYMMDD', 'YYMMDD'
   * @member {string} groupHeaderTimeFormat The group header time format.
   * Possible values include: 'NotSpecified', 'HHMM', 'HHMMSS', 'HHMMSSdd',
   * 'HHMMSSd'
   * @member {string} usageIndicator The usage indicator. Possible values
   * include: 'NotSpecified', 'Test', 'Information', 'Production'
   */
  constructor() {
  }

  /**
   * Defines the metadata of X12EnvelopeSettings
   *
   * @returns {object} metadata of X12EnvelopeSettings
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'X12EnvelopeSettings',
      type: {
        name: 'Composite',
        className: 'X12EnvelopeSettings',
        modelProperties: {
          controlStandardsId: {
            required: true,
            serializedName: 'controlStandardsId',
            type: {
              name: 'Number'
            }
          },
          useControlStandardsIdAsRepetitionCharacter: {
            required: true,
            serializedName: 'useControlStandardsIdAsRepetitionCharacter',
            type: {
              name: 'Boolean'
            }
          },
          senderApplicationId: {
            required: true,
            serializedName: 'senderApplicationId',
            type: {
              name: 'String'
            }
          },
          receiverApplicationId: {
            required: true,
            serializedName: 'receiverApplicationId',
            type: {
              name: 'String'
            }
          },
          controlVersionNumber: {
            required: true,
            serializedName: 'controlVersionNumber',
            type: {
              name: 'String'
            }
          },
          interchangeControlNumberLowerBound: {
            required: true,
            serializedName: 'interchangeControlNumberLowerBound',
            type: {
              name: 'Number'
            }
          },
          interchangeControlNumberUpperBound: {
            required: true,
            serializedName: 'interchangeControlNumberUpperBound',
            type: {
              name: 'Number'
            }
          },
          rolloverInterchangeControlNumber: {
            required: true,
            serializedName: 'rolloverInterchangeControlNumber',
            type: {
              name: 'Boolean'
            }
          },
          enableDefaultGroupHeaders: {
            required: true,
            serializedName: 'enableDefaultGroupHeaders',
            type: {
              name: 'Boolean'
            }
          },
          functionalGroupId: {
            required: false,
            serializedName: 'functionalGroupId',
            type: {
              name: 'String'
            }
          },
          groupControlNumberLowerBound: {
            required: true,
            serializedName: 'groupControlNumberLowerBound',
            type: {
              name: 'Number'
            }
          },
          groupControlNumberUpperBound: {
            required: true,
            serializedName: 'groupControlNumberUpperBound',
            type: {
              name: 'Number'
            }
          },
          rolloverGroupControlNumber: {
            required: true,
            serializedName: 'rolloverGroupControlNumber',
            type: {
              name: 'Boolean'
            }
          },
          groupHeaderAgencyCode: {
            required: true,
            serializedName: 'groupHeaderAgencyCode',
            type: {
              name: 'String'
            }
          },
          groupHeaderVersion: {
            required: true,
            serializedName: 'groupHeaderVersion',
            type: {
              name: 'String'
            }
          },
          transactionSetControlNumberLowerBound: {
            required: true,
            serializedName: 'transactionSetControlNumberLowerBound',
            type: {
              name: 'Number'
            }
          },
          transactionSetControlNumberUpperBound: {
            required: true,
            serializedName: 'transactionSetControlNumberUpperBound',
            type: {
              name: 'Number'
            }
          },
          rolloverTransactionSetControlNumber: {
            required: true,
            serializedName: 'rolloverTransactionSetControlNumber',
            type: {
              name: 'Boolean'
            }
          },
          transactionSetControlNumberPrefix: {
            required: false,
            serializedName: 'transactionSetControlNumberPrefix',
            type: {
              name: 'String'
            }
          },
          transactionSetControlNumberSuffix: {
            required: false,
            serializedName: 'transactionSetControlNumberSuffix',
            type: {
              name: 'String'
            }
          },
          overwriteExistingTransactionSetControlNumber: {
            required: true,
            serializedName: 'overwriteExistingTransactionSetControlNumber',
            type: {
              name: 'Boolean'
            }
          },
          groupHeaderDateFormat: {
            required: true,
            serializedName: 'groupHeaderDateFormat',
            type: {
              name: 'Enum',
              allowedValues: [ 'NotSpecified', 'CCYYMMDD', 'YYMMDD' ]
            }
          },
          groupHeaderTimeFormat: {
            required: true,
            serializedName: 'groupHeaderTimeFormat',
            type: {
              name: 'Enum',
              allowedValues: [ 'NotSpecified', 'HHMM', 'HHMMSS', 'HHMMSSdd', 'HHMMSSd' ]
            }
          },
          usageIndicator: {
            required: true,
            serializedName: 'usageIndicator',
            type: {
              name: 'Enum',
              allowedValues: [ 'NotSpecified', 'Test', 'Information', 'Production' ]
            }
          }
        }
      }
    };
  }
}

module.exports = X12EnvelopeSettings;
