/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The X12 oneway agreement.
 *
 */
class X12OneWayAgreement {
  /**
   * Create a X12OneWayAgreement.
   * @member {object} senderBusinessIdentity The sender business identity
   * @member {string} [senderBusinessIdentity.qualifier] The business identity
   * qualifier e.g. as2identity, ZZ, ZZZ, 31, 32
   * @member {string} [senderBusinessIdentity.value] The user defined business
   * identity value.
   * @member {object} receiverBusinessIdentity The receiver business identity
   * @member {string} [receiverBusinessIdentity.qualifier] The business
   * identity qualifier e.g. as2identity, ZZ, ZZZ, 31, 32
   * @member {string} [receiverBusinessIdentity.value] The user defined
   * business identity value.
   * @member {object} protocolSettings The X12 protocol settings.
   * @member {object} [protocolSettings.validationSettings] The X12 validation
   * settings.
   * @member {boolean}
   * [protocolSettings.validationSettings.validateCharacterSet] The value
   * indicating whether to validate character set in the message.
   * @member {boolean}
   * [protocolSettings.validationSettings.checkDuplicateInterchangeControlNumber]
   * The value indicating whether to check for duplicate interchange control
   * number.
   * @member {number}
   * [protocolSettings.validationSettings.interchangeControlNumberValidityDays]
   * The validity period of interchange control number.
   * @member {boolean}
   * [protocolSettings.validationSettings.checkDuplicateGroupControlNumber] The
   * value indicating whether to check for duplicate group control number.
   * @member {boolean}
   * [protocolSettings.validationSettings.checkDuplicateTransactionSetControlNumber]
   * The value indicating whether to check for duplicate transaction set
   * control number.
   * @member {boolean} [protocolSettings.validationSettings.validateEdiTypes]
   * The value indicating whether to Whether to validate EDI types.
   * @member {boolean} [protocolSettings.validationSettings.validateXsdTypes]
   * The value indicating whether to Whether to validate XSD types.
   * @member {boolean}
   * [protocolSettings.validationSettings.allowLeadingAndTrailingSpacesAndZeroes]
   * The value indicating whether to allow leading and trailing spaces and
   * zeroes.
   * @member {boolean}
   * [protocolSettings.validationSettings.trimLeadingAndTrailingSpacesAndZeroes]
   * The value indicating whether to trim leading and trailing spaces and
   * zeroes.
   * @member {string}
   * [protocolSettings.validationSettings.trailingSeparatorPolicy] The trailing
   * separator policy. Possible values include: 'NotSpecified', 'NotAllowed',
   * 'Optional', 'Mandatory'
   * @member {object} [protocolSettings.framingSettings] The X12 framing
   * settings.
   * @member {number} [protocolSettings.framingSettings.dataElementSeparator]
   * The data element separator.
   * @member {number} [protocolSettings.framingSettings.componentSeparator] The
   * component separator.
   * @member {boolean}
   * [protocolSettings.framingSettings.replaceSeparatorsInPayload] The value
   * indicating whether to replace separators in payload.
   * @member {number} [protocolSettings.framingSettings.replaceCharacter] The
   * replacement character.
   * @member {number} [protocolSettings.framingSettings.segmentTerminator] The
   * segment terminator.
   * @member {string} [protocolSettings.framingSettings.characterSet] The X12
   * character set. Possible values include: 'NotSpecified', 'Basic',
   * 'Extended', 'UTF8'
   * @member {string}
   * [protocolSettings.framingSettings.segmentTerminatorSuffix] The segment
   * terminator suffix. Possible values include: 'NotSpecified', 'None', 'CR',
   * 'LF', 'CRLF'
   * @member {object} [protocolSettings.envelopeSettings] The X12 envelope
   * settings.
   * @member {number} [protocolSettings.envelopeSettings.controlStandardsId]
   * The controls standards id.
   * @member {boolean}
   * [protocolSettings.envelopeSettings.useControlStandardsIdAsRepetitionCharacter]
   * The value indicating whether to use control standards id as repetition
   * character.
   * @member {string} [protocolSettings.envelopeSettings.senderApplicationId]
   * The sender application id.
   * @member {string} [protocolSettings.envelopeSettings.receiverApplicationId]
   * The receiver application id.
   * @member {string} [protocolSettings.envelopeSettings.controlVersionNumber]
   * The control version number.
   * @member {number}
   * [protocolSettings.envelopeSettings.interchangeControlNumberLowerBound] The
   * interchange  control number lower bound.
   * @member {number}
   * [protocolSettings.envelopeSettings.interchangeControlNumberUpperBound] The
   * interchange  control number upper bound.
   * @member {boolean}
   * [protocolSettings.envelopeSettings.rolloverInterchangeControlNumber] The
   * value indicating whether to rollover interchange control number.
   * @member {boolean}
   * [protocolSettings.envelopeSettings.enableDefaultGroupHeaders] The value
   * indicating whether to enable default group headers.
   * @member {string} [protocolSettings.envelopeSettings.functionalGroupId] The
   * functional group id.
   * @member {number}
   * [protocolSettings.envelopeSettings.groupControlNumberLowerBound] The group
   * control number lower bound.
   * @member {number}
   * [protocolSettings.envelopeSettings.groupControlNumberUpperBound] The group
   * control number upper bound.
   * @member {boolean}
   * [protocolSettings.envelopeSettings.rolloverGroupControlNumber] The value
   * indicating whether to rollover group control number.
   * @member {string} [protocolSettings.envelopeSettings.groupHeaderAgencyCode]
   * The group header agency code.
   * @member {string} [protocolSettings.envelopeSettings.groupHeaderVersion]
   * The group header version.
   * @member {number}
   * [protocolSettings.envelopeSettings.transactionSetControlNumberLowerBound]
   * The transaction set control number lower bound.
   * @member {number}
   * [protocolSettings.envelopeSettings.transactionSetControlNumberUpperBound]
   * The transaction set control number upper bound.
   * @member {boolean}
   * [protocolSettings.envelopeSettings.rolloverTransactionSetControlNumber]
   * The value indicating whether to rollover transaction set control number.
   * @member {string}
   * [protocolSettings.envelopeSettings.transactionSetControlNumberPrefix] The
   * transaction set control number prefix.
   * @member {string}
   * [protocolSettings.envelopeSettings.transactionSetControlNumberSuffix] The
   * transaction set control number suffix.
   * @member {boolean}
   * [protocolSettings.envelopeSettings.overwriteExistingTransactionSetControlNumber]
   * The value indicating whether to overwrite existing transaction set control
   * number.
   * @member {string} [protocolSettings.envelopeSettings.groupHeaderDateFormat]
   * The group header date format. Possible values include: 'NotSpecified',
   * 'CCYYMMDD', 'YYMMDD'
   * @member {string} [protocolSettings.envelopeSettings.groupHeaderTimeFormat]
   * The group header time format. Possible values include: 'NotSpecified',
   * 'HHMM', 'HHMMSS', 'HHMMSSdd', 'HHMMSSd'
   * @member {string} [protocolSettings.envelopeSettings.usageIndicator] The
   * usage indicator. Possible values include: 'NotSpecified', 'Test',
   * 'Information', 'Production'
   * @member {object} [protocolSettings.acknowledgementSettings] The X12
   * acknowledgment settings.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.needTechnicalAcknowledgement]
   * The value indicating whether technical acknowledgement is needed.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.batchTechnicalAcknowledgements]
   * The value indicating whether to batch the technical acknowledgements.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.needFunctionalAcknowledgement]
   * The value indicating whether functional acknowledgement is needed.
   * @member {string}
   * [protocolSettings.acknowledgementSettings.functionalAcknowledgementVersion]
   * The functional acknowledgement version.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.batchFunctionalAcknowledgements]
   * The value indicating whether to batch functional acknowledgements.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.needImplementationAcknowledgement]
   * The value indicating whether implementation acknowledgement is needed.
   * @member {string}
   * [protocolSettings.acknowledgementSettings.implementationAcknowledgementVersion]
   * The implementation acknowledgement version.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.batchImplementationAcknowledgements]
   * The value indicating whether to batch implementation acknowledgements.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.needLoopForValidMessages] The
   * value indicating whether a loop is needed for valid messages.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.sendSynchronousAcknowledgement]
   * The value indicating whether to send synchronous acknowledgement.
   * @member {string}
   * [protocolSettings.acknowledgementSettings.acknowledgementControlNumberPrefix]
   * The acknowledgement control number prefix.
   * @member {string}
   * [protocolSettings.acknowledgementSettings.acknowledgementControlNumberSuffix]
   * The acknowledgement control number suffix.
   * @member {number}
   * [protocolSettings.acknowledgementSettings.acknowledgementControlNumberLowerBound]
   * The acknowledgement control number lower bound.
   * @member {number}
   * [protocolSettings.acknowledgementSettings.acknowledgementControlNumberUpperBound]
   * The acknowledgement control number upper bound.
   * @member {boolean}
   * [protocolSettings.acknowledgementSettings.rolloverAcknowledgementControlNumber]
   * The value indicating whether to rollover acknowledgement control number.
   * @member {object} [protocolSettings.messageFilter] The X12 message filter.
   * @member {string} [protocolSettings.messageFilter.messageFilterType] The
   * message filter type. Possible values include: 'NotSpecified', 'Include',
   * 'Exclude'
   * @member {object} [protocolSettings.securitySettings] The X12 security
   * settings.
   * @member {string}
   * [protocolSettings.securitySettings.authorizationQualifier] The
   * authorization qualifier.
   * @member {string} [protocolSettings.securitySettings.authorizationValue]
   * The authorization value.
   * @member {string} [protocolSettings.securitySettings.securityQualifier] The
   * security qualifier.
   * @member {string} [protocolSettings.securitySettings.passwordValue] The
   * password value.
   * @member {object} [protocolSettings.processingSettings] The X12 processing
   * settings.
   * @member {boolean} [protocolSettings.processingSettings.maskSecurityInfo]
   * The value indicating whether to mask security information.
   * @member {boolean}
   * [protocolSettings.processingSettings.convertImpliedDecimal] The value
   * indicating whether to convert numerical type to implied decimal.
   * @member {boolean}
   * [protocolSettings.processingSettings.preserveInterchange] The value
   * indicating whether to preserve interchange.
   * @member {boolean}
   * [protocolSettings.processingSettings.suspendInterchangeOnError] The value
   * indicating whether to suspend interchange on error.
   * @member {boolean}
   * [protocolSettings.processingSettings.createEmptyXmlTagsForTrailingSeparators]
   * The value indicating whether to create empty xml tags for trailing
   * separators.
   * @member {boolean}
   * [protocolSettings.processingSettings.useDotAsDecimalSeparator] The value
   * indicating whether to use dot as decimal separator.
   * @member {array} [protocolSettings.envelopeOverrides] The X12 envelope
   * override settings.
   * @member {array} [protocolSettings.validationOverrides] The X12 validation
   * override settings.
   * @member {array} [protocolSettings.messageFilterList] The X12 message
   * filter list.
   * @member {array} [protocolSettings.schemaReferences] The X12 schema
   * references.
   * @member {array} [protocolSettings.x12DelimiterOverrides] The X12 delimiter
   * override settings.
   */
  constructor() {
  }

  /**
   * Defines the metadata of X12OneWayAgreement
   *
   * @returns {object} metadata of X12OneWayAgreement
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'X12OneWayAgreement',
      type: {
        name: 'Composite',
        className: 'X12OneWayAgreement',
        modelProperties: {
          senderBusinessIdentity: {
            required: true,
            serializedName: 'senderBusinessIdentity',
            type: {
              name: 'Composite',
              className: 'BusinessIdentity'
            }
          },
          receiverBusinessIdentity: {
            required: true,
            serializedName: 'receiverBusinessIdentity',
            type: {
              name: 'Composite',
              className: 'BusinessIdentity'
            }
          },
          protocolSettings: {
            required: true,
            serializedName: 'protocolSettings',
            type: {
              name: 'Composite',
              className: 'X12ProtocolSettings'
            }
          }
        }
      }
    };
  }
}

module.exports = X12OneWayAgreement;
