/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * CommitmentAssociations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMLCommitmentPlansManagementClient.
 */
export interface CommitmentAssociations {


    /**
     * Get a commitment association.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {string} commitmentAssociationName The commitment association name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentAssociation>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentAssociation>>;

    /**
     * Get a commitment association.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {string} commitmentAssociationName The commitment association name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentAssociation} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentAssociation} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentAssociation} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentAssociation>;
    get(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, callback: ServiceCallback<models.CommitmentAssociation>): void;
    get(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentAssociation>): void;


    /**
     * Get all commitment associations for a parent commitment plan.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentAssociationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, commitmentPlanName: string, options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentAssociationListResult>>;

    /**
     * Get all commitment associations for a parent commitment plan.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentAssociationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentAssociationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentAssociationListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, commitmentPlanName: string, options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentAssociationListResult>;
    list(resourceGroupName: string, commitmentPlanName: string, callback: ServiceCallback<models.CommitmentAssociationListResult>): void;
    list(resourceGroupName: string, commitmentPlanName: string, options: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentAssociationListResult>): void;


    /**
     * Re-parent a commitment association from one commitment plan to another.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {string} commitmentAssociationName The commitment association name.
     *
     * @param {object} movePayload The move request payload.
     *
     * @param {string} [movePayload.destinationPlanId] The ARM ID of the commitment
     * plan to re-parent the commitment association to.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentAssociation>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    moveWithHttpOperationResponse(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, movePayload: models.MoveCommitmentAssociationRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentAssociation>>;

    /**
     * Re-parent a commitment association from one commitment plan to another.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {string} commitmentAssociationName The commitment association name.
     *
     * @param {object} movePayload The move request payload.
     *
     * @param {string} [movePayload.destinationPlanId] The ARM ID of the commitment
     * plan to re-parent the commitment association to.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentAssociation} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentAssociation} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentAssociation} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    move(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, movePayload: models.MoveCommitmentAssociationRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentAssociation>;
    move(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, movePayload: models.MoveCommitmentAssociationRequest, callback: ServiceCallback<models.CommitmentAssociation>): void;
    move(resourceGroupName: string, commitmentPlanName: string, commitmentAssociationName: string, movePayload: models.MoveCommitmentAssociationRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentAssociation>): void;


    /**
     * Get all commitment associations for a parent commitment plan.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentAssociationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentAssociationListResult>>;

    /**
     * Get all commitment associations for a parent commitment plan.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentAssociationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentAssociationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentAssociationListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentAssociationListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.CommitmentAssociationListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentAssociationListResult>): void;
}

/**
 * @class
 * CommitmentPlans
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMLCommitmentPlansManagementClient.
 */
export interface CommitmentPlans {


    /**
     * Retrieve an Azure ML commitment plan by its subscription, resource group and
     * name.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlan>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlan>>;

    /**
     * Retrieve an Azure ML commitment plan by its subscription, resource group and
     * name.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlan} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlan} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlan} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlan>;
    get(resourceGroupName: string, commitmentPlanName: string, callback: ServiceCallback<models.CommitmentPlan>): void;
    get(resourceGroupName: string, commitmentPlanName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlan>): void;


    /**
     * Create a new Azure ML commitment plan resource or updates an existing one.
     *
     * @param {object} createOrUpdatePayload The payload to create or update the
     * Azure ML commitment plan.
     *
     * @param {string} [createOrUpdatePayload.etag] An entity tag used to enforce
     * optimistic concurrency.
     *
     * @param {object} [createOrUpdatePayload.sku] The commitment plan SKU.
     *
     * @param {number} [createOrUpdatePayload.sku.capacity] The scale-out capacity
     * of the resource. 1 is 1x, 2 is 2x, etc. This impacts the quantities and cost
     * of any commitment plan resource.
     *
     * @param {string} [createOrUpdatePayload.sku.name] The SKU name. Along with
     * tier, uniquely identifies the SKU.
     *
     * @param {string} [createOrUpdatePayload.sku.tier] The SKU tier. Along with
     * name, uniquely identifies the SKU.
     *
     * @param {string} createOrUpdatePayload.location Resource location.
     *
     * @param {object} [createOrUpdatePayload.tags] User-defined tags for the
     * resource.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlan>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(createOrUpdatePayload: models.CommitmentPlan, resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlan>>;

    /**
     * Create a new Azure ML commitment plan resource or updates an existing one.
     *
     * @param {object} createOrUpdatePayload The payload to create or update the
     * Azure ML commitment plan.
     *
     * @param {string} [createOrUpdatePayload.etag] An entity tag used to enforce
     * optimistic concurrency.
     *
     * @param {object} [createOrUpdatePayload.sku] The commitment plan SKU.
     *
     * @param {number} [createOrUpdatePayload.sku.capacity] The scale-out capacity
     * of the resource. 1 is 1x, 2 is 2x, etc. This impacts the quantities and cost
     * of any commitment plan resource.
     *
     * @param {string} [createOrUpdatePayload.sku.name] The SKU name. Along with
     * tier, uniquely identifies the SKU.
     *
     * @param {string} [createOrUpdatePayload.sku.tier] The SKU tier. Along with
     * name, uniquely identifies the SKU.
     *
     * @param {string} createOrUpdatePayload.location Resource location.
     *
     * @param {object} [createOrUpdatePayload.tags] User-defined tags for the
     * resource.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlan} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlan} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlan} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(createOrUpdatePayload: models.CommitmentPlan, resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlan>;
    createOrUpdate(createOrUpdatePayload: models.CommitmentPlan, resourceGroupName: string, commitmentPlanName: string, callback: ServiceCallback<models.CommitmentPlan>): void;
    createOrUpdate(createOrUpdatePayload: models.CommitmentPlan, resourceGroupName: string, commitmentPlanName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlan>): void;


    /**
     * Remove an existing Azure ML commitment plan.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    removeWithHttpOperationResponse(resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Remove an existing Azure ML commitment plan.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    remove(resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    remove(resourceGroupName: string, commitmentPlanName: string, callback: ServiceCallback<void>): void;
    remove(resourceGroupName: string, commitmentPlanName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Patch an existing Azure ML commitment plan resource.
     *
     * @param {object} patchPayload The payload to use to patch the Azure ML
     * commitment plan. Only tags and SKU may be modified on an existing commitment
     * plan.
     *
     * @param {object} [patchPayload.tags] User-defined tags for the commitment
     * plan.
     *
     * @param {object} [patchPayload.sku] The commitment plan SKU.
     *
     * @param {number} [patchPayload.sku.capacity] The scale-out capacity of the
     * resource. 1 is 1x, 2 is 2x, etc. This impacts the quantities and cost of any
     * commitment plan resource.
     *
     * @param {string} [patchPayload.sku.name] The SKU name. Along with tier,
     * uniquely identifies the SKU.
     *
     * @param {string} [patchPayload.sku.tier] The SKU tier. Along with name,
     * uniquely identifies the SKU.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlan>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    patchWithHttpOperationResponse(patchPayload: models.CommitmentPlanPatchPayload, resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlan>>;

    /**
     * Patch an existing Azure ML commitment plan resource.
     *
     * @param {object} patchPayload The payload to use to patch the Azure ML
     * commitment plan. Only tags and SKU may be modified on an existing commitment
     * plan.
     *
     * @param {object} [patchPayload.tags] User-defined tags for the commitment
     * plan.
     *
     * @param {object} [patchPayload.sku] The commitment plan SKU.
     *
     * @param {number} [patchPayload.sku.capacity] The scale-out capacity of the
     * resource. 1 is 1x, 2 is 2x, etc. This impacts the quantities and cost of any
     * commitment plan resource.
     *
     * @param {string} [patchPayload.sku.name] The SKU name. Along with tier,
     * uniquely identifies the SKU.
     *
     * @param {string} [patchPayload.sku.tier] The SKU tier. Along with name,
     * uniquely identifies the SKU.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlan} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlan} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlan} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    patch(patchPayload: models.CommitmentPlanPatchPayload, resourceGroupName: string, commitmentPlanName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlan>;
    patch(patchPayload: models.CommitmentPlanPatchPayload, resourceGroupName: string, commitmentPlanName: string, callback: ServiceCallback<models.CommitmentPlan>): void;
    patch(patchPayload: models.CommitmentPlanPatchPayload, resourceGroupName: string, commitmentPlanName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlan>): void;


    /**
     * Retrieve all Azure ML commitment plans in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlanListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlanListResult>>;

    /**
     * Retrieve all Azure ML commitment plans in a subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlanListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlanListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlanListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlanListResult>;
    list(callback: ServiceCallback<models.CommitmentPlanListResult>): void;
    list(options: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlanListResult>): void;


    /**
     * Retrieve all Azure ML commitment plans in a resource group.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlanListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listInResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlanListResult>>;

    /**
     * Retrieve all Azure ML commitment plans in a resource group.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlanListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlanListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlanListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listInResourceGroup(resourceGroupName: string, options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlanListResult>;
    listInResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.CommitmentPlanListResult>): void;
    listInResourceGroup(resourceGroupName: string, options: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlanListResult>): void;


    /**
     * Retrieve all Azure ML commitment plans in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlanListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlanListResult>>;

    /**
     * Retrieve all Azure ML commitment plans in a subscription.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlanListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlanListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlanListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlanListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.CommitmentPlanListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlanListResult>): void;


    /**
     * Retrieve all Azure ML commitment plans in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CommitmentPlanListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listInResourceGroupNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CommitmentPlanListResult>>;

    /**
     * Retrieve all Azure ML commitment plans in a resource group.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CommitmentPlanListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CommitmentPlanListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CommitmentPlanListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listInResourceGroupNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CommitmentPlanListResult>;
    listInResourceGroupNext(nextPageLink: string, callback: ServiceCallback<models.CommitmentPlanListResult>): void;
    listInResourceGroupNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CommitmentPlanListResult>): void;
}

/**
 * @class
 * UsageHistory
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the AzureMLCommitmentPlansManagementClient.
 */
export interface UsageHistory {


    /**
     * Retrieve the usage history for an Azure ML commitment plan.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<PlanUsageHistoryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, commitmentPlanName: string, options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.PlanUsageHistoryListResult>>;

    /**
     * Retrieve the usage history for an Azure ML commitment plan.
     *
     * @param {string} resourceGroupName The resource group name.
     *
     * @param {string} commitmentPlanName The Azure ML commitment plan name.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {string} [options.skipToken] Continuation token for pagination.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {PlanUsageHistoryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {PlanUsageHistoryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link PlanUsageHistoryListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, commitmentPlanName: string, options?: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }): Promise<models.PlanUsageHistoryListResult>;
    list(resourceGroupName: string, commitmentPlanName: string, callback: ServiceCallback<models.PlanUsageHistoryListResult>): void;
    list(resourceGroupName: string, commitmentPlanName: string, options: { skipToken? : string, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.PlanUsageHistoryListResult>): void;


    /**
     * Retrieve the usage history for an Azure ML commitment plan.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<PlanUsageHistoryListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listNextWithHttpOperationResponse(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.PlanUsageHistoryListResult>>;

    /**
     * Retrieve the usage history for an Azure ML commitment plan.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {PlanUsageHistoryListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {PlanUsageHistoryListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link PlanUsageHistoryListResult} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listNext(nextPageLink: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.PlanUsageHistoryListResult>;
    listNext(nextPageLink: string, callback: ServiceCallback<models.PlanUsageHistoryListResult>): void;
    listNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.PlanUsageHistoryListResult>): void;
}
