/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Class representing a CampaignStatisticsResult.
 */
class CampaignStatisticsResult {
  /**
   * Create a CampaignStatisticsResult.
   * @member {number} [queued] Number of times the campaign was registered to
   * be pushed.
   * @member {number} [pushed] Number of pushes performed.
   * @member {number} [pushedNative] Total number of native pushes. Information
   * only available on Android, iOS, Windows Phone and Windows applications.
   * @member {number} [pushedNativeGoogle] Number of C2DM/GCM pushes (available
   * only on Android applications).
   * @member {number} [pushedNativeAdm] Number of ADM pushes (available only on
   * Android applications).
   * @member {number} [delivered] Number of times the campaign was received by
   * the application (Not present in case of a native-push).
   * @member {number} [dropped] Number of times the campaign was dropped by the
   * application. It can happen if the SDK failed to parse the campaign payload
   * or if an error occurred while trying to notify the end-user (Not present
   * in case of a native-push).
   * @member {number} [systemNotificationDisplayed] Number of times the system
   * notification was displayed (Not present in case of a data-push or a
   * native-push). On Android it corresponds to a status bar notification.
   * @member {number} [inAppNotificationDisplayed] Number of times the in-app
   * notification was displayed (Not present in case of a data-push or a
   * native-push).
   * @member {number} [contentDisplayed] Number of times the campaign’s content
   * view was displayed (Not present in case of a notification-only
   * announcement, a data-push or a native-push).
   * @member {number} [systemNotificationActioned] Number of times the system
   * notification (On Android it corresponds to a status bar notification. On
   * iOS, it is the Apple Push notification) was actioned.
   * @member {number} [systemNotificationExited] Number of times the system
   * notification was exited (Not present in case of a data-push or a
   * native-push).
   * @member {number} [inAppNotificationActioned] Number of times the in-app
   * notification was actioned (Not present in case of a data-push or a
   * native-push).
   * @member {number} [inAppNotificationExited] Number of times the in-app
   * notification was exited (Not present in case of a data-push or a
   * native-push).
   * @member {number} [contentActioned] Number of times the campaign’s content
   * view was actioned (Not present in case of a notification-only announcement
   * or a native-push).
   * @member {number} [contentExited] Number of times the campaign’s content
   * view was exited (Not present in case of a notification-only announcement
   * or a native-push).
   * @member {object} [answers] Poll specific statistics.
   */
  constructor() {
  }

  /**
   * Defines the metadata of CampaignStatisticsResult
   *
   * @returns {object} metadata of CampaignStatisticsResult
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CampaignStatisticsResult',
      type: {
        name: 'Composite',
        className: 'CampaignStatisticsResult',
        modelProperties: {
          queued: {
            required: false,
            serializedName: 'queued',
            type: {
              name: 'Number'
            }
          },
          pushed: {
            required: false,
            serializedName: 'pushed',
            type: {
              name: 'Number'
            }
          },
          pushedNative: {
            required: false,
            serializedName: 'pushed-native',
            type: {
              name: 'Number'
            }
          },
          pushedNativeGoogle: {
            required: false,
            serializedName: 'pushed-native-google',
            type: {
              name: 'Number'
            }
          },
          pushedNativeAdm: {
            required: false,
            serializedName: 'pushed-native-adm',
            type: {
              name: 'Number'
            }
          },
          delivered: {
            required: false,
            serializedName: 'delivered',
            type: {
              name: 'Number'
            }
          },
          dropped: {
            required: false,
            serializedName: 'dropped',
            type: {
              name: 'Number'
            }
          },
          systemNotificationDisplayed: {
            required: false,
            serializedName: 'system-notification-displayed',
            type: {
              name: 'Number'
            }
          },
          inAppNotificationDisplayed: {
            required: false,
            serializedName: 'in-app-notification-displayed',
            type: {
              name: 'Number'
            }
          },
          contentDisplayed: {
            required: false,
            serializedName: 'content-displayed',
            type: {
              name: 'Number'
            }
          },
          systemNotificationActioned: {
            required: false,
            serializedName: 'system-notification-actioned',
            type: {
              name: 'Number'
            }
          },
          systemNotificationExited: {
            required: false,
            serializedName: 'system-notification-exited',
            type: {
              name: 'Number'
            }
          },
          inAppNotificationActioned: {
            required: false,
            serializedName: 'in-app-notification-actioned',
            type: {
              name: 'Number'
            }
          },
          inAppNotificationExited: {
            required: false,
            serializedName: 'in-app-notification-exited',
            type: {
              name: 'Number'
            }
          },
          contentActioned: {
            required: false,
            serializedName: 'content-actioned',
            type: {
              name: 'Number'
            }
          },
          contentExited: {
            required: false,
            serializedName: 'content-exited',
            type: {
              name: 'Number'
            }
          },
          answers: {
            required: false,
            serializedName: 'answers',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Object'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = CampaignStatisticsResult;
