/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Class representing a Device.
 */
class Device {
  /**
   * Create a Device.
   * @member {string} deviceId The device result.
   * @member {object} [meta] Application usage data.
   * @member {number} [meta.firstSeen] First time the device used the
   * application in milliseconds since January 1st, 1970 UTC.
   * @member {number} [meta.lastSeen] Last time the device used the application
   * in milliseconds since January 1st, 1970 UTC.
   * @member {number} [meta.lastInfo] Timestamp corresponding to the info
   * object in milliseconds since January 1st, 1970 UTC.
   * @member {number} [meta.lastLocation] Timestamp corresponding to the
   * location object in milliseconds since January 1st, 1970 UTC.
   * @member {boolean} [meta.nativePushEnabled] Boolean indicating if native
   * push notifications (like Android’s GCM or Apple’s APNS) are enabled for
   * the application. This boolean is set to true when the application
   * registers successfully to the native push service, and set to false when
   * the native push service reports to Mobile Engagement that the application
   * can no longer be pushed (which means that it has been uninstalled). This
   * report is performed a few hours after Mobile Engagement has tried to
   * perform a native push to a device on which the application has been
   * uninstalled.
   * @member {object} [info] Last technical data received (concerning device,
   * system, network and application identification).
   * @member {string} [info.phoneModel] Phone model.
   * @member {string} [info.phoneManufacturer] Phone manufacturer
   * @member {string} [info.firmwareVersion] Firmware version.
   * @member {string} [info.firmwareName] Firmware name.
   * @member {number} [info.androidAPILevel] Android API level.
   * @member {string} [info.carrierCountry] Carrier country.
   * @member {string} [info.locale] Locale code.
   * @member {string} [info.carrierName] Carrier name.
   * @member {string} [info.networkType] Network type.
   * @member {string} [info.networkSubtype] Network sub-type.
   * @member {string} [info.applicationVersionName] Application version name.
   * @member {number} [info.applicationVersionCode] Application version code.
   * @member {number} [info.timeZoneOffset] The offset in minutes from UTC for
   * the device time zone, including daylight savings time.
   * @member {string} [info.serviceVersion] SDK version.
   * @member {object} [location] Last geo-location data received.
   * @member {string} [location.countrycode] The ISO 3166 two-letter country
   * code.
   * @member {string} [location.region] An administrative region of the nation,
   * such as a state or province.
   * @member {string} [location.locality] A locality within the administrative
   * region, such as a town or city.
   * @member {object} [appInfo] Also known as tags, a key-value set as a JSON
   * object.
   */
  constructor() {
  }

  /**
   * Defines the metadata of Device
   *
   * @returns {object} metadata of Device
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Device',
      type: {
        name: 'Composite',
        className: 'Device',
        modelProperties: {
          deviceId: {
            required: true,
            serializedName: 'deviceId',
            type: {
              name: 'String'
            }
          },
          meta: {
            required: false,
            serializedName: 'meta',
            type: {
              name: 'Composite',
              className: 'DeviceMeta'
            }
          },
          info: {
            required: false,
            serializedName: 'info',
            type: {
              name: 'Composite',
              className: 'DeviceInfo'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'Composite',
              className: 'DeviceLocation'
            }
          },
          appInfo: {
            required: false,
            serializedName: 'appInfo',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = Device;
