/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Application gateway resource
 *
 * @extends models['Resource']
 */
class ApplicationGateway extends models['Resource'] {
  /**
   * Create a ApplicationGateway.
   * @member {object} [sku] SKU of the application gateway resource.
   * @member {string} [sku.name] Name of an application gateway SKU. Possible
   * values include: 'Standard_Small', 'Standard_Medium', 'Standard_Large',
   * 'WAF_Medium', 'WAF_Large'
   * @member {string} [sku.tier] Tier of an application gateway. Possible
   * values include: 'Standard', 'WAF'
   * @member {number} [sku.capacity] Capacity (instance count) of an
   * application gateway.
   * @member {object} [sslPolicy] SSL policy of the application gateway
   * resource.
   * @member {array} [sslPolicy.disabledSslProtocols] Ssl protocols to be
   * disabled on application gateway.
   * @member {string} [sslPolicy.policyType] Type of Ssl Policy. Possible
   * values include: 'Predefined', 'Custom'
   * @member {string} [sslPolicy.policyName] Name of Ssl predefined policy.
   * Possible values include: 'AppGwSslPolicy20150501',
   * 'AppGwSslPolicy20170401', 'AppGwSslPolicy20170401S'
   * @member {array} [sslPolicy.cipherSuites] Ssl cipher suites to be enabled
   * in the specified order to application gateway.
   * @member {string} [sslPolicy.minProtocolVersion] Minimum version of Ssl
   * protocol to be supported on application gateway. Possible values include:
   * 'TLSv1_0', 'TLSv1_1', 'TLSv1_2'
   * @member {string} [operationalState] Operational state of the application
   * gateway resource. Possible values include: 'Stopped', 'Starting',
   * 'Running', 'Stopping'
   * @member {array} [gatewayIPConfigurations] Subnets of application the
   * gateway resource.
   * @member {array} [authenticationCertificates] Authentication certificates
   * of the application gateway resource.
   * @member {array} [sslCertificates] SSL certificates of the application
   * gateway resource.
   * @member {array} [frontendIPConfigurations] Frontend IP addresses of the
   * application gateway resource.
   * @member {array} [frontendPorts] Frontend ports of the application gateway
   * resource.
   * @member {array} [probes] Probes of the application gateway resource.
   * @member {array} [backendAddressPools] Backend address pool of the
   * application gateway resource.
   * @member {array} [backendHttpSettingsCollection] Backend http settings of
   * the application gateway resource.
   * @member {array} [httpListeners] Http listeners of the application gateway
   * resource.
   * @member {array} [urlPathMaps] URL path map of the application gateway
   * resource.
   * @member {array} [requestRoutingRules] Request routing rules of the
   * application gateway resource.
   * @member {array} [redirectConfigurations] Redirect configurations of the
   * application gateway resource.
   * @member {object} [webApplicationFirewallConfiguration] Web application
   * firewall configuration.
   * @member {boolean} [webApplicationFirewallConfiguration.enabled] Whether
   * the web application firewall is enabled or not.
   * @member {string} [webApplicationFirewallConfiguration.firewallMode] Web
   * application firewall mode. Possible values include: 'Detection',
   * 'Prevention'
   * @member {string} [webApplicationFirewallConfiguration.ruleSetType] The
   * type of the web application firewall rule set. Possible values are:
   * 'OWASP'.
   * @member {string} [webApplicationFirewallConfiguration.ruleSetVersion] The
   * version of the rule set type.
   * @member {array} [webApplicationFirewallConfiguration.disabledRuleGroups]
   * The disabled rule groups.
   * @member {string} [resourceGuid] Resource GUID property of the application
   * gateway resource.
   * @member {string} [provisioningState] Provisioning state of the application
   * gateway resource. Possible values are: 'Updating', 'Deleting', and
   * 'Failed'.
   * @member {string} [etag] A unique read-only string that changes whenever
   * the resource is updated.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ApplicationGateway
   *
   * @returns {object} metadata of ApplicationGateway
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ApplicationGateway',
      type: {
        name: 'Composite',
        className: 'ApplicationGateway',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          sku: {
            required: false,
            serializedName: 'properties.sku',
            type: {
              name: 'Composite',
              className: 'ApplicationGatewaySku'
            }
          },
          sslPolicy: {
            required: false,
            serializedName: 'properties.sslPolicy',
            type: {
              name: 'Composite',
              className: 'ApplicationGatewaySslPolicy'
            }
          },
          operationalState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.operationalState',
            type: {
              name: 'String'
            }
          },
          gatewayIPConfigurations: {
            required: false,
            serializedName: 'properties.gatewayIPConfigurations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayIPConfigurationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayIPConfiguration'
                  }
              }
            }
          },
          authenticationCertificates: {
            required: false,
            serializedName: 'properties.authenticationCertificates',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayAuthenticationCertificateElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayAuthenticationCertificate'
                  }
              }
            }
          },
          sslCertificates: {
            required: false,
            serializedName: 'properties.sslCertificates',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewaySslCertificateElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewaySslCertificate'
                  }
              }
            }
          },
          frontendIPConfigurations: {
            required: false,
            serializedName: 'properties.frontendIPConfigurations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayFrontendIPConfigurationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayFrontendIPConfiguration'
                  }
              }
            }
          },
          frontendPorts: {
            required: false,
            serializedName: 'properties.frontendPorts',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayFrontendPortElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayFrontendPort'
                  }
              }
            }
          },
          probes: {
            required: false,
            serializedName: 'properties.probes',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayProbeElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayProbe'
                  }
              }
            }
          },
          backendAddressPools: {
            required: false,
            serializedName: 'properties.backendAddressPools',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayBackendAddressPoolElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayBackendAddressPool'
                  }
              }
            }
          },
          backendHttpSettingsCollection: {
            required: false,
            serializedName: 'properties.backendHttpSettingsCollection',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayBackendHttpSettingsElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayBackendHttpSettings'
                  }
              }
            }
          },
          httpListeners: {
            required: false,
            serializedName: 'properties.httpListeners',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayHttpListenerElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayHttpListener'
                  }
              }
            }
          },
          urlPathMaps: {
            required: false,
            serializedName: 'properties.urlPathMaps',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayUrlPathMapElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayUrlPathMap'
                  }
              }
            }
          },
          requestRoutingRules: {
            required: false,
            serializedName: 'properties.requestRoutingRules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayRequestRoutingRuleElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayRequestRoutingRule'
                  }
              }
            }
          },
          redirectConfigurations: {
            required: false,
            serializedName: 'properties.redirectConfigurations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayRedirectConfigurationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayRedirectConfiguration'
                  }
              }
            }
          },
          webApplicationFirewallConfiguration: {
            required: false,
            serializedName: 'properties.webApplicationFirewallConfiguration',
            type: {
              name: 'Composite',
              className: 'ApplicationGatewayWebApplicationFirewallConfiguration'
            }
          },
          resourceGuid: {
            required: false,
            serializedName: 'properties.resourceGuid',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ApplicationGateway;
