/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * WorkspaceCollections
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the PowerBIEmbeddedManagementClient.
 */
export interface WorkspaceCollections {


    /**
     * Retrieves an existing Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getByNameWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollection>>;

    /**
     * Retrieves an existing Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getByName(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollection>;
    getByName(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<models.WorkspaceCollection>): void;
    getByName(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollection>): void;


    /**
     * Creates a new Power BI Workspace Collection with the specified properties. A
     * Power BI Workspace Collection contains one or more workspaces, and can be
     * used to provision keys that provide API access to those workspaces.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} body Create workspace collection request
     *
     * @param {string} [body.location] Azure location
     *
     * @param {object} [body.tags]
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, body: models.CreateWorkspaceCollectionRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollection>>;

    /**
     * Creates a new Power BI Workspace Collection with the specified properties. A
     * Power BI Workspace Collection contains one or more workspaces, and can be
     * used to provision keys that provide API access to those workspaces.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} body Create workspace collection request
     *
     * @param {string} [body.location] Azure location
     *
     * @param {object} [body.tags]
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    create(resourceGroupName: string, workspaceCollectionName: string, body: models.CreateWorkspaceCollectionRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollection>;
    create(resourceGroupName: string, workspaceCollectionName: string, body: models.CreateWorkspaceCollectionRequest, callback: ServiceCallback<models.WorkspaceCollection>): void;
    create(resourceGroupName: string, workspaceCollectionName: string, body: models.CreateWorkspaceCollectionRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollection>): void;


    /**
     * Update an existing Power BI Workspace Collection with the specified
     * properties.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} body Update workspace collection request
     *
     * @param {object} [body.tags]
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollection>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    updateWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, body: models.UpdateWorkspaceCollectionRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollection>>;

    /**
     * Update an existing Power BI Workspace Collection with the specified
     * properties.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} body Update workspace collection request
     *
     * @param {object} [body.tags]
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollection} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollection} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollection} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    update(resourceGroupName: string, workspaceCollectionName: string, body: models.UpdateWorkspaceCollectionRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollection>;
    update(resourceGroupName: string, workspaceCollectionName: string, body: models.UpdateWorkspaceCollectionRequest, callback: ServiceCallback<models.WorkspaceCollection>): void;
    update(resourceGroupName: string, workspaceCollectionName: string, body: models.UpdateWorkspaceCollectionRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollection>): void;


    /**
     * Delete a Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete a Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Verify the specified Power BI Workspace Collection name is valid and not
     * already in use.
     *
     * @param {string} location Azure location
     *
     * @param {object} body Check name availability request
     *
     * @param {string} [body.name] Workspace collection name
     *
     * @param {string} [body.type] Resource type
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<CheckNameResponse>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    checkNameAvailabilityWithHttpOperationResponse(location: string, body: models.CheckNameRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CheckNameResponse>>;

    /**
     * Verify the specified Power BI Workspace Collection name is valid and not
     * already in use.
     *
     * @param {string} location Azure location
     *
     * @param {object} body Check name availability request
     *
     * @param {string} [body.name] Workspace collection name
     *
     * @param {string} [body.type] Resource type
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {CheckNameResponse} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {CheckNameResponse} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link CheckNameResponse} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    checkNameAvailability(location: string, body: models.CheckNameRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CheckNameResponse>;
    checkNameAvailability(location: string, body: models.CheckNameRequest, callback: ServiceCallback<models.CheckNameResponse>): void;
    checkNameAvailability(location: string, body: models.CheckNameRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CheckNameResponse>): void;


    /**
     * Retrieves all existing Power BI workspace collections in the specified
     * resource group.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollectionList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollectionList>>;

    /**
     * Retrieves all existing Power BI workspace collections in the specified
     * resource group.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollectionList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollectionList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollectionList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollectionList>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.WorkspaceCollectionList>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionList>): void;


    /**
     * Retrieves all existing Power BI workspace collections in the specified
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollectionList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollectionList>>;

    /**
     * Retrieves all existing Power BI workspace collections in the specified
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollectionList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollectionList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollectionList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscription(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollectionList>;
    listBySubscription(callback: ServiceCallback<models.WorkspaceCollectionList>): void;
    listBySubscription(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionList>): void;


    /**
     * Retrieves the primary and secondary access keys for the specified Power BI
     * Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollectionAccessKeys>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getAccessKeysWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollectionAccessKeys>>;

    /**
     * Retrieves the primary and secondary access keys for the specified Power BI
     * Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollectionAccessKeys} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollectionAccessKeys} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollectionAccessKeys} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    getAccessKeys(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollectionAccessKeys>;
    getAccessKeys(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;
    getAccessKeys(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;


    /**
     * Regenerates the primary or secondary access key for the specified Power BI
     * Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} body Access key to regenerate
     *
     * @param {string} [body.keyName] Key name. Possible values include: 'key1',
     * 'key2'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceCollectionAccessKeys>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    regenerateKeyWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, body: models.WorkspaceCollectionAccessKey, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceCollectionAccessKeys>>;

    /**
     * Regenerates the primary or secondary access key for the specified Power BI
     * Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} body Access key to regenerate
     *
     * @param {string} [body.keyName] Key name. Possible values include: 'key1',
     * 'key2'
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceCollectionAccessKeys} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceCollectionAccessKeys} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceCollectionAccessKeys} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    regenerateKey(resourceGroupName: string, workspaceCollectionName: string, body: models.WorkspaceCollectionAccessKey, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceCollectionAccessKeys>;
    regenerateKey(resourceGroupName: string, workspaceCollectionName: string, body: models.WorkspaceCollectionAccessKey, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;
    regenerateKey(resourceGroupName: string, workspaceCollectionName: string, body: models.WorkspaceCollectionAccessKey, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;


    /**
     * Migrates an existing Power BI Workspace Collection to a different resource
     * group and/or subscription.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {object} body Workspace migration request
     *
     * @param {string} [body.targetResourceGroup] Name of the resource group the
     * Power BI workspace collections will be migrated to.
     *
     * @param {array} [body.resources]
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    migrateWithHttpOperationResponse(resourceGroupName: string, body: models.MigrateWorkspaceCollectionRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Migrates an existing Power BI Workspace Collection to a different resource
     * group and/or subscription.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {object} body Workspace migration request
     *
     * @param {string} [body.targetResourceGroup] Name of the resource group the
     * Power BI workspace collections will be migrated to.
     *
     * @param {array} [body.resources]
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    migrate(resourceGroupName: string, body: models.MigrateWorkspaceCollectionRequest, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    migrate(resourceGroupName: string, body: models.MigrateWorkspaceCollectionRequest, callback: ServiceCallback<void>): void;
    migrate(resourceGroupName: string, body: models.MigrateWorkspaceCollectionRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * Delete a Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    beginDeleteMethodWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * Delete a Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    beginDeleteMethod(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    beginDeleteMethod(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * Workspaces
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the PowerBIEmbeddedManagementClient.
 */
export interface Workspaces {


    /**
     * Retrieves all existing Power BI workspaces in the specified workspace
     * collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<WorkspaceList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.WorkspaceList>>;

    /**
     * Retrieves all existing Power BI workspaces in the specified workspace
     * collection.
     *
     * @param {string} resourceGroupName Azure resource group
     *
     * @param {string} workspaceCollectionName Power BI Embedded Workspace
     * Collection name
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {WorkspaceList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {WorkspaceList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link WorkspaceList} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(resourceGroupName: string, workspaceCollectionName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.WorkspaceList>;
    list(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<models.WorkspaceList>): void;
    list(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceList>): void;
}
