/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Provides the ability to override other stemming filters with custom
 * dictionary-based stemming. Any dictionary-stemmed terms will be marked as
 * keywords so that they will not be stemmed with stemmers down the chain. Must
 * be placed before any stemming filters. This token filter is implemented
 * using Apache Lucene.
 *
 * @extends models['TokenFilter']
 */
class StemmerOverrideTokenFilter extends models['TokenFilter'] {
  /**
   * Create a StemmerOverrideTokenFilter.
   * @member {array} rules A list of stemming rules in the following format:
   * "word => stem", for example: "ran => run"
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of StemmerOverrideTokenFilter
   *
   * @returns {object} metadata of StemmerOverrideTokenFilter
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: '#Microsoft.Azure.Search.StemmerOverrideTokenFilter',
      type: {
        name: 'Composite',
        className: 'StemmerOverrideTokenFilter',
        modelProperties: {
          name: {
            required: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          odatatype: {
            required: true,
            serializedName: '@odata\\.type',
            type: {
              name: 'String'
            }
          },
          rules: {
            required: true,
            serializedName: 'rules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = StemmerOverrideTokenFilter;
