/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;

/**
 * Resets the change tracking state associated with an Azure Search indexer.
 *
 * @param {string} indexerName The name of the indexer to reset.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _reset(indexerName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexerName === null || indexerName === undefined || typeof indexerName.valueOf() !== 'string') {
      throw new Error('indexerName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers(\'{indexerName}\')/search.reset';
  requestUrl = requestUrl.replace('{indexerName}', encodeURIComponent(indexerName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 204) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Runs an Azure Search indexer on-demand.
 *
 * @param {string} indexerName The name of the indexer to run.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _run(indexerName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexerName === null || indexerName === undefined || typeof indexerName.valueOf() !== 'string') {
      throw new Error('indexerName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers(\'{indexerName}\')/search.run';
  requestUrl = requestUrl.replace('{indexerName}', encodeURIComponent(indexerName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Creates a new Azure Search indexer or updates an indexer if it already
 * exists.
 *
 * @param {string} indexerName The name of the indexer to create or update.
 *
 * @param {object} indexer The definition of the indexer to create or update.
 *
 * @param {string} indexer.name The name of the indexer.
 *
 * @param {string} [indexer.description] The description of the indexer.
 *
 * @param {string} indexer.dataSourceName The name of the datasource from which
 * this indexer reads data.
 *
 * @param {string} indexer.targetIndexName The name of the index to which this
 * indexer writes data.
 *
 * @param {object} [indexer.schedule] The schedule for this indexer.
 *
 * @param {moment.duration} indexer.schedule.interval The interval of time
 * between indexer executions.
 *
 * @param {date} [indexer.schedule.startTime] The time when an indexer should
 * start running.
 *
 * @param {object} [indexer.parameters] Parameters for indexer execution.
 *
 * @param {number} [indexer.parameters.batchSize] The number of items that are
 * read from the data source and indexed as a single batch in order to improve
 * performance. The default depends on the data source type.
 *
 * @param {number} [indexer.parameters.maxFailedItems] The maximum number of
 * items that can fail indexing for indexer execution to still be considered
 * successful. -1 means no limit. Default is 0.
 *
 * @param {number} [indexer.parameters.maxFailedItemsPerBatch] The maximum
 * number of items in a single batch that can fail indexing for the batch to
 * still be considered successful. -1 means no limit. Default is 0.
 *
 * @param {boolean} [indexer.parameters.base64EncodeKeys] Whether indexer will
 * base64-encode all values that are inserted into key field of the target
 * index. This is needed if keys can contain characters that are invalid in
 * keys (such as dot '.'). Default is false.
 *
 * @param {object} [indexer.parameters.configuration] A dictionary of
 * indexer-specific configuration properties. Each name is the name of a
 * specific property. Each value must be of a primitive type.
 *
 * @param {array} [indexer.fieldMappings] Defines mappings between fields in
 * the data source and corresponding target fields in the index.
 *
 * @param {boolean} [indexer.isDisabled] A value indicating whether the indexer
 * is disabled. Default is false.
 *
 * @param {string} [indexer.eTag] The ETag of the Indexer.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.accessCondition] Additional parameters for the
 * operation
 *
 * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
 * condition. The operation will be performed only if the ETag on the server
 * matches this value.
 *
 * @param {string} [options.accessCondition.ifNoneMatch] Defines the
 * If-None-Match condition. The operation will be performed only if the ETag on
 * the server does not match this value.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Indexer} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _createOrUpdate(indexerName, indexer, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  let accessCondition = (options && options.accessCondition !== undefined) ? options.accessCondition : undefined;
  // Validate
  try {
    if (indexerName === null || indexerName === undefined || typeof indexerName.valueOf() !== 'string') {
      throw new Error('indexerName cannot be null or undefined and it must be of type string.');
    }
    if (indexer === null || indexer === undefined) {
      throw new Error('indexer cannot be null or undefined.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  let ifMatch;
  let ifNoneMatch;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifMatch = accessCondition.ifMatch;
      if (ifMatch !== null && ifMatch !== undefined && typeof ifMatch.valueOf() !== 'string') {
        throw new Error('ifMatch must be of type string.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifNoneMatch = accessCondition.ifNoneMatch;
      if (ifNoneMatch !== null && ifNoneMatch !== undefined && typeof ifNoneMatch.valueOf() !== 'string') {
        throw new Error('ifNoneMatch must be of type string.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers(\'{indexerName}\')';
  requestUrl = requestUrl.replace('{indexerName}', encodeURIComponent(indexerName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PUT';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if (ifMatch !== undefined && ifMatch !== null) {
    httpRequest.headers['If-Match'] = ifMatch;
  }
  if (ifNoneMatch !== undefined && ifNoneMatch !== null) {
    httpRequest.headers['If-None-Match'] = ifNoneMatch;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (indexer !== null && indexer !== undefined) {
      let requestModelMapper = new client.models['Indexer']().mapper();
      requestModel = client.serialize(requestModelMapper, indexer, 'indexer');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(indexer, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 201 && statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Indexer']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Indexer']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError1 = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError1.request = msRest.stripRequest(httpRequest);
        deserializationError1.response = msRest.stripResponse(response);
        return callback(deserializationError1);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deletes an Azure Search indexer.
 *
 * @param {string} indexerName The name of the indexer to delete.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.accessCondition] Additional parameters for the
 * operation
 *
 * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
 * condition. The operation will be performed only if the ETag on the server
 * matches this value.
 *
 * @param {string} [options.accessCondition.ifNoneMatch] Defines the
 * If-None-Match condition. The operation will be performed only if the ETag on
 * the server does not match this value.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deleteMethod(indexerName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  let accessCondition = (options && options.accessCondition !== undefined) ? options.accessCondition : undefined;
  // Validate
  try {
    if (indexerName === null || indexerName === undefined || typeof indexerName.valueOf() !== 'string') {
      throw new Error('indexerName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  let ifMatch;
  let ifNoneMatch;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifMatch = accessCondition.ifMatch;
      if (ifMatch !== null && ifMatch !== undefined && typeof ifMatch.valueOf() !== 'string') {
        throw new Error('ifMatch must be of type string.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifNoneMatch = accessCondition.ifNoneMatch;
      if (ifNoneMatch !== null && ifNoneMatch !== undefined && typeof ifNoneMatch.valueOf() !== 'string') {
        throw new Error('ifNoneMatch must be of type string.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers(\'{indexerName}\')';
  requestUrl = requestUrl.replace('{indexerName}', encodeURIComponent(indexerName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'DELETE';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if (ifMatch !== undefined && ifMatch !== null) {
    httpRequest.headers['If-Match'] = ifMatch;
  }
  if (ifNoneMatch !== undefined && ifNoneMatch !== null) {
    httpRequest.headers['If-None-Match'] = ifNoneMatch;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 404 && statusCode !== 204) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Retrieves an indexer definition from Azure Search.
 *
 * @param {string} indexerName The name of the indexer to retrieve.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Indexer} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _get(indexerName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexerName === null || indexerName === undefined || typeof indexerName.valueOf() !== 'string') {
      throw new Error('indexerName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers(\'{indexerName}\')';
  requestUrl = requestUrl.replace('{indexerName}', encodeURIComponent(indexerName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Indexer']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Lists all indexers available for an Azure Search service.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link IndexerListResult} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _list(options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers';
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['IndexerListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Creates a new Azure Search indexer.
 *
 * @param {object} indexer The definition of the indexer to create.
 *
 * @param {string} indexer.name The name of the indexer.
 *
 * @param {string} [indexer.description] The description of the indexer.
 *
 * @param {string} indexer.dataSourceName The name of the datasource from which
 * this indexer reads data.
 *
 * @param {string} indexer.targetIndexName The name of the index to which this
 * indexer writes data.
 *
 * @param {object} [indexer.schedule] The schedule for this indexer.
 *
 * @param {moment.duration} indexer.schedule.interval The interval of time
 * between indexer executions.
 *
 * @param {date} [indexer.schedule.startTime] The time when an indexer should
 * start running.
 *
 * @param {object} [indexer.parameters] Parameters for indexer execution.
 *
 * @param {number} [indexer.parameters.batchSize] The number of items that are
 * read from the data source and indexed as a single batch in order to improve
 * performance. The default depends on the data source type.
 *
 * @param {number} [indexer.parameters.maxFailedItems] The maximum number of
 * items that can fail indexing for indexer execution to still be considered
 * successful. -1 means no limit. Default is 0.
 *
 * @param {number} [indexer.parameters.maxFailedItemsPerBatch] The maximum
 * number of items in a single batch that can fail indexing for the batch to
 * still be considered successful. -1 means no limit. Default is 0.
 *
 * @param {boolean} [indexer.parameters.base64EncodeKeys] Whether indexer will
 * base64-encode all values that are inserted into key field of the target
 * index. This is needed if keys can contain characters that are invalid in
 * keys (such as dot '.'). Default is false.
 *
 * @param {object} [indexer.parameters.configuration] A dictionary of
 * indexer-specific configuration properties. Each name is the name of a
 * specific property. Each value must be of a primitive type.
 *
 * @param {array} [indexer.fieldMappings] Defines mappings between fields in
 * the data source and corresponding target fields in the index.
 *
 * @param {boolean} [indexer.isDisabled] A value indicating whether the indexer
 * is disabled. Default is false.
 *
 * @param {string} [indexer.eTag] The ETag of the Indexer.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Indexer} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _create(indexer, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexer === null || indexer === undefined) {
      throw new Error('indexer cannot be null or undefined.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers';
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (indexer !== null && indexer !== undefined) {
      let requestModelMapper = new client.models['Indexer']().mapper();
      requestModel = client.serialize(requestModelMapper, indexer, 'indexer');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(indexer, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Indexer']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Returns the current status and execution history of an indexer.
 *
 * @param {string} indexerName The name of the indexer for which to retrieve
 * status.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link IndexerExecutionInfo} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getStatus(indexerName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexerName === null || indexerName === undefined || typeof indexerName.valueOf() !== 'string') {
      throw new Error('indexerName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexers(\'{indexerName}\')/search.status';
  requestUrl = requestUrl.replace('{indexerName}', encodeURIComponent(indexerName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['IndexerExecutionInfo']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a Indexers. */
class Indexers {
  /**
   * Create a Indexers.
   * @param {SearchServiceClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._reset = _reset;
    this._run = _run;
    this._createOrUpdate = _createOrUpdate;
    this._deleteMethod = _deleteMethod;
    this._get = _get;
    this._list = _list;
    this._create = _create;
    this._getStatus = _getStatus;
  }

  /**
   * Resets the change tracking state associated with an Azure Search indexer.
   *
   * @param {string} indexerName The name of the indexer to reset.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  resetWithHttpOperationResponse(indexerName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._reset(indexerName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Resets the change tracking state associated with an Azure Search indexer.
   *
   * @param {string} indexerName The name of the indexer to reset.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  reset(indexerName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._reset(indexerName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._reset(indexerName, options, optionalCallback);
    }
  }

  /**
   * Runs an Azure Search indexer on-demand.
   *
   * @param {string} indexerName The name of the indexer to run.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  runWithHttpOperationResponse(indexerName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._run(indexerName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Runs an Azure Search indexer on-demand.
   *
   * @param {string} indexerName The name of the indexer to run.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  run(indexerName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._run(indexerName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._run(indexerName, options, optionalCallback);
    }
  }

  /**
   * Creates a new Azure Search indexer or updates an indexer if it already
   * exists.
   *
   * @param {string} indexerName The name of the indexer to create or update.
   *
   * @param {object} indexer The definition of the indexer to create or update.
   *
   * @param {string} indexer.name The name of the indexer.
   *
   * @param {string} [indexer.description] The description of the indexer.
   *
   * @param {string} indexer.dataSourceName The name of the datasource from which
   * this indexer reads data.
   *
   * @param {string} indexer.targetIndexName The name of the index to which this
   * indexer writes data.
   *
   * @param {object} [indexer.schedule] The schedule for this indexer.
   *
   * @param {moment.duration} indexer.schedule.interval The interval of time
   * between indexer executions.
   *
   * @param {date} [indexer.schedule.startTime] The time when an indexer should
   * start running.
   *
   * @param {object} [indexer.parameters] Parameters for indexer execution.
   *
   * @param {number} [indexer.parameters.batchSize] The number of items that are
   * read from the data source and indexed as a single batch in order to improve
   * performance. The default depends on the data source type.
   *
   * @param {number} [indexer.parameters.maxFailedItems] The maximum number of
   * items that can fail indexing for indexer execution to still be considered
   * successful. -1 means no limit. Default is 0.
   *
   * @param {number} [indexer.parameters.maxFailedItemsPerBatch] The maximum
   * number of items in a single batch that can fail indexing for the batch to
   * still be considered successful. -1 means no limit. Default is 0.
   *
   * @param {boolean} [indexer.parameters.base64EncodeKeys] Whether indexer will
   * base64-encode all values that are inserted into key field of the target
   * index. This is needed if keys can contain characters that are invalid in
   * keys (such as dot '.'). Default is false.
   *
   * @param {object} [indexer.parameters.configuration] A dictionary of
   * indexer-specific configuration properties. Each name is the name of a
   * specific property. Each value must be of a primitive type.
   *
   * @param {array} [indexer.fieldMappings] Defines mappings between fields in
   * the data source and corresponding target fields in the index.
   *
   * @param {boolean} [indexer.isDisabled] A value indicating whether the indexer
   * is disabled. Default is false.
   *
   * @param {string} [indexer.eTag] The ETag of the Indexer.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Indexer>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createOrUpdateWithHttpOperationResponse(indexerName, indexer, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._createOrUpdate(indexerName, indexer, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates a new Azure Search indexer or updates an indexer if it already
   * exists.
   *
   * @param {string} indexerName The name of the indexer to create or update.
   *
   * @param {object} indexer The definition of the indexer to create or update.
   *
   * @param {string} indexer.name The name of the indexer.
   *
   * @param {string} [indexer.description] The description of the indexer.
   *
   * @param {string} indexer.dataSourceName The name of the datasource from which
   * this indexer reads data.
   *
   * @param {string} indexer.targetIndexName The name of the index to which this
   * indexer writes data.
   *
   * @param {object} [indexer.schedule] The schedule for this indexer.
   *
   * @param {moment.duration} indexer.schedule.interval The interval of time
   * between indexer executions.
   *
   * @param {date} [indexer.schedule.startTime] The time when an indexer should
   * start running.
   *
   * @param {object} [indexer.parameters] Parameters for indexer execution.
   *
   * @param {number} [indexer.parameters.batchSize] The number of items that are
   * read from the data source and indexed as a single batch in order to improve
   * performance. The default depends on the data source type.
   *
   * @param {number} [indexer.parameters.maxFailedItems] The maximum number of
   * items that can fail indexing for indexer execution to still be considered
   * successful. -1 means no limit. Default is 0.
   *
   * @param {number} [indexer.parameters.maxFailedItemsPerBatch] The maximum
   * number of items in a single batch that can fail indexing for the batch to
   * still be considered successful. -1 means no limit. Default is 0.
   *
   * @param {boolean} [indexer.parameters.base64EncodeKeys] Whether indexer will
   * base64-encode all values that are inserted into key field of the target
   * index. This is needed if keys can contain characters that are invalid in
   * keys (such as dot '.'). Default is false.
   *
   * @param {object} [indexer.parameters.configuration] A dictionary of
   * indexer-specific configuration properties. Each name is the name of a
   * specific property. Each value must be of a primitive type.
   *
   * @param {array} [indexer.fieldMappings] Defines mappings between fields in
   * the data source and corresponding target fields in the index.
   *
   * @param {boolean} [indexer.isDisabled] A value indicating whether the indexer
   * is disabled. Default is false.
   *
   * @param {string} [indexer.eTag] The ETag of the Indexer.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Indexer} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Indexer} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  createOrUpdate(indexerName, indexer, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._createOrUpdate(indexerName, indexer, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._createOrUpdate(indexerName, indexer, options, optionalCallback);
    }
  }

  /**
   * Deletes an Azure Search indexer.
   *
   * @param {string} indexerName The name of the indexer to delete.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deleteMethodWithHttpOperationResponse(indexerName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deleteMethod(indexerName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes an Azure Search indexer.
   *
   * @param {string} indexerName The name of the indexer to delete.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deleteMethod(indexerName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deleteMethod(indexerName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deleteMethod(indexerName, options, optionalCallback);
    }
  }

  /**
   * Retrieves an indexer definition from Azure Search.
   *
   * @param {string} indexerName The name of the indexer to retrieve.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Indexer>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getWithHttpOperationResponse(indexerName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._get(indexerName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Retrieves an indexer definition from Azure Search.
   *
   * @param {string} indexerName The name of the indexer to retrieve.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Indexer} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Indexer} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  get(indexerName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._get(indexerName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._get(indexerName, options, optionalCallback);
    }
  }

  /**
   * Lists all indexers available for an Azure Search service.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<IndexerListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listWithHttpOperationResponse(options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._list(options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Lists all indexers available for an Azure Search service.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {IndexerListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link IndexerListResult} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  list(options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._list(options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._list(options, optionalCallback);
    }
  }

  /**
   * Creates a new Azure Search indexer.
   *
   * @param {object} indexer The definition of the indexer to create.
   *
   * @param {string} indexer.name The name of the indexer.
   *
   * @param {string} [indexer.description] The description of the indexer.
   *
   * @param {string} indexer.dataSourceName The name of the datasource from which
   * this indexer reads data.
   *
   * @param {string} indexer.targetIndexName The name of the index to which this
   * indexer writes data.
   *
   * @param {object} [indexer.schedule] The schedule for this indexer.
   *
   * @param {moment.duration} indexer.schedule.interval The interval of time
   * between indexer executions.
   *
   * @param {date} [indexer.schedule.startTime] The time when an indexer should
   * start running.
   *
   * @param {object} [indexer.parameters] Parameters for indexer execution.
   *
   * @param {number} [indexer.parameters.batchSize] The number of items that are
   * read from the data source and indexed as a single batch in order to improve
   * performance. The default depends on the data source type.
   *
   * @param {number} [indexer.parameters.maxFailedItems] The maximum number of
   * items that can fail indexing for indexer execution to still be considered
   * successful. -1 means no limit. Default is 0.
   *
   * @param {number} [indexer.parameters.maxFailedItemsPerBatch] The maximum
   * number of items in a single batch that can fail indexing for the batch to
   * still be considered successful. -1 means no limit. Default is 0.
   *
   * @param {boolean} [indexer.parameters.base64EncodeKeys] Whether indexer will
   * base64-encode all values that are inserted into key field of the target
   * index. This is needed if keys can contain characters that are invalid in
   * keys (such as dot '.'). Default is false.
   *
   * @param {object} [indexer.parameters.configuration] A dictionary of
   * indexer-specific configuration properties. Each name is the name of a
   * specific property. Each value must be of a primitive type.
   *
   * @param {array} [indexer.fieldMappings] Defines mappings between fields in
   * the data source and corresponding target fields in the index.
   *
   * @param {boolean} [indexer.isDisabled] A value indicating whether the indexer
   * is disabled. Default is false.
   *
   * @param {string} [indexer.eTag] The ETag of the Indexer.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Indexer>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createWithHttpOperationResponse(indexer, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._create(indexer, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates a new Azure Search indexer.
   *
   * @param {object} indexer The definition of the indexer to create.
   *
   * @param {string} indexer.name The name of the indexer.
   *
   * @param {string} [indexer.description] The description of the indexer.
   *
   * @param {string} indexer.dataSourceName The name of the datasource from which
   * this indexer reads data.
   *
   * @param {string} indexer.targetIndexName The name of the index to which this
   * indexer writes data.
   *
   * @param {object} [indexer.schedule] The schedule for this indexer.
   *
   * @param {moment.duration} indexer.schedule.interval The interval of time
   * between indexer executions.
   *
   * @param {date} [indexer.schedule.startTime] The time when an indexer should
   * start running.
   *
   * @param {object} [indexer.parameters] Parameters for indexer execution.
   *
   * @param {number} [indexer.parameters.batchSize] The number of items that are
   * read from the data source and indexed as a single batch in order to improve
   * performance. The default depends on the data source type.
   *
   * @param {number} [indexer.parameters.maxFailedItems] The maximum number of
   * items that can fail indexing for indexer execution to still be considered
   * successful. -1 means no limit. Default is 0.
   *
   * @param {number} [indexer.parameters.maxFailedItemsPerBatch] The maximum
   * number of items in a single batch that can fail indexing for the batch to
   * still be considered successful. -1 means no limit. Default is 0.
   *
   * @param {boolean} [indexer.parameters.base64EncodeKeys] Whether indexer will
   * base64-encode all values that are inserted into key field of the target
   * index. This is needed if keys can contain characters that are invalid in
   * keys (such as dot '.'). Default is false.
   *
   * @param {object} [indexer.parameters.configuration] A dictionary of
   * indexer-specific configuration properties. Each name is the name of a
   * specific property. Each value must be of a primitive type.
   *
   * @param {array} [indexer.fieldMappings] Defines mappings between fields in
   * the data source and corresponding target fields in the index.
   *
   * @param {boolean} [indexer.isDisabled] A value indicating whether the indexer
   * is disabled. Default is false.
   *
   * @param {string} [indexer.eTag] The ETag of the Indexer.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Indexer} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Indexer} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  create(indexer, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._create(indexer, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._create(indexer, options, optionalCallback);
    }
  }

  /**
   * Returns the current status and execution history of an indexer.
   *
   * @param {string} indexerName The name of the indexer for which to retrieve
   * status.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<IndexerExecutionInfo>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getStatusWithHttpOperationResponse(indexerName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getStatus(indexerName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Returns the current status and execution history of an indexer.
   *
   * @param {string} indexerName The name of the indexer for which to retrieve
   * status.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {IndexerExecutionInfo} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link IndexerExecutionInfo} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getStatus(indexerName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getStatus(indexerName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getStatus(indexerName, options, optionalCallback);
    }
  }

}

module.exports = Indexers;
