/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;

/**
 * Creates a new Azure Search index.
 *
 * @param {object} index The definition of the index to create.
 *
 * @param {string} index.name The name of the index.
 *
 * @param {array} index.fields The fields of the index.
 *
 * @param {array} [index.scoringProfiles] The scoring profiles for the index.
 *
 * @param {string} [index.defaultScoringProfile] The name of the scoring
 * profile to use if none is specified in the query. If this property is not
 * set and no scoring profile is specified in the query, then default scoring
 * (tf-idf) will be used.
 *
 * @param {object} [index.corsOptions] Options to control Cross-Origin Resource
 * Sharing (CORS) for the index.
 *
 * @param {array} index.corsOptions.allowedOrigins The list of origins from
 * which JavaScript code will be granted access to your index. Can contain a
 * list of hosts of the form
 * {protocol}://{fully-qualified-domain-name}[:{port#}], or a single '*' to
 * allow all origins (not recommended).
 *
 * @param {number} [index.corsOptions.maxAgeInSeconds] The duration for which
 * browsers should cache CORS preflight responses. Defaults to 5 mintues.
 *
 * @param {array} [index.suggesters] The suggesters for the index.
 *
 * @param {array} [index.analyzers] The analyzers for the index.
 *
 * @param {array} [index.tokenizers] The tokenizers for the index.
 *
 * @param {array} [index.tokenFilters] The token filters for the index.
 *
 * @param {array} [index.charFilters] The character filters for the index.
 *
 * @param {string} [index.eTag] The ETag of the index.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Index} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _create(index, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (index === null || index === undefined) {
      throw new Error('index cannot be null or undefined.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes';
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (index !== null && index !== undefined) {
      let requestModelMapper = new client.models['Index']().mapper();
      requestModel = client.serialize(requestModelMapper, index, 'index');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(index, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Index']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Lists all indexes available for an Azure Search service.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {string} [options.select] Selects which properties of the index
 * definitions to retrieve. Specified as a comma-separated list of JSON
 * property names, or '*' for all properties. The default is all properties.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link IndexListResult} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _list(options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let select = (options && options.select !== undefined) ? options.select : undefined;
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (select !== null && select !== undefined && typeof select.valueOf() !== 'string') {
      throw new Error('select must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes';
  let queryParameters = [];
  if (select !== null && select !== undefined) {
    queryParameters.push('$select=' + encodeURIComponent(select));
  }
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['IndexListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Creates a new Azure Search index or updates an index if it already exists.
 *
 * @param {string} indexName The definition of the index to create or update.
 *
 * @param {object} index The definition of the index to create or update.
 *
 * @param {string} index.name The name of the index.
 *
 * @param {array} index.fields The fields of the index.
 *
 * @param {array} [index.scoringProfiles] The scoring profiles for the index.
 *
 * @param {string} [index.defaultScoringProfile] The name of the scoring
 * profile to use if none is specified in the query. If this property is not
 * set and no scoring profile is specified in the query, then default scoring
 * (tf-idf) will be used.
 *
 * @param {object} [index.corsOptions] Options to control Cross-Origin Resource
 * Sharing (CORS) for the index.
 *
 * @param {array} index.corsOptions.allowedOrigins The list of origins from
 * which JavaScript code will be granted access to your index. Can contain a
 * list of hosts of the form
 * {protocol}://{fully-qualified-domain-name}[:{port#}], or a single '*' to
 * allow all origins (not recommended).
 *
 * @param {number} [index.corsOptions.maxAgeInSeconds] The duration for which
 * browsers should cache CORS preflight responses. Defaults to 5 mintues.
 *
 * @param {array} [index.suggesters] The suggesters for the index.
 *
 * @param {array} [index.analyzers] The analyzers for the index.
 *
 * @param {array} [index.tokenizers] The tokenizers for the index.
 *
 * @param {array} [index.tokenFilters] The token filters for the index.
 *
 * @param {array} [index.charFilters] The character filters for the index.
 *
 * @param {string} [index.eTag] The ETag of the index.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {boolean} [options.allowIndexDowntime] Allows new analyzers,
 * tokenizers, token filters, or char filters to be added to an index by taking
 * the index offline for at least a few seconds. This temporarily causes
 * indexing and query requests to fail. Performance and write availability of
 * the index can be impaired for several minutes after the index is updated, or
 * longer for very large indexes.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.accessCondition] Additional parameters for the
 * operation
 *
 * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
 * condition. The operation will be performed only if the ETag on the server
 * matches this value.
 *
 * @param {string} [options.accessCondition.ifNoneMatch] Defines the
 * If-None-Match condition. The operation will be performed only if the ETag on
 * the server does not match this value.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Index} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _createOrUpdate(indexName, index, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let allowIndexDowntime = (options && options.allowIndexDowntime !== undefined) ? options.allowIndexDowntime : undefined;
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  let accessCondition = (options && options.accessCondition !== undefined) ? options.accessCondition : undefined;
  // Validate
  try {
    if (indexName === null || indexName === undefined || typeof indexName.valueOf() !== 'string') {
      throw new Error('indexName cannot be null or undefined and it must be of type string.');
    }
    if (index === null || index === undefined) {
      throw new Error('index cannot be null or undefined.');
    }
    if (allowIndexDowntime !== null && allowIndexDowntime !== undefined && typeof allowIndexDowntime !== 'boolean') {
      throw new Error('allowIndexDowntime must be of type boolean.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  let ifMatch;
  let ifNoneMatch;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifMatch = accessCondition.ifMatch;
      if (ifMatch !== null && ifMatch !== undefined && typeof ifMatch.valueOf() !== 'string') {
        throw new Error('ifMatch must be of type string.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifNoneMatch = accessCondition.ifNoneMatch;
      if (ifNoneMatch !== null && ifNoneMatch !== undefined && typeof ifNoneMatch.valueOf() !== 'string') {
        throw new Error('ifNoneMatch must be of type string.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes(\'{indexName}\')';
  requestUrl = requestUrl.replace('{indexName}', encodeURIComponent(indexName));
  let queryParameters = [];
  if (allowIndexDowntime !== null && allowIndexDowntime !== undefined) {
    queryParameters.push('allowIndexDowntime=' + encodeURIComponent(allowIndexDowntime.toString()));
  }
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PUT';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if (ifMatch !== undefined && ifMatch !== null) {
    httpRequest.headers['If-Match'] = ifMatch;
  }
  if (ifNoneMatch !== undefined && ifNoneMatch !== null) {
    httpRequest.headers['If-None-Match'] = ifNoneMatch;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (index !== null && index !== undefined) {
      let requestModelMapper = new client.models['Index']().mapper();
      requestModel = client.serialize(requestModelMapper, index, 'index');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(index, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Index']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Index']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError1 = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError1.request = msRest.stripRequest(httpRequest);
        deserializationError1.response = msRest.stripResponse(response);
        return callback(deserializationError1);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deletes an Azure Search index and all the documents it contains.
 *
 * @param {string} indexName The name of the index to delete.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.accessCondition] Additional parameters for the
 * operation
 *
 * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
 * condition. The operation will be performed only if the ETag on the server
 * matches this value.
 *
 * @param {string} [options.accessCondition.ifNoneMatch] Defines the
 * If-None-Match condition. The operation will be performed only if the ETag on
 * the server does not match this value.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deleteMethod(indexName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  let accessCondition = (options && options.accessCondition !== undefined) ? options.accessCondition : undefined;
  // Validate
  try {
    if (indexName === null || indexName === undefined || typeof indexName.valueOf() !== 'string') {
      throw new Error('indexName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  let ifMatch;
  let ifNoneMatch;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifMatch = accessCondition.ifMatch;
      if (ifMatch !== null && ifMatch !== undefined && typeof ifMatch.valueOf() !== 'string') {
        throw new Error('ifMatch must be of type string.');
      }
    }
    if (accessCondition !== null && accessCondition !== undefined)
    {
      ifNoneMatch = accessCondition.ifNoneMatch;
      if (ifNoneMatch !== null && ifNoneMatch !== undefined && typeof ifNoneMatch.valueOf() !== 'string') {
        throw new Error('ifNoneMatch must be of type string.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes(\'{indexName}\')';
  requestUrl = requestUrl.replace('{indexName}', encodeURIComponent(indexName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'DELETE';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if (ifMatch !== undefined && ifMatch !== null) {
    httpRequest.headers['If-Match'] = ifMatch;
  }
  if (ifNoneMatch !== undefined && ifNoneMatch !== null) {
    httpRequest.headers['If-None-Match'] = ifNoneMatch;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 204 && statusCode !== 404) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Retrieves an index definition from Azure Search.
 *
 * @param {string} indexName The name of the index to retrieve.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link Index} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _get(indexName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexName === null || indexName === undefined || typeof indexName.valueOf() !== 'string') {
      throw new Error('indexName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes(\'{indexName}\')';
  requestUrl = requestUrl.replace('{indexName}', encodeURIComponent(indexName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['Index']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Returns statistics for the given index, including a document count and
 * storage usage.
 *
 * @param {string} indexName The name of the index for which to retrieve
 * statistics.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link IndexGetStatisticsResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getStatistics(indexName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexName === null || indexName === undefined || typeof indexName.valueOf() !== 'string') {
      throw new Error('indexName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes(\'{indexName}\')/search.stats';
  requestUrl = requestUrl.replace('{indexName}', encodeURIComponent(indexName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['IndexGetStatisticsResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Shows how an analyzer breaks text into tokens.
 *
 * @param {string} indexName The name of the index for which to test an
 * analyzer.
 *
 * @param {object} request The text and analyzer or analysis components to
 * test.
 *
 * @param {string} request.text The text to break into tokens.
 *
 * @param {object} [request.analyzer] The name of the analyzer to use to break
 * the given text. If this parameter is not specified, you must specify a
 * tokenizer instead. The tokenizer and analyzer parameters are mutually
 * exclusive.
 *
 * @param {string} [request.analyzer.name]
 *
 * @param {object} [request.tokenizer] The name of the tokenizer to use to
 * break the given text. If this parameter is not specified, you must specify
 * an analyzer instead. The tokenizer and analyzer parameters are mutually
 * exclusive.
 *
 * @param {string} [request.tokenizer.name]
 *
 * @param {array} [request.tokenFilters] An optional list of token filters to
 * use when breaking the given text. This parameter can only be set when using
 * the tokenizer parameter.
 *
 * @param {array} [request.charFilters] An optional list of character filters
 * to use when breaking the given text. This parameter can only be set when
 * using the tokenizer parameter.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchRequestOptions] Additional parameters for the
 * operation
 *
 * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
 * sent with the request to help with debugging.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link AnalyzeResult} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _analyze(indexName, request, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchRequestOptions = (options && options.searchRequestOptions !== undefined) ? options.searchRequestOptions : undefined;
  // Validate
  try {
    if (indexName === null || indexName === undefined || typeof indexName.valueOf() !== 'string') {
      throw new Error('indexName cannot be null or undefined and it must be of type string.');
    }
    if (request === null || request === undefined) {
      throw new Error('request cannot be null or undefined.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchRequestOptions !== null && searchRequestOptions !== undefined)
    {
      clientRequestId = searchRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'indexes(\'{indexName}\')/search.analyze';
  requestUrl = requestUrl.replace('{indexName}', encodeURIComponent(indexName));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (request !== null && request !== undefined) {
      let requestModelMapper = new client.models['AnalyzeRequest']().mapper();
      requestModel = client.serialize(requestModelMapper, request, 'request');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(request, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['AnalyzeResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a Indexes. */
class Indexes {
  /**
   * Create a Indexes.
   * @param {SearchServiceClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._create = _create;
    this._list = _list;
    this._createOrUpdate = _createOrUpdate;
    this._deleteMethod = _deleteMethod;
    this._get = _get;
    this._getStatistics = _getStatistics;
    this._analyze = _analyze;
  }

  /**
   * Creates a new Azure Search index.
   *
   * @param {object} index The definition of the index to create.
   *
   * @param {string} index.name The name of the index.
   *
   * @param {array} index.fields The fields of the index.
   *
   * @param {array} [index.scoringProfiles] The scoring profiles for the index.
   *
   * @param {string} [index.defaultScoringProfile] The name of the scoring
   * profile to use if none is specified in the query. If this property is not
   * set and no scoring profile is specified in the query, then default scoring
   * (tf-idf) will be used.
   *
   * @param {object} [index.corsOptions] Options to control Cross-Origin Resource
   * Sharing (CORS) for the index.
   *
   * @param {array} index.corsOptions.allowedOrigins The list of origins from
   * which JavaScript code will be granted access to your index. Can contain a
   * list of hosts of the form
   * {protocol}://{fully-qualified-domain-name}[:{port#}], or a single '*' to
   * allow all origins (not recommended).
   *
   * @param {number} [index.corsOptions.maxAgeInSeconds] The duration for which
   * browsers should cache CORS preflight responses. Defaults to 5 mintues.
   *
   * @param {array} [index.suggesters] The suggesters for the index.
   *
   * @param {array} [index.analyzers] The analyzers for the index.
   *
   * @param {array} [index.tokenizers] The tokenizers for the index.
   *
   * @param {array} [index.tokenFilters] The token filters for the index.
   *
   * @param {array} [index.charFilters] The character filters for the index.
   *
   * @param {string} [index.eTag] The ETag of the index.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Index>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createWithHttpOperationResponse(index, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._create(index, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates a new Azure Search index.
   *
   * @param {object} index The definition of the index to create.
   *
   * @param {string} index.name The name of the index.
   *
   * @param {array} index.fields The fields of the index.
   *
   * @param {array} [index.scoringProfiles] The scoring profiles for the index.
   *
   * @param {string} [index.defaultScoringProfile] The name of the scoring
   * profile to use if none is specified in the query. If this property is not
   * set and no scoring profile is specified in the query, then default scoring
   * (tf-idf) will be used.
   *
   * @param {object} [index.corsOptions] Options to control Cross-Origin Resource
   * Sharing (CORS) for the index.
   *
   * @param {array} index.corsOptions.allowedOrigins The list of origins from
   * which JavaScript code will be granted access to your index. Can contain a
   * list of hosts of the form
   * {protocol}://{fully-qualified-domain-name}[:{port#}], or a single '*' to
   * allow all origins (not recommended).
   *
   * @param {number} [index.corsOptions.maxAgeInSeconds] The duration for which
   * browsers should cache CORS preflight responses. Defaults to 5 mintues.
   *
   * @param {array} [index.suggesters] The suggesters for the index.
   *
   * @param {array} [index.analyzers] The analyzers for the index.
   *
   * @param {array} [index.tokenizers] The tokenizers for the index.
   *
   * @param {array} [index.tokenFilters] The token filters for the index.
   *
   * @param {array} [index.charFilters] The character filters for the index.
   *
   * @param {string} [index.eTag] The ETag of the index.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Index} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Index} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  create(index, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._create(index, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._create(index, options, optionalCallback);
    }
  }

  /**
   * Lists all indexes available for an Azure Search service.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.select] Selects which properties of the index
   * definitions to retrieve. Specified as a comma-separated list of JSON
   * property names, or '*' for all properties. The default is all properties.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<IndexListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listWithHttpOperationResponse(options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._list(options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Lists all indexes available for an Azure Search service.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {string} [options.select] Selects which properties of the index
   * definitions to retrieve. Specified as a comma-separated list of JSON
   * property names, or '*' for all properties. The default is all properties.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {IndexListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link IndexListResult} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  list(options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._list(options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._list(options, optionalCallback);
    }
  }

  /**
   * Creates a new Azure Search index or updates an index if it already exists.
   *
   * @param {string} indexName The definition of the index to create or update.
   *
   * @param {object} index The definition of the index to create or update.
   *
   * @param {string} index.name The name of the index.
   *
   * @param {array} index.fields The fields of the index.
   *
   * @param {array} [index.scoringProfiles] The scoring profiles for the index.
   *
   * @param {string} [index.defaultScoringProfile] The name of the scoring
   * profile to use if none is specified in the query. If this property is not
   * set and no scoring profile is specified in the query, then default scoring
   * (tf-idf) will be used.
   *
   * @param {object} [index.corsOptions] Options to control Cross-Origin Resource
   * Sharing (CORS) for the index.
   *
   * @param {array} index.corsOptions.allowedOrigins The list of origins from
   * which JavaScript code will be granted access to your index. Can contain a
   * list of hosts of the form
   * {protocol}://{fully-qualified-domain-name}[:{port#}], or a single '*' to
   * allow all origins (not recommended).
   *
   * @param {number} [index.corsOptions.maxAgeInSeconds] The duration for which
   * browsers should cache CORS preflight responses. Defaults to 5 mintues.
   *
   * @param {array} [index.suggesters] The suggesters for the index.
   *
   * @param {array} [index.analyzers] The analyzers for the index.
   *
   * @param {array} [index.tokenizers] The tokenizers for the index.
   *
   * @param {array} [index.tokenFilters] The token filters for the index.
   *
   * @param {array} [index.charFilters] The character filters for the index.
   *
   * @param {string} [index.eTag] The ETag of the index.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {boolean} [options.allowIndexDowntime] Allows new analyzers,
   * tokenizers, token filters, or char filters to be added to an index by taking
   * the index offline for at least a few seconds. This temporarily causes
   * indexing and query requests to fail. Performance and write availability of
   * the index can be impaired for several minutes after the index is updated, or
   * longer for very large indexes.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Index>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createOrUpdateWithHttpOperationResponse(indexName, index, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._createOrUpdate(indexName, index, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates a new Azure Search index or updates an index if it already exists.
   *
   * @param {string} indexName The definition of the index to create or update.
   *
   * @param {object} index The definition of the index to create or update.
   *
   * @param {string} index.name The name of the index.
   *
   * @param {array} index.fields The fields of the index.
   *
   * @param {array} [index.scoringProfiles] The scoring profiles for the index.
   *
   * @param {string} [index.defaultScoringProfile] The name of the scoring
   * profile to use if none is specified in the query. If this property is not
   * set and no scoring profile is specified in the query, then default scoring
   * (tf-idf) will be used.
   *
   * @param {object} [index.corsOptions] Options to control Cross-Origin Resource
   * Sharing (CORS) for the index.
   *
   * @param {array} index.corsOptions.allowedOrigins The list of origins from
   * which JavaScript code will be granted access to your index. Can contain a
   * list of hosts of the form
   * {protocol}://{fully-qualified-domain-name}[:{port#}], or a single '*' to
   * allow all origins (not recommended).
   *
   * @param {number} [index.corsOptions.maxAgeInSeconds] The duration for which
   * browsers should cache CORS preflight responses. Defaults to 5 mintues.
   *
   * @param {array} [index.suggesters] The suggesters for the index.
   *
   * @param {array} [index.analyzers] The analyzers for the index.
   *
   * @param {array} [index.tokenizers] The tokenizers for the index.
   *
   * @param {array} [index.tokenFilters] The token filters for the index.
   *
   * @param {array} [index.charFilters] The character filters for the index.
   *
   * @param {string} [index.eTag] The ETag of the index.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {boolean} [options.allowIndexDowntime] Allows new analyzers,
   * tokenizers, token filters, or char filters to be added to an index by taking
   * the index offline for at least a few seconds. This temporarily causes
   * indexing and query requests to fail. Performance and write availability of
   * the index can be impaired for several minutes after the index is updated, or
   * longer for very large indexes.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Index} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Index} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  createOrUpdate(indexName, index, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._createOrUpdate(indexName, index, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._createOrUpdate(indexName, index, options, optionalCallback);
    }
  }

  /**
   * Deletes an Azure Search index and all the documents it contains.
   *
   * @param {string} indexName The name of the index to delete.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deleteMethodWithHttpOperationResponse(indexName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deleteMethod(indexName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes an Azure Search index and all the documents it contains.
   *
   * @param {string} indexName The name of the index to delete.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.accessCondition] Additional parameters for the
   * operation
   *
   * @param {string} [options.accessCondition.ifMatch] Defines the If-Match
   * condition. The operation will be performed only if the ETag on the server
   * matches this value.
   *
   * @param {string} [options.accessCondition.ifNoneMatch] Defines the
   * If-None-Match condition. The operation will be performed only if the ETag on
   * the server does not match this value.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deleteMethod(indexName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deleteMethod(indexName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deleteMethod(indexName, options, optionalCallback);
    }
  }

  /**
   * Retrieves an index definition from Azure Search.
   *
   * @param {string} indexName The name of the index to retrieve.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<Index>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getWithHttpOperationResponse(indexName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._get(indexName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Retrieves an index definition from Azure Search.
   *
   * @param {string} indexName The name of the index to retrieve.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {Index} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Index} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  get(indexName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._get(indexName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._get(indexName, options, optionalCallback);
    }
  }

  /**
   * Returns statistics for the given index, including a document count and
   * storage usage.
   *
   * @param {string} indexName The name of the index for which to retrieve
   * statistics.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<IndexGetStatisticsResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getStatisticsWithHttpOperationResponse(indexName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getStatistics(indexName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Returns statistics for the given index, including a document count and
   * storage usage.
   *
   * @param {string} indexName The name of the index for which to retrieve
   * statistics.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {IndexGetStatisticsResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link IndexGetStatisticsResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getStatistics(indexName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getStatistics(indexName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getStatistics(indexName, options, optionalCallback);
    }
  }

  /**
   * Shows how an analyzer breaks text into tokens.
   *
   * @param {string} indexName The name of the index for which to test an
   * analyzer.
   *
   * @param {object} request The text and analyzer or analysis components to
   * test.
   *
   * @param {string} request.text The text to break into tokens.
   *
   * @param {object} [request.analyzer] The name of the analyzer to use to break
   * the given text. If this parameter is not specified, you must specify a
   * tokenizer instead. The tokenizer and analyzer parameters are mutually
   * exclusive.
   *
   * @param {string} [request.analyzer.name]
   *
   * @param {object} [request.tokenizer] The name of the tokenizer to use to
   * break the given text. If this parameter is not specified, you must specify
   * an analyzer instead. The tokenizer and analyzer parameters are mutually
   * exclusive.
   *
   * @param {string} [request.tokenizer.name]
   *
   * @param {array} [request.tokenFilters] An optional list of token filters to
   * use when breaking the given text. This parameter can only be set when using
   * the tokenizer parameter.
   *
   * @param {array} [request.charFilters] An optional list of character filters
   * to use when breaking the given text. This parameter can only be set when
   * using the tokenizer parameter.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<AnalyzeResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  analyzeWithHttpOperationResponse(indexName, request, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._analyze(indexName, request, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Shows how an analyzer breaks text into tokens.
   *
   * @param {string} indexName The name of the index for which to test an
   * analyzer.
   *
   * @param {object} request The text and analyzer or analysis components to
   * test.
   *
   * @param {string} request.text The text to break into tokens.
   *
   * @param {object} [request.analyzer] The name of the analyzer to use to break
   * the given text. If this parameter is not specified, you must specify a
   * tokenizer instead. The tokenizer and analyzer parameters are mutually
   * exclusive.
   *
   * @param {string} [request.analyzer.name]
   *
   * @param {object} [request.tokenizer] The name of the tokenizer to use to
   * break the given text. If this parameter is not specified, you must specify
   * an analyzer instead. The tokenizer and analyzer parameters are mutually
   * exclusive.
   *
   * @param {string} [request.tokenizer.name]
   *
   * @param {array} [request.tokenFilters] An optional list of token filters to
   * use when breaking the given text. This parameter can only be set when using
   * the tokenizer parameter.
   *
   * @param {array} [request.charFilters] An optional list of character filters
   * to use when breaking the given text. This parameter can only be set when
   * using the tokenizer parameter.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchRequestOptions] Additional parameters for the
   * operation
   *
   * @param {uuid} [options.searchRequestOptions.clientRequestId] The tracking ID
   * sent with the request to help with debugging.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {AnalyzeResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link AnalyzeResult} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  analyze(indexName, request, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._analyze(indexName, request, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._analyze(indexName, request, options, optionalCallback);
    }
  }

}

module.exports = Indexes;
