/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';
var moment = require('moment');

/**
 * Describes the parameters for a standalone cluster configuration upgrade.
 *
 */
class ClusterConfigurationUpgradeDescription {
  /**
   * Create a ClusterConfigurationUpgradeDescription.
   * @member {string} clusterConfig The cluster configuration.
   * @member {moment.duration} [healthCheckRetryTimeout] The length of time
   * between attempts to perform a health checks if the application or cluster
   * is not healthy. Default value: moment.duration('PT0H0M0S') .
   * @member {moment.duration} [healthCheckWaitDurationInSeconds] The length of
   * time to wait after completing an upgrade domain before starting the health
   * checks process. Default value: moment.duration('PT0H0M0S') .
   * @member {moment.duration} [healthCheckStableDurationInSeconds] The length
   * of time that the application or cluster must remain healthy. Default
   * value: moment.duration('PT0H0M0S') .
   * @member {moment.duration} [upgradeDomainTimeoutInSeconds] The timeout for
   * the upgrade domain. Default value: moment.duration('PT0H0M0S') .
   * @member {moment.duration} [upgradeTimeoutInSeconds] The upgrade timeout.
   * Default value: moment.duration('PT0H0M0S') .
   * @member {number} [maxPercentUnhealthyApplications] The maximum allowed
   * percentage of unhealthy applications during the upgrade. Allowed values
   * are integer values from zero to 100. Default value: 0 .
   * @member {number} [maxPercentUnhealthyNodes] The maximum allowed percentage
   * of unhealthy nodes during the upgrade. Allowed values are integer values
   * from zero to 100. Default value: 0 .
   * @member {number} [maxPercentDeltaUnhealthyNodes] The maximum allowed
   * percentage of delta health degradation during the upgrade. Allowed values
   * are integer values from zero to 100. Default value: 0 .
   * @member {number} [maxPercentUpgradeDomainDeltaUnhealthyNodes] The maximum
   * allowed percentage of upgrade domain delta health degradation during the
   * upgrade. Allowed values are integer values from zero to 100. Default
   * value: 0 .
   */
  constructor() {
  }

  /**
   * Defines the metadata of ClusterConfigurationUpgradeDescription
   *
   * @returns {object} metadata of ClusterConfigurationUpgradeDescription
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ClusterConfigurationUpgradeDescription',
      type: {
        name: 'Composite',
        className: 'ClusterConfigurationUpgradeDescription',
        modelProperties: {
          clusterConfig: {
            required: true,
            serializedName: 'ClusterConfig',
            type: {
              name: 'String'
            }
          },
          healthCheckRetryTimeout: {
            required: false,
            serializedName: 'HealthCheckRetryTimeout',
            defaultValue: moment.duration('PT0H0M0S'),
            type: {
              name: 'TimeSpan'
            }
          },
          healthCheckWaitDurationInSeconds: {
            required: false,
            serializedName: 'HealthCheckWaitDurationInSeconds',
            defaultValue: moment.duration('PT0H0M0S'),
            type: {
              name: 'TimeSpan'
            }
          },
          healthCheckStableDurationInSeconds: {
            required: false,
            serializedName: 'HealthCheckStableDurationInSeconds',
            defaultValue: moment.duration('PT0H0M0S'),
            type: {
              name: 'TimeSpan'
            }
          },
          upgradeDomainTimeoutInSeconds: {
            required: false,
            serializedName: 'UpgradeDomainTimeoutInSeconds',
            defaultValue: moment.duration('PT0H0M0S'),
            type: {
              name: 'TimeSpan'
            }
          },
          upgradeTimeoutInSeconds: {
            required: false,
            serializedName: 'UpgradeTimeoutInSeconds',
            defaultValue: moment.duration('PT0H0M0S'),
            type: {
              name: 'TimeSpan'
            }
          },
          maxPercentUnhealthyApplications: {
            required: false,
            serializedName: 'MaxPercentUnhealthyApplications',
            defaultValue: 0,
            type: {
              name: 'Number'
            }
          },
          maxPercentUnhealthyNodes: {
            required: false,
            serializedName: 'MaxPercentUnhealthyNodes',
            defaultValue: 0,
            type: {
              name: 'Number'
            }
          },
          maxPercentDeltaUnhealthyNodes: {
            required: false,
            serializedName: 'MaxPercentDeltaUnhealthyNodes',
            defaultValue: 0,
            type: {
              name: 'Number'
            }
          },
          maxPercentUpgradeDomainDeltaUnhealthyNodes: {
            required: false,
            serializedName: 'MaxPercentUpgradeDomainDeltaUnhealthyNodes',
            defaultValue: 0,
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = ClusterConfigurationUpgradeDescription;
