/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for deployed applications, containing health
 * evaluations for each unhealthy deployed application that impacted current
 * aggregated health state.
 * Can be returned when evaluating application health and the aggregated health
 * state is either Error or Warning.
 *
 *
 * @extends models['HealthEvaluation']
 */
class DeployedApplicationsHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a DeployedApplicationsHealthEvaluation.
   * @member {number} [maxPercentUnhealthyDeployedApplications] Maximum allowed
   * percentage of unhealthy deployed applications from the
   * ApplicationHealthPolicy.
   * @member {number} [totalCount] Total number of deployed applications of the
   * application in the health store.
   * @member {array} [unhealthyEvaluations]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DeployedApplicationsHealthEvaluation
   *
   * @returns {object} metadata of DeployedApplicationsHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'DeployedApplications',
      type: {
        name: 'Composite',
        className: 'DeployedApplicationsHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            type: {
              name: 'String'
            }
          },
          maxPercentUnhealthyDeployedApplications: {
            required: false,
            serializedName: 'MaxPercentUnhealthyDeployedApplications',
            type: {
              name: 'Number'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = DeployedApplicationsHealthEvaluation;
