/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for deployed service packages, containing
 * health evaluations for each unhealthy deployed service package that impacted
 * current aggregated health state. Can be returned when evaluating deployed
 * application health and the aggregated health state is either Error or
 * Warning.
 *
 * @extends models['HealthEvaluation']
 */
class DeployedServicePackagesHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a DeployedServicePackagesHealthEvaluation.
   * @member {number} [totalCount] Total number of deployed service packages of
   * the deployed application in the health store.
   * @member {array} [unhealthyEvaluations]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DeployedServicePackagesHealthEvaluation
   *
   * @returns {object} metadata of DeployedServicePackagesHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'DeployedServicePackages',
      type: {
        name: 'Composite',
        className: 'DeployedServicePackagesHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            type: {
              name: 'String'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = DeployedServicePackagesHealthEvaluation;
