/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for replicas, containing health evaluations for
 * each unhealthy replica that impacted current aggregated health state. Can be
 * returned when evaluating partition health and the aggregated health state is
 * either Error or Warning.
 *
 * @extends models['HealthEvaluation']
 */
class ReplicasHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a ReplicasHealthEvaluation.
   * @member {number} [maxPercentUnhealthyReplicasPerPartition] Maximum allowed
   * percentage of unhealthy replicas per partition from the
   * ApplicationHealthPolicy.
   * @member {number} [totalCount] Total number of replicas in the partition
   * from the health store.
   * @member {array} [unhealthyEvaluations]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ReplicasHealthEvaluation
   *
   * @returns {object} metadata of ReplicasHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Replicas',
      type: {
        name: 'Composite',
        className: 'ReplicasHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            type: {
              name: 'String'
            }
          },
          maxPercentUnhealthyReplicasPerPartition: {
            required: false,
            serializedName: 'MaxPercentUnhealthyReplicasPerPartition',
            type: {
              name: 'Number'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ReplicasHealthEvaluation;
