/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes the policy to be used for placement of a Service Fabric service
 * where the service's Primary replicas should optimally be placed in a
 * particular domain.
 *
 * This placement policy is usually used with fault domains in scenarios where
 * the Service Fabric cluster is geographically distributed in order to
 * indicate that a service's primary replica should be located in a particular
 * fault domain, which in geo-distributed scenarios usually aligns with
 * regional or datacenter boundaries. Note that since this is an optimization
 * it is possible that the Primary replica may not end up located in this
 * domain due to failures, capacity limits, or other constraints.
 *
 *
 * @extends models['ServicePlacementPolicyDescription']
 */
class ServicePlacementPreferPrimaryDomainPolicyDescription extends models['ServicePlacementPolicyDescription'] {
  /**
   * Create a ServicePlacementPreferPrimaryDomainPolicyDescription.
   * @member {string} [domainName] The name of the domain that should used for
   * placement as per this policy.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ServicePlacementPreferPrimaryDomainPolicyDescription
   *
   * @returns {object} metadata of ServicePlacementPreferPrimaryDomainPolicyDescription
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'PreferPrimaryDomain',
      type: {
        name: 'Composite',
        className: 'ServicePlacementPreferPrimaryDomainPolicyDescription',
        modelProperties: {
          type: {
            required: true,
            serializedName: 'Type',
            type: {
              name: 'String'
            }
          },
          domainName: {
            required: false,
            serializedName: 'DomainName',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ServicePlacementPreferPrimaryDomainPolicyDescription;
