/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for services of a certain service type
 * belonging to an application, containing health evaluations for each
 * unhealthy service that impacted current aggregated health state. Can be
 * returned when evaluating application health and the aggregated health state
 * is either Error or Warning.
 *
 * @extends models['HealthEvaluation']
 */
class ServicesHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a ServicesHealthEvaluation.
   * @member {string} [serviceTypeName] Name of the service type of the
   * services.
   * @member {number} [maxPercentUnhealthyServices] Maximum allowed percentage
   * of unhealthy services from the ServiceTypeHealthPolicy.
   * @member {number} [totalCount] Total number of services of the current
   * service type in the application from the health store.
   * @member {array} [unhealthyEvaluations]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ServicesHealthEvaluation
   *
   * @returns {object} metadata of ServicesHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Services',
      type: {
        name: 'Composite',
        className: 'ServicesHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            type: {
              name: 'String'
            }
          },
          serviceTypeName: {
            required: false,
            serializedName: 'ServiceTypeName',
            type: {
              name: 'String'
            }
          },
          maxPercentUnhealthyServices: {
            required: false,
            serializedName: 'MaxPercentUnhealthyServices',
            type: {
              name: 'Number'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ServicesHealthEvaluation;
