/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Describes the delta health policies for the cluster upgrade.
 *
 */
class ClusterUpgradeDeltaHealthPolicy {
  /**
   * Create a ClusterUpgradeDeltaHealthPolicy.
   * @member {number} maxPercentDeltaUnhealthyNodes The maximum allowed
   * percentage of nodes health degradation allowed during cluster upgrades.
   * The delta is measured between the state of the nodes at the beginning of
   * upgrade and the state of the nodes at the time of the health evaluation.
   * The check is performed after every upgrade domain upgrade completion to
   * make sure the global state of the cluster is within tolerated limits.
   * @member {number} maxPercentUpgradeDomainDeltaUnhealthyNodes The maximum
   * allowed percentage of upgrade domain nodes health degradation allowed
   * during cluster upgrades. The delta is measured between the state of the
   * upgrade domain nodes at the beginning of upgrade and the state of the
   * upgrade domain nodes at the time of the health evaluation. The check is
   * performed after every upgrade domain upgrade completion for all completed
   * upgrade domains to make sure the state of the upgrade domains is within
   * tolerated limits.
   * @member {number} maxPercentDeltaUnhealthyApplications The maximum allowed
   * percentage of applications health degradation allowed during cluster
   * upgrades. The delta is measured between the state of the applications at
   * the beginning of upgrade and the state of the applications at the time of
   * the health evaluation. The check is performed after every upgrade domain
   * upgrade completion to make sure the global state of the cluster is within
   * tolerated limits. System services are not included in this.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ClusterUpgradeDeltaHealthPolicy
   *
   * @returns {object} metadata of ClusterUpgradeDeltaHealthPolicy
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ClusterUpgradeDeltaHealthPolicy',
      type: {
        name: 'Composite',
        className: 'ClusterUpgradeDeltaHealthPolicy',
        modelProperties: {
          maxPercentDeltaUnhealthyNodes: {
            required: true,
            serializedName: 'maxPercentDeltaUnhealthyNodes',
            constraints: {
              InclusiveMaximum: 100,
              InclusiveMinimum: 0
            },
            type: {
              name: 'Number'
            }
          },
          maxPercentUpgradeDomainDeltaUnhealthyNodes: {
            required: true,
            serializedName: 'maxPercentUpgradeDomainDeltaUnhealthyNodes',
            constraints: {
              InclusiveMaximum: 100,
              InclusiveMinimum: 0
            },
            type: {
              name: 'Number'
            }
          },
          maxPercentDeltaUnhealthyApplications: {
            required: true,
            serializedName: 'maxPercentDeltaUnhealthyApplications',
            constraints: {
              InclusiveMaximum: 100,
              InclusiveMinimum: 0
            },
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = ClusterUpgradeDeltaHealthPolicy;
