/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A network connection.
 *
 * @extends models['Relationship']
 */
class Connection extends models['Relationship'] {
  /**
   * Create a Connection.
   * @member {object} source Source resource of the relationship.
   * @member {string} [source.id] Resource URI.
   * @member {string} [source.type] Resource type qualifier.
   * @member {string} [source.name] Resource name.
   * @member {string} [source.kind] Polymorphic Discriminator
   * @member {object} destination Destination resource of the relationship.
   * @member {string} [destination.id] Resource URI.
   * @member {string} [destination.type] Resource type qualifier.
   * @member {string} [destination.name] Resource name.
   * @member {string} [destination.kind] Polymorphic Discriminator
   * @member {date} [startTime] Relationship start time.
   * @member {date} [endTime] Relationship end time.
   * @member {object} [serverPort] Reference to the server port via which this
   * connection has been established.
   * @member {object} [serverPort.machine] Machine hosting the port.
   * @member {string} [serverPort.ipAddress] IP address of the port.
   * @member {number} [serverPort.portNumber] Port number.
   * @member {string} [failureState] Specifies whether there are only
   * successful, failed or a mixture of both connections represented by this
   * resource. Possible values include: 'ok', 'failed', 'mixed'
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Connection
   *
   * @returns {object} metadata of Connection
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'rel:connection',
      type: {
        name: 'Composite',
        className: 'Connection',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'kind',
            type: {
              name: 'String'
            }
          },
          source: {
            required: true,
            serializedName: 'properties.source',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'kind',
                clientName: 'kind'
              },
              uberParent: 'ResourceReference',
              className: 'ResourceReference'
            }
          },
          destination: {
            required: true,
            serializedName: 'properties.destination',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'kind',
                clientName: 'kind'
              },
              uberParent: 'ResourceReference',
              className: 'ResourceReference'
            }
          },
          startTime: {
            required: false,
            serializedName: 'properties.startTime',
            type: {
              name: 'DateTime'
            }
          },
          endTime: {
            required: false,
            serializedName: 'properties.endTime',
            type: {
              name: 'DateTime'
            }
          },
          serverPort: {
            required: false,
            serializedName: 'properties.serverPort',
            type: {
              name: 'Composite',
              className: 'PortReference'
            }
          },
          failureState: {
            required: false,
            serializedName: 'properties.failureState',
            type: {
              name: 'Enum',
              allowedValues: [ 'ok', 'failed', 'mixed' ]
            }
          }
        }
      }
    };
  }
}

module.exports = Connection;
