/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes the updatable properties of the job
 *
 */
class MutableJob {
  /**
   * Create a MutableJob.
   * @member {object} [tags]
   * @member {boolean} [cancelRequested] If specified, the value must be true.
   * The service will attempt to cancel the job.
   * @member {string} [state] If specified, the value must be Shipping, which
   * tells the Import/Export service that the package for the job has been
   * shipped. The ReturnAddress and DeliveryPackage properties must have been
   * set either in this request or in a previous request, otherwise the request
   * will fail. Possible values include: 'Shipping'
   * @member {object} [returnAddress] Specifies the return address information
   * for the job.
   * @member {string} [returnAddress.recipientName] The name of the recipient
   * who will receive the hard drives when they are returned.
   * @member {string} [returnAddress.streetAddress1] The first line of the
   * street address to use when returning the drives.
   * @member {string} [returnAddress.streetAddress2] The second line of the
   * street address to use when returning the drives.
   * @member {string} [returnAddress.city] The city name to use when returning
   * the drives.
   * @member {string} [returnAddress.stateOrProvince] The state or province to
   * use when returning the drives.
   * @member {string} [returnAddress.postalCode] The postal code to use when
   * returning the drives.
   * @member {string} [returnAddress.countryOrRegion] The country or region to
   * use when returning the drives.
   * @member {string} [returnAddress.phone] Phone number of the recipient of
   * the returned drives.
   * @member {string} [returnAddress.email] Email address of the recipient of
   * the returned drives.
   * @member {object} [returnShipping] Specifies the return carrier and
   * customer's account with the carrier.
   * @member {string} [returnShipping.carrierName] The carrier's name.
   * @member {string} [returnShipping.carrierAccountNumber] The customer's
   * account number with the carrier.
   * @member {object} [deliveryPackage] Contains information about the package
   * being shipped by the customer to the Microsoft data center.
   * @member {string} [deliveryPackage.carrierName] The name of the carrier
   * that is used to ship the import or export drives.
   * @member {string} [deliveryPackage.trackingNumber] The tracking number of
   * the package.
   * @member {number} [deliveryPackage.driveCount] The number of drives
   * included in the package.
   * @member {string} [deliveryPackage.shipDate] The date the package is
   * shipped.
   * @member {string} [logLevel] Indicates whether error logging or verbose
   * logging is enabled.
   * @member {boolean} [backupDriveManifest] Indicates whether the manifest
   * files on the drives should be copied to block blobs.
   */
  constructor() {
  }

  /**
   * Defines the metadata of MutableJob
   *
   * @returns {object} metadata of MutableJob
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'MutableJob',
      type: {
        name: 'Composite',
        className: 'MutableJob',
        modelProperties: {
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Object'
            }
          },
          cancelRequested: {
            required: false,
            serializedName: 'properties.cancelRequested',
            type: {
              name: 'Boolean'
            }
          },
          state: {
            required: false,
            serializedName: 'properties.state',
            type: {
              name: 'String'
            }
          },
          returnAddress: {
            required: false,
            serializedName: 'properties.returnAddress',
            type: {
              name: 'Composite',
              className: 'ReturnAddress'
            }
          },
          returnShipping: {
            required: false,
            serializedName: 'properties.returnShipping',
            type: {
              name: 'Composite',
              className: 'ReturnShipping'
            }
          },
          deliveryPackage: {
            required: false,
            serializedName: 'properties.deliveryPackage',
            type: {
              name: 'Composite',
              className: 'PackageInfomation'
            }
          },
          logLevel: {
            required: false,
            serializedName: 'properties.logLevel',
            type: {
              name: 'String'
            }
          },
          backupDriveManifest: {
            required: false,
            serializedName: 'properties.backupDriveManifest',
            type: {
              name: 'Boolean'
            }
          }
        }
      }
    };
  }
}

module.exports = MutableJob;
