/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Parameter supplied to Update Api Management Service.
 *
 * @extends models['ApimResource']
 */
class ApiManagementServiceUpdateParameters extends models['ApimResource'] {
  /**
   * Create a ApiManagementServiceUpdateParameters.
   * @member {string} [notificationSenderEmail] Email address from which the
   * notification will be sent.
   * @member {string} [provisioningState] The current provisioning state of the
   * API Management service which can be one of the following:
   * Created/Activating/Succeeded/Updating/Failed/Stopped/Terminating/TerminationFailed/Deleted.
   * @member {string} [targetProvisioningState] The provisioning state of the
   * API Management service, which is targeted by the long running operation
   * started on the service.
   * @member {date} [createdAtUtc] Creation UTC date of the API Management
   * service.The date conforms to the following format: `yyyy-MM-ddTHH:mm:ssZ`
   * as specified by the ISO 8601 standard.
   * @member {string} [gatewayUrl] Gateway URL of the API Management service.
   * @member {string} [portalUrl] Publisher portal endpoint Url of the API
   * Management service.
   * @member {string} [managementApiUrl] Management API endpoint URL of the API
   * Management service.
   * @member {string} [scmUrl] SCM endpoint URL of the API Management service.
   * @member {array} [hostnameConfigurations] Custom hostname configuration of
   * the API Management service.
   * @member {array} [staticIps] Static IP addresses of the API Management
   * service virtual machines. Available only for Standard and Premium SKU.
   * @member {object} [virtualNetworkConfiguration] Virtual network
   * configuration of the API Management service.
   * @member {string} [virtualNetworkConfiguration.vnetid] The virtual network
   * ID. This is typically a GUID. Expect a null GUID by default.
   * @member {string} [virtualNetworkConfiguration.subnetname] The name of the
   * subnet.
   * @member {string} [virtualNetworkConfiguration.subnetResourceId] The full
   * resource ID of a subnet in a virtual network to deploy the API Management
   * service in.
   * @member {array} [additionalLocations] Additional datacenter locations of
   * the API Management service.
   * @member {object} [customProperties] Custom properties of the API
   * Management service. Setting
   * `Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Ciphers.TripleDes168`
   * will disable the cipher TLS_RSA_WITH_3DES_EDE_CBC_SHA for all TLS(1.0, 1.1
   * and 1.2). Setting
   * `Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Protocols.Tls11`
   * can be used to disable just TLS 1.1 and setting
   * `Microsoft.WindowsAzure.ApiManagement.Gateway.Security.Protocols.Tls10`
   * can be used to disable TLS 1.0 on an API Management service.
   * @member {array} [certificates] List of Certificates that need to be
   * installed in the API Management service. Max supported certificates that
   * can be installed is 10.
   * @member {string} [virtualNetworkType] The type of VPN in which API
   * Managemet service needs to be configured in. None (Default Value) means
   * the API Management service is not part of any Virtual Network, External
   * means the API Management deployment is set up inside a Virtual Network
   * having an Internet Facing Endpoint, and Internal means that API Management
   * deployment is setup inside a Virtual Network having an Intranet Facing
   * Endpoint only. Possible values include: 'None', 'External', 'Internal'.
   * Default value: 'None' .
   * @member {string} [publisherEmail] Publisher email.
   * @member {string} [publisherName] Publisher name.
   * @member {object} [sku] SKU properties of the API Management service.
   * @member {string} [sku.name] Name of the Sku. Possible values include:
   * 'Developer', 'Standard', 'Premium'
   * @member {number} [sku.capacity] Capacity of the SKU (number of deployed
   * units of the SKU). The default value is 1.
   * @member {string} [etag] ETag of the resource.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ApiManagementServiceUpdateParameters
   *
   * @returns {object} metadata of ApiManagementServiceUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ApiManagementServiceUpdateParameters',
      type: {
        name: 'Composite',
        className: 'ApiManagementServiceUpdateParameters',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          notificationSenderEmail: {
            required: false,
            serializedName: 'properties.notificationSenderEmail',
            constraints: {
              MaxLength: 100
            },
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          targetProvisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.targetProvisioningState',
            type: {
              name: 'String'
            }
          },
          createdAtUtc: {
            required: false,
            readOnly: true,
            serializedName: 'properties.createdAtUtc',
            type: {
              name: 'DateTime'
            }
          },
          gatewayUrl: {
            required: false,
            readOnly: true,
            serializedName: 'properties.gatewayUrl',
            type: {
              name: 'String'
            }
          },
          portalUrl: {
            required: false,
            readOnly: true,
            serializedName: 'properties.portalUrl',
            type: {
              name: 'String'
            }
          },
          managementApiUrl: {
            required: false,
            readOnly: true,
            serializedName: 'properties.managementApiUrl',
            type: {
              name: 'String'
            }
          },
          scmUrl: {
            required: false,
            readOnly: true,
            serializedName: 'properties.scmUrl',
            type: {
              name: 'String'
            }
          },
          hostnameConfigurations: {
            required: false,
            serializedName: 'properties.hostnameConfigurations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HostnameConfigurationElementType',
                  type: {
                    name: 'Composite',
                    className: 'HostnameConfiguration'
                  }
              }
            }
          },
          staticIps: {
            required: false,
            readOnly: true,
            serializedName: 'properties.staticIps',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          virtualNetworkConfiguration: {
            required: false,
            serializedName: 'properties.virtualNetworkConfiguration',
            type: {
              name: 'Composite',
              className: 'VirtualNetworkConfiguration'
            }
          },
          additionalLocations: {
            required: false,
            serializedName: 'properties.additionalLocations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'AdditionalLocationElementType',
                  type: {
                    name: 'Composite',
                    className: 'AdditionalLocation'
                  }
              }
            }
          },
          customProperties: {
            required: false,
            serializedName: 'properties.customProperties',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          certificates: {
            required: false,
            serializedName: 'properties.certificates',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'CertificateConfigurationElementType',
                  type: {
                    name: 'Composite',
                    className: 'CertificateConfiguration'
                  }
              }
            }
          },
          virtualNetworkType: {
            required: false,
            serializedName: 'properties.virtualNetworkType',
            defaultValue: 'None',
            type: {
              name: 'Enum',
              allowedValues: [ 'None', 'External', 'Internal' ]
            }
          },
          publisherEmail: {
            required: false,
            serializedName: 'properties.publisherEmail',
            constraints: {
              MaxLength: 100
            },
            type: {
              name: 'String'
            }
          },
          publisherName: {
            required: false,
            serializedName: 'properties.publisherName',
            constraints: {
              MaxLength: 100
            },
            type: {
              name: 'String'
            }
          },
          sku: {
            required: false,
            serializedName: 'sku',
            type: {
              name: 'Composite',
              className: 'ApiManagementServiceSkuProperties'
            }
          },
          etag: {
            required: false,
            readOnly: true,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ApiManagementServiceUpdateParameters;
