/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * User update parameters.
 *
 */
class UserUpdateParameters {
  /**
   * Create a UserUpdateParameters.
   * @member {string} [state] Account state. Specifies whether the user is
   * active or not. Blocked users are unable to sign into the developer portal
   * or call any APIs of subscribed products. Default state is Active. Possible
   * values include: 'active', 'blocked'. Default value: 'active' .
   * @member {string} [note] Optional note about a user set by the
   * administrator.
   * @member {array} [identities] Collection of user identities.
   * @member {string} [email] Email address. Must not be empty and must be
   * unique within the service instance.
   * @member {string} [password] User Password.
   * @member {string} [firstName] First name.
   * @member {string} [lastName] Last name.
   */
  constructor() {
  }

  /**
   * Defines the metadata of UserUpdateParameters
   *
   * @returns {object} metadata of UserUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'UserUpdateParameters',
      type: {
        name: 'Composite',
        className: 'UserUpdateParameters',
        modelProperties: {
          state: {
            required: false,
            serializedName: 'properties.state',
            defaultValue: 'active',
            type: {
              name: 'Enum',
              allowedValues: [ 'active', 'blocked' ]
            }
          },
          note: {
            required: false,
            serializedName: 'properties.note',
            type: {
              name: 'String'
            }
          },
          identities: {
            required: false,
            readOnly: true,
            serializedName: 'properties.identities',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'UserIdentityContractElementType',
                  type: {
                    name: 'Composite',
                    className: 'UserIdentityContract'
                  }
              }
            }
          },
          email: {
            required: false,
            serializedName: 'properties.email',
            constraints: {
              MaxLength: 254,
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          password: {
            required: false,
            serializedName: 'properties.password',
            type: {
              name: 'String'
            }
          },
          firstName: {
            required: false,
            serializedName: 'properties.firstName',
            constraints: {
              MaxLength: 100,
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          },
          lastName: {
            required: false,
            serializedName: 'properties.lastName',
            constraints: {
              MaxLength: 100,
              MinLength: 1
            },
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = UserUpdateParameters;
