/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Error reponse indicates Insights service is not able to process the incoming
 * request. The reason is provided in the error message.
 *
 * @member {string} [code] Error code.
 * @member {string} [message] Error message indicating why the operation
 * failed.
 */
export interface ErrorResponse {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @member {string} [provider] Service provider: Microsoft.Cdn
 * @member {string} [resource] Resource on which the operation is performed:
 * Profile, endpoint, etc.
 * @member {string} [operation] Operation type: Read, write, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * CDN REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] The object that represents the operation.
 * @member {string} [display.provider] Service provider: Microsoft.Cdn
 * @member {string} [display.resource] Resource on which the operation is
 * performed: Profile, endpoint, etc.
 * @member {string} [display.operation] Operation type: Read, write, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * An azure resource object
 *
 * @member {string} [id] Azure resource Id
 * @member {string} [name] Azure resource name
 * @member {string} [type] Azure resource type
 * @member {string} location Resource location
 * @member {object} [tags] Resource tags
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the TagsResource class.
 * @constructor
 * A container holding only the Tags for a resource, allowing the user to
 * update the tags on a WebTest instance.
 *
 * @member {object} [tags] Resource tags
 */
export interface TagsResource {
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ApplicationInsightsComponent class.
 * @constructor
 * An Application Insights component definition.
 *
 * @member {string} kind The kind of application that this component refers to,
 * used to customize UI. This value is a freeform string, values should
 * typically be one of the following: web, ios, other, store, java, phone.
 * @member {string} [applicationId] The unique ID of your application. This
 * field mirrors the 'Name' field and cannot be changed.
 * @member {string} [appId] Application Insights Unique ID for your
 * Application.
 * @member {string} applicationType Type of application being monitored.
 * Possible values include: 'web', 'other'. Default value: 'web' .
 * @member {string} [flowType] Used by the Application Insights system to
 * determine what kind of flow this component was created by. This is to be set
 * to 'Bluefield' when creating/updating a component via the REST API. Possible
 * values include: 'Bluefield'. Default value: 'Bluefield' .
 * @member {string} [requestSource] Describes what tool created this
 * Application Insights component. Customers using this API should set this to
 * the default 'rest'. Possible values include: 'rest'. Default value: 'rest' .
 * @member {string} [instrumentationKey] Application Insights Instrumentation
 * key. A read-only value that applications can use to identify the destination
 * for all telemetry sent to Azure Application Insights. This value will be
 * supplied upon construction of each new Application Insights component.
 * @member {date} [creationDate] Creation Date for the Application Insights
 * component, in ISO 8601 format.
 * @member {string} [tenantId] Azure Tenant Id.
 * @member {string} [hockeyAppId] The unique application ID created when a new
 * application is added to HockeyApp, used for communications with HockeyApp.
 * @member {string} [hockeyAppToken] Token used to authenticate communications
 * with between Application Insights and HockeyApp.
 * @member {string} [provisioningState] Current state of this component:
 * whether or not is has been provisioned within the resource group it is
 * defined. Users cannot change this value but are able to read from it. Values
 * will include Succeeded, Deploying, Canceled, and Failed.
 * @member {number} [samplingPercentage] Percentage of the data produced by the
 * application being monitored that is being sampled for Application Insights
 * telemetry.
 */
export interface ApplicationInsightsComponent extends Resource {
  kind: string;
  readonly applicationId?: string;
  readonly appId?: string;
  applicationType: string;
  flowType?: string;
  requestSource?: string;
  readonly instrumentationKey?: string;
  readonly creationDate?: Date;
  readonly tenantId?: string;
  hockeyAppId?: string;
  readonly hockeyAppToken?: string;
  readonly provisioningState?: string;
  samplingPercentage?: number;
}

/**
 * @class
 * Initializes a new instance of the WebTestGeolocation class.
 * @constructor
 * Geo-physical location to run a web test from. You must specify one or more
 * locations for the test to run from.
 *
 * @member {string} [location] Location ID for the webtest to run from.
 */
export interface WebTestGeolocation {
  location?: string;
}

/**
 * @class
 * Initializes a new instance of the WebTestPropertiesConfiguration class.
 * @constructor
 * An XML configuration specification for a WebTest.
 *
 * @member {string} [webTest] The XML specification of a WebTest to run against
 * an application.
 */
export interface WebTestPropertiesConfiguration {
  webTest?: string;
}

/**
 * @class
 * Initializes a new instance of the WebTest class.
 * @constructor
 * An Application Insights web test definition.
 *
 * @member {string} [kind] The kind of web test that this web test watches.
 * Choices are ping and multistep. Possible values include: 'ping',
 * 'multistep'. Default value: 'ping' .
 * @member {string} syntheticMonitorId Unique ID of this WebTest. This is
 * typically the same value as the Name field.
 * @member {string} webTestName User defined name if this WebTest.
 * @member {string} [description] Purpose/user defined descriptive test for
 * this WebTest.
 * @member {boolean} [enabled] Is the test actively being monitored.
 * @member {number} [frequency] Interval in seconds between test runs for this
 * WebTest. Default value is 300. Default value: 300 .
 * @member {number} [timeout] Seconds until this WebTest will timeout and fail.
 * Default value is 30. Default value: 30 .
 * @member {string} webTestKind The kind of web test this is, valid choices are
 * ping and multistep. Possible values include: 'ping', 'multistep'. Default
 * value: 'ping' .
 * @member {boolean} [retryEnabled] Allow for retries should this WebTest fail.
 * @member {array} locations A list of where to physically run the tests from
 * to give global coverage for accessibility of your application.
 * @member {object} [configuration] An XML configuration specification for a
 * WebTest.
 * @member {string} [configuration.webTest] The XML specification of a WebTest
 * to run against an application.
 * @member {string} [provisioningState] Current state of this component,
 * whether or not is has been provisioned within the resource group it is
 * defined. Users cannot change this value but are able to read from it. Values
 * will include Succeeded, Deploying, Canceled, and Failed.
 */
export interface WebTest extends Resource {
  kind?: string;
  syntheticMonitorId: string;
  webTestName: string;
  description?: string;
  enabled?: boolean;
  frequency?: number;
  timeout?: number;
  webTestKind: string;
  retryEnabled?: boolean;
  locations: WebTestGeolocation[];
  configuration?: WebTestPropertiesConfiguration;
  readonly provisioningState?: string;
}


/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result of the request to list CDN operations. It contains a list of
 * operations and a URL link to get the next set of results.
 *
 * @member {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult extends Array<Operation> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ApplicationInsightsComponentListResult class.
 * @constructor
 * Describes the list of Application Insights Resources.
 *
 * @member {string} [nextLink] The URI to get the next set of Application
 * Insights component defintions if too many components where returned in the
 * result set.
 */
export interface ApplicationInsightsComponentListResult extends Array<ApplicationInsightsComponent> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WebTestListResult class.
 * @constructor
 * A list of 0 or more Application Insights web test definitions.
 *
 * @member {string} [nextLink] The link to get the next part of the returned
 * list of web tests, should the return set be too large for a single request.
 * May be null.
 */
export interface WebTestListResult extends Array<WebTest> {
  nextLink?: string;
}
