/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the RoleAssignmentFilter class.
 * @constructor
 * Role Assignments filter
 *
 * @member {string} [principalId] Returns role assignment of the specific
 * principal.
 */
export interface RoleAssignmentFilter {
  principalId?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleDefinitionFilter class.
 * @constructor
 * Role Definitions filter
 *
 * @member {string} [roleName] Returns role definition with the specific name.
 */
export interface RoleDefinitionFilter {
  roleName?: string;
}

/**
 * @class
 * Initializes a new instance of the ClassicAdministratorProperties class.
 * @constructor
 * Classic Administrator properties.
 *
 * @member {string} [emailAddress] The email address of the administrator.
 * @member {string} [role] The role of the administrator.
 */
export interface ClassicAdministratorProperties {
  emailAddress?: string;
  role?: string;
}

/**
 * @class
 * Initializes a new instance of the ClassicAdministrator class.
 * @constructor
 * Classic Administrators
 *
 * @member {string} [id] The ID of the administrator.
 * @member {string} [name] The name of the administrator.
 * @member {string} [type] The type of the administrator.
 * @member {object} [properties] Properties for the classic administrator.
 * @member {string} [properties.emailAddress] The email address of the
 * administrator.
 * @member {string} [properties.role] The role of the administrator.
 */
export interface ClassicAdministrator {
  id?: string;
  name?: string;
  type?: string;
  properties?: ClassicAdministratorProperties;
}

/**
 * @class
 * Initializes a new instance of the Permission class.
 * @constructor
 * Role definition permissions.
 *
 * @member {array} [actions] Allowed actions.
 * @member {array} [notActions] Denied actions.
 */
export interface Permission {
  actions?: string[];
  notActions?: string[];
}

/**
 * @class
 * Initializes a new instance of the ProviderOperation class.
 * @constructor
 * Operation
 *
 * @member {string} [name] The operation name.
 * @member {string} [displayName] The operation display name.
 * @member {string} [description] The operation description.
 * @member {string} [origin] The operation origin.
 * @member {object} [properties] The operation properties.
 */
export interface ProviderOperation {
  name?: string;
  displayName?: string;
  description?: string;
  origin?: string;
  properties?: any;
}

/**
 * @class
 * Initializes a new instance of the ResourceType class.
 * @constructor
 * Resource Type
 *
 * @member {string} [name] The resource type name.
 * @member {string} [displayName] The resource type display name.
 * @member {array} [operations] The resource type operations.
 */
export interface ResourceType {
  name?: string;
  displayName?: string;
  operations?: ProviderOperation[];
}

/**
 * @class
 * Initializes a new instance of the ProviderOperationsMetadata class.
 * @constructor
 * Provider Operations metadata
 *
 * @member {string} [id] The provider id.
 * @member {string} [name] The provider name.
 * @member {string} [type] The provider type.
 * @member {string} [displayName] The provider display name.
 * @member {array} [resourceTypes] The provider resource types
 * @member {array} [operations] The provider operations.
 */
export interface ProviderOperationsMetadata {
  id?: string;
  name?: string;
  type?: string;
  displayName?: string;
  resourceTypes?: ResourceType[];
  operations?: ProviderOperation[];
}

/**
 * @class
 * Initializes a new instance of the RoleAssignmentPropertiesWithScope class.
 * @constructor
 * Role assignment properties with scope.
 *
 * @member {string} [scope] The role assignment scope.
 * @member {string} [roleDefinitionId] The role definition ID.
 * @member {string} [principalId] The principal ID.
 */
export interface RoleAssignmentPropertiesWithScope {
  scope?: string;
  roleDefinitionId?: string;
  principalId?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleAssignment class.
 * @constructor
 * Role Assignments
 *
 * @member {string} [id] The role assignment ID.
 * @member {string} [name] The role assignment name.
 * @member {string} [type] The role assignment type.
 * @member {object} [properties] Role assignment properties.
 * @member {string} [properties.scope] The role assignment scope.
 * @member {string} [properties.roleDefinitionId] The role definition ID.
 * @member {string} [properties.principalId] The principal ID.
 */
export interface RoleAssignment {
  id?: string;
  name?: string;
  type?: string;
  properties?: RoleAssignmentPropertiesWithScope;
}

/**
 * @class
 * Initializes a new instance of the RoleAssignmentProperties class.
 * @constructor
 * Role assignment properties.
 *
 * @member {string} [roleDefinitionId] The role definition ID used in the role
 * assignment.
 * @member {string} [principalId] The principal ID assigned to the role. This
 * maps to the ID inside the Active Directory. It can point to a user, service
 * principal, or security group.
 */
export interface RoleAssignmentProperties {
  roleDefinitionId?: string;
  principalId?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleAssignmentCreateParameters class.
 * @constructor
 * Role assignment create parameters.
 *
 * @member {object} [properties] Role assignment properties.
 * @member {string} [properties.roleDefinitionId] The role definition ID used
 * in the role assignment.
 * @member {string} [properties.principalId] The principal ID assigned to the
 * role. This maps to the ID inside the Active Directory. It can point to a
 * user, service principal, or security group.
 */
export interface RoleAssignmentCreateParameters {
  properties?: RoleAssignmentProperties;
}

/**
 * @class
 * Initializes a new instance of the RoleDefinitionProperties class.
 * @constructor
 * Role definition properties.
 *
 * @member {string} [roleName] The role name.
 * @member {string} [description] The role definition description.
 * @member {string} [type] The role type.
 * @member {array} [permissions] Role definition permissions.
 * @member {array} [assignableScopes] Role definition assignable scopes.
 */
export interface RoleDefinitionProperties {
  roleName?: string;
  description?: string;
  type?: string;
  permissions?: Permission[];
  assignableScopes?: string[];
}

/**
 * @class
 * Initializes a new instance of the RoleDefinition class.
 * @constructor
 * Role definition.
 *
 * @member {string} [id] The role definition ID.
 * @member {string} [name] The role definition name.
 * @member {string} [type] The role definition type.
 * @member {object} [properties] Role definition properties.
 * @member {string} [properties.roleName] The role name.
 * @member {string} [properties.description] The role definition description.
 * @member {string} [properties.type] The role type.
 * @member {array} [properties.permissions] Role definition permissions.
 * @member {array} [properties.assignableScopes] Role definition assignable
 * scopes.
 */
export interface RoleDefinition {
  id?: string;
  name?: string;
  type?: string;
  properties?: RoleDefinitionProperties;
}


/**
 * @class
 * Initializes a new instance of the ClassicAdministratorListResult class.
 * @constructor
 * ClassicAdministrator list result information.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface ClassicAdministratorListResult extends Array<ClassicAdministrator> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the PermissionGetResult class.
 * @constructor
 * Permissions information.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface PermissionGetResult extends Array<Permission> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ProviderOperationsMetadataListResult class.
 * @constructor
 * Provider operations metadata list
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface ProviderOperationsMetadataListResult extends Array<ProviderOperationsMetadata> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleAssignmentListResult class.
 * @constructor
 * Role assignment list operation result.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface RoleAssignmentListResult extends Array<RoleAssignment> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleDefinitionListResult class.
 * @constructor
 * Role definition list operation result.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface RoleDefinitionListResult extends Array<RoleDefinition> {
  nextLink?: string;
}
