/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * Error response of an operation failure
 *
 * @member {string} [code] Error code
 * @member {string} [message] Error message indicating why the operation
 * failed.
 */
export interface ErrorResponse {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the UsageCounterName class.
 * @constructor
 * Definition of usage counter name.
 *
 * @member {string} [value] Gets or sets the usage counter name.
 * @member {string} [localizedValue] Gets or sets the localized usage counter
 * name.
 */
export interface UsageCounterName {
  value?: string;
  localizedValue?: string;
}

/**
 * @class
 * Initializes a new instance of the Usage class.
 * @constructor
 * Definition of Usage.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {object} [name] Gets or sets the usage counter name.
 * @member {string} [name.value] Gets or sets the usage counter name.
 * @member {string} [name.localizedValue] Gets or sets the localized usage
 * counter name.
 * @member {string} [unit] Gets or sets the usage unit name.
 * @member {number} [currentValue] Gets or sets the current usage value.
 * @member {number} [limit] Gets or sets max limit. -1 for unlimited
 * @member {string} [throttleStatus] Gets or sets the throttle status.
 */
export interface Usage {
  id?: string;
  name?: UsageCounterName;
  unit?: string;
  currentValue?: number;
  limit?: number;
  throttleStatus?: string;
}

/**
 * @class
 * Initializes a new instance of the Statistics class.
 * @constructor
 * Definition of the statistic.
 *
 * @member {string} [counterProperty] Gets the property value of the statistic.
 * @member {number} [counterValue] Gets the value of the statistic.
 * @member {date} [startTime] Gets the startTime of the statistic.
 * @member {date} [endTime] Gets the endTime of the statistic.
 * @member {string} [id] Gets the id.
 */
export interface Statistics {
  readonly counterProperty?: string;
  readonly counterValue?: number;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly id?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookParameter class.
 * @constructor
 * Definition of the runbook parameter type.
 *
 * @member {string} [type] Gets or sets the type of the parameter.
 * @member {boolean} [isMandatory] Gets or sets a Boolean value to indicate
 * whether the parameter is madatory or not.
 * @member {number} [position] Get or sets the position of the parameter.
 * @member {string} [defaultValue] Gets or sets the default value of parameter.
 */
export interface RunbookParameter {
  type?: string;
  isMandatory?: boolean;
  position?: number;
  defaultValue?: string;
}

/**
 * @class
 * Initializes a new instance of the ContentHash class.
 * @constructor
 * Definition of the runbook property type.
 *
 * @member {string} algorithm Gets or sets the content hash algorithm used to
 * hash the content.
 * @member {string} value Gets or sets expected hash value of the content.
 */
export interface ContentHash {
  algorithm: string;
  value: string;
}

/**
 * @class
 * Initializes a new instance of the ContentLink class.
 * @constructor
 * Definition of the content link.
 *
 * @member {string} [uri] Gets or sets the uri of the runbook content.
 * @member {object} [contentHash] Gets or sets the hash.
 * @member {string} [contentHash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [contentHash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [version] Gets or sets the version of the content.
 */
export interface ContentLink {
  uri?: string;
  contentHash?: ContentHash;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookDraft class.
 * @constructor
 * Definition of the runbook type.
 *
 * @member {boolean} [inEdit] Gets or sets whether runbook is in edit mode.
 * @member {object} [draftContentLink] Gets or sets the draft runbook content
 * link.
 * @member {string} [draftContentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draftContentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [draftContentLink.contentHash.value] Gets or sets expected
 * hash value of the content.
 * @member {string} [draftContentLink.version] Gets or sets the version of the
 * content.
 * @member {date} [creationTime] Gets or sets the creation time of the runbook
 * draft.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time of the
 * runbook draft.
 * @member {object} [parameters] Gets or sets the runbook draft parameters.
 * @member {array} [outputTypes] Gets or sets the runbook output types.
 */
export interface RunbookDraft {
  inEdit?: boolean;
  draftContentLink?: ContentLink;
  creationTime?: Date;
  lastModifiedTime?: Date;
  parameters?: { [propertyName: string]: RunbookParameter };
  outputTypes?: string[];
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * The Resource definition.
 *
 * @member {string} [id] Resource Id
 * @member {string} [name] Resource name
 * @member {string} [type] Resource type
 * @member {string} location Resource location
 * @member {object} [tags] Resource tags
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Runbook class.
 * @constructor
 * Definition of the runbook type.
 *
 * @member {string} [runbookType] Gets or sets the type of the runbook.
 * Possible values include: 'Script', 'Graph', 'PowerShellWorkflow',
 * 'PowerShell', 'GraphPowerShellWorkflow', 'GraphPowerShell'
 * @member {object} [publishContentLink] Gets or sets the published runbook
 * content link.
 * @member {string} [publishContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [publishContentLink.contentHash] Gets or sets the hash.
 * @member {string} [publishContentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [publishContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [publishContentLink.version] Gets or sets the version of
 * the content.
 * @member {string} [state] Gets or sets the state of the runbook. Possible
 * values include: 'New', 'Edit', 'Published'
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {number} [logActivityTrace] Gets or sets the option to log activity
 * trace of the runbook.
 * @member {number} [jobCount] Gets or sets the job count of the runbook.
 * @member {object} [parameters] Gets or sets the runbook parameters.
 * @member {array} [outputTypes] Gets or sets the runbook output types.
 * @member {object} [draft] Gets or sets the draft runbook properties.
 * @member {boolean} [draft.inEdit] Gets or sets whether runbook is in edit
 * mode.
 * @member {object} [draft.draftContentLink] Gets or sets the draft runbook
 * content link.
 * @member {string} [draft.draftContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [draft.draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draft.draftContentLink.contentHash.algorithm] Gets or sets
 * the content hash algorithm used to hash the content.
 * @member {string} [draft.draftContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [draft.draftContentLink.version] Gets or sets the version
 * of the content.
 * @member {date} [draft.creationTime] Gets or sets the creation time of the
 * runbook draft.
 * @member {date} [draft.lastModifiedTime] Gets or sets the last modified time
 * of the runbook draft.
 * @member {object} [draft.parameters] Gets or sets the runbook draft
 * parameters.
 * @member {array} [draft.outputTypes] Gets or sets the runbook output types.
 * @member {string} [provisioningState] Gets or sets the provisioning state of
 * the runbook. Possible values include: 'Succeeded'
 * @member {string} [lastModifiedBy] Gets or sets the last modified by.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface Runbook extends Resource {
  runbookType?: string;
  publishContentLink?: ContentLink;
  state?: string;
  logVerbose?: boolean;
  logProgress?: boolean;
  logActivityTrace?: number;
  jobCount?: number;
  parameters?: { [propertyName: string]: RunbookParameter };
  outputTypes?: string[];
  draft?: RunbookDraft;
  provisioningState?: string;
  lastModifiedBy?: string;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the ModuleErrorInfo class.
 * @constructor
 * Definition of the module error info type.
 *
 * @member {string} [code] Gets or sets the error code.
 * @member {string} [message] Gets or sets the error message.
 */
export interface ModuleErrorInfo {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the Module class.
 * @constructor
 * Definition of the module type.
 *
 * @member {boolean} [isGlobal] Gets or sets the isGlobal flag of the module.
 * @member {string} [version] Gets or sets the version of the module.
 * @member {number} [sizeInBytes] Gets or sets the size in bytes of the module.
 * @member {number} [activityCount] Gets or sets the activity count of the
 * module.
 * @member {string} [provisioningState] Gets or sets the provisioning state of
 * the module. Possible values include: 'Created', 'Creating',
 * 'StartingImportModuleRunbook', 'RunningImportModuleRunbook',
 * 'ContentRetrieved', 'ContentDownloaded', 'ContentValidated',
 * 'ConnectionTypeImported', 'ContentStored', 'ModuleDataStored',
 * 'ActivitiesStored', 'ModuleImportRunbookComplete', 'Succeeded', 'Failed',
 * 'Cancelled', 'Updating'
 * @member {object} [contentLink] Gets or sets the contentLink of the module.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {object} [error] Gets or sets the error info of the module.
 * @member {string} [error.code] Gets or sets the error code.
 * @member {string} [error.message] Gets or sets the error message.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface Module extends Resource {
  isGlobal?: boolean;
  version?: string;
  sizeInBytes?: number;
  activityCount?: number;
  provisioningState?: string;
  contentLink?: ContentLink;
  error?: ModuleErrorInfo;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfigurationAssociationProperty class.
 * @constructor
 * The dsc nodeconfiguration property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the dsc nodeconfiguration.
 */
export interface DscNodeConfigurationAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNode class.
 * @constructor
 * Definition of the dsc node type.
 *
 * @member {date} [lastSeen] Gets or sets the last seen time of the node.
 * @member {date} [registrationTime] Gets or sets the registration time of the
 * node.
 * @member {string} [ip] Gets or sets the ip of the node.
 * @member {string} [accountId] Gets or sets the account id of the node.
 * @member {object} [nodeConfiguration] Gets or sets the configuration of the
 * node.
 * @member {string} [nodeConfiguration.name] Gets or sets the name of the dsc
 * nodeconfiguration.
 * @member {string} [status] Gets or sets the status of the node.
 * @member {string} [nodeId] Gets or sets the node id.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface DscNode extends Resource {
  lastSeen?: Date;
  registrationTime?: Date;
  ip?: string;
  accountId?: string;
  nodeConfiguration?: DscNodeConfigurationAssociationProperty;
  status?: string;
  nodeId?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the ContentSource class.
 * @constructor
 * Definition of the content source.
 *
 * @member {object} [hash] Gets or sets the hash.
 * @member {string} [hash.algorithm] Gets or sets the content hash algorithm
 * used to hash the content.
 * @member {string} [hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [type] Gets or sets the content source type. Possible
 * values include: 'embeddedContent', 'uri'
 * @member {string} [value] Gets or sets the value of the content. This is
 * based on the content source type.
 * @member {string} [version] Gets or sets the version of the content.
 */
export interface ContentSource {
  hash?: ContentHash;
  type?: string;
  value?: string;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationParameter class.
 * @constructor
 * Definition of the configuration parameter type.
 *
 * @member {string} [type] Gets or sets the type of the parameter.
 * @member {boolean} [isMandatory] Gets or sets a Boolean value to indicate
 * whether the parameter is madatory or not.
 * @member {number} [position] Get or sets the position of the parameter.
 * @member {string} [defaultValue] Gets or sets the default value of parameter.
 */
export interface DscConfigurationParameter {
  type?: string;
  isMandatory?: boolean;
  position?: number;
  defaultValue?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfiguration class.
 * @constructor
 * Definition of the configuration type.
 *
 * @member {string} [provisioningState] Gets or sets the provisioning state of
 * the configuration. Possible values include: 'Succeeded'
 * @member {number} [jobCount] Gets or sets the job count of the configuration.
 * @member {object} [parameters] Gets or sets the configuration parameters.
 * @member {object} [source] Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {string} [state] Gets or sets the state of the configuration.
 * Possible values include: 'New', 'Edit', 'Published'
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface DscConfiguration extends Resource {
  provisioningState?: string;
  jobCount?: number;
  parameters?: { [propertyName: string]: DscConfigurationParameter };
  source?: ContentSource;
  state?: string;
  logVerbose?: boolean;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the Sku class.
 * @constructor
 * The account SKU.
 *
 * @member {string} name Gets or sets the SKU name of the account. Possible
 * values include: 'Free', 'Basic'
 * @member {string} [family] Gets or sets the SKU family.
 * @member {number} [capacity] Gets or sets the SKU capacity.
 */
export interface Sku {
  name: string;
  family?: string;
  capacity?: number;
}

/**
 * @class
 * Initializes a new instance of the AutomationAccount class.
 * @constructor
 * Definition of the automation account type.
 *
 * @member {object} [sku] Gets or sets the SKU of account.
 * @member {string} [sku.name] Gets or sets the SKU name of the account.
 * Possible values include: 'Free', 'Basic'
 * @member {string} [sku.family] Gets or sets the SKU family.
 * @member {number} [sku.capacity] Gets or sets the SKU capacity.
 * @member {string} [lastModifiedBy] Gets or sets the last modified by.
 * @member {string} [state] Gets status of account. Possible values include:
 * 'Ok', 'Unavailable', 'Suspended'
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 * @member {string} [etag] Gets or sets the etag of the resource.
 */
export interface AutomationAccount extends Resource {
  sku?: Sku;
  lastModifiedBy?: string;
  readonly state?: string;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
  etag?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Provider, Resource and Operation values
 *
 * @member {string} [provider] Service provider: Microsoft.Automation
 * @member {string} [resource] Resource on which the operation is performed:
 * Runbooks, Jobs etc.
 * @member {string} [operation] Operation type: Read, write, delete, etc.
 */
export interface OperationDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Automation REST API operation
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}
 * @member {object} [display] Provider, Resource and Operation values
 * @member {string} [display.provider] Service provider: Microsoft.Automation
 * @member {string} [display.resource] Resource on which the operation is
 * performed: Runbooks, Jobs etc.
 * @member {string} [display.operation] Operation type: Read, write, delete,
 * etc.
 */
export interface Operation {
  name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the AutomationAccountCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update automation account
 * operation.
 *
 * @member {object} [sku] Gets or sets account SKU.
 * @member {string} [sku.name] Gets or sets the SKU name of the account.
 * Possible values include: 'Free', 'Basic'
 * @member {string} [sku.family] Gets or sets the SKU family.
 * @member {number} [sku.capacity] Gets or sets the SKU capacity.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface AutomationAccountCreateOrUpdateParameters {
  sku?: Sku;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the AutomationAccountUpdateParameters class.
 * @constructor
 * The parameters supplied to the update automation account operation.
 *
 * @member {object} [sku] Gets or sets account SKU.
 * @member {string} [sku.name] Gets or sets the SKU name of the account.
 * Possible values include: 'Free', 'Basic'
 * @member {string} [sku.family] Gets or sets the SKU family.
 * @member {number} [sku.capacity] Gets or sets the SKU capacity.
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface AutomationAccountUpdateParameters {
  sku?: Sku;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Certificate class.
 * @constructor
 * Definition of the certificate.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {string} [name] Gets the name of the certificate.
 * @member {string} [thumbprint] Gets the thumbprint of the certificate.
 * @member {date} [expiryTime] Gets the expiry time of the certificate.
 * @member {boolean} [isExportable] Gets the is exportable flag of the
 * certificate.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Certificate {
  readonly id?: string;
  readonly name?: string;
  readonly thumbprint?: string;
  readonly expiryTime?: Date;
  readonly isExportable?: boolean;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the CertificateUpdateParameters class.
 * @constructor
 * The parameters supplied to the update certificate operation.
 *
 * @member {string} name Gets or sets the name of the certificate.
 * @member {string} [description] Gets or sets the description of the
 * certificate.
 */
export interface CertificateUpdateParameters {
  name: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the CertificateCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update or replace certificate
 * operation.
 *
 * @member {string} name Gets or sets the name of the certificate.
 * @member {string} base64Value Gets or sets the base64 encoded value of the
 * certificate.
 * @member {string} [description] Gets or sets the description of the
 * certificate.
 * @member {string} [thumbprint] Gets or sets the thumbprint of the
 * certificate.
 * @member {boolean} [isExportable] Gets or sets the is exportable flag of the
 * certificate.
 */
export interface CertificateCreateOrUpdateParameters {
  name: string;
  base64Value: string;
  description?: string;
  thumbprint?: string;
  isExportable?: boolean;
}

/**
 * @class
 * Initializes a new instance of the ConnectionTypeAssociationProperty class.
 * @constructor
 * The connection type property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the connection type.
 */
export interface ConnectionTypeAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the Connection class.
 * @constructor
 * Definition of the connection.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {string} [name] Gets the name of the connection.
 * @member {object} [connectionType] Gets or sets the connectionType of the
 * connection.
 * @member {string} [connectionType.name] Gets or sets the name of the
 * connection type.
 * @member {object} [fieldDefinitionValues] Gets the field definition values of
 * the connection.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Connection {
  readonly id?: string;
  readonly name?: string;
  connectionType?: ConnectionTypeAssociationProperty;
  readonly fieldDefinitionValues?: { [propertyName: string]: string };
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionUpdateParameters class.
 * @constructor
 * The parameters supplied to the update connection operation.
 *
 * @member {string} [name] Gets or sets the name of the connection.
 * @member {string} [description] Gets or sets the description of the
 * connection.
 * @member {object} [fieldDefinitionValues] Gets or sets the field definition
 * values of the connection.
 */
export interface ConnectionUpdateParameters {
  name?: string;
  description?: string;
  fieldDefinitionValues?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ConnectionCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update connection operation.
 *
 * @member {string} name Gets or sets the name of the connection.
 * @member {string} [description] Gets or sets the description of the
 * connection.
 * @member {object} connectionType Gets or sets the connectionType of the
 * connection.
 * @member {string} [connectionType.name] Gets or sets the name of the
 * connection type.
 * @member {object} [fieldDefinitionValues] Gets or sets the field definition
 * properties of the connection.
 */
export interface ConnectionCreateOrUpdateParameters {
  name: string;
  description?: string;
  connectionType: ConnectionTypeAssociationProperty;
  fieldDefinitionValues?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the FieldDefinition class.
 * @constructor
 * Definition of the connection fields.
 *
 * @member {boolean} [isEncrypted] Gets or sets the isEncrypted flag of the
 * connection field definition.
 * @member {boolean} [isOptional] Gets or sets the isOptional flag of the
 * connection field definition.
 * @member {string} type Gets or sets the type of the connection field
 * definition.
 */
export interface FieldDefinition {
  isEncrypted?: boolean;
  isOptional?: boolean;
  type: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionType class.
 * @constructor
 * Definition of the connection type.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {string} [name] Gets the name of the connection type.
 * @member {boolean} [isGlobal] Gets or sets a Boolean value to indicate if the
 * connection type is global.
 * @member {object} [fieldDefinitions] Gets the field definitions of the
 * connection type.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface ConnectionType {
  readonly id?: string;
  readonly name?: string;
  isGlobal?: boolean;
  readonly fieldDefinitions?: { [propertyName: string]: FieldDefinition };
  readonly creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionTypeCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update connection type operation.
 *
 * @member {string} name Gets or sets the name of the connection type.
 * @member {boolean} [isGlobal] Gets or sets a Boolean value to indicate if the
 * connection type is global.
 * @member {object} fieldDefinitions Gets or sets the field definitions of the
 * connection type.
 */
export interface ConnectionTypeCreateOrUpdateParameters {
  name: string;
  isGlobal?: boolean;
  fieldDefinitions: { [propertyName: string]: FieldDefinition };
}

/**
 * @class
 * Initializes a new instance of the Credential class.
 * @constructor
 * Definition of the credential.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {string} [name] Gets the name of the credential.
 * @member {string} [userName] Gets the user name of the credential.
 * @member {date} [creationTime] Gets the creation time.
 * @member {date} [lastModifiedTime] Gets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Credential {
  readonly id?: string;
  name?: string;
  readonly userName?: string;
  readonly creationTime?: Date;
  readonly lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the CredentialUpdateParameters class.
 * @constructor
 * The parameters supplied to the Update credential operation.
 *
 * @member {string} name Gets or sets the name of the credential.
 * @member {string} [userName] Gets or sets the user name of the credential.
 * @member {string} [password] Gets or sets the password of the credential.
 * @member {string} [description] Gets or sets the description of the
 * credential.
 */
export interface CredentialUpdateParameters {
  name: string;
  userName?: string;
  password?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the CredentialCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update credential operation.
 *
 * @member {string} name Gets or sets the name of the credential.
 * @member {string} userName Gets or sets the user name of the credential.
 * @member {string} password Gets or sets the password of the credential.
 * @member {string} [description] Gets or sets the description of the
 * credential.
 */
export interface CredentialCreateOrUpdateParameters {
  name: string;
  userName: string;
  password: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityParameter class.
 * @constructor
 * Definition of the activity parameter.
 *
 * @member {string} [name] Gets or sets the name of the activity parameter.
 * @member {string} [type] Gets or sets the type of the activity parameter.
 * @member {boolean} [isMandatory] Gets or sets a Boolean value that indicates
 * true if the parameter is required. If the value is false, the parameter is
 * optional.
 * @member {boolean} [isDynamic] Gets or sets a Boolean value that indicates
 * true if the parameter is dynamic.
 * @member {boolean} [position] Gets or sets the position of the activity
 * parameter.
 * @member {boolean} [valueFromPipeline] Gets or sets a Boolean value that
 * indicates true if the parameter can take values from the incoming pipeline
 * objects. This setting is used if the cmdlet must access the complete input
 * object. false indicates that the parameter cannot take values from the
 * complete input object.
 * @member {boolean} [valueFromPipelineByPropertyName] Gets or sets a Boolean
 * value that indicates true if the parameter can be filled from a property of
 * the incoming pipeline object that has the same name as this parameter. false
 * indicates that the parameter cannot be filled from the incoming pipeline
 * object property with the same name.
 * @member {boolean} [valueFromRemainingArguments] Gets or sets a Boolean value
 * that indicates true if the cmdlet parameter accepts all the remaining
 * command-line arguments that are associated with this parameter in the form
 * of an array. false if the cmdlet parameter does not accept all the remaining
 * argument values.
 */
export interface ActivityParameter {
  name?: string;
  type?: string;
  isMandatory?: boolean;
  isDynamic?: boolean;
  position?: boolean;
  valueFromPipeline?: boolean;
  valueFromPipelineByPropertyName?: boolean;
  valueFromRemainingArguments?: boolean;
}

/**
 * @class
 * Initializes a new instance of the ActivityParameterSet class.
 * @constructor
 * Definition of the activity parameter set.
 *
 * @member {string} [name] Gets or sets the name of the activity parameter set.
 * @member {array} [parameters] Gets or sets the parameters of the activity
 * parameter set.
 */
export interface ActivityParameterSet {
  name?: string;
  parameters?: ActivityParameter[];
}

/**
 * @class
 * Initializes a new instance of the ActivityOutputType class.
 * @constructor
 * Definition of the activity output type.
 *
 * @member {string} [name] Gets or sets the name of the activity output type.
 * @member {string} [type] Gets or sets the type of the activity output type.
 */
export interface ActivityOutputType {
  name?: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the Activity class.
 * @constructor
 * Definition of the activity.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets the name of the activity.
 * @member {string} [definition] Gets or sets the user name of the activity.
 * @member {array} [parameterSets] Gets or sets the parameter sets of the
 * activity.
 * @member {array} [outputTypes] Gets or sets the output types of the activity.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Activity {
  id?: string;
  readonly name?: string;
  definition?: string;
  parameterSets?: ActivityParameterSet[];
  outputTypes?: ActivityOutputType[];
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the AdvancedScheduleMonthlyOccurrence class.
 * @constructor
 * The properties of the create advanced schedule monthly occurrence.
 *
 * @member {number} [occurrence] Occurrence of the week within the month. Must
 * be between 1 and 5
 * @member {string} [day] Day of the occurrence. Must be one of monday,
 * tuesday, wednesday,thursday, friday, saturday, sunday. Possible values
 * include: 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday',
 * 'Sunday'
 */
export interface AdvancedScheduleMonthlyOccurrence {
  occurrence?: number;
  day?: string;
}

/**
 * @class
 * Initializes a new instance of the AdvancedSchedule class.
 * @constructor
 * The properties of the create Advanced Schedule.
 *
 * @member {array} [weekDays] Days of the week that the job should execute on.
 * @member {array} [monthDays] Days of the month that the job should execute
 * on. Must be between 1 and 31.
 * @member {array} [monthlyOccurrences] Occurrences of days within a month.
 */
export interface AdvancedSchedule {
  weekDays?: string[];
  monthDays?: number[];
  monthlyOccurrences?: AdvancedScheduleMonthlyOccurrence[];
}

/**
 * @class
 * Initializes a new instance of the AgentRegistrationKeys class.
 * @constructor
 * Definition of the agent registration keys.
 *
 * @member {string} [primary] Gets or sets the primary key.
 * @member {string} [secondary] Gets or sets the secondary key.
 */
export interface AgentRegistrationKeys {
  primary?: string;
  secondary?: string;
}

/**
 * @class
 * Initializes a new instance of the AgentRegistration class.
 * @constructor
 * Definition of the agent registration infomration type.
 *
 * @member {string} [dscMetaConfiguration] Gets or sets the dsc meta
 * configuration.
 * @member {string} [endpoint] Gets or sets the dsc server endpoint.
 * @member {object} [keys] Gets or sets the agent registration keys.
 * @member {string} [keys.primary] Gets or sets the primary key.
 * @member {string} [keys.secondary] Gets or sets the secondary key.
 * @member {string} [id] Gets or sets the id.
 */
export interface AgentRegistration {
  dscMetaConfiguration?: string;
  endpoint?: string;
  keys?: AgentRegistrationKeys;
  id?: string;
}

/**
 * @class
 * Initializes a new instance of the AgentRegistrationRegenerateKeyParameter class.
 * @constructor
 * The parameters supplied to the regenerate keys operation.
 *
 * @member {string} keyName Gets or sets the agent registration key name -
 * Primary or Secondary. Possible values include: 'Primary', 'Secondary'
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface AgentRegistrationRegenerateKeyParameter {
  keyName: string;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationAssociationProperty class.
 * @constructor
 * The Dsc configuration property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the Dsc configuration.
 */
export interface DscConfigurationAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the DscCompilationJobCreateParameters class.
 * @constructor
 * The parameters supplied to the create compilation job operation.
 *
 * @member {object} configuration Gets or sets the configuration.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface DscCompilationJobCreateParameters {
  configuration: DscConfigurationAssociationProperty;
  parameters?: { [propertyName: string]: string };
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscCompilationJob class.
 * @constructor
 * Definition of the Dsc Compilation job.
 *
 * @member {string} [id] Gets the id of the resource.
 * @member {object} [configuration] Gets or sets the configuration.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {string} [startedBy] Gets the compilation job started by.
 * @member {uuid} [jobId] Gets the id of the job.
 * @member {date} [creationTime] Gets the creation time of the job.
 * @member {string} [status] Gets or sets the status of the job. Possible
 * values include: 'New', 'Activating', 'Running', 'Completed', 'Failed',
 * 'Stopped', 'Blocked', 'Suspended', 'Disconnected', 'Suspending', 'Stopping',
 * 'Resuming', 'Removing'
 * @member {string} [statusDetails] Gets or sets the status details of the job.
 * @member {date} [startTime] Gets the start time of the job.
 * @member {date} [endTime] Gets the end time of the job.
 * @member {string} [exception] Gets the exception of the job.
 * @member {date} [lastModifiedTime] Gets the last modified time of the job.
 * @member {date} [lastStatusModifiedTime] Gets the last status modified time
 * of the job.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 */
export interface DscCompilationJob {
  readonly id?: string;
  configuration?: DscConfigurationAssociationProperty;
  readonly startedBy?: string;
  readonly jobId?: string;
  readonly creationTime?: Date;
  status?: string;
  statusDetails?: string;
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly exception?: string;
  readonly lastModifiedTime?: Date;
  readonly lastStatusModifiedTime?: Date;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update configuration operation.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {object} source Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {object} [parameters] Gets or sets the configuration parameters.
 * @member {string} [description] Gets or sets the description of the
 * configuration.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface DscConfigurationCreateOrUpdateParameters {
  logVerbose?: boolean;
  logProgress?: boolean;
  source: ContentSource;
  parameters?: { [propertyName: string]: DscConfigurationParameter };
  description?: string;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DscMetaConfiguration class.
 * @constructor
 * Definition of the DSC Meta Configuration.
 *
 * @member {number} [configurationModeFrequencyMins] Gets or sets the
 * ConfigurationModeFrequencyMins value of the meta configuration.
 * @member {boolean} [rebootNodeIfNeeded] Gets or sets the RebootNodeIfNeeded
 * value of the meta configuration.
 * @member {string} [configurationMode] Gets or sets the ConfigurationMode
 * value of the meta configuration.
 * @member {string} [actionAfterReboot] Gets or sets the ActionAfterReboot
 * value of the meta configuration.
 * @member {string} [certificateId] Gets or sets the CertificateId value of the
 * meta configuration.
 * @member {number} [refreshFrequencyMins] Gets or sets the
 * RefreshFrequencyMins value of the meta configuration.
 * @member {boolean} [allowModuleOverwrite] Gets or sets the
 * AllowModuleOverwrite value of the meta configuration.
 */
export interface DscMetaConfiguration {
  configurationModeFrequencyMins?: number;
  rebootNodeIfNeeded?: boolean;
  configurationMode?: string;
  actionAfterReboot?: string;
  certificateId?: string;
  refreshFrequencyMins?: number;
  allowModuleOverwrite?: boolean;
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfigurationCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update node configuration
 * operation.
 *
 * @member {object} source Gets or sets the source.
 * @member {object} [source.hash] Gets or sets the hash.
 * @member {string} [source.hash.algorithm] Gets or sets the content hash
 * algorithm used to hash the content.
 * @member {string} [source.hash.value] Gets or sets expected hash value of the
 * content.
 * @member {string} [source.type] Gets or sets the content source type.
 * Possible values include: 'embeddedContent', 'uri'
 * @member {string} [source.value] Gets or sets the value of the content. This
 * is based on the content source type.
 * @member {string} [source.version] Gets or sets the version of the content.
 * @member {string} name Gets or sets the type of the parameter.
 * @member {object} configuration Gets or sets the configuration of the node.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 */
export interface DscNodeConfigurationCreateOrUpdateParameters {
  source: ContentSource;
  name: string;
  configuration: DscConfigurationAssociationProperty;
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfiguration class.
 * @constructor
 * Definition of the dsc node configuration.
 *
 * @member {string} [name] Gets or sets the node configuration name.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {date} [creationTime] Gets or sets creation time.
 * @member {object} [configuration] Gets or sets the configuration of the node.
 * @member {string} [configuration.name] Gets or sets the name of the Dsc
 * configuration.
 * @member {string} [id] Gets or sets the id of the resource.
 */
export interface DscNodeConfiguration {
  name?: string;
  lastModifiedTime?: Date;
  creationTime?: Date;
  configuration?: DscConfigurationAssociationProperty;
  id?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeUpdateParameters class.
 * @constructor
 * The parameters supplied to the update dsc node operation.
 *
 * @member {string} [nodeId] Gets or sets the id of the dsc node.
 * @member {object} [nodeConfiguration] Gets or sets the configuration of the
 * node.
 * @member {string} [nodeConfiguration.name] Gets or sets the name of the dsc
 * nodeconfiguration.
 */
export interface DscNodeUpdateParameters {
  nodeId?: string;
  nodeConfiguration?: DscNodeConfigurationAssociationProperty;
}

/**
 * @class
 * Initializes a new instance of the DscReportError class.
 * @constructor
 * Definition of the dsc node report error type.
 *
 * @member {string} [errorSource] Gets or sets the source of the error.
 * @member {string} [resourceId] Gets or sets the resource ID which generated
 * the error.
 * @member {string} [errorCode] Gets or sets the error code.
 * @member {string} [errorMessage] Gets or sets the error message.
 * @member {string} [locale] Gets or sets the locale of the error.
 * @member {string} [errorDetails] Gets or sets the error details.
 */
export interface DscReportError {
  errorSource?: string;
  resourceId?: string;
  errorCode?: string;
  errorMessage?: string;
  locale?: string;
  errorDetails?: string;
}

/**
 * @class
 * Initializes a new instance of the DscReportResourceNavigation class.
 * @constructor
 * Navigation for DSC Report Resource.
 *
 * @member {string} [resourceId] Gets or sets the ID of the resource to
 * navigate to.
 */
export interface DscReportResourceNavigation {
  resourceId?: string;
}

/**
 * @class
 * Initializes a new instance of the DscReportResource class.
 * @constructor
 * Definition of the DSC Report Resource.
 *
 * @member {string} [resourceId] Gets or sets the ID of the resource.
 * @member {string} [sourceInfo] Gets or sets the source info of the resource.
 * @member {array} [dependsOn] Gets or sets the Resource Navigation values for
 * resources the resource depends on.
 * @member {string} [moduleName] Gets or sets the module name of the resource.
 * @member {string} [moduleVersion] Gets or sets the module version of the
 * resource.
 * @member {string} [resourceName] Gets or sets the name of the resource.
 * @member {string} [error] Gets or sets the error of the resource.
 * @member {string} [status] Gets or sets the status of the resource.
 * @member {number} [durationInSeconds] Gets or sets the duration in seconds
 * for the resource.
 * @member {date} [startDate] Gets or sets the start date of the resource.
 */
export interface DscReportResource {
  resourceId?: string;
  sourceInfo?: string;
  dependsOn?: DscReportResourceNavigation[];
  moduleName?: string;
  moduleVersion?: string;
  resourceName?: string;
  error?: string;
  status?: string;
  durationInSeconds?: number;
  startDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the DscNodeReport class.
 * @constructor
 * Definition of the dsc node report type.
 *
 * @member {date} [endTime] Gets or sets the end time of the node report.
 * @member {date} [lastModifiedTime] Gets or sets the lastModifiedTime of the
 * node report.
 * @member {date} [startTime] Gets or sets the start time of the node report.
 * @member {string} [type] Gets or sets the type of the node report.
 * @member {string} [reportId] Gets or sets the id of the node report.
 * @member {string} [status] Gets or sets the status of the node report.
 * @member {string} [refreshMode] Gets or sets the refreshMode of the node
 * report.
 * @member {string} [rebootRequested] Gets or sets the rebootRequested of the
 * node report.
 * @member {string} [reportFormatVersion] Gets or sets the reportFormatVersion
 * of the node report.
 * @member {string} [configurationVersion] Gets or sets the
 * configurationVersion of the node report.
 * @member {string} [id] Gets or sets the id.
 * @member {array} [errors] Gets or sets the errors for the node report.
 * @member {array} [resources] Gets or sets the resource for the node report.
 * @member {object} [metaConfiguration] Gets or sets the metaConfiguration of
 * the node at the time of the report.
 * @member {number} [metaConfiguration.configurationModeFrequencyMins] Gets or
 * sets the ConfigurationModeFrequencyMins value of the meta configuration.
 * @member {boolean} [metaConfiguration.rebootNodeIfNeeded] Gets or sets the
 * RebootNodeIfNeeded value of the meta configuration.
 * @member {string} [metaConfiguration.configurationMode] Gets or sets the
 * ConfigurationMode value of the meta configuration.
 * @member {string} [metaConfiguration.actionAfterReboot] Gets or sets the
 * ActionAfterReboot value of the meta configuration.
 * @member {string} [metaConfiguration.certificateId] Gets or sets the
 * CertificateId value of the meta configuration.
 * @member {number} [metaConfiguration.refreshFrequencyMins] Gets or sets the
 * RefreshFrequencyMins value of the meta configuration.
 * @member {boolean} [metaConfiguration.allowModuleOverwrite] Gets or sets the
 * AllowModuleOverwrite value of the meta configuration.
 * @member {string} [hostName] Gets or sets the hostname of the node that sent
 * the report.
 * @member {array} [iPV4Addresses] Gets or sets the IPv4 address of the node
 * that sent the report.
 * @member {array} [iPV6Addresses] Gets or sets the IPv6 address of the node
 * that sent the report.
 * @member {number} [numberOfResources] Gets or sets the number of resource in
 * the node report.
 * @member {string} [rawErrors] Gets or sets the unparsed errors for the node
 * report.
 */
export interface DscNodeReport {
  endTime?: Date;
  lastModifiedTime?: Date;
  startTime?: Date;
  type?: string;
  reportId?: string;
  status?: string;
  refreshMode?: string;
  rebootRequested?: string;
  reportFormatVersion?: string;
  configurationVersion?: string;
  id?: string;
  errors?: DscReportError[];
  resources?: DscReportResource[];
  metaConfiguration?: DscMetaConfiguration;
  hostName?: string;
  iPV4Addresses?: string[];
  iPV6Addresses?: string[];
  numberOfResources?: number;
  rawErrors?: string;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorker class.
 * @constructor
 * Definition of hybrid runbook worker.
 *
 * @member {string} [name] Gets or sets the worker machine name.
 * @member {string} [ip] Gets or sets the assigned machine IP address.
 * @member {date} [registrationTime] Gets or sets the registration time of the
 * worker machine.
 */
export interface HybridRunbookWorker {
  name?: string;
  ip?: string;
  registrationTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the RunAsCredentialAssociationProperty class.
 * @constructor
 * Definition of runas credential to use for hybrid worker.
 *
 * @member {string} [name] Gets or sets the name of the credential.
 */
export interface RunAsCredentialAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorkerGroup class.
 * @constructor
 * Definition of hybrid runbook worker group.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets or sets the name of the group.
 * @member {array} [hybridRunbookWorkers] Gets or sets the list of hybrid
 * runbook workers.
 * @member {object} [credential] Sets the credential of a worker group.
 * @member {string} [credential.name] Gets or sets the name of the credential.
 */
export interface HybridRunbookWorkerGroup {
  id?: string;
  name?: string;
  hybridRunbookWorkers?: HybridRunbookWorker[];
  credential?: RunAsCredentialAssociationProperty;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorkerGroupUpdateParameters class.
 * @constructor
 * Parameters supplied to the update operation.
 *
 * @member {object} [credential] Sets the credential of a worker group.
 * @member {string} [credential.name] Gets or sets the name of the credential.
 */
export interface HybridRunbookWorkerGroupUpdateParameters {
  credential?: RunAsCredentialAssociationProperty;
}

/**
 * @class
 * Initializes a new instance of the RunbookAssociationProperty class.
 * @constructor
 * The runbook property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the runbook.
 */
export interface RunbookAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the Job class.
 * @constructor
 * Definition of the job.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [startedBy] Gets or sets the job started by.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 * @member {uuid} [jobId] Gets or sets the id of the job.
 * @member {date} [creationTime] Gets or sets the creation time of the job.
 * @member {string} [status] Gets or sets the status of the job. Possible
 * values include: 'New', 'Activating', 'Running', 'Completed', 'Failed',
 * 'Stopped', 'Blocked', 'Suspended', 'Disconnected', 'Suspending', 'Stopping',
 * 'Resuming', 'Removing'
 * @member {string} [statusDetails] Gets or sets the status details of the job.
 * @member {date} [startTime] Gets or sets the start time of the job.
 * @member {date} [endTime] Gets or sets the end time of the job.
 * @member {string} [exception] Gets or sets the exception of the job.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time of the
 * job.
 * @member {date} [lastStatusModifiedTime] Gets or sets the last status
 * modified time of the job.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 */
export interface Job {
  id?: string;
  runbook?: RunbookAssociationProperty;
  startedBy?: string;
  runOn?: string;
  jobId?: string;
  creationTime?: Date;
  status?: string;
  statusDetails?: string;
  startTime?: Date;
  endTime?: Date;
  exception?: string;
  lastModifiedTime?: Date;
  lastStatusModifiedTime?: Date;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the JobCreateParameters class.
 * @constructor
 * The parameters supplied to the create job operation.
 *
 * @member {object} runbook Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface JobCreateParameters {
  runbook: RunbookAssociationProperty;
  parameters?: { [propertyName: string]: string };
  runOn?: string;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ScheduleAssociationProperty class.
 * @constructor
 * The schedule property associated with the entity.
 *
 * @member {string} [name] Gets or sets the name of the schedule.
 */
export interface ScheduleAssociationProperty {
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the JobScheduleCreateParameters class.
 * @constructor
 * The parameters supplied to the create job schedule operation.
 *
 * @member {object} schedule Gets or sets the schedule.
 * @member {string} [schedule.name] Gets or sets the name of the schedule.
 * @member {object} runbook Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the hybrid worker group that the
 * scheduled job should run on.
 * @member {object} [parameters] Gets or sets a list of job properties.
 */
export interface JobScheduleCreateParameters {
  schedule: ScheduleAssociationProperty;
  runbook: RunbookAssociationProperty;
  runOn?: string;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the JobSchedule class.
 * @constructor
 * Definition of the job schedule.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [jobScheduleId] Gets or sets the id of job schedule.
 * @member {object} [schedule] Gets or sets the schedule.
 * @member {string} [schedule.name] Gets or sets the name of the schedule.
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the hybrid worker group that the
 * scheduled job should run on.
 * @member {object} [parameters] Gets or sets the parameters of the job
 * schedule.
 */
export interface JobSchedule {
  id?: string;
  jobScheduleId?: string;
  schedule?: ScheduleAssociationProperty;
  runbook?: RunbookAssociationProperty;
  runOn?: string;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the JobStream class.
 * @constructor
 * Definition of the job stream.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [jobStreamId] Gets or sets the id of the job stream.
 * @member {date} [time] Gets or sets the creation time of the job.
 * @member {string} [streamType] Gets or sets the stream type. Possible values
 * include: 'Progress', 'Output', 'Warning', 'Error', 'Debug', 'Verbose', 'Any'
 * @member {string} [streamText] Gets or sets the stream text.
 * @member {string} [summary] Gets or sets the summary.
 * @member {object} [value] Gets or sets the values of the job stream.
 */
export interface JobStream {
  id?: string;
  jobStreamId?: string;
  time?: Date;
  streamType?: string;
  streamText?: string;
  summary?: string;
  value?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the ModuleCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update module operation.
 *
 * @member {object} contentLink Gets or sets the module content link.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface ModuleCreateOrUpdateParameters {
  contentLink: ContentLink;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ModuleUpdateParameters class.
 * @constructor
 * The parameters supplied to the update module operation.
 *
 * @member {object} [contentLink] Gets or sets the module content link.
 * @member {string} [contentLink.uri] Gets or sets the uri of the runbook
 * content.
 * @member {object} [contentLink.contentHash] Gets or sets the hash.
 * @member {string} [contentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [contentLink.contentHash.value] Gets or sets expected hash
 * value of the content.
 * @member {string} [contentLink.version] Gets or sets the version of the
 * content.
 * @member {string} [name] Gets or sets name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface ModuleUpdateParameters {
  contentLink?: ContentLink;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the RunbookDraftUndoEditResult class.
 * @constructor
 * The response model for the undoedit runbook operation.
 *
 * @member {string} [statusCode] Possible values include: 'Continue',
 * 'SwitchingProtocols', 'OK', 'Created', 'Accepted',
 * 'NonAuthoritativeInformation', 'NoContent', 'ResetContent',
 * 'PartialContent', 'MultipleChoices', 'Ambiguous', 'MovedPermanently',
 * 'Moved', 'Found', 'Redirect', 'SeeOther', 'RedirectMethod', 'NotModified',
 * 'UseProxy', 'Unused', 'TemporaryRedirect', 'RedirectKeepVerb', 'BadRequest',
 * 'Unauthorized', 'PaymentRequired', 'Forbidden', 'NotFound',
 * 'MethodNotAllowed', 'NotAcceptable', 'ProxyAuthenticationRequired',
 * 'RequestTimeout', 'Conflict', 'Gone', 'LengthRequired',
 * 'PreconditionFailed', 'RequestEntityTooLarge', 'RequestUriTooLong',
 * 'UnsupportedMediaType', 'RequestedRangeNotSatisfiable', 'ExpectationFailed',
 * 'UpgradeRequired', 'InternalServerError', 'NotImplemented', 'BadGateway',
 * 'ServiceUnavailable', 'GatewayTimeout', 'HttpVersionNotSupported'
 * @member {string} [requestId]
 */
export interface RunbookDraftUndoEditResult {
  statusCode?: string;
  requestId?: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update runbook operation.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {string} runbookType Gets or sets the type of the runbook. Possible
 * values include: 'Script', 'Graph', 'PowerShellWorkflow', 'PowerShell',
 * 'GraphPowerShellWorkflow', 'GraphPowerShell'
 * @member {object} [draft] Gets or sets the draft runbook properties.
 * @member {boolean} [draft.inEdit] Gets or sets whether runbook is in edit
 * mode.
 * @member {object} [draft.draftContentLink] Gets or sets the draft runbook
 * content link.
 * @member {string} [draft.draftContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [draft.draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draft.draftContentLink.contentHash.algorithm] Gets or sets
 * the content hash algorithm used to hash the content.
 * @member {string} [draft.draftContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [draft.draftContentLink.version] Gets or sets the version
 * of the content.
 * @member {date} [draft.creationTime] Gets or sets the creation time of the
 * runbook draft.
 * @member {date} [draft.lastModifiedTime] Gets or sets the last modified time
 * of the runbook draft.
 * @member {object} [draft.parameters] Gets or sets the runbook draft
 * parameters.
 * @member {array} [draft.outputTypes] Gets or sets the runbook output types.
 * @member {object} [publishContentLink] Gets or sets the published runbook
 * content link.
 * @member {string} [publishContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [publishContentLink.contentHash] Gets or sets the hash.
 * @member {string} [publishContentLink.contentHash.algorithm] Gets or sets the
 * content hash algorithm used to hash the content.
 * @member {string} [publishContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [publishContentLink.version] Gets or sets the version of
 * the content.
 * @member {string} [description] Gets or sets the description of the runbook.
 * @member {number} [logActivityTrace] Gets or sets the activity-level tracing
 * options of the runbook.
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface RunbookCreateOrUpdateParameters {
  logVerbose?: boolean;
  logProgress?: boolean;
  runbookType: string;
  draft?: RunbookDraft;
  publishContentLink?: ContentLink;
  description?: string;
  logActivityTrace?: number;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the RunbookCreateOrUpdateDraftProperties class.
 * @constructor
 * The parameters supplied to the create or update dratft runbook properties.
 *
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {string} runbookType Gets or sets the type of the runbook. Possible
 * values include: 'Script', 'Graph', 'PowerShellWorkflow', 'PowerShell',
 * 'GraphPowerShellWorkflow', 'GraphPowerShell'
 * @member {object} draft Gets or sets the draft runbook properties.
 * @member {boolean} [draft.inEdit] Gets or sets whether runbook is in edit
 * mode.
 * @member {object} [draft.draftContentLink] Gets or sets the draft runbook
 * content link.
 * @member {string} [draft.draftContentLink.uri] Gets or sets the uri of the
 * runbook content.
 * @member {object} [draft.draftContentLink.contentHash] Gets or sets the hash.
 * @member {string} [draft.draftContentLink.contentHash.algorithm] Gets or sets
 * the content hash algorithm used to hash the content.
 * @member {string} [draft.draftContentLink.contentHash.value] Gets or sets
 * expected hash value of the content.
 * @member {string} [draft.draftContentLink.version] Gets or sets the version
 * of the content.
 * @member {date} [draft.creationTime] Gets or sets the creation time of the
 * runbook draft.
 * @member {date} [draft.lastModifiedTime] Gets or sets the last modified time
 * of the runbook draft.
 * @member {object} [draft.parameters] Gets or sets the runbook draft
 * parameters.
 * @member {array} [draft.outputTypes] Gets or sets the runbook output types.
 * @member {string} [description] Gets or sets the description of the runbook.
 * @member {number} [logActivityTrace] Gets or sets the activity-level tracing
 * options of the runbook.
 */
export interface RunbookCreateOrUpdateDraftProperties {
  logVerbose?: boolean;
  logProgress?: boolean;
  runbookType: string;
  draft: RunbookDraft;
  description?: string;
  logActivityTrace?: number;
}

/**
 * @class
 * Initializes a new instance of the RunbookCreateOrUpdateDraftParameters class.
 * @constructor
 * The parameters supplied to the create or update runbook operation.
 *
 * @member {string} runbookContent Content of the Runbook.
 */
export interface RunbookCreateOrUpdateDraftParameters {
  runbookContent: string;
}

/**
 * @class
 * Initializes a new instance of the RunbookUpdateParameters class.
 * @constructor
 * The parameters supplied to the update runbook operation.
 *
 * @member {string} [description] Gets or sets the description of the runbook.
 * @member {boolean} [logVerbose] Gets or sets verbose log option.
 * @member {boolean} [logProgress] Gets or sets progress log option.
 * @member {number} [logActivityTrace] Gets or sets the activity-level tracing
 * options of the runbook.
 * @member {string} [name] Gets or sets the name of the resource.
 * @member {string} [location] Gets or sets the location of the resource.
 * @member {object} [tags] Gets or sets the tags attached to the resource.
 */
export interface RunbookUpdateParameters {
  description?: string;
  logVerbose?: boolean;
  logProgress?: boolean;
  logActivityTrace?: number;
  name?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ScheduleCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update schedule operation.
 *
 * @member {string} name Gets or sets the name of the schedule.
 * @member {string} [description] Gets or sets the description of the schedule.
 * @member {date} startTime Gets or sets the start time of the schedule.
 * @member {date} [expiryTime] Gets or sets the end time of the schedule.
 * @member {object} [interval] Gets or sets the interval of the schedule.
 * @member {string} frequency Gets or sets the frequency of the schedule.
 * Possible values include: 'OneTime', 'Day', 'Hour', 'Week', 'Month'
 * @member {string} [timeZone] Gets or sets the time zone of the schedule.
 * @member {object} [advancedSchedule] Gets or sets the AdvancedSchedule.
 * @member {array} [advancedSchedule.weekDays] Days of the week that the job
 * should execute on.
 * @member {array} [advancedSchedule.monthDays] Days of the month that the job
 * should execute on. Must be between 1 and 31.
 * @member {array} [advancedSchedule.monthlyOccurrences] Occurrences of days
 * within a month.
 */
export interface ScheduleCreateOrUpdateParameters {
  name: string;
  description?: string;
  startTime: Date;
  expiryTime?: Date;
  interval?: any;
  frequency: string;
  timeZone?: string;
  advancedSchedule?: AdvancedSchedule;
}

/**
 * @class
 * Initializes a new instance of the Schedule class.
 * @constructor
 * Definition of the schedule.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets or sets the name of the schedule.
 * @member {date} [startTime] Gets or sets the start time of the schedule.
 * @member {number} [startTimeOffsetMinutes] Gets the start time's offset in
 * minutes.
 * @member {date} [expiryTime] Gets or sets the end time of the schedule.
 * @member {number} [expiryTimeOffsetMinutes] Gets or sets the expiry time's
 * offset in minutes.
 * @member {boolean} [isEnabled] Gets or sets a value indicating whether this
 * schedule is enabled. Default value: false .
 * @member {date} [nextRun] Gets or sets the next run time of the schedule.
 * @member {number} [nextRunOffsetMinutes] Gets or sets the next run time's
 * offset in minutes.
 * @member {object} [interval] Gets or sets the interval of the schedule.
 * @member {string} [frequency] Gets or sets the frequency of the schedule.
 * Possible values include: 'OneTime', 'Day', 'Hour', 'Week', 'Month'
 * @member {string} [timeZone] Gets or sets the time zone of the schedule.
 * @member {object} [advancedSchedule] Gets or sets the advanced schedule.
 * @member {array} [advancedSchedule.weekDays] Days of the week that the job
 * should execute on.
 * @member {array} [advancedSchedule.monthDays] Days of the month that the job
 * should execute on. Must be between 1 and 31.
 * @member {array} [advancedSchedule.monthlyOccurrences] Occurrences of days
 * within a month.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Schedule {
  id?: string;
  name?: string;
  startTime?: Date;
  readonly startTimeOffsetMinutes?: number;
  expiryTime?: Date;
  expiryTimeOffsetMinutes?: number;
  isEnabled?: boolean;
  nextRun?: Date;
  nextRunOffsetMinutes?: number;
  interval?: any;
  frequency?: string;
  timeZone?: string;
  advancedSchedule?: AdvancedSchedule;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ScheduleUpdateParameters class.
 * @constructor
 * The parameters supplied to the update schedule operation.
 *
 * @member {string} name Gets or sets the name of the schedule.
 * @member {string} [description] Gets or sets the description of the schedule.
 * @member {boolean} [isEnabled] Gets or sets a value indicating whether this
 * schedule is enabled.
 */
export interface ScheduleUpdateParameters {
  name: string;
  description?: string;
  isEnabled?: boolean;
}

/**
 * @class
 * Initializes a new instance of the SubResource class.
 * @constructor
 * The Sub Resource definition.
 *
 * @member {string} [id] Resource Id
 */
export interface SubResource extends BaseResource {
  id?: string;
}

/**
 * @class
 * Initializes a new instance of the TestJobCreateParameters class.
 * @constructor
 * The parameters supplied to the create test job operation.
 *
 * @member {string} runbookName Gets or sets the runbook name.
 * @member {object} [parameters] Gets or sets the parameters of the test job.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 */
export interface TestJobCreateParameters {
  runbookName: string;
  parameters?: { [propertyName: string]: string };
  runOn?: string;
}

/**
 * @class
 * Initializes a new instance of the TestJob class.
 * @constructor
 * Definition of the test job.
 *
 * @member {date} [creationTime] Gets or sets the creation time of the test
 * job.
 * @member {string} [status] Gets or sets the status of the test job.
 * @member {string} [statusDetails] Gets or sets the status details of the test
 * job.
 * @member {string} [runOn] Gets or sets the runOn which specifies the group
 * name where the job is to be executed.
 * @member {date} [startTime] Gets or sets the start time of the test job.
 * @member {date} [endTime] Gets or sets the end time of the test job.
 * @member {string} [exception] Gets or sets the exception of the test job.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time of the
 * test job.
 * @member {date} [lastStatusModifiedTime] Gets or sets the last status
 * modified time of the test job.
 * @member {object} [parameters] Gets or sets the parameters of the test job.
 */
export interface TestJob {
  creationTime?: Date;
  status?: string;
  statusDetails?: string;
  runOn?: string;
  startTime?: Date;
  endTime?: Date;
  exception?: string;
  lastModifiedTime?: Date;
  lastStatusModifiedTime?: Date;
  parameters?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the TypeField class.
 * @constructor
 * Information about a field of a type.
 *
 * @member {string} [name] Gets or sets the name of the field.
 * @member {string} [type] Gets or sets the type of the field.
 */
export interface TypeField {
  name?: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the VariableCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update variable operation.
 *
 * @member {string} name Gets or sets the name of the variable.
 * @member {string} [value] Gets or sets the value of the variable.
 * @member {string} [description] Gets or sets the description of the variable.
 * @member {boolean} [isEncrypted] Gets or sets the encrypted flag of the
 * variable.
 */
export interface VariableCreateOrUpdateParameters {
  name: string;
  value?: string;
  description?: string;
  isEncrypted?: boolean;
}

/**
 * @class
 * Initializes a new instance of the Variable class.
 * @constructor
 * Definition of the varible.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets or sets the name of the variable.
 * @member {string} [value] Gets or sets the value of the variable.
 * @member {boolean} [isEncrypted] Gets or sets the encrypted flag of the
 * variable.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Variable {
  id?: string;
  name?: string;
  value?: string;
  isEncrypted?: boolean;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the VariableUpdateParameters class.
 * @constructor
 * The parameters supplied to the update variable operation.
 *
 * @member {string} name Gets or sets the name of the variable.
 * @member {string} [value] Gets or sets the value of the variable.
 * @member {string} [description] Gets or sets the description of the variable.
 */
export interface VariableUpdateParameters {
  name: string;
  value?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the WebhookCreateOrUpdateParameters class.
 * @constructor
 * The parameters supplied to the create or update webhook operation.
 *
 * @member {string} name Gets or sets the name of the webhook.
 * @member {boolean} [isEnabled] Gets or sets the value of the enabled flag of
 * webhook.
 * @member {string} [uri] Gets or sets the uri.
 * @member {date} [expiryTime] Gets or sets the expiry time.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {object} [runbook] Gets or sets the runbook.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the name of the hybrid worker group
 * the webhook job will run on.
 */
export interface WebhookCreateOrUpdateParameters {
  name: string;
  isEnabled?: boolean;
  uri?: string;
  expiryTime?: Date;
  parameters?: { [propertyName: string]: string };
  runbook?: RunbookAssociationProperty;
  runOn?: string;
}

/**
 * @class
 * Initializes a new instance of the Webhook class.
 * @constructor
 * Definition of the webhook type.
 *
 * @member {string} [id] Gets or sets the id of the resource.
 * @member {string} [name] Gets or sets the name of the webhook.
 * @member {boolean} [isEnabled] Gets or sets the value of the enabled flag of
 * the webhook. Default value: false .
 * @member {string} [uri] Gets or sets the webhook uri.
 * @member {date} [expiryTime] Gets or sets the expiry time.
 * @member {date} [lastInvokedTime] Gets or sets the last invoked time.
 * @member {object} [parameters] Gets or sets the parameters of the job that is
 * created when the webhook calls the runbook it is associated with.
 * @member {object} [runbook] Gets or sets the runbook the webhook is
 * associated with.
 * @member {string} [runbook.name] Gets or sets the name of the runbook.
 * @member {string} [runOn] Gets or sets the name of the hybrid worker group
 * the webhook job will run on.
 * @member {date} [creationTime] Gets or sets the creation time.
 * @member {date} [lastModifiedTime] Gets or sets the last modified time.
 * @member {string} [description] Gets or sets the description.
 */
export interface Webhook {
  id?: string;
  name?: string;
  isEnabled?: boolean;
  uri?: string;
  expiryTime?: Date;
  lastInvokedTime?: Date;
  parameters?: { [propertyName: string]: string };
  runbook?: RunbookAssociationProperty;
  runOn?: string;
  creationTime?: Date;
  lastModifiedTime?: Date;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the WebhookUpdateParameters class.
 * @constructor
 * The parameters supplied to the update webhook operation.
 *
 * @member {string} name Gets or sets the name of the webhook.
 * @member {boolean} [isEnabled] Gets or sets the value of the enabled flag of
 * webhook.
 * @member {string} [runOn] Gets or sets the name of the hybrid worker group
 * the webhook job will run on.
 * @member {object} [parameters] Gets or sets the parameters of the job.
 * @member {string} [description] Gets or sets the description of the webhook.
 */
export interface WebhookUpdateParameters {
  name: string;
  isEnabled?: boolean;
  runOn?: string;
  parameters?: { [propertyName: string]: string };
  description?: string;
}


/**
 * @class
 * Initializes a new instance of the AutomationAccountListResult class.
 * @constructor
 * The response model for the list account operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface AutomationAccountListResult extends Array<AutomationAccount> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * The response model for the list of Automation operations
 *
 */
export interface OperationListResult extends Array<Operation> {
}

/**
 * @class
 * Initializes a new instance of the StatisticsListResult class.
 * @constructor
 * The response model for the list statistics operation.
 *
 */
export interface StatisticsListResult extends Array<Statistics> {
}

/**
 * @class
 * Initializes a new instance of the UsageListResult class.
 * @constructor
 * The response model for the get usage operation.
 *
 */
export interface UsageListResult extends Array<Usage> {
}

/**
 * @class
 * Initializes a new instance of the CertificateListResult class.
 * @constructor
 * The response model for the list certificate operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface CertificateListResult extends Array<Certificate> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionListResult class.
 * @constructor
 * The response model for the list connection operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ConnectionListResult extends Array<Connection> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectionTypeListResult class.
 * @constructor
 * The response model for the list connection type operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ConnectionTypeListResult extends Array<ConnectionType> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the CredentialListResult class.
 * @constructor
 * The response model for the list credential operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface CredentialListResult extends Array<Credential> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscCompilationJobListResult class.
 * @constructor
 * The response model for the list job operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscCompilationJobListResult extends Array<DscCompilationJob> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscConfigurationListResult class.
 * @constructor
 * The response model for the list configuration operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscConfigurationListResult extends Array<DscConfiguration> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeListResult class.
 * @constructor
 * The response model for the list dsc nodes operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscNodeListResult extends Array<DscNode> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeReportListResult class.
 * @constructor
 * The response model for the list dsc nodes operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscNodeReportListResult extends Array<DscNodeReport> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DscNodeConfigurationListResult class.
 * @constructor
 * The response model for the list job operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface DscNodeConfigurationListResult extends Array<DscNodeConfiguration> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the HybridRunbookWorkerGroupsListResult class.
 * @constructor
 * The response model for the list hybrid runbook worker groups.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface HybridRunbookWorkerGroupsListResult extends Array<HybridRunbookWorkerGroup> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobListResult class.
 * @constructor
 * The response model for the list job operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface JobListResult extends Array<Job> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobStreamListResult class.
 * @constructor
 * The response model for the list job stream operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface JobStreamListResult extends Array<JobStream> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the JobScheduleListResult class.
 * @constructor
 * The response model for the list job schedule operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface JobScheduleListResult extends Array<JobSchedule> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ActivityListResult class.
 * @constructor
 * The response model for the list activity operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ActivityListResult extends Array<Activity> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ModuleListResult class.
 * @constructor
 * The response model for the list module operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ModuleListResult extends Array<Module> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the TypeFieldListResult class.
 * @constructor
 * The response model for the list fields operation.
 *
 */
export interface TypeFieldListResult extends Array<TypeField> {
}

/**
 * @class
 * Initializes a new instance of the RunbookListResult class.
 * @constructor
 * The response model for the list runbook operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface RunbookListResult extends Array<Runbook> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ScheduleListResult class.
 * @constructor
 * The response model for the list schedule operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface ScheduleListResult extends Array<Schedule> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the VariableListResult class.
 * @constructor
 * The response model for the list variables operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface VariableListResult extends Array<Variable> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WebhookListResult class.
 * @constructor
 * The response model for the list webhook operation.
 *
 * @member {string} [nextLink] Gets or sets the next link.
 */
export interface WebhookListResult extends Array<Webhook> {
  nextLink?: string;
}
