/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Contains information about an Azure Batch account.
 *
 * @extends models['Resource']
 */
class BatchAccount extends models['Resource'] {
  /**
   * Create a BatchAccount.
   * @member {string} [accountEndpoint] The account endpoint used to interact
   * with the Batch service.
   * @member {string} [provisioningState] The provisioned state of the
   * resource. Possible values include: 'Invalid', 'Creating', 'Deleting',
   * 'Succeeded', 'Failed', 'Cancelled'
   * @member {string} [poolAllocationMode] The allocation mode to use for
   * creating pools in the Batch account. Possible values include:
   * 'BatchService', 'UserSubscription'
   * @member {object} [keyVaultReference] A reference to the Azure key vault
   * associated with the Batch account.
   * @member {string} [keyVaultReference.id] The resource ID of the Azure key
   * vault associated with the Batch account.
   * @member {string} [keyVaultReference.url] The URL of the Azure key vault
   * associated with the Batch account.
   * @member {object} [autoStorage] The properties and status of any
   * auto-storage account associated with the Batch account.
   * @member {date} [autoStorage.lastKeySync] The UTC time at which storage
   * keys were last synchronized with the Batch account.
   * @member {number} [dedicatedCoreQuota] The dedicated core quota for this
   * Batch account.
   * @member {number} [lowPriorityCoreQuota] The low-priority core quota for
   * this Batch account.
   * @member {number} [poolQuota] The pool quota for this Batch account.
   * @member {number} [activeJobAndJobScheduleQuota] The active job and job
   * schedule quota for this Batch account.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of BatchAccount
   *
   * @returns {object} metadata of BatchAccount
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'BatchAccount',
      type: {
        name: 'Composite',
        className: 'BatchAccount',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            readOnly: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            readOnly: true,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          accountEndpoint: {
            required: false,
            readOnly: true,
            serializedName: 'properties.accountEndpoint',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'Enum',
              allowedValues: [ 'Invalid', 'Creating', 'Deleting', 'Succeeded', 'Failed', 'Cancelled' ]
            }
          },
          poolAllocationMode: {
            required: false,
            readOnly: true,
            serializedName: 'properties.poolAllocationMode',
            type: {
              name: 'Enum',
              allowedValues: [ 'BatchService', 'UserSubscription' ]
            }
          },
          keyVaultReference: {
            required: false,
            readOnly: true,
            serializedName: 'properties.keyVaultReference',
            type: {
              name: 'Composite',
              className: 'KeyVaultReference'
            }
          },
          autoStorage: {
            required: false,
            readOnly: true,
            serializedName: 'properties.autoStorage',
            type: {
              name: 'Composite',
              className: 'AutoStorageProperties'
            }
          },
          dedicatedCoreQuota: {
            required: false,
            readOnly: true,
            serializedName: 'properties.dedicatedCoreQuota',
            type: {
              name: 'Number'
            }
          },
          lowPriorityCoreQuota: {
            required: false,
            readOnly: true,
            serializedName: 'properties.lowPriorityCoreQuota',
            type: {
              name: 'Number'
            }
          },
          poolQuota: {
            required: false,
            readOnly: true,
            serializedName: 'properties.poolQuota',
            type: {
              name: 'Number'
            }
          },
          activeJobAndJobScheduleQuota: {
            required: false,
            readOnly: true,
            serializedName: 'properties.activeJobAndJobScheduleQuota',
            type: {
              name: 'Number'
            }
          }
        }
      }
    };
  }
}

module.exports = BatchAccount;
