/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Parameters supplied to the Create operation.
 *
 */
class BatchAccountCreateParameters {
  /**
   * Create a BatchAccountCreateParameters.
   * @member {string} location The region in which to create the account.
   * @member {object} [tags] The user-specified tags associated with the
   * account.
   * @member {object} [autoStorage] The properties related to the auto-storage
   * account.
   * @member {string} [autoStorage.storageAccountId] The resource ID of the
   * storage account to be used for auto-storage account.
   * @member {string} [poolAllocationMode] The allocation mode to use for
   * creating pools in the Batch account. The pool allocation mode also affects
   * how clients may authenticate to the Batch Service API. If the mode is
   * BatchService, clients may authenticate using access keys or Azure Active
   * Directory. If the mode is UserSubscription, clients must use Azure Active
   * Directory. The default is BatchService. Possible values include:
   * 'BatchService', 'UserSubscription'
   * @member {object} [keyVaultReference] A reference to the Azure key vault
   * associated with the Batch account.
   * @member {string} [keyVaultReference.id] The resource ID of the Azure key
   * vault associated with the Batch account.
   * @member {string} [keyVaultReference.url] The URL of the Azure key vault
   * associated with the Batch account.
   */
  constructor() {
  }

  /**
   * Defines the metadata of BatchAccountCreateParameters
   *
   * @returns {object} metadata of BatchAccountCreateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'BatchAccountCreateParameters',
      type: {
        name: 'Composite',
        className: 'BatchAccountCreateParameters',
        modelProperties: {
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          autoStorage: {
            required: false,
            serializedName: 'properties.autoStorage',
            type: {
              name: 'Composite',
              className: 'AutoStorageBaseProperties'
            }
          },
          poolAllocationMode: {
            required: false,
            serializedName: 'properties.poolAllocationMode',
            type: {
              name: 'Enum',
              allowedValues: [ 'BatchService', 'UserSubscription' ]
            }
          },
          keyVaultReference: {
            required: false,
            serializedName: 'properties.keyVaultReference',
            type: {
              name: 'Composite',
              className: 'KeyVaultReference'
            }
          }
        }
      }
    };
  }
}

module.exports = BatchAccountCreateParameters;
