/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { ServiceClient, ServiceClientOptions, ServiceCallback, HttpOperationResponse, ServiceClientCredentials } from 'ms-rest';
import { AzureServiceClient, AzureServiceClientOptions } from 'ms-rest-azure';
import * as models from "./models";
import * as operations from "./operations";

declare class CdnManagementClient extends AzureServiceClient {
  /**
   * Initializes a new instance of the CdnManagementClient class.
   * @constructor
   *
   * @class
   * @param {credentials} credentials - Credentials needed for the client to connect to Azure.
   *
   * @param {string} subscriptionId - Azure Subscription ID.
   *
   * @param {string} [baseUri] - The base URI of the service.
   *
   * @param {object} [options] - The parameter options
   *
   * @param {Array} [options.filters] - Filters to be added to the request pipeline
   *
   * @param {object} [options.requestOptions] - Options for the underlying request object
   * {@link https://github.com/request/request#requestoptions-callback Options doc}
   *
   * @param {boolean} [options.noRetryPolicy] - If set to true, turn off default retry policy
   *
   * @param {string} [options.acceptLanguage] - Gets or sets the preferred language for the response.
   *
   * @param {number} [options.longRunningOperationRetryTimeout] - Gets or sets the retry timeout in seconds for Long Running Operations. Default value is 30.
   *
   * @param {boolean} [options.generateClientRequestId] - When set to true a unique x-ms-client-request-id value is generated and included in each request. Default is true.
   *
   */
  constructor(credentials: ServiceClientCredentials, subscriptionId: string, baseUri?: string, options?: AzureServiceClientOptions);

  credentials: ServiceClientCredentials;

  subscriptionId: string;

  apiVersion: string;

  acceptLanguage: string;

  longRunningOperationRetryTimeout: number;

  generateClientRequestId: boolean;

  // Operation groups
  profiles: operations.Profiles;
  endpoints: operations.Endpoints;
  origins: operations.Origins;
  customDomains: operations.CustomDomains;
  resourceUsageOperations: operations.ResourceUsageOperations;
  operations: operations.Operations;
  edgeNodes: operations.EdgeNodes;


  /**
   * Check the availability of a resource name. This is needed for resources
   * where name is globally unique, such as a CDN endpoint.
   *
   * @param {string} name The resource name to validate.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<CheckNameAvailabilityOutput>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  checkNameAvailabilityWithHttpOperationResponse(name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.CheckNameAvailabilityOutput>>;

  /**
   * Check the availability of a resource name. This is needed for resources
   * where name is globally unique, such as a CDN endpoint.
   *
   * @param {string} name The resource name to validate.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {CheckNameAvailabilityOutput} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {CheckNameAvailabilityOutput} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link CheckNameAvailabilityOutput} for more
   *                      information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  checkNameAvailability(name: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.CheckNameAvailabilityOutput>;
  checkNameAvailability(name: string, callback: ServiceCallback<models.CheckNameAvailabilityOutput>): void;
  checkNameAvailability(name: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CheckNameAvailabilityOutput>): void;


  /**
   * Check if the probe path is a valid path and the file can be accessed. Probe
   * path is the path to a file hosted on the origin server to help accelerate
   * the delivery of dynamic content via the CDN endpoint. This path is relative
   * to the origin path specified in the endpoint configuration.
   *
   * @param {string} probeURL The probe URL to validate.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<ValidateProbeOutput>} - The deserialized result object.
   *
   * @reject {Error|ServiceError} - The error object.
   */
  validateProbeWithHttpOperationResponse(probeURL: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.ValidateProbeOutput>>;

  /**
   * Check if the probe path is a valid path and the file can be accessed. Probe
   * path is the path to a file hosted on the origin server to help accelerate
   * the delivery of dynamic content via the CDN endpoint. This path is relative
   * to the origin path specified in the endpoint configuration.
   *
   * @param {string} probeURL The probe URL to validate.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {ServiceCallback} [optionalCallback] - The optional callback.
   *
   * @returns {ServiceCallback|Promise} If a callback was passed as the last
   * parameter then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned.
   *
   *                      @resolve {ValidateProbeOutput} - The deserialized result object.
   *
   *                      @reject {Error|ServiceError} - The error object.
   *
   * {ServiceCallback} optionalCallback(err, result, request, response)
   *
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {ValidateProbeOutput} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link ValidateProbeOutput} for more information.
   *
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
   */
  validateProbe(probeURL: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.ValidateProbeOutput>;
  validateProbe(probeURL: string, callback: ServiceCallback<models.ValidateProbeOutput>): void;
  validateProbe(probeURL: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.ValidateProbeOutput>): void;
}

export = CdnManagementClient;
