/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the HubBillingInfoFormat class.
 * @constructor
 * Hub billing info.
 *
 * @member {string} [skuName] The sku name.
 * @member {number} [minUnits] The minimum number of units will be billed. One
 * unit is 10,000 Profiles and 100,000 Interactions.
 * @member {number} [maxUnits] The maximum number of units can be used.  One
 * unit is 10,000 Profiles and 100,000 Interactions.
 */
export interface HubBillingInfoFormat {
  skuName?: string;
  minUnits?: number;
  maxUnits?: number;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Common properties of Azure resource.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 * @member {string} [location] Resource location.
 * @member {object} [tags] Resource tags.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Hub class.
 * @constructor
 * Hub resource.
 *
 * @member {string} [apiEndpoint] API endpoint URL of the hub.
 * @member {string} [webEndpoint] Web endpoint URL of the hub.
 * @member {string} [provisioningState] Provisioning state of the hub.
 * @member {number} [tenantFeatures] The bit flags for enabled hub features.
 * Bit 0 is set to 1 indicates graph is enabled, or disabled if set to 0. Bit 1
 * is set to 1 indicates the hub is disabled, or enabled if set to 0.
 * @member {object} [hubBillingInfo] Billing settings of the hub.
 * @member {string} [hubBillingInfo.skuName] The sku name.
 * @member {number} [hubBillingInfo.minUnits] The minimum number of units will
 * be billed. One unit is 10,000 Profiles and 100,000 Interactions.
 * @member {number} [hubBillingInfo.maxUnits] The maximum number of units can
 * be used.  One unit is 10,000 Profiles and 100,000 Interactions.
 */
export interface Hub extends Resource {
  readonly apiEndpoint?: string;
  readonly webEndpoint?: string;
  readonly provisioningState?: string;
  tenantFeatures?: number;
  hubBillingInfo?: HubBillingInfoFormat;
}

/**
 * @class
 * Initializes a new instance of the MetadataDefinitionBase class.
 * @constructor
 * The Metadata definition base.
 *
 * @member {object} [attributes] The attributes for the Type.
 * @member {object} [description] Localized descriptions for the property.
 * @member {object} [displayName] Localized display names for the property.
 * @member {object} [localizedAttributes] Any custom localized attributes for
 * the Type.
 * @member {string} [smallImage] Small Image associated with the Property or
 * EntityType.
 * @member {string} [mediumImage] Medium Image associated with the Property or
 * EntityType.
 * @member {string} [largeImage] Large Image associated with the Property or
 * EntityType.
 */
export interface MetadataDefinitionBase {
  attributes?: { [propertyName: string]: string[] };
  description?: { [propertyName: string]: string };
  displayName?: { [propertyName: string]: string };
  localizedAttributes?: { [propertyName: string]: { [propertyName: string]: string } };
  smallImage?: string;
  mediumImage?: string;
  largeImage?: string;
}

/**
 * @class
 * Initializes a new instance of the ProfileEnumValidValuesFormat class.
 * @constructor
 * Valid enum values in case of an enum property.
 *
 * @member {number} [value] The integer value of the enum member.
 * @member {object} [localizedValueNames] Localized names of the enum member.
 */
export interface ProfileEnumValidValuesFormat {
  value?: number;
  localizedValueNames?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the DataSourcePrecedence class.
 * @constructor
 * The data source precedence is a way to know the precedence of each data
 * source.
 *
 * @member {string} [name] The data source name
 * @member {string} [dataSourceType] The data source type. Possible values
 * include: 'Connector', 'LinkInteraction', 'SystemDefault'
 * @member {string} [status] The data source status. Possible values include:
 * 'None', 'Active', 'Deleted'
 * @member {number} [id] The data source ID.
 * @member {string} [dataSourceReferenceId] The data source reference id.
 * @member {number} [precedence] the precedence value.
 */
export interface DataSourcePrecedence {
  readonly name?: string;
  readonly dataSourceType?: string;
  readonly status?: string;
  readonly id?: number;
  readonly dataSourceReferenceId?: string;
  precedence?: number;
}

/**
 * @class
 * Initializes a new instance of the PropertyDefinition class.
 * @constructor
 * Property definition.
 *
 * @member {string} [arrayValueSeparator] Array value separator for properties
 * with isArray set.
 * @member {array} [enumValidValues] Describes valid values for an enum
 * property.
 * @member {string} fieldName Name of the property.
 * @member {string} fieldType Type of the property.
 * @member {boolean} [isArray] Indicates if the property is actually an array
 * of the fieldType above on the data api.
 * @member {boolean} [isEnum] Indicates if the property is an enum.
 * @member {boolean} [isFlagEnum] Indicates if the property is an flag enum.
 * @member {boolean} [isImage] Whether the property is an Image.
 * @member {boolean} [isLocalizedString] Whether the property is a localized
 * string.
 * @member {boolean} [isName] Whether the property is a name or a part of name.
 * @member {boolean} [isRequired] Whether property value is required on
 * instances, IsRequired field only for Intercation. Profile Instance will not
 * check for required field.
 * @member {string} [propertyId] The ID associated with the property.
 * @member {string} [schemaItemPropLink] URL encoded schema.org item prop link
 * for the property.
 * @member {number} [maxLength] Max length of string. Used only if type is
 * string.
 * @member {boolean} [isAvailableInGraph] Whether property is available in
 * graph or not.
 * @member {array} [dataSourcePrecedenceRules] This is specific to interactions
 * modeled as activities. Data sources are used to determine where data is
 * stored and also in precedence rules.
 */
export interface PropertyDefinition {
  arrayValueSeparator?: string;
  enumValidValues?: ProfileEnumValidValuesFormat[];
  fieldName: string;
  fieldType: string;
  isArray?: boolean;
  isEnum?: boolean;
  isFlagEnum?: boolean;
  isImage?: boolean;
  isLocalizedString?: boolean;
  isName?: boolean;
  isRequired?: boolean;
  propertyId?: string;
  schemaItemPropLink?: string;
  maxLength?: number;
  isAvailableInGraph?: boolean;
  readonly dataSourcePrecedenceRules?: DataSourcePrecedence[];
}

/**
 * @class
 * Initializes a new instance of the EntityTypeDefinition class.
 * @constructor
 * Describes an entity.
 *
 * @member {string} [apiEntitySetName] The api entity set name. This becomes
 * the odata entity set name for the entity Type being refered in this object.
 * @member {string} [entityType] Type of entity. Possible values include:
 * 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {array} [fields] The properties of the Profile.
 * @member {number} [instancesCount] The instance count.
 * @member {date} [lastChangedUtc] The last changed time for the type
 * definition.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {string} [schemaItemTypeLink] The schema org link. This helps ACI
 * identify and suggest semantic models.
 * @member {string} [tenantId] The hub name.
 * @member {string} [timestampFieldName] The timestamp property name.
 * Represents the time when the interaction or profile update happened.
 * @member {string} [typeName] The name of the entity.
 */
export interface EntityTypeDefinition extends MetadataDefinitionBase {
  apiEntitySetName?: string;
  entityType?: string;
  fields?: PropertyDefinition[];
  instancesCount?: number;
  readonly lastChangedUtc?: Date;
  readonly provisioningState?: string;
  schemaItemTypeLink?: string;
  readonly tenantId?: string;
  timestampFieldName?: string;
  typeName?: string;
}

/**
 * @class
 * Initializes a new instance of the AuthorizationPolicy class.
 * @constructor
 * The authorization policy.
 *
 * @member {string} [policyName] Name of the policy.
 * @member {array} permissions The permissions associated with the policy.
 * @member {string} [primaryKey] Primary key assiciated with the policy.
 * @member {string} [secondaryKey] Secondary key assiciated with the policy.
 */
export interface AuthorizationPolicy {
  readonly policyName?: string;
  permissions: string[];
  primaryKey?: string;
  secondaryKey?: string;
}

/**
 * @class
 * Initializes a new instance of the SalesforceDiscoverSetting class.
 * @constructor
 * Salesforce discover setting.
 *
 * @member {string} salesforceConnectionStringSecretUrl The salesforce
 * connection string secret URL.
 */
export interface SalesforceDiscoverSetting {
  salesforceConnectionStringSecretUrl: string;
}

/**
 * @class
 * Initializes a new instance of the SalesforceTable class.
 * @constructor
 * Salesforce table.
 *
 * @member {string} [isProfile] Indicating whether this instance is profile.
 * @member {string} tableCategory The table category.
 * @member {string} tableName The name of the table.
 * @member {string} [tableRemarks] The table remarks.
 * @member {string} tableSchema The table schema.
 */
export interface SalesforceTable {
  isProfile?: string;
  tableCategory: string;
  tableName: string;
  tableRemarks?: string;
  tableSchema: string;
}

/**
 * @class
 * Initializes a new instance of the SalesforceConnectorProperties class.
 * @constructor
 * The Salesforce connector properties.
 *
 * @member {object} usersetting The user settings.
 * @member {string} [usersetting.salesforceConnectionStringSecretUrl] The
 * salesforce connection string secret URL.
 * @member {array} salesforcetables The Salesforce tables.
 */
export interface SalesforceConnectorProperties {
  usersetting: SalesforceDiscoverSetting;
  salesforcetables: SalesforceTable[];
}

/**
 * @class
 * Initializes a new instance of the AzureBlobConnectorProperties class.
 * @constructor
 * The Azure Blob connector properties.
 *
 * @member {string} connectionKeyVaultUrl The connection KeyVault URL.
 */
export interface AzureBlobConnectorProperties {
  connectionKeyVaultUrl: string;
}

/**
 * @class
 * Initializes a new instance of the CrmConnectorEntities class.
 * @constructor
 * The CRM connector entities.
 *
 * @member {string} logicalName The logical name.
 * @member {string} [displayName] The display name.
 * @member {boolean} [isProfile] Indicating whether this is profile.
 */
export interface CrmConnectorEntities {
  logicalName: string;
  displayName?: string;
  isProfile?: boolean;
}

/**
 * @class
 * Initializes a new instance of the CrmConnectorProperties class.
 * @constructor
 * The CRM connector properties.
 *
 * @member {string} [connectionString] The connection string.
 * @member {string} organizationId The organization ID.
 * @member {string} organizationUrl The organization URL.
 * @member {array} entities The entities like account, contact, opportunity.
 * @member {string} [accessToken] The access token.
 */
export interface CrmConnectorProperties {
  connectionString?: string;
  organizationId: string;
  organizationUrl: string;
  entities: CrmConnectorEntities[];
  accessToken?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingErrorManagement class.
 * @constructor
 * The error mangement.
 *
 * @member {string} errorManagementType The type of error management to use for
 * the mapping. Possible values include: 'RejectAndContinue', 'StopImport',
 * 'RejectUntilLimit'
 * @member {number} [errorLimit] The error limit allowed while importing data.
 */
export interface ConnectorMappingErrorManagement {
  errorManagementType: string;
  errorLimit?: number;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingFormat class.
 * @constructor
 * Connector mapping property format.
 *
 * @member {string} [columnDelimiter] The character that signifies a break
 * between columns.
 * @member {string} [acceptLanguage] The oData language.
 * @member {string} [quoteCharacter] Quote character, used to indicate enquoted
 * fields.
 * @member {string} [quoteEscapeCharacter] Escape character for quotes, can be
 * the same as the quoteCharacter.
 * @member {string} [arraySeparator] Character separating array elements.
 */
export interface ConnectorMappingFormat {
  columnDelimiter?: string;
  acceptLanguage?: string;
  quoteCharacter?: string;
  quoteEscapeCharacter?: string;
  arraySeparator?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingAvailability class.
 * @constructor
 * Connector mapping property availability.
 *
 * @member {string} [frequency] The frequency to update. Possible values
 * include: 'Minute', 'Hour', 'Day', 'Week', 'Month'
 * @member {number} interval The interval of the given frequency to use.
 */
export interface ConnectorMappingAvailability {
  frequency?: string;
  interval: number;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingStructure class.
 * @constructor
 * Connector mapping property structure.
 *
 * @member {string} propertyName The property name of the mapping entity.
 * @member {string} columnName The column name of the import file.
 * @member {string} [customFormatSpecifier] Custom format specifier for input
 * parsing.
 * @member {boolean} [isEncrypted] Indicates if the column is encrypted.
 */
export interface ConnectorMappingStructure {
  propertyName: string;
  columnName: string;
  customFormatSpecifier?: string;
  isEncrypted?: boolean;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingCompleteOperation class.
 * @constructor
 * The complete operation.
 *
 * @member {string} [completionOperationType] The type of completion operation.
 * Possible values include: 'DoNothing', 'DeleteFile', 'MoveFile'
 * @member {string} [destinationFolder] The destination folder where files will
 * be moved to once the import is done.
 */
export interface ConnectorMappingCompleteOperation {
  completionOperationType?: string;
  destinationFolder?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingProperties class.
 * @constructor
 * The connector mapping properties.
 *
 * @member {string} [folderPath] The folder path for the mapping.
 * @member {string} [fileFilter] The file filter for the mapping.
 * @member {boolean} [hasHeader] If the file contains a header or not.
 * @member {object} errorManagement The error management setting for the
 * mapping.
 * @member {string} [errorManagement.errorManagementType] The type of error
 * management to use for the mapping. Possible values include:
 * 'RejectAndContinue', 'StopImport', 'RejectUntilLimit'
 * @member {number} [errorManagement.errorLimit] The error limit allowed while
 * importing data.
 * @member {object} format The format of mapping property.
 * @member {string} [format.columnDelimiter] The character that signifies a
 * break between columns.
 * @member {string} [format.acceptLanguage] The oData language.
 * @member {string} [format.quoteCharacter] Quote character, used to indicate
 * enquoted fields.
 * @member {string} [format.quoteEscapeCharacter] Escape character for quotes,
 * can be the same as the quoteCharacter.
 * @member {string} [format.arraySeparator] Character separating array
 * elements.
 * @member {object} availability The availability of mapping property.
 * @member {string} [availability.frequency] The frequency to update. Possible
 * values include: 'Minute', 'Hour', 'Day', 'Week', 'Month'
 * @member {number} [availability.interval] The interval of the given frequency
 * to use.
 * @member {array} structure Ingestion mapping information at property level.
 * @member {object} completeOperation The operation after import is done.
 * @member {string} [completeOperation.completionOperationType] The type of
 * completion operation. Possible values include: 'DoNothing', 'DeleteFile',
 * 'MoveFile'
 * @member {string} [completeOperation.destinationFolder] The destination
 * folder where files will be moved to once the import is done.
 */
export interface ConnectorMappingProperties {
  folderPath?: string;
  fileFilter?: string;
  hasHeader?: boolean;
  errorManagement: ConnectorMappingErrorManagement;
  format: ConnectorMappingFormat;
  availability: ConnectorMappingAvailability;
  structure: ConnectorMappingStructure[];
  completeOperation: ConnectorMappingCompleteOperation;
}

/**
 * @class
 * Initializes a new instance of the KpiThresholds class.
 * @constructor
 * Defines the KPI Threshold limits.
 *
 * @member {number} lowerLimit The lower threshold limit.
 * @member {number} upperLimit The upper threshold limit.
 * @member {boolean} increasingKpi Whether or not the KPI is an increasing KPI.
 */
export interface KpiThresholds {
  lowerLimit: number;
  upperLimit: number;
  increasingKpi: boolean;
}

/**
 * @class
 * Initializes a new instance of the KpiGroupByMetadata class.
 * @constructor
 * The KPI GroupBy field metadata.
 *
 * @member {object} [displayName] The display name.
 * @member {string} [fieldName] The name of the field.
 * @member {string} [fieldType] The type of the field.
 */
export interface KpiGroupByMetadata {
  displayName?: { [propertyName: string]: string };
  fieldName?: string;
  fieldType?: string;
}

/**
 * @class
 * Initializes a new instance of the KpiParticipantProfilesMetadata class.
 * @constructor
 * The KPI participant profile metadata.
 *
 * @member {string} typeName Name of the type.
 */
export interface KpiParticipantProfilesMetadata {
  typeName: string;
}

/**
 * @class
 * Initializes a new instance of the KpiAlias class.
 * @constructor
 * The KPI alias.
 *
 * @member {string} aliasName KPI alias name.
 * @member {string} expression The expression.
 */
export interface KpiAlias {
  aliasName: string;
  expression: string;
}

/**
 * @class
 * Initializes a new instance of the KpiExtract class.
 * @constructor
 * The KPI extract.
 *
 * @member {string} extractName KPI extract name.
 * @member {string} expression The expression.
 */
export interface KpiExtract {
  extractName: string;
  expression: string;
}

/**
 * @class
 * Initializes a new instance of the KpiDefinition class.
 * @constructor
 * Defines the KPI Threshold limits.
 *
 * @member {string} entityType The mapping entity type. Possible values
 * include: 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {string} entityTypeName The mapping entity name.
 * @member {string} [tenantId] The hub name.
 * @member {string} [kpiName] The KPI name.
 * @member {object} [displayName] Localized display name for the KPI.
 * @member {object} [description] Localized description for the KPI.
 * @member {string} calculationWindow The calculation window. Possible values
 * include: 'Lifetime', 'Hour', 'Day', 'Week', 'Month'
 * @member {string} [calculationWindowFieldName] Name of calculation window
 * field.
 * @member {string} functionProperty The computation function for the KPI.
 * Possible values include: 'Sum', 'Avg', 'Min', 'Max', 'Last', 'Count',
 * 'None', 'CountDistinct'
 * @member {string} expression The computation expression for the KPI.
 * @member {string} [unit] The unit of measurement for the KPI.
 * @member {string} [filter] The filter expression for the KPI.
 * @member {array} [groupBy] the group by properties for the KPI.
 * @member {array} [groupByMetadata] The KPI GroupByMetadata.
 * @member {array} [participantProfilesMetadata] The participant profiles.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {object} [thresHolds] The KPI thresholds.
 * @member {number} [thresHolds.lowerLimit] The lower threshold limit.
 * @member {number} [thresHolds.upperLimit] The upper threshold limit.
 * @member {boolean} [thresHolds.increasingKpi] Whether or not the KPI is an
 * increasing KPI.
 * @member {array} [aliases] The aliases.
 * @member {array} [extracts] The KPI extracts.
 */
export interface KpiDefinition {
  entityType: string;
  entityTypeName: string;
  readonly tenantId?: string;
  readonly kpiName?: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  calculationWindow: string;
  calculationWindowFieldName?: string;
  functionProperty: string;
  expression: string;
  unit?: string;
  filter?: string;
  groupBy?: string[];
  readonly groupByMetadata?: KpiGroupByMetadata[];
  readonly participantProfilesMetadata?: KpiParticipantProfilesMetadata[];
  readonly provisioningState?: string;
  thresHolds?: KpiThresholds;
  aliases?: KpiAlias[];
  extracts?: KpiExtract[];
}

/**
 * @class
 * Initializes a new instance of the ProxyResource class.
 * @constructor
 * Common properties of proxy resource.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 */
export interface ProxyResource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the TypePropertiesMapping class.
 * @constructor
 * Metadata for a Link's property mapping.
 *
 * @member {string} sourcePropertyName Property name on the source Entity Type.
 * @member {string} targetPropertyName Property name on the target Entity Type.
 * @member {string} [linkType] Link type. Possible values include:
 * 'UpdateAlways', 'CopyIfNull'
 */
export interface TypePropertiesMapping {
  sourcePropertyName: string;
  targetPropertyName: string;
  linkType?: string;
}

/**
 * @class
 * Initializes a new instance of the ParticipantPropertyReference class.
 * @constructor
 * The participant property reference.
 *
 * @member {string} sourcePropertyName The source property that maps to the
 * target property.
 * @member {string} targetPropertyName The target property that maps to the
 * source property.
 */
export interface ParticipantPropertyReference {
  sourcePropertyName: string;
  targetPropertyName: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipTypeFieldMapping class.
 * @constructor
 * Map a field of profile to its corresponding StrongId in Related Profile.
 *
 * @member {string} profileFieldName Specifies the fieldName in profile.
 * @member {string} relatedProfileKeyProperty Specifies the KeyProperty (from
 * StrongId) of the related profile.
 */
export interface RelationshipTypeFieldMapping {
  profileFieldName: string;
  relatedProfileKeyProperty: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipTypeMapping class.
 * @constructor
 * Maps fields in Profile to their corresponding StrongIds in Related Profile.
 *
 * @member {array} fieldMappings Maps a profile property with the StrongId of
 * related profile. This is an array to support StrongIds that are composite
 * key as well.
 */
export interface RelationshipTypeMapping {
  fieldMappings: RelationshipTypeFieldMapping[];
}

/**
 * @class
 * Initializes a new instance of the RelationshipLinkFieldMapping class.
 * @constructor
 * The fields mapping for Relationships.
 *
 * @member {string} interactionFieldName The field name on the Interaction
 * Type.
 * @member {string} [linkType] Link type. Possible values include:
 * 'UpdateAlways', 'CopyIfNull'
 * @member {string} relationshipFieldName The field name on the Relationship
 * metadata.
 */
export interface RelationshipLinkFieldMapping {
  interactionFieldName: string;
  linkType?: string;
  relationshipFieldName: string;
}

/**
 * @class
 * Initializes a new instance of the ParticipantProfilePropertyReference class.
 * @constructor
 * The participant profile property reference.
 *
 * @member {string} interactionPropertyName The source interaction property
 * that maps to the target profile property.
 * @member {string} profilePropertyName The target profile property that maps
 * to the source interaction property.
 */
export interface ParticipantProfilePropertyReference {
  interactionPropertyName: string;
  profilePropertyName: string;
}

/**
 * @class
 * Initializes a new instance of the Participant class.
 * @constructor
 * Describes a profile type participating in an interaction.
 *
 * @member {string} profileTypeName Profile type name.
 * @member {array} participantPropertyReferences The property references.
 * @member {string} participantName Participant name.
 * @member {object} [displayName] Localized display name.
 * @member {object} [description] Localized descriptions.
 * @member {string} [role] The role that the participant is playing in the
 * interaction.
 */
export interface Participant {
  profileTypeName: string;
  participantPropertyReferences: ParticipantPropertyReference[];
  participantName: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  role?: string;
}

/**
 * @class
 * Initializes a new instance of the StrongId class.
 * @constructor
 * Property/Properties which represent a unique ID.
 *
 * @member {array} keyPropertyNames The properties which make up the unique ID.
 * @member {string} strongIdName The Name identifying the strong ID.
 * @member {object} [displayName] Localized display name.
 * @member {object} [description] Localized descriptions.
 */
export interface StrongId {
  keyPropertyNames: string[];
  strongIdName: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ProfileResourceFormat class.
 * @constructor
 * The profile resource format.
 *
 * @member {object} [attributes] The attributes for the Type.
 * @member {object} [description] Localized descriptions for the property.
 * @member {object} [displayName] Localized display names for the property.
 * @member {object} [localizedAttributes] Any custom localized attributes for
 * the Type.
 * @member {string} [smallImage] Small Image associated with the Property or
 * EntityType.
 * @member {string} [mediumImage] Medium Image associated with the Property or
 * EntityType.
 * @member {string} [largeImage] Large Image associated with the Property or
 * EntityType.
 * @member {string} [apiEntitySetName] The api entity set name. This becomes
 * the odata entity set name for the entity Type being refered in this object.
 * @member {string} [entityType] Type of entity. Possible values include:
 * 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {array} [fields] The properties of the Profile.
 * @member {number} [instancesCount] The instance count.
 * @member {date} [lastChangedUtc] The last changed time for the type
 * definition.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {string} [schemaItemTypeLink] The schema org link. This helps ACI
 * identify and suggest semantic models.
 * @member {string} [tenantId] The hub name.
 * @member {string} [timestampFieldName] The timestamp property name.
 * Represents the time when the interaction or profile update happened.
 * @member {string} [typeName] The name of the entity.
 * @member {array} [strongIds] The strong IDs.
 */
export interface ProfileResourceFormat extends ProxyResource {
  attributes?: { [propertyName: string]: string[] };
  description?: { [propertyName: string]: string };
  displayName?: { [propertyName: string]: string };
  localizedAttributes?: { [propertyName: string]: { [propertyName: string]: string } };
  smallImage?: string;
  mediumImage?: string;
  largeImage?: string;
  apiEntitySetName?: string;
  entityType?: string;
  fields?: PropertyDefinition[];
  instancesCount?: number;
  readonly lastChangedUtc?: Date;
  readonly provisioningState?: string;
  schemaItemTypeLink?: string;
  readonly tenantId?: string;
  timestampFieldName?: string;
  typeName?: string;
  strongIds?: StrongId[];
}

/**
 * @class
 * Initializes a new instance of the InteractionResourceFormat class.
 * @constructor
 * The interaction resource format.
 *
 * @member {object} [attributes] The attributes for the Type.
 * @member {object} [description] Localized descriptions for the property.
 * @member {object} [displayName] Localized display names for the property.
 * @member {object} [localizedAttributes] Any custom localized attributes for
 * the Type.
 * @member {string} [smallImage] Small Image associated with the Property or
 * EntityType.
 * @member {string} [mediumImage] Medium Image associated with the Property or
 * EntityType.
 * @member {string} [largeImage] Large Image associated with the Property or
 * EntityType.
 * @member {string} [apiEntitySetName] The api entity set name. This becomes
 * the odata entity set name for the entity Type being refered in this object.
 * @member {string} [entityType] Type of entity. Possible values include:
 * 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {array} [fields] The properties of the Profile.
 * @member {number} [instancesCount] The instance count.
 * @member {date} [lastChangedUtc] The last changed time for the type
 * definition.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {string} [schemaItemTypeLink] The schema org link. This helps ACI
 * identify and suggest semantic models.
 * @member {string} [tenantId] The hub name.
 * @member {string} [timestampFieldName] The timestamp property name.
 * Represents the time when the interaction or profile update happened.
 * @member {string} [typeName] The name of the entity.
 * @member {array} [idPropertyNames] The id property names. Properties which
 * uniquely identify an interaction instance.
 * @member {array} [participantProfiles] Profiles that participated in the
 * interaction.
 * @member {string} [primaryParticipantProfilePropertyName] The primary
 * participant property name for an interaction ,This is used to logically
 * represent the agent of the interaction, Specify the participant name here
 * from ParticipantName.
 * @member {array} [dataSourcePrecedenceRules] This is specific to interactions
 * modeled as activities. Data sources are used to determine where data is
 * stored and also in precedence rules.
 * @member {string} [interactionResourceFormatName] The data source name
 * @member {string} [dataSourceType] The data source type. Possible values
 * include: 'Connector', 'LinkInteraction', 'SystemDefault'
 * @member {string} [status] The data source status. Possible values include:
 * 'None', 'Active', 'Deleted'
 * @member {number} [interactionResourceFormatId] The data source ID.
 * @member {string} [dataSourceReferenceId] The data source reference id.
 * @member {boolean} [isActivity] An interaction can be tagged as an activity
 * only during create. This enables the interaction to be editable and can
 * enable merging of properties from multiple data sources based on precedence,
 * which is defined at a link level.
 */
export interface InteractionResourceFormat extends ProxyResource {
  attributes?: { [propertyName: string]: string[] };
  description?: { [propertyName: string]: string };
  displayName?: { [propertyName: string]: string };
  localizedAttributes?: { [propertyName: string]: { [propertyName: string]: string } };
  smallImage?: string;
  mediumImage?: string;
  largeImage?: string;
  apiEntitySetName?: string;
  entityType?: string;
  fields?: PropertyDefinition[];
  instancesCount?: number;
  readonly lastChangedUtc?: Date;
  readonly provisioningState?: string;
  schemaItemTypeLink?: string;
  readonly tenantId?: string;
  timestampFieldName?: string;
  typeName?: string;
  idPropertyNames?: string[];
  participantProfiles?: Participant[];
  primaryParticipantProfilePropertyName?: string;
  readonly dataSourcePrecedenceRules?: DataSourcePrecedence[];
  readonly interactionResourceFormatName?: string;
  readonly dataSourceType?: string;
  readonly status?: string;
  readonly interactionResourceFormatId?: number;
  readonly dataSourceReferenceId?: string;
  isActivity?: boolean;
}

/**
 * @class
 * Initializes a new instance of the KpiResourceFormat class.
 * @constructor
 * The KPI resource format.
 *
 * @member {string} entityType The mapping entity type. Possible values
 * include: 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {string} entityTypeName The mapping entity name.
 * @member {string} [tenantId] The hub name.
 * @member {string} [kpiName] The KPI name.
 * @member {object} [displayName] Localized display name for the KPI.
 * @member {object} [description] Localized description for the KPI.
 * @member {string} calculationWindow The calculation window. Possible values
 * include: 'Lifetime', 'Hour', 'Day', 'Week', 'Month'
 * @member {string} [calculationWindowFieldName] Name of calculation window
 * field.
 * @member {string} functionProperty The computation function for the KPI.
 * Possible values include: 'Sum', 'Avg', 'Min', 'Max', 'Last', 'Count',
 * 'None', 'CountDistinct'
 * @member {string} expression The computation expression for the KPI.
 * @member {string} [unit] The unit of measurement for the KPI.
 * @member {string} [filter] The filter expression for the KPI.
 * @member {array} [groupBy] the group by properties for the KPI.
 * @member {array} [groupByMetadata] The KPI GroupByMetadata.
 * @member {array} [participantProfilesMetadata] The participant profiles.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {object} [thresHolds] The KPI thresholds.
 * @member {number} [thresHolds.lowerLimit] The lower threshold limit.
 * @member {number} [thresHolds.upperLimit] The upper threshold limit.
 * @member {boolean} [thresHolds.increasingKpi] Whether or not the KPI is an
 * increasing KPI.
 * @member {array} [aliases] The aliases.
 * @member {array} [extracts] The KPI extracts.
 */
export interface KpiResourceFormat extends ProxyResource {
  entityType: string;
  entityTypeName: string;
  readonly tenantId?: string;
  readonly kpiName?: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  calculationWindow: string;
  calculationWindowFieldName?: string;
  functionProperty: string;
  expression: string;
  unit?: string;
  filter?: string;
  groupBy?: string[];
  readonly groupByMetadata?: KpiGroupByMetadata[];
  readonly participantProfilesMetadata?: KpiParticipantProfilesMetadata[];
  readonly provisioningState?: string;
  thresHolds?: KpiThresholds;
  aliases?: KpiAlias[];
  extracts?: KpiExtract[];
}

/**
 * @class
 * Initializes a new instance of the EnrichingKpi class.
 * @constructor
 * The enriching KPI definition.
 *
 */
export interface EnrichingKpi extends KpiDefinition {
}

/**
 * @class
 * Initializes a new instance of the ConnectorResourceFormat class.
 * @constructor
 * The connector resource format.
 *
 * @member {number} [connectorId] ID of the connector.
 * @member {string} [connectorName] Name of the connector.
 * @member {string} connectorType Type of connector. Possible values include:
 * 'None', 'CRM', 'AzureBlob', 'Salesforce', 'ExchangeOnline', 'Outbound'
 * @member {string} [displayName] Display name of the connector.
 * @member {string} [description] Description of the connector.
 * @member {object} connectorProperties The connector properties.
 * @member {date} [created] The created time.
 * @member {date} [lastModified] The last monified time.
 * @member {string} [state] State of connector. Possible values include:
 * 'Creating', 'Created', 'Ready', 'Expiring', 'Deleting', 'Failed'
 * @member {string} [tenantId] The hub name.
 * @member {boolean} [isInternal] If this is an internal connector.
 */
export interface ConnectorResourceFormat extends ProxyResource {
  readonly connectorId?: number;
  connectorName?: string;
  connectorType: string;
  displayName?: string;
  description?: string;
  connectorProperties: { [propertyName: string]: any };
  readonly created?: Date;
  readonly lastModified?: Date;
  readonly state?: string;
  readonly tenantId?: string;
  isInternal?: boolean;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingResourceFormat class.
 * @constructor
 * The c onnector mapping resource format.
 *
 * @member {string} [connectorName] The connector name.
 * @member {string} [connectorType] Type of connector. Possible values include:
 * 'None', 'CRM', 'AzureBlob', 'Salesforce', 'ExchangeOnline', 'Outbound'
 * @member {date} [created] The created time.
 * @member {date} [lastModified] The last monified time.
 * @member {string} entityType Defines which entity type the file should map
 * to. Possible values include: 'None', 'Profile', 'Interaction',
 * 'Relationship'
 * @member {string} entityTypeName The mapping entity name.
 * @member {string} [connectorMappingName] The connector mapping name
 * @member {string} [displayName] Display name for the connector mapping.
 * @member {string} [description] The description of the connector mapping.
 * @member {string} [dataFormatId] The DataFormat ID.
 * @member {object} mappingProperties The properties of the mapping.
 * @member {string} [mappingProperties.folderPath] The folder path for the
 * mapping.
 * @member {string} [mappingProperties.fileFilter] The file filter for the
 * mapping.
 * @member {boolean} [mappingProperties.hasHeader] If the file contains a
 * header or not.
 * @member {object} [mappingProperties.errorManagement] The error management
 * setting for the mapping.
 * @member {string} [mappingProperties.errorManagement.errorManagementType] The
 * type of error management to use for the mapping. Possible values include:
 * 'RejectAndContinue', 'StopImport', 'RejectUntilLimit'
 * @member {number} [mappingProperties.errorManagement.errorLimit] The error
 * limit allowed while importing data.
 * @member {object} [mappingProperties.format] The format of mapping property.
 * @member {string} [mappingProperties.format.columnDelimiter] The character
 * that signifies a break between columns.
 * @member {string} [mappingProperties.format.acceptLanguage] The oData
 * language.
 * @member {string} [mappingProperties.format.quoteCharacter] Quote character,
 * used to indicate enquoted fields.
 * @member {string} [mappingProperties.format.quoteEscapeCharacter] Escape
 * character for quotes, can be the same as the quoteCharacter.
 * @member {string} [mappingProperties.format.arraySeparator] Character
 * separating array elements.
 * @member {object} [mappingProperties.availability] The availability of
 * mapping property.
 * @member {string} [mappingProperties.availability.frequency] The frequency to
 * update. Possible values include: 'Minute', 'Hour', 'Day', 'Week', 'Month'
 * @member {number} [mappingProperties.availability.interval] The interval of
 * the given frequency to use.
 * @member {array} [mappingProperties.structure] Ingestion mapping information
 * at property level.
 * @member {object} [mappingProperties.completeOperation] The operation after
 * import is done.
 * @member {string}
 * [mappingProperties.completeOperation.completionOperationType] The type of
 * completion operation. Possible values include: 'DoNothing', 'DeleteFile',
 * 'MoveFile'
 * @member {string} [mappingProperties.completeOperation.destinationFolder] The
 * destination folder where files will be moved to once the import is done.
 * @member {date} [nextRunTime] The next run time based on customer's settings.
 * @member {string} [runId] The RunId.
 * @member {string} [state] State of connector mapping. Possible values
 * include: 'Creating', 'Created', 'Failed', 'Ready', 'Running', 'Stopped',
 * 'Expiring'
 * @member {string} [tenantId] The hub name.
 */
export interface ConnectorMappingResourceFormat extends ProxyResource {
  readonly connectorName?: string;
  connectorType?: string;
  readonly created?: Date;
  readonly lastModified?: Date;
  entityType: string;
  entityTypeName: string;
  readonly connectorMappingName?: string;
  displayName?: string;
  description?: string;
  readonly dataFormatId?: string;
  mappingProperties: ConnectorMappingProperties;
  readonly nextRunTime?: Date;
  readonly runId?: string;
  readonly state?: string;
  readonly tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the AuthorizationPolicyResourceFormat class.
 * @constructor
 * The authorization policy resource format.
 *
 * @member {string} [policyName] Name of the policy.
 * @member {array} permissions The permissions associated with the policy.
 * @member {string} [primaryKey] Primary key assiciated with the policy.
 * @member {string} [secondaryKey] Secondary key assiciated with the policy.
 */
export interface AuthorizationPolicyResourceFormat extends ProxyResource {
  readonly policyName?: string;
  permissions: string[];
  primaryKey?: string;
  secondaryKey?: string;
}

/**
 * @class
 * Initializes a new instance of the LinkResourceFormat class.
 * @constructor
 * The link resource format.
 *
 * @member {string} [tenantId] The hub name.
 * @member {string} [linkName] The link name.
 * @member {string} sourceEntityType Type of source entity. Possible values
 * include: 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {string} targetEntityType Type of target entity. Possible values
 * include: 'None', 'Profile', 'Interaction', 'Relationship'
 * @member {string} sourceEntityTypeName Name of the source Entity Type.
 * @member {string} targetEntityTypeName Name of the target Entity Type.
 * @member {object} [displayName] Localized display name for the Link.
 * @member {object} [description] Localized descriptions for the Link.
 * @member {array} [mappings] The set of properties mappings between the source
 * and target Types.
 * @member {array} participantPropertyReferences The properties that represent
 * the participating profile.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {boolean} [referenceOnly] Indicating whether the link is reference
 * only link. This flag is ingored if the Mappings are defined. If the mappings
 * are not defined and it is set to true, links processing will not create or
 * update profiles.
 * @member {string} [operationType] Determines whether this link is supposed to
 * create or delete instances if Link is NOT Reference Only. Possible values
 * include: 'Upsert', 'Delete'
 */
export interface LinkResourceFormat extends ProxyResource {
  readonly tenantId?: string;
  readonly linkName?: string;
  sourceEntityType: string;
  targetEntityType: string;
  sourceEntityTypeName: string;
  targetEntityTypeName: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  mappings?: TypePropertiesMapping[];
  participantPropertyReferences: ParticipantPropertyReference[];
  readonly provisioningState?: string;
  referenceOnly?: boolean;
  operationType?: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipResourceFormat class.
 * @constructor
 * The relationship resource format.
 *
 * @member {string} [cardinality] The Relationship Cardinality. Possible values
 * include: 'OneToOne', 'OneToMany', 'ManyToMany'
 * @member {object} [displayName] Localized display name for the Relationship.
 * @member {object} [description] Localized descriptions for the Relationship.
 * @member {date} [expiryDateTimeUtc] The expiry date time in UTC.
 * @member {array} [fields] The properties of the Relationship.
 * @member {array} [lookupMappings] Optional property to be used to map fields
 * in profile to their strong ids in related profile.
 * @member {string} profileType Profile type.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {string} [relationshipName] The Relationship name.
 * @member {string} relatedProfileType Related profile being referenced.
 * @member {string} [relationshipGuidId] The relationship guid id.
 * @member {string} [tenantId] The hub name.
 */
export interface RelationshipResourceFormat extends ProxyResource {
  cardinality?: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  expiryDateTimeUtc?: Date;
  fields?: PropertyDefinition[];
  lookupMappings?: RelationshipTypeMapping[];
  profileType: string;
  readonly provisioningState?: string;
  readonly relationshipName?: string;
  relatedProfileType: string;
  readonly relationshipGuidId?: string;
  readonly tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipLinkResourceFormat class.
 * @constructor
 * The relationship link resource format.
 *
 * @member {object} [displayName] Localized display name for the Relationship
 * Link.
 * @member {object} [description] Localized descriptions for the Relationship
 * Link.
 * @member {string} interactionType The InteractionType associated with the
 * Relationship Link.
 * @member {string} [linkName] The name of the Relationship Link.
 * @member {array} [mappings] The mappings between Interaction and Relationship
 * fields.
 * @member {array} profilePropertyReferences The property references for the
 * Profile of the Relationship.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {array} relatedProfilePropertyReferences The property references for
 * the Related Profile of the Relationship.
 * @member {string} relationshipName The Relationship associated with the Link.
 * @member {string} [relationshipGuidId] The relationship guid id.
 * @member {string} [tenantId] The hub name.
 */
export interface RelationshipLinkResourceFormat extends ProxyResource {
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  interactionType: string;
  readonly linkName?: string;
  mappings?: RelationshipLinkFieldMapping[];
  profilePropertyReferences: ParticipantProfilePropertyReference[];
  readonly provisioningState?: string;
  relatedProfilePropertyReferences: ParticipantProfilePropertyReference[];
  relationshipName: string;
  readonly relationshipGuidId?: string;
  readonly tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the ViewResourceFormat class.
 * @constructor
 * The view resource format.
 *
 * @member {string} [viewName] Name of the view.
 * @member {string} [userId] the user ID.
 * @member {string} [tenantId] the hub name.
 * @member {object} [displayName] Localized display name for the view.
 * @member {string} definition View definition.
 * @member {date} [changed] Date time when view was last modified.
 * @member {date} [created] Date time when view was created.
 */
export interface ViewResourceFormat extends ProxyResource {
  readonly viewName?: string;
  userId?: string;
  readonly tenantId?: string;
  displayName?: { [propertyName: string]: string };
  definition: string;
  readonly changed?: Date;
  readonly created?: Date;
}

/**
 * @class
 * Initializes a new instance of the WidgetTypeResourceFormat class.
 * @constructor
 * The WidgetTypeResourceFormat
 *
 * @member {string} [widgetTypeName] Name of the widget type.
 * @member {string} definition Definition for widget type.
 * @member {string} [description] Description for widget type.
 * @member {object} [displayName] Localized display name for the widget type.
 * @member {string} [imageUrl] The image URL.
 * @member {string} [tenantId] The hub name.
 * @member {string} [widgetVersion] The widget version.
 * @member {date} [changed] Date time when widget type was last modified.
 * @member {date} [created] Date time when widget type was created.
 */
export interface WidgetTypeResourceFormat extends ProxyResource {
  readonly widgetTypeName?: string;
  definition: string;
  description?: string;
  displayName?: { [propertyName: string]: string };
  imageUrl?: string;
  readonly tenantId?: string;
  widgetVersion?: string;
  readonly changed?: Date;
  readonly created?: Date;
}

/**
 * @class
 * Initializes a new instance of the AssignmentPrincipal class.
 * @constructor
 * The AssignmentPrincipal
 *
 * @member {string} principalId The principal id being assigned to.
 * @member {string} principalType The Type of the principal ID.
 * @member {object} [principalMetadata] Other metadata for the principal.
 */
export interface AssignmentPrincipal {
  principalId: string;
  principalType: string;
  principalMetadata?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ResourceSetDescription class.
 * @constructor
 * The resource set description.
 *
 * @member {array} [elements] The elements included in the set.
 * @member {array} [exceptions] The elements that are not included in the set,
 * in case elements contains '*' indicating 'all'.
 */
export interface ResourceSetDescription {
  elements?: string[];
  exceptions?: string[];
}

/**
 * @class
 * Initializes a new instance of the RoleAssignmentResourceFormat class.
 * @constructor
 * The Role Assignment resource format.
 *
 * @member {string} [tenantId] The hub name.
 * @member {string} [assignmentName] The name of the metadata object.
 * @member {object} [displayName] Localized display names for the metadata.
 * @member {object} [description] Localized description for the metadata.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {string} role Type of roles. Possible values include: 'Admin',
 * 'Reader', 'ManageAdmin', 'ManageReader', 'DataAdmin', 'DataReader'
 * @member {array} principals The principals being assigned to.
 * @member {object} [profiles] Profiles set for the assignment.
 * @member {array} [profiles.elements] The elements included in the set.
 * @member {array} [profiles.exceptions] The elements that are not included in
 * the set, in case elements contains '*' indicating 'all'.
 * @member {object} [interactions] Interactions set for the assignment.
 * @member {array} [interactions.elements] The elements included in the set.
 * @member {array} [interactions.exceptions] The elements that are not included
 * in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [links] Links set for the assignment.
 * @member {array} [links.elements] The elements included in the set.
 * @member {array} [links.exceptions] The elements that are not included in the
 * set, in case elements contains '*' indicating 'all'.
 * @member {object} [kpis] Kpis set for the assignment.
 * @member {array} [kpis.elements] The elements included in the set.
 * @member {array} [kpis.exceptions] The elements that are not included in the
 * set, in case elements contains '*' indicating 'all'.
 * @member {object} [sasPolicies] Sas Policies set for the assignment.
 * @member {array} [sasPolicies.elements] The elements included in the set.
 * @member {array} [sasPolicies.exceptions] The elements that are not included
 * in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [connectors] Connectors set for the assignment.
 * @member {array} [connectors.elements] The elements included in the set.
 * @member {array} [connectors.exceptions] The elements that are not included
 * in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [views] Views set for the assignment.
 * @member {array} [views.elements] The elements included in the set.
 * @member {array} [views.exceptions] The elements that are not included in the
 * set, in case elements contains '*' indicating 'all'.
 * @member {object} [relationshipLinks] The Role assignments set for the
 * relationship links.
 * @member {array} [relationshipLinks.elements] The elements included in the
 * set.
 * @member {array} [relationshipLinks.exceptions] The elements that are not
 * included in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [relationships] The Role assignments set for the
 * relationships.
 * @member {array} [relationships.elements] The elements included in the set.
 * @member {array} [relationships.exceptions] The elements that are not
 * included in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [widgetTypes] Widget types set for the assignment.
 * @member {array} [widgetTypes.elements] The elements included in the set.
 * @member {array} [widgetTypes.exceptions] The elements that are not included
 * in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [roleAssignments] The Role assignments set for the
 * assignment.
 * @member {array} [roleAssignments.elements] The elements included in the set.
 * @member {array} [roleAssignments.exceptions] The elements that are not
 * included in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [conflationPolicies] Widget types set for the assignment.
 * @member {array} [conflationPolicies.elements] The elements included in the
 * set.
 * @member {array} [conflationPolicies.exceptions] The elements that are not
 * included in the set, in case elements contains '*' indicating 'all'.
 * @member {object} [segments] The Role assignments set for the assignment.
 * @member {array} [segments.elements] The elements included in the set.
 * @member {array} [segments.exceptions] The elements that are not included in
 * the set, in case elements contains '*' indicating 'all'.
 */
export interface RoleAssignmentResourceFormat extends ProxyResource {
  readonly tenantId?: string;
  readonly assignmentName?: string;
  displayName?: { [propertyName: string]: string };
  description?: { [propertyName: string]: string };
  readonly provisioningState?: string;
  role: string;
  principals: AssignmentPrincipal[];
  profiles?: ResourceSetDescription;
  interactions?: ResourceSetDescription;
  links?: ResourceSetDescription;
  kpis?: ResourceSetDescription;
  sasPolicies?: ResourceSetDescription;
  connectors?: ResourceSetDescription;
  views?: ResourceSetDescription;
  relationshipLinks?: ResourceSetDescription;
  relationships?: ResourceSetDescription;
  widgetTypes?: ResourceSetDescription;
  roleAssignments?: ResourceSetDescription;
  conflationPolicies?: ResourceSetDescription;
  segments?: ResourceSetDescription;
}

/**
 * @class
 * Initializes a new instance of the RoleResourceFormat class.
 * @constructor
 * The role resource format.
 *
 * @member {string} [roleName] The role name.
 * @member {string} [description] The description of the role.
 */
export interface RoleResourceFormat extends ProxyResource {
  roleName?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the GetImageUploadUrlInput class.
 * @constructor
 * Input type for getting image upload url.
 *
 * @member {string} [entityType] Type of entity. Can be Profile or Interaction.
 * @member {string} [entityTypeName] Name of the entity type.
 * @member {string} [relativePath] Relative path of the image.
 */
export interface GetImageUploadUrlInput {
  entityType?: string;
  entityTypeName?: string;
  relativePath?: string;
}

/**
 * @class
 * Initializes a new instance of the ImageDefinition class.
 * @constructor
 * The image definition.
 *
 * @member {boolean} [imageExists] Whether image exists already.
 * @member {string} [contentUrl] Content URL for the image blob.
 * @member {string} [relativePath] Relative path of the image.
 */
export interface ImageDefinition {
  imageExists?: boolean;
  contentUrl?: string;
  relativePath?: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipsLookup class.
 * @constructor
 * The definition of suggested relationship for the type.
 *
 * @member {string} [profileName] The relationship profile.
 * @member {array} [profilePropertyReferences] The property references for the
 * profile type.
 * @member {string} [relatedProfileName] The related profile.
 * @member {array} [relatedProfilePropertyReferences] The property references
 * for the related profile type.
 * @member {string} [existingRelationshipName] The name of existing
 * Relationship.
 */
export interface RelationshipsLookup {
  readonly profileName?: string;
  readonly profilePropertyReferences?: ParticipantProfilePropertyReference[];
  readonly relatedProfileName?: string;
  readonly relatedProfilePropertyReferences?: ParticipantProfilePropertyReference[];
  readonly existingRelationshipName?: string;
}

/**
 * @class
 * Initializes a new instance of the SuggestRelationshipLinksResponse class.
 * @constructor
 * The response of suggest relationship links operation.
 *
 * @member {string} [interactionName] The interaction name.
 * @member {array} [suggestedRelationships] Suggested relationships for the
 * type.
 */
export interface SuggestRelationshipLinksResponse {
  readonly interactionName?: string;
  readonly suggestedRelationships?: RelationshipsLookup[];
}

/**
 * @class
 * Initializes a new instance of the PredictionMappings class.
 * @constructor
 * Definition of the link mapping of prediction.
 *
 * @member {string} [score] The score of the link mapping.
 * @member {string} [grade] The grade of the link mapping.
 * @member {string} [reason] The reason of the link mapping.
 */
export interface PredictionMappings {
  score?: string;
  grade?: string;
  reason?: string;
}

/**
 * @class
 * Initializes a new instance of the PredictionGradesItem class.
 * @constructor
 * The definition of a prediction grade.
 *
 * @member {string} [gradeName] Name of the grade.
 * @member {number} [minScoreThreshold] Minimum score threshold.
 * @member {number} [maxScoreThreshold] Maximum score threshold.
 */
export interface PredictionGradesItem {
  gradeName?: string;
  minScoreThreshold?: number;
  maxScoreThreshold?: number;
}

/**
 * @class
 * Initializes a new instance of the PredictionSystemGeneratedEntities class.
 * @constructor
 * System generated entities.
 *
 * @member {array} [generatedInteractionTypes] Generated interaction types.
 * @member {array} [generatedLinks] Generated links.
 * @member {object} [generatedKpis] Generated KPIs.
 */
export interface PredictionSystemGeneratedEntities {
  generatedInteractionTypes?: string[];
  generatedLinks?: string[];
  generatedKpis?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the PredictionDistributionDefinitionDistributionsItem class.
 * @constructor
 * The definition of a prediction distribution.
 *
 * @member {number} [scoreThreshold] Score threshold.
 * @member {number} [positives] Number of positives.
 * @member {number} [negatives] Number of negatives.
 * @member {number} [positivesAboveThreshold] Number of positives above
 * threshold.
 * @member {number} [negativesAboveThreshold] Number of negatives above
 * threshold.
 */
export interface PredictionDistributionDefinitionDistributionsItem {
  scoreThreshold?: number;
  positives?: number;
  negatives?: number;
  positivesAboveThreshold?: number;
  negativesAboveThreshold?: number;
}

/**
 * @class
 * Initializes a new instance of the PredictionDistributionDefinition class.
 * @constructor
 * The definition of the prediction distribution.
 *
 * @member {number} [totalPositives] Total positive in the distribution.
 * @member {number} [totalNegatives] Total negatives in the distribution.
 * @member {array} [distributions] Distributions of the prediction.
 */
export interface PredictionDistributionDefinition {
  totalPositives?: number;
  totalNegatives?: number;
  distributions?: PredictionDistributionDefinitionDistributionsItem[];
}

/**
 * @class
 * Initializes a new instance of the CanonicalProfileDefinitionPropertiesItem class.
 * @constructor
 * The definition of a canonical profile property.
 *
 * @member {string} [profileName] Profile name.
 * @member {string} [profilePropertyName] Property name of profile.
 * @member {number} [rank] The rank.
 * @member {string} [type] Type of canonical property value. Possible values
 * include: 'Numeric', 'Categorical', 'DerivedCategorical', 'DerivedNumeric'
 * @member {string} [value] Value of the canonical property.
 */
export interface CanonicalProfileDefinitionPropertiesItem {
  profileName?: string;
  profilePropertyName?: string;
  rank?: number;
  type?: string;
  value?: string;
}

/**
 * @class
 * Initializes a new instance of the CanonicalProfileDefinition class.
 * @constructor
 * Definition of canonical profile.
 *
 * @member {number} [canonicalProfileId] Canonical profile ID.
 * @member {array} [properties] Properties of the canonical profile.
 */
export interface CanonicalProfileDefinition {
  canonicalProfileId?: number;
  properties?: CanonicalProfileDefinitionPropertiesItem[];
}

/**
 * @class
 * Initializes a new instance of the PredictionTrainingResults class.
 * @constructor
 * The training results of the prediction.
 *
 * @member {string} [tenantId] The hub name.
 * @member {string} [scoreName] Score name.
 * @member {object} [predictionDistribution] Prediction distribution.
 * @member {number} [predictionDistribution.totalPositives] Total positive in
 * the distribution.
 * @member {number} [predictionDistribution.totalNegatives] Total negatives in
 * the distribution.
 * @member {array} [predictionDistribution.distributions] Distributions of the
 * prediction.
 * @member {array} [canonicalProfiles] Canonical profiles.
 * @member {number} [primaryProfileInstanceCount] Instance count of the primary
 * profile.
 */
export interface PredictionTrainingResults {
  readonly tenantId?: string;
  readonly scoreName?: string;
  readonly predictionDistribution?: PredictionDistributionDefinition;
  readonly canonicalProfiles?: CanonicalProfileDefinition[];
  readonly primaryProfileInstanceCount?: number;
}

/**
 * @class
 * Initializes a new instance of the PredictionModelStatus class.
 * @constructor
 * The prediction model status.
 *
 * @member {string} [tenantId] The hub name.
 * @member {string} [predictionName] The prediction name.
 * @member {string} [predictionGuidId] The prediction GUID ID.
 * @member {string} status Prediction model life cycle.  When prediction is in
 * PendingModelConfirmation status, it is allowed to update the status to
 * PendingFeaturing or Active through API. Possible values include: 'New',
 * 'Provisioning', 'ProvisioningFailed', 'PendingDiscovering', 'Discovering',
 * 'PendingFeaturing', 'Featuring', 'FeaturingFailed', 'PendingTraining',
 * 'Training', 'TrainingFailed', 'Evaluating', 'EvaluatingFailed',
 * 'PendingModelConfirmation', 'Active', 'Deleted', 'HumanIntervention',
 * 'Failed'
 * @member {string} [message] The model status message.
 * @member {number} [trainingSetCount] Count of the training set.
 * @member {number} [testSetCount] Count of the test set.
 * @member {number} [validationSetCount] Count of the validation set.
 * @member {number} [trainingAccuracy] The training accuracy.
 * @member {number} [signalsUsed] The singnas used.
 * @member {string} [modelVersion] Version of the model.
 */
export interface PredictionModelStatus {
  readonly tenantId?: string;
  readonly predictionName?: string;
  readonly predictionGuidId?: string;
  status: string;
  readonly message?: string;
  readonly trainingSetCount?: number;
  readonly testSetCount?: number;
  readonly validationSetCount?: number;
  readonly trainingAccuracy?: number;
  readonly signalsUsed?: number;
  readonly modelVersion?: string;
}

/**
 * @class
 * Initializes a new instance of the PredictionResourceFormat class.
 * @constructor
 * The prediction resource format.
 *
 * @member {object} [description] Description of the prediction.
 * @member {object} [displayName] Display name of the prediction.
 * @member {array} [involvedInteractionTypes] Interaction types involved in the
 * prediction.
 * @member {array} [involvedKpiTypes] KPI types involved in the prediction.
 * @member {array} [involvedRelationships] Relationships involved in the
 * prediction.
 * @member {string} negativeOutcomeExpression Negative outcome expression.
 * @member {string} positiveOutcomeExpression Positive outcome expression.
 * @member {string} primaryProfileType Primary profile type.
 * @member {string} [provisioningState] Provisioning state. Possible values
 * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
 * 'HumanIntervention', 'Failed'
 * @member {string} [predictionName] Name of the prediction.
 * @member {string} scopeExpression Scope expression.
 * @member {string} [tenantId] The hub name.
 * @member {boolean} autoAnalyze Whether do auto analyze.
 * @member {object} mappings Definition of the link mapping of prediction.
 * @member {string} [mappings.score] The score of the link mapping.
 * @member {string} [mappings.grade] The grade of the link mapping.
 * @member {string} [mappings.reason] The reason of the link mapping.
 * @member {string} scoreLabel Score label.
 * @member {array} [grades] The prediction grades.
 * @member {object} [systemGeneratedEntities] System generated entities.
 * @member {array} [systemGeneratedEntities.generatedInteractionTypes]
 * Generated interaction types.
 * @member {array} [systemGeneratedEntities.generatedLinks] Generated links.
 * @member {object} [systemGeneratedEntities.generatedKpis] Generated KPIs.
 */
export interface PredictionResourceFormat extends ProxyResource {
  description?: { [propertyName: string]: string };
  displayName?: { [propertyName: string]: string };
  involvedInteractionTypes?: string[];
  involvedKpiTypes?: string[];
  involvedRelationships?: string[];
  negativeOutcomeExpression: string;
  positiveOutcomeExpression: string;
  primaryProfileType: string;
  readonly provisioningState?: string;
  predictionName?: string;
  scopeExpression: string;
  readonly tenantId?: string;
  autoAnalyze: boolean;
  mappings: PredictionMappings;
  scoreLabel: string;
  grades?: PredictionGradesItem[];
  readonly systemGeneratedEntities?: PredictionSystemGeneratedEntities;
}


/**
 * @class
 * Initializes a new instance of the HubListResult class.
 * @constructor
 * Response of list hub operation.
 *
 * @member {string} [nextLink] Link for next set of results.
 */
export interface HubListResult extends Array<Hub> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ProfileListResult class.
 * @constructor
 * The response of list profile operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface ProfileListResult extends Array<ProfileResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the InteractionListResult class.
 * @constructor
 * The response of list interaction operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface InteractionListResult extends Array<InteractionResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipListResult class.
 * @constructor
 * The response of list relationship operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface RelationshipListResult extends Array<RelationshipResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RelationshipLinkListResult class.
 * @constructor
 * The response of list relationship link operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface RelationshipLinkListResult extends Array<RelationshipLinkResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the AuthorizationPolicyListResult class.
 * @constructor
 * The response of list authorization policy operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface AuthorizationPolicyListResult extends Array<AuthorizationPolicyResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectorListResult class.
 * @constructor
 * The response of list connector operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface ConnectorListResult extends Array<ConnectorResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ConnectorMappingListResult class.
 * @constructor
 * The response of list connector mapping operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface ConnectorMappingListResult extends Array<ConnectorMappingResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the KpiListResult class.
 * @constructor
 * The response of list KPI operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface KpiListResult extends Array<KpiResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the WidgetTypeListResult class.
 * @constructor
 * The response of list widget type operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface WidgetTypeListResult extends Array<WidgetTypeResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ViewListResult class.
 * @constructor
 * The response of list view operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface ViewListResult extends Array<ViewResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the LinkListResult class.
 * @constructor
 * The response of list link operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface LinkListResult extends Array<LinkResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleListResult class.
 * @constructor
 * The response of list role assignment operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface RoleListResult extends Array<RoleResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the RoleAssignmentListResult class.
 * @constructor
 * The response of list role assignment operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface RoleAssignmentListResult extends Array<RoleAssignmentResourceFormat> {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the PredictionListResult class.
 * @constructor
 * The response of list predictions operation.
 *
 * @member {string} [nextLink] Link to the next set of results.
 */
export interface PredictionListResult extends Array<PredictionResourceFormat> {
  nextLink?: string;
}
