/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * The profile resource format.
 *
 * @extends models['ProxyResource']
 */
class ProfileResourceFormat extends models['ProxyResource'] {
  /**
   * Create a ProfileResourceFormat.
   * @member {object} [attributes] The attributes for the Type.
   * @member {object} [description] Localized descriptions for the property.
   * @member {object} [displayName] Localized display names for the property.
   * @member {object} [localizedAttributes] Any custom localized attributes for
   * the Type.
   * @member {string} [smallImage] Small Image associated with the Property or
   * EntityType.
   * @member {string} [mediumImage] Medium Image associated with the Property
   * or EntityType.
   * @member {string} [largeImage] Large Image associated with the Property or
   * EntityType.
   * @member {string} [apiEntitySetName] The api entity set name. This becomes
   * the odata entity set name for the entity Type being refered in this
   * object.
   * @member {string} [entityType] Type of entity. Possible values include:
   * 'None', 'Profile', 'Interaction', 'Relationship'
   * @member {array} [fields] The properties of the Profile.
   * @member {number} [instancesCount] The instance count.
   * @member {date} [lastChangedUtc] The last changed time for the type
   * definition.
   * @member {string} [provisioningState] Provisioning state. Possible values
   * include: 'Provisioning', 'Succeeded', 'Expiring', 'Deleting',
   * 'HumanIntervention', 'Failed'
   * @member {string} [schemaItemTypeLink] The schema org link. This helps ACI
   * identify and suggest semantic models.
   * @member {string} [tenantId] The hub name.
   * @member {string} [timestampFieldName] The timestamp property name.
   * Represents the time when the interaction or profile update happened.
   * @member {string} [typeName] The name of the entity.
   * @member {array} [strongIds] The strong IDs.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ProfileResourceFormat
   *
   * @returns {object} metadata of ProfileResourceFormat
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ProfileResourceFormat',
      type: {
        name: 'Composite',
        className: 'ProfileResourceFormat',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          attributes: {
            required: false,
            serializedName: 'properties.attributes',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ArrayElementType',
                  type: {
                    name: 'Sequence',
                    element: {
                        required: false,
                        serializedName: 'StringElementType',
                        type: {
                          name: 'String'
                        }
                    }
                  }
              }
            }
          },
          description: {
            required: false,
            serializedName: 'properties.description',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          displayName: {
            required: false,
            serializedName: 'properties.displayName',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          localizedAttributes: {
            required: false,
            serializedName: 'properties.localizedAttributes',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ObjectElementType',
                  type: {
                    name: 'Dictionary',
                    value: {
                        required: false,
                        serializedName: 'StringElementType',
                        type: {
                          name: 'String'
                        }
                    }
                  }
              }
            }
          },
          smallImage: {
            required: false,
            serializedName: 'properties.smallImage',
            type: {
              name: 'String'
            }
          },
          mediumImage: {
            required: false,
            serializedName: 'properties.mediumImage',
            type: {
              name: 'String'
            }
          },
          largeImage: {
            required: false,
            serializedName: 'properties.largeImage',
            type: {
              name: 'String'
            }
          },
          apiEntitySetName: {
            required: false,
            serializedName: 'properties.apiEntitySetName',
            type: {
              name: 'String'
            }
          },
          entityType: {
            required: false,
            serializedName: 'properties.entityType',
            type: {
              name: 'Enum',
              allowedValues: [ 'None', 'Profile', 'Interaction', 'Relationship' ]
            }
          },
          fields: {
            required: false,
            serializedName: 'properties.fields',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'PropertyDefinitionElementType',
                  type: {
                    name: 'Composite',
                    className: 'PropertyDefinition'
                  }
              }
            }
          },
          instancesCount: {
            required: false,
            serializedName: 'properties.instancesCount',
            type: {
              name: 'Number'
            }
          },
          lastChangedUtc: {
            required: false,
            readOnly: true,
            serializedName: 'properties.lastChangedUtc',
            type: {
              name: 'DateTime'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          schemaItemTypeLink: {
            required: false,
            serializedName: 'properties.schemaItemTypeLink',
            type: {
              name: 'String'
            }
          },
          tenantId: {
            required: false,
            readOnly: true,
            serializedName: 'properties.tenantId',
            type: {
              name: 'String'
            }
          },
          timestampFieldName: {
            required: false,
            serializedName: 'properties.timestampFieldName',
            type: {
              name: 'String'
            }
          },
          typeName: {
            required: false,
            serializedName: 'properties.typeName',
            type: {
              name: 'String'
            }
          },
          strongIds: {
            required: false,
            serializedName: 'properties.strongIds',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StrongIdElementType',
                  type: {
                    name: 'Composite',
                    className: 'StrongId'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ProfileResourceFormat;
