/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A single Amazon Simple Storage Service (S3) object or a set of S3 objects.
 *
 * @extends models['Dataset']
 */
class AmazonS3Dataset extends models['Dataset'] {
  /**
   * Create a AmazonS3Dataset.
   * @member {object} bucketName The name of the Amazon S3 bucket. Type: string
   * (or Expression with resultType string).
   * @member {object} [key] The key of the Amazon S3 object. Type: string (or
   * Expression with resultType string).
   * @member {object} [prefix] The prefix filter for the S3 object name. Type:
   * string (or Expression with resultType string).
   * @member {object} [version] The version for the S3 object. Type: string (or
   * Expression with resultType string).
   * @member {object} [format] The format of files.
   * @member {object} [format.serializer] Serializer. Type: string (or
   * Expression with resultType string).
   * @member {object} [format.deserializer] Deserializer. Type: string (or
   * Expression with resultType string).
   * @member {string} [format.type] Polymorphic Discriminator
   * @member {object} [compression] The data compression method used for the
   * Amazon S3 object.
   * @member {string} [compression.type] Polymorphic Discriminator
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AmazonS3Dataset
   *
   * @returns {object} metadata of AmazonS3Dataset
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AmazonS3Object',
      type: {
        name: 'Composite',
        className: 'AmazonS3Dataset',
        modelProperties: {
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          structure: {
            required: false,
            serializedName: 'structure',
            type: {
              name: 'Object'
            }
          },
          linkedServiceName: {
            required: true,
            serializedName: 'linkedServiceName',
            defaultValue: {},
            type: {
              name: 'Composite',
              className: 'LinkedServiceReference'
            }
          },
          parameters: {
            required: false,
            serializedName: 'parameters',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'ParameterSpecificationElementType',
                  type: {
                    name: 'Composite',
                    className: 'ParameterSpecification'
                  }
              }
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          bucketName: {
            required: true,
            serializedName: 'typeProperties.bucketName',
            type: {
              name: 'Object'
            }
          },
          key: {
            required: false,
            serializedName: 'typeProperties.key',
            type: {
              name: 'Object'
            }
          },
          prefix: {
            required: false,
            serializedName: 'typeProperties.prefix',
            type: {
              name: 'Object'
            }
          },
          version: {
            required: false,
            serializedName: 'typeProperties.version',
            type: {
              name: 'Object'
            }
          },
          format: {
            required: false,
            serializedName: 'typeProperties.format',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'DatasetStorageFormat',
              className: 'DatasetStorageFormat'
            }
          },
          compression: {
            required: false,
            serializedName: 'typeProperties.compression',
            type: {
              name: 'Composite',
              polymorphicDiscriminator: {
                serializedName: 'type',
                clientName: 'type'
              },
              uberParent: 'DatasetCompression',
              className: 'DatasetCompression'
            }
          }
        }
      }
    };
  }
}

module.exports = AmazonS3Dataset;
