/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Linked service for an HTTP source.
 *
 * @extends models['LinkedService']
 */
class HttpLinkedService extends models['LinkedService'] {
  /**
   * Create a HttpLinkedService.
   * @member {object} url The base URL of the HTTP endpoint, e.g.
   * http://www.microsoft.com. Type: string (or Expression with resultType
   * string).
   * @member {string} [authenticationType] The authentication type to be used
   * to connect to the HTTP server. Possible values include: 'Basic',
   * 'Anonymous', 'Digest', 'Windows', 'ClientCertificate'
   * @member {object} [userName] User name for Basic, Digest, or Windows
   * authentication. Type: string (or Expression with resultType string).
   * @member {object} [password] Password for Basic, Digest, Windows, or
   * ClientCertificate with EmbeddedCertData authentication.
   * @member {string} [password.value] Value of secure string.
   * @member {object} [embeddedCertData] Base64 encoded certificate data for
   * ClientCertificate authentication. For on-premises copy with
   * ClientCertificate authentication, either CertThumbprint or
   * EmbeddedCertData/Password should be specified. Type: string (or Expression
   * with resultType string).
   * @member {object} [certThumbprint] Thumbprint of certificate for
   * ClientCertificate authentication. Only valid for on-premises copy. For
   * on-premises copy with ClientCertificate authentication, either
   * CertThumbprint or EmbeddedCertData/Password should be specified. Type:
   * string (or Expression with resultType string).
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   * @member {object} [enableServerCertificateValidation] If true, validate the
   * HTTPS server SSL certificate. Default value is true. Type: boolean (or
   * Expression with resultType boolean).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of HttpLinkedService
   *
   * @returns {object} metadata of HttpLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'HttpServer',
      type: {
        name: 'Composite',
        className: 'HttpLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          url: {
            required: true,
            serializedName: 'typeProperties.url',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: false,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'String'
            }
          },
          userName: {
            required: false,
            serializedName: 'typeProperties.userName',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              className: 'SecureString'
            }
          },
          embeddedCertData: {
            required: false,
            serializedName: 'typeProperties.embeddedCertData',
            type: {
              name: 'Object'
            }
          },
          certThumbprint: {
            required: false,
            serializedName: 'typeProperties.certThumbprint',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          },
          enableServerCertificateValidation: {
            required: false,
            serializedName: 'typeProperties.enableServerCertificateValidation',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = HttpLinkedService;
