/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Linked service for MongoDb data source.
 *
 * @extends models['LinkedService']
 */
class MongoDbLinkedService extends models['LinkedService'] {
  /**
   * Create a MongoDbLinkedService.
   * @member {object} server The IP address or server name of the MongoDB
   * server. Type: string (or Expression with resultType string).
   * @member {string} [authenticationType] The authentication type to be used
   * to connect to the MongoDB database. Possible values include: 'Basic',
   * 'Anonymous'
   * @member {object} databaseName The name of the MongoDB database that you
   * want to access. Type: string (or Expression with resultType string).
   * @member {object} [username] Username for authentication. Type: string (or
   * Expression with resultType string).
   * @member {object} [password] Password for authentication.
   * @member {string} [password.value] Value of secure string.
   * @member {object} [authSource] Database to verify the username and
   * password. Type: string (or Expression with resultType string).
   * @member {object} [port] The TCP port number that the MongoDB server uses
   * to listen for client connections. The default value is 27017. Type:
   * integer (or Expression with resultType integer), minimum: 0.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of MongoDbLinkedService
   *
   * @returns {object} metadata of MongoDbLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'MongoDb',
      type: {
        name: 'Composite',
        className: 'MongoDbLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          server: {
            required: true,
            serializedName: 'typeProperties.server',
            type: {
              name: 'Object'
            }
          },
          authenticationType: {
            required: false,
            serializedName: 'typeProperties.authenticationType',
            type: {
              name: 'String'
            }
          },
          databaseName: {
            required: true,
            serializedName: 'typeProperties.databaseName',
            type: {
              name: 'Object'
            }
          },
          username: {
            required: false,
            serializedName: 'typeProperties.username',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              className: 'SecureString'
            }
          },
          authSource: {
            required: false,
            serializedName: 'typeProperties.authSource',
            type: {
              name: 'Object'
            }
          },
          port: {
            required: false,
            serializedName: 'typeProperties.port',
            type: {
              name: 'Object'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = MongoDbLinkedService;
