/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Linked service for Salesforce.
 *
 * @extends models['LinkedService']
 */
class SalesforceLinkedService extends models['LinkedService'] {
  /**
   * Create a SalesforceLinkedService.
   * @member {object} [environmentUrl] The URL of Salesforce instance. Default
   * is 'https://login.salesforce.com'. To copy data from sandbox, specify
   * 'https://test.salesforce.com'. To copy data from custom domain, specify,
   * for example, 'https://[domain].my.salesforce.com'. Type: string (or
   * Expression with resultType string).
   * @member {object} [username] The username for Basic authentication of the
   * Salesforce source. Type: string (or Expression with resultType string).
   * @member {object} [password] The password for Basic authentication of the
   * Salesforce source.
   * @member {string} [password.value] Value of secure string.
   * @member {object} [securityToken] The security token is required to
   * remotely access Salesforce source.
   * @member {string} [securityToken.value] Value of secure string.
   * @member {object} [encryptedCredential] The encrypted credential used for
   * authentication. Credentials are encrypted using the integration runtime
   * credential manager. Type: string (or Expression with resultType string).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of SalesforceLinkedService
   *
   * @returns {object} metadata of SalesforceLinkedService
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Salesforce',
      type: {
        name: 'Composite',
        className: 'SalesforceLinkedService',
        modelProperties: {
          connectVia: {
            required: false,
            serializedName: 'connectVia',
            type: {
              name: 'Composite',
              className: 'IntegrationRuntimeReference'
            }
          },
          description: {
            required: false,
            serializedName: 'description',
            type: {
              name: 'String'
            }
          },
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          environmentUrl: {
            required: false,
            serializedName: 'typeProperties.environmentUrl',
            type: {
              name: 'Object'
            }
          },
          username: {
            required: false,
            serializedName: 'typeProperties.username',
            type: {
              name: 'Object'
            }
          },
          password: {
            required: false,
            serializedName: 'typeProperties.password',
            type: {
              name: 'Composite',
              className: 'SecureString'
            }
          },
          securityToken: {
            required: false,
            serializedName: 'typeProperties.securityToken',
            type: {
              name: 'Composite',
              className: 'SecureString'
            }
          },
          encryptedCredential: {
            required: false,
            serializedName: 'typeProperties.encryptedCredential',
            type: {
              name: 'Object'
            }
          }
        }
      }
    };
  }
}

module.exports = SalesforceLinkedService;
