/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A formula for creating a VM, specifying an image base and other parameters
 *
 * @extends models['Resource']
 */
class Formula extends models['Resource'] {
  /**
   * Create a Formula.
   * @member {string} [description] The description of the formula.
   * @member {string} [author] The author of the formula.
   * @member {string} [osType] The OS type of the formula.
   * @member {date} [creationDate] The creation date of the formula.
   * @member {object} [formulaContent] The content of the formula.
   * @member {object} [formulaContent.bulkCreationParameters] The number of
   * virtual machine instances to create.
   * @member {number} [formulaContent.bulkCreationParameters.instanceCount] The
   * number of virtual machine instances to create.
   * @member {string} [formulaContent.notes] The notes of the virtual machine.
   * @member {string} [formulaContent.ownerObjectId] The object identifier of
   * the owner of the virtual machine.
   * @member {string} [formulaContent.ownerUserPrincipalName] The user
   * principal name of the virtual machine owner.
   * @member {string} [formulaContent.createdByUserId] The object identifier of
   * the creator of the virtual machine.
   * @member {string} [formulaContent.createdByUser] The email address of
   * creator of the virtual machine.
   * @member {date} [formulaContent.createdDate] The creation date of the
   * virtual machine.
   * @member {string} [formulaContent.customImageId] The custom image
   * identifier of the virtual machine.
   * @member {string} [formulaContent.osType] The OS type of the virtual
   * machine.
   * @member {string} [formulaContent.size] The size of the virtual machine.
   * @member {string} [formulaContent.userName] The user name of the virtual
   * machine.
   * @member {string} [formulaContent.password] The password of the virtual
   * machine administrator.
   * @member {string} [formulaContent.sshKey] The SSH key of the virtual
   * machine administrator.
   * @member {boolean} [formulaContent.isAuthenticationWithSshKey] Indicates
   * whether this virtual machine uses an SSH key for authentication.
   * @member {string} [formulaContent.fqdn] The fully-qualified domain name of
   * the virtual machine.
   * @member {string} [formulaContent.labSubnetName] The lab subnet name of the
   * virtual machine.
   * @member {string} [formulaContent.labVirtualNetworkId] The lab virtual
   * network identifier of the virtual machine.
   * @member {boolean} [formulaContent.disallowPublicIpAddress] Indicates
   * whether the virtual machine is to be created without a public IP address.
   * @member {array} [formulaContent.artifacts] The artifacts to be installed
   * on the virtual machine.
   * @member {object} [formulaContent.artifactDeploymentStatus] The artifact
   * deployment status for the virtual machine.
   * @member {string}
   * [formulaContent.artifactDeploymentStatus.deploymentStatus] The deployment
   * status of the artifact.
   * @member {number}
   * [formulaContent.artifactDeploymentStatus.artifactsApplied] The total count
   * of the artifacts that were successfully applied.
   * @member {number} [formulaContent.artifactDeploymentStatus.totalArtifacts]
   * The total count of the artifacts that were tentatively applied.
   * @member {object} [formulaContent.galleryImageReference] The Microsoft
   * Azure Marketplace image reference of the virtual machine.
   * @member {string} [formulaContent.galleryImageReference.offer] The offer of
   * the gallery image.
   * @member {string} [formulaContent.galleryImageReference.publisher] The
   * publisher of the gallery image.
   * @member {string} [formulaContent.galleryImageReference.sku] The SKU of the
   * gallery image.
   * @member {string} [formulaContent.galleryImageReference.osType] The OS type
   * of the gallery image.
   * @member {string} [formulaContent.galleryImageReference.version] The
   * version of the gallery image.
   * @member {object} [formulaContent.computeVm] The compute virtual machine
   * properties.
   * @member {array} [formulaContent.computeVm.statuses] Gets the statuses of
   * the virtual machine.
   * @member {string} [formulaContent.computeVm.osType] Gets the OS type of the
   * virtual machine.
   * @member {string} [formulaContent.computeVm.vmSize] Gets the size of the
   * virtual machine.
   * @member {string} [formulaContent.computeVm.networkInterfaceId] Gets the
   * network interface ID of the virtual machine.
   * @member {string} [formulaContent.computeVm.osDiskId] Gets OS disk blob uri
   * for the virtual machine.
   * @member {array} [formulaContent.computeVm.dataDiskIds] Gets data disks
   * blob uri for the virtual machine.
   * @member {array} [formulaContent.computeVm.dataDisks] Gets all data disks
   * attached to the virtual machine.
   * @member {object} [formulaContent.networkInterface] The network interface
   * properties.
   * @member {string} [formulaContent.networkInterface.virtualNetworkId] The
   * resource ID of the virtual network.
   * @member {string} [formulaContent.networkInterface.subnetId] The resource
   * ID of the sub net.
   * @member {string} [formulaContent.networkInterface.publicIpAddressId] The
   * resource ID of the public IP address.
   * @member {string} [formulaContent.networkInterface.publicIpAddress] The
   * public IP address.
   * @member {string} [formulaContent.networkInterface.privateIpAddress] The
   * private IP address.
   * @member {string} [formulaContent.networkInterface.dnsName] The DNS name.
   * @member {string} [formulaContent.networkInterface.rdpAuthority] The
   * RdpAuthority property is a server DNS host name or IP address followed by
   * the service port number for RDP (Remote Desktop Protocol).
   * @member {string} [formulaContent.networkInterface.sshAuthority] The
   * SshAuthority property is a server DNS host name or IP address followed by
   * the service port number for SSH.
   * @member {object}
   * [formulaContent.networkInterface.sharedPublicIpAddressConfiguration] The
   * configuration for sharing a public IP address across multiple virtual
   * machines.
   * @member {array}
   * [formulaContent.networkInterface.sharedPublicIpAddressConfiguration.inboundNatRules]
   * The incoming NAT rules
   * @member {object} [formulaContent.applicableSchedule] The applicable
   * schedule for the virtual machine.
   * @member {object} [formulaContent.applicableSchedule.labVmsShutdown] The
   * auto-shutdown schedule, if one has been set at the lab or lab resource
   * level.
   * @member {string} [formulaContent.applicableSchedule.labVmsShutdown.status]
   * The status of the schedule (i.e. Enabled, Disabled). Possible values
   * include: 'Enabled', 'Disabled'
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.taskType] The task type
   * of the schedule (e.g. LabVmsShutdownTask, LabVmAutoStart).
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsShutdown.weeklyRecurrence] If the
   * schedule will occur only some days of the week, specify the weekly
   * recurrence.
   * @member {array}
   * [formulaContent.applicableSchedule.labVmsShutdown.weeklyRecurrence.weekdays]
   * The days of the week for which the schedule is set (e.g. Sunday, Monday,
   * Tuesday, etc.).
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.weeklyRecurrence.time]
   * The time of the day the schedule will occur.
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsShutdown.dailyRecurrence] If the
   * schedule will occur once each day of the week, specify the daily
   * recurrence.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.dailyRecurrence.time]
   * The time of day the schedule will occur.
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsShutdown.hourlyRecurrence] If the
   * schedule will occur multiple times a day, specify the hourly recurrence.
   * @member {number}
   * [formulaContent.applicableSchedule.labVmsShutdown.hourlyRecurrence.minute]
   * Minutes of the hour the schedule will run.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.timeZoneId] The time
   * zone ID (e.g. Pacific Standard time).
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsShutdown.notificationSettings]
   * Notification settings.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.notificationSettings.status]
   * If notifications are enabled for this schedule (i.e. Enabled, Disabled).
   * Possible values include: 'Disabled', 'Enabled'
   * @member {number}
   * [formulaContent.applicableSchedule.labVmsShutdown.notificationSettings.timeInMinutes]
   * Time in minutes before event at which notification will be sent.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.notificationSettings.webhookUrl]
   * The webhook URL to which the notification will be sent.
   * @member {date}
   * [formulaContent.applicableSchedule.labVmsShutdown.createdDate] The
   * creation date of the schedule.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.targetResourceId] The
   * resource ID to which the schedule belongs
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.provisioningState] The
   * provisioning status of the resource.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsShutdown.uniqueIdentifier] The
   * unique immutable identifier of a resource (Guid).
   * @member {object} [formulaContent.applicableSchedule.labVmsStartup] The
   * auto-startup schedule, if one has been set at the lab or lab resource
   * level.
   * @member {string} [formulaContent.applicableSchedule.labVmsStartup.status]
   * The status of the schedule (i.e. Enabled, Disabled). Possible values
   * include: 'Enabled', 'Disabled'
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.taskType] The task type
   * of the schedule (e.g. LabVmsShutdownTask, LabVmAutoStart).
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsStartup.weeklyRecurrence] If the
   * schedule will occur only some days of the week, specify the weekly
   * recurrence.
   * @member {array}
   * [formulaContent.applicableSchedule.labVmsStartup.weeklyRecurrence.weekdays]
   * The days of the week for which the schedule is set (e.g. Sunday, Monday,
   * Tuesday, etc.).
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.weeklyRecurrence.time]
   * The time of the day the schedule will occur.
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsStartup.dailyRecurrence] If the
   * schedule will occur once each day of the week, specify the daily
   * recurrence.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.dailyRecurrence.time] The
   * time of day the schedule will occur.
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsStartup.hourlyRecurrence] If the
   * schedule will occur multiple times a day, specify the hourly recurrence.
   * @member {number}
   * [formulaContent.applicableSchedule.labVmsStartup.hourlyRecurrence.minute]
   * Minutes of the hour the schedule will run.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.timeZoneId] The time zone
   * ID (e.g. Pacific Standard time).
   * @member {object}
   * [formulaContent.applicableSchedule.labVmsStartup.notificationSettings]
   * Notification settings.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.notificationSettings.status]
   * If notifications are enabled for this schedule (i.e. Enabled, Disabled).
   * Possible values include: 'Disabled', 'Enabled'
   * @member {number}
   * [formulaContent.applicableSchedule.labVmsStartup.notificationSettings.timeInMinutes]
   * Time in minutes before event at which notification will be sent.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.notificationSettings.webhookUrl]
   * The webhook URL to which the notification will be sent.
   * @member {date}
   * [formulaContent.applicableSchedule.labVmsStartup.createdDate] The creation
   * date of the schedule.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.targetResourceId] The
   * resource ID to which the schedule belongs
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.provisioningState] The
   * provisioning status of the resource.
   * @member {string}
   * [formulaContent.applicableSchedule.labVmsStartup.uniqueIdentifier] The
   * unique immutable identifier of a resource (Guid).
   * @member {date} [formulaContent.expirationDate] The expiration date for VM.
   * @member {boolean} [formulaContent.allowClaim] Indicates whether another
   * user can take ownership of the virtual machine
   * @member {string} [formulaContent.storageType] Storage type to use for
   * virtual machine (i.e. Standard, Premium).
   * @member {string} [formulaContent.virtualMachineCreationSource] Tells
   * source of creation of lab virtual machine. Output property only. Possible
   * values include: 'FromCustomImage', 'FromGalleryImage'
   * @member {string} [formulaContent.environmentId] The resource ID of the
   * environment that contains this virtual machine, if any.
   * @member {string} [formulaContent.provisioningState] The provisioning
   * status of the resource.
   * @member {string} [formulaContent.uniqueIdentifier] The unique immutable
   * identifier of a resource (Guid).
   * @member {string} [formulaContent.name] The name of the virtual machine or
   * environment
   * @member {string} [formulaContent.location] The location of the new virtual
   * machine or environment
   * @member {object} [formulaContent.tags] The tags of the resource.
   * @member {object} [vm] Information about a VM from which a formula is to be
   * created.
   * @member {string} [vm.labVmId] The identifier of the VM from which a
   * formula is to be created.
   * @member {string} [provisioningState] The provisioning status of the
   * resource.
   * @member {string} [uniqueIdentifier] The unique immutable identifier of a
   * resource (Guid).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Formula
   *
   * @returns {object} metadata of Formula
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Formula',
      type: {
        name: 'Composite',
        className: 'Formula',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          description: {
            required: false,
            serializedName: 'properties.description',
            type: {
              name: 'String'
            }
          },
          author: {
            required: false,
            serializedName: 'properties.author',
            type: {
              name: 'String'
            }
          },
          osType: {
            required: false,
            serializedName: 'properties.osType',
            type: {
              name: 'String'
            }
          },
          creationDate: {
            required: false,
            readOnly: true,
            serializedName: 'properties.creationDate',
            type: {
              name: 'DateTime'
            }
          },
          formulaContent: {
            required: false,
            serializedName: 'properties.formulaContent',
            type: {
              name: 'Composite',
              className: 'LabVirtualMachineCreationParameter'
            }
          },
          vm: {
            required: false,
            serializedName: 'properties.vm',
            type: {
              name: 'Composite',
              className: 'FormulaPropertiesFromVm'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          uniqueIdentifier: {
            required: false,
            serializedName: 'properties.uniqueIdentifier',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = Formula;
