/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A lab.
 *
 * @extends models['Resource']
 */
class Lab extends models['Resource'] {
  /**
   * Create a Lab.
   * @member {string} [defaultStorageAccount] The lab's default storage
   * account.
   * @member {string} [defaultPremiumStorageAccount] The lab's default premium
   * storage account.
   * @member {string} [artifactsStorageAccount] The lab's artifact storage
   * account.
   * @member {string} [premiumDataDiskStorageAccount] The lab's premium data
   * disk storage account.
   * @member {string} [vaultName] The lab's Key vault.
   * @member {string} [labStorageType] Type of storage used by the lab. It can
   * be either Premium or Standard. Default is Premium. Possible values
   * include: 'Standard', 'Premium'
   * @member {date} [createdDate] The creation date of the lab.
   * @member {string} [premiumDataDisks] The setting to enable usage of premium
   * data disks.
   * When its value is 'Enabled', creation of standard or premium data disks is
   * allowed.
   * When its value is 'Disabled', only creation of standard data disks is
   * allowed. Possible values include: 'Disabled', 'Enabled'
   * @member {string} [provisioningState] The provisioning status of the
   * resource.
   * @member {string} [uniqueIdentifier] The unique immutable identifier of a
   * resource (Guid).
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Lab
   *
   * @returns {object} metadata of Lab
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Lab',
      type: {
        name: 'Composite',
        className: 'Lab',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          defaultStorageAccount: {
            required: false,
            readOnly: true,
            serializedName: 'properties.defaultStorageAccount',
            type: {
              name: 'String'
            }
          },
          defaultPremiumStorageAccount: {
            required: false,
            readOnly: true,
            serializedName: 'properties.defaultPremiumStorageAccount',
            type: {
              name: 'String'
            }
          },
          artifactsStorageAccount: {
            required: false,
            readOnly: true,
            serializedName: 'properties.artifactsStorageAccount',
            type: {
              name: 'String'
            }
          },
          premiumDataDiskStorageAccount: {
            required: false,
            readOnly: true,
            serializedName: 'properties.premiumDataDiskStorageAccount',
            type: {
              name: 'String'
            }
          },
          vaultName: {
            required: false,
            readOnly: true,
            serializedName: 'properties.vaultName',
            type: {
              name: 'String'
            }
          },
          labStorageType: {
            required: false,
            serializedName: 'properties.labStorageType',
            type: {
              name: 'String'
            }
          },
          createdDate: {
            required: false,
            readOnly: true,
            serializedName: 'properties.createdDate',
            type: {
              name: 'DateTime'
            }
          },
          premiumDataDisks: {
            required: false,
            serializedName: 'properties.premiumDataDisks',
            type: {
              name: 'String'
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          uniqueIdentifier: {
            required: false,
            serializedName: 'properties.uniqueIdentifier',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = Lab;
