/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * An action group resource.
 *
 * @extends models['Resource']
 */
class ActionGroupResource extends models['Resource'] {
  /**
   * Create a ActionGroupResource.
   * @member {string} groupShortName The short name of the action group. This
   * will be used in SMS messages.
   * @member {boolean} enabled Indicates whether this action group is enabled.
   * If an action group is not enabled, then none of its receivers will receive
   * communications. Default value: true .
   * @member {array} [emailReceivers] The list of email receivers that are part
   * of this action group.
   * @member {array} [smsReceivers] The list of SMS receivers that are part of
   * this action group.
   * @member {array} [webhookReceivers] The list of webhook receivers that are
   * part of this action group.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ActionGroupResource
   *
   * @returns {object} metadata of ActionGroupResource
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ActionGroupResource',
      type: {
        name: 'Composite',
        className: 'ActionGroupResource',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          groupShortName: {
            required: true,
            serializedName: 'properties.groupShortName',
            constraints: {
              MaxLength: 15
            },
            type: {
              name: 'String'
            }
          },
          enabled: {
            required: true,
            serializedName: 'properties.enabled',
            defaultValue: true,
            type: {
              name: 'Boolean'
            }
          },
          emailReceivers: {
            required: false,
            serializedName: 'properties.emailReceivers',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'EmailReceiverElementType',
                  type: {
                    name: 'Composite',
                    className: 'EmailReceiver'
                  }
              }
            }
          },
          smsReceivers: {
            required: false,
            serializedName: 'properties.smsReceivers',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'SmsReceiverElementType',
                  type: {
                    name: 'Composite',
                    className: 'SmsReceiver'
                  }
              }
            }
          },
          webhookReceivers: {
            required: false,
            serializedName: 'properties.webhookReceivers',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'WebhookReceiverElementType',
                  type: {
                    name: 'Composite',
                    className: 'WebhookReceiver'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ActionGroupResource;
