/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * IPConfiguration in a network interface.
 *
 * @extends models['SubResource']
 */
class NetworkInterfaceIPConfiguration extends models['SubResource'] {
  /**
   * Create a NetworkInterfaceIPConfiguration.
   * @member {array} [applicationGatewayBackendAddressPools] The reference of
   * ApplicationGatewayBackendAddressPool resource.
   * @member {array} [loadBalancerBackendAddressPools] The reference of
   * LoadBalancerBackendAddressPool resource.
   * @member {array} [loadBalancerInboundNatRules] A list of references of
   * LoadBalancerInboundNatRules.
   * @member {string} [privateIPAddress] Private IP address of the IP
   * configuration.
   * @member {string} [privateIPAllocationMethod] Defines how a private IP
   * address is assigned. Possible values are: 'Static' and 'Dynamic'. Possible
   * values include: 'Static', 'Dynamic'
   * @member {string} [privateIPAddressVersion] Available from Api-Version
   * 2016-03-30 onwards, it represents whether the specific ipconfiguration is
   * IPv4 or IPv6. Default is taken as IPv4.  Possible values are: 'IPv4' and
   * 'IPv6'. Possible values include: 'IPv4', 'IPv6'
   * @member {object} [subnet] Subnet bound to the IP configuration.
   * @member {string} [subnet.addressPrefix] The address prefix for the subnet.
   * @member {object} [subnet.networkSecurityGroup] The reference of the
   * NetworkSecurityGroup resource.
   * @member {array} [subnet.networkSecurityGroup.securityRules] A collection
   * of security rules of the network security group.
   * @member {array} [subnet.networkSecurityGroup.defaultSecurityRules] The
   * default security rules of network security group.
   * @member {array} [subnet.networkSecurityGroup.networkInterfaces] A
   * collection of references to network interfaces.
   * @member {array} [subnet.networkSecurityGroup.subnets] A collection of
   * references to subnets.
   * @member {string} [subnet.networkSecurityGroup.resourceGuid] The resource
   * GUID property of the network security group resource.
   * @member {string} [subnet.networkSecurityGroup.provisioningState] The
   * provisioning state of the public IP resource. Possible values are:
   * 'Updating', 'Deleting', and 'Failed'.
   * @member {string} [subnet.networkSecurityGroup.etag] A unique read-only
   * string that changes whenever the resource is updated.
   * @member {object} [subnet.routeTable] The reference of the RouteTable
   * resource.
   * @member {array} [subnet.routeTable.routes] Collection of routes contained
   * within a route table.
   * @member {array} [subnet.routeTable.subnets] A collection of references to
   * subnets.
   * @member {string} [subnet.routeTable.provisioningState] The provisioning
   * state of the resource. Possible values are: 'Updating', 'Deleting', and
   * 'Failed'.
   * @member {string} [subnet.routeTable.etag] Gets a unique read-only string
   * that changes whenever the resource is updated.
   * @member {array} [subnet.serviceEndpoints] An array of service endpoints.
   * @member {array} [subnet.ipConfigurations] Gets an array of references to
   * the network interface IP configurations using subnet.
   * @member {array} [subnet.resourceNavigationLinks] Gets an array of
   * references to the external resources using subnet.
   * @member {string} [subnet.provisioningState] The provisioning state of the
   * resource.
   * @member {string} [subnet.name] The name of the resource that is unique
   * within a resource group. This name can be used to access the resource.
   * @member {string} [subnet.etag] A unique read-only string that changes
   * whenever the resource is updated.
   * @member {boolean} [primary] Gets whether this is a primary customer
   * address on the network interface.
   * @member {object} [publicIPAddress] Public IP address bound to the IP
   * configuration.
   * @member {object} [publicIPAddress.sku] The public IP address SKU.
   * @member {string} [publicIPAddress.sku.name] Name of a public IP address
   * SKU. Possible values include: 'Basic', 'Standard'
   * @member {string} [publicIPAddress.publicIPAllocationMethod] The public IP
   * allocation method. Possible values are: 'Static' and 'Dynamic'. Possible
   * values include: 'Static', 'Dynamic'
   * @member {string} [publicIPAddress.publicIPAddressVersion] The public IP
   * address version. Possible values are: 'IPv4' and 'IPv6'. Possible values
   * include: 'IPv4', 'IPv6'
   * @member {object} [publicIPAddress.ipConfiguration] The IP configuration
   * associated with the public IP address.
   * @member {string} [publicIPAddress.ipConfiguration.privateIPAddress] The
   * private IP address of the IP configuration.
   * @member {string}
   * [publicIPAddress.ipConfiguration.privateIPAllocationMethod] The private IP
   * allocation method. Possible values are 'Static' and 'Dynamic'. Possible
   * values include: 'Static', 'Dynamic'
   * @member {object} [publicIPAddress.ipConfiguration.subnet] The reference of
   * the subnet resource.
   * @member {string} [publicIPAddress.ipConfiguration.subnet.addressPrefix]
   * The address prefix for the subnet.
   * @member {object}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup] The
   * reference of the NetworkSecurityGroup resource.
   * @member {array}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.securityRules]
   * A collection of security rules of the network security group.
   * @member {array}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.defaultSecurityRules]
   * The default security rules of network security group.
   * @member {array}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.networkInterfaces]
   * A collection of references to network interfaces.
   * @member {array}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.subnets] A
   * collection of references to subnets.
   * @member {string}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.resourceGuid]
   * The resource GUID property of the network security group resource.
   * @member {string}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.provisioningState]
   * The provisioning state of the public IP resource. Possible values are:
   * 'Updating', 'Deleting', and 'Failed'.
   * @member {string}
   * [publicIPAddress.ipConfiguration.subnet.networkSecurityGroup.etag] A
   * unique read-only string that changes whenever the resource is updated.
   * @member {object} [publicIPAddress.ipConfiguration.subnet.routeTable] The
   * reference of the RouteTable resource.
   * @member {array} [publicIPAddress.ipConfiguration.subnet.routeTable.routes]
   * Collection of routes contained within a route table.
   * @member {array}
   * [publicIPAddress.ipConfiguration.subnet.routeTable.subnets] A collection
   * of references to subnets.
   * @member {string}
   * [publicIPAddress.ipConfiguration.subnet.routeTable.provisioningState] The
   * provisioning state of the resource. Possible values are: 'Updating',
   * 'Deleting', and 'Failed'.
   * @member {string} [publicIPAddress.ipConfiguration.subnet.routeTable.etag]
   * Gets a unique read-only string that changes whenever the resource is
   * updated.
   * @member {array} [publicIPAddress.ipConfiguration.subnet.serviceEndpoints]
   * An array of service endpoints.
   * @member {array} [publicIPAddress.ipConfiguration.subnet.ipConfigurations]
   * Gets an array of references to the network interface IP configurations
   * using subnet.
   * @member {array}
   * [publicIPAddress.ipConfiguration.subnet.resourceNavigationLinks] Gets an
   * array of references to the external resources using subnet.
   * @member {string}
   * [publicIPAddress.ipConfiguration.subnet.provisioningState] The
   * provisioning state of the resource.
   * @member {string} [publicIPAddress.ipConfiguration.subnet.name] The name of
   * the resource that is unique within a resource group. This name can be used
   * to access the resource.
   * @member {string} [publicIPAddress.ipConfiguration.subnet.etag] A unique
   * read-only string that changes whenever the resource is updated.
   * @member {object} [publicIPAddress.ipConfiguration.publicIPAddress] The
   * reference of the public IP resource.
   * @member {string} [publicIPAddress.ipConfiguration.provisioningState] Gets
   * the provisioning state of the public IP resource. Possible values are:
   * 'Updating', 'Deleting', and 'Failed'.
   * @member {string} [publicIPAddress.ipConfiguration.name] The name of the
   * resource that is unique within a resource group. This name can be used to
   * access the resource.
   * @member {string} [publicIPAddress.ipConfiguration.etag] A unique read-only
   * string that changes whenever the resource is updated.
   * @member {object} [publicIPAddress.dnsSettings] The FQDN of the DNS record
   * associated with the public IP address.
   * @member {string} [publicIPAddress.dnsSettings.domainNameLabel] Gets or
   * sets the Domain name label.The concatenation of the domain name label and
   * the regionalized DNS zone make up the fully qualified domain name
   * associated with the public IP address. If a domain name label is
   * specified, an A DNS record is created for the public IP in the Microsoft
   * Azure DNS system.
   * @member {string} [publicIPAddress.dnsSettings.fqdn] Gets the FQDN, Fully
   * qualified domain name of the A DNS record associated with the public IP.
   * This is the concatenation of the domainNameLabel and the regionalized DNS
   * zone.
   * @member {string} [publicIPAddress.dnsSettings.reverseFqdn] Gets or Sets
   * the Reverse FQDN. A user-visible, fully qualified domain name that
   * resolves to this public IP address. If the reverseFqdn is specified, then
   * a PTR DNS record is created pointing from the IP address in the
   * in-addr.arpa domain to the reverse FQDN.
   * @member {string} [publicIPAddress.ipAddress] The IP address associated
   * with the public IP address resource.
   * @member {number} [publicIPAddress.idleTimeoutInMinutes] The idle timeout
   * of the public IP address.
   * @member {string} [publicIPAddress.resourceGuid] The resource GUID property
   * of the public IP resource.
   * @member {string} [publicIPAddress.provisioningState] The provisioning
   * state of the PublicIP resource. Possible values are: 'Updating',
   * 'Deleting', and 'Failed'.
   * @member {string} [publicIPAddress.etag] A unique read-only string that
   * changes whenever the resource is updated.
   * @member {array} [publicIPAddress.zones] A list of availability zones
   * denoting the IP allocated for the resource needs to come from.
   * @member {array} [applicationSecurityGroups] Application security groups in
   * which the IP configuration is included.
   * @member {string} [provisioningState] The provisioning state of the network
   * interface IP configuration. Possible values are: 'Updating', 'Deleting',
   * and 'Failed'.
   * @member {string} [name] The name of the resource that is unique within a
   * resource group. This name can be used to access the resource.
   * @member {string} [etag] A unique read-only string that changes whenever
   * the resource is updated.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of NetworkInterfaceIPConfiguration
   *
   * @returns {object} metadata of NetworkInterfaceIPConfiguration
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'NetworkInterfaceIPConfiguration',
      type: {
        name: 'Composite',
        className: 'NetworkInterfaceIPConfiguration',
        modelProperties: {
          id: {
            required: false,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          applicationGatewayBackendAddressPools: {
            required: false,
            serializedName: 'properties.applicationGatewayBackendAddressPools',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationGatewayBackendAddressPoolElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationGatewayBackendAddressPool'
                  }
              }
            }
          },
          loadBalancerBackendAddressPools: {
            required: false,
            serializedName: 'properties.loadBalancerBackendAddressPools',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'BackendAddressPoolElementType',
                  type: {
                    name: 'Composite',
                    className: 'BackendAddressPool'
                  }
              }
            }
          },
          loadBalancerInboundNatRules: {
            required: false,
            serializedName: 'properties.loadBalancerInboundNatRules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'InboundNatRuleElementType',
                  type: {
                    name: 'Composite',
                    className: 'InboundNatRule'
                  }
              }
            }
          },
          privateIPAddress: {
            required: false,
            serializedName: 'properties.privateIPAddress',
            type: {
              name: 'String'
            }
          },
          privateIPAllocationMethod: {
            required: false,
            serializedName: 'properties.privateIPAllocationMethod',
            type: {
              name: 'String'
            }
          },
          privateIPAddressVersion: {
            required: false,
            serializedName: 'properties.privateIPAddressVersion',
            type: {
              name: 'String'
            }
          },
          subnet: {
            required: false,
            serializedName: 'properties.subnet',
            type: {
              name: 'Composite',
              className: 'Subnet'
            }
          },
          primary: {
            required: false,
            serializedName: 'properties.primary',
            type: {
              name: 'Boolean'
            }
          },
          publicIPAddress: {
            required: false,
            serializedName: 'properties.publicIPAddress',
            type: {
              name: 'Composite',
              className: 'PublicIPAddress'
            }
          },
          applicationSecurityGroups: {
            required: false,
            serializedName: 'properties.applicationSecurityGroups',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ApplicationSecurityGroupElementType',
                  type: {
                    name: 'Composite',
                    className: 'ApplicationSecurityGroup'
                  }
              }
            }
          },
          provisioningState: {
            required: false,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          etag: {
            required: false,
            serializedName: 'etag',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = NetworkInterfaceIPConfiguration;
