/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback, HttpOperationResponse } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Solutions
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the OperationsManagementClient.
 */
export interface Solutions {


    /**
     * @summary Create/Update Solution.
     *
     * Creates or updates the Solution.
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} parameters The parameters required to create OMS Solution.
     *
     * @param {string} [parameters.location] Resource location
     *
     * @param {object} [parameters.plan] Plan for solution object supported by the
     * OperationsManagement resource provider.
     *
     * @param {string} [parameters.plan.name] name of the solution to be created.
     * For Microsoft published solution it should be in the format of
     * solutionType(workspaceName). SolutionType part is case sensitive. For third
     * party solution, it can be anything.
     *
     * @param {string} [parameters.plan.publisher] Publisher name. For gallery
     * solution, it is Microsoft.
     *
     * @param {string} [parameters.plan.promotionCode] promotionCode, Not really
     * used now, can you left as empty
     *
     * @param {string} [parameters.plan.product] name of the solution to
     * enabled/add. For Microsoft published gallery solution it should be in the
     * format of OMSGallery/<solutionType>. This is case sensitive
     *
     * @param {object} [parameters.properties] Properties for solution object
     * supported by the OperationsManagement resource provider.
     *
     * @param {string} parameters.properties.workspaceResourceId The azure
     * resourceId for the workspace where the solution will be deployed/enabled.
     *
     * @param {array} [parameters.properties.containedResources] The azure
     * resources that will be contained within the solutions. They will be locked
     * and gets deleted automatically when the solution is deleted.
     *
     * @param {array} [parameters.properties.referencedResources] The resources
     * that will be referenced from this solution. Deleting any of those solution
     * out of band will break the solution.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Solution>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    createOrUpdateWithHttpOperationResponse(resourceGroupName: string, parameters: models.Solution, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Solution>>;

    /**
     * @summary Create/Update Solution.
     *
     * Creates or updates the Solution.
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} parameters The parameters required to create OMS Solution.
     *
     * @param {string} [parameters.location] Resource location
     *
     * @param {object} [parameters.plan] Plan for solution object supported by the
     * OperationsManagement resource provider.
     *
     * @param {string} [parameters.plan.name] name of the solution to be created.
     * For Microsoft published solution it should be in the format of
     * solutionType(workspaceName). SolutionType part is case sensitive. For third
     * party solution, it can be anything.
     *
     * @param {string} [parameters.plan.publisher] Publisher name. For gallery
     * solution, it is Microsoft.
     *
     * @param {string} [parameters.plan.promotionCode] promotionCode, Not really
     * used now, can you left as empty
     *
     * @param {string} [parameters.plan.product] name of the solution to
     * enabled/add. For Microsoft published gallery solution it should be in the
     * format of OMSGallery/<solutionType>. This is case sensitive
     *
     * @param {object} [parameters.properties] Properties for solution object
     * supported by the OperationsManagement resource provider.
     *
     * @param {string} parameters.properties.workspaceResourceId The azure
     * resourceId for the workspace where the solution will be deployed/enabled.
     *
     * @param {array} [parameters.properties.containedResources] The azure
     * resources that will be contained within the solutions. They will be locked
     * and gets deleted automatically when the solution is deleted.
     *
     * @param {array} [parameters.properties.referencedResources] The resources
     * that will be referenced from this solution. Deleting any of those solution
     * out of band will break the solution.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Solution} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Solution} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Solution} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    createOrUpdate(resourceGroupName: string, parameters: models.Solution, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Solution>;
    createOrUpdate(resourceGroupName: string, parameters: models.Solution, callback: ServiceCallback<models.Solution>): void;
    createOrUpdate(resourceGroupName: string, parameters: models.Solution, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Solution>): void;


    /**
     * @summary Deletes the solution
     *
     * Deletes the solution in the subscription.
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<null>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    deleteMethodWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<void>>;

    /**
     * @summary Deletes the solution
     *
     * Deletes the solution in the subscription.
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {null} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {null} [result]   - The deserialized result object if an error did not occur.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    deleteMethod(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<void>;
    deleteMethod(resourceGroupName: string, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;


    /**
     * @summary Retrieve solution.
     *
     * Retrieves the user solution.
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<Solution>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    getWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.Solution>>;

    /**
     * @summary Retrieve solution.
     *
     * Retrieves the user solution.
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {Solution} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {Solution} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link Solution} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    get(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.Solution>;
    get(resourceGroupName: string, callback: ServiceCallback<models.Solution>): void;
    get(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.Solution>): void;


    /**
     * @summary Retrieves the solution list for the subscription
     *
     * Retrieves the solution list. It will retrieve both first party and third
     * party solutions
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SolutionPropertiesList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listByResourceGroupWithHttpOperationResponse(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SolutionPropertiesList>>;

    /**
     * @summary Retrieves the solution list for the subscription
     *
     * Retrieves the solution list. It will retrieve both first party and third
     * party solutions
     *
     * @param {string} resourceGroupName The name of the resource group to get. The
     * name is case insensitive.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SolutionPropertiesList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SolutionPropertiesList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SolutionPropertiesList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listByResourceGroup(resourceGroupName: string, options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SolutionPropertiesList>;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.SolutionPropertiesList>): void;
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SolutionPropertiesList>): void;


    /**
     * @summary Retrieves the solution list for the subscription
     *
     * Retrieves the solution list. It will retrieve both first party and third
     * party solutions
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<SolutionPropertiesList>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listBySubscriptionWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.SolutionPropertiesList>>;

    /**
     * @summary Retrieves the solution list for the subscription
     *
     * Retrieves the solution list. It will retrieve both first party and third
     * party solutions
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {SolutionPropertiesList} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {SolutionPropertiesList} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link SolutionPropertiesList} for more
     *                      information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    listBySubscription(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.SolutionPropertiesList>;
    listBySubscription(callback: ServiceCallback<models.SolutionPropertiesList>): void;
    listBySubscription(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.SolutionPropertiesList>): void;
}

/**
 * @class
 * Operations
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the OperationsManagementClient.
 */
export interface Operations {


    /**
     * Lists all of the available OperationsManagement Rest API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @returns {Promise} A promise is returned
     *
     * @resolve {HttpOperationResponse<OperationListResult>} - The deserialized result object.
     *
     * @reject {Error|ServiceError} - The error object.
     */
    listWithHttpOperationResponse(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<HttpOperationResponse<models.OperationListResult>>;

    /**
     * Lists all of the available OperationsManagement Rest API operations.
     *
     * @param {object} [options] Optional Parameters.
     *
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     *
     * @param {ServiceCallback} [optionalCallback] - The optional callback.
     *
     * @returns {ServiceCallback|Promise} If a callback was passed as the last
     * parameter then it returns the callback else returns a Promise.
     *
     * {Promise} A promise is returned.
     *
     *                      @resolve {OperationListResult} - The deserialized result object.
     *
     *                      @reject {Error|ServiceError} - The error object.
     *
     * {ServiceCallback} optionalCallback(err, result, request, response)
     *
     *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
     *
     *                      {OperationListResult} [result]   - The deserialized result object if an error did not occur.
     *                      See {@link OperationListResult} for more information.
     *
     *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
     *
     *                      {http.IncomingMessage} [response] - The HTTP Response stream if an error did not occur.
     */
    list(options?: { customHeaders? : { [headerName: string]: string; } }): Promise<models.OperationListResult>;
    list(callback: ServiceCallback<models.OperationListResult>): void;
    list(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.OperationListResult>): void;
}
