/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * IaaS VM workload-specific backup item.
 *
 * @extends models['ProtectedItem']
 */
class AzureIaaSVMProtectedItem extends models['ProtectedItem'] {
  /**
   * Create a AzureIaaSVMProtectedItem.
   * @member {string} [friendlyName] Friendly name of the VM represented by
   * this backup item.
   * @member {string} [virtualMachineId] Fully qualified ARM ID of the virtual
   * machine represented by this item.
   * @member {string} [protectionStatus] Backup status of this backup item.
   * @member {string} [protectionState] Backup state of this backup item.
   * Possible values include: 'Invalid', 'IRPending', 'Protected',
   * 'ProtectionError', 'ProtectionStopped', 'ProtectionPaused'
   * @member {string} [healthStatus] Health status of protected item. Possible
   * values include: 'Passed', 'ActionRequired', 'ActionSuggested', 'Invalid'
   * @member {array} [healthDetails] Health details on this backup item.
   * @member {string} [lastBackupStatus] Last backup operation status. Possible
   * values: Healthy, Unhealthy.
   * @member {date} [lastBackupTime] Timestamp of the last backup operation on
   * this backup item.
   * @member {string} [protectedItemDataId] Data ID of the protected item.
   * @member {object} [extendedInfo] Additional information for this backup
   * item.
   * @member {date} [extendedInfo.oldestRecoveryPoint] The oldest backup copy
   * available for this backup item.
   * @member {number} [extendedInfo.recoveryPointCount] Number of backup copies
   * available for this backup item.
   * @member {boolean} [extendedInfo.policyInconsistent] Specifies if backup
   * policy associated with the backup item is inconsistent.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AzureIaaSVMProtectedItem
   *
   * @returns {object} metadata of AzureIaaSVMProtectedItem
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'AzureIaaSVMProtectedItem',
      type: {
        name: 'Composite',
        className: 'AzureIaaSVMProtectedItem',
        modelProperties: {
          backupManagementType: {
            required: false,
            serializedName: 'backupManagementType',
            type: {
              name: 'String'
            }
          },
          workloadType: {
            required: false,
            serializedName: 'workloadType',
            type: {
              name: 'String'
            }
          },
          containerName: {
            required: false,
            serializedName: 'containerName',
            type: {
              name: 'String'
            }
          },
          sourceResourceId: {
            required: false,
            serializedName: 'sourceResourceId',
            type: {
              name: 'String'
            }
          },
          policyId: {
            required: false,
            serializedName: 'policyId',
            type: {
              name: 'String'
            }
          },
          lastRecoveryPoint: {
            required: false,
            serializedName: 'lastRecoveryPoint',
            type: {
              name: 'DateTime'
            }
          },
          protectedItemType: {
            required: true,
            serializedName: 'protectedItemType',
            type: {
              name: 'String'
            }
          },
          friendlyName: {
            required: false,
            serializedName: 'friendlyName',
            type: {
              name: 'String'
            }
          },
          virtualMachineId: {
            required: false,
            serializedName: 'virtualMachineId',
            type: {
              name: 'String'
            }
          },
          protectionStatus: {
            required: false,
            serializedName: 'protectionStatus',
            type: {
              name: 'String'
            }
          },
          protectionState: {
            required: false,
            serializedName: 'protectionState',
            type: {
              name: 'String'
            }
          },
          healthStatus: {
            required: false,
            serializedName: 'healthStatus',
            type: {
              name: 'String'
            }
          },
          healthDetails: {
            required: false,
            serializedName: 'healthDetails',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'AzureIaaSVMHealthDetailsElementType',
                  type: {
                    name: 'Composite',
                    className: 'AzureIaaSVMHealthDetails'
                  }
              }
            }
          },
          lastBackupStatus: {
            required: false,
            serializedName: 'lastBackupStatus',
            type: {
              name: 'String'
            }
          },
          lastBackupTime: {
            required: false,
            serializedName: 'lastBackupTime',
            type: {
              name: 'DateTime'
            }
          },
          protectedItemDataId: {
            required: false,
            serializedName: 'protectedItemDataId',
            type: {
              name: 'String'
            }
          },
          extendedInfo: {
            required: false,
            serializedName: 'extendedInfo',
            type: {
              name: 'Composite',
              className: 'AzureIaaSVMProtectedItemExtendedInfo'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureIaaSVMProtectedItem;
