/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the AzureIaaSVMErrorInfo class.
 * @constructor
 * Azure IaaS VM workload-specific error information.
 *
 * @member {number} [errorCode] Error code.
 * @member {string} [errorTitle] Title: Typically, the entity that the error
 * pertains to.
 * @member {string} [errorString] Localized error string.
 * @member {array} [recommendations] List of localized recommendations for
 * above error code.
 */
export interface AzureIaaSVMErrorInfo {
  errorCode?: number;
  errorTitle?: string;
  errorString?: string;
  recommendations?: string[];
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMJobTaskDetails class.
 * @constructor
 * Azure IaaS VM workload-specific job task details.
 *
 * @member {string} [taskId] The task display name.
 * @member {date} [startTime] The start time.
 * @member {date} [endTime] The end time.
 * @member {string} [instanceId] The instanceId.
 * @member {moment.duration} [duration] Time elapsed for task.
 * @member {string} [status] The status.
 * @member {number} [progressPercentage] Progress of the task.
 */
export interface AzureIaaSVMJobTaskDetails {
  taskId?: string;
  startTime?: Date;
  endTime?: Date;
  instanceId?: string;
  duration?: moment.Duration;
  status?: string;
  progressPercentage?: number;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMJobExtendedInfo class.
 * @constructor
 * Azure IaaS VM workload-specific additional information for job.
 *
 * @member {array} [tasksList] List of tasks associated with this job.
 * @member {object} [propertyBag] Job properties.
 * @member {number} [progressPercentage] Indicates progress of the job. Null if
 * it has not started or completed.
 * @member {string} [dynamicErrorMessage] Non localized error message on job
 * execution.
 */
export interface AzureIaaSVMJobExtendedInfo {
  tasksList?: AzureIaaSVMJobTaskDetails[];
  propertyBag?: { [propertyName: string]: string };
  progressPercentage?: number;
  dynamicErrorMessage?: string;
}

/**
 * @class
 * Initializes a new instance of the Job class.
 * @constructor
 * Defines workload agnostic properties for a job.
 *
 * @member {string} [entityFriendlyName] Friendly name of the entity on which
 * the current job is executing.
 * @member {string} [backupManagementType] Backup management type to execute
 * the current job. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB',
 * 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [operation] The operation name.
 * @member {string} [status] Job status.
 * @member {date} [startTime] The start time.
 * @member {date} [endTime] The end time.
 * @member {string} [activityId] ActivityId of job.
 * @member {string} jobType Polymorphic Discriminator
 */
export interface Job {
  entityFriendlyName?: string;
  backupManagementType?: string;
  operation?: string;
  status?: string;
  startTime?: Date;
  endTime?: Date;
  activityId?: string;
  jobType: string;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMJob class.
 * @constructor
 * Azure IaaS VM workload-specifc job object.
 *
 * @member {moment.duration} [duration] Time elapsed during the execution of
 * this job.
 * @member {array} [actionsInfo] Gets or sets the state/actions applicable on
 * this job like cancel/retry.
 * @member {array} [errorDetails] Error details on execution of this job.
 * @member {string} [virtualMachineVersion] Specifies whether the backup item
 * is a Classic or an Azure Resource Manager VM.
 * @member {object} [extendedInfo] Additional information for this job.
 * @member {array} [extendedInfo.tasksList] List of tasks associated with this
 * job.
 * @member {object} [extendedInfo.propertyBag] Job properties.
 * @member {number} [extendedInfo.progressPercentage] Indicates progress of the
 * job. Null if it has not started or completed.
 * @member {string} [extendedInfo.dynamicErrorMessage] Non localized error
 * message on job execution.
 */
export interface AzureIaaSVMJob extends Job {
  duration?: moment.Duration;
  actionsInfo?: string[];
  errorDetails?: AzureIaaSVMErrorInfo[];
  virtualMachineVersion?: string;
  extendedInfo?: AzureIaaSVMJobExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the DpmErrorInfo class.
 * @constructor
 * DPM workload-specific error information.
 *
 * @member {string} [errorString] Localized error string.
 * @member {array} [recommendations] List of localized recommendations for
 * above error code.
 */
export interface DpmErrorInfo {
  errorString?: string;
  recommendations?: string[];
}

/**
 * @class
 * Initializes a new instance of the DpmJobTaskDetails class.
 * @constructor
 * DPM workload-specific job task details.
 *
 * @member {string} [taskId] The task display name.
 * @member {date} [startTime] The start time.
 * @member {date} [endTime] The end time.
 * @member {moment.duration} [duration] Time elapsed for task.
 * @member {string} [status] The status.
 */
export interface DpmJobTaskDetails {
  taskId?: string;
  startTime?: Date;
  endTime?: Date;
  duration?: moment.Duration;
  status?: string;
}

/**
 * @class
 * Initializes a new instance of the DpmJobExtendedInfo class.
 * @constructor
 * Additional information on the DPM workload-specific job.
 *
 * @member {array} [tasksList] List of tasks associated with this job.
 * @member {object} [propertyBag] The job properties.
 * @member {string} [dynamicErrorMessage] Non localized error message on job
 * execution.
 */
export interface DpmJobExtendedInfo {
  tasksList?: DpmJobTaskDetails[];
  propertyBag?: { [propertyName: string]: string };
  dynamicErrorMessage?: string;
}

/**
 * @class
 * Initializes a new instance of the DpmJob class.
 * @constructor
 * DPM workload-specifc job object.
 *
 * @member {moment.duration} [duration] Time elapsed for job.
 * @member {string} [dpmServerName] DPM server name managing the backup item or
 * backup job.
 * @member {string} [containerName] Name of cluster/server protecting current
 * backup item, if any.
 * @member {string} [containerType] Type of container.
 * @member {string} [workloadType] Type of backup item.
 * @member {array} [actionsInfo] The state/actions applicable on this job like
 * cancel/retry.
 * @member {array} [errorDetails] The errors.
 * @member {object} [extendedInfo] Additional information for this job.
 * @member {array} [extendedInfo.tasksList] List of tasks associated with this
 * job.
 * @member {object} [extendedInfo.propertyBag] The job properties.
 * @member {string} [extendedInfo.dynamicErrorMessage] Non localized error
 * message on job execution.
 */
export interface DpmJob extends Job {
  duration?: moment.Duration;
  dpmServerName?: string;
  containerName?: string;
  containerType?: string;
  workloadType?: string;
  actionsInfo?: string[];
  errorDetails?: DpmErrorInfo[];
  extendedInfo?: DpmJobExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the JobQueryObject class.
 * @constructor
 * Filters to list the jobs.
 *
 * @member {string} [status] Status of the job. Possible values include:
 * 'Invalid', 'InProgress', 'Completed', 'Failed', 'CompletedWithWarnings',
 * 'Cancelled', 'Cancelling'
 * @member {string} [backupManagementType] Type of backup managmenent for the
 * job. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB', 'DPM',
 * 'AzureBackupServer', 'AzureSql'
 * @member {string} [operation] Type of operation. Possible values include:
 * 'Invalid', 'Register', 'UnRegister', 'ConfigureBackup', 'Backup', 'Restore',
 * 'DisableBackup', 'DeleteBackupData'
 * @member {string} [jobId] JobID represents the job uniquely.
 * @member {date} [startTime] Job has started at this time. Value is in UTC.
 * @member {date} [endTime] Job has ended at this time. Value is in UTC.
 */
export interface JobQueryObject {
  status?: string;
  backupManagementType?: string;
  operation?: string;
  jobId?: string;
  startTime?: Date;
  endTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * ARM Resource.
 *
 * @member {string} [id] Resource Id represents the complete path to the
 * resource.
 * @member {string} [name] Resource name associated with the resource.
 * @member {string} [type] Resource type represents the complete path of the
 * form Namespace/ResourceType/ResourceType/...
 * @member {string} [location] Resource location.
 * @member {object} [tags] Resource tags.
 * @member {string} [eTag] Optional ETag.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
  eTag?: string;
}

/**
 * @class
 * Initializes a new instance of the JobResource class.
 * @constructor
 * Defines workload agnostic properties for a job.
 *
 * @member {object} [properties] JobResource properties
 * @member {string} [properties.entityFriendlyName] Friendly name of the entity
 * on which the current job is executing.
 * @member {string} [properties.backupManagementType] Backup management type to
 * execute the current job. Possible values include: 'Invalid', 'AzureIaasVM',
 * 'MAB', 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [properties.operation] The operation name.
 * @member {string} [properties.status] Job status.
 * @member {date} [properties.startTime] The start time.
 * @member {date} [properties.endTime] The end time.
 * @member {string} [properties.activityId] ActivityId of job.
 * @member {string} [properties.jobType] Polymorphic Discriminator
 */
export interface JobResource extends Resource {
  properties?: Job;
}

/**
 * @class
 * Initializes a new instance of the MabErrorInfo class.
 * @constructor
 * MAB workload-specific error information.
 *
 * @member {string} [errorString] Localized error string.
 * @member {array} [recommendations] List of localized recommendations.
 */
export interface MabErrorInfo {
  errorString?: string;
  recommendations?: string[];
}

/**
 * @class
 * Initializes a new instance of the MabJobTaskDetails class.
 * @constructor
 * MAB workload-specific job task details.
 *
 * @member {string} [taskId] The task display name.
 * @member {date} [startTime] The start time.
 * @member {date} [endTime] The end time.
 * @member {moment.duration} [duration] Time elapsed for task.
 * @member {string} [status] The status.
 */
export interface MabJobTaskDetails {
  taskId?: string;
  startTime?: Date;
  endTime?: Date;
  duration?: moment.Duration;
  status?: string;
}

/**
 * @class
 * Initializes a new instance of the MabJobExtendedInfo class.
 * @constructor
 * Additional information for the MAB workload-specific job.
 *
 * @member {array} [tasksList] List of tasks for this job.
 * @member {object} [propertyBag] The job properties.
 * @member {string} [dynamicErrorMessage] Non localized error message specific
 * to this job.
 */
export interface MabJobExtendedInfo {
  tasksList?: MabJobTaskDetails[];
  propertyBag?: { [propertyName: string]: string };
  dynamicErrorMessage?: string;
}

/**
 * @class
 * Initializes a new instance of the MabJob class.
 * @constructor
 * MAB workload-specific job.
 *
 * @member {moment.duration} [duration] Time taken by job to run.
 * @member {array} [actionsInfo] The state/actions applicable on jobs like
 * cancel/retry.
 * @member {string} [mabServerName] Name of server protecting the DS.
 * @member {string} [mabServerType] Server type of MAB container. Possible
 * values include: 'Invalid', 'Unknown', 'IaasVMContainer',
 * 'IaasVMServiceContainer', 'DPMContainer', 'AzureBackupServerContainer',
 * 'MABContainer', 'Cluster', 'AzureSqlContainer', 'Windows', 'VCenter'
 * @member {string} [workloadType] Workload type of backup item. Possible
 * values include: 'Invalid', 'VM', 'FileFolder', 'AzureSqlDb', 'SQLDB',
 * 'Exchange', 'Sharepoint', 'VMwareVM', 'SystemState', 'Client',
 * 'GenericDataSource'
 * @member {array} [errorDetails] The errors.
 * @member {object} [extendedInfo] Additional information on the job.
 * @member {array} [extendedInfo.tasksList] List of tasks for this job.
 * @member {object} [extendedInfo.propertyBag] The job properties.
 * @member {string} [extendedInfo.dynamicErrorMessage] Non localized error
 * message specific to this job.
 */
export interface MabJob extends Job {
  duration?: moment.Duration;
  actionsInfo?: string[];
  mabServerName?: string;
  mabServerType?: string;
  workloadType?: string;
  errorDetails?: MabErrorInfo[];
  extendedInfo?: MabJobExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the ResourceList class.
 * @constructor
 * Base for all lists of resources.
 *
 * @member {string} [nextLink] The uri to fetch the next page of resources.
 * Call ListNext() fetches next page of resources.
 */
export interface ResourceList {
  nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DPMContainerExtendedInfo class.
 * @constructor
 * Additional information of the DPMContainer.
 *
 * @member {date} [lastRefreshedAt] Last refresh time of the DPMContainer.
 */
export interface DPMContainerExtendedInfo {
  lastRefreshedAt?: Date;
}

/**
 * @class
 * Initializes a new instance of the ProtectionContainer class.
 * @constructor
 * Base class for container with backup items. Containers with specific
 * workloads are derived from this class.
 *
 * @member {string} [friendlyName] Friendly name of the container.
 * @member {string} [backupManagementType] Type of backup managemenent for the
 * container. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB', 'DPM',
 * 'AzureBackupServer', 'AzureSql'
 * @member {string} [registrationStatus] Status of registration of the
 * container with the Recovery Services Vault.
 * @member {string} [healthStatus] Status of health of the container.
 * @member {string} [containerType] Type of the container. The value of this
 * property for: 1. Compute Azure VM is Microsoft.Compute/virtualMachines 2.
 * Classic Compute Azure VM is Microsoft.ClassicCompute/virtualMachines 3.
 * Windows machines (like MAB, DPM etc) is Windows 4. Azure SQL instance is
 * AzureSqlContainer. Possible values include: 'Invalid', 'Unknown',
 * 'IaasVMContainer', 'IaasVMServiceContainer', 'DPMContainer',
 * 'AzureBackupServerContainer', 'MABContainer', 'Cluster',
 * 'AzureSqlContainer', 'Windows', 'VCenter'
 * @member {string} protectableObjectType Polymorphic Discriminator
 */
export interface ProtectionContainer {
  friendlyName?: string;
  backupManagementType?: string;
  registrationStatus?: string;
  healthStatus?: string;
  readonly containerType?: string;
  protectableObjectType: string;
}

/**
 * @class
 * Initializes a new instance of the AzureBackupServerContainer class.
 * @constructor
 * AzureBackupServer (DPMVenus) workload-specific protection container.
 *
 * @member {boolean} [canReRegister] Specifies whether the container is
 * re-registrable.
 * @member {string} [containerId] ID of container.
 * @member {number} [protectedItemCount] Number of protected items in the
 * BackupEngine
 * @member {string} [dpmAgentVersion] Backup engine Agent version
 * @member {array} [dPMServers] List of BackupEngines protecting the container
 * @member {boolean} [upgradeAvailable] To check if upgrade available
 * @member {string} [protectionStatus] Protection status of the container.
 * @member {object} [extendedInfo] Extended Info of the container.
 * @member {date} [extendedInfo.lastRefreshedAt] Last refresh time of the
 * DPMContainer.
 */
export interface AzureBackupServerContainer extends ProtectionContainer {
  canReRegister?: boolean;
  containerId?: string;
  protectedItemCount?: number;
  dpmAgentVersion?: string;
  dPMServers?: string[];
  upgradeAvailable?: boolean;
  protectionStatus?: string;
  extendedInfo?: DPMContainerExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the BackupEngineBase class.
 * @constructor
 * The base backup engine class. All workload specific backup engines derive
 * from this class.
 *
 * @member {string} [friendlyName] Friendly name of the backup engine.
 * @member {string} [backupManagementType] Type of backup management for the
 * backup engine. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB',
 * 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [registrationStatus] Registration status of the backup
 * engine with the Recovery Services Vault.
 * @member {string} [backupEngineState] Status of the backup engine with the
 * Recovery Services Vault. = {Active/Deleting/DeleteFailed}
 * @member {string} [healthStatus] Backup status of the backup engine.
 * @member {boolean} [canReRegister] Flag indicating if the backup engine be
 * registered, once already registered.
 * @member {string} [backupEngineId] ID of the backup engine.
 * @member {string} [dpmVersion] Backup engine version
 * @member {string} [azureBackupAgentVersion] Backup agent version
 * @member {boolean} [isAzureBackupAgentUpgradeAvailable] To check if backup
 * agent upgrade available
 * @member {boolean} [isDPMUpgradeAvailable] To check if backup engine upgrade
 * available
 * @member {object} [extendedInfo] Extended info of the backupengine
 * @member {string} [extendedInfo.databaseName] Database name of backup engine.
 * @member {number} [extendedInfo.protectedItemsCount] Number of protected
 * items in the backup engine.
 * @member {number} [extendedInfo.protectedServersCount] Number of protected
 * servers in the backup engine.
 * @member {number} [extendedInfo.diskCount] Number of disks in the backup
 * engine.
 * @member {number} [extendedInfo.usedDiskSpace] Diskspace used in the backup
 * engine.
 * @member {number} [extendedInfo.availableDiskSpace] Diskspace currently
 * available in the backup engine.
 * @member {date} [extendedInfo.refreshedAt] Last refresh time in the backup
 * engine.
 * @member {number} [extendedInfo.azureProtectedInstances] Protected instances
 * in the backup engine.
 * @member {string} backupEngineType Polymorphic Discriminator
 */
export interface BackupEngineBase {
  friendlyName?: string;
  backupManagementType?: string;
  registrationStatus?: string;
  backupEngineState?: string;
  healthStatus?: string;
  canReRegister?: boolean;
  backupEngineId?: string;
  dpmVersion?: string;
  azureBackupAgentVersion?: string;
  isAzureBackupAgentUpgradeAvailable?: boolean;
  isDPMUpgradeAvailable?: boolean;
  extendedInfo?: BackupEngineExtendedInfo;
  backupEngineType: string;
}

/**
 * @class
 * Initializes a new instance of the AzureBackupServerEngine class.
 * @constructor
 * Backup engine type when Azure Backup Server is used to manage the backups.
 *
 */
export interface AzureBackupServerEngine extends BackupEngineBase {
}

/**
 * @class
 * Initializes a new instance of the IaaSVMContainer class.
 * @constructor
 * IaaS VM workload-specific container.
 *
 * @member {string} [virtualMachineId] Fully qualified ARM url of the virtual
 * machine represented by this Azure IaaS VM container.
 * @member {string} [virtualMachineVersion] Specifies whether the container
 * represents a Classic or an Azure Resource Manager VM.
 * @member {string} [resourceGroup] Resource group name of Recovery Services
 * Vault.
 */
export interface IaaSVMContainer extends ProtectionContainer {
  virtualMachineId?: string;
  virtualMachineVersion?: string;
  resourceGroup?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSClassicComputeVMContainer class.
 * @constructor
 * IaaS VM workload-specific backup item representing a classic virtual
 * machine.
 *
 */
export interface AzureIaaSClassicComputeVMContainer extends IaaSVMContainer {
}

/**
 * @class
 * Initializes a new instance of the WorkloadProtectableItem class.
 * @constructor
 * Base class for backup item. Workload-specific backup items are derived from
 * this class.
 *
 * @member {string} [backupManagementType] Type of backup managemenent to
 * backup an item.
 * @member {string} [friendlyName] Friendly name of the backup item.
 * @member {string} [protectionState] State of the back up item. Possible
 * values include: 'Invalid', 'NotProtected', 'Protecting', 'Protected'
 * @member {string} protectableItemType Polymorphic Discriminator
 */
export interface WorkloadProtectableItem {
  backupManagementType?: string;
  friendlyName?: string;
  protectionState?: string;
  protectableItemType: string;
}

/**
 * @class
 * Initializes a new instance of the IaaSVMProtectableItem class.
 * @constructor
 * IaaS VM workload-specific backup item.
 *
 * @member {string} [virtualMachineId] Fully qualified ARM ID of the virtual
 * machine.
 */
export interface IaaSVMProtectableItem extends WorkloadProtectableItem {
  virtualMachineId?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSClassicComputeVMProtectableItem class.
 * @constructor
 * IaaS VM workload-specific backup item representing the Classic Compute VM.
 *
 */
export interface AzureIaaSClassicComputeVMProtectableItem extends IaaSVMProtectableItem {
}

/**
 * @class
 * Initializes a new instance of the ProtectedItem class.
 * @constructor
 * Base class for backup items.
 *
 * @member {string} [backupManagementType] Type of backup managemenent for the
 * backed up item. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB',
 * 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [workloadType] Type of workload this item represents.
 * Possible values include: 'Invalid', 'VM', 'FileFolder', 'AzureSqlDb',
 * 'SQLDB', 'Exchange', 'Sharepoint', 'VMwareVM', 'SystemState', 'Client',
 * 'GenericDataSource'
 * @member {string} [containerName] Unique name of container
 * @member {string} [sourceResourceId] ARM ID of the resource to be backed up.
 * @member {string} [policyId] ID of the backup policy with which this item is
 * backed up.
 * @member {date} [lastRecoveryPoint] Timestamp when the last (latest) backup
 * copy was created for this backup item.
 * @member {string} protectedItemType Polymorphic Discriminator
 */
export interface ProtectedItem {
  backupManagementType?: string;
  workloadType?: string;
  containerName?: string;
  sourceResourceId?: string;
  policyId?: string;
  lastRecoveryPoint?: Date;
  protectedItemType: string;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMProtectedItem class.
 * @constructor
 * IaaS VM workload-specific backup item.
 *
 * @member {string} [friendlyName] Friendly name of the VM represented by this
 * backup item.
 * @member {string} [virtualMachineId] Fully qualified ARM ID of the virtual
 * machine represented by this item.
 * @member {string} [protectionStatus] Backup status of this backup item.
 * @member {string} [protectionState] Backup state of this backup item.
 * Possible values include: 'Invalid', 'IRPending', 'Protected',
 * 'ProtectionError', 'ProtectionStopped', 'ProtectionPaused'
 * @member {string} [healthStatus] Health status of protected item. Possible
 * values include: 'Passed', 'ActionRequired', 'ActionSuggested', 'Invalid'
 * @member {array} [healthDetails] Health details on this backup item.
 * @member {string} [lastBackupStatus] Last backup operation status. Possible
 * values: Healthy, Unhealthy.
 * @member {date} [lastBackupTime] Timestamp of the last backup operation on
 * this backup item.
 * @member {string} [protectedItemDataId] Data ID of the protected item.
 * @member {object} [extendedInfo] Additional information for this backup item.
 * @member {date} [extendedInfo.oldestRecoveryPoint] The oldest backup copy
 * available for this backup item.
 * @member {number} [extendedInfo.recoveryPointCount] Number of backup copies
 * available for this backup item.
 * @member {boolean} [extendedInfo.policyInconsistent] Specifies if backup
 * policy associated with the backup item is inconsistent.
 */
export interface AzureIaaSVMProtectedItem extends ProtectedItem {
  friendlyName?: string;
  virtualMachineId?: string;
  protectionStatus?: string;
  protectionState?: string;
  healthStatus?: string;
  healthDetails?: AzureIaaSVMHealthDetails[];
  lastBackupStatus?: string;
  lastBackupTime?: Date;
  protectedItemDataId?: string;
  extendedInfo?: AzureIaaSVMProtectedItemExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSClassicComputeVMProtectedItem class.
 * @constructor
 * IaaS VM workload-specific backup item representing the Classic Compute VM.
 *
 */
export interface AzureIaaSClassicComputeVMProtectedItem extends AzureIaaSVMProtectedItem {
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSComputeVMContainer class.
 * @constructor
 * IaaS VM workload-specific backup item representing an Azure Resource Manager
 * virtual machine.
 *
 */
export interface AzureIaaSComputeVMContainer extends IaaSVMContainer {
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSComputeVMProtectableItem class.
 * @constructor
 * IaaS VM workload-specific backup item representing the Azure Resource
 * Manager VM.
 *
 */
export interface AzureIaaSComputeVMProtectableItem extends IaaSVMProtectableItem {
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSComputeVMProtectedItem class.
 * @constructor
 * IaaS VM workload-specific backup item representing the Azure Resource
 * Manager VM.
 *
 */
export interface AzureIaaSComputeVMProtectedItem extends AzureIaaSVMProtectedItem {
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMHealthDetails class.
 * @constructor
 * Azure IaaS VM workload-specific Health Details.
 *
 * @member {number} [code] Health Code
 * @member {string} [title] Health Title
 * @member {string} [message] Health Message
 * @member {array} [recommendations] Health Recommended Actions
 */
export interface AzureIaaSVMHealthDetails {
  code?: number;
  title?: string;
  message?: string;
  recommendations?: string[];
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMProtectedItemExtendedInfo class.
 * @constructor
 * Additional information on Azure IaaS VM specific backup item.
 *
 * @member {date} [oldestRecoveryPoint] The oldest backup copy available for
 * this backup item.
 * @member {number} [recoveryPointCount] Number of backup copies available for
 * this backup item.
 * @member {boolean} [policyInconsistent] Specifies if backup policy associated
 * with the backup item is inconsistent.
 */
export interface AzureIaaSVMProtectedItemExtendedInfo {
  oldestRecoveryPoint?: Date;
  recoveryPointCount?: number;
  policyInconsistent?: boolean;
}

/**
 * @class
 * Initializes a new instance of the SchedulePolicy class.
 * @constructor
 * Base class for backup schedule.
 *
 * @member {string} schedulePolicyType Polymorphic Discriminator
 */
export interface SchedulePolicy {
  schedulePolicyType: string;
}

/**
 * @class
 * Initializes a new instance of the RetentionPolicy class.
 * @constructor
 * Base class for retention policy.
 *
 * @member {string} retentionPolicyType Polymorphic Discriminator
 */
export interface RetentionPolicy {
  retentionPolicyType: string;
}

/**
 * @class
 * Initializes a new instance of the ProtectionPolicy class.
 * @constructor
 * Base class for backup policy. Workload-specific backup policies are derived
 * from this class.
 *
 * @member {number} [protectedItemsCount] Number of items associated with this
 * policy.
 * @member {string} backupManagementType Polymorphic Discriminator
 */
export interface ProtectionPolicy {
  protectedItemsCount?: number;
  backupManagementType: string;
}

/**
 * @class
 * Initializes a new instance of the AzureIaaSVMProtectionPolicy class.
 * @constructor
 * IaaS VM workload-specific backup policy.
 *
 * @member {object} [schedulePolicy] Backup schedule specified as part of
 * backup policy.
 * @member {string} [schedulePolicy.schedulePolicyType] Polymorphic
 * Discriminator
 * @member {object} [retentionPolicy] Retention policy with the details on
 * backup copy retention ranges.
 * @member {string} [retentionPolicy.retentionPolicyType] Polymorphic
 * Discriminator
 * @member {string} [timeZone] TimeZone optional input as string. For example:
 * TimeZone = "Pacific Standard Time".
 */
export interface AzureIaaSVMProtectionPolicy extends ProtectionPolicy {
  schedulePolicy?: SchedulePolicy;
  retentionPolicy?: RetentionPolicy;
  timeZone?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlContainer class.
 * @constructor
 * Azure Sql workload-specific container.
 *
 */
export interface AzureSqlContainer extends ProtectionContainer {
}

/**
 * @class
 * Initializes a new instance of the AzureSqlProtectedItemExtendedInfo class.
 * @constructor
 * Additional information on Azure Sql specific protected item.
 *
 * @member {date} [oldestRecoveryPoint] The oldest backup copy available for
 * this item in the service.
 * @member {number} [recoveryPointCount] Number of available backup copies
 * associated with this backup item.
 * @member {string} [policyState] State of the backup policy associated with
 * this backup item.
 */
export interface AzureSqlProtectedItemExtendedInfo {
  oldestRecoveryPoint?: Date;
  recoveryPointCount?: number;
  policyState?: string;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlProtectedItem class.
 * @constructor
 * Azure SQL workload-specific backup item.
 *
 * @member {string} [protectedItemDataId] Internal ID of a backup item. Used by
 * Azure SQL Backup engine to contact Recovery Services.
 * @member {string} [protectionState] Backup state of the backed up item.
 * Possible values include: 'Invalid', 'IRPending', 'Protected',
 * 'ProtectionError', 'ProtectionStopped', 'ProtectionPaused'
 * @member {object} [extendedInfo] Additional information for this backup item.
 * @member {date} [extendedInfo.oldestRecoveryPoint] The oldest backup copy
 * available for this item in the service.
 * @member {number} [extendedInfo.recoveryPointCount] Number of available
 * backup copies associated with this backup item.
 * @member {string} [extendedInfo.policyState] State of the backup policy
 * associated with this backup item.
 */
export interface AzureSqlProtectedItem extends ProtectedItem {
  protectedItemDataId?: string;
  protectionState?: string;
  extendedInfo?: AzureSqlProtectedItemExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the AzureSqlProtectionPolicy class.
 * @constructor
 * Azure SQL workload-specific backup policy.
 *
 * @member {object} [retentionPolicy] Retention policy details.
 * @member {string} [retentionPolicy.retentionPolicyType] Polymorphic
 * Discriminator
 */
export interface AzureSqlProtectionPolicy extends ProtectionPolicy {
  retentionPolicy?: RetentionPolicy;
}

/**
 * @class
 * Initializes a new instance of the BackupEngineExtendedInfo class.
 * @constructor
 * Additional information on backup engine.
 *
 * @member {string} [databaseName] Database name of backup engine.
 * @member {number} [protectedItemsCount] Number of protected items in the
 * backup engine.
 * @member {number} [protectedServersCount] Number of protected servers in the
 * backup engine.
 * @member {number} [diskCount] Number of disks in the backup engine.
 * @member {number} [usedDiskSpace] Diskspace used in the backup engine.
 * @member {number} [availableDiskSpace] Diskspace currently available in the
 * backup engine.
 * @member {date} [refreshedAt] Last refresh time in the backup engine.
 * @member {number} [azureProtectedInstances] Protected instances in the backup
 * engine.
 */
export interface BackupEngineExtendedInfo {
  databaseName?: string;
  protectedItemsCount?: number;
  protectedServersCount?: number;
  diskCount?: number;
  usedDiskSpace?: number;
  availableDiskSpace?: number;
  refreshedAt?: Date;
  azureProtectedInstances?: number;
}

/**
 * @class
 * Initializes a new instance of the BackupEngineBaseResource class.
 * @constructor
 * The base backup engine class. All workload specific backup engines derive
 * from this class.
 *
 * @member {object} [properties] BackupEngineBaseResource properties
 * @member {string} [properties.friendlyName] Friendly name of the backup
 * engine.
 * @member {string} [properties.backupManagementType] Type of backup management
 * for the backup engine. Possible values include: 'Invalid', 'AzureIaasVM',
 * 'MAB', 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [properties.registrationStatus] Registration status of the
 * backup engine with the Recovery Services Vault.
 * @member {string} [properties.backupEngineState] Status of the backup engine
 * with the Recovery Services Vault. = {Active/Deleting/DeleteFailed}
 * @member {string} [properties.healthStatus] Backup status of the backup
 * engine.
 * @member {boolean} [properties.canReRegister] Flag indicating if the backup
 * engine be registered, once already registered.
 * @member {string} [properties.backupEngineId] ID of the backup engine.
 * @member {string} [properties.dpmVersion] Backup engine version
 * @member {string} [properties.azureBackupAgentVersion] Backup agent version
 * @member {boolean} [properties.isAzureBackupAgentUpgradeAvailable] To check
 * if backup agent upgrade available
 * @member {boolean} [properties.isDPMUpgradeAvailable] To check if backup
 * engine upgrade available
 * @member {object} [properties.extendedInfo] Extended info of the backupengine
 * @member {string} [properties.extendedInfo.databaseName] Database name of
 * backup engine.
 * @member {number} [properties.extendedInfo.protectedItemsCount] Number of
 * protected items in the backup engine.
 * @member {number} [properties.extendedInfo.protectedServersCount] Number of
 * protected servers in the backup engine.
 * @member {number} [properties.extendedInfo.diskCount] Number of disks in the
 * backup engine.
 * @member {number} [properties.extendedInfo.usedDiskSpace] Diskspace used in
 * the backup engine.
 * @member {number} [properties.extendedInfo.availableDiskSpace] Diskspace
 * currently available in the backup engine.
 * @member {date} [properties.extendedInfo.refreshedAt] Last refresh time in
 * the backup engine.
 * @member {number} [properties.extendedInfo.azureProtectedInstances] Protected
 * instances in the backup engine.
 * @member {string} [properties.backupEngineType] Polymorphic Discriminator
 */
export interface BackupEngineBaseResource extends Resource {
  properties?: BackupEngineBase;
}

/**
 * @class
 * Initializes a new instance of the NameInfo class.
 * @constructor
 * The name of usage.
 *
 * @member {string} [value] Value of usage.
 * @member {string} [localizedValue] Localized value of usage.
 */
export interface NameInfo {
  value?: string;
  localizedValue?: string;
}

/**
 * @class
 * Initializes a new instance of the BackupManagementUsage class.
 * @constructor
 * Backup management usages of a vault.
 *
 * @member {string} [unit] Unit of the usage. Possible values include: 'Count',
 * 'Bytes', 'Seconds', 'Percent', 'CountPerSecond', 'BytesPerSecond'
 * @member {string} [quotaPeriod] Quota period of usage.
 * @member {date} [nextResetTime] Next reset time of usage.
 * @member {number} [currentValue] Current value of usage.
 * @member {number} [limit] Limit of usage.
 * @member {object} [name] Name of usage.
 * @member {string} [name.value] Value of usage.
 * @member {string} [name.localizedValue] Localized value of usage.
 */
export interface BackupManagementUsage {
  unit?: string;
  quotaPeriod?: string;
  nextResetTime?: Date;
  currentValue?: number;
  limit?: number;
  name?: NameInfo;
}

/**
 * @class
 * Initializes a new instance of the BackupRequest class.
 * @constructor
 * Base class for backup request. Workload-specific backup requests are derived
 * from this class.
 *
 * @member {string} objectType Polymorphic Discriminator
 */
export interface BackupRequest {
  objectType: string;
}

/**
 * @class
 * Initializes a new instance of the BackupRequestResource class.
 * @constructor
 * Base class for backup request. Workload-specific backup requests are derived
 * from this class.
 *
 * @member {object} [properties] BackupRequestResource properties
 * @member {string} [properties.objectType] Polymorphic Discriminator
 */
export interface BackupRequestResource extends Resource {
  properties?: BackupRequest;
}

/**
 * @class
 * Initializes a new instance of the BackupResourceConfig class.
 * @constructor
 * The resource storage details.
 *
 * @member {string} [storageType] Storage type. Possible values include:
 * 'Invalid', 'GeoRedundant', 'LocallyRedundant'
 * @member {string} [storageTypeState] Locked or Unlocked. Once a machine is
 * registered against a resource, the storageTypeState is always Locked.
 * Possible values include: 'Invalid', 'Locked', 'Unlocked'
 */
export interface BackupResourceConfig {
  storageType?: string;
  storageTypeState?: string;
}

/**
 * @class
 * Initializes a new instance of the BackupResourceConfigResource class.
 * @constructor
 * The resource storage details.
 *
 * @member {object} [properties] BackupResourceConfigResource properties
 * @member {string} [properties.storageType] Storage type. Possible values
 * include: 'Invalid', 'GeoRedundant', 'LocallyRedundant'
 * @member {string} [properties.storageTypeState] Locked or Unlocked. Once a
 * machine is registered against a resource, the storageTypeState is always
 * Locked. Possible values include: 'Invalid', 'Locked', 'Unlocked'
 */
export interface BackupResourceConfigResource extends Resource {
  properties?: BackupResourceConfig;
}

/**
 * @class
 * Initializes a new instance of the BackupResourceVaultConfig class.
 * @constructor
 * Backup resource vault config details.
 *
 * @member {string} [storageType] Storage type. Possible values include:
 * 'Invalid', 'GeoRedundant', 'LocallyRedundant'
 * @member {string} [storageTypeState] Locked or Unlocked. Once a machine is
 * registered against a resource, the storageTypeState is always Locked.
 * Possible values include: 'Invalid', 'Locked', 'Unlocked'
 * @member {string} [enhancedSecurityState] Enabled or Disabled. Possible
 * values include: 'Invalid', 'Enabled', 'Disabled'
 */
export interface BackupResourceVaultConfig {
  storageType?: string;
  storageTypeState?: string;
  enhancedSecurityState?: string;
}

/**
 * @class
 * Initializes a new instance of the BackupResourceVaultConfigResource class.
 * @constructor
 * Backup resource vault config details.
 *
 * @member {object} [properties] BackupResourceVaultConfigResource properties
 * @member {string} [properties.storageType] Storage type. Possible values
 * include: 'Invalid', 'GeoRedundant', 'LocallyRedundant'
 * @member {string} [properties.storageTypeState] Locked or Unlocked. Once a
 * machine is registered against a resource, the storageTypeState is always
 * Locked. Possible values include: 'Invalid', 'Locked', 'Unlocked'
 * @member {string} [properties.enhancedSecurityState] Enabled or Disabled.
 * Possible values include: 'Invalid', 'Enabled', 'Disabled'
 */
export interface BackupResourceVaultConfigResource extends Resource {
  properties?: BackupResourceVaultConfig;
}

/**
 * @class
 * Initializes a new instance of the BEKDetails class.
 * @constructor
 * BEK is bitlocker encrpytion key.
 *
 * @member {string} [secretUrl] Secret is BEK.
 * @member {string} [secretVaultId] ID of the Key Vault where this Secret is
 * stored.
 * @member {string} [secretData] BEK data.
 */
export interface BEKDetails {
  secretUrl?: string;
  secretVaultId?: string;
  secretData?: string;
}

/**
 * @class
 * Initializes a new instance of the BMSBackupEngineQueryObject class.
 * @constructor
 * Query parameters to fetch list of backup engines.
 *
 * @member {string} [expand] attribute to add extended info
 */
export interface BMSBackupEngineQueryObject {
  expand?: string;
}

/**
 * @class
 * Initializes a new instance of the BMSBackupEnginesQueryObject class.
 * @constructor
 * Query parameters to fetch list of backup engines.
 *
 * @member {string} [backupManagementType] Backup management type for the
 * backup engine. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB',
 * 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [friendlyName] Friendly name of the backup engine.
 * @member {string} [expand] Attribute to add extended info.
 */
export interface BMSBackupEnginesQueryObject {
  backupManagementType?: string;
  friendlyName?: string;
  expand?: string;
}

/**
 * @class
 * Initializes a new instance of the BMSBackupSummariesQueryObject class.
 * @constructor
 * Query parameters to fetch backup summaries.
 *
 * @member {string} [type] Backup management type for this container. Possible
 * values include: 'Invalid', 'BackupProtectedItemCountSummary',
 * 'BackupProtectionContainerCountSummary'
 */
export interface BMSBackupSummariesQueryObject {
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the BMSContainerQueryObject class.
 * @constructor
 * The query filters that can be used with the list containers API.
 *
 * @member {string} backupManagementType Backup management type for this
 * container. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB', 'DPM',
 * 'AzureBackupServer', 'AzureSql'
 * @member {string} [containerType] Type of container for filter. Possible
 * values include: 'Invalid', 'Unknown', 'IaasVMContainer',
 * 'IaasVMServiceContainer', 'DPMContainer', 'AzureBackupServerContainer',
 * 'MABContainer', 'Cluster', 'AzureSqlContainer', 'Windows', 'VCenter'
 * @member {string} [backupEngineName] Backup engine name
 * @member {string} [status] Status of registration of this container with the
 * Recovery Services Vault.
 * @member {string} [friendlyName] Friendly name of this container.
 */
export interface BMSContainerQueryObject {
  backupManagementType: string;
  containerType?: string;
  backupEngineName?: string;
  status?: string;
  friendlyName?: string;
}

/**
 * @class
 * Initializes a new instance of the BMSPOQueryObject class.
 * @constructor
 * Filters to list items that can be backed up.
 *
 * @member {string} [backupManagementType] Backup management type. Possible
 * values include: 'Invalid', 'AzureIaasVM', 'MAB', 'DPM', 'AzureBackupServer',
 * 'AzureSql'
 * @member {string} [status] Backup status query parameter.
 * @member {string} [friendlyName] Friendly name.
 */
export interface BMSPOQueryObject {
  backupManagementType?: string;
  status?: string;
  friendlyName?: string;
}

/**
 * @class
 * Initializes a new instance of the BMSRPQueryObject class.
 * @constructor
 * Filters to list backup copies.
 *
 * @member {date} [startDate] Backup copies created after this time.
 * @member {date} [endDate] Backup copies created before this time.
 */
export interface BMSRPQueryObject {
  startDate?: Date;
  endDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the ClientScriptForConnect class.
 * @constructor
 * Client script details for file / folder restore.
 *
 * @member {string} [scriptContent] File content of the client script for file
 * / folder restore.
 * @member {string} [scriptExtension] File extension of the client script for
 * file / folder restore - .ps1 , .sh , etc.
 * @member {string} [osType] OS type - Windows, Linux etc. for which this file
 * / folder restore client script works.
 * @member {string} [url] URL of Executable from where to source the content.
 * If this is not null then ScriptContent should not be used
 * @member {string} [scriptNameSuffix] Mandator suffix that should be added to
 * the name of script that is given for download to user.
 * If its null or empty then , ignore it.
 */
export interface ClientScriptForConnect {
  scriptContent?: string;
  scriptExtension?: string;
  osType?: string;
  url?: string;
  scriptNameSuffix?: string;
}

/**
 * @class
 * Initializes a new instance of the Day class.
 * @constructor
 * Day of the week.
 *
 * @member {number} [date] Date of the month
 * @member {boolean} [isLast] Whether Date is last date of month
 */
export interface Day {
  date?: number;
  isLast?: boolean;
}

/**
 * @class
 * Initializes a new instance of the DailyRetentionFormat class.
 * @constructor
 * Daily retention format.
 *
 * @member {array} [daysOfTheMonth] List of days of the month.
 */
export interface DailyRetentionFormat {
  daysOfTheMonth?: Day[];
}

/**
 * @class
 * Initializes a new instance of the RetentionDuration class.
 * @constructor
 * Retention duration.
 *
 * @member {number} [count] Count of duration types. Retention duration is
 * obtained by the counting the duration type Count times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [durationType] Retention duration type of retention policy.
 * Possible values include: 'Invalid', 'Days', 'Weeks', 'Months', 'Years'
 */
export interface RetentionDuration {
  count?: number;
  durationType?: string;
}

/**
 * @class
 * Initializes a new instance of the DailyRetentionSchedule class.
 * @constructor
 * Daily retention schedule.
 *
 * @member {array} [retentionTimes] Retention times of retention policy.
 * @member {object} [retentionDuration] Retention duration of retention Policy.
 * @member {number} [retentionDuration.count] Count of duration types.
 * Retention duration is obtained by the counting the duration type Count
 * times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [retentionDuration.durationType] Retention duration type of
 * retention policy. Possible values include: 'Invalid', 'Days', 'Weeks',
 * 'Months', 'Years'
 */
export interface DailyRetentionSchedule {
  retentionTimes?: Date[];
  retentionDuration?: RetentionDuration;
}

/**
 * @class
 * Initializes a new instance of the DpmBackupEngine class.
 * @constructor
 * Data Protection Manager (DPM) specific backup engine.
 *
 */
export interface DpmBackupEngine extends BackupEngineBase {
}

/**
 * @class
 * Initializes a new instance of the DpmContainer class.
 * @constructor
 * DPM workload-specific protection container.
 *
 * @member {boolean} [canReRegister] Specifies whether the container is
 * re-registrable.
 * @member {string} [containerId] ID of container.
 * @member {number} [protectedItemCount] Number of protected items in the
 * BackupEngine
 * @member {string} [dpmAgentVersion] Backup engine Agent version
 * @member {array} [dPMServers] List of BackupEngines protecting the container
 * @member {boolean} [upgradeAvailable] To check if upgrade available
 * @member {string} [protectionStatus] Protection status of the container.
 * @member {object} [extendedInfo] Extended Info of the container.
 * @member {date} [extendedInfo.lastRefreshedAt] Last refresh time of the
 * DPMContainer.
 */
export interface DpmContainer extends ProtectionContainer {
  canReRegister?: boolean;
  containerId?: string;
  protectedItemCount?: number;
  dpmAgentVersion?: string;
  dPMServers?: string[];
  upgradeAvailable?: boolean;
  protectionStatus?: string;
  extendedInfo?: DPMContainerExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the DPMProtectedItemExtendedInfo class.
 * @constructor
 * Additional information of DPM Protected item.
 *
 * @member {object} [protectableObjectLoadPath] Attribute to provide
 * information on various DBs.
 * @member {boolean} [protectedProperty] To check if backup item is disk
 * protected.
 * @member {boolean} [isPresentOnCloud] To check if backup item is cloud
 * protected.
 * @member {string} [lastBackupStatus] Last backup status information on backup
 * item.
 * @member {date} [lastRefreshedAt] Last refresh time on backup item.
 * @member {date} [oldestRecoveryPoint] Oldest cloud recovery point time.
 * @member {number} [recoveryPointCount] cloud recovery point count.
 * @member {date} [onPremiseOldestRecoveryPoint] Oldest disk recovery point
 * time.
 * @member {date} [onPremiseLatestRecoveryPoint] latest disk recovery point
 * time.
 * @member {number} [onPremiseRecoveryPointCount] disk recovery point count.
 * @member {boolean} [isCollocated] To check if backup item is collocated.
 * @member {string} [protectionGroupName] Protection group name of the backup
 * item.
 * @member {string} [diskStorageUsedInBytes] Used Disk storage in bytes.
 * @member {string} [totalDiskStorageSizeInBytes] total Disk storage in bytes.
 */
export interface DPMProtectedItemExtendedInfo {
  protectableObjectLoadPath?: { [propertyName: string]: string };
  protectedProperty?: boolean;
  isPresentOnCloud?: boolean;
  lastBackupStatus?: string;
  lastRefreshedAt?: Date;
  oldestRecoveryPoint?: Date;
  recoveryPointCount?: number;
  onPremiseOldestRecoveryPoint?: Date;
  onPremiseLatestRecoveryPoint?: Date;
  onPremiseRecoveryPointCount?: number;
  isCollocated?: boolean;
  protectionGroupName?: string;
  diskStorageUsedInBytes?: string;
  totalDiskStorageSizeInBytes?: string;
}

/**
 * @class
 * Initializes a new instance of the DPMProtectedItem class.
 * @constructor
 * Additional information on Backup engine specific backup item.
 *
 * @member {string} [friendlyName] Friendly name of the managed item
 * @member {string} [backupEngineName] Backup Management server protecting this
 * backup item
 * @member {string} [protectionState] Protection state of the backupengine.
 * Possible values include: 'Invalid', 'IRPending', 'Protected',
 * 'ProtectionError', 'ProtectionStopped', 'ProtectionPaused'
 * @member {boolean} [isScheduledForDeferredDelete] To check if backup item is
 * scheduled for deferred delete
 * @member {object} [extendedInfo] Extended info of the backup item.
 * @member {object} [extendedInfo.protectableObjectLoadPath] Attribute to
 * provide information on various DBs.
 * @member {boolean} [extendedInfo.protectedProperty] To check if backup item
 * is disk protected.
 * @member {boolean} [extendedInfo.isPresentOnCloud] To check if backup item is
 * cloud protected.
 * @member {string} [extendedInfo.lastBackupStatus] Last backup status
 * information on backup item.
 * @member {date} [extendedInfo.lastRefreshedAt] Last refresh time on backup
 * item.
 * @member {date} [extendedInfo.oldestRecoveryPoint] Oldest cloud recovery
 * point time.
 * @member {number} [extendedInfo.recoveryPointCount] cloud recovery point
 * count.
 * @member {date} [extendedInfo.onPremiseOldestRecoveryPoint] Oldest disk
 * recovery point time.
 * @member {date} [extendedInfo.onPremiseLatestRecoveryPoint] latest disk
 * recovery point time.
 * @member {number} [extendedInfo.onPremiseRecoveryPointCount] disk recovery
 * point count.
 * @member {boolean} [extendedInfo.isCollocated] To check if backup item is
 * collocated.
 * @member {string} [extendedInfo.protectionGroupName] Protection group name of
 * the backup item.
 * @member {string} [extendedInfo.diskStorageUsedInBytes] Used Disk storage in
 * bytes.
 * @member {string} [extendedInfo.totalDiskStorageSizeInBytes] total Disk
 * storage in bytes.
 */
export interface DPMProtectedItem extends ProtectedItem {
  friendlyName?: string;
  backupEngineName?: string;
  protectionState?: string;
  isScheduledForDeferredDelete?: boolean;
  extendedInfo?: DPMProtectedItemExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the EncryptionDetails class.
 * @constructor
 * Details needed if the VM was encrypted at the time of backup.
 *
 * @member {boolean} [encryptionEnabled] Identifies whether this backup copy
 * represents an encrypted VM at the time of backup.
 * @member {string} [kekUrl] Key Url.
 * @member {string} [secretKeyUrl] Secret Url.
 * @member {string} [kekVaultId] ID of Key Vault where KEK is stored.
 * @member {string} [secretKeyVaultId] ID of Key Vault where Secret is stored.
 */
export interface EncryptionDetails {
  encryptionEnabled?: boolean;
  kekUrl?: string;
  secretKeyUrl?: string;
  kekVaultId?: string;
  secretKeyVaultId?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationResultInfoBase class.
 * @constructor
 * Base class for operation result info.
 *
 * @member {string} objectType Polymorphic Discriminator
 */
export interface OperationResultInfoBase {
  objectType: string;
}

/**
 * @class
 * Initializes a new instance of the ExportJobsOperationResultInfo class.
 * @constructor
 * This class is used to send blob details after exporting jobs.
 *
 * @member {string} [blobUrl] URL of the blob into which the serialized string
 * of list of jobs is exported.
 * @member {string} [blobSasKey] SAS key to access the blob. It expires in 15
 * mins.
 */
export interface ExportJobsOperationResultInfo extends OperationResultInfoBase {
  blobUrl?: string;
  blobSasKey?: string;
}

/**
 * @class
 * Initializes a new instance of the RecoveryPoint class.
 * @constructor
 * Base class for backup copies. Workload-specific backup copies are derived
 * from this class.
 *
 * @member {string} objectType Polymorphic Discriminator
 */
export interface RecoveryPoint {
  objectType: string;
}

/**
 * @class
 * Initializes a new instance of the GenericRecoveryPoint class.
 * @constructor
 * Generic backup copy.
 *
 * @member {string} [friendlyName] Friendly name of the backup copy.
 * @member {string} [recoveryPointType] Type of the backup copy.
 * @member {date} [recoveryPointTime] Time at which this backup copy was
 * created.
 * @member {string} [recoveryPointAdditionalInfo] Additional information
 * associated with this backup copy.
 */
export interface GenericRecoveryPoint extends RecoveryPoint {
  friendlyName?: string;
  recoveryPointType?: string;
  recoveryPointTime?: Date;
  recoveryPointAdditionalInfo?: string;
}

/**
 * @class
 * Initializes a new instance of the GetProtectedItemQueryObject class.
 * @constructor
 * Filters to list backup items.
 *
 * @member {string} [expand] Specifies if the additional information should be
 * provided for this item.
 */
export interface GetProtectedItemQueryObject {
  expand?: string;
}

/**
 * @class
 * Initializes a new instance of the IaasVMBackupRequest class.
 * @constructor
 * IaaS VM workload-specific backup request.
 *
 * @member {date} [recoveryPointExpiryTimeInUTC] Backup copy will expire after
 * the time specified (UTC).
 */
export interface IaasVMBackupRequest extends BackupRequest {
  recoveryPointExpiryTimeInUTC?: Date;
}

/**
 * @class
 * Initializes a new instance of the ILRRequest class.
 * @constructor
 * Parameters to restore file/folders API.
 *
 * @member {string} objectType Polymorphic Discriminator
 */
export interface ILRRequest {
  objectType: string;
}

/**
 * @class
 * Initializes a new instance of the IaasVMILRRegistrationRequest class.
 * @constructor
 * Restore files/folders from a backup copy of IaaS VM.
 *
 * @member {string} [recoveryPointId] ID of the IaaS VM backup copy from where
 * the files/folders have to be restored.
 * @member {string} [virtualMachineId] Fully qualified ARM ID of the virtual
 * machine whose the files / folders have to be restored.
 * @member {string} [initiatorName] iSCSI initiator name.
 * @member {boolean} [renewExistingRegistration] Whether to renew existing
 * registration with the iSCSI server.
 */
export interface IaasVMILRRegistrationRequest extends ILRRequest {
  recoveryPointId?: string;
  virtualMachineId?: string;
  initiatorName?: string;
  renewExistingRegistration?: boolean;
}

/**
 * @class
 * Initializes a new instance of the KEKDetails class.
 * @constructor
 * KEK is encryption key for BEK.
 *
 * @member {string} [keyUrl] Key is KEK.
 * @member {string} [keyVaultId] Key Vault ID where this Key is stored.
 * @member {string} [keyBackupData] KEK data.
 */
export interface KEKDetails {
  keyUrl?: string;
  keyVaultId?: string;
  keyBackupData?: string;
}

/**
 * @class
 * Initializes a new instance of the KeyAndSecretDetails class.
 * @constructor
 * BEK is bitlocker key.
 * KEK is encryption key for BEK
 * If the VM was encrypted then we will store follwing details :
 * 1. Secret(BEK) - Url + Backup Data + vaultId.
 * 2. Key(KEK) - Url + Backup Data + vaultId.
 * BEK and KEK can potentiallty have different vault ids.
 *
 * @member {object} [kekDetails] KEK is encryption key for BEK.
 * @member {string} [kekDetails.keyUrl] Key is KEK.
 * @member {string} [kekDetails.keyVaultId] Key Vault ID where this Key is
 * stored.
 * @member {string} [kekDetails.keyBackupData] KEK data.
 * @member {object} [bekDetails] BEK is bitlocker encrpytion key.
 * @member {string} [bekDetails.secretUrl] Secret is BEK.
 * @member {string} [bekDetails.secretVaultId] ID of the Key Vault where this
 * Secret is stored.
 * @member {string} [bekDetails.secretData] BEK data.
 */
export interface KeyAndSecretDetails {
  kekDetails?: KEKDetails;
  bekDetails?: BEKDetails;
}

/**
 * @class
 * Initializes a new instance of the RecoveryPointTierInformation class.
 * @constructor
 * Recovery point tier information.
 *
 * @member {string} [type] Recovery point tier type. Possible values include:
 * 'Invalid', 'InstantRP', 'HardenedRP'
 * @member {string} [status] Recovery point tier status. Possible values
 * include: 'Invalid', 'Valid', 'Disabled', 'Deleted'
 */
export interface RecoveryPointTierInformation {
  type?: string;
  status?: string;
}

/**
 * @class
 * Initializes a new instance of the IaasVMRecoveryPoint class.
 * @constructor
 * IaaS VM workload specific backup copy.
 *
 * @member {string} [recoveryPointType] Type of the backup copy.
 * @member {date} [recoveryPointTime] Time at which this backup copy was
 * created.
 * @member {string} [recoveryPointAdditionalInfo] Additional information
 * associated with this backup copy.
 * @member {string} [sourceVMStorageType] Storage type of the VM whose backup
 * copy is created.
 * @member {boolean} [isSourceVMEncrypted] Identifies whether the VM was
 * encrypted when the backup copy is created.
 * @member {object} [keyAndSecret] Required details for recovering an encrypted
 * VM. Applicable only when IsSourceVMEncrypted is true.
 * @member {object} [keyAndSecret.kekDetails] KEK is encryption key for BEK.
 * @member {string} [keyAndSecret.kekDetails.keyUrl] Key is KEK.
 * @member {string} [keyAndSecret.kekDetails.keyVaultId] Key Vault ID where
 * this Key is stored.
 * @member {string} [keyAndSecret.kekDetails.keyBackupData] KEK data.
 * @member {object} [keyAndSecret.bekDetails] BEK is bitlocker encrpytion key.
 * @member {string} [keyAndSecret.bekDetails.secretUrl] Secret is BEK.
 * @member {string} [keyAndSecret.bekDetails.secretVaultId] ID of the Key Vault
 * where this Secret is stored.
 * @member {string} [keyAndSecret.bekDetails.secretData] BEK data.
 * @member {boolean} [isInstantILRSessionActive] Is the session to recover
 * items from this backup copy still active.
 * @member {array} [recoveryPointTierDetails] Recovery point tier information.
 * @member {boolean} [isManagedVirtualMachine] Whether VM is with Managed Disks
 * @member {string} [virtualMachineSize] Virtual Machine Size
 * @member {boolean} [originalStorageAccountOption] Original SA Option
 */
export interface IaasVMRecoveryPoint extends RecoveryPoint {
  recoveryPointType?: string;
  recoveryPointTime?: Date;
  recoveryPointAdditionalInfo?: string;
  sourceVMStorageType?: string;
  isSourceVMEncrypted?: boolean;
  keyAndSecret?: KeyAndSecretDetails;
  isInstantILRSessionActive?: boolean;
  recoveryPointTierDetails?: RecoveryPointTierInformation[];
  isManagedVirtualMachine?: boolean;
  virtualMachineSize?: string;
  originalStorageAccountOption?: boolean;
}

/**
 * @class
 * Initializes a new instance of the RestoreRequest class.
 * @constructor
 * Base class for restore request. Workload-specific restore requests are
 * derived from this class.
 *
 * @member {string} objectType Polymorphic Discriminator
 */
export interface RestoreRequest {
  objectType: string;
}

/**
 * @class
 * Initializes a new instance of the IaasVMRestoreRequest class.
 * @constructor
 * IaaS VM workload-specific restore.
 *
 * @member {string} [recoveryPointId] ID of the backup copy to be recovered.
 * @member {string} [recoveryType] Type of this recovery. Possible values
 * include: 'Invalid', 'OriginalLocation', 'AlternateLocation', 'RestoreDisks'
 * @member {string} [sourceResourceId] Fully qualified ARM ID of the VM which
 * is being recovered.
 * @member {string} [targetVirtualMachineId] This is the complete ARM Id of the
 * VM that will be created.
 * For e.g.
 * /subscriptions/{subId}/resourcegroups/{rg}/provider/Microsoft.Compute/virtualmachines/{vm}
 * @member {string} [targetResourceGroupId] This is the ARM Id of the resource
 * group that you want to create for this Virtual machine and other artifacts.
 * For e.g. /subscriptions/{subId}/resourcegroups/{rg}
 * @member {string} [storageAccountId] Fully qualified ARM ID of the storage
 * account to which the VM has to be restored.
 * @member {string} [virtualNetworkId] This is the virtual network Id of the
 * vnet that will be attached to the virtual machine.
 * User will be validated for join action permissions in the linked access.
 * @member {string} [subnetId] Subnet ID, is the subnet ID associated with the
 * to be restored VM. For Classic VMs it would be {VnetID}/Subnet/{SubnetName}
 * and, for the Azure Resource Manager VMs it would be ARM resource ID used to
 * represent the subnet.
 * @member {string} [targetDomainNameId] Fully qualified ARM ID of the domain
 * name to be associated to the VM being restored. This applies only to Classic
 * Virtual Machines.
 * @member {string} [region] Region in which the virtual machine is restored.
 * @member {string} [affinityGroup] Affinity group associated to VM to be
 * restored. Used only for Classic Compute Virtual Machines.
 * @member {boolean} [createNewCloudService] Should a new cloud service be
 * created while restoring the VM. If this is false, VM will be restored to the
 * same cloud service as it was at the time of backup.
 * @member {boolean} [originalStorageAccountOption]
 * @member {object} [encryptionDetails] Details needed if the VM was encrypted
 * at the time of backup.
 * @member {boolean} [encryptionDetails.encryptionEnabled] Identifies whether
 * this backup copy represents an encrypted VM at the time of backup.
 * @member {string} [encryptionDetails.kekUrl] Key Url.
 * @member {string} [encryptionDetails.secretKeyUrl] Secret Url.
 * @member {string} [encryptionDetails.kekVaultId] ID of Key Vault where KEK is
 * stored.
 * @member {string} [encryptionDetails.secretKeyVaultId] ID of Key Vault where
 * Secret is stored.
 */
export interface IaasVMRestoreRequest extends RestoreRequest {
  recoveryPointId?: string;
  recoveryType?: string;
  sourceResourceId?: string;
  targetVirtualMachineId?: string;
  targetResourceGroupId?: string;
  storageAccountId?: string;
  virtualNetworkId?: string;
  subnetId?: string;
  targetDomainNameId?: string;
  region?: string;
  affinityGroup?: string;
  createNewCloudService?: boolean;
  originalStorageAccountOption?: boolean;
  encryptionDetails?: EncryptionDetails;
}

/**
 * @class
 * Initializes a new instance of the ILRRequestResource class.
 * @constructor
 * Parameters to restore file/folders API.
 *
 * @member {object} [properties] ILRRequestResource properties
 * @member {string} [properties.objectType] Polymorphic Discriminator
 */
export interface ILRRequestResource extends Resource {
  properties?: ILRRequest;
}

/**
 * @class
 * Initializes a new instance of the InstantItemRecoveryTarget class.
 * @constructor
 * Target details for file / folder restore.
 *
 * @member {array} [clientScripts] List of client scripts.
 */
export interface InstantItemRecoveryTarget {
  clientScripts?: ClientScriptForConnect[];
}

/**
 * @class
 * Initializes a new instance of the WeeklyRetentionSchedule class.
 * @constructor
 * Weekly retention schedule.
 *
 * @member {array} [daysOfTheWeek] List of days of week for weekly retention
 * policy.
 * @member {array} [retentionTimes] Retention times of retention policy.
 * @member {object} [retentionDuration] Retention duration of retention Policy.
 * @member {number} [retentionDuration.count] Count of duration types.
 * Retention duration is obtained by the counting the duration type Count
 * times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [retentionDuration.durationType] Retention duration type of
 * retention policy. Possible values include: 'Invalid', 'Days', 'Weeks',
 * 'Months', 'Years'
 */
export interface WeeklyRetentionSchedule {
  daysOfTheWeek?: string[];
  retentionTimes?: Date[];
  retentionDuration?: RetentionDuration;
}

/**
 * @class
 * Initializes a new instance of the WeeklyRetentionFormat class.
 * @constructor
 * Weekly retention format.
 *
 * @member {array} [daysOfTheWeek] List of days of the week.
 * @member {array} [weeksOfTheMonth] List of weeks of month.
 */
export interface WeeklyRetentionFormat {
  daysOfTheWeek?: string[];
  weeksOfTheMonth?: string[];
}

/**
 * @class
 * Initializes a new instance of the MonthlyRetentionSchedule class.
 * @constructor
 * Monthly retention schedule.
 *
 * @member {string} [retentionScheduleFormatType] Retention schedule format
 * type for monthly retention policy. Possible values include: 'Invalid',
 * 'Daily', 'Weekly'
 * @member {object} [retentionScheduleDaily] Daily retention format for monthly
 * retention policy.
 * @member {array} [retentionScheduleDaily.daysOfTheMonth] List of days of the
 * month.
 * @member {object} [retentionScheduleWeekly] Weekly retention format for
 * monthly retention policy.
 * @member {array} [retentionScheduleWeekly.daysOfTheWeek] List of days of the
 * week.
 * @member {array} [retentionScheduleWeekly.weeksOfTheMonth] List of weeks of
 * month.
 * @member {array} [retentionTimes] Retention times of retention policy.
 * @member {object} [retentionDuration] Retention duration of retention Policy.
 * @member {number} [retentionDuration.count] Count of duration types.
 * Retention duration is obtained by the counting the duration type Count
 * times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [retentionDuration.durationType] Retention duration type of
 * retention policy. Possible values include: 'Invalid', 'Days', 'Weeks',
 * 'Months', 'Years'
 */
export interface MonthlyRetentionSchedule {
  retentionScheduleFormatType?: string;
  retentionScheduleDaily?: DailyRetentionFormat;
  retentionScheduleWeekly?: WeeklyRetentionFormat;
  retentionTimes?: Date[];
  retentionDuration?: RetentionDuration;
}

/**
 * @class
 * Initializes a new instance of the YearlyRetentionSchedule class.
 * @constructor
 * Yearly retention schedule.
 *
 * @member {string} [retentionScheduleFormatType] Retention schedule format for
 * yearly retention policy. Possible values include: 'Invalid', 'Daily',
 * 'Weekly'
 * @member {array} [monthsOfYear] List of months of year of yearly retention
 * policy.
 * @member {object} [retentionScheduleDaily] Daily retention format for yearly
 * retention policy.
 * @member {array} [retentionScheduleDaily.daysOfTheMonth] List of days of the
 * month.
 * @member {object} [retentionScheduleWeekly] Weekly retention format for
 * yearly retention policy.
 * @member {array} [retentionScheduleWeekly.daysOfTheWeek] List of days of the
 * week.
 * @member {array} [retentionScheduleWeekly.weeksOfTheMonth] List of weeks of
 * month.
 * @member {array} [retentionTimes] Retention times of retention policy.
 * @member {object} [retentionDuration] Retention duration of retention Policy.
 * @member {number} [retentionDuration.count] Count of duration types.
 * Retention duration is obtained by the counting the duration type Count
 * times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [retentionDuration.durationType] Retention duration type of
 * retention policy. Possible values include: 'Invalid', 'Days', 'Weeks',
 * 'Months', 'Years'
 */
export interface YearlyRetentionSchedule {
  retentionScheduleFormatType?: string;
  monthsOfYear?: string[];
  retentionScheduleDaily?: DailyRetentionFormat;
  retentionScheduleWeekly?: WeeklyRetentionFormat;
  retentionTimes?: Date[];
  retentionDuration?: RetentionDuration;
}

/**
 * @class
 * Initializes a new instance of the LongTermRetentionPolicy class.
 * @constructor
 * Long term retention policy.
 *
 * @member {object} [dailySchedule] Daily retention schedule of the protection
 * policy.
 * @member {array} [dailySchedule.retentionTimes] Retention times of retention
 * policy.
 * @member {object} [dailySchedule.retentionDuration] Retention duration of
 * retention Policy.
 * @member {number} [dailySchedule.retentionDuration.count] Count of duration
 * types. Retention duration is obtained by the counting the duration type
 * Count times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [dailySchedule.retentionDuration.durationType] Retention
 * duration type of retention policy. Possible values include: 'Invalid',
 * 'Days', 'Weeks', 'Months', 'Years'
 * @member {object} [weeklySchedule] Weekly retention schedule of the
 * protection policy.
 * @member {array} [weeklySchedule.daysOfTheWeek] List of days of week for
 * weekly retention policy.
 * @member {array} [weeklySchedule.retentionTimes] Retention times of retention
 * policy.
 * @member {object} [weeklySchedule.retentionDuration] Retention duration of
 * retention Policy.
 * @member {number} [weeklySchedule.retentionDuration.count] Count of duration
 * types. Retention duration is obtained by the counting the duration type
 * Count times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [weeklySchedule.retentionDuration.durationType] Retention
 * duration type of retention policy. Possible values include: 'Invalid',
 * 'Days', 'Weeks', 'Months', 'Years'
 * @member {object} [monthlySchedule] Monthly retention schedule of the
 * protection policy.
 * @member {string} [monthlySchedule.retentionScheduleFormatType] Retention
 * schedule format type for monthly retention policy. Possible values include:
 * 'Invalid', 'Daily', 'Weekly'
 * @member {object} [monthlySchedule.retentionScheduleDaily] Daily retention
 * format for monthly retention policy.
 * @member {array} [monthlySchedule.retentionScheduleDaily.daysOfTheMonth] List
 * of days of the month.
 * @member {object} [monthlySchedule.retentionScheduleWeekly] Weekly retention
 * format for monthly retention policy.
 * @member {array} [monthlySchedule.retentionScheduleWeekly.daysOfTheWeek] List
 * of days of the week.
 * @member {array} [monthlySchedule.retentionScheduleWeekly.weeksOfTheMonth]
 * List of weeks of month.
 * @member {array} [monthlySchedule.retentionTimes] Retention times of
 * retention policy.
 * @member {object} [monthlySchedule.retentionDuration] Retention duration of
 * retention Policy.
 * @member {number} [monthlySchedule.retentionDuration.count] Count of duration
 * types. Retention duration is obtained by the counting the duration type
 * Count times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [monthlySchedule.retentionDuration.durationType] Retention
 * duration type of retention policy. Possible values include: 'Invalid',
 * 'Days', 'Weeks', 'Months', 'Years'
 * @member {object} [yearlySchedule] Yearly retention schedule of the
 * protection policy.
 * @member {string} [yearlySchedule.retentionScheduleFormatType] Retention
 * schedule format for yearly retention policy. Possible values include:
 * 'Invalid', 'Daily', 'Weekly'
 * @member {array} [yearlySchedule.monthsOfYear] List of months of year of
 * yearly retention policy.
 * @member {object} [yearlySchedule.retentionScheduleDaily] Daily retention
 * format for yearly retention policy.
 * @member {array} [yearlySchedule.retentionScheduleDaily.daysOfTheMonth] List
 * of days of the month.
 * @member {object} [yearlySchedule.retentionScheduleWeekly] Weekly retention
 * format for yearly retention policy.
 * @member {array} [yearlySchedule.retentionScheduleWeekly.daysOfTheWeek] List
 * of days of the week.
 * @member {array} [yearlySchedule.retentionScheduleWeekly.weeksOfTheMonth]
 * List of weeks of month.
 * @member {array} [yearlySchedule.retentionTimes] Retention times of retention
 * policy.
 * @member {object} [yearlySchedule.retentionDuration] Retention duration of
 * retention Policy.
 * @member {number} [yearlySchedule.retentionDuration.count] Count of duration
 * types. Retention duration is obtained by the counting the duration type
 * Count times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [yearlySchedule.retentionDuration.durationType] Retention
 * duration type of retention policy. Possible values include: 'Invalid',
 * 'Days', 'Weeks', 'Months', 'Years'
 */
export interface LongTermRetentionPolicy extends RetentionPolicy {
  dailySchedule?: DailyRetentionSchedule;
  weeklySchedule?: WeeklyRetentionSchedule;
  monthlySchedule?: MonthlyRetentionSchedule;
  yearlySchedule?: YearlyRetentionSchedule;
}

/**
 * @class
 * Initializes a new instance of the LongTermSchedulePolicy class.
 * @constructor
 * Long term policy schedule.
 *
 */
export interface LongTermSchedulePolicy extends SchedulePolicy {
}

/**
 * @class
 * Initializes a new instance of the MabContainerExtendedInfo class.
 * @constructor
 * Additional information of the container.
 *
 * @member {date} [lastRefreshedAt] Time stamp when this container was
 * refreshed.
 * @member {string} [backupItemType] Type of backup items associated with this
 * container. Possible values include: 'Invalid', 'VM', 'FileFolder',
 * 'AzureSqlDb', 'SQLDB', 'Exchange', 'Sharepoint', 'VMwareVM', 'SystemState',
 * 'Client', 'GenericDataSource'
 * @member {array} [backupItems] List of backup items associated with this
 * container.
 * @member {string} [policyName] Backup policy associated with this container.
 * @member {string} [lastBackupStatus] Latest backup status of this container.
 */
export interface MabContainerExtendedInfo {
  lastRefreshedAt?: Date;
  backupItemType?: string;
  backupItems?: string[];
  policyName?: string;
  lastBackupStatus?: string;
}

/**
 * @class
 * Initializes a new instance of the MabContainer class.
 * @constructor
 * Container with items backed up using MAB backup engine.
 *
 * @member {boolean} [canReRegister] Can the container be registered one more
 * time.
 * @member {number} [containerId] ContainerID represents the container.
 * @member {number} [protectedItemCount] Number of items backed up in this
 * container.
 * @member {string} [agentVersion] Agent version of this container.
 * @member {object} [extendedInfo] Additional information for this container
 * @member {date} [extendedInfo.lastRefreshedAt] Time stamp when this container
 * was refreshed.
 * @member {string} [extendedInfo.backupItemType] Type of backup items
 * associated with this container. Possible values include: 'Invalid', 'VM',
 * 'FileFolder', 'AzureSqlDb', 'SQLDB', 'Exchange', 'Sharepoint', 'VMwareVM',
 * 'SystemState', 'Client', 'GenericDataSource'
 * @member {array} [extendedInfo.backupItems] List of backup items associated
 * with this container.
 * @member {string} [extendedInfo.policyName] Backup policy associated with
 * this container.
 * @member {string} [extendedInfo.lastBackupStatus] Latest backup status of
 * this container.
 */
export interface MabContainer extends ProtectionContainer {
  canReRegister?: boolean;
  containerId?: number;
  protectedItemCount?: number;
  agentVersion?: string;
  extendedInfo?: MabContainerExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the MabFileFolderProtectedItemExtendedInfo class.
 * @constructor
 * Additional information on the backed up item.
 *
 * @member {date} [lastRefreshedAt] Last time when the agent data synced to
 * service.
 * @member {date} [oldestRecoveryPoint] The oldest backup copy available.
 * @member {number} [recoveryPointCount] Number of backup copies associated
 * with the backup item.
 */
export interface MabFileFolderProtectedItemExtendedInfo {
  lastRefreshedAt?: Date;
  oldestRecoveryPoint?: Date;
  recoveryPointCount?: number;
}

/**
 * @class
 * Initializes a new instance of the MabFileFolderProtectedItem class.
 * @constructor
 * MAB workload-specific backup item.
 *
 * @member {string} [friendlyName] Friendly name of this backup item.
 * @member {string} [computerName] Name of the computer associated with this
 * backup item.
 * @member {string} [lastBackupStatus] Status of last backup operation.
 * @member {string} [protectionState] Protected, ProtectionStopped, IRPending
 * or ProtectionError
 * @member {boolean} [isScheduledForDeferredDelete] Specifies if the item is
 * scheduled for deferred deletion.
 * @member {number} [deferredDeleteSyncTimeInUTC] Sync time for deferred
 * deletion.
 * @member {object} [extendedInfo] Additional information with this backup
 * item.
 * @member {date} [extendedInfo.lastRefreshedAt] Last time when the agent data
 * synced to service.
 * @member {date} [extendedInfo.oldestRecoveryPoint] The oldest backup copy
 * available.
 * @member {number} [extendedInfo.recoveryPointCount] Number of backup copies
 * associated with the backup item.
 */
export interface MabFileFolderProtectedItem extends ProtectedItem {
  friendlyName?: string;
  computerName?: string;
  lastBackupStatus?: string;
  protectionState?: string;
  isScheduledForDeferredDelete?: boolean;
  deferredDeleteSyncTimeInUTC?: number;
  extendedInfo?: MabFileFolderProtectedItemExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the MabProtectionPolicy class.
 * @constructor
 * Mab container-specific backup policy.
 *
 * @member {object} [schedulePolicy] Backup schedule of backup policy.
 * @member {string} [schedulePolicy.schedulePolicyType] Polymorphic
 * Discriminator
 * @member {object} [retentionPolicy] Retention policy details.
 * @member {string} [retentionPolicy.retentionPolicyType] Polymorphic
 * Discriminator
 */
export interface MabProtectionPolicy extends ProtectionPolicy {
  schedulePolicy?: SchedulePolicy;
  retentionPolicy?: RetentionPolicy;
}

/**
 * @class
 * Initializes a new instance of the OperationResultInfo class.
 * @constructor
 * Operation result info.
 *
 * @member {array} [jobList] List of jobs created by this operation.
 */
export interface OperationResultInfo extends OperationResultInfoBase {
  jobList?: string[];
}

/**
 * @class
 * Initializes a new instance of the OperationWorkerResponse class.
 * @constructor
 * This is the base class for operation result responses.
 *
 * @member {string} [statusCode] HTTP Status Code of the operation. Possible
 * values include: 'Continue', 'SwitchingProtocols', 'OK', 'Created',
 * 'Accepted', 'NonAuthoritativeInformation', 'NoContent', 'ResetContent',
 * 'PartialContent', 'MultipleChoices', 'Ambiguous', 'MovedPermanently',
 * 'Moved', 'Found', 'Redirect', 'SeeOther', 'RedirectMethod', 'NotModified',
 * 'UseProxy', 'Unused', 'TemporaryRedirect', 'RedirectKeepVerb', 'BadRequest',
 * 'Unauthorized', 'PaymentRequired', 'Forbidden', 'NotFound',
 * 'MethodNotAllowed', 'NotAcceptable', 'ProxyAuthenticationRequired',
 * 'RequestTimeout', 'Conflict', 'Gone', 'LengthRequired',
 * 'PreconditionFailed', 'RequestEntityTooLarge', 'RequestUriTooLong',
 * 'UnsupportedMediaType', 'RequestedRangeNotSatisfiable', 'ExpectationFailed',
 * 'UpgradeRequired', 'InternalServerError', 'NotImplemented', 'BadGateway',
 * 'ServiceUnavailable', 'GatewayTimeout', 'HttpVersionNotSupported'
 * @member {object} [headers] HTTP headers associated with this operation.
 */
export interface OperationWorkerResponse {
  statusCode?: string;
  headers?: { [propertyName: string]: string[] };
}

/**
 * @class
 * Initializes a new instance of the OperationResultInfoBaseResource class.
 * @constructor
 * Base class for operation result info.
 *
 * @member {object} [operation] OperationResultInfoBaseResource operation
 * @member {string} [operation.objectType] Polymorphic Discriminator
 */
export interface OperationResultInfoBaseResource extends OperationWorkerResponse {
  operation?: OperationResultInfoBase;
}

/**
 * @class
 * Initializes a new instance of the OperationStatusError class.
 * @constructor
 * Error information associated with operation status call.
 *
 * @member {string} [code] Error code of the operation failure.
 * @member {string} [message] Error message displayed if the operation failure.
 */
export interface OperationStatusError {
  code?: string;
  message?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationStatusExtendedInfo class.
 * @constructor
 * Base class for additional information of operation status.
 *
 * @member {string} objectType Polymorphic Discriminator
 */
export interface OperationStatusExtendedInfo {
  objectType: string;
}

/**
 * @class
 * Initializes a new instance of the OperationStatus class.
 * @constructor
 * Operation status.
 *
 * @member {string} [id] ID of the operation.
 * @member {string} [name] Name of the operation.
 * @member {string} [status] Operation status. Possible values include:
 * 'Invalid', 'InProgress', 'Succeeded', 'Failed', 'Canceled'
 * @member {date} [startTime] Operation start time. Format: ISO-8601.
 * @member {date} [endTime] Operation end time. Format: ISO-8601.
 * @member {object} [error] Error information related to this operation.
 * @member {string} [error.code] Error code of the operation failure.
 * @member {string} [error.message] Error message displayed if the operation
 * failure.
 * @member {object} [properties] Additional information associated with this
 * operation.
 * @member {string} [properties.objectType] Polymorphic Discriminator
 */
export interface OperationStatus {
  id?: string;
  name?: string;
  status?: string;
  startTime?: Date;
  endTime?: Date;
  error?: OperationStatusError;
  properties?: OperationStatusExtendedInfo;
}

/**
 * @class
 * Initializes a new instance of the OperationStatusJobExtendedInfo class.
 * @constructor
 * Operation status job extended info.
 *
 * @member {string} [jobId] ID of the job created for this protected item.
 */
export interface OperationStatusJobExtendedInfo extends OperationStatusExtendedInfo {
  jobId?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationStatusJobsExtendedInfo class.
 * @constructor
 * Operation status extended info for list of jobs.
 *
 * @member {array} [jobIds] IDs of the jobs created for the protected item.
 * @member {object} [failedJobsError] Stores all the failed jobs along with the
 * corresponding error codes.
 */
export interface OperationStatusJobsExtendedInfo extends OperationStatusExtendedInfo {
  jobIds?: string[];
  failedJobsError?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the OperationStatusProvisionILRExtendedInfo class.
 * @constructor
 * Operation status extended info for ILR provision action.
 *
 * @member {object} [recoveryTarget] Target details for file / folder restore.
 * @member {array} [recoveryTarget.clientScripts] List of client scripts.
 */
export interface OperationStatusProvisionILRExtendedInfo extends OperationStatusExtendedInfo {
  recoveryTarget?: InstantItemRecoveryTarget;
}

/**
 * @class
 * Initializes a new instance of the ProtectedItemQueryObject class.
 * @constructor
 * Filters to list backup items.
 *
 * @member {string} [healthState] Health State for the backed up item. Possible
 * values include: 'Passed', 'ActionRequired', 'ActionSuggested', 'Invalid'
 * @member {string} [backupManagementType] Backup management type for the
 * backed up item. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB',
 * 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [itemType] Type of workload this item represents. Possible
 * values include: 'Invalid', 'VM', 'FileFolder', 'AzureSqlDb', 'SQLDB',
 * 'Exchange', 'Sharepoint', 'VMwareVM', 'SystemState', 'Client',
 * 'GenericDataSource'
 * @member {string} [policyName] Backup policy name associated with the backup
 * item.
 * @member {string} [containerName] Name of the container.
 * @member {string} [backupEngineName] Backup Engine name
 * @member {string} [friendlyName] Friendly name of protected item
 */
export interface ProtectedItemQueryObject {
  healthState?: string;
  backupManagementType?: string;
  itemType?: string;
  policyName?: string;
  containerName?: string;
  backupEngineName?: string;
  friendlyName?: string;
}

/**
 * @class
 * Initializes a new instance of the ProtectedItemResource class.
 * @constructor
 * Base class for backup items.
 *
 * @member {object} [properties] ProtectedItemResource properties
 * @member {string} [properties.backupManagementType] Type of backup
 * managemenent for the backed up item. Possible values include: 'Invalid',
 * 'AzureIaasVM', 'MAB', 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [properties.workloadType] Type of workload this item
 * represents. Possible values include: 'Invalid', 'VM', 'FileFolder',
 * 'AzureSqlDb', 'SQLDB', 'Exchange', 'Sharepoint', 'VMwareVM', 'SystemState',
 * 'Client', 'GenericDataSource'
 * @member {string} [properties.containerName] Unique name of container
 * @member {string} [properties.sourceResourceId] ARM ID of the resource to be
 * backed up.
 * @member {string} [properties.policyId] ID of the backup policy with which
 * this item is backed up.
 * @member {date} [properties.lastRecoveryPoint] Timestamp when the last
 * (latest) backup copy was created for this backup item.
 * @member {string} [properties.protectedItemType] Polymorphic Discriminator
 */
export interface ProtectedItemResource extends Resource {
  properties?: ProtectedItem;
}

/**
 * @class
 * Initializes a new instance of the ProtectionContainerResource class.
 * @constructor
 * Base class for container with backup items. Containers with specific
 * workloads are derived from this class.
 *
 * @member {object} [properties] ProtectionContainerResource properties
 * @member {string} [properties.friendlyName] Friendly name of the container.
 * @member {string} [properties.backupManagementType] Type of backup
 * managemenent for the container. Possible values include: 'Invalid',
 * 'AzureIaasVM', 'MAB', 'DPM', 'AzureBackupServer', 'AzureSql'
 * @member {string} [properties.registrationStatus] Status of registration of
 * the container with the Recovery Services Vault.
 * @member {string} [properties.healthStatus] Status of health of the
 * container.
 * @member {string} [properties.containerType] Type of the container. The value
 * of this property for: 1. Compute Azure VM is
 * Microsoft.Compute/virtualMachines 2. Classic Compute Azure VM is
 * Microsoft.ClassicCompute/virtualMachines 3. Windows machines (like MAB, DPM
 * etc) is Windows 4. Azure SQL instance is AzureSqlContainer. Possible values
 * include: 'Invalid', 'Unknown', 'IaasVMContainer', 'IaasVMServiceContainer',
 * 'DPMContainer', 'AzureBackupServerContainer', 'MABContainer', 'Cluster',
 * 'AzureSqlContainer', 'Windows', 'VCenter'
 * @member {string} [properties.protectableObjectType] Polymorphic
 * Discriminator
 */
export interface ProtectionContainerResource extends Resource {
  properties?: ProtectionContainer;
}

/**
 * @class
 * Initializes a new instance of the ProtectionPolicyQueryObject class.
 * @constructor
 * Filters the list backup policies API.
 *
 * @member {string} [backupManagementType] Backup management type for the
 * backup policy. Possible values include: 'Invalid', 'AzureIaasVM', 'MAB',
 * 'DPM', 'AzureBackupServer', 'AzureSql'
 */
export interface ProtectionPolicyQueryObject {
  backupManagementType?: string;
}

/**
 * @class
 * Initializes a new instance of the ProtectionPolicyResource class.
 * @constructor
 * Base class for backup policy. Workload-specific backup policies are derived
 * from this class.
 *
 * @member {object} [properties] ProtectionPolicyResource properties
 * @member {number} [properties.protectedItemsCount] Number of items associated
 * with this policy.
 * @member {string} [properties.backupManagementType] Polymorphic Discriminator
 */
export interface ProtectionPolicyResource extends Resource {
  properties?: ProtectionPolicy;
}

/**
 * @class
 * Initializes a new instance of the RecoveryPointResource class.
 * @constructor
 * Base class for backup copies. Workload-specific backup copies are derived
 * from this class.
 *
 * @member {object} [properties] RecoveryPointResource properties
 * @member {string} [properties.objectType] Polymorphic Discriminator
 */
export interface RecoveryPointResource extends Resource {
  properties?: RecoveryPoint;
}

/**
 * @class
 * Initializes a new instance of the RestoreRequestResource class.
 * @constructor
 * Base class for restore request. Workload-specific restore requests are
 * derived from this class.
 *
 * @member {object} [properties] RestoreRequestResource properties
 * @member {string} [properties.objectType] Polymorphic Discriminator
 */
export interface RestoreRequestResource extends Resource {
  properties?: RestoreRequest;
}

/**
 * @class
 * Initializes a new instance of the SimpleRetentionPolicy class.
 * @constructor
 * Simple policy retention.
 *
 * @member {object} [retentionDuration] Retention duration of the protection
 * policy.
 * @member {number} [retentionDuration.count] Count of duration types.
 * Retention duration is obtained by the counting the duration type Count
 * times.
 * For example, when Count = 3 and DurationType = Weeks, retention duration
 * will be three weeks.
 * @member {string} [retentionDuration.durationType] Retention duration type of
 * retention policy. Possible values include: 'Invalid', 'Days', 'Weeks',
 * 'Months', 'Years'
 */
export interface SimpleRetentionPolicy extends RetentionPolicy {
  retentionDuration?: RetentionDuration;
}

/**
 * @class
 * Initializes a new instance of the SimpleSchedulePolicy class.
 * @constructor
 * Simple policy schedule.
 *
 * @member {string} [scheduleRunFrequency] Frequency of the schedule operation
 * of this policy. Possible values include: 'Invalid', 'Daily', 'Weekly'
 * @member {array} [scheduleRunDays] List of days of week this schedule has to
 * be run.
 * @member {array} [scheduleRunTimes] List of times of day this schedule has to
 * be run.
 * @member {number} [scheduleWeeklyFrequency] At every number weeks this
 * schedule has to be run.
 */
export interface SimpleSchedulePolicy extends SchedulePolicy {
  scheduleRunFrequency?: string;
  scheduleRunDays?: string[];
  scheduleRunTimes?: Date[];
  scheduleWeeklyFrequency?: number;
}

/**
 * @class
 * Initializes a new instance of the TokenInformation class.
 * @constructor
 * The token information details.
 *
 * @member {string} [token] Token value.
 * @member {number} [expiryTimeInUtcTicks] Expiry time of token.
 * @member {string} [securityPIN] Security PIN
 */
export interface TokenInformation {
  token?: string;
  expiryTimeInUtcTicks?: number;
  securityPIN?: string;
}

/**
 * @class
 * Initializes a new instance of the WorkloadProtectableItemResource class.
 * @constructor
 * Base class for backup item. Workload-specific backup items are derived from
 * this class.
 *
 * @member {object} [properties] WorkloadProtectableItemResource properties
 * @member {string} [properties.backupManagementType] Type of backup
 * managemenent to backup an item.
 * @member {string} [properties.friendlyName] Friendly name of the backup item.
 * @member {string} [properties.protectionState] State of the back up item.
 * Possible values include: 'Invalid', 'NotProtected', 'Protecting',
 * 'Protected'
 * @member {string} [properties.protectableItemType] Polymorphic Discriminator
 */
export interface WorkloadProtectableItemResource extends Resource {
  properties?: WorkloadProtectableItem;
}

/**
 * @class
 * Initializes a new instance of the ClientDiscoveryDisplay class.
 * @constructor
 * Localized display information of an operation.
 *
 * @member {string} [provider] Name of the provider for display purposes
 * @member {string} [resource] Name of the resource type for display purposes
 * @member {string} [operation] Name of the operation for display purposes
 * @member {string} [description] Description of the operation for display
 * purposes
 */
export interface ClientDiscoveryDisplay {
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the ClientDiscoveryForLogSpecification class.
 * @constructor
 * Class to represent shoebox log specification in json client discovery.
 *
 * @member {string} [name] Name
 * @member {string} [displayName] Localized display name
 * @member {string} [blobDuration] blob duration
 */
export interface ClientDiscoveryForLogSpecification {
  name?: string;
  displayName?: string;
  blobDuration?: string;
}

/**
 * @class
 * Initializes a new instance of the ClientDiscoveryForServiceSpecification class.
 * @constructor
 * Class to represent shoebox service specification in json client discovery.
 *
 * @member {array} [logSpecifications] List of log specifications of this
 * operation.
 */
export interface ClientDiscoveryForServiceSpecification {
  logSpecifications?: ClientDiscoveryForLogSpecification[];
}

/**
 * @class
 * Initializes a new instance of the ClientDiscoveryForProperties class.
 * @constructor
 * Class to represent shoebox properties in json client discovery.
 *
 * @member {object} [serviceSpecification] Operation properties.
 * @member {array} [serviceSpecification.logSpecifications] List of log
 * specifications of this operation.
 */
export interface ClientDiscoveryForProperties {
  serviceSpecification?: ClientDiscoveryForServiceSpecification;
}

/**
 * @class
 * Initializes a new instance of the ClientDiscoveryValueForSingleApi class.
 * @constructor
 * Available operation details.
 *
 * @member {string} [name] Name
 * @member {object} [display] Contains the localized display information for
 * this particular operation
 * @member {string} [display.provider] Name of the provider for display
 * purposes
 * @member {string} [display.resource] Name of the resource type for display
 * purposes
 * @member {string} [display.operation] Name of the operation for display
 * purposes
 * @member {string} [display.description] Description of the operation for
 * display purposes
 * @member {string} [origin] The intended executor of the operation;governs the
 * display of the operation in the RBAC UX and the audit logs UX
 * @member {object} [properties] Properties
 * @member {object} [properties.serviceSpecification] Operation properties.
 * @member {array} [properties.serviceSpecification.logSpecifications] List of
 * log specifications of this operation.
 */
export interface ClientDiscoveryValueForSingleApi {
  name?: string;
  display?: ClientDiscoveryDisplay;
  origin?: string;
  properties?: ClientDiscoveryForProperties;
}


/**
 * @class
 * Initializes a new instance of the JobResourceList class.
 * @constructor
 * List of Job resources
 *
 */
export interface JobResourceList extends Array<JobResource> {
}

/**
 * @class
 * Initializes a new instance of the BackupEngineBaseResourceList class.
 * @constructor
 * List of BackupEngineBase resources
 *
 */
export interface BackupEngineBaseResourceList extends Array<BackupEngineBaseResource> {
}

/**
 * @class
 * Initializes a new instance of the RecoveryPointResourceList class.
 * @constructor
 * List of RecoveryPoint resources
 *
 */
export interface RecoveryPointResourceList extends Array<RecoveryPointResource> {
}

/**
 * @class
 * Initializes a new instance of the ProtectionPolicyResourceList class.
 * @constructor
 * List of ProtectionPolicy resources
 *
 */
export interface ProtectionPolicyResourceList extends Array<ProtectionPolicyResource> {
}

/**
 * @class
 * Initializes a new instance of the WorkloadProtectableItemResourceList class.
 * @constructor
 * List of WorkloadProtectableItem resources
 *
 */
export interface WorkloadProtectableItemResourceList extends Array<WorkloadProtectableItemResource> {
}

/**
 * @class
 * Initializes a new instance of the ProtectedItemResourceList class.
 * @constructor
 * List of ProtectedItem resources
 *
 */
export interface ProtectedItemResourceList extends Array<ProtectedItemResource> {
}

/**
 * @class
 * Initializes a new instance of the ProtectionContainerResourceList class.
 * @constructor
 * List of ProtectionContainer resources
 *
 */
export interface ProtectionContainerResourceList extends Array<ProtectionContainerResource> {
}

/**
 * @class
 * Initializes a new instance of the BackupManagementUsageList class.
 * @constructor
 * Backup management usage for vault.
 *
 */
export interface BackupManagementUsageList extends Array<BackupManagementUsage> {
}

/**
 * @class
 * Initializes a new instance of the ClientDiscoveryResponse class.
 * @constructor
 * Operations List response which contains list of available APIs.
 *
 * @member {string} [nextLink] NextLink
 */
export interface ClientDiscoveryResponse extends Array<ClientDiscoveryValueForSingleApi> {
  nextLink?: string;
}
