/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the ErrorDetails class.
 * @constructor
 * The details of the error.
 *
 * @member {string} [code] One of a server-defined set of error codes.
 * @member {string} [message] A human-readable representation of the error.
 * @member {string} [target] (Optional) The target of the error.
 */
export interface ErrorDetails {
  code?: string;
  message?: string;
  target?: string;
}

/**
 * @class
 * Initializes a new instance of the ErrorResponse class.
 * @constructor
 * The error object.
 *
 * @member {object} [error] Error.
 * @member {string} [error.code] One of a server-defined set of error codes.
 * @member {string} [error.message] A human-readable representation of the
 * error.
 * @member {string} [error.target] (Optional) The target of the error.
 */
export interface ErrorResponse {
  error?: ErrorDetails;
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * The object that represents the operation.
 *
 * @member {string} [provider] Service provider: Microsoft.Management.
 * @member {string} [resource] Resource on which the operation is performed:
 * Invoice, etc.
 * @member {string} [operation] Operation type: Read, write, delete, etc.
 */
export interface OperationDisplay {
  readonly provider?: string;
  readonly resource?: string;
  readonly operation?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * A Management REST API operation.
 *
 * @member {string} [name] Operation name: {provider}/{resource}/{operation}.
 * @member {object} [display] The object that represents the operation.
 * @member {string} [display.provider] Service provider: Microsoft.Management.
 * @member {string} [display.resource] Resource on which the operation is
 * performed: Invoice, etc.
 * @member {string} [display.operation] Operation type: Read, write, delete,
 * etc.
 */
export interface Operation {
  readonly name?: string;
  display?: OperationDisplay;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupInfo class.
 * @constructor
 * The management group.
 *
 * @member {string} [id] The ID of the management group. E.g.
 * /providers/Microsoft.Management/managementGroups/20000000-0000-0000-0000-000000000000
 * @member {string} [type] The type of the resource. E.g.
 * /providers/Microsoft.Management/managementGroups
 * @member {uuid} [name] The name of the management group. E.g.
 * 20000000-0000-0000-0000-000000000000
 * @member {uuid} [tenantId] The AAD Tenant ID associated with the management
 * group. E.g. 10000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 */
export interface ManagementGroupInfo {
  readonly id?: string;
  readonly type?: string;
  readonly name?: string;
  tenantId?: string;
  displayName?: string;
}

/**
 * @class
 * Initializes a new instance of the ParentGroupInfo class.
 * @constructor
 * (Optional) The ID of the parent management group.
 *
 * @member {string} [parentId] The ID of the parent management group. E.g.
 * /providers/Microsoft.Management/managementGroups/30000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 */
export interface ParentGroupInfo {
  parentId?: string;
  displayName?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupDetailsProperties class.
 * @constructor
 * The details properties of a management group.
 *
 * @member {number} [version] The version number of the object.
 * @member {date} [updatedTime] The date and time when this object was last
 * updated.
 * @member {string} [updatedBy] The identity of the principal or process that
 * updated the object.
 * @member {object} [parent] Parent.
 * @member {string} [parent.parentId] The ID of the parent management group.
 * E.g.
 * /providers/Microsoft.Management/managementGroups/30000000-0000-0000-0000-000000000000
 * @member {string} [parent.displayName] The friendly name of the management
 * group.
 * @member {string} [managementGroupType] Management Group Type. Possible
 * values include: 'Enrollment', 'Department', 'Account', 'Subscription'
 */
export interface ManagementGroupDetailsProperties {
  version?: number;
  updatedTime?: Date;
  updatedBy?: string;
  parent?: ParentGroupInfo;
  managementGroupType?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroup class.
 * @constructor
 * The management group details.
 *
 * @member {string} [id] The ID of the management group. E.g.
 * /providers/Microsoft.Management/managementGroups/20000000-0000-0000-0000-000000000000
 * @member {string} [type] The type of the resource. E.g.
 * /providers/Microsoft.Management/managementGroups
 * @member {uuid} [name] The name of the management group. E.g.
 * 20000000-0000-0000-0000-000000000000
 * @member {uuid} [tenantId] The AAD Tenant ID associated with the management
 * group. E.g. 10000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 * @member {object} [details] Details.
 * @member {number} [details.version] The version number of the object.
 * @member {date} [details.updatedTime] The date and time when this object was
 * last updated.
 * @member {string} [details.updatedBy] The identity of the principal or
 * process that updated the object.
 * @member {object} [details.parent]
 * @member {string} [details.parent.parentId] The ID of the parent management
 * group. E.g.
 * /providers/Microsoft.Management/managementGroups/30000000-0000-0000-0000-000000000000
 * @member {string} [details.parent.displayName] The friendly name of the
 * management group.
 * @member {string} [details.managementGroupType] Possible values include:
 * 'Enrollment', 'Department', 'Account', 'Subscription'
 */
export interface ManagementGroup {
  readonly id?: string;
  readonly type?: string;
  readonly name?: string;
  tenantId?: string;
  displayName?: string;
  details?: ManagementGroupDetailsProperties;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupChildInfo class.
 * @constructor
 * The unique identifier (ID) of a management group.
 *
 * @member {string} [childType] Managment Group Child Info. Possible values
 * include: 'Enrollment', 'Department', 'Account', 'Subscription'
 * @member {string} [childId] The ID of the child resource (management group or
 * subscription). E.g.
 * /providers/Microsoft.Management/managementGroups/40000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the child resource.
 * @member {uuid} [tenantId] (Optional) The AAD Tenant ID associated with the
 * child resource.
 */
export interface ManagementGroupChildInfo {
  childType?: string;
  childId?: string;
  displayName?: string;
  tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupWithChildren class.
 * @constructor
 * The management group details.
 *
 * @member {string} [id] The ID of the management group. E.g.
 * /providers/Microsoft.Management/managementGroups/20000000-0000-0000-0000-000000000000
 * @member {string} [type] The type of the resource. E.g.
 * /providers/Microsoft.Management/managementGroups
 * @member {uuid} [name] The name of the management group. E.g.
 * 20000000-0000-0000-0000-000000000000
 * @member {uuid} [tenantId] The AAD Tenant ID associated with the management
 * group. E.g. 10000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 * @member {object} [details] Details.
 * @member {number} [details.version] The version number of the object.
 * @member {date} [details.updatedTime] The date and time when this object was
 * last updated.
 * @member {string} [details.updatedBy] The identity of the principal or
 * process that updated the object.
 * @member {object} [details.parent]
 * @member {string} [details.parent.parentId] The ID of the parent management
 * group. E.g.
 * /providers/Microsoft.Management/managementGroups/30000000-0000-0000-0000-000000000000
 * @member {string} [details.parent.displayName] The friendly name of the
 * management group.
 * @member {string} [details.managementGroupType] Possible values include:
 * 'Enrollment', 'Department', 'Account', 'Subscription'
 * @member {array} [children] The list of children.
 */
export interface ManagementGroupWithChildren {
  readonly id?: string;
  readonly type?: string;
  readonly name?: string;
  tenantId?: string;
  displayName?: string;
  details?: ManagementGroupDetailsProperties;
  children?: ManagementGroupChildInfo[];
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupRecursiveChildInfo class.
 * @constructor
 * The unique identifier (ID) of a management group.
 *
 * @member {string} [childType] Managment Group Recursive Child Info. Possible
 * values include: 'Enrollment', 'Department', 'Account', 'Subscription'
 * @member {string} [childId] The ID of the child resource (management group or
 * subscription). E.g.
 * /providers/Microsoft.Management/managementGroups/40000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the child resource.
 * @member {uuid} [tenantId] (Optional) The AAD Tenant ID associated with the
 * child resource.
 * @member {array} [children] The list of children.
 */
export interface ManagementGroupRecursiveChildInfo {
  childType?: string;
  childId?: string;
  displayName?: string;
  tenantId?: string;
  children?: ManagementGroupRecursiveChildInfo[];
}

/**
 * @class
 * Initializes a new instance of the ManagementGroupWithHierarchy class.
 * @constructor
 * The management group details.
 *
 * @member {string} [id] The ID of the management group. E.g.
 * /providers/Microsoft.Management/managementGroups/20000000-0000-0000-0000-000000000000
 * @member {string} [type] The type of the resource. E.g.
 * /providers/Microsoft.Management/managementGroups
 * @member {uuid} [name] The name of the management group. E.g.
 * 20000000-0000-0000-0000-000000000000
 * @member {uuid} [tenantId] The AAD Tenant ID associated with the management
 * group. E.g. 10000000-0000-0000-0000-000000000000
 * @member {string} [displayName] The friendly name of the management group.
 * @member {object} [details] Details.
 * @member {number} [details.version] The version number of the object.
 * @member {date} [details.updatedTime] The date and time when this object was
 * last updated.
 * @member {string} [details.updatedBy] The identity of the principal or
 * process that updated the object.
 * @member {object} [details.parent]
 * @member {string} [details.parent.parentId] The ID of the parent management
 * group. E.g.
 * /providers/Microsoft.Management/managementGroups/30000000-0000-0000-0000-000000000000
 * @member {string} [details.parent.displayName] The friendly name of the
 * management group.
 * @member {string} [details.managementGroupType] Possible values include:
 * 'Enrollment', 'Department', 'Account', 'Subscription'
 * @member {array} [children] The list of children.
 */
export interface ManagementGroupWithHierarchy {
  readonly id?: string;
  readonly type?: string;
  readonly name?: string;
  tenantId?: string;
  displayName?: string;
  details?: ManagementGroupDetailsProperties;
  children?: ManagementGroupRecursiveChildInfo[];
}


/**
 * @class
 * Initializes a new instance of the ManagementGroupListResult class.
 * @constructor
 * The result of listing management groups.
 *
 * @member {string} [nextLink] The URL to the next page of results.
 */
export interface ManagementGroupListResult extends Array<ManagementGroupInfo> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result listing  operations. It contains a list of operations and a URL link
 * to get the next set of results.
 *
 * @member {string} [nextLink] URL to get the next set of operation list
 * results if there are any.
 */
export interface OperationListResult extends Array<Operation> {
  readonly nextLink?: string;
}
