/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the DeploymentExtendedFilter class.
 * @constructor
 * Deployment filter.
 *
 * @member {string} [provisioningState] The provisioning state.
 */
export interface DeploymentExtendedFilter {
  provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the GenericResourceFilter class.
 * @constructor
 * Resource filter.
 *
 * @member {string} [resourceType] The resource type.
 * @member {string} [tagname] The tag name.
 * @member {string} [tagvalue] The tag value.
 */
export interface GenericResourceFilter {
  resourceType?: string;
  tagname?: string;
  tagvalue?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceGroupFilter class.
 * @constructor
 * Resource group filter.
 *
 * @member {string} [tagName] The tag name.
 * @member {string} [tagValue] The tag value.
 */
export interface ResourceGroupFilter {
  tagName?: string;
  tagValue?: string;
}

/**
 * @class
 * Initializes a new instance of the TemplateLink class.
 * @constructor
 * Entity representing the reference to the template.
 *
 * @member {string} uri The URI of the template to deploy.
 * @member {string} [contentVersion] If included, must match the ContentVersion
 * in the template.
 */
export interface TemplateLink {
  uri: string;
  contentVersion?: string;
}

/**
 * @class
 * Initializes a new instance of the ParametersLink class.
 * @constructor
 * Entity representing the reference to the deployment paramaters.
 *
 * @member {string} uri The URI of the parameters file.
 * @member {string} [contentVersion] If included, must match the ContentVersion
 * in the template.
 */
export interface ParametersLink {
  uri: string;
  contentVersion?: string;
}

/**
 * @class
 * Initializes a new instance of the DebugSetting class.
 * @constructor
 * @member {string} [detailLevel] Specifies the type of information to log for
 * debugging. The permitted values are none, requestContent, responseContent,
 * or both requestContent and responseContent separated by a comma. The default
 * is none. When setting this value, carefully consider the type of information
 * you are passing in during deployment. By logging information about the
 * request or response, you could potentially expose sensitive data that is
 * retrieved through the deployment operations.
 */
export interface DebugSetting {
  detailLevel?: string;
}

/**
 * @class
 * Initializes a new instance of the DeploymentProperties class.
 * @constructor
 * Deployment properties.
 *
 * @member {object} [template] The template content. You use this element when
 * you want to pass the template syntax directly in the request rather than
 * link to an existing template. It can be a JObject or well-formed JSON
 * string. Use either the templateLink property or the template property, but
 * not both.
 * @member {object} [templateLink] The URI of the template. Use either the
 * templateLink property or the template property, but not both.
 * @member {string} [templateLink.uri] The URI of the template to deploy.
 * @member {string} [templateLink.contentVersion] If included, must match the
 * ContentVersion in the template.
 * @member {object} [parameters] Name and value pairs that define the
 * deployment parameters for the template. You use this element when you want
 * to provide the parameter values directly in the request rather than link to
 * an existing parameter file. Use either the parametersLink property or the
 * parameters property, but not both. It can be a JObject or a well formed JSON
 * string.
 * @member {object} [parametersLink] The URI of parameters file. You use this
 * element to link to an existing parameters file. Use either the
 * parametersLink property or the parameters property, but not both.
 * @member {string} [parametersLink.uri] The URI of the parameters file.
 * @member {string} [parametersLink.contentVersion] If included, must match the
 * ContentVersion in the template.
 * @member {string} mode The mode that is used to deploy resources. This value
 * can be either Incremental or Complete. In Incremental mode, resources are
 * deployed without deleting existing resources that are not included in the
 * template. In Complete mode, resources are deployed and existing resources in
 * the resource group that are not included in the template are deleted. Be
 * careful when using Complete mode as you may unintentionally delete
 * resources. Possible values include: 'Incremental', 'Complete'
 * @member {object} [debugSetting] The debug setting of the deployment.
 * @member {string} [debugSetting.detailLevel] Specifies the type of
 * information to log for debugging. The permitted values are none,
 * requestContent, responseContent, or both requestContent and responseContent
 * separated by a comma. The default is none. When setting this value,
 * carefully consider the type of information you are passing in during
 * deployment. By logging information about the request or response, you could
 * potentially expose sensitive data that is retrieved through the deployment
 * operations.
 */
export interface DeploymentProperties {
  template?: any;
  templateLink?: TemplateLink;
  parameters?: any;
  parametersLink?: ParametersLink;
  mode: string;
  debugSetting?: DebugSetting;
}

/**
 * @class
 * Initializes a new instance of the Deployment class.
 * @constructor
 * Deployment operation parameters.
 *
 * @member {object} properties The deployment properties.
 * @member {object} [properties.template] The template content. You use this
 * element when you want to pass the template syntax directly in the request
 * rather than link to an existing template. It can be a JObject or well-formed
 * JSON string. Use either the templateLink property or the template property,
 * but not both.
 * @member {object} [properties.templateLink] The URI of the template. Use
 * either the templateLink property or the template property, but not both.
 * @member {string} [properties.templateLink.uri] The URI of the template to
 * deploy.
 * @member {string} [properties.templateLink.contentVersion] If included, must
 * match the ContentVersion in the template.
 * @member {object} [properties.parameters] Name and value pairs that define
 * the deployment parameters for the template. You use this element when you
 * want to provide the parameter values directly in the request rather than
 * link to an existing parameter file. Use either the parametersLink property
 * or the parameters property, but not both. It can be a JObject or a well
 * formed JSON string.
 * @member {object} [properties.parametersLink] The URI of parameters file. You
 * use this element to link to an existing parameters file. Use either the
 * parametersLink property or the parameters property, but not both.
 * @member {string} [properties.parametersLink.uri] The URI of the parameters
 * file.
 * @member {string} [properties.parametersLink.contentVersion] If included,
 * must match the ContentVersion in the template.
 * @member {string} [properties.mode] The mode that is used to deploy
 * resources. This value can be either Incremental or Complete. In Incremental
 * mode, resources are deployed without deleting existing resources that are
 * not included in the template. In Complete mode, resources are deployed and
 * existing resources in the resource group that are not included in the
 * template are deleted. Be careful when using Complete mode as you may
 * unintentionally delete resources. Possible values include: 'Incremental',
 * 'Complete'
 * @member {object} [properties.debugSetting] The debug setting of the
 * deployment.
 * @member {string} [properties.debugSetting.detailLevel] Specifies the type of
 * information to log for debugging. The permitted values are none,
 * requestContent, responseContent, or both requestContent and responseContent
 * separated by a comma. The default is none. When setting this value,
 * carefully consider the type of information you are passing in during
 * deployment. By logging information about the request or response, you could
 * potentially expose sensitive data that is retrieved through the deployment
 * operations.
 */
export interface Deployment {
  properties: DeploymentProperties;
}

/**
 * @class
 * Initializes a new instance of the DeploymentExportResult class.
 * @constructor
 * The deployment export result.
 *
 * @member {object} [template] The template content.
 */
export interface DeploymentExportResult {
  template?: any;
}

/**
 * @class
 * Initializes a new instance of the ResourceManagementErrorWithDetails class.
 * @constructor
 * The detailed error message of resource management.
 *
 * @member {string} [code] The error code returned when exporting the template.
 * @member {string} [message] The error message describing the export error.
 * @member {string} [target] The target of the error.
 * @member {array} [details] Validation error.
 */
export interface ResourceManagementErrorWithDetails {
  readonly code?: string;
  readonly message?: string;
  readonly target?: string;
  readonly details?: ResourceManagementErrorWithDetails[];
}

/**
 * @class
 * Initializes a new instance of the AliasPathType class.
 * @constructor
 * The type of the paths for alias.
 *
 * @member {string} [path] The path of an alias.
 * @member {array} [apiVersions] The API versions.
 */
export interface AliasPathType {
  path?: string;
  apiVersions?: string[];
}

/**
 * @class
 * Initializes a new instance of the AliasType class.
 * @constructor
 * The alias type.
 *
 * @member {string} [name] The alias name.
 * @member {array} [paths] The paths for an alias.
 */
export interface AliasType {
  name?: string;
  paths?: AliasPathType[];
}

/**
 * @class
 * Initializes a new instance of the ProviderResourceType class.
 * @constructor
 * Resource type managed by the resource provider.
 *
 * @member {string} [resourceType] The resource type.
 * @member {array} [locations] The collection of locations where this resource
 * type can be created.
 * @member {array} [aliases] The aliases that are supported by this resource
 * type.
 * @member {array} [apiVersions] The API version.
 * @member {object} [properties] The properties.
 */
export interface ProviderResourceType {
  resourceType?: string;
  locations?: string[];
  aliases?: AliasType[];
  apiVersions?: string[];
  properties?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the Provider class.
 * @constructor
 * Resource provider information.
 *
 * @member {string} [id] The provider ID.
 * @member {string} [namespace] The namespace of the resource provider.
 * @member {string} [registrationState] The registration state of the provider.
 * @member {array} [resourceTypes] The collection of provider resource types.
 */
export interface Provider {
  readonly id?: string;
  namespace?: string;
  readonly registrationState?: string;
  readonly resourceTypes?: ProviderResourceType[];
}

/**
 * @class
 * Initializes a new instance of the BasicDependency class.
 * @constructor
 * Deployment dependency information.
 *
 * @member {string} [id] The ID of the dependency.
 * @member {string} [resourceType] The dependency resource type.
 * @member {string} [resourceName] The dependency resource name.
 */
export interface BasicDependency {
  id?: string;
  resourceType?: string;
  resourceName?: string;
}

/**
 * @class
 * Initializes a new instance of the Dependency class.
 * @constructor
 * Deployment dependency information.
 *
 * @member {array} [dependsOn] The list of dependencies.
 * @member {string} [id] The ID of the dependency.
 * @member {string} [resourceType] The dependency resource type.
 * @member {string} [resourceName] The dependency resource name.
 */
export interface Dependency {
  dependsOn?: BasicDependency[];
  id?: string;
  resourceType?: string;
  resourceName?: string;
}

/**
 * @class
 * Initializes a new instance of the DeploymentPropertiesExtended class.
 * @constructor
 * Deployment properties with additional details.
 *
 * @member {string} [provisioningState] The state of the provisioning.
 * @member {string} [correlationId] The correlation ID of the deployment.
 * @member {date} [timestamp] The timestamp of the template deployment.
 * @member {object} [outputs] Key/value pairs that represent deploymentoutput.
 * @member {array} [providers] The list of resource providers needed for the
 * deployment.
 * @member {array} [dependencies] The list of deployment dependencies.
 * @member {object} [template] The template content. Use only one of Template
 * or TemplateLink.
 * @member {object} [templateLink] The URI referencing the template. Use only
 * one of Template or TemplateLink.
 * @member {string} [templateLink.uri] The URI of the template to deploy.
 * @member {string} [templateLink.contentVersion] If included, must match the
 * ContentVersion in the template.
 * @member {object} [parameters] Deployment parameters. Use only one of
 * Parameters or ParametersLink.
 * @member {object} [parametersLink] The URI referencing the parameters. Use
 * only one of Parameters or ParametersLink.
 * @member {string} [parametersLink.uri] The URI of the parameters file.
 * @member {string} [parametersLink.contentVersion] If included, must match the
 * ContentVersion in the template.
 * @member {string} [mode] The deployment mode. Possible values are Incremental
 * and Complete. Possible values include: 'Incremental', 'Complete'
 * @member {object} [debugSetting] The debug setting of the deployment.
 * @member {string} [debugSetting.detailLevel] Specifies the type of
 * information to log for debugging. The permitted values are none,
 * requestContent, responseContent, or both requestContent and responseContent
 * separated by a comma. The default is none. When setting this value,
 * carefully consider the type of information you are passing in during
 * deployment. By logging information about the request or response, you could
 * potentially expose sensitive data that is retrieved through the deployment
 * operations.
 */
export interface DeploymentPropertiesExtended {
  readonly provisioningState?: string;
  readonly correlationId?: string;
  readonly timestamp?: Date;
  outputs?: any;
  providers?: Provider[];
  dependencies?: Dependency[];
  template?: any;
  templateLink?: TemplateLink;
  parameters?: any;
  parametersLink?: ParametersLink;
  mode?: string;
  debugSetting?: DebugSetting;
}

/**
 * @class
 * Initializes a new instance of the DeploymentValidateResult class.
 * @constructor
 * Information from validate template deployment response.
 *
 * @member {object} [error] Validation error.
 * @member {string} [error.code] The error code returned when exporting the
 * template.
 * @member {string} [error.message] The error message describing the export
 * error.
 * @member {string} [error.target] The target of the error.
 * @member {array} [error.details] Validation error.
 * @member {object} [properties] The template deployment properties.
 * @member {string} [properties.provisioningState] The state of the
 * provisioning.
 * @member {string} [properties.correlationId] The correlation ID of the
 * deployment.
 * @member {date} [properties.timestamp] The timestamp of the template
 * deployment.
 * @member {object} [properties.outputs] Key/value pairs that represent
 * deploymentoutput.
 * @member {array} [properties.providers] The list of resource providers needed
 * for the deployment.
 * @member {array} [properties.dependencies] The list of deployment
 * dependencies.
 * @member {object} [properties.template] The template content. Use only one of
 * Template or TemplateLink.
 * @member {object} [properties.templateLink] The URI referencing the template.
 * Use only one of Template or TemplateLink.
 * @member {string} [properties.templateLink.uri] The URI of the template to
 * deploy.
 * @member {string} [properties.templateLink.contentVersion] If included, must
 * match the ContentVersion in the template.
 * @member {object} [properties.parameters] Deployment parameters. Use only one
 * of Parameters or ParametersLink.
 * @member {object} [properties.parametersLink] The URI referencing the
 * parameters. Use only one of Parameters or ParametersLink.
 * @member {string} [properties.parametersLink.uri] The URI of the parameters
 * file.
 * @member {string} [properties.parametersLink.contentVersion] If included,
 * must match the ContentVersion in the template.
 * @member {string} [properties.mode] The deployment mode. Possible values are
 * Incremental and Complete. Possible values include: 'Incremental', 'Complete'
 * @member {object} [properties.debugSetting] The debug setting of the
 * deployment.
 * @member {string} [properties.debugSetting.detailLevel] Specifies the type of
 * information to log for debugging. The permitted values are none,
 * requestContent, responseContent, or both requestContent and responseContent
 * separated by a comma. The default is none. When setting this value,
 * carefully consider the type of information you are passing in during
 * deployment. By logging information about the request or response, you could
 * potentially expose sensitive data that is retrieved through the deployment
 * operations.
 */
export interface DeploymentValidateResult {
  error?: ResourceManagementErrorWithDetails;
  properties?: DeploymentPropertiesExtended;
}

/**
 * @class
 * Initializes a new instance of the DeploymentExtended class.
 * @constructor
 * Deployment information.
 *
 * @member {string} [id] The ID of the deployment.
 * @member {string} name The name of the deployment.
 * @member {object} [properties] Deployment properties.
 * @member {string} [properties.provisioningState] The state of the
 * provisioning.
 * @member {string} [properties.correlationId] The correlation ID of the
 * deployment.
 * @member {date} [properties.timestamp] The timestamp of the template
 * deployment.
 * @member {object} [properties.outputs] Key/value pairs that represent
 * deploymentoutput.
 * @member {array} [properties.providers] The list of resource providers needed
 * for the deployment.
 * @member {array} [properties.dependencies] The list of deployment
 * dependencies.
 * @member {object} [properties.template] The template content. Use only one of
 * Template or TemplateLink.
 * @member {object} [properties.templateLink] The URI referencing the template.
 * Use only one of Template or TemplateLink.
 * @member {string} [properties.templateLink.uri] The URI of the template to
 * deploy.
 * @member {string} [properties.templateLink.contentVersion] If included, must
 * match the ContentVersion in the template.
 * @member {object} [properties.parameters] Deployment parameters. Use only one
 * of Parameters or ParametersLink.
 * @member {object} [properties.parametersLink] The URI referencing the
 * parameters. Use only one of Parameters or ParametersLink.
 * @member {string} [properties.parametersLink.uri] The URI of the parameters
 * file.
 * @member {string} [properties.parametersLink.contentVersion] If included,
 * must match the ContentVersion in the template.
 * @member {string} [properties.mode] The deployment mode. Possible values are
 * Incremental and Complete. Possible values include: 'Incremental', 'Complete'
 * @member {object} [properties.debugSetting] The debug setting of the
 * deployment.
 * @member {string} [properties.debugSetting.detailLevel] Specifies the type of
 * information to log for debugging. The permitted values are none,
 * requestContent, responseContent, or both requestContent and responseContent
 * separated by a comma. The default is none. When setting this value,
 * carefully consider the type of information you are passing in during
 * deployment. By logging information about the request or response, you could
 * potentially expose sensitive data that is retrieved through the deployment
 * operations.
 */
export interface DeploymentExtended {
  id?: string;
  name: string;
  properties?: DeploymentPropertiesExtended;
}

/**
 * @class
 * Initializes a new instance of the Plan class.
 * @constructor
 * Plan for the resource.
 *
 * @member {string} [name] The plan ID.
 * @member {string} [publisher] The publisher ID.
 * @member {string} [product] The offer ID.
 * @member {string} [promotionCode] The promotion code.
 */
export interface Plan {
  name?: string;
  publisher?: string;
  product?: string;
  promotionCode?: string;
}

/**
 * @class
 * Initializes a new instance of the Sku class.
 * @constructor
 * SKU for the resource.
 *
 * @member {string} [name] The SKU name.
 * @member {string} [tier] The SKU tier.
 * @member {string} [size] The SKU size.
 * @member {string} [family] The SKU family.
 * @member {string} [model] The SKU model.
 * @member {number} [capacity] The SKU capacity.
 */
export interface Sku {
  name?: string;
  tier?: string;
  size?: string;
  family?: string;
  model?: string;
  capacity?: number;
}

/**
 * @class
 * Initializes a new instance of the Identity class.
 * @constructor
 * Identity for the resource.
 *
 * @member {string} [principalId] The principal ID of resource identity.
 * @member {string} [tenantId] The tenant ID of resource.
 * @member {string} [type] The identity type. Possible values include:
 * 'SystemAssigned'
 */
export interface Identity {
  readonly principalId?: string;
  readonly tenantId?: string;
  type?: string;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * Resource.
 *
 * @member {string} [id] Resource ID
 * @member {string} [name] Resource name
 * @member {string} [type] Resource type
 * @member {string} [location] Resource location
 * @member {object} [tags] Resource tags
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
  location?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the GenericResource class.
 * @constructor
 * Resource information.
 *
 * @member {object} [plan] The plan of the resource.
 * @member {string} [plan.name] The plan ID.
 * @member {string} [plan.publisher] The publisher ID.
 * @member {string} [plan.product] The offer ID.
 * @member {string} [plan.promotionCode] The promotion code.
 * @member {object} [properties] The resource properties.
 * @member {string} [kind] The kind of the resource.
 * @member {string} [managedBy] ID of the resource that manages this resource.
 * @member {object} [sku] The SKU of the resource.
 * @member {string} [sku.name] The SKU name.
 * @member {string} [sku.tier] The SKU tier.
 * @member {string} [sku.size] The SKU size.
 * @member {string} [sku.family] The SKU family.
 * @member {string} [sku.model] The SKU model.
 * @member {number} [sku.capacity] The SKU capacity.
 * @member {object} [identity] The identity of the resource.
 * @member {string} [identity.principalId] The principal ID of resource
 * identity.
 * @member {string} [identity.tenantId] The tenant ID of resource.
 * @member {string} [identity.type] The identity type. Possible values include:
 * 'SystemAssigned'
 */
export interface GenericResource extends Resource {
  plan?: Plan;
  properties?: any;
  kind?: string;
  managedBy?: string;
  sku?: Sku;
  identity?: Identity;
}

/**
 * @class
 * Initializes a new instance of the ResourceGroupProperties class.
 * @constructor
 * The resource group properties.
 *
 * @member {string} [provisioningState] The provisioning state.
 */
export interface ResourceGroupProperties {
  readonly provisioningState?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceGroup class.
 * @constructor
 * Resource group information.
 *
 * @member {string} [id] The ID of the resource group.
 * @member {string} [name] The name of the resource group.
 * @member {object} [properties]
 * @member {string} [properties.provisioningState] The provisioning state.
 * @member {string} location The location of the resource group. It cannot be
 * changed after the resource group has been created. It muct be one of the
 * supported Azure locations.
 * @member {string} [managedBy] The ID of the resource that manages this
 * resource group.
 * @member {object} [tags] The tags attached to the resource group.
 */
export interface ResourceGroup {
  readonly id?: string;
  name?: string;
  properties?: ResourceGroupProperties;
  location: string;
  managedBy?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ResourceGroupPatchable class.
 * @constructor
 * Resource group information.
 *
 * @member {string} [name] The name of the resource group.
 * @member {object} [properties]
 * @member {string} [properties.provisioningState] The provisioning state.
 * @member {string} [managedBy] The ID of the resource that manages this
 * resource group.
 * @member {object} [tags] The tags attached to the resource group.
 */
export interface ResourceGroupPatchable {
  name?: string;
  properties?: ResourceGroupProperties;
  managedBy?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ResourcesMoveInfo class.
 * @constructor
 * Parameters of move resources.
 *
 * @member {array} [resources] The IDs of the resources.
 * @member {string} [targetResourceGroup] The target resource group.
 */
export interface ResourcesMoveInfo {
  resources?: string[];
  targetResourceGroup?: string;
}

/**
 * @class
 * Initializes a new instance of the ExportTemplateRequest class.
 * @constructor
 * Export resource group template request parameters.
 *
 * @member {array} [resources] The IDs of the resources. The only supported
 * string currently is '*' (all resources). Future updates will support
 * exporting specific resources.
 * @member {string} [options] The export template options. Supported values
 * include 'IncludeParameterDefaultValue', 'IncludeComments' or
 * 'IncludeParameterDefaultValue, IncludeComments
 */
export interface ExportTemplateRequest {
  resources?: string[];
  options?: string;
}

/**
 * @class
 * Initializes a new instance of the TagCount class.
 * @constructor
 * Tag count.
 *
 * @member {string} [type] Type of count.
 * @member {number} [value] Value of count.
 */
export interface TagCount {
  type?: string;
  value?: number;
}

/**
 * @class
 * Initializes a new instance of the TagValue class.
 * @constructor
 * Tag information.
 *
 * @member {string} [id] The tag ID.
 * @member {string} [tagValue] The tag value.
 * @member {object} [count] The tag value count.
 * @member {string} [count.type] Type of count.
 * @member {number} [count.value] Value of count.
 */
export interface TagValue {
  id?: string;
  tagValue?: string;
  count?: TagCount;
}

/**
 * @class
 * Initializes a new instance of the TagDetails class.
 * @constructor
 * Tag details.
 *
 * @member {string} [id] The tag ID.
 * @member {string} [tagName] The tag name.
 * @member {object} [count] The total number of resources that use the resource
 * tag. When a tag is initially created and has no associated resources, the
 * value is 0.
 * @member {string} [count.type] Type of count.
 * @member {number} [count.value] Value of count.
 * @member {array} [values] The list of tag values.
 */
export interface TagDetails {
  id?: string;
  tagName?: string;
  count?: TagCount;
  values?: TagValue[];
}

/**
 * @class
 * Initializes a new instance of the TargetResource class.
 * @constructor
 * Target resource.
 *
 * @member {string} [id] The ID of the resource.
 * @member {string} [resourceName] The name of the resource.
 * @member {string} [resourceType] The type of the resource.
 */
export interface TargetResource {
  id?: string;
  resourceName?: string;
  resourceType?: string;
}

/**
 * @class
 * Initializes a new instance of the HttpMessage class.
 * @constructor
 * HTTP message.
 *
 * @member {object} [content] HTTP message content.
 */
export interface HttpMessage {
  content?: any;
}

/**
 * @class
 * Initializes a new instance of the DeploymentOperationProperties class.
 * @constructor
 * Deployment operation properties.
 *
 * @member {string} [provisioningState] The state of the provisioning.
 * @member {date} [timestamp] The date and time of the operation.
 * @member {string} [serviceRequestId] Deployment operation service request id.
 * @member {string} [statusCode] Operation status code.
 * @member {object} [statusMessage] Operation status message.
 * @member {object} [targetResource] The target resource.
 * @member {string} [targetResource.id] The ID of the resource.
 * @member {string} [targetResource.resourceName] The name of the resource.
 * @member {string} [targetResource.resourceType] The type of the resource.
 * @member {object} [request] The HTTP request message.
 * @member {object} [request.content] HTTP message content.
 * @member {object} [response] The HTTP response message.
 * @member {object} [response.content] HTTP message content.
 */
export interface DeploymentOperationProperties {
  readonly provisioningState?: string;
  readonly timestamp?: Date;
  readonly serviceRequestId?: string;
  readonly statusCode?: string;
  readonly statusMessage?: any;
  readonly targetResource?: TargetResource;
  readonly request?: HttpMessage;
  readonly response?: HttpMessage;
}

/**
 * @class
 * Initializes a new instance of the DeploymentOperation class.
 * @constructor
 * Deployment operation information.
 *
 * @member {string} [id] Full deployment operation ID.
 * @member {string} [operationId] Deployment operation ID.
 * @member {object} [properties] Deployment properties.
 * @member {string} [properties.provisioningState] The state of the
 * provisioning.
 * @member {date} [properties.timestamp] The date and time of the operation.
 * @member {string} [properties.serviceRequestId] Deployment operation service
 * request id.
 * @member {string} [properties.statusCode] Operation status code.
 * @member {object} [properties.statusMessage] Operation status message.
 * @member {object} [properties.targetResource] The target resource.
 * @member {string} [properties.targetResource.id] The ID of the resource.
 * @member {string} [properties.targetResource.resourceName] The name of the
 * resource.
 * @member {string} [properties.targetResource.resourceType] The type of the
 * resource.
 * @member {object} [properties.request] The HTTP request message.
 * @member {object} [properties.request.content] HTTP message content.
 * @member {object} [properties.response] The HTTP response message.
 * @member {object} [properties.response.content] HTTP message content.
 */
export interface DeploymentOperation {
  readonly id?: string;
  readonly operationId?: string;
  properties?: DeploymentOperationProperties;
}

/**
 * @class
 * Initializes a new instance of the ResourceProviderOperationDisplayProperties class.
 * @constructor
 * Resource provider operation's display properties.
 *
 * @member {string} [publisher] Operation description.
 * @member {string} [provider] Operation provider.
 * @member {string} [resource] Operation resource.
 * @member {string} [operation] Operation.
 * @member {string} [description] Operation description.
 */
export interface ResourceProviderOperationDisplayProperties {
  publisher?: string;
  provider?: string;
  resource?: string;
  operation?: string;
  description?: string;
}

/**
 * @class
 * Initializes a new instance of the SubResource class.
 * @constructor
 * Sub-resource.
 *
 * @member {string} [id] Resource ID
 */
export interface SubResource extends BaseResource {
  id?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceGroupExportResult class.
 * @constructor
 * Resource group export result.
 *
 * @member {object} [template] The template content.
 * @member {object} [error] The error.
 * @member {string} [error.code] The error code returned when exporting the
 * template.
 * @member {string} [error.message] The error message describing the export
 * error.
 * @member {string} [error.target] The target of the error.
 * @member {array} [error.details] Validation error.
 */
export interface ResourceGroupExportResult {
  template?: any;
  error?: ResourceManagementErrorWithDetails;
}


/**
 * @class
 * Initializes a new instance of the DeploymentListResult class.
 * @constructor
 * List of deployments.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface DeploymentListResult extends Array<DeploymentExtended> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ProviderListResult class.
 * @constructor
 * List of resource providers.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface ProviderListResult extends Array<Provider> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceListResult class.
 * @constructor
 * List of resource groups.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface ResourceListResult extends Array<GenericResource> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ResourceGroupListResult class.
 * @constructor
 * List of resource groups.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface ResourceGroupListResult extends Array<ResourceGroup> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the TagsListResult class.
 * @constructor
 * List of subscription tags.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface TagsListResult extends Array<TagDetails> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DeploymentOperationsListResult class.
 * @constructor
 * List of deployment operations.
 *
 * @member {string} [nextLink] The URL to use for getting the next set of
 * results.
 */
export interface DeploymentOperationsListResult extends Array<DeploymentOperation> {
  readonly nextLink?: string;
}
