/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;


/**
 * Creates or updates a Search service in the given resource group. If the
 * Search service already exists, all properties will be updated with the given
 * values.
 *
 * @param {string} resourceGroupName The name of the resource group within the
 * current subscription. You can obtain this value from the Azure Resource
 * Manager API or the portal.
 *
 * @param {string} searchServiceName The name of the Azure Search service to
 * create or update. Search service names must only contain lowercase letters,
 * digits or dashes, cannot use dash as the first two or last one characters,
 * cannot contain consecutive dashes, and must be between 2 and 60 characters
 * in length. Search service names must be globally unique since they are part
 * of the service URI (https://<name>.search.windows.net). You cannot change
 * the service name after the service is created.
 *
 * @param {object} service The definition of the Search service to create or
 * update.
 *
 * @param {number} [service.replicaCount] The number of replicas in the Search
 * service. If specified, it must be a value between 1 and 12 inclusive for
 * standard SKUs or between 1 and 3 inclusive for basic SKU.
 *
 * @param {number} [service.partitionCount] The number of partitions in the
 * Search service; if specified, it can be 1, 2, 3, 4, 6, or 12. Values greater
 * than 1 are only valid for standard SKUs. For 'standard3' services with
 * hostingMode set to 'highDensity', the allowed values are between 1 and 3.
 *
 * @param {string} [service.hostingMode] Applicable only for the standard3 SKU.
 * You can set this property to enable up to 3 high density partitions that
 * allow up to 1000 indexes, which is much higher than the maximum indexes
 * allowed for any other SKU. For the standard3 SKU, the value is either
 * 'default' or 'highDensity'. For all other SKUs, this value must be
 * 'default'. Possible values include: 'default', 'highDensity'
 *
 * @param {object} service.sku The SKU of the Search Service, which determines
 * price tier and capacity limits.
 *
 * @param {string} [service.sku.name] The SKU of the Search service. Valid
 * values include: 'free': Shared service. 'basic': Dedicated service with up
 * to 3 replicas. 'standard': Dedicated service with up to 12 partitions and 12
 * replicas. 'standard2': Similar to standard, but with more capacity per
 * search unit. 'standard3': Offers maximum capacity per search unit with up to
 * 12 partitions and 12 replicas (or up to 3 partitions with more indexes if
 * you also set the hostingMode property to 'highDensity'). Possible values
 * include: 'free', 'basic', 'standard', 'standard2', 'standard3'
 *
 * @param {string} service.location The geographic location of the resource.
 * This must be one of the supported and registered Azure Geo Regions (for
 * example, West US, East US, Southeast Asia, and so forth).
 *
 * @param {object} [service.tags] Tags to help categorize the resource in the
 * Azure portal.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchManagementRequestOptions] Additional
 * parameters for the operation
 *
 * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
 * client-generated GUID value that identifies this request. If specified, this
 * will be included in response information as a way to track the request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SearchService} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _createOrUpdate(resourceGroupName, searchServiceName, service, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginCreateOrUpdate(resourceGroupName, searchServiceName, service, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SearchService']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Gets the Search service with the given name in the given resource group.
 *
 * @param {string} resourceGroupName The name of the resource group within the
 * current subscription. You can obtain this value from the Azure Resource
 * Manager API or the portal.
 *
 * @param {string} searchServiceName The name of the Azure Search service
 * associated with the specified resource group.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchManagementRequestOptions] Additional
 * parameters for the operation
 *
 * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
 * client-generated GUID value that identifies this request. If specified, this
 * will be included in response information as a way to track the request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SearchService} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _get(resourceGroupName, searchServiceName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchManagementRequestOptions = (options && options.searchManagementRequestOptions !== undefined) ? options.searchManagementRequestOptions : undefined;
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (searchServiceName === null || searchServiceName === undefined || typeof searchServiceName.valueOf() !== 'string') {
      throw new Error('searchServiceName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchManagementRequestOptions !== null && searchManagementRequestOptions !== undefined)
    {
      clientRequestId = searchManagementRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Search/searchServices/{searchServiceName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{searchServiceName}', encodeURIComponent(searchServiceName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['x-ms-client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SearchService']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deletes a Search service in the given resource group, along with its
 * associated resources.
 *
 * @param {string} resourceGroupName The name of the resource group within the
 * current subscription. You can obtain this value from the Azure Resource
 * Manager API or the portal.
 *
 * @param {string} searchServiceName The name of the Azure Search service
 * associated with the specified resource group.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchManagementRequestOptions] Additional
 * parameters for the operation
 *
 * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
 * client-generated GUID value that identifies this request. If specified, this
 * will be included in response information as a way to track the request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deleteMethod(resourceGroupName, searchServiceName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchManagementRequestOptions = (options && options.searchManagementRequestOptions !== undefined) ? options.searchManagementRequestOptions : undefined;
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (searchServiceName === null || searchServiceName === undefined || typeof searchServiceName.valueOf() !== 'string') {
      throw new Error('searchServiceName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchManagementRequestOptions !== null && searchManagementRequestOptions !== undefined)
    {
      clientRequestId = searchManagementRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Search/searchServices/{searchServiceName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{searchServiceName}', encodeURIComponent(searchServiceName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'DELETE';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['x-ms-client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 204 && statusCode !== 404) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of all Search services in the given resource group.
 *
 * @param {string} resourceGroupName The name of the resource group within the
 * current subscription. You can obtain this value from the Azure Resource
 * Manager API or the portal.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchManagementRequestOptions] Additional
 * parameters for the operation
 *
 * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
 * client-generated GUID value that identifies this request. If specified, this
 * will be included in response information as a way to track the request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SearchServiceListResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listByResourceGroup(resourceGroupName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchManagementRequestOptions = (options && options.searchManagementRequestOptions !== undefined) ? options.searchManagementRequestOptions : undefined;
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchManagementRequestOptions !== null && searchManagementRequestOptions !== undefined)
    {
      clientRequestId = searchManagementRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Search/searchServices';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['x-ms-client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SearchServiceListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Checks whether or not the given Search service name is available for use.
 * Search service names must be globally unique since they are part of the
 * service URI (https://<name>.search.windows.net).
 *
 * @param {string} name The Search service name to validate. Search service
 * names must only contain lowercase letters, digits or dashes, cannot use dash
 * as the first two or last one characters, cannot contain consecutive dashes,
 * and must be between 2 and 60 characters in length.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchManagementRequestOptions] Additional
 * parameters for the operation
 *
 * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
 * client-generated GUID value that identifies this request. If specified, this
 * will be included in response information as a way to track the request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link CheckNameAvailabilityOutput} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _checkNameAvailability(name, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchManagementRequestOptions = (options && options.searchManagementRequestOptions !== undefined) ? options.searchManagementRequestOptions : undefined;
  // Validate
  try {
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (name === null || name === undefined || typeof name.valueOf() !== 'string') {
      throw new Error('name cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  let checkNameAvailabilityInput = new client.models['CheckNameAvailabilityInput']();
  try {
    if (searchManagementRequestOptions !== null && searchManagementRequestOptions !== undefined)
    {
      clientRequestId = searchManagementRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
    if (name !== null && name !== undefined)
    {
      checkNameAvailabilityInput.name = name;
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/providers/Microsoft.Search/checkNameAvailability';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['x-ms-client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (checkNameAvailabilityInput !== null && checkNameAvailabilityInput !== undefined) {
      let requestModelMapper = new client.models['CheckNameAvailabilityInput']().mapper();
      requestModel = client.serialize(requestModelMapper, checkNameAvailabilityInput, 'checkNameAvailabilityInput');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(checkNameAvailabilityInput, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['CheckNameAvailabilityOutput']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Creates or updates a Search service in the given resource group. If the
 * Search service already exists, all properties will be updated with the given
 * values.
 *
 * @param {string} resourceGroupName The name of the resource group within the
 * current subscription. You can obtain this value from the Azure Resource
 * Manager API or the portal.
 *
 * @param {string} searchServiceName The name of the Azure Search service to
 * create or update. Search service names must only contain lowercase letters,
 * digits or dashes, cannot use dash as the first two or last one characters,
 * cannot contain consecutive dashes, and must be between 2 and 60 characters
 * in length. Search service names must be globally unique since they are part
 * of the service URI (https://<name>.search.windows.net). You cannot change
 * the service name after the service is created.
 *
 * @param {object} service The definition of the Search service to create or
 * update.
 *
 * @param {number} [service.replicaCount] The number of replicas in the Search
 * service. If specified, it must be a value between 1 and 12 inclusive for
 * standard SKUs or between 1 and 3 inclusive for basic SKU.
 *
 * @param {number} [service.partitionCount] The number of partitions in the
 * Search service; if specified, it can be 1, 2, 3, 4, 6, or 12. Values greater
 * than 1 are only valid for standard SKUs. For 'standard3' services with
 * hostingMode set to 'highDensity', the allowed values are between 1 and 3.
 *
 * @param {string} [service.hostingMode] Applicable only for the standard3 SKU.
 * You can set this property to enable up to 3 high density partitions that
 * allow up to 1000 indexes, which is much higher than the maximum indexes
 * allowed for any other SKU. For the standard3 SKU, the value is either
 * 'default' or 'highDensity'. For all other SKUs, this value must be
 * 'default'. Possible values include: 'default', 'highDensity'
 *
 * @param {object} service.sku The SKU of the Search Service, which determines
 * price tier and capacity limits.
 *
 * @param {string} [service.sku.name] The SKU of the Search service. Valid
 * values include: 'free': Shared service. 'basic': Dedicated service with up
 * to 3 replicas. 'standard': Dedicated service with up to 12 partitions and 12
 * replicas. 'standard2': Similar to standard, but with more capacity per
 * search unit. 'standard3': Offers maximum capacity per search unit with up to
 * 12 partitions and 12 replicas (or up to 3 partitions with more indexes if
 * you also set the hostingMode property to 'highDensity'). Possible values
 * include: 'free', 'basic', 'standard', 'standard2', 'standard3'
 *
 * @param {string} service.location The geographic location of the resource.
 * This must be one of the supported and registered Azure Geo Regions (for
 * example, West US, East US, Southeast Asia, and so forth).
 *
 * @param {object} [service.tags] Tags to help categorize the resource in the
 * Azure portal.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.searchManagementRequestOptions] Additional
 * parameters for the operation
 *
 * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
 * client-generated GUID value that identifies this request. If specified, this
 * will be included in response information as a way to track the request.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link SearchService} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginCreateOrUpdate(resourceGroupName, searchServiceName, service, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let searchManagementRequestOptions = (options && options.searchManagementRequestOptions !== undefined) ? options.searchManagementRequestOptions : undefined;
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (searchServiceName === null || searchServiceName === undefined || typeof searchServiceName.valueOf() !== 'string') {
      throw new Error('searchServiceName cannot be null or undefined and it must be of type string.');
    }
    if (service === null || service === undefined) {
      throw new Error('service cannot be null or undefined.');
    }
    if (this.client.apiVersion === null || this.client.apiVersion === undefined || typeof this.client.apiVersion.valueOf() !== 'string') {
      throw new Error('this.client.apiVersion cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let clientRequestId;
  try {
    if (searchManagementRequestOptions !== null && searchManagementRequestOptions !== undefined)
    {
      clientRequestId = searchManagementRequestOptions.clientRequestId;
      if (clientRequestId !== null && clientRequestId !== undefined && !(typeof clientRequestId.valueOf() === 'string' && msRest.isValidUuid(clientRequestId))) {
        throw new Error('clientRequestId must be of type string and must be a valid uuid.');
      }
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Search/searchServices/{searchServiceName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{searchServiceName}', encodeURIComponent(searchServiceName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(this.client.apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PUT';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if (clientRequestId !== undefined && clientRequestId !== null) {
    httpRequest.headers['x-ms-client-request-id'] = clientRequestId.toString();
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (service !== null && service !== undefined) {
      let requestModelMapper = new client.models['SearchService']().mapper();
      requestModel = client.serialize(requestModelMapper, service, 'service');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(service, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SearchService']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['SearchService']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError1 = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError1.request = msRest.stripRequest(httpRequest);
        deserializationError1.response = msRest.stripResponse(response);
        return callback(deserializationError1);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a Services. */
class Services {
  /**
   * Create a Services.
   * @param {SearchManagementClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._createOrUpdate = _createOrUpdate;
    this._get = _get;
    this._deleteMethod = _deleteMethod;
    this._listByResourceGroup = _listByResourceGroup;
    this._checkNameAvailability = _checkNameAvailability;
    this._beginCreateOrUpdate = _beginCreateOrUpdate;
  }

  /**
   * Creates or updates a Search service in the given resource group. If the
   * Search service already exists, all properties will be updated with the given
   * values.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service to
   * create or update. Search service names must only contain lowercase letters,
   * digits or dashes, cannot use dash as the first two or last one characters,
   * cannot contain consecutive dashes, and must be between 2 and 60 characters
   * in length. Search service names must be globally unique since they are part
   * of the service URI (https://<name>.search.windows.net). You cannot change
   * the service name after the service is created.
   *
   * @param {object} service The definition of the Search service to create or
   * update.
   *
   * @param {number} [service.replicaCount] The number of replicas in the Search
   * service. If specified, it must be a value between 1 and 12 inclusive for
   * standard SKUs or between 1 and 3 inclusive for basic SKU.
   *
   * @param {number} [service.partitionCount] The number of partitions in the
   * Search service; if specified, it can be 1, 2, 3, 4, 6, or 12. Values greater
   * than 1 are only valid for standard SKUs. For 'standard3' services with
   * hostingMode set to 'highDensity', the allowed values are between 1 and 3.
   *
   * @param {string} [service.hostingMode] Applicable only for the standard3 SKU.
   * You can set this property to enable up to 3 high density partitions that
   * allow up to 1000 indexes, which is much higher than the maximum indexes
   * allowed for any other SKU. For the standard3 SKU, the value is either
   * 'default' or 'highDensity'. For all other SKUs, this value must be
   * 'default'. Possible values include: 'default', 'highDensity'
   *
   * @param {object} service.sku The SKU of the Search Service, which determines
   * price tier and capacity limits.
   *
   * @param {string} [service.sku.name] The SKU of the Search service. Valid
   * values include: 'free': Shared service. 'basic': Dedicated service with up
   * to 3 replicas. 'standard': Dedicated service with up to 12 partitions and 12
   * replicas. 'standard2': Similar to standard, but with more capacity per
   * search unit. 'standard3': Offers maximum capacity per search unit with up to
   * 12 partitions and 12 replicas (or up to 3 partitions with more indexes if
   * you also set the hostingMode property to 'highDensity'). Possible values
   * include: 'free', 'basic', 'standard', 'standard2', 'standard3'
   *
   * @param {string} service.location The geographic location of the resource.
   * This must be one of the supported and registered Azure Geo Regions (for
   * example, West US, East US, Southeast Asia, and so forth).
   *
   * @param {object} [service.tags] Tags to help categorize the resource in the
   * Azure portal.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SearchService>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createOrUpdateWithHttpOperationResponse(resourceGroupName, searchServiceName, service, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._createOrUpdate(resourceGroupName, searchServiceName, service, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates or updates a Search service in the given resource group. If the
   * Search service already exists, all properties will be updated with the given
   * values.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service to
   * create or update. Search service names must only contain lowercase letters,
   * digits or dashes, cannot use dash as the first two or last one characters,
   * cannot contain consecutive dashes, and must be between 2 and 60 characters
   * in length. Search service names must be globally unique since they are part
   * of the service URI (https://<name>.search.windows.net). You cannot change
   * the service name after the service is created.
   *
   * @param {object} service The definition of the Search service to create or
   * update.
   *
   * @param {number} [service.replicaCount] The number of replicas in the Search
   * service. If specified, it must be a value between 1 and 12 inclusive for
   * standard SKUs or between 1 and 3 inclusive for basic SKU.
   *
   * @param {number} [service.partitionCount] The number of partitions in the
   * Search service; if specified, it can be 1, 2, 3, 4, 6, or 12. Values greater
   * than 1 are only valid for standard SKUs. For 'standard3' services with
   * hostingMode set to 'highDensity', the allowed values are between 1 and 3.
   *
   * @param {string} [service.hostingMode] Applicable only for the standard3 SKU.
   * You can set this property to enable up to 3 high density partitions that
   * allow up to 1000 indexes, which is much higher than the maximum indexes
   * allowed for any other SKU. For the standard3 SKU, the value is either
   * 'default' or 'highDensity'. For all other SKUs, this value must be
   * 'default'. Possible values include: 'default', 'highDensity'
   *
   * @param {object} service.sku The SKU of the Search Service, which determines
   * price tier and capacity limits.
   *
   * @param {string} [service.sku.name] The SKU of the Search service. Valid
   * values include: 'free': Shared service. 'basic': Dedicated service with up
   * to 3 replicas. 'standard': Dedicated service with up to 12 partitions and 12
   * replicas. 'standard2': Similar to standard, but with more capacity per
   * search unit. 'standard3': Offers maximum capacity per search unit with up to
   * 12 partitions and 12 replicas (or up to 3 partitions with more indexes if
   * you also set the hostingMode property to 'highDensity'). Possible values
   * include: 'free', 'basic', 'standard', 'standard2', 'standard3'
   *
   * @param {string} service.location The geographic location of the resource.
   * This must be one of the supported and registered Azure Geo Regions (for
   * example, West US, East US, Southeast Asia, and so forth).
   *
   * @param {object} [service.tags] Tags to help categorize the resource in the
   * Azure portal.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SearchService} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SearchService} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  createOrUpdate(resourceGroupName, searchServiceName, service, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._createOrUpdate(resourceGroupName, searchServiceName, service, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._createOrUpdate(resourceGroupName, searchServiceName, service, options, optionalCallback);
    }
  }

  /**
   * Gets the Search service with the given name in the given resource group.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service
   * associated with the specified resource group.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SearchService>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getWithHttpOperationResponse(resourceGroupName, searchServiceName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._get(resourceGroupName, searchServiceName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets the Search service with the given name in the given resource group.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service
   * associated with the specified resource group.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SearchService} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SearchService} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  get(resourceGroupName, searchServiceName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._get(resourceGroupName, searchServiceName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._get(resourceGroupName, searchServiceName, options, optionalCallback);
    }
  }

  /**
   * Deletes a Search service in the given resource group, along with its
   * associated resources.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service
   * associated with the specified resource group.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deleteMethodWithHttpOperationResponse(resourceGroupName, searchServiceName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deleteMethod(resourceGroupName, searchServiceName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes a Search service in the given resource group, along with its
   * associated resources.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service
   * associated with the specified resource group.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deleteMethod(resourceGroupName, searchServiceName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deleteMethod(resourceGroupName, searchServiceName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deleteMethod(resourceGroupName, searchServiceName, options, optionalCallback);
    }
  }

  /**
   * Gets a list of all Search services in the given resource group.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SearchServiceListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listByResourceGroupWithHttpOperationResponse(resourceGroupName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listByResourceGroup(resourceGroupName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of all Search services in the given resource group.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SearchServiceListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SearchServiceListResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listByResourceGroup(resourceGroupName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listByResourceGroup(resourceGroupName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listByResourceGroup(resourceGroupName, options, optionalCallback);
    }
  }

  /**
   * Checks whether or not the given Search service name is available for use.
   * Search service names must be globally unique since they are part of the
   * service URI (https://<name>.search.windows.net).
   *
   * @param {string} name The Search service name to validate. Search service
   * names must only contain lowercase letters, digits or dashes, cannot use dash
   * as the first two or last one characters, cannot contain consecutive dashes,
   * and must be between 2 and 60 characters in length.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<CheckNameAvailabilityOutput>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  checkNameAvailabilityWithHttpOperationResponse(name, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._checkNameAvailability(name, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Checks whether or not the given Search service name is available for use.
   * Search service names must be globally unique since they are part of the
   * service URI (https://<name>.search.windows.net).
   *
   * @param {string} name The Search service name to validate. Search service
   * names must only contain lowercase letters, digits or dashes, cannot use dash
   * as the first two or last one characters, cannot contain consecutive dashes,
   * and must be between 2 and 60 characters in length.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {CheckNameAvailabilityOutput} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link CheckNameAvailabilityOutput} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  checkNameAvailability(name, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._checkNameAvailability(name, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._checkNameAvailability(name, options, optionalCallback);
    }
  }

  /**
   * Creates or updates a Search service in the given resource group. If the
   * Search service already exists, all properties will be updated with the given
   * values.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service to
   * create or update. Search service names must only contain lowercase letters,
   * digits or dashes, cannot use dash as the first two or last one characters,
   * cannot contain consecutive dashes, and must be between 2 and 60 characters
   * in length. Search service names must be globally unique since they are part
   * of the service URI (https://<name>.search.windows.net). You cannot change
   * the service name after the service is created.
   *
   * @param {object} service The definition of the Search service to create or
   * update.
   *
   * @param {number} [service.replicaCount] The number of replicas in the Search
   * service. If specified, it must be a value between 1 and 12 inclusive for
   * standard SKUs or between 1 and 3 inclusive for basic SKU.
   *
   * @param {number} [service.partitionCount] The number of partitions in the
   * Search service; if specified, it can be 1, 2, 3, 4, 6, or 12. Values greater
   * than 1 are only valid for standard SKUs. For 'standard3' services with
   * hostingMode set to 'highDensity', the allowed values are between 1 and 3.
   *
   * @param {string} [service.hostingMode] Applicable only for the standard3 SKU.
   * You can set this property to enable up to 3 high density partitions that
   * allow up to 1000 indexes, which is much higher than the maximum indexes
   * allowed for any other SKU. For the standard3 SKU, the value is either
   * 'default' or 'highDensity'. For all other SKUs, this value must be
   * 'default'. Possible values include: 'default', 'highDensity'
   *
   * @param {object} service.sku The SKU of the Search Service, which determines
   * price tier and capacity limits.
   *
   * @param {string} [service.sku.name] The SKU of the Search service. Valid
   * values include: 'free': Shared service. 'basic': Dedicated service with up
   * to 3 replicas. 'standard': Dedicated service with up to 12 partitions and 12
   * replicas. 'standard2': Similar to standard, but with more capacity per
   * search unit. 'standard3': Offers maximum capacity per search unit with up to
   * 12 partitions and 12 replicas (or up to 3 partitions with more indexes if
   * you also set the hostingMode property to 'highDensity'). Possible values
   * include: 'free', 'basic', 'standard', 'standard2', 'standard3'
   *
   * @param {string} service.location The geographic location of the resource.
   * This must be one of the supported and registered Azure Geo Regions (for
   * example, West US, East US, Southeast Asia, and so forth).
   *
   * @param {object} [service.tags] Tags to help categorize the resource in the
   * Azure portal.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<SearchService>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName, searchServiceName, service, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginCreateOrUpdate(resourceGroupName, searchServiceName, service, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Creates or updates a Search service in the given resource group. If the
   * Search service already exists, all properties will be updated with the given
   * values.
   *
   * @param {string} resourceGroupName The name of the resource group within the
   * current subscription. You can obtain this value from the Azure Resource
   * Manager API or the portal.
   *
   * @param {string} searchServiceName The name of the Azure Search service to
   * create or update. Search service names must only contain lowercase letters,
   * digits or dashes, cannot use dash as the first two or last one characters,
   * cannot contain consecutive dashes, and must be between 2 and 60 characters
   * in length. Search service names must be globally unique since they are part
   * of the service URI (https://<name>.search.windows.net). You cannot change
   * the service name after the service is created.
   *
   * @param {object} service The definition of the Search service to create or
   * update.
   *
   * @param {number} [service.replicaCount] The number of replicas in the Search
   * service. If specified, it must be a value between 1 and 12 inclusive for
   * standard SKUs or between 1 and 3 inclusive for basic SKU.
   *
   * @param {number} [service.partitionCount] The number of partitions in the
   * Search service; if specified, it can be 1, 2, 3, 4, 6, or 12. Values greater
   * than 1 are only valid for standard SKUs. For 'standard3' services with
   * hostingMode set to 'highDensity', the allowed values are between 1 and 3.
   *
   * @param {string} [service.hostingMode] Applicable only for the standard3 SKU.
   * You can set this property to enable up to 3 high density partitions that
   * allow up to 1000 indexes, which is much higher than the maximum indexes
   * allowed for any other SKU. For the standard3 SKU, the value is either
   * 'default' or 'highDensity'. For all other SKUs, this value must be
   * 'default'. Possible values include: 'default', 'highDensity'
   *
   * @param {object} service.sku The SKU of the Search Service, which determines
   * price tier and capacity limits.
   *
   * @param {string} [service.sku.name] The SKU of the Search service. Valid
   * values include: 'free': Shared service. 'basic': Dedicated service with up
   * to 3 replicas. 'standard': Dedicated service with up to 12 partitions and 12
   * replicas. 'standard2': Similar to standard, but with more capacity per
   * search unit. 'standard3': Offers maximum capacity per search unit with up to
   * 12 partitions and 12 replicas (or up to 3 partitions with more indexes if
   * you also set the hostingMode property to 'highDensity'). Possible values
   * include: 'free', 'basic', 'standard', 'standard2', 'standard3'
   *
   * @param {string} service.location The geographic location of the resource.
   * This must be one of the supported and registered Azure Geo Regions (for
   * example, West US, East US, Southeast Asia, and so forth).
   *
   * @param {object} [service.tags] Tags to help categorize the resource in the
   * Azure portal.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.searchManagementRequestOptions] Additional
   * parameters for the operation
   *
   * @param {uuid} [options.searchManagementRequestOptions.clientRequestId] A
   * client-generated GUID value that identifies this request. If specified, this
   * will be included in response information as a way to track the request.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {SearchService} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link SearchService} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginCreateOrUpdate(resourceGroupName, searchServiceName, service, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginCreateOrUpdate(resourceGroupName, searchServiceName, service, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginCreateOrUpdate(resourceGroupName, searchServiceName, service, options, optionalCallback);
    }
  }

}

module.exports = Services;
