/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Cluster update request
 *
 */
class ClusterUpdateParameters {
  /**
   * Create a ClusterUpdateParameters.
   * @member {string} [reliabilityLevel] The reliability level sets the replica
   * set size of system services. Learn about
   * [ReliabilityLevel](https://docs.microsoft.com/en-us/azure/service-fabric/service-fabric-cluster-capacity).
   * Possible values include: 'Bronze', 'Silver', 'Gold'
   * @member {string} [upgradeMode] The upgrade mode of the cluster. This
   * indicates if the cluster should be automatically upgraded when new Service
   * Fabric runtime version is available. Possible values include: 'Automatic',
   * 'Manual'
   * @member {string} [clusterCodeVersion] The Service Fabric runtime version
   * of the cluster. This property can only by set the user when
   * **upgradeMode** is set to 'Manual'. To get list of available Service
   * Fabric versions for new clusters use [ClusterVersion
   * API](./ClusterVersion.md). To get the list of available version for
   * existing clusters use **availableClusterVersions**.
   * @member {object} [certificate] The certificate to use for securing the
   * cluster. The certificate provided will be used for  node to node security
   * within the cluster, SSL certificate for cluster management endpoint and
   * default  admin client.
   * @member {string} [certificate.thumbprint] Thumbprint of the primary
   * certificate.
   * @member {string} [certificate.thumbprintSecondary] Thumbprint of the
   * secondary certificate.
   * @member {string} [certificate.x509StoreName] The local certificate store
   * location. Possible values include: 'AddressBook', 'AuthRoot',
   * 'CertificateAuthority', 'Disallowed', 'My', 'Root', 'TrustedPeople',
   * 'TrustedPublisher'
   * @member {array} [clientCertificateThumbprints] The list of client
   * certificates referenced by thumbprint that are allowed to manage the
   * cluster. This will overwrite the existing list.
   * @member {array} [clientCertificateCommonNames] The list of client
   * certificates referenced by common name that are allowed to manage the
   * cluster. This will overwrite the existing list.
   * @member {array} [fabricSettings] The list of custom fabric settings to
   * configure the cluster. This will overwrite the existing list.
   * @member {object} [reverseProxyCertificate] The server certificate used by
   * reverse proxy.
   * @member {string} [reverseProxyCertificate.thumbprint] Thumbprint of the
   * primary certificate.
   * @member {string} [reverseProxyCertificate.thumbprintSecondary] Thumbprint
   * of the secondary certificate.
   * @member {string} [reverseProxyCertificate.x509StoreName] The local
   * certificate store location. Possible values include: 'AddressBook',
   * 'AuthRoot', 'CertificateAuthority', 'Disallowed', 'My', 'Root',
   * 'TrustedPeople', 'TrustedPublisher'
   * @member {array} [nodeTypes] The list of node types in the cluster. This
   * will overwrite the existing list.
   * @member {object} [upgradeDescription] The policy to use when upgrading the
   * cluster.
   * @member {boolean} [upgradeDescription.forceRestart] If true, then
   * processes are forcefully restarted during upgrade even when the code
   * version has not changed (the upgrade only changes configuration or data).
   * @member {string} [upgradeDescription.upgradeReplicaSetCheckTimeout] The
   * maximum amount of time to block processing of an upgrade domain and revent
   * loss of availability when there are unexpected issues. When this timeout
   * expires, processing of the upgrade domain will proceed regardless of
   * availability loss issues. The timeout is reset at the start of each
   * upgrade domain. The timeout can be in either hh:mm:ss or in d.hh:mm:ss.ms
   * format.
   * @member {string} [upgradeDescription.healthCheckWaitDuration] The length
   * of time to wait after completing an upgrade domain before performing
   * health checks. The duration can be in either hh:mm:ss or in d.hh:mm:ss.ms
   * format.
   * @member {string} [upgradeDescription.healthCheckStableDuration] The amount
   * of time that the application or cluster must remain healthy before the
   * upgrade proceeds to the next upgrade domain. The duration can be in either
   * hh:mm:ss or in d.hh:mm:ss.ms format.
   * @member {string} [upgradeDescription.healthCheckRetryTimeout] The amount
   * of time to retry health evaluation when the application or cluster is
   * unhealthy before the upgrade rolls back. The timeout can be in either
   * hh:mm:ss or in d.hh:mm:ss.ms format.
   * @member {string} [upgradeDescription.upgradeTimeout] The amount of time
   * the overall upgrade has to complete before the upgrade rolls back. The
   * timeout can be in either hh:mm:ss or in d.hh:mm:ss.ms format.
   * @member {string} [upgradeDescription.upgradeDomainTimeout] The amount of
   * time each upgrade domain has to complete before the upgrade rolls back.
   * The timeout can be in either hh:mm:ss or in d.hh:mm:ss.ms format.
   * @member {object} [upgradeDescription.healthPolicy] The cluster health
   * policy used when upgrading the cluster.
   * @member {number}
   * [upgradeDescription.healthPolicy.maxPercentUnhealthyNodes] The maximum
   * allowed percentage of unhealthy nodes before reporting an error. For
   * example, to allow 10% of nodes to be unhealthy, this value would be 10.
   * @member {number}
   * [upgradeDescription.healthPolicy.maxPercentUnhealthyApplications] The
   * maximum allowed percentage of unhealthy applications before reporting an
   * error. For example, to allow 10% of applications to be unhealthy, this
   * value would be 10.
   * @member {object} [upgradeDescription.deltaHealthPolicy] The delta health
   * policy used when upgrading the cluster.
   * @member {number}
   * [upgradeDescription.deltaHealthPolicy.maxPercentDeltaUnhealthyNodes] The
   * maximum allowed percentage of nodes health degradation allowed during
   * cluster upgrades. The delta is measured between the state of the nodes at
   * the beginning of upgrade and the state of the nodes at the time of the
   * health evaluation. The check is performed after every upgrade domain
   * upgrade completion to make sure the global state of the cluster is within
   * tolerated limits.
   * @member {number}
   * [upgradeDescription.deltaHealthPolicy.maxPercentUpgradeDomainDeltaUnhealthyNodes]
   * The maximum allowed percentage of upgrade domain nodes health degradation
   * allowed during cluster upgrades. The delta is measured between the state
   * of the upgrade domain nodes at the beginning of upgrade and the state of
   * the upgrade domain nodes at the time of the health evaluation. The check
   * is performed after every upgrade domain upgrade completion for all
   * completed upgrade domains to make sure the state of the upgrade domains is
   * within tolerated limits.
   * @member {number}
   * [upgradeDescription.deltaHealthPolicy.maxPercentDeltaUnhealthyApplications]
   * The maximum allowed percentage of applications health degradation allowed
   * during cluster upgrades. The delta is measured between the state of the
   * applications at the beginning of upgrade and the state of the applications
   * at the time of the health evaluation. The check is performed after every
   * upgrade domain upgrade completion to make sure the global state of the
   * cluster is within tolerated limits. System services are not included in
   * this.
   * @member {array} [addOnFeatures] The list of add-on features to enable in
   * the cluster.
   * @member {object} [tags] Cluster update parameters
   */
  constructor() {
  }

  /**
   * Defines the metadata of ClusterUpdateParameters
   *
   * @returns {object} metadata of ClusterUpdateParameters
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ClusterUpdateParameters',
      type: {
        name: 'Composite',
        className: 'ClusterUpdateParameters',
        modelProperties: {
          reliabilityLevel: {
            required: false,
            serializedName: 'properties.reliabilityLevel',
            type: {
              name: 'String'
            }
          },
          upgradeMode: {
            required: false,
            serializedName: 'properties.upgradeMode',
            type: {
              name: 'String'
            }
          },
          clusterCodeVersion: {
            required: false,
            serializedName: 'properties.clusterCodeVersion',
            type: {
              name: 'String'
            }
          },
          certificate: {
            required: false,
            serializedName: 'properties.certificate',
            type: {
              name: 'Composite',
              className: 'CertificateDescription'
            }
          },
          clientCertificateThumbprints: {
            required: false,
            serializedName: 'properties.clientCertificateThumbprints',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ClientCertificateThumbprintElementType',
                  type: {
                    name: 'Composite',
                    className: 'ClientCertificateThumbprint'
                  }
              }
            }
          },
          clientCertificateCommonNames: {
            required: false,
            serializedName: 'properties.clientCertificateCommonNames',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ClientCertificateCommonNameElementType',
                  type: {
                    name: 'Composite',
                    className: 'ClientCertificateCommonName'
                  }
              }
            }
          },
          fabricSettings: {
            required: false,
            serializedName: 'properties.fabricSettings',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'SettingsSectionDescriptionElementType',
                  type: {
                    name: 'Composite',
                    className: 'SettingsSectionDescription'
                  }
              }
            }
          },
          reverseProxyCertificate: {
            required: false,
            serializedName: 'properties.reverseProxyCertificate',
            type: {
              name: 'Composite',
              className: 'CertificateDescription'
            }
          },
          nodeTypes: {
            required: false,
            serializedName: 'properties.nodeTypes',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'NodeTypeDescriptionElementType',
                  type: {
                    name: 'Composite',
                    className: 'NodeTypeDescription'
                  }
              }
            }
          },
          upgradeDescription: {
            required: false,
            serializedName: 'properties.upgradeDescription',
            type: {
              name: 'Composite',
              className: 'ClusterUpgradePolicy'
            }
          },
          addOnFeatures: {
            required: false,
            serializedName: 'properties.addOnFeatures',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ClusterUpdateParameters;
