/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Contains information about a database Threat Detection policy.
 *
 * @extends models['ProxyResource']
 */
class DatabaseSecurityAlertPolicy extends models['ProxyResource'] {
  /**
   * Create a DatabaseSecurityAlertPolicy.
   * @member {string} [location] The geo-location where the resource lives
   * @member {string} [kind] Resource kind.
   * @member {string} state Specifies the state of the policy. If state is
   * Enabled, storageEndpoint and storageAccountAccessKey are required.
   * Possible values include: 'New', 'Enabled', 'Disabled'
   * @member {string} [disabledAlerts] Specifies the semicolon-separated list
   * of alerts that are disabled, or empty string to disable no alerts.
   * Possible values: Sql_Injection; Sql_Injection_Vulnerability;
   * Access_Anomaly; Usage_Anomaly.
   * @member {string} [emailAddresses] Specifies the semicolon-separated list
   * of e-mail addresses to which the alert is sent.
   * @member {string} [emailAccountAdmins] Specifies that the alert is sent to
   * the account administrators. Possible values include: 'Enabled', 'Disabled'
   * @member {string} [storageEndpoint] Specifies the blob storage endpoint
   * (e.g. https://MyAccount.blob.core.windows.net). This blob storage will
   * hold all Threat Detection audit logs. If state is Enabled, storageEndpoint
   * is required.
   * @member {string} [storageAccountAccessKey] Specifies the identifier key of
   * the Threat Detection audit storage account. If state is Enabled,
   * storageAccountAccessKey is required.
   * @member {number} [retentionDays] Specifies the number of days to keep in
   * the Threat Detection audit logs.
   * @member {string} [useServerDefault] Specifies whether to use the default
   * server policy. Possible values include: 'Enabled', 'Disabled'
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of DatabaseSecurityAlertPolicy
   *
   * @returns {object} metadata of DatabaseSecurityAlertPolicy
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'DatabaseSecurityAlertPolicy',
      type: {
        name: 'Composite',
        className: 'DatabaseSecurityAlertPolicy',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: false,
            readOnly: true,
            serializedName: 'kind',
            type: {
              name: 'String'
            }
          },
          state: {
            required: true,
            serializedName: 'properties.state',
            type: {
              name: 'Enum',
              allowedValues: [ 'New', 'Enabled', 'Disabled' ]
            }
          },
          disabledAlerts: {
            required: false,
            serializedName: 'properties.disabledAlerts',
            type: {
              name: 'String'
            }
          },
          emailAddresses: {
            required: false,
            serializedName: 'properties.emailAddresses',
            type: {
              name: 'String'
            }
          },
          emailAccountAdmins: {
            required: false,
            serializedName: 'properties.emailAccountAdmins',
            type: {
              name: 'Enum',
              allowedValues: [ 'Enabled', 'Disabled' ]
            }
          },
          storageEndpoint: {
            required: false,
            serializedName: 'properties.storageEndpoint',
            type: {
              name: 'String'
            }
          },
          storageAccountAccessKey: {
            required: false,
            serializedName: 'properties.storageAccountAccessKey',
            type: {
              name: 'String'
            }
          },
          retentionDays: {
            required: false,
            serializedName: 'properties.retentionDays',
            type: {
              name: 'Number'
            }
          },
          useServerDefault: {
            required: false,
            serializedName: 'properties.useServerDefault',
            type: {
              name: 'Enum',
              allowedValues: [ 'Enabled', 'Disabled' ]
            }
          }
        }
      }
    };
  }
}

module.exports = DatabaseSecurityAlertPolicy;
