/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * ARM resource.
 *
 * @member {string} [id] Resource ID.
 * @member {string} [name] Resource name.
 * @member {string} [type] Resource type.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  readonly name?: string;
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the ProxyResource class.
 * @constructor
 * ARM proxy resource.
 *
 */
export interface ProxyResource extends Resource {
}

/**
 * @class
 * Initializes a new instance of the BackupLongTermRetentionPolicy class.
 * @constructor
 * A backup long term retention policy
 *
 * @member {string} [location] The geo-location where the resource lives
 * @member {string} state The status of the backup long term retention policy.
 * Possible values include: 'Disabled', 'Enabled'
 * @member {string} recoveryServicesBackupPolicyResourceId The azure recovery
 * services backup protection policy resource id
 */
export interface BackupLongTermRetentionPolicy extends ProxyResource {
  readonly location?: string;
  state: string;
  recoveryServicesBackupPolicyResourceId: string;
}

/**
 * @class
 * Initializes a new instance of the BackupLongTermRetentionVault class.
 * @constructor
 * A backup long term retention vault
 *
 * @member {string} [location] The geo-location where the resource lives
 * @member {string} recoveryServicesVaultResourceId The azure recovery services
 * vault resource id
 */
export interface BackupLongTermRetentionVault extends ProxyResource {
  readonly location?: string;
  recoveryServicesVaultResourceId: string;
}

/**
 * @class
 * Initializes a new instance of the TrackedResource class.
 * @constructor
 * ARM tracked top level resource.
 *
 * @member {object} [tags] Resource tags.
 * @member {string} location Resource location.
 */
export interface TrackedResource extends Resource {
  tags?: { [propertyName: string]: string };
  location: string;
}

/**
 * @class
 * Initializes a new instance of the RestorePoint class.
 * @constructor
 * A database restore point.
 *
 * @member {string} [restorePointType] The restore point type of the database
 * restore point. Possible values include: 'DISCRETE', 'CONTINUOUS'
 * @member {date} [restorePointCreationDate] Restore point creation time
 * (ISO8601 format). Populated when restorePointType = CONTINUOUS. Null
 * otherwise.
 * @member {date} [earliestRestoreDate] Earliest restore time (ISO8601 format).
 * Populated when restorePointType = DISCRETE. Null otherwise.
 */
export interface RestorePoint extends ProxyResource {
  readonly restorePointType?: string;
  readonly restorePointCreationDate?: Date;
  readonly earliestRestoreDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the RecoverableDatabase class.
 * @constructor
 * A recoverable database
 *
 * @member {string} [edition] The edition of the database
 * @member {string} [serviceLevelObjective] The service level objective name of
 * the database
 * @member {string} [elasticPoolName] The elastic pool name of the database
 * @member {date} [lastAvailableBackupDate] The last available backup date of
 * the database (ISO8601 format)
 */
export interface RecoverableDatabase extends ProxyResource {
  readonly edition?: string;
  readonly serviceLevelObjective?: string;
  readonly elasticPoolName?: string;
  readonly lastAvailableBackupDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the RestorableDroppedDatabase class.
 * @constructor
 * A restorable dropped database
 *
 * @member {string} [location] The geo-location where the resource lives
 * @member {string} [databaseName] The name of the database
 * @member {string} [edition] The edition of the database
 * @member {string} [maxSizeBytes] The max size in bytes of the database
 * @member {string} [serviceLevelObjective] The service level objective name of
 * the database
 * @member {string} [elasticPoolName] The elastic pool name of the database
 * @member {date} [creationDate] The creation date of the database (ISO8601
 * format)
 * @member {date} [deletionDate] The deletion date of the database (ISO8601
 * format)
 * @member {date} [earliestRestoreDate] The earliest restore date of the
 * database (ISO8601 format)
 */
export interface RestorableDroppedDatabase extends ProxyResource {
  readonly location?: string;
  readonly databaseName?: string;
  readonly edition?: string;
  readonly maxSizeBytes?: string;
  readonly serviceLevelObjective?: string;
  readonly elasticPoolName?: string;
  readonly creationDate?: Date;
  readonly deletionDate?: Date;
  readonly earliestRestoreDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the MaxSizeCapability class.
 * @constructor
 * The maximum size limits for a database.
 *
 * @member {number} [limit] The maximum size of the database (see 'unit' for
 * the units).
 * @member {string} [unit] The units that the limit is expressed in. Possible
 * values include: 'Megabytes', 'Gigabytes', 'Terabytes', 'Petabytes'
 * @member {string} [status] The status of the maximum size capability.
 * Possible values include: 'Visible', 'Available', 'Default', 'Disabled'
 */
export interface MaxSizeCapability {
  readonly limit?: number;
  readonly unit?: string;
  readonly status?: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceObjectiveCapability class.
 * @constructor
 * The service objectives capability.
 *
 * @member {string} [name] The service objective name.
 * @member {string} [status] The status of the service objective. Possible
 * values include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {string} [unit] Unit type used to measure service objective
 * performance level. Possible values include: 'DTU'
 * @member {number} [value] Performance level value.
 * @member {uuid} [id] The unique ID of the service objective.
 * @member {array} [supportedMaxSizes] The list of supported maximum database
 * sizes for this service objective.
 * @member {object} [includedMaxSize] The included (free) max size for this
 * service level objective.
 * @member {number} [includedMaxSize.limit] The maximum size of the database
 * (see 'unit' for the units).
 * @member {string} [includedMaxSize.unit] The units that the limit is
 * expressed in. Possible values include: 'Megabytes', 'Gigabytes',
 * 'Terabytes', 'Petabytes'
 * @member {string} [includedMaxSize.status] The status of the maximum size
 * capability. Possible values include: 'Visible', 'Available', 'Default',
 * 'Disabled'
 */
export interface ServiceObjectiveCapability {
  readonly name?: string;
  readonly status?: string;
  readonly unit?: string;
  readonly value?: number;
  readonly id?: string;
  readonly supportedMaxSizes?: MaxSizeCapability[];
  readonly includedMaxSize?: MaxSizeCapability;
}

/**
 * @class
 * Initializes a new instance of the EditionCapability class.
 * @constructor
 * The database edition capabilities.
 *
 * @member {string} [name] The edition name.
 * @member {string} [status] The status of the edition. Possible values
 * include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {array} [supportedServiceLevelObjectives] The list of supported
 * service objectives for the edition.
 */
export interface EditionCapability {
  readonly name?: string;
  readonly status?: string;
  readonly supportedServiceLevelObjectives?: ServiceObjectiveCapability[];
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolPerDatabaseMinDtuCapability class.
 * @constructor
 * The minimum per-database DTU capability.
 *
 * @member {number} [limit] The maximum DTUs per database.
 * @member {string} [status] The status of the capability. Possible values
 * include: 'Visible', 'Available', 'Default', 'Disabled'
 */
export interface ElasticPoolPerDatabaseMinDtuCapability {
  readonly limit?: number;
  readonly status?: string;
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolPerDatabaseMaxDtuCapability class.
 * @constructor
 * The max per-database DTU capability.
 *
 * @member {number} [limit] The maximum DTUs per database.
 * @member {string} [status] The status of the capability. Possible values
 * include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {array} [supportedPerDatabaseMinDtus] The list of supported min
 * database DTUs.
 */
export interface ElasticPoolPerDatabaseMaxDtuCapability {
  readonly limit?: number;
  readonly status?: string;
  readonly supportedPerDatabaseMinDtus?: ElasticPoolPerDatabaseMinDtuCapability[];
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolDtuCapability class.
 * @constructor
 * The Elastic Pool DTU capability.
 *
 * @member {number} [limit] The maximum size of the database (see 'unit' for
 * the units).
 * @member {number} [maxDatabaseCount] The maximum number of databases
 * supported.
 * @member {string} [status] The status of the capability. Possible values
 * include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {array} [supportedMaxSizes] The list of supported max sizes.
 * @member {object} [includedMaxSize] The included (free) max size for this
 * service level objective.
 * @member {number} [includedMaxSize.limit] The maximum size of the database
 * (see 'unit' for the units).
 * @member {string} [includedMaxSize.unit] The units that the limit is
 * expressed in. Possible values include: 'Megabytes', 'Gigabytes',
 * 'Terabytes', 'Petabytes'
 * @member {string} [includedMaxSize.status] The status of the maximum size
 * capability. Possible values include: 'Visible', 'Available', 'Default',
 * 'Disabled'
 * @member {array} [supportedPerDatabaseMaxSizes] The list of supported max
 * database sizes.
 * @member {array} [supportedPerDatabaseMaxDtus] The list of supported max
 * database DTUs.
 */
export interface ElasticPoolDtuCapability {
  readonly limit?: number;
  readonly maxDatabaseCount?: number;
  readonly status?: string;
  readonly supportedMaxSizes?: MaxSizeCapability[];
  readonly includedMaxSize?: MaxSizeCapability;
  readonly supportedPerDatabaseMaxSizes?: MaxSizeCapability[];
  readonly supportedPerDatabaseMaxDtus?: ElasticPoolPerDatabaseMaxDtuCapability[];
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolEditionCapability class.
 * @constructor
 * The elastic pool edition capabilities.
 *
 * @member {string} [name] The elastic pool edition name.
 * @member {string} [status] The status of the elastic pool edition. Possible
 * values include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {array} [supportedElasticPoolDtus] The list of supported elastic
 * pool DTU levels for the edition.
 */
export interface ElasticPoolEditionCapability {
  readonly name?: string;
  readonly status?: string;
  readonly supportedElasticPoolDtus?: ElasticPoolDtuCapability[];
}

/**
 * @class
 * Initializes a new instance of the ServerVersionCapability class.
 * @constructor
 * The server capabilities.
 *
 * @member {string} [name] The server version name.
 * @member {string} [status] The status of the server version. Possible values
 * include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {array} [supportedEditions] The list of supported database editions.
 * @member {array} [supportedElasticPoolEditions] The list of supported elastic
 * pool editions.
 */
export interface ServerVersionCapability {
  readonly name?: string;
  readonly status?: string;
  readonly supportedEditions?: EditionCapability[];
  readonly supportedElasticPoolEditions?: ElasticPoolEditionCapability[];
}

/**
 * @class
 * Initializes a new instance of the LocationCapabilities class.
 * @constructor
 * The capabilities for a location.
 *
 * @member {string} [name] The location name.
 * @member {string} [status] Azure SQL Database's status for the location.
 * Possible values include: 'Visible', 'Available', 'Default', 'Disabled'
 * @member {array} [supportedServerVersions] The list of supported server
 * versions.
 */
export interface LocationCapabilities {
  readonly name?: string;
  readonly status?: string;
  readonly supportedServerVersions?: ServerVersionCapability[];
}

/**
 * @class
 * Initializes a new instance of the ServerConnectionPolicy class.
 * @constructor
 * A server secure connection policy.
 *
 * @member {string} [kind] Metadata used for the Azure portal experience.
 * @member {string} [location] Resource location.
 * @member {string} connectionType The server connection type. Possible values
 * include: 'Default', 'Proxy', 'Redirect'
 */
export interface ServerConnectionPolicy extends ProxyResource {
  readonly kind?: string;
  readonly location?: string;
  connectionType: string;
}

/**
 * @class
 * Initializes a new instance of the DatabaseSecurityAlertPolicy class.
 * @constructor
 * Contains information about a database Threat Detection policy.
 *
 * @member {string} [location] The geo-location where the resource lives
 * @member {string} [kind] Resource kind.
 * @member {string} state Specifies the state of the policy. If state is
 * Enabled, storageEndpoint and storageAccountAccessKey are required. Possible
 * values include: 'New', 'Enabled', 'Disabled'
 * @member {string} [disabledAlerts] Specifies the semicolon-separated list of
 * alerts that are disabled, or empty string to disable no alerts. Possible
 * values: Sql_Injection; Sql_Injection_Vulnerability; Access_Anomaly;
 * Usage_Anomaly.
 * @member {string} [emailAddresses] Specifies the semicolon-separated list of
 * e-mail addresses to which the alert is sent.
 * @member {string} [emailAccountAdmins] Specifies that the alert is sent to
 * the account administrators. Possible values include: 'Enabled', 'Disabled'
 * @member {string} [storageEndpoint] Specifies the blob storage endpoint (e.g.
 * https://MyAccount.blob.core.windows.net). This blob storage will hold all
 * Threat Detection audit logs. If state is Enabled, storageEndpoint is
 * required.
 * @member {string} [storageAccountAccessKey] Specifies the identifier key of
 * the Threat Detection audit storage account. If state is Enabled,
 * storageAccountAccessKey is required.
 * @member {number} [retentionDays] Specifies the number of days to keep in the
 * Threat Detection audit logs.
 * @member {string} [useServerDefault] Specifies whether to use the default
 * server policy. Possible values include: 'Enabled', 'Disabled'
 */
export interface DatabaseSecurityAlertPolicy extends ProxyResource {
  location?: string;
  readonly kind?: string;
  state: string;
  disabledAlerts?: string;
  emailAddresses?: string;
  emailAccountAdmins?: string;
  storageEndpoint?: string;
  storageAccountAccessKey?: string;
  retentionDays?: number;
  useServerDefault?: string;
}

/**
 * @class
 * Initializes a new instance of the DataMaskingPolicy class.
 * @constructor
 * Represents a database data masking policy.
 *
 * @member {string} dataMaskingState The state of the data masking policy.
 * Possible values include: 'Disabled', 'Enabled'
 * @member {string} [exemptPrincipals] The list of the exempt principals.
 * Specifies the semicolon-separated list of database users for which the data
 * masking policy does not apply. The specified users receive data results
 * without masking for all of the database queries.
 * @member {string} [applicationPrincipals] The list of the application
 * principals. This is a legacy parameter and is no longer used.
 * @member {string} [maskingLevel] The masking level. This is a legacy
 * parameter and is no longer used.
 * @member {string} [location] The location of the data masking policy.
 * @member {string} [kind] The kind of data masking policy. Metadata, used for
 * Azure portal.
 */
export interface DataMaskingPolicy extends ProxyResource {
  dataMaskingState: string;
  exemptPrincipals?: string;
  readonly applicationPrincipals?: string;
  readonly maskingLevel?: string;
  readonly location?: string;
  readonly kind?: string;
}

/**
 * @class
 * Initializes a new instance of the DataMaskingRule class.
 * @constructor
 * Represents a database data masking rule.
 *
 * @member {string} [dataMaskingRuleId] The rule Id.
 * @member {string} [aliasName] The alias name. This is a legacy parameter and
 * is no longer used.
 * @member {string} [ruleState] The rule state. Used to delete a rule. To
 * delete an existing rule, specify the schemaName, tableName, columnName,
 * maskingFunction, and specify ruleState as disabled. However, if the rule
 * doesn't already exist, the rule will be created with ruleState set to
 * enabled, regardless of the provided value of ruleState. Possible values
 * include: 'Disabled', 'Enabled'
 * @member {string} schemaName The schema name on which the data masking rule
 * is applied.
 * @member {string} tableName The table name on which the data masking rule is
 * applied.
 * @member {string} columnName The column name on which the data masking rule
 * is applied.
 * @member {string} maskingFunction The masking function that is used for the
 * data masking rule. Possible values include: 'Default', 'CCN', 'Email',
 * 'Number', 'SSN', 'Text'
 * @member {string} [numberFrom] The numberFrom property of the masking rule.
 * Required if maskingFunction is set to Number, otherwise this parameter will
 * be ignored.
 * @member {string} [numberTo] The numberTo property of the data masking rule.
 * Required if maskingFunction is set to Number, otherwise this parameter will
 * be ignored.
 * @member {string} [prefixSize] If maskingFunction is set to Text, the number
 * of characters to show unmasked in the beginning of the string. Otherwise,
 * this parameter will be ignored.
 * @member {string} [suffixSize] If maskingFunction is set to Text, the number
 * of characters to show unmasked at the end of the string. Otherwise, this
 * parameter will be ignored.
 * @member {string} [replacementString] If maskingFunction is set to Text, the
 * character to use for masking the unexposed part of the string. Otherwise,
 * this parameter will be ignored.
 * @member {string} [location] The location of the data masking rule.
 * @member {string} [kind] The kind of Data Masking Rule. Metadata, used for
 * Azure portal.
 */
export interface DataMaskingRule extends ProxyResource {
  readonly dataMaskingRuleId?: string;
  aliasName?: string;
  ruleState?: string;
  schemaName: string;
  tableName: string;
  columnName: string;
  maskingFunction: string;
  numberFrom?: string;
  numberTo?: string;
  prefixSize?: string;
  suffixSize?: string;
  replacementString?: string;
  readonly location?: string;
  readonly kind?: string;
}

/**
 * @class
 * Initializes a new instance of the FirewallRule class.
 * @constructor
 * Represents a server firewall rule.
 *
 * @member {string} [kind] Kind of server that contains this firewall rule.
 * @member {string} [location] Location of the server that contains this
 * firewall rule.
 * @member {string} startIpAddress The start IP address of the firewall rule.
 * Must be IPv4 format. Use value '0.0.0.0' to represent all Azure-internal IP
 * addresses.
 * @member {string} endIpAddress The end IP address of the firewall rule. Must
 * be IPv4 format. Must be greater than or equal to startIpAddress. Use value
 * '0.0.0.0' to represent all Azure-internal IP addresses.
 */
export interface FirewallRule extends ProxyResource {
  readonly kind?: string;
  readonly location?: string;
  startIpAddress: string;
  endIpAddress: string;
}

/**
 * @class
 * Initializes a new instance of the GeoBackupPolicy class.
 * @constructor
 * A database geo backup policy.
 *
 * @member {string} state The state of the geo backup policy. Possible values
 * include: 'Disabled', 'Enabled'
 * @member {string} [storageType] The storage type of the geo backup policy.
 * @member {string} [kind] Kind of geo backup policy.  This is metadata used
 * for the Azure portal experience.
 * @member {string} [location] Backup policy location.
 */
export interface GeoBackupPolicy extends ProxyResource {
  state: string;
  readonly storageType?: string;
  readonly kind?: string;
  readonly location?: string;
}

/**
 * @class
 * Initializes a new instance of the ImportExtensionRequest class.
 * @constructor
 * Import database parameters.
 *
 * @member {string} [name] The name of the extension.
 * @member {string} [type] The type of the extension.
 * @member {string} storageKeyType The type of the storage key to use. Possible
 * values include: 'StorageAccessKey', 'SharedAccessKey'
 * @member {string} storageKey The storage key to use.  If storage key type is
 * SharedAccessKey, it must be preceded with a "?."
 * @member {string} storageUri The storage uri to use.
 * @member {string} administratorLogin The name of the SQL administrator.
 * @member {string} administratorLoginPassword The password of the SQL
 * administrator.
 * @member {string} [authenticationType] The authentication type. Possible
 * values include: 'SQL', 'ADPassword'. Default value: 'SQL' .
 */
export interface ImportExtensionRequest {
  name?: string;
  type?: string;
  storageKeyType: string;
  storageKey: string;
  storageUri: string;
  administratorLogin: string;
  administratorLoginPassword: string;
  authenticationType?: string;
}

/**
 * @class
 * Initializes a new instance of the ImportExportResponse class.
 * @constructor
 * Response for Import/Export Get operation.
 *
 * @member {string} [requestType] The request type of the operation.
 * @member {uuid} [requestId] The request type of the operation.
 * @member {string} [serverName] The name of the server.
 * @member {string} [databaseName] The name of the database.
 * @member {string} [status] The status message returned from the server.
 * @member {string} [lastModifiedTime] The operation status last modified time.
 * @member {string} [queuedTime] The operation queued time.
 * @member {string} [blobUri] The blob uri.
 * @member {string} [errorMessage] The error message returned from the server.
 */
export interface ImportExportResponse extends ProxyResource {
  readonly requestType?: string;
  readonly requestId?: string;
  readonly serverName?: string;
  readonly databaseName?: string;
  readonly status?: string;
  readonly lastModifiedTime?: string;
  readonly queuedTime?: string;
  readonly blobUri?: string;
  readonly errorMessage?: string;
}

/**
 * @class
 * Initializes a new instance of the ExportRequest class.
 * @constructor
 * Export database parameters.
 *
 * @member {string} storageKeyType The type of the storage key to use. Possible
 * values include: 'StorageAccessKey', 'SharedAccessKey'
 * @member {string} storageKey The storage key to use.  If storage key type is
 * SharedAccessKey, it must be preceded with a "?."
 * @member {string} storageUri The storage uri to use.
 * @member {string} administratorLogin The name of the SQL administrator.
 * @member {string} administratorLoginPassword The password of the SQL
 * administrator.
 * @member {string} [authenticationType] The authentication type. Possible
 * values include: 'SQL', 'ADPassword'. Default value: 'SQL' .
 */
export interface ExportRequest {
  storageKeyType: string;
  storageKey: string;
  storageUri: string;
  administratorLogin: string;
  administratorLoginPassword: string;
  authenticationType?: string;
}

/**
 * @class
 * Initializes a new instance of the ImportRequest class.
 * @constructor
 * Import database parameters.
 *
 * @member {string} databaseName The name of the database to import.
 * @member {string} edition The edition for the database being created.
 * Possible values include: 'Web', 'Business', 'Basic', 'Standard', 'Premium',
 * 'Free', 'Stretch', 'DataWarehouse', 'System', 'System2'
 * @member {string} serviceObjectiveName The name of the service objective to
 * assign to the database. Possible values include: 'Basic', 'S0', 'S1', 'S2',
 * 'S3', 'P1', 'P2', 'P3', 'P4', 'P6', 'P11', 'P15', 'System', 'System2',
 * 'ElasticPool'
 * @member {string} maxSizeBytes The maximum size for the newly imported
 * database.
 */
export interface ImportRequest extends ExportRequest {
  databaseName: string;
  edition: string;
  serviceObjectiveName: string;
  maxSizeBytes: string;
}

/**
 * @class
 * Initializes a new instance of the MetricValue class.
 * @constructor
 * Represents database metrics.
 *
 * @member {number} [count] The number of values for the metric.
 * @member {number} [average] The average value of the metric.
 * @member {number} [maximum] The max value of the metric.
 * @member {number} [minimum] The min value of the metric.
 * @member {date} [timestamp] The metric timestamp (ISO-8601 format).
 * @member {number} [total] The total value of the metric.
 */
export interface MetricValue {
  readonly count?: number;
  readonly average?: number;
  readonly maximum?: number;
  readonly minimum?: number;
  readonly timestamp?: Date;
  readonly total?: number;
}

/**
 * @class
 * Initializes a new instance of the MetricName class.
 * @constructor
 * A database metric name.
 *
 * @member {string} [value] The name of the database metric.
 * @member {string} [localizedValue] The friendly name of the database metric.
 */
export interface MetricName {
  readonly value?: string;
  readonly localizedValue?: string;
}

/**
 * @class
 * Initializes a new instance of the Metric class.
 * @constructor
 * Database metrics.
 *
 * @member {date} [startTime] The start time for the metric (ISO-8601 format).
 * @member {date} [endTime] The end time for the metric (ISO-8601 format).
 * @member {string} [timeGrain] The time step to be used to summarize the
 * metric values.
 * @member {string} [unit] The unit of the metric. Possible values include:
 * 'count', 'bytes', 'seconds', 'percent', 'countPerSecond', 'bytesPerSecond'
 * @member {object} [name] The name information for the metric.
 * @member {string} [name.value] The name of the database metric.
 * @member {string} [name.localizedValue] The friendly name of the database
 * metric.
 * @member {array} [metricValues] The metric values for the specified time
 * window and timestep.
 */
export interface Metric {
  readonly startTime?: Date;
  readonly endTime?: Date;
  readonly timeGrain?: string;
  readonly unit?: string;
  readonly name?: MetricName;
  readonly metricValues?: MetricValue[];
}

/**
 * @class
 * Initializes a new instance of the MetricAvailability class.
 * @constructor
 * A metric availability value.
 *
 * @member {string} [retention] The length of retention for the database
 * metric.
 * @member {string} [timeGrain] The granularity of the database metric.
 */
export interface MetricAvailability {
  readonly retention?: string;
  readonly timeGrain?: string;
}

/**
 * @class
 * Initializes a new instance of the MetricDefinition class.
 * @constructor
 * A database metric definition.
 *
 * @member {object} [name] The name information for the metric.
 * @member {string} [name.value] The name of the database metric.
 * @member {string} [name.localizedValue] The friendly name of the database
 * metric.
 * @member {string} [primaryAggregationType] The primary aggregation type
 * defining how metric values are displayed. Possible values include: 'None',
 * 'Average', 'Count', 'Minimum', 'Maximum', 'Total'
 * @member {string} [resourceUri] The resource uri of the database.
 * @member {string} [unit] The unit of the metric. Possible values include:
 * 'Count', 'Bytes', 'Seconds', 'Percent', 'CountPerSecond', 'BytesPerSecond'
 * @member {array} [metricAvailabilities] The list of database metric
 * availabities for the metric.
 */
export interface MetricDefinition {
  readonly name?: MetricName;
  readonly primaryAggregationType?: string;
  readonly resourceUri?: string;
  readonly unit?: string;
  readonly metricAvailabilities?: MetricAvailability[];
}

/**
 * @class
 * Initializes a new instance of the ReplicationLink class.
 * @constructor
 * Represents a database replication link.
 *
 * @member {string} [location] Location of the server that contains this
 * firewall rule.
 * @member {boolean} [isTerminationAllowed] Legacy value indicating whether
 * termination is allowed.  Currently always returns true.
 * @member {string} [replicationMode] Replication mode of this replication
 * link.
 * @member {string} [partnerServer] The name of the server hosting the partner
 * database.
 * @member {string} [partnerDatabase] The name of the partner database.
 * @member {string} [partnerLocation] The Azure Region of the partner database.
 * @member {string} [role] The role of the database in the replication link.
 * Possible values include: 'Primary', 'Secondary', 'NonReadableSecondary',
 * 'Source', 'Copy'
 * @member {string} [partnerRole] The role of the partner database in the
 * replication link. Possible values include: 'Primary', 'Secondary',
 * 'NonReadableSecondary', 'Source', 'Copy'
 * @member {date} [startTime] The start time for the replication link.
 * @member {number} [percentComplete] The percentage of seeding complete for
 * the replication link.
 * @member {string} [replicationState] The replication state for the
 * replication link. Possible values include: 'PENDING', 'SEEDING', 'CATCH_UP',
 * 'SUSPENDED'
 */
export interface ReplicationLink extends ProxyResource {
  readonly location?: string;
  readonly isTerminationAllowed?: boolean;
  readonly replicationMode?: string;
  readonly partnerServer?: string;
  readonly partnerDatabase?: string;
  readonly partnerLocation?: string;
  readonly role?: string;
  readonly partnerRole?: string;
  readonly startTime?: Date;
  readonly percentComplete?: number;
  readonly replicationState?: string;
}

/**
 * @class
 * Initializes a new instance of the ServerAzureADAdministrator class.
 * @constructor
 * An server Active Directory Administrator.
 *
 * @member {string} login The server administrator login value.
 * @member {uuid} sid The server administrator Sid (Secure ID).
 * @member {uuid} tenantId The server Active Directory Administrator tenant id.
 */
export interface ServerAzureADAdministrator extends ProxyResource {
  login: string;
  sid: string;
  tenantId: string;
}

/**
 * @class
 * Initializes a new instance of the ServerCommunicationLink class.
 * @constructor
 * Server communication link.
 *
 * @member {string} [state] The state.
 * @member {string} partnerServer The name of the partner server.
 * @member {string} [location] Communication link location.
 * @member {string} [kind] Communication link kind.  This property is used for
 * Azure Portal metadata.
 */
export interface ServerCommunicationLink extends ProxyResource {
  readonly state?: string;
  partnerServer: string;
  readonly location?: string;
  readonly kind?: string;
}

/**
 * @class
 * Initializes a new instance of the ServiceObjective class.
 * @constructor
 * Represents a database service objective.
 *
 * @member {string} [serviceObjectiveName] The name for the service objective.
 * @member {boolean} [isDefault] Gets whether the service level objective is
 * the default service objective.
 * @member {boolean} [isSystem] Gets whether the service level objective is a
 * system service objective.
 * @member {string} [description] The description for the service level
 * objective.
 * @member {boolean} [enabled] Gets whether the service level objective is
 * enabled.
 */
export interface ServiceObjective extends ProxyResource {
  readonly serviceObjectiveName?: string;
  readonly isDefault?: boolean;
  readonly isSystem?: boolean;
  readonly description?: string;
  readonly enabled?: boolean;
}

/**
 * @class
 * Initializes a new instance of the CheckNameAvailabilityRequest class.
 * @constructor
 * A request to check whether the specified name for a resource is available.
 *
 * @member {string} name The name whose availability is to be checked.
 */
export interface CheckNameAvailabilityRequest {
  name: string;
}

/**
 * @class
 * Initializes a new instance of the CheckNameAvailabilityResponse class.
 * @constructor
 * A response indicating whether the specified name for a resource is
 * available.
 *
 * @member {boolean} [available] True if the name is available, otherwise
 * false.
 * @member {string} [message] A message explaining why the name is unavailable.
 * Will be null if the name is available.
 * @member {string} [name] The name whose availability was checked.
 * @member {string} [reason] The reason code explaining why the name is
 * unavailable. Will be null if the name is available. Possible values include:
 * 'Invalid', 'AlreadyExists'
 */
export interface CheckNameAvailabilityResponse {
  readonly available?: boolean;
  readonly message?: string;
  readonly name?: string;
  readonly reason?: string;
}

/**
 * @class
 * Initializes a new instance of the RecommendedElasticPoolMetric class.
 * @constructor
 * Represents recommended elastic pool metric.
 *
 * @member {date} [dateTime] The time of metric (ISO8601 format).
 * @member {number} [dtu] Gets or sets the DTUs (Database Transaction Units).
 * See
 * https://azure.microsoft.com/documentation/articles/sql-database-what-is-a-dtu/
 * @member {number} [sizeGB] Gets or sets size in gigabytes.
 */
export interface RecommendedElasticPoolMetric {
  dateTime?: Date;
  dtu?: number;
  sizeGB?: number;
}

/**
 * @class
 * Initializes a new instance of the SloUsageMetric class.
 * @constructor
 * A Slo Usage Metric.
 *
 * @member {string} [serviceLevelObjective] The serviceLevelObjective for SLO
 * usage metric. Possible values include: 'Basic', 'S0', 'S1', 'S2', 'S3',
 * 'P1', 'P2', 'P3', 'P4', 'P6', 'P11', 'P15', 'System', 'System2',
 * 'ElasticPool'
 * @member {uuid} [serviceLevelObjectiveId] The serviceLevelObjectiveId for SLO
 * usage metric.
 * @member {number} [inRangeTimeRatio] Gets or sets inRangeTimeRatio for SLO
 * usage metric.
 */
export interface SloUsageMetric {
  readonly serviceLevelObjective?: string;
  readonly serviceLevelObjectiveId?: string;
  readonly inRangeTimeRatio?: number;
}

/**
 * @class
 * Initializes a new instance of the ServiceTierAdvisor class.
 * @constructor
 * Represents a Service Tier Advisor.
 *
 * @member {date} [observationPeriodStart] The observation period start
 * (ISO8601 format).
 * @member {date} [observationPeriodEnd] The observation period start (ISO8601
 * format).
 * @member {number} [activeTimeRatio] The activeTimeRatio for service tier
 * advisor.
 * @member {number} [minDtu] Gets or sets minDtu for service tier advisor.
 * @member {number} [avgDtu] Gets or sets avgDtu for service tier advisor.
 * @member {number} [maxDtu] Gets or sets maxDtu for service tier advisor.
 * @member {number} [maxSizeInGB] Gets or sets maxSizeInGB for service tier
 * advisor.
 * @member {array} [serviceLevelObjectiveUsageMetrics] Gets or sets
 * serviceLevelObjectiveUsageMetrics for the service tier advisor.
 * @member {string} [currentServiceLevelObjective] Gets or sets
 * currentServiceLevelObjective for service tier advisor.
 * @member {uuid} [currentServiceLevelObjectiveId] Gets or sets
 * currentServiceLevelObjectiveId for service tier advisor.
 * @member {string} [usageBasedRecommendationServiceLevelObjective] Gets or
 * sets usageBasedRecommendationServiceLevelObjective for service tier advisor.
 * @member {uuid} [usageBasedRecommendationServiceLevelObjectiveId] Gets or
 * sets usageBasedRecommendationServiceLevelObjectiveId for service tier
 * advisor.
 * @member {string} [databaseSizeBasedRecommendationServiceLevelObjective] Gets
 * or sets databaseSizeBasedRecommendationServiceLevelObjective for service
 * tier advisor.
 * @member {uuid} [databaseSizeBasedRecommendationServiceLevelObjectiveId] Gets
 * or sets databaseSizeBasedRecommendationServiceLevelObjectiveId for service
 * tier advisor.
 * @member {string} [disasterPlanBasedRecommendationServiceLevelObjective] Gets
 * or sets disasterPlanBasedRecommendationServiceLevelObjective for service
 * tier advisor.
 * @member {uuid} [disasterPlanBasedRecommendationServiceLevelObjectiveId] Gets
 * or sets disasterPlanBasedRecommendationServiceLevelObjectiveId for service
 * tier advisor.
 * @member {string} [overallRecommendationServiceLevelObjective] Gets or sets
 * overallRecommendationServiceLevelObjective for service tier advisor.
 * @member {uuid} [overallRecommendationServiceLevelObjectiveId] Gets or sets
 * overallRecommendationServiceLevelObjectiveId for service tier advisor.
 * @member {number} [confidence] Gets or sets confidence for service tier
 * advisor.
 */
export interface ServiceTierAdvisor extends ProxyResource {
  readonly observationPeriodStart?: Date;
  readonly observationPeriodEnd?: Date;
  readonly activeTimeRatio?: number;
  readonly minDtu?: number;
  readonly avgDtu?: number;
  readonly maxDtu?: number;
  readonly maxSizeInGB?: number;
  readonly serviceLevelObjectiveUsageMetrics?: SloUsageMetric[];
  readonly currentServiceLevelObjective?: string;
  readonly currentServiceLevelObjectiveId?: string;
  readonly usageBasedRecommendationServiceLevelObjective?: string;
  readonly usageBasedRecommendationServiceLevelObjectiveId?: string;
  readonly databaseSizeBasedRecommendationServiceLevelObjective?: string;
  readonly databaseSizeBasedRecommendationServiceLevelObjectiveId?: string;
  readonly disasterPlanBasedRecommendationServiceLevelObjective?: string;
  readonly disasterPlanBasedRecommendationServiceLevelObjectiveId?: string;
  readonly overallRecommendationServiceLevelObjective?: string;
  readonly overallRecommendationServiceLevelObjectiveId?: string;
  readonly confidence?: number;
}

/**
 * @class
 * Initializes a new instance of the TransparentDataEncryption class.
 * @constructor
 * Represents a database transparent data encryption configuration.
 *
 * @member {string} [location] Resource location.
 * @member {string} [status] The status of the database transparent data
 * encryption. Possible values include: 'Enabled', 'Disabled'
 */
export interface TransparentDataEncryption extends ProxyResource {
  readonly location?: string;
  status?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationImpact class.
 * @constructor
 * The impact of an operation, both in absolute and relative terms.
 *
 * @member {string} [name] The name of the impact dimension.
 * @member {string} [unit] The unit in which estimated impact to dimension is
 * measured.
 * @member {number} [changeValueAbsolute] The absolute impact to dimension.
 * @member {number} [changeValueRelative] The relative impact to dimension
 * (null if not applicable)
 */
export interface OperationImpact {
  readonly name?: string;
  readonly unit?: string;
  readonly changeValueAbsolute?: number;
  readonly changeValueRelative?: number;
}

/**
 * @class
 * Initializes a new instance of the RecommendedIndex class.
 * @constructor
 * Represents a database recommended index.
 *
 * @member {string} [action] The proposed index action. You can create a
 * missing index, drop an unused index, or rebuild an existing index to improve
 * its performance. Possible values include: 'Create', 'Drop', 'Rebuild'
 * @member {string} [state] The current recommendation state. Possible values
 * include: 'Active', 'Pending', 'Executing', 'Verifying', 'Pending Revert',
 * 'Reverting', 'Reverted', 'Ignored', 'Expired', 'Blocked', 'Success'
 * @member {date} [created] The UTC datetime showing when this resource was
 * created (ISO8601 format).
 * @member {date} [lastModified] The UTC datetime of when was this resource
 * last changed (ISO8601 format).
 * @member {string} [indexType] The type of index (CLUSTERED, NONCLUSTERED,
 * COLUMNSTORE, CLUSTERED COLUMNSTORE). Possible values include: 'CLUSTERED',
 * 'NONCLUSTERED', 'COLUMNSTORE', 'CLUSTERED COLUMNSTORE'
 * @member {string} [schema] The schema where table to build index over resides
 * @member {string} [table] The table on which to build index.
 * @member {array} [columns] Columns over which to build index
 * @member {array} [includedColumns] The list of column names to be included in
 * the index
 * @member {string} [indexScript] The full build index script
 * @member {array} [estimatedImpact] The estimated impact of doing recommended
 * index action.
 * @member {array} [reportedImpact] The values reported after index action is
 * complete.
 */
export interface RecommendedIndex extends ProxyResource {
  readonly action?: string;
  readonly state?: string;
  readonly created?: Date;
  readonly lastModified?: Date;
  readonly indexType?: string;
  readonly schema?: string;
  readonly table?: string;
  readonly columns?: string[];
  readonly includedColumns?: string[];
  readonly indexScript?: string;
  readonly estimatedImpact?: OperationImpact[];
  readonly reportedImpact?: OperationImpact[];
}

/**
 * @class
 * Initializes a new instance of the Database class.
 * @constructor
 * Represents a database.
 *
 * @member {string} [kind] Kind of database.  This is metadata used for the
 * Azure portal experience.
 * @member {string} [collation] The collation of the database. If createMode is
 * not Default, this value is ignored.
 * @member {date} [creationDate] The creation date of the database (ISO8601
 * format).
 * @member {number} [containmentState] The containment state of the database.
 * @member {uuid} [currentServiceObjectiveId] The current service level
 * objective ID of the database. This is the ID of the service level objective
 * that is currently active.
 * @member {uuid} [databaseId] The ID of the database.
 * @member {date} [earliestRestoreDate] This records the earliest start date
 * and time that restore is available for this database (ISO8601 format).
 * @member {string} [createMode] Specifies the mode of database creation.
 *
 * Default: regular database creation.
 *
 * Copy: creates a database as a copy of an existing database. sourceDatabaseId
 * must be specified as the resource ID of the source database.
 *
 * OnlineSecondary/NonReadableSecondary: creates a database as a (readable or
 * nonreadable) secondary replica of an existing database. sourceDatabaseId
 * must be specified as the resource ID of the existing primary database.
 *
 * PointInTimeRestore: Creates a database by restoring a point in time backup
 * of an existing database. sourceDatabaseId must be specified as the resource
 * ID of the existing database, and restorePointInTime must be specified.
 *
 * Recovery: Creates a database by restoring a geo-replicated backup.
 * sourceDatabaseId must be specified as the recoverable database resource ID
 * to restore.
 *
 * Restore: Creates a database by restoring a backup of a deleted database.
 * sourceDatabaseId must be specified. If sourceDatabaseId is the database's
 * original resource ID, then sourceDatabaseDeletionDate must be specified.
 * Otherwise sourceDatabaseId must be the restorable dropped database resource
 * ID and sourceDatabaseDeletionDate is ignored. restorePointInTime may also be
 * specified to restore from an earlier point in time.
 *
 * RestoreLongTermRetentionBackup: Creates a database by restoring from a long
 * term retention vault. recoveryServicesRecoveryPointResourceId must be
 * specified as the recovery point resource ID.
 *
 * Copy, NonReadableSecondary, OnlineSecondary and
 * RestoreLongTermRetentionBackup are not supported for DataWarehouse edition.
 * Possible values include: 'Copy', 'Default', 'NonReadableSecondary',
 * 'OnlineSecondary', 'PointInTimeRestore', 'Recovery', 'Restore',
 * 'RestoreLongTermRetentionBackup'
 * @member {string} [sourceDatabaseId] Conditional. If createMode is Copy,
 * NonReadableSecondary, OnlineSecondary, PointInTimeRestore, Recovery, or
 * Restore, then this value is required. Specifies the resource ID of the
 * source database. If createMode is NonReadableSecondary or OnlineSecondary,
 * the name of the source database must be the same as the new database being
 * created.
 * @member {date} [sourceDatabaseDeletionDate] Conditional. If createMode is
 * Restore and sourceDatabaseId is the deleted database's original resource id
 * when it existed (as opposed to its current restorable dropped database id),
 * then this value is required. Specifies the time that the database was
 * deleted.
 * @member {date} [restorePointInTime] Conditional. If createMode is
 * PointInTimeRestore, this value is required. If createMode is Restore, this
 * value is optional. Specifies the point in time (ISO8601 format) of the
 * source database that will be restored to create the new database. Must be
 * greater than or equal to the source database's earliestRestoreDate value.
 * @member {string} [recoveryServicesRecoveryPointResourceId] Conditional. If
 * createMode is RestoreLongTermRetentionBackup, then this value is required.
 * Specifies the resource ID of the recovery point to restore from.
 * @member {string} [edition] The edition of the database. The DatabaseEditions
 * enumeration contains all the valid editions. If createMode is
 * NonReadableSecondary or OnlineSecondary, this value is ignored. To see
 * possible values, query the capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation.". Possible values
 * include: 'Web', 'Business', 'Basic', 'Standard', 'Premium', 'Free',
 * 'Stretch', 'DataWarehouse', 'System', 'System2'
 * @member {string} [maxSizeBytes] The max size of the database expressed in
 * bytes. If createMode is not Default, this value is ignored. To see possible
 * values, query the capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation."
 * @member {uuid} [requestedServiceObjectiveId] The configured service level
 * objective ID of the database. This is the service level objective that is in
 * the process of being applied to the database. Once successfully updated, it
 * will match the value of currentServiceObjectiveId property. If
 * requestedServiceObjectiveId and requestedServiceObjectiveName are both
 * updated, the value of requestedServiceObjectiveId overrides the value of
 * requestedServiceObjectiveName. To see possible values, query the
 * capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation."
 * @member {string} [requestedServiceObjectiveName] The name of the configured
 * service level objective of the database. This is the service level objective
 * that is in the process of being applied to the database. Once successfully
 * updated, it will match the value of serviceLevelObjective property. To see
 * possible values, query the capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation.". Possible values
 * include: 'Basic', 'S0', 'S1', 'S2', 'S3', 'P1', 'P2', 'P3', 'P4', 'P6',
 * 'P11', 'P15', 'System', 'System2', 'ElasticPool'
 * @member {string} [serviceLevelObjective] The current service level objective
 * of the database. Possible values include: 'Basic', 'S0', 'S1', 'S2', 'S3',
 * 'P1', 'P2', 'P3', 'P4', 'P6', 'P11', 'P15', 'System', 'System2',
 * 'ElasticPool'
 * @member {string} [status] The status of the database.
 * @member {string} [elasticPoolName] The name of the elastic pool the database
 * is in. If elasticPoolName and requestedServiceObjectiveName are both
 * updated, the value of requestedServiceObjectiveName is ignored. Not
 * supported for DataWarehouse edition.
 * @member {string} [defaultSecondaryLocation] The default secondary region for
 * this database.
 * @member {array} [serviceTierAdvisors] The list of service tier advisors for
 * this database. Expanded property
 * @member {array} [transparentDataEncryption] The transparent data encryption
 * info for this database.
 * @member {array} [recommendedIndex] The recommended indices for this
 * database.
 * @member {string} [failoverGroupId] The resource identifier of the failover
 * group containing this database.
 * @member {string} [readScale] Conditional. If the database is a
 * geo-secondary, readScale indicates whether read-only connections are allowed
 * to this database or not. Not supported for DataWarehouse edition. Possible
 * values include: 'Enabled', 'Disabled'
 * @member {string} [sampleName] Indicates the name of the sample schema to
 * apply when creating this database. If createMode is not Default, this value
 * is ignored. Not supported for DataWarehouse edition. Possible values
 * include: 'AdventureWorksLT'
 */
export interface Database extends TrackedResource {
  readonly kind?: string;
  collation?: string;
  readonly creationDate?: Date;
  readonly containmentState?: number;
  readonly currentServiceObjectiveId?: string;
  readonly databaseId?: string;
  readonly earliestRestoreDate?: Date;
  createMode?: string;
  sourceDatabaseId?: string;
  sourceDatabaseDeletionDate?: Date;
  restorePointInTime?: Date;
  recoveryServicesRecoveryPointResourceId?: string;
  edition?: string;
  maxSizeBytes?: string;
  requestedServiceObjectiveId?: string;
  requestedServiceObjectiveName?: string;
  readonly serviceLevelObjective?: string;
  readonly status?: string;
  elasticPoolName?: string;
  readonly defaultSecondaryLocation?: string;
  readonly serviceTierAdvisors?: ServiceTierAdvisor[];
  readonly transparentDataEncryption?: TransparentDataEncryption[];
  readonly recommendedIndex?: RecommendedIndex[];
  readonly failoverGroupId?: string;
  readScale?: string;
  sampleName?: string;
}

/**
 * @class
 * Initializes a new instance of the RecommendedElasticPool class.
 * @constructor
 * Represents a recommented elastic pool.
 *
 * @member {string} [databaseEdition] The edition of the recommended elastic
 * pool. The ElasticPoolEdition enumeration contains all the valid editions.
 * Possible values include: 'Basic', 'Standard', 'Premium'
 * @member {number} [dtu] The DTU for the recommended elastic pool.
 * @member {number} [databaseDtuMin] The minimum DTU for the database.
 * @member {number} [databaseDtuMax] The maximum DTU for the database.
 * @member {number} [storageMB] Gets storage size in megabytes.
 * @member {date} [observationPeriodStart] The observation period start
 * (ISO8601 format).
 * @member {date} [observationPeriodEnd] The observation period start (ISO8601
 * format).
 * @member {number} [maxObservedDtu] Gets maximum observed DTU.
 * @member {number} [maxObservedStorageMB] Gets maximum observed storage in
 * megabytes.
 * @member {array} [databases] The list of databases in this pool. Expanded
 * property
 * @member {array} [metrics] The list of databases housed in the server.
 * Expanded property
 */
export interface RecommendedElasticPool extends ProxyResource {
  readonly databaseEdition?: string;
  dtu?: number;
  databaseDtuMin?: number;
  databaseDtuMax?: number;
  storageMB?: number;
  readonly observationPeriodStart?: Date;
  readonly observationPeriodEnd?: Date;
  readonly maxObservedDtu?: number;
  readonly maxObservedStorageMB?: number;
  readonly databases?: Database[];
  readonly metrics?: RecommendedElasticPoolMetric[];
}

/**
 * @class
 * Initializes a new instance of the ElasticPool class.
 * @constructor
 * Represents a database elastic pool.
 *
 * @member {date} [creationDate] The creation date of the elastic pool (ISO8601
 * format).
 * @member {string} [state] The state of the elastic pool. Possible values
 * include: 'Creating', 'Ready', 'Disabled'
 * @member {string} [edition] The edition of the elastic pool. Possible values
 * include: 'Basic', 'Standard', 'Premium'
 * @member {number} [dtu] The total shared DTU for the database elastic pool.
 * @member {number} [databaseDtuMax] The maximum DTU any one database can
 * consume.
 * @member {number} [databaseDtuMin] The minimum DTU all databases are
 * guaranteed.
 * @member {number} [storageMB] Gets storage limit for the database elastic
 * pool in MB.
 * @member {string} [kind] Kind of elastic pool.  This is metadata used for the
 * Azure portal experience.
 */
export interface ElasticPool extends TrackedResource {
  readonly creationDate?: Date;
  readonly state?: string;
  edition?: string;
  dtu?: number;
  databaseDtuMax?: number;
  databaseDtuMin?: number;
  storageMB?: number;
  readonly kind?: string;
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolUpdate class.
 * @constructor
 * Represents an elastic pool update.
 *
 * @member {object} [tags] Resource tags.
 * @member {date} [creationDate] The creation date of the elastic pool (ISO8601
 * format).
 * @member {string} [state] The state of the elastic pool. Possible values
 * include: 'Creating', 'Ready', 'Disabled'
 * @member {string} [edition] The edition of the elastic pool. Possible values
 * include: 'Basic', 'Standard', 'Premium'
 * @member {number} [dtu] The total shared DTU for the database elastic pool.
 * @member {number} [databaseDtuMax] The maximum DTU any one database can
 * consume.
 * @member {number} [databaseDtuMin] The minimum DTU all databases are
 * guaranteed.
 * @member {number} [storageMB] Gets storage limit for the database elastic
 * pool in MB.
 */
export interface ElasticPoolUpdate extends Resource {
  tags?: { [propertyName: string]: string };
  readonly creationDate?: Date;
  readonly state?: string;
  edition?: string;
  dtu?: number;
  databaseDtuMax?: number;
  databaseDtuMin?: number;
  storageMB?: number;
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolActivity class.
 * @constructor
 * Represents the activity on an elastic pool.
 *
 * @member {string} [location] The geo-location where the resource lives
 * @member {date} [endTime] The time the operation finished (ISO8601 format).
 * @member {number} [errorCode] The error code if available.
 * @member {string} [errorMessage] The error message if available.
 * @member {number} [errorSeverity] The error severity if available.
 * @member {string} [operation] The operation name.
 * @member {uuid} [operationId] The unique operation ID.
 * @member {number} [percentComplete] The percentage complete if available.
 * @member {number} [requestedDatabaseDtuMax] The requested max DTU per
 * database if available.
 * @member {number} [requestedDatabaseDtuMin] The requested min DTU per
 * database if available.
 * @member {number} [requestedDtu] The requested DTU for the pool if available.
 * @member {string} [requestedElasticPoolName] The requested name for the
 * elastic pool if available.
 * @member {number} [requestedStorageLimitInGB] The requested storage limit for
 * the pool in GB if available.
 * @member {string} [elasticPoolName] The name of the elastic pool.
 * @member {string} [serverName] The name of the server the elastic pool is in.
 * @member {date} [startTime] The time the operation started (ISO8601 format).
 * @member {string} [state] The current state of the operation.
 * @member {number} [requestedStorageLimitInMB] The requested storage limit in
 * MB.
 * @member {number} [requestedDatabaseDtuGuarantee] The requested per database
 * DTU guarantee.
 * @member {number} [requestedDatabaseDtuCap] The requested per database DTU
 * cap.
 * @member {number} [requestedDtuGuarantee] The requested DTU guarantee.
 */
export interface ElasticPoolActivity extends ProxyResource {
  location?: string;
  readonly endTime?: Date;
  readonly errorCode?: number;
  readonly errorMessage?: string;
  readonly errorSeverity?: number;
  readonly operation?: string;
  readonly operationId?: string;
  readonly percentComplete?: number;
  readonly requestedDatabaseDtuMax?: number;
  readonly requestedDatabaseDtuMin?: number;
  readonly requestedDtu?: number;
  readonly requestedElasticPoolName?: string;
  readonly requestedStorageLimitInGB?: number;
  readonly elasticPoolName?: string;
  readonly serverName?: string;
  readonly startTime?: Date;
  readonly state?: string;
  readonly requestedStorageLimitInMB?: number;
  readonly requestedDatabaseDtuGuarantee?: number;
  readonly requestedDatabaseDtuCap?: number;
  readonly requestedDtuGuarantee?: number;
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolDatabaseActivity class.
 * @constructor
 * Represents the activity on an elastic pool.
 *
 * @member {string} [location] The geo-location where the resource lives
 * @member {string} [databaseName] The database name.
 * @member {date} [endTime] The time the operation finished (ISO8601 format).
 * @member {number} [errorCode] The error code if available.
 * @member {string} [errorMessage] The error message if available.
 * @member {number} [errorSeverity] The error severity if available.
 * @member {string} [operation] The operation name.
 * @member {uuid} [operationId] The unique operation ID.
 * @member {number} [percentComplete] The percentage complete if available.
 * @member {string} [requestedElasticPoolName] The name for the elastic pool
 * the database is moving into if available.
 * @member {string} [currentElasticPoolName] The name of the current elastic
 * pool the database is in if available.
 * @member {string} [currentServiceObjective] The name of the current service
 * objective if available.
 * @member {string} [requestedServiceObjective] The name of the requested
 * service objective if available.
 * @member {string} [serverName] The name of the server the elastic pool is in.
 * @member {date} [startTime] The time the operation started (ISO8601 format).
 * @member {string} [state] The current state of the operation.
 */
export interface ElasticPoolDatabaseActivity extends ProxyResource {
  location?: string;
  readonly databaseName?: string;
  readonly endTime?: Date;
  readonly errorCode?: number;
  readonly errorMessage?: string;
  readonly errorSeverity?: number;
  readonly operation?: string;
  readonly operationId?: string;
  readonly percentComplete?: number;
  readonly requestedElasticPoolName?: string;
  readonly currentElasticPoolName?: string;
  readonly currentServiceObjective?: string;
  readonly requestedServiceObjective?: string;
  readonly serverName?: string;
  readonly startTime?: Date;
  readonly state?: string;
}

/**
 * @class
 * Initializes a new instance of the DatabaseUpdate class.
 * @constructor
 * Represents a database update.
 *
 * @member {object} [tags] Resource tags.
 * @member {string} [collation] The collation of the database. If createMode is
 * not Default, this value is ignored.
 * @member {date} [creationDate] The creation date of the database (ISO8601
 * format).
 * @member {number} [containmentState] The containment state of the database.
 * @member {uuid} [currentServiceObjectiveId] The current service level
 * objective ID of the database. This is the ID of the service level objective
 * that is currently active.
 * @member {uuid} [databaseId] The ID of the database.
 * @member {date} [earliestRestoreDate] This records the earliest start date
 * and time that restore is available for this database (ISO8601 format).
 * @member {string} [createMode] Specifies the mode of database creation.
 *
 * Default: regular database creation.
 *
 * Copy: creates a database as a copy of an existing database. sourceDatabaseId
 * must be specified as the resource ID of the source database.
 *
 * OnlineSecondary/NonReadableSecondary: creates a database as a (readable or
 * nonreadable) secondary replica of an existing database. sourceDatabaseId
 * must be specified as the resource ID of the existing primary database.
 *
 * PointInTimeRestore: Creates a database by restoring a point in time backup
 * of an existing database. sourceDatabaseId must be specified as the resource
 * ID of the existing database, and restorePointInTime must be specified.
 *
 * Recovery: Creates a database by restoring a geo-replicated backup.
 * sourceDatabaseId must be specified as the recoverable database resource ID
 * to restore.
 *
 * Restore: Creates a database by restoring a backup of a deleted database.
 * sourceDatabaseId must be specified. If sourceDatabaseId is the database's
 * original resource ID, then sourceDatabaseDeletionDate must be specified.
 * Otherwise sourceDatabaseId must be the restorable dropped database resource
 * ID and sourceDatabaseDeletionDate is ignored. restorePointInTime may also be
 * specified to restore from an earlier point in time.
 *
 * RestoreLongTermRetentionBackup: Creates a database by restoring from a long
 * term retention vault. recoveryServicesRecoveryPointResourceId must be
 * specified as the recovery point resource ID.
 *
 * Copy, NonReadableSecondary, OnlineSecondary and
 * RestoreLongTermRetentionBackup are not supported for DataWarehouse edition.
 * Possible values include: 'Copy', 'Default', 'NonReadableSecondary',
 * 'OnlineSecondary', 'PointInTimeRestore', 'Recovery', 'Restore',
 * 'RestoreLongTermRetentionBackup'
 * @member {string} [sourceDatabaseId] Conditional. If createMode is Copy,
 * NonReadableSecondary, OnlineSecondary, PointInTimeRestore, Recovery, or
 * Restore, then this value is required. Specifies the resource ID of the
 * source database. If createMode is NonReadableSecondary or OnlineSecondary,
 * the name of the source database must be the same as the new database being
 * created.
 * @member {date} [sourceDatabaseDeletionDate] Conditional. If createMode is
 * Restore and sourceDatabaseId is the deleted database's original resource id
 * when it existed (as opposed to its current restorable dropped database id),
 * then this value is required. Specifies the time that the database was
 * deleted.
 * @member {date} [restorePointInTime] Conditional. If createMode is
 * PointInTimeRestore, this value is required. If createMode is Restore, this
 * value is optional. Specifies the point in time (ISO8601 format) of the
 * source database that will be restored to create the new database. Must be
 * greater than or equal to the source database's earliestRestoreDate value.
 * @member {string} [recoveryServicesRecoveryPointResourceId] Conditional. If
 * createMode is RestoreLongTermRetentionBackup, then this value is required.
 * Specifies the resource ID of the recovery point to restore from.
 * @member {string} [edition] The edition of the database. The DatabaseEditions
 * enumeration contains all the valid editions. If createMode is
 * NonReadableSecondary or OnlineSecondary, this value is ignored. To see
 * possible values, query the capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation.". Possible values
 * include: 'Web', 'Business', 'Basic', 'Standard', 'Premium', 'Free',
 * 'Stretch', 'DataWarehouse', 'System', 'System2'
 * @member {string} [maxSizeBytes] The max size of the database expressed in
 * bytes. If createMode is not Default, this value is ignored. To see possible
 * values, query the capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation."
 * @member {uuid} [requestedServiceObjectiveId] The configured service level
 * objective ID of the database. This is the service level objective that is in
 * the process of being applied to the database. Once successfully updated, it
 * will match the value of currentServiceObjectiveId property. If
 * requestedServiceObjectiveId and requestedServiceObjectiveName are both
 * updated, the value of requestedServiceObjectiveId overrides the value of
 * requestedServiceObjectiveName. To see possible values, query the
 * capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation."
 * @member {string} [requestedServiceObjectiveName] The name of the configured
 * service level objective of the database. This is the service level objective
 * that is in the process of being applied to the database. Once successfully
 * updated, it will match the value of serviceLevelObjective property. To see
 * possible values, query the capabilities API
 * (/subscriptions/{subscriptionId}/providers/Microsoft.Sql/locations/{locationID}/capabilities)
 * referred to by operationId: "Capabilities_ListByLocation.". Possible values
 * include: 'Basic', 'S0', 'S1', 'S2', 'S3', 'P1', 'P2', 'P3', 'P4', 'P6',
 * 'P11', 'P15', 'System', 'System2', 'ElasticPool'
 * @member {string} [serviceLevelObjective] The current service level objective
 * of the database. Possible values include: 'Basic', 'S0', 'S1', 'S2', 'S3',
 * 'P1', 'P2', 'P3', 'P4', 'P6', 'P11', 'P15', 'System', 'System2',
 * 'ElasticPool'
 * @member {string} [status] The status of the database.
 * @member {string} [elasticPoolName] The name of the elastic pool the database
 * is in. If elasticPoolName and requestedServiceObjectiveName are both
 * updated, the value of requestedServiceObjectiveName is ignored. Not
 * supported for DataWarehouse edition.
 * @member {string} [defaultSecondaryLocation] The default secondary region for
 * this database.
 * @member {array} [serviceTierAdvisors] The list of service tier advisors for
 * this database. Expanded property
 * @member {array} [transparentDataEncryption] The transparent data encryption
 * info for this database.
 * @member {array} [recommendedIndex] The recommended indices for this
 * database.
 * @member {string} [failoverGroupId] The resource identifier of the failover
 * group containing this database.
 * @member {string} [readScale] Conditional. If the database is a
 * geo-secondary, readScale indicates whether read-only connections are allowed
 * to this database or not. Not supported for DataWarehouse edition. Possible
 * values include: 'Enabled', 'Disabled'
 * @member {string} [sampleName] Indicates the name of the sample schema to
 * apply when creating this database. If createMode is not Default, this value
 * is ignored. Not supported for DataWarehouse edition. Possible values
 * include: 'AdventureWorksLT'
 */
export interface DatabaseUpdate extends Resource {
  tags?: { [propertyName: string]: string };
  collation?: string;
  readonly creationDate?: Date;
  readonly containmentState?: number;
  readonly currentServiceObjectiveId?: string;
  readonly databaseId?: string;
  readonly earliestRestoreDate?: Date;
  createMode?: string;
  sourceDatabaseId?: string;
  sourceDatabaseDeletionDate?: Date;
  restorePointInTime?: Date;
  recoveryServicesRecoveryPointResourceId?: string;
  edition?: string;
  maxSizeBytes?: string;
  requestedServiceObjectiveId?: string;
  requestedServiceObjectiveName?: string;
  readonly serviceLevelObjective?: string;
  readonly status?: string;
  elasticPoolName?: string;
  readonly defaultSecondaryLocation?: string;
  readonly serviceTierAdvisors?: ServiceTierAdvisor[];
  readonly transparentDataEncryption?: TransparentDataEncryption[];
  readonly recommendedIndex?: RecommendedIndex[];
  readonly failoverGroupId?: string;
  readScale?: string;
  sampleName?: string;
}

/**
 * @class
 * Initializes a new instance of the TransparentDataEncryptionActivity class.
 * @constructor
 * Represents a database transparent data encryption Scan.
 *
 * @member {string} [location] Resource location.
 * @member {string} [status] The status of the database. Possible values
 * include: 'Encrypting', 'Decrypting'
 * @member {number} [percentComplete] The percent complete of the transparent
 * data encryption scan for a database.
 */
export interface TransparentDataEncryptionActivity extends ProxyResource {
  readonly location?: string;
  readonly status?: string;
  readonly percentComplete?: number;
}

/**
 * @class
 * Initializes a new instance of the ServerUsage class.
 * @constructor
 * Represents server metrics.
 *
 * @member {string} [name] Name of the server usage metric.
 * @member {string} [resourceName] The name of the resource.
 * @member {string} [displayName] The metric display name.
 * @member {number} [currentValue] The current value of the metric.
 * @member {number} [limit] The current limit of the metric.
 * @member {string} [unit] The units of the metric.
 * @member {date} [nextResetTime] The next reset time for the metric (ISO8601
 * format).
 */
export interface ServerUsage {
  readonly name?: string;
  readonly resourceName?: string;
  readonly displayName?: string;
  readonly currentValue?: number;
  readonly limit?: number;
  readonly unit?: string;
  readonly nextResetTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the DatabaseUsage class.
 * @constructor
 * The database usages.
 *
 * @member {string} [name] The name of the usage metric.
 * @member {string} [resourceName] The name of the resource.
 * @member {string} [displayName] The usage metric display name.
 * @member {number} [currentValue] The current value of the usage metric.
 * @member {number} [limit] The current limit of the usage metric.
 * @member {string} [unit] The units of the usage metric.
 * @member {date} [nextResetTime] The next reset time for the usage metric
 * (ISO8601 format).
 */
export interface DatabaseUsage {
  readonly name?: string;
  readonly resourceName?: string;
  readonly displayName?: string;
  readonly currentValue?: number;
  readonly limit?: number;
  readonly unit?: string;
  readonly nextResetTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the DatabaseBlobAuditingPolicy class.
 * @constructor
 * A database blob auditing policy.
 *
 * @member {string} [kind] Resource kind.
 * @member {string} state Specifies the state of the policy. If state is
 * Enabled, storageEndpoint and storageAccountAccessKey are required. Possible
 * values include: 'Enabled', 'Disabled'
 * @member {string} [storageEndpoint] Specifies the blob storage endpoint (e.g.
 * https://MyAccount.blob.core.windows.net). If state is Enabled,
 * storageEndpoint is required.
 * @member {string} [storageAccountAccessKey] Specifies the identifier key of
 * the auditing storage account. If state is Enabled, storageAccountAccessKey
 * is required.
 * @member {number} [retentionDays] Specifies the number of days to keep in the
 * audit logs.
 * @member {array} [auditActionsAndGroups] Specifies the Actions and
 * Actions-Groups to audit.
 * @member {uuid} [storageAccountSubscriptionId] Specifies the blob storage
 * subscription Id.
 * @member {boolean} [isStorageSecondaryKeyInUse] Specifies whether
 * storageAccountAccessKey value is the storage’s secondary key.
 */
export interface DatabaseBlobAuditingPolicy extends ProxyResource {
  readonly kind?: string;
  state: string;
  storageEndpoint?: string;
  storageAccountAccessKey?: string;
  retentionDays?: number;
  auditActionsAndGroups?: string[];
  storageAccountSubscriptionId?: string;
  isStorageSecondaryKeyInUse?: boolean;
}

/**
 * @class
 * Initializes a new instance of the EncryptionProtector class.
 * @constructor
 * The server encryption protector.
 *
 * @member {string} [kind] Kind of encryption protector. This is metadata used
 * for the Azure portal experience.
 * @member {string} [location] Resource location.
 * @member {string} [subregion] Subregion of the encryption protector.
 * @member {string} [serverKeyName] The name of the server key.
 * @member {string} serverKeyType The encryption protector type like
 * 'ServiceManaged', 'AzureKeyVault'. Possible values include:
 * 'ServiceManaged', 'AzureKeyVault'
 * @member {string} [uri] The URI of the server key.
 * @member {string} [thumbprint] Thumbprint of the server key.
 */
export interface EncryptionProtector extends ProxyResource {
  kind?: string;
  readonly location?: string;
  readonly subregion?: string;
  serverKeyName?: string;
  serverKeyType: string;
  readonly uri?: string;
  readonly thumbprint?: string;
}

/**
 * @class
 * Initializes a new instance of the FailoverGroupReadWriteEndpoint class.
 * @constructor
 * Read-write endpoint of the failover group instance.
 *
 * @member {string} failoverPolicy Failover policy of the read-write endpoint
 * for the failover group. If failoverPolicy is Automatic then
 * failoverWithDataLossGracePeriodMinutes is required. Possible values include:
 * 'Manual', 'Automatic'
 * @member {number} [failoverWithDataLossGracePeriodMinutes] Grace period
 * before failover with data loss is attempted for the read-write endpoint. If
 * failoverPolicy is Automatic then failoverWithDataLossGracePeriodMinutes is
 * required.
 */
export interface FailoverGroupReadWriteEndpoint {
  failoverPolicy: string;
  failoverWithDataLossGracePeriodMinutes?: number;
}

/**
 * @class
 * Initializes a new instance of the FailoverGroupReadOnlyEndpoint class.
 * @constructor
 * Read-only endpoint of the failover group instance.
 *
 * @member {string} [failoverPolicy] Failover policy of the read-only endpoint
 * for the failover group. Possible values include: 'Disabled', 'Enabled'
 */
export interface FailoverGroupReadOnlyEndpoint {
  failoverPolicy?: string;
}

/**
 * @class
 * Initializes a new instance of the PartnerInfo class.
 * @constructor
 * Partner server information for the failover group.
 *
 * @member {string} id Resource identifier of the partner server.
 * @member {string} [location] Geo location of the partner server.
 * @member {string} [replicationRole] Replication role of the partner server.
 * Possible values include: 'Primary', 'Secondary'
 */
export interface PartnerInfo {
  id: string;
  readonly location?: string;
  readonly replicationRole?: string;
}

/**
 * @class
 * Initializes a new instance of the FailoverGroup class.
 * @constructor
 * A failover group.
 *
 * @member {string} [location] Resource location.
 * @member {object} [tags] Resource tags.
 * @member {object} readWriteEndpoint Read-write endpoint of the failover group
 * instance.
 * @member {string} [readWriteEndpoint.failoverPolicy] Failover policy of the
 * read-write endpoint for the failover group. If failoverPolicy is Automatic
 * then failoverWithDataLossGracePeriodMinutes is required. Possible values
 * include: 'Manual', 'Automatic'
 * @member {number} [readWriteEndpoint.failoverWithDataLossGracePeriodMinutes]
 * Grace period before failover with data loss is attempted for the read-write
 * endpoint. If failoverPolicy is Automatic then
 * failoverWithDataLossGracePeriodMinutes is required.
 * @member {object} [readOnlyEndpoint] Read-only endpoint of the failover group
 * instance.
 * @member {string} [readOnlyEndpoint.failoverPolicy] Failover policy of the
 * read-only endpoint for the failover group. Possible values include:
 * 'Disabled', 'Enabled'
 * @member {string} [replicationRole] Local replication role of the failover
 * group instance. Possible values include: 'Primary', 'Secondary'
 * @member {string} [replicationState] Replication state of the failover group
 * instance.
 * @member {array} partnerServers List of partner server information for the
 * failover group.
 * @member {array} [databases] List of databases in the failover group.
 */
export interface FailoverGroup extends ProxyResource {
  readonly location?: string;
  tags?: { [propertyName: string]: string };
  readWriteEndpoint: FailoverGroupReadWriteEndpoint;
  readOnlyEndpoint?: FailoverGroupReadOnlyEndpoint;
  readonly replicationRole?: string;
  readonly replicationState?: string;
  partnerServers: PartnerInfo[];
  databases?: string[];
}

/**
 * @class
 * Initializes a new instance of the FailoverGroupUpdate class.
 * @constructor
 * A failover group update request.
 *
 * @member {object} [readWriteEndpoint] Read-write endpoint of the failover
 * group instance.
 * @member {string} [readWriteEndpoint.failoverPolicy] Failover policy of the
 * read-write endpoint for the failover group. If failoverPolicy is Automatic
 * then failoverWithDataLossGracePeriodMinutes is required. Possible values
 * include: 'Manual', 'Automatic'
 * @member {number} [readWriteEndpoint.failoverWithDataLossGracePeriodMinutes]
 * Grace period before failover with data loss is attempted for the read-write
 * endpoint. If failoverPolicy is Automatic then
 * failoverWithDataLossGracePeriodMinutes is required.
 * @member {object} [readOnlyEndpoint] Read-only endpoint of the failover group
 * instance.
 * @member {string} [readOnlyEndpoint.failoverPolicy] Failover policy of the
 * read-only endpoint for the failover group. Possible values include:
 * 'Disabled', 'Enabled'
 * @member {array} [databases] List of databases in the failover group.
 * @member {object} [tags] Resource tags.
 */
export interface FailoverGroupUpdate {
  readWriteEndpoint?: FailoverGroupReadWriteEndpoint;
  readOnlyEndpoint?: FailoverGroupReadOnlyEndpoint;
  databases?: string[];
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the OperationDisplay class.
 * @constructor
 * Display metadata associated with the operation.
 *
 * @member {string} [provider] The localized friendly form of the resource
 * provider name.
 * @member {string} [resource] The localized friendly form of the resource type
 * related to this action/operation.
 * @member {string} [operation] The localized friendly name for the operation.
 * @member {string} [description] The localized friendly description for the
 * operation.
 */
export interface OperationDisplay {
  readonly provider?: string;
  readonly resource?: string;
  readonly operation?: string;
  readonly description?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * SQL REST API operation definition.
 *
 * @member {string} [name] The name of the operation being performed on this
 * particular object.
 * @member {object} [display] The localized display information for this
 * particular operation / action.
 * @member {string} [display.provider] The localized friendly form of the
 * resource provider name.
 * @member {string} [display.resource] The localized friendly form of the
 * resource type related to this action/operation.
 * @member {string} [display.operation] The localized friendly name for the
 * operation.
 * @member {string} [display.description] The localized friendly description
 * for the operation.
 * @member {string} [origin] The intended executor of the operation. Possible
 * values include: 'user', 'system'
 * @member {object} [properties] Additional descriptions for the operation.
 */
export interface Operation {
  readonly name?: string;
  readonly display?: OperationDisplay;
  readonly origin?: string;
  readonly properties?: { [propertyName: string]: any };
}

/**
 * @class
 * Initializes a new instance of the ServerKey class.
 * @constructor
 * A server key.
 *
 * @member {string} [kind] Kind of encryption protector. This is metadata used
 * for the Azure portal experience.
 * @member {string} [location] Resource location.
 * @member {string} [subregion] Subregion of the server key.
 * @member {string} serverKeyType The server key type like 'ServiceManaged',
 * 'AzureKeyVault'. Possible values include: 'ServiceManaged', 'AzureKeyVault'
 * @member {string} [uri] The URI of the server key.
 * @member {string} [thumbprint] Thumbprint of the server key.
 * @member {date} [creationDate] The server key creation date.
 */
export interface ServerKey extends ProxyResource {
  kind?: string;
  readonly location?: string;
  readonly subregion?: string;
  serverKeyType: string;
  uri?: string;
  thumbprint?: string;
  creationDate?: Date;
}

/**
 * @class
 * Initializes a new instance of the ResourceIdentity class.
 * @constructor
 * Azure Active Directory identity configuration for a resource.
 *
 * @member {uuid} [principalId] The Azure Active Directory principal id.
 * @member {string} [type] The identity type. Set this to 'SystemAssigned' in
 * order to automatically create and assign an Azure Active Directory principal
 * for the resource. Possible values include: 'SystemAssigned'
 * @member {uuid} [tenantId] The Azure Active Directory tenant id.
 */
export interface ResourceIdentity {
  readonly principalId?: string;
  type?: string;
  readonly tenantId?: string;
}

/**
 * @class
 * Initializes a new instance of the Server class.
 * @constructor
 * An Azure SQL Database server.
 *
 * @member {object} [identity] The Azure Active Directory identity of the
 * server.
 * @member {uuid} [identity.principalId] The Azure Active Directory principal
 * id.
 * @member {string} [identity.type] The identity type. Set this to
 * 'SystemAssigned' in order to automatically create and assign an Azure Active
 * Directory principal for the resource. Possible values include:
 * 'SystemAssigned'
 * @member {uuid} [identity.tenantId] The Azure Active Directory tenant id.
 * @member {string} [kind] Kind of sql server. This is metadata used for the
 * Azure portal experience.
 * @member {string} [administratorLogin] Administrator username for the server.
 * Once created it cannot be changed.
 * @member {string} [administratorLoginPassword] The administrator login
 * password (required for server creation).
 * @member {string} [version] The version of the server.
 * @member {string} [state] The state of the server.
 * @member {string} [fullyQualifiedDomainName] The fully qualified domain name
 * of the server.
 */
export interface Server extends TrackedResource {
  identity?: ResourceIdentity;
  readonly kind?: string;
  administratorLogin?: string;
  administratorLoginPassword?: string;
  version?: string;
  readonly state?: string;
  readonly fullyQualifiedDomainName?: string;
}

/**
 * @class
 * Initializes a new instance of the ServerUpdate class.
 * @constructor
 * An update request for an Azure SQL Database server.
 *
 * @member {string} [administratorLogin] Administrator username for the server.
 * Once created it cannot be changed.
 * @member {string} [administratorLoginPassword] The administrator login
 * password (required for server creation).
 * @member {string} [version] The version of the server.
 * @member {string} [state] The state of the server.
 * @member {string} [fullyQualifiedDomainName] The fully qualified domain name
 * of the server.
 * @member {object} [tags] Resource tags.
 */
export interface ServerUpdate {
  administratorLogin?: string;
  administratorLoginPassword?: string;
  version?: string;
  readonly state?: string;
  readonly fullyQualifiedDomainName?: string;
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the SyncAgent class.
 * @constructor
 * An Azure SQL Database sync agent.
 *
 * @member {string} [syncAgentName] Name of the sync agent.
 * @member {string} [syncDatabaseId] ARM resource id of the sync database in
 * the sync agent.
 * @member {date} [lastAliveTime] Last alive time of the sync agent.
 * @member {string} [state] State of the sync agent. Possible values include:
 * 'Online', 'Offline', 'NeverConnected'
 * @member {boolean} [isUpToDate] If the sync agent version is up to date.
 * @member {date} [expiryTime] Expiration time of the sync agent version.
 * @member {string} [version] Version of the sync agent.
 */
export interface SyncAgent extends ProxyResource {
  readonly syncAgentName?: string;
  syncDatabaseId?: string;
  readonly lastAliveTime?: Date;
  readonly state?: string;
  readonly isUpToDate?: boolean;
  readonly expiryTime?: Date;
  readonly version?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncAgentKeyProperties class.
 * @constructor
 * Properties of an Azure SQL Database sync agent key.
 *
 * @member {string} [syncAgentKey] Key of sync agent.
 */
export interface SyncAgentKeyProperties {
  readonly syncAgentKey?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncAgentLinkedDatabase class.
 * @constructor
 * An Azure SQL Database sync agent linked database.
 *
 * @member {string} [databaseType] Type of the sync agent linked database.
 * Possible values include: 'AzureSqlDatabase', 'SqlServerDatabase'
 * @member {string} [databaseId] Id of the sync agent linked database.
 * @member {string} [description] Description of the sync agent linked
 * database.
 * @member {string} [serverName] Server name of the sync agent linked database.
 * @member {string} [databaseName] Database name of the sync agent linked
 * database.
 * @member {string} [userName] User name of the sync agent linked database.
 */
export interface SyncAgentLinkedDatabase extends ProxyResource {
  readonly databaseType?: string;
  readonly databaseId?: string;
  readonly description?: string;
  readonly serverName?: string;
  readonly databaseName?: string;
  readonly userName?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncDatabaseIdProperties class.
 * @constructor
 * Properties of the sync database id.
 *
 * @member {string} [id] ARM resource id of sync database.
 */
export interface SyncDatabaseIdProperties {
  readonly id?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncFullSchemaTableColumn class.
 * @constructor
 * Properties of the column in the table of database full schema.
 *
 * @member {string} [dataSize] Data size of the column.
 * @member {string} [dataType] Data type of the column.
 * @member {string} [errorId] Error id of the column.
 * @member {boolean} [hasError] If there is error in the table.
 * @member {boolean} [isPrimaryKey] If it is the primary key of the table.
 * @member {string} [name] Name of the column.
 * @member {string} [quotedName] Quoted name of the column.
 */
export interface SyncFullSchemaTableColumn {
  readonly dataSize?: string;
  readonly dataType?: string;
  readonly errorId?: string;
  readonly hasError?: boolean;
  readonly isPrimaryKey?: boolean;
  readonly name?: string;
  readonly quotedName?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncFullSchemaTable class.
 * @constructor
 * Properties of the table in the database full schema.
 *
 * @member {array} [columns] List of columns in the table of database full
 * schema.
 * @member {string} [errorId] Error id of the table.
 * @member {boolean} [hasError] If there is error in the table.
 * @member {string} [name] Name of the table.
 * @member {string} [quotedName] Quoted name of the table.
 */
export interface SyncFullSchemaTable {
  readonly columns?: SyncFullSchemaTableColumn[];
  readonly errorId?: string;
  readonly hasError?: boolean;
  readonly name?: string;
  readonly quotedName?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncFullSchemaProperties class.
 * @constructor
 * Properties of the database full schema.
 *
 * @member {array} [tables] List of tables in the database full schema.
 * @member {date} [lastUpdateTime] Last update time of the database schema.
 */
export interface SyncFullSchemaProperties {
  readonly tables?: SyncFullSchemaTable[];
  readonly lastUpdateTime?: Date;
}

/**
 * @class
 * Initializes a new instance of the SyncGroupLogProperties class.
 * @constructor
 * Properties of an Azure SQL Database sync group log.
 *
 * @member {date} [timestamp] Timestamp of the sync group log.
 * @member {string} [type] Type of the sync group log. Possible values include:
 * 'All', 'Error', 'Warning', 'Success'
 * @member {string} [source] Source of the sync group log.
 * @member {string} [details] Details of the sync group log.
 * @member {uuid} [tracingId] TracingId of the sync group log.
 * @member {string} [operationStatus] OperationStatus of the sync group log.
 */
export interface SyncGroupLogProperties {
  readonly timestamp?: Date;
  readonly type?: string;
  readonly source?: string;
  readonly details?: string;
  readonly tracingId?: string;
  readonly operationStatus?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncGroupSchemaTableColumn class.
 * @constructor
 * Properties of column in sync group table.
 *
 * @member {string} [quotedName] Quoted name of sync group table column.
 * @member {string} [dataSize] Data size of the column.
 * @member {string} [dataType] Data type of the column.
 */
export interface SyncGroupSchemaTableColumn {
  quotedName?: string;
  dataSize?: string;
  dataType?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncGroupSchemaTable class.
 * @constructor
 * Properties of table in sync group schema.
 *
 * @member {array} [columns] List of columns in sync group schema.
 * @member {string} [quotedName] Quoted name of sync group schema table.
 */
export interface SyncGroupSchemaTable {
  columns?: SyncGroupSchemaTableColumn[];
  quotedName?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncGroupSchema class.
 * @constructor
 * Properties of sync group schema.
 *
 * @member {array} [tables] List of tables in sync group schema.
 * @member {string} [masterSyncMemberName] Name of master sync member where the
 * schema is from.
 */
export interface SyncGroupSchema {
  tables?: SyncGroupSchemaTable[];
  masterSyncMemberName?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncGroup class.
 * @constructor
 * An Azure SQL Database sync group.
 *
 * @member {number} [interval] Sync interval of the sync group.
 * @member {date} [lastSyncTime] Last sync time of the sync group.
 * @member {string} [conflictResolutionPolicy] Conflict resolution policy of
 * the sync group. Possible values include: 'HubWin', 'MemberWin'
 * @member {string} [syncDatabaseId] ARM resource id of the sync database in
 * the sync group.
 * @member {string} [hubDatabaseUserName] User name for the sync group hub
 * database credential.
 * @member {string} [hubDatabasePassword] Password for the sync group hub
 * database credential.
 * @member {string} [syncState] Sync state of the sync group. Possible values
 * include: 'NotReady', 'Error', 'Warning', 'Progressing', 'Good'
 * @member {object} [schema] Sync schema of the sync group.
 * @member {array} [schema.tables] List of tables in sync group schema.
 * @member {string} [schema.masterSyncMemberName] Name of master sync member
 * where the schema is from.
 */
export interface SyncGroup extends ProxyResource {
  interval?: number;
  readonly lastSyncTime?: Date;
  conflictResolutionPolicy?: string;
  syncDatabaseId?: string;
  hubDatabaseUserName?: string;
  hubDatabasePassword?: string;
  readonly syncState?: string;
  schema?: SyncGroupSchema;
}

/**
 * @class
 * Initializes a new instance of the SyncMember class.
 * @constructor
 * An Azure SQL Database sync member.
 *
 * @member {string} [databaseType] Database type of the sync member. Possible
 * values include: 'AzureSqlDatabase', 'SqlServerDatabase'
 * @member {string} [syncAgentId] ARM resource id of the sync agent in the sync
 * member.
 * @member {uuid} [sqlServerDatabaseId] SQL Server database id of the sync
 * member.
 * @member {string} [serverName] Server name of the member database in the sync
 * member
 * @member {string} [databaseName] Database name of the member database in the
 * sync member.
 * @member {string} [userName] User name of the member database in the sync
 * member.
 * @member {string} [password] Password of the member database in the sync
 * member.
 * @member {string} [syncDirection] Sync direction of the sync member. Possible
 * values include: 'Bidirectional', 'OneWayMemberToHub', 'OneWayHubToMember'
 * @member {string} [syncState] Sync state of the sync member. Possible values
 * include: 'SyncInProgress', 'SyncSucceeded', 'SyncFailed',
 * 'DisabledTombstoneCleanup', 'DisabledBackupRestore',
 * 'SyncSucceededWithWarnings', 'SyncCancelling', 'SyncCancelled',
 * 'UnProvisioned', 'Provisioning', 'Provisioned', 'ProvisionFailed',
 * 'DeProvisioning', 'DeProvisioned', 'DeProvisionFailed', 'Reprovisioning',
 * 'ReprovisionFailed', 'UnReprovisioned'
 */
export interface SyncMember extends ProxyResource {
  databaseType?: string;
  syncAgentId?: string;
  sqlServerDatabaseId?: string;
  serverName?: string;
  databaseName?: string;
  userName?: string;
  password?: string;
  syncDirection?: string;
  readonly syncState?: string;
}

/**
 * @class
 * Initializes a new instance of the VirtualNetworkRule class.
 * @constructor
 * A virtual network rule.
 *
 * @member {string} virtualNetworkSubnetId The ARM resource id of the virtual
 * network subnet.
 * @member {boolean} [ignoreMissingVnetServiceEndpoint] Create firewall rule
 * before the virtual network has vnet service endpoint enabled.
 * @member {string} [state] Virtual Network Rule State. Possible values
 * include: 'Initializing', 'InProgress', 'Ready', 'Deleting', 'Unknown'
 */
export interface VirtualNetworkRule extends ProxyResource {
  virtualNetworkSubnetId: string;
  ignoreMissingVnetServiceEndpoint?: boolean;
  readonly state?: string;
}

/**
 * @class
 * Initializes a new instance of the DatabaseOperation class.
 * @constructor
 * A database operation.
 *
 * @member {string} [databaseName] The name of the database the operation is
 * being performed on.
 * @member {string} [operation] The name of operation.
 * @member {string} [operationFriendlyName] The friendly name of operation.
 * @member {number} [percentComplete] The percentage of the operation
 * completed.
 * @member {string} [serverName] The name of the server.
 * @member {date} [startTime] The operation start time.
 * @member {string} [state] The operation state. Possible values include:
 * 'Pending', 'InProgress', 'Succeeded', 'Failed', 'CancelInProgress',
 * 'Cancelled'
 * @member {number} [errorCode] The operation error code.
 * @member {string} [errorDescription] The operation error description.
 * @member {number} [errorSeverity] The operation error severity.
 * @member {boolean} [isUserError] Whether or not the error is a user error.
 */
export interface DatabaseOperation extends ProxyResource {
  readonly databaseName?: string;
  readonly operation?: string;
  readonly operationFriendlyName?: string;
  readonly percentComplete?: number;
  readonly serverName?: string;
  readonly startTime?: Date;
  readonly state?: string;
  readonly errorCode?: number;
  readonly errorDescription?: string;
  readonly errorSeverity?: number;
  readonly isUserError?: boolean;
}


/**
 * @class
 * Initializes a new instance of the RestorePointListResult class.
 * @constructor
 * The response to a list database restore points request.
 *
 */
export interface RestorePointListResult extends Array<RestorePoint> {
}

/**
 * @class
 * Initializes a new instance of the RecoverableDatabaseListResult class.
 * @constructor
 * The response to a list recoverable databases request
 *
 */
export interface RecoverableDatabaseListResult extends Array<RecoverableDatabase> {
}

/**
 * @class
 * Initializes a new instance of the RestorableDroppedDatabaseListResult class.
 * @constructor
 * The response to a list restorable dropped databases request
 *
 */
export interface RestorableDroppedDatabaseListResult extends Array<RestorableDroppedDatabase> {
}

/**
 * @class
 * Initializes a new instance of the DataMaskingRuleListResult class.
 * @constructor
 * The response to a list data masking rules request.
 *
 */
export interface DataMaskingRuleListResult extends Array<DataMaskingRule> {
}

/**
 * @class
 * Initializes a new instance of the FirewallRuleListResult class.
 * @constructor
 * Represents the response to a List Firewall Rules request.
 *
 */
export interface FirewallRuleListResult extends Array<FirewallRule> {
}

/**
 * @class
 * Initializes a new instance of the GeoBackupPolicyListResult class.
 * @constructor
 * The response to a list geo backup policies request.
 *
 */
export interface GeoBackupPolicyListResult extends Array<GeoBackupPolicy> {
}

/**
 * @class
 * Initializes a new instance of the MetricListResult class.
 * @constructor
 * The response to a list database metrics request.
 *
 */
export interface MetricListResult extends Array<Metric> {
}

/**
 * @class
 * Initializes a new instance of the MetricDefinitionListResult class.
 * @constructor
 * The response to a list database metric definitions request.
 *
 */
export interface MetricDefinitionListResult extends Array<MetricDefinition> {
}

/**
 * @class
 * Initializes a new instance of the DatabaseListResult class.
 * @constructor
 * Represents the response to a list database request.
 *
 */
export interface DatabaseListResult extends Array<Database> {
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolListResult class.
 * @constructor
 * Represents the response to a list elastic pool request.
 *
 */
export interface ElasticPoolListResult extends Array<ElasticPool> {
}

/**
 * @class
 * Initializes a new instance of the ReplicationLinkListResult class.
 * @constructor
 * Represents the response to a List database replication link request.
 *
 */
export interface ReplicationLinkListResult extends Array<ReplicationLink> {
}

/**
 * @class
 * Initializes a new instance of the ServerAdministratorListResult class.
 * @constructor
 * The response to a list Active Directory Administrators request.
 *
 */
export interface ServerAdministratorListResult extends Array<ServerAzureADAdministrator> {
}

/**
 * @class
 * Initializes a new instance of the ServerCommunicationLinkListResult class.
 * @constructor
 * A list of server communication links.
 *
 */
export interface ServerCommunicationLinkListResult extends Array<ServerCommunicationLink> {
}

/**
 * @class
 * Initializes a new instance of the ServiceObjectiveListResult class.
 * @constructor
 * Represents the response to a get database service objectives request.
 *
 */
export interface ServiceObjectiveListResult extends Array<ServiceObjective> {
}

/**
 * @class
 * Initializes a new instance of the ServerListResult class.
 * @constructor
 * A list of servers.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface ServerListResult extends Array<Server> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolActivityListResult class.
 * @constructor
 * Represents the response to a list elastic pool activity request.
 *
 */
export interface ElasticPoolActivityListResult extends Array<ElasticPoolActivity> {
}

/**
 * @class
 * Initializes a new instance of the ElasticPoolDatabaseActivityListResult class.
 * @constructor
 * Represents the response to a list elastic pool database activity request.
 *
 */
export interface ElasticPoolDatabaseActivityListResult extends Array<ElasticPoolDatabaseActivity> {
}

/**
 * @class
 * Initializes a new instance of the RecommendedElasticPoolListResult class.
 * @constructor
 * Represents the response to a list recommended elastic pool request.
 *
 */
export interface RecommendedElasticPoolListResult extends Array<RecommendedElasticPool> {
}

/**
 * @class
 * Initializes a new instance of the RecommendedElasticPoolListMetricsResult class.
 * @constructor
 * Represents the response to a list recommended elastic pool metrics request.
 *
 */
export interface RecommendedElasticPoolListMetricsResult extends Array<RecommendedElasticPoolMetric> {
}

/**
 * @class
 * Initializes a new instance of the ServiceTierAdvisorListResult class.
 * @constructor
 * Represents the response to a list service tier advisor request.
 *
 */
export interface ServiceTierAdvisorListResult extends Array<ServiceTierAdvisor> {
}

/**
 * @class
 * Initializes a new instance of the TransparentDataEncryptionActivityListResult class.
 * @constructor
 * Represents the response to a list database transparent data encryption
 * activity request.
 *
 */
export interface TransparentDataEncryptionActivityListResult extends Array<TransparentDataEncryptionActivity> {
}

/**
 * @class
 * Initializes a new instance of the ServerUsageListResult class.
 * @constructor
 * Represents the response to a list server metrics request.
 *
 */
export interface ServerUsageListResult extends Array<ServerUsage> {
}

/**
 * @class
 * Initializes a new instance of the DatabaseUsageListResult class.
 * @constructor
 * The response to a list database metrics request.
 *
 */
export interface DatabaseUsageListResult extends Array<DatabaseUsage> {
}

/**
 * @class
 * Initializes a new instance of the EncryptionProtectorListResult class.
 * @constructor
 * A list of server encryption protectors.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface EncryptionProtectorListResult extends Array<EncryptionProtector> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the FailoverGroupListResult class.
 * @constructor
 * A list of failover groups.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface FailoverGroupListResult extends Array<FailoverGroup> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Result of the request to list SQL operations.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface OperationListResult extends Array<Operation> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the ServerKeyListResult class.
 * @constructor
 * A list of server keys.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface ServerKeyListResult extends Array<ServerKey> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncAgentListResult class.
 * @constructor
 * A list of sync agents.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncAgentListResult extends Array<SyncAgent> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncAgentLinkedDatabaseListResult class.
 * @constructor
 * A list of sync agent linked databases.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncAgentLinkedDatabaseListResult extends Array<SyncAgentLinkedDatabase> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncDatabaseIdListResult class.
 * @constructor
 * A list of sync database ID properties.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncDatabaseIdListResult extends Array<SyncDatabaseIdProperties> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncFullSchemaPropertiesListResult class.
 * @constructor
 * A list of sync schema properties.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncFullSchemaPropertiesListResult extends Array<SyncFullSchemaProperties> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncGroupLogListResult class.
 * @constructor
 * A list of sync group log properties.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncGroupLogListResult extends Array<SyncGroupLogProperties> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncGroupListResult class.
 * @constructor
 * A list of sync groups.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncGroupListResult extends Array<SyncGroup> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the SyncMemberListResult class.
 * @constructor
 * A list of Azure SQL Database sync members.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface SyncMemberListResult extends Array<SyncMember> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the VirtualNetworkRuleListResult class.
 * @constructor
 * A list of virtual network rules.
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface VirtualNetworkRuleListResult extends Array<VirtualNetworkRule> {
  readonly nextLink?: string;
}

/**
 * @class
 * Initializes a new instance of the DatabaseOperationListResult class.
 * @constructor
 * The response to a list database operations request
 *
 * @member {string} [nextLink] Link to retrieve next page of results.
 */
export interface DatabaseOperationListResult extends Array<DatabaseOperation> {
  readonly nextLink?: string;
}
