/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes an import/export job.
 *
 * @extends models['Resource']
 */
class Job extends models['Resource'] {
  /**
   * Create a Job.
   * @member {string} storageAccountId The resource identifier of the storage
   * account where data will be imported to or exported from.
   * @member {string} [containerSas] The container shared access signature
   * (SAS) to be used to import or export data to or from the storage account.
   * @member {string} jobType The type of job: Import or Export. Possible
   * values include: 'Import', 'Export'
   * @member {object} returnAddress Specifies the return address information
   * for the job.
   * @member {string} [returnAddress.recipientName] The name of the recipient
   * who will receive the hard drives when they are returned.
   * @member {string} [returnAddress.streetAddress1] The first line of the
   * street address to use when returning the drives.
   * @member {string} [returnAddress.streetAddress2] The second line of the
   * street address to use when returning the drives.
   * @member {string} [returnAddress.city] The city name to use when returning
   * the drives.
   * @member {string} [returnAddress.stateOrProvince] The state or province to
   * use when returning the drives.
   * @member {string} [returnAddress.postalCode] The postal code to use when
   * returning the drives.
   * @member {string} [returnAddress.countryOrRegion] The country or region to
   * use when returning the drives.
   * @member {string} [returnAddress.phone] Phone number of the recipient of
   * the returned drives.
   * @member {string} [returnAddress.email] Email address of the recipient of
   * the returned drives.
   * @member {object} returnShipping Specifies the return carrier and
   * customer's account with the carrier.
   * @member {string} [returnShipping.carrierName] The carrier's name.
   * @member {string} [returnShipping.carrierAccountNumber] The customer's
   * account number with the carrier.
   * @member {object} [shippingInformation] Contains information about the
   * Microsoft datacenter to which the drives should be shipped.
   * @member {string} [shippingInformation.name] The name of the recipient at
   * the datacenter.
   * @member {string} [shippingInformation.address] The address of the
   * datacenter.
   * @member {object} [deliveryPackage] Contains information about the package
   * being shipped by the customer to the Microsoft data center.
   * @member {string} [deliveryPackage.carrierName] The name of the carrier
   * that is used to ship the import or export drives.
   * @member {string} [deliveryPackage.trackingNumber] The tracking number of
   * the package.
   * @member {number} [deliveryPackage.driveCount] The number of drives
   * included in the package.
   * @member {string} [deliveryPackage.shipDate] The date the package is
   * shipped.
   * @member {object} [returnPackage] Contains information about the package
   * being shipped from the Microsoft data center to the customer to return the
   * drives. The format is the same as the deliveryPackage property above. This
   * property is not included if the drives have not yet been returned.
   * @member {string} [returnPackage.carrierName] The name of the carrier that
   * is used to ship the import or export drives.
   * @member {string} [returnPackage.trackingNumber] The tracking number of the
   * package.
   * @member {number} [returnPackage.driveCount] The number of drives included
   * in the package.
   * @member {string} [returnPackage.shipDate] The date the package is shipped.
   * @member {string} diagnosticsPath The virtual blob directory to which the
   * copy logs and backups of drive manifest files (if enabled) will be stored.
   * @member {string} [logLevel] Default value is Error. Indicates whether
   * error logging or verbose logging will be enabled. Possible values include:
   * 'Error', 'Verbose'. Default value: 'Error' .
   * @member {boolean} [backupDriveManifest] Default value is false. Indicates
   * whether the manifest files on the drives should be copied to block blobs.
   * Default value: false .
   * @member {string} [state] Current state of the job. Possible values
   * include: 'Creating', 'Shipping', 'Received', 'Transferring', 'Packaging',
   * 'Closed', 'Completed'
   * @member {boolean} [cancelRequested] Indicates whether a request has been
   * submitted to cancel the job.
   * @member {number} [percentComplete] Overall percentage completed for the
   * job.
   * @member {string} [incompleteBlobListUri] A blob path that points to a
   * block blob containing a list of blob names that were not exported due to
   * insufficient drive space. If all blobs were exported successfully, then
   * this element is not included in the response.
   * @member {array} [driveList] List of up to ten drives that comprise the
   * job. The drive list is a required element for an import job; it is not
   * specified for export jobs.
   * @member {object} [exportProperty] A property containing information about
   * the blobs to be exported for an export job. This property is included for
   * export jobs only.
   * @member {array} [exportProperty.blobPath] A collection of blob-path
   * strings.
   * @member {array} [exportProperty.blobPathPrefix] A collection of
   * blob-prefix strings.
   * @member {string} [exportProperty.blobListblobPath] The relative URI to the
   * block blob that contains the list of blob paths or blob path prefixes as
   * defined above, beginning with the container name. If the blob is in the
   * root container, the URI must begin with $root.
   * @member {string} [provisioningState] Specifies the provisioning state of
   * the job.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of Job
   *
   * @returns {object} metadata of Job
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Job',
      type: {
        name: 'Composite',
        className: 'Job',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          },
          tags: {
            required: false,
            serializedName: 'tags',
            type: {
              name: 'Object'
            }
          },
          storageAccountId: {
            required: true,
            serializedName: 'properties.storageAccountId',
            type: {
              name: 'String'
            }
          },
          containerSas: {
            required: false,
            serializedName: 'properties.containerSas',
            type: {
              name: 'String'
            }
          },
          jobType: {
            required: true,
            serializedName: 'properties.jobType',
            type: {
              name: 'Enum',
              allowedValues: [ 'Import', 'Export' ]
            }
          },
          returnAddress: {
            required: true,
            serializedName: 'properties.returnAddress',
            type: {
              name: 'Composite',
              className: 'ReturnAddress'
            }
          },
          returnShipping: {
            required: true,
            serializedName: 'properties.returnShipping',
            type: {
              name: 'Composite',
              className: 'ReturnShipping'
            }
          },
          shippingInformation: {
            required: false,
            readOnly: true,
            serializedName: 'properties.shippingInformation',
            type: {
              name: 'Composite',
              className: 'ShippingInformation'
            }
          },
          deliveryPackage: {
            required: false,
            serializedName: 'properties.deliveryPackage',
            type: {
              name: 'Composite',
              className: 'PackageInfomation'
            }
          },
          returnPackage: {
            required: false,
            readOnly: true,
            serializedName: 'properties.returnPackage',
            type: {
              name: 'Composite',
              className: 'PackageInfomation'
            }
          },
          diagnosticsPath: {
            required: true,
            serializedName: 'properties.diagnosticsPath',
            type: {
              name: 'String'
            }
          },
          logLevel: {
            required: false,
            serializedName: 'properties.logLevel',
            defaultValue: 'Error',
            type: {
              name: 'String'
            }
          },
          backupDriveManifest: {
            required: false,
            serializedName: 'properties.backupDriveManifest',
            defaultValue: false,
            type: {
              name: 'Boolean'
            }
          },
          state: {
            required: false,
            readOnly: true,
            serializedName: 'properties.state',
            type: {
              name: 'String'
            }
          },
          cancelRequested: {
            required: false,
            serializedName: 'properties.cancelRequested',
            type: {
              name: 'Boolean'
            }
          },
          percentComplete: {
            required: false,
            readOnly: true,
            serializedName: 'properties.percentComplete',
            type: {
              name: 'Number'
            }
          },
          incompleteBlobListUri: {
            required: false,
            readOnly: true,
            serializedName: 'properties.incompleteBlobListUri',
            type: {
              name: 'String'
            }
          },
          driveList: {
            required: false,
            serializedName: 'properties.driveList',
            constraints: {
              MaxItems: 10,
              MinItems: 0
            },
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'DriveStatusElementType',
                  type: {
                    name: 'Composite',
                    className: 'DriveStatus'
                  }
              }
            }
          },
          exportProperty: {
            required: false,
            serializedName: 'properties.export',
            type: {
              name: 'Composite',
              className: 'ExportModel'
            }
          },
          provisioningState: {
            required: false,
            readOnly: true,
            serializedName: 'properties.provisioningState',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = Job;
