/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes an Azure Data Lake Store output data source.
 *
 * @extends models['OutputDataSource']
 */
class AzureDataLakeStoreOutputDataSource extends models['OutputDataSource'] {
  /**
   * Create a AzureDataLakeStoreOutputDataSource.
   * @member {string} [refreshToken] A refresh token that can be used to obtain
   * a valid access token that can then be used to authenticate with the data
   * source. A valid refresh token is currently only obtainable via the Azure
   * Portal. It is recommended to put a dummy string value here when creating
   * the data source and then going to the Azure Portal to authenticate the
   * data source which will update this property with a valid refresh token.
   * Required on PUT (CreateOrReplace) requests.
   * @member {string} [tokenUserPrincipalName] The user principal name (UPN) of
   * the user that was used to obtain the refresh token. Use this property to
   * help remember which user was used to obtain the refresh token.
   * @member {string} [tokenUserDisplayName] The user display name of the user
   * that was used to obtain the refresh token. Use this property to help
   * remember which user was used to obtain the refresh token.
   * @member {string} [accountName] The name of the Azure Data Lake Store
   * account. Required on PUT (CreateOrReplace) requests.
   * @member {string} [tenantId] The tenant id of the user used to obtain the
   * refresh token. Required on PUT (CreateOrReplace) requests.
   * @member {string} [filePathPrefix] The location of the file to which the
   * output should be written to. Required on PUT (CreateOrReplace) requests.
   * @member {string} [dateFormat] The date format. Wherever {date} appears in
   * filePathPrefix, the value of this property is used as the date format
   * instead.
   * @member {string} [timeFormat] The time format. Wherever {time} appears in
   * filePathPrefix, the value of this property is used as the time format
   * instead.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of AzureDataLakeStoreOutputDataSource
   *
   * @returns {object} metadata of AzureDataLakeStoreOutputDataSource
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Microsoft.DataLake/Accounts',
      type: {
        name: 'Composite',
        className: 'AzureDataLakeStoreOutputDataSource',
        modelProperties: {
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          refreshToken: {
            required: false,
            serializedName: 'properties.refreshToken',
            type: {
              name: 'String'
            }
          },
          tokenUserPrincipalName: {
            required: false,
            serializedName: 'properties.tokenUserPrincipalName',
            type: {
              name: 'String'
            }
          },
          tokenUserDisplayName: {
            required: false,
            serializedName: 'properties.tokenUserDisplayName',
            type: {
              name: 'String'
            }
          },
          accountName: {
            required: false,
            serializedName: 'properties.accountName',
            type: {
              name: 'String'
            }
          },
          tenantId: {
            required: false,
            serializedName: 'properties.tenantId',
            type: {
              name: 'String'
            }
          },
          filePathPrefix: {
            required: false,
            serializedName: 'properties.filePathPrefix',
            type: {
              name: 'String'
            }
          },
          dateFormat: {
            required: false,
            serializedName: 'properties.dateFormat',
            type: {
              name: 'String'
            }
          },
          timeFormat: {
            required: false,
            serializedName: 'properties.timeFormat',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = AzureDataLakeStoreOutputDataSource;
