/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes an Event Hub output data source.
 *
 * @extends models['OutputDataSource']
 */
class EventHubOutputDataSource extends models['OutputDataSource'] {
  /**
   * Create a EventHubOutputDataSource.
   * @member {string} [serviceBusNamespace] The namespace that is associated
   * with the desired Event Hub, Service Bus Queue, Service Bus Topic, etc.
   * Required on PUT (CreateOrReplace) requests.
   * @member {string} [sharedAccessPolicyName] The shared access policy name
   * for the Event Hub, Service Bus Queue, Service Bus Topic, etc. Required on
   * PUT (CreateOrReplace) requests.
   * @member {string} [sharedAccessPolicyKey] The shared access policy key for
   * the specified shared access policy. Required on PUT (CreateOrReplace)
   * requests.
   * @member {string} [eventHubName] The name of the Event Hub. Required on PUT
   * (CreateOrReplace) requests.
   * @member {string} [partitionKey] The key/column that is used to determine
   * to which partition to send event data.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of EventHubOutputDataSource
   *
   * @returns {object} metadata of EventHubOutputDataSource
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Microsoft.ServiceBus/EventHub',
      type: {
        name: 'Composite',
        className: 'EventHubOutputDataSource',
        modelProperties: {
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          serviceBusNamespace: {
            required: false,
            serializedName: 'properties.serviceBusNamespace',
            type: {
              name: 'String'
            }
          },
          sharedAccessPolicyName: {
            required: false,
            serializedName: 'properties.sharedAccessPolicyName',
            type: {
              name: 'String'
            }
          },
          sharedAccessPolicyKey: {
            required: false,
            serializedName: 'properties.sharedAccessPolicyKey',
            type: {
              name: 'String'
            }
          },
          eventHubName: {
            required: false,
            serializedName: 'properties.eventHubName',
            type: {
              name: 'String'
            }
          },
          partitionKey: {
            required: false,
            serializedName: 'properties.partitionKey',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = EventHubOutputDataSource;
