/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes a Service Bus Topic output data source.
 *
 * @extends models['OutputDataSource']
 */
class ServiceBusTopicOutputDataSource extends models['OutputDataSource'] {
  /**
   * Create a ServiceBusTopicOutputDataSource.
   * @member {string} [serviceBusNamespace] The namespace that is associated
   * with the desired Event Hub, Service Bus Queue, Service Bus Topic, etc.
   * Required on PUT (CreateOrReplace) requests.
   * @member {string} [sharedAccessPolicyName] The shared access policy name
   * for the Event Hub, Service Bus Queue, Service Bus Topic, etc. Required on
   * PUT (CreateOrReplace) requests.
   * @member {string} [sharedAccessPolicyKey] The shared access policy key for
   * the specified shared access policy. Required on PUT (CreateOrReplace)
   * requests.
   * @member {string} [topicName] The name of the Service Bus Topic. Required
   * on PUT (CreateOrReplace) requests.
   * @member {array} [propertyColumns] A string array of the names of output
   * columns to be attached to Service Bus messages as custom properties.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ServiceBusTopicOutputDataSource
   *
   * @returns {object} metadata of ServiceBusTopicOutputDataSource
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Microsoft.ServiceBus/Topic',
      type: {
        name: 'Composite',
        className: 'ServiceBusTopicOutputDataSource',
        modelProperties: {
          type: {
            required: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          serviceBusNamespace: {
            required: false,
            serializedName: 'properties.serviceBusNamespace',
            type: {
              name: 'String'
            }
          },
          sharedAccessPolicyName: {
            required: false,
            serializedName: 'properties.sharedAccessPolicyName',
            type: {
              name: 'String'
            }
          },
          sharedAccessPolicyKey: {
            required: false,
            serializedName: 'properties.sharedAccessPolicyKey',
            type: {
              name: 'String'
            }
          },
          topicName: {
            required: false,
            serializedName: 'properties.topicName',
            type: {
              name: 'String'
            }
          },
          propertyColumns: {
            required: false,
            serializedName: 'properties.propertyColumns',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = ServiceBusTopicOutputDataSource;
