/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import { BaseResource } from 'ms-rest-azure';
import { CloudError } from 'ms-rest-azure';
import * as moment from 'moment';

export { BaseResource } from 'ms-rest-azure';
export { CloudError } from 'ms-rest-azure';


/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * A generic Azure Resource Manager resource.
 *
 * @member {string} [id] Unique identifier of the resource.
 * @member {string} [location] Resource location.
 * @member {string} [name] Resource name.
 * @member {object} [tags] Resource tags.
 * @member {string} [type] Resource type.
 */
export interface Resource extends BaseResource {
  readonly id?: string;
  location?: string;
  readonly name?: string;
  tags?: { [propertyName: string]: string };
  readonly type?: string;
}

/**
 * @class
 * Initializes a new instance of the AccountResource class.
 * @constructor
 * The response to an account resource GET request.
 *
 * @member {object} [properties] Resource properties.
 */
export interface AccountResource extends Resource {
  properties?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the AccountResourceListResult class.
 * @constructor
 * The response to an account resource list GET request.
 *
 * @member {array} [value] Array of resource details.
 */
export interface AccountResourceListResult {
  value?: AccountResource[];
}

/**
 * @class
 * Initializes a new instance of the AccountResourceRequest class.
 * @constructor
 * The body of a PUT request to modify a Visual Studio account resource.
 *
 * @member {string} [accountName] The account name.
 * @member {string} [location] The Azure instance location.
 * @member {object} [operationType] The type of the operation.
 * @member {object} [properties] The custom properties of the resource.
 * @member {object} [tags] The custom tags of the resource.
 */
export interface AccountResourceRequest {
  accountName?: string;
  location?: string;
  operationType?: any;
  properties?: { [propertyName: string]: string };
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the CheckNameAvailabilityParameter class.
 * @constructor
 * The body of a POST request to check name availability.
 *
 * @member {string} [resourceName] The name of the resource to check
 * availability for.
 * @member {string} [resourceType] The type of resource to check availability
 * for.
 */
export interface CheckNameAvailabilityParameter {
  resourceName?: string;
  resourceType?: string;
}

/**
 * @class
 * Initializes a new instance of the CheckNameAvailabilityResult class.
 * @constructor
 * The response to a name availability request.
 *
 * @member {string} [message] The message describing the detailed reason.
 * @member {boolean} [nameAvailable] The value which indicates whether the
 * provided name is available.
 */
export interface CheckNameAvailabilityResult {
  message?: string;
  nameAvailable?: boolean;
}

/**
 * @class
 * Initializes a new instance of the ExtensionResourcePlan class.
 * @constructor
 * Plan data for an extension resource.
 *
 * @member {string} [name] Name of the plan.
 * @member {string} [product] Product name.
 * @member {string} [promotionCode] Optional: the promotion code associated
 * with the plan.
 * @member {string} [publisher] Name of the extension publisher.
 * @member {string} [version] A string that uniquely identifies the plan
 * version.
 */
export interface ExtensionResourcePlan {
  name?: string;
  product?: string;
  promotionCode?: string;
  publisher?: string;
  version?: string;
}

/**
 * @class
 * Initializes a new instance of the ExtensionResource class.
 * @constructor
 * The response to an extension resource GET request.
 *
 * @member {object} [plan] The extension plan that was purchased.
 * @member {string} [plan.name] Name of the plan.
 * @member {string} [plan.product] Product name.
 * @member {string} [plan.promotionCode] Optional: the promotion code
 * associated with the plan.
 * @member {string} [plan.publisher] Name of the extension publisher.
 * @member {string} [plan.version] A string that uniquely identifies the plan
 * version.
 * @member {object} [properties] Resource properties.
 */
export interface ExtensionResource extends Resource {
  plan?: ExtensionResourcePlan;
  properties?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ExtensionResourceListResult class.
 * @constructor
 * The response to an extension resource list GET request.
 *
 * @member {array} [value] Array of extension resource details.
 */
export interface ExtensionResourceListResult {
  value?: ExtensionResource[];
}

/**
 * @class
 * Initializes a new instance of the ExtensionResourceRequest class.
 * @constructor
 * The body of an extension resource PUT request.
 *
 * @member {string} [location] The Azure region of the Visual Studio account
 * associated with this request (i.e 'southcentralus'.)
 * @member {object} [plan] Extended information about the plan being purchased
 * for this extension resource.
 * @member {string} [plan.name] Name of the plan.
 * @member {string} [plan.product] Product name.
 * @member {string} [plan.promotionCode] Optional: the promotion code
 * associated with the plan.
 * @member {string} [plan.publisher] Name of the extension publisher.
 * @member {string} [plan.version] A string that uniquely identifies the plan
 * version.
 * @member {object} [properties] A dictionary of extended properties. This
 * property is currently unused.
 * @member {object} [tags] A dictionary of user-defined tags to be stored with
 * the extension resource.
 */
export interface ExtensionResourceRequest {
  location?: string;
  plan?: ExtensionResourcePlan;
  properties?: { [propertyName: string]: string };
  tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the OperationProperties class.
 * @constructor
 * Properties of an operation supported by the resource provider.
 *
 * @member {string} [description] The description of the resource operation.
 * @member {string} [operation] The operation name.
 * @member {string} [provider] The provider name.
 * @member {string} [resource] The resource name.
 */
export interface OperationProperties {
  description?: string;
  operation?: string;
  provider?: string;
  resource?: string;
}

/**
 * @class
 * Initializes a new instance of the Operation class.
 * @constructor
 * Properties of an operation supported by the resource provider.
 *
 * @member {object} [display] The properties of the resource operation.
 * @member {string} [display.description] The description of the resource
 * operation.
 * @member {string} [display.operation] The operation name.
 * @member {string} [display.provider] The provider name.
 * @member {string} [display.resource] The resource name.
 * @member {string} [name] The name of the resource operation.
 */
export interface Operation {
  display?: OperationProperties;
  name?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationListResult class.
 * @constructor
 * Container for a list of operations supported by a resource provider.
 *
 * @member {array} [value] A list of operations supported by a resource
 * provider.
 */
export interface OperationListResult {
  value?: Operation[];
}

/**
 * @class
 * Initializes a new instance of the ProjectResource class.
 * @constructor
 * A Visual Studio Team Services project resource.
 *
 * @member {object} [properties] Key/value pair of resource properties.
 */
export interface ProjectResource extends Resource {
  properties?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the ProjectResourceListResult class.
 * @constructor
 * The response to a request to list Team Services project resources in a
 * resource group/account.
 *
 * @member {array} [value] List of project resource details.
 */
export interface ProjectResourceListResult {
  value?: ProjectResource[];
}

