/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Process Information.
 *
 * @extends models['ProxyOnlyResource']
 */
class ProcessInfo extends models['ProxyOnlyResource'] {
  /**
   * Create a ProcessInfo.
   * @member {number} [processInfoId] ARM Identifier for deployment.
   * @member {string} [processInfoName] Deployment name.
   * @member {string} [href] HRef URI.
   * @member {string} [miniDump] Minidump URI.
   * @member {boolean} [isProfileRunning] Is profile running?
   * @member {boolean} [isIisProfileRunning] Is the IIS Profile running?
   * @member {number} [iisProfileTimeoutInSeconds] IIS Profile timeout
   * (seconds).
   * @member {string} [parent] Parent process.
   * @member {array} [children] Child process list.
   * @member {array} [threads] Thread list.
   * @member {array} [openFileHandles] List of open files.
   * @member {array} [modules] List of modules.
   * @member {string} [fileName] File name of this process.
   * @member {string} [commandLine] Command line.
   * @member {string} [userName] User name.
   * @member {number} [handleCount] Handle count.
   * @member {number} [moduleCount] Module count.
   * @member {number} [threadCount] Thread count.
   * @member {date} [startTime] Start time.
   * @member {string} [totalProcessorTime] Total CPU time.
   * @member {string} [userProcessorTime] User CPU time.
   * @member {string} [privilegedProcessorTime] Privileged CPU time.
   * @member {number} [workingSet64] Working set.
   * @member {number} [peakWorkingSet64] Peak working set.
   * @member {number} [privateMemorySize64] Private memory size.
   * @member {number} [virtualMemorySize64] Virtual memory size.
   * @member {number} [peakVirtualMemorySize64] Peak virtual memory usage.
   * @member {number} [pagedSystemMemorySize64] Paged system memory.
   * @member {number} [nonpagedSystemMemorySize64] Non-paged system memory.
   * @member {number} [pagedMemorySize64] Paged memory.
   * @member {number} [peakPagedMemorySize64] Peak paged memory.
   * @member {date} [timeStamp] Time stamp.
   * @member {object} [environmentVariables] List of environment variables.
   * @member {boolean} [isScmSite] Is this the SCM site?
   * @member {boolean} [isWebJob] Is this a Web Job?
   * @member {string} [description] Description of process.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of ProcessInfo
   *
   * @returns {object} metadata of ProcessInfo
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ProcessInfo',
      type: {
        name: 'Composite',
        className: 'ProcessInfo',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: false,
            serializedName: 'kind',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          processInfoId: {
            required: false,
            serializedName: 'properties.id',
            type: {
              name: 'Number'
            }
          },
          processInfoName: {
            required: false,
            serializedName: 'properties.name',
            type: {
              name: 'String'
            }
          },
          href: {
            required: false,
            serializedName: 'properties.href',
            type: {
              name: 'String'
            }
          },
          miniDump: {
            required: false,
            serializedName: 'properties.miniDump',
            type: {
              name: 'String'
            }
          },
          isProfileRunning: {
            required: false,
            serializedName: 'properties.isProfileRunning',
            type: {
              name: 'Boolean'
            }
          },
          isIisProfileRunning: {
            required: false,
            serializedName: 'properties.isIisProfileRunning',
            type: {
              name: 'Boolean'
            }
          },
          iisProfileTimeoutInSeconds: {
            required: false,
            serializedName: 'properties.iisProfileTimeoutInSeconds',
            type: {
              name: 'Number'
            }
          },
          parent: {
            required: false,
            serializedName: 'properties.parent',
            type: {
              name: 'String'
            }
          },
          children: {
            required: false,
            serializedName: 'properties.children',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          threads: {
            required: false,
            serializedName: 'properties.threads',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ProcessThreadInfoElementType',
                  type: {
                    name: 'Composite',
                    className: 'ProcessThreadInfo'
                  }
              }
            }
          },
          openFileHandles: {
            required: false,
            serializedName: 'properties.openFileHandles',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          modules: {
            required: false,
            serializedName: 'properties.modules',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'ProcessModuleInfoElementType',
                  type: {
                    name: 'Composite',
                    className: 'ProcessModuleInfo'
                  }
              }
            }
          },
          fileName: {
            required: false,
            serializedName: 'properties.fileName',
            type: {
              name: 'String'
            }
          },
          commandLine: {
            required: false,
            serializedName: 'properties.commandLine',
            type: {
              name: 'String'
            }
          },
          userName: {
            required: false,
            serializedName: 'properties.userName',
            type: {
              name: 'String'
            }
          },
          handleCount: {
            required: false,
            serializedName: 'properties.handleCount',
            type: {
              name: 'Number'
            }
          },
          moduleCount: {
            required: false,
            serializedName: 'properties.moduleCount',
            type: {
              name: 'Number'
            }
          },
          threadCount: {
            required: false,
            serializedName: 'properties.threadCount',
            type: {
              name: 'Number'
            }
          },
          startTime: {
            required: false,
            serializedName: 'properties.startTime',
            type: {
              name: 'DateTime'
            }
          },
          totalProcessorTime: {
            required: false,
            serializedName: 'properties.totalProcessorTime',
            type: {
              name: 'String'
            }
          },
          userProcessorTime: {
            required: false,
            serializedName: 'properties.userProcessorTime',
            type: {
              name: 'String'
            }
          },
          privilegedProcessorTime: {
            required: false,
            serializedName: 'properties.privilegedProcessorTime',
            type: {
              name: 'String'
            }
          },
          workingSet64: {
            required: false,
            serializedName: 'properties.workingSet64',
            type: {
              name: 'Number'
            }
          },
          peakWorkingSet64: {
            required: false,
            serializedName: 'properties.peakWorkingSet64',
            type: {
              name: 'Number'
            }
          },
          privateMemorySize64: {
            required: false,
            serializedName: 'properties.privateMemorySize64',
            type: {
              name: 'Number'
            }
          },
          virtualMemorySize64: {
            required: false,
            serializedName: 'properties.virtualMemorySize64',
            type: {
              name: 'Number'
            }
          },
          peakVirtualMemorySize64: {
            required: false,
            serializedName: 'properties.peakVirtualMemorySize64',
            type: {
              name: 'Number'
            }
          },
          pagedSystemMemorySize64: {
            required: false,
            serializedName: 'properties.pagedSystemMemorySize64',
            type: {
              name: 'Number'
            }
          },
          nonpagedSystemMemorySize64: {
            required: false,
            serializedName: 'properties.nonpagedSystemMemorySize64',
            type: {
              name: 'Number'
            }
          },
          pagedMemorySize64: {
            required: false,
            serializedName: 'properties.pagedMemorySize64',
            type: {
              name: 'Number'
            }
          },
          peakPagedMemorySize64: {
            required: false,
            serializedName: 'properties.peakPagedMemorySize64',
            type: {
              name: 'Number'
            }
          },
          timeStamp: {
            required: false,
            serializedName: 'properties.timeStamp',
            type: {
              name: 'DateTime'
            }
          },
          environmentVariables: {
            required: false,
            serializedName: 'properties.environmentVariables',
            type: {
              name: 'Dictionary',
              value: {
                  required: false,
                  serializedName: 'StringElementType',
                  type: {
                    name: 'String'
                  }
              }
            }
          },
          isScmSite: {
            required: false,
            serializedName: 'properties.isScmSite',
            type: {
              name: 'Boolean'
            }
          },
          isWebJob: {
            required: false,
            serializedName: 'properties.isWebJob',
            type: {
              name: 'Boolean'
            }
          },
          description: {
            required: false,
            serializedName: 'properties.description',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ProcessInfo;
