/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Details about app recovery operation.
 *
 * @extends models['ProxyOnlyResource']
 */
class SnapshotRecoveryRequest extends models['ProxyOnlyResource'] {
  /**
   * Create a SnapshotRecoveryRequest.
   * @member {string} [snapshotTime] Point in time in which the app recovery
   * should be attempted, formatted as a DateTime string.
   * @member {object} [recoveryTarget] Specifies the web app that snapshot
   * contents will be written to.
   * @member {string} [recoveryTarget.location] Geographical location of the
   * target web app, e.g. SouthEastAsia, SouthCentralUS
   * @member {string} [recoveryTarget.id] ARM resource ID of the target app.
   * /subscriptions/{subId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/sites/{siteName}
   * for production slots and
   * /subscriptions/{subId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Web/sites/{siteName}/slots/{slotName}
   * for other slots.
   * @member {boolean} [overwrite] If <code>true</code> the recovery operation
   * can overwrite source app; otherwise, <code>false</code>.
   * @member {boolean} [recoverConfiguration] If true, site configuration, in
   * addition to content, will be reverted.
   * @member {boolean} [ignoreConflictingHostNames] If true, custom hostname
   * conflicts will be ignored when recovering to a target web app.
   * This setting is only necessary when RecoverConfiguration is enabled.
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of SnapshotRecoveryRequest
   *
   * @returns {object} metadata of SnapshotRecoveryRequest
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'SnapshotRecoveryRequest',
      type: {
        name: 'Composite',
        className: 'SnapshotRecoveryRequest',
        modelProperties: {
          id: {
            required: false,
            readOnly: true,
            serializedName: 'id',
            type: {
              name: 'String'
            }
          },
          name: {
            required: false,
            readOnly: true,
            serializedName: 'name',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: false,
            serializedName: 'kind',
            type: {
              name: 'String'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          snapshotTime: {
            required: false,
            serializedName: 'properties.snapshotTime',
            type: {
              name: 'String'
            }
          },
          recoveryTarget: {
            required: false,
            serializedName: 'properties.recoveryTarget',
            type: {
              name: 'Composite',
              className: 'SnapshotRecoveryTarget'
            }
          },
          overwrite: {
            required: false,
            serializedName: 'properties.overwrite',
            type: {
              name: 'Boolean'
            }
          },
          recoverConfiguration: {
            required: false,
            serializedName: 'properties.recoverConfiguration',
            type: {
              name: 'Boolean'
            }
          },
          ignoreConflictingHostNames: {
            required: false,
            serializedName: 'properties.ignoreConflictingHostNames',
            type: {
              name: 'Boolean'
            }
          }
        }
      }
    };
  }
}

module.exports = SnapshotRecoveryRequest;
