/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * A certificate that can be installed on compute nodes and can be used to
 * authenticate operations on the machine.
 *
 */
class Certificate {
  /**
   * Create a Certificate.
   * @member {string} [thumbprint] The X.509 thumbprint of the certificate.
   * This is a sequence of up to 40 hex digits.
   * @member {string} [thumbprintAlgorithm] The algorithm used to derive the
   * thumbprint.
   * @member {string} [url] The URL of the certificate.
   * @member {string} [state] The current state of the certificate. Possible
   * values include: 'active', 'deleting', 'deleteFailed'
   * @member {date} [stateTransitionTime] The time at which the certificate
   * entered its current state.
   * @member {string} [previousState] The previous state of the certificate.
   * This property is not set if the certificate is in its initial active
   * state. Possible values include: 'active', 'deleting', 'deleteFailed'
   * @member {date} [previousStateTransitionTime] The time at which the
   * certificate entered its previous state. This property is not set if the
   * certificate is in its initial Active state.
   * @member {string} [publicData] The public part of the certificate as a
   * base-64 encoded .cer file.
   * @member {object} [deleteCertificateError] The error that occurred on the
   * last attempt to delete this certificate. This property is set only if the
   * certificate is in the DeleteFailed state.
   * @member {string} [deleteCertificateError.code]
   * @member {string} [deleteCertificateError.message]
   * @member {array} [deleteCertificateError.values] This list includes details
   * such as the active pools and nodes referencing this certificate. However,
   * if a large number of resources reference the certificate, the list
   * contains only about the first hundred.
   */
  constructor() {
  }

  /**
   * Defines the metadata of Certificate
   *
   * @returns {object} metadata of Certificate
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Certificate',
      type: {
        name: 'Composite',
        className: 'Certificate',
        modelProperties: {
          thumbprint: {
            required: false,
            serializedName: 'thumbprint',
            type: {
              name: 'String'
            }
          },
          thumbprintAlgorithm: {
            required: false,
            serializedName: 'thumbprintAlgorithm',
            type: {
              name: 'String'
            }
          },
          url: {
            required: false,
            serializedName: 'url',
            type: {
              name: 'String'
            }
          },
          state: {
            required: false,
            serializedName: 'state',
            type: {
              name: 'Enum',
              allowedValues: [ 'active', 'deleting', 'deleteFailed' ]
            }
          },
          stateTransitionTime: {
            required: false,
            serializedName: 'stateTransitionTime',
            type: {
              name: 'DateTime'
            }
          },
          previousState: {
            required: false,
            serializedName: 'previousState',
            type: {
              name: 'Enum',
              allowedValues: [ 'active', 'deleting', 'deleteFailed' ]
            }
          },
          previousStateTransitionTime: {
            required: false,
            serializedName: 'previousStateTransitionTime',
            type: {
              name: 'DateTime'
            }
          },
          publicData: {
            required: false,
            serializedName: 'publicData',
            type: {
              name: 'String'
            }
          },
          deleteCertificateError: {
            required: false,
            serializedName: 'deleteCertificateError',
            type: {
              name: 'Composite',
              className: 'DeleteCertificateError'
            }
          }
        }
      }
    };
  }
}

module.exports = Certificate;
