/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Describes the parameters for an application upgrade.
 *
 */
class ApplicationUpgradeProgressInfo {
  /**
   * Create a ApplicationUpgradeProgressInfo.
   * @member {string} [name]
   * @member {string} [typeName]
   * @member {string} [targetApplicationTypeVersion]
   * @member {array} [upgradeDomains]
   * @member {string} [upgradeState] Possible values include: 'Invalid',
   * 'RollingBackInProgress', 'RollingBackCompleted', 'RollingForwardPending',
   * 'RollingForwardInProgress', 'RollingForwardCompleted', 'Failed'
   * @member {string} [nextUpgradeDomain]
   * @member {string} [rollingUpgradeMode] Possible values include: 'Invalid',
   * 'UnmonitoredAuto', 'UnmonitoredManual', 'Monitored'. Default value:
   * 'UnmonitoredAuto' .
   * @member {object} [upgradeDescription]
   * @member {string} [upgradeDescription.name]
   * @member {string} [upgradeDescription.targetApplicationTypeVersion]
   * @member {array} [upgradeDescription.parameters]
   * @member {string} [upgradeDescription.upgradeKind] Possible values include:
   * 'Invalid', 'Rolling'
   * @member {string} [upgradeDescription.rollingUpgradeMode] Possible values
   * include: 'Invalid', 'UnmonitoredAuto', 'UnmonitoredManual', 'Monitored'
   * @member {number}
   * [upgradeDescription.upgradeReplicaSetCheckTimeoutInSeconds]
   * @member {boolean} [upgradeDescription.forceRestart]
   * @member {object} [upgradeDescription.monitoringPolicy]
   * @member {string} [upgradeDescription.monitoringPolicy.failureAction]
   * Possible values include: 'Invalid', 'Rollback', 'Manual'
   * @member {string}
   * [upgradeDescription.monitoringPolicy.healthCheckWaitDurationInMilliseconds]
   * @member {string}
   * [upgradeDescription.monitoringPolicy.healthCheckStableDurationInMilliseconds]
   * @member {string}
   * [upgradeDescription.monitoringPolicy.healthCheckRetryTimeoutInMilliseconds]
   * @member {string}
   * [upgradeDescription.monitoringPolicy.upgradeTimeoutInMilliseconds]
   * @member {string}
   * [upgradeDescription.monitoringPolicy.upgradeDomainTimeoutInMilliseconds]
   * @member {object} [upgradeDescription.applicationHealthPolicy]
   * @member {boolean}
   * [upgradeDescription.applicationHealthPolicy.considerWarningAsError]
   * Indicates whether warnings are treated with the same severity as errors.
   * @member {number}
   * [upgradeDescription.applicationHealthPolicy.maxPercentUnhealthyDeployedApplications]
   * The maximum allowed percentage of unhealthy deployed applications. Allowed
   * values are Byte values from zero to 100.
   * The percentage represents the maximum tolerated percentage of deployed
   * applications that can be unhealthy before the application is considered in
   * error.
   * This is calculated by dividing the number of unhealthy deployed
   * applications over the number of nodes where the application is currently
   * deployed on in the cluster.
   * The computation rounds up to tolerate one failure on small numbers of
   * nodes. Default percentage is zero.
   * @member {object}
   * [upgradeDescription.applicationHealthPolicy.defaultServiceTypeHealthPolicy]
   * @member {number}
   * [upgradeDescription.applicationHealthPolicy.defaultServiceTypeHealthPolicy.maxPercentUnhealthyPartitionsPerService]
   * The maximum allowed percentage of unhealthy partitions per service.
   * Allowed values are Byte values from zero to 100
   *
   * The percentage represents the maximum tolerated percentage of partitions
   * that can be unhealthy before the service is considered in error.
   * If the percentage is respected but there is at least one unhealthy
   * partition, the health is evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy
   * partitions over the total number of partitions in the service.
   * The computation rounds up to tolerate one failure on small numbers of
   * partitions. Default percentage is zero.
   * @member {number}
   * [upgradeDescription.applicationHealthPolicy.defaultServiceTypeHealthPolicy.maxPercentUnhealthyReplicasPerPartition]
   * The maximum allowed percentage of unhealthy replicas per partition.
   * Allowed values are Byte values from zero to 100.
   *
   * The percentage represents the maximum tolerated percentage of replicas
   * that can be unhealthy before the partition is considered in error.
   * If the percentage is respected but there is at least one unhealthy
   * replica, the health is evaluated as Warning.
   * The percentage is calculated by dividing the number of unhealthy replicas
   * over the total number of replicas in the partition.
   * The computation rounds up to tolerate one failure on small numbers of
   * replicas. Default percentage is zero.
   * @member {number}
   * [upgradeDescription.applicationHealthPolicy.defaultServiceTypeHealthPolicy.maxPercentUnhealthyServices]
   * The maximum maximum allowed percentage of unhealthy services. Allowed
   * values are Byte values from zero to 100.
   *
   * The percentage represents the maximum tolerated percentage of services
   * that can be unhealthy before the application is considered in error.
   * If the percentage is respected but there is at least one unhealthy
   * service, the health is evaluated as Warning.
   * This is calculated by dividing the number of unhealthy services of the
   * specific service type over the total number of services of the specific
   * service type.
   * The computation rounds up to tolerate one failure on small numbers of
   * services. Default percentage is zero.
   * @member {array}
   * [upgradeDescription.applicationHealthPolicy.serviceTypeHealthPolicyMap]
   * @member {string} [upgradeDurationInMilliseconds] The estimated total
   * amount of time spent processing the overall upgrade.
   * @member {string} [upgradeDomainDurationInMilliseconds] The estimated total
   * amount of time spent processing the current upgrade domain.
   * @member {array} [unhealthyEvaluations]
   * @member {object} [currentUpgradeDomainProgress]
   * @member {string} [currentUpgradeDomainProgress.domainName]
   * @member {array} [currentUpgradeDomainProgress.nodeUpgradeProgressList]
   * @member {string} [startTimestampUtc] The estimated UTC datetime when the
   * upgrade started.
   * @member {string} [failureTimestampUtc] The estimated UTC datetime when the
   * upgrade failed and FailureAction was executed.
   * @member {string} [failureReason] Possible values include: 'None',
   * 'Interrupted', 'HealthCheck', 'UpgradeDomainTimeout', 'UpgradeTimeout'
   * @member {object} [upgradeDomainProgressAtFailure]
   * @member {string} [upgradeDomainProgressAtFailure.domainName]
   * @member {array} [upgradeDomainProgressAtFailure.nodeUpgradeProgressList]
   * @member {string} [upgradeStatusDetails] Additional detailed information
   * about the status of the pending upgrade.
   */
  constructor() {
  }

  /**
   * Defines the metadata of ApplicationUpgradeProgressInfo
   *
   * @returns {object} metadata of ApplicationUpgradeProgressInfo
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'ApplicationUpgradeProgressInfo',
      type: {
        name: 'Composite',
        className: 'ApplicationUpgradeProgressInfo',
        modelProperties: {
          name: {
            required: false,
            serializedName: 'Name',
            type: {
              name: 'String'
            }
          },
          typeName: {
            required: false,
            serializedName: 'TypeName',
            type: {
              name: 'String'
            }
          },
          targetApplicationTypeVersion: {
            required: false,
            serializedName: 'TargetApplicationTypeVersion',
            type: {
              name: 'String'
            }
          },
          upgradeDomains: {
            required: false,
            serializedName: 'UpgradeDomains',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'UpgradeDomainInfoElementType',
                  type: {
                    name: 'Composite',
                    className: 'UpgradeDomainInfo'
                  }
              }
            }
          },
          upgradeState: {
            required: false,
            serializedName: 'UpgradeState',
            type: {
              name: 'String'
            }
          },
          nextUpgradeDomain: {
            required: false,
            serializedName: 'NextUpgradeDomain',
            type: {
              name: 'String'
            }
          },
          rollingUpgradeMode: {
            required: false,
            serializedName: 'RollingUpgradeMode',
            defaultValue: 'UnmonitoredAuto',
            type: {
              name: 'String'
            }
          },
          upgradeDescription: {
            required: false,
            serializedName: 'UpgradeDescription',
            type: {
              name: 'Composite',
              className: 'ApplicationUpgradeDescription'
            }
          },
          upgradeDurationInMilliseconds: {
            required: false,
            serializedName: 'UpgradeDurationInMilliseconds',
            type: {
              name: 'String'
            }
          },
          upgradeDomainDurationInMilliseconds: {
            required: false,
            serializedName: 'UpgradeDomainDurationInMilliseconds',
            type: {
              name: 'String'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          },
          currentUpgradeDomainProgress: {
            required: false,
            serializedName: 'CurrentUpgradeDomainProgress',
            type: {
              name: 'Composite',
              className: 'CurrentUpgradeDomainProgressInfo'
            }
          },
          startTimestampUtc: {
            required: false,
            serializedName: 'StartTimestampUtc',
            type: {
              name: 'String'
            }
          },
          failureTimestampUtc: {
            required: false,
            serializedName: 'FailureTimestampUtc',
            type: {
              name: 'String'
            }
          },
          failureReason: {
            required: false,
            serializedName: 'FailureReason',
            type: {
              name: 'String'
            }
          },
          upgradeDomainProgressAtFailure: {
            required: false,
            serializedName: 'UpgradeDomainProgressAtFailure',
            type: {
              name: 'Composite',
              className: 'FailureUpgradeDomainProgressInfo'
            }
          },
          upgradeStatusDetails: {
            required: false,
            serializedName: 'UpgradeStatusDetails',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = ApplicationUpgradeProgressInfo;
