/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Information about setup or main entry point of a code package deployed on a
 * Service Fabric node.
 *
 */
class CodePackageEntryPoint {
  /**
   * Create a CodePackageEntryPoint.
   * @member {string} [entryPointLocation] The location of entry point
   * executable on the node.
   * @member {string} [processId] The process id of the entry point.
   * @member {string} [runAsUserName] The user name under which entry point
   * executable is run on the node.
   * @member {object} [codePackageEntryPointStatistics]
   * @member {string} [codePackageEntryPointStatistics.lastExitCode] The last
   * exit code of the entry point.
   * @member {date} [codePackageEntryPointStatistics.lastActivationTime] The
   * last time (in UTC) when Service Fabric attempted to run the entry point.
   * @member {date} [codePackageEntryPointStatistics.lastExitTime] The last
   * time (in UTC) when the entry point finished running.
   * @member {date}
   * [codePackageEntryPointStatistics.lastSuccessfulActivationTime] The last
   * time (in UTC) when the entry point ran successfully.
   * @member {date} [codePackageEntryPointStatistics.lastSuccessfulExitTime]
   * The last time (in UTC) when the entry point finished running gracefully.
   * @member {string} [codePackageEntryPointStatistics.activationCount] Number
   * of times the entry point has run.
   * @member {string} [codePackageEntryPointStatistics.activationFailureCount]
   * Number of times the entry point failed to run.
   * @member {string}
   * [codePackageEntryPointStatistics.continuousActivationFailureCount] Number
   * of times the entry point continuously failed to run.
   * @member {string} [codePackageEntryPointStatistics.exitCount] Number of
   * times the entry point finished running.
   * @member {string} [codePackageEntryPointStatistics.exitFailureCount] Number
   * of times the entry point failed to exit gracefully.
   * @member {string}
   * [codePackageEntryPointStatistics.continuousExitFailureCount] Number of
   * times the entry point continuously failed to exit gracefully.
   * @member {string} [status] Possible values include: 'Invalid', 'Pending',
   * 'Starting', 'Started', 'Stopping', 'Stopped'
   * @member {date} [nextActivationTime] The time (in UTC) when the entry point
   * executable will be run next.
   * @member {string} [instanceId]
   */
  constructor() {
  }

  /**
   * Defines the metadata of CodePackageEntryPoint
   *
   * @returns {object} metadata of CodePackageEntryPoint
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'CodePackageEntryPoint',
      type: {
        name: 'Composite',
        className: 'CodePackageEntryPoint',
        modelProperties: {
          entryPointLocation: {
            required: false,
            serializedName: 'EntryPointLocation',
            type: {
              name: 'String'
            }
          },
          processId: {
            required: false,
            serializedName: 'ProcessId',
            type: {
              name: 'String'
            }
          },
          runAsUserName: {
            required: false,
            serializedName: 'RunAsUserName',
            type: {
              name: 'String'
            }
          },
          codePackageEntryPointStatistics: {
            required: false,
            serializedName: 'CodePackageEntryPointStatistics',
            type: {
              name: 'Composite',
              className: 'CodePackageEntryPointStatistics'
            }
          },
          status: {
            required: false,
            serializedName: 'Status',
            type: {
              name: 'String'
            }
          },
          nextActivationTime: {
            required: false,
            serializedName: 'NextActivationTime',
            type: {
              name: 'DateTime'
            }
          },
          instanceId: {
            required: false,
            serializedName: 'InstanceId',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = CodePackageEntryPoint;
