/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for the partitions of a service, containing
 * health evaluations for each unhealthy partition that impacts current
 * aggregated health state. Can be returned when evaluating service health and
 * the aggregated health state is either Error or Warning.
 *
 * @extends models['HealthEvaluation']
 */
class PartitionsHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a PartitionsHealthEvaluation.
   * @member {number} [maxPercentUnhealthyPartitionsPerService] Maximum allowed
   * percentage of unhealthy partitions per service from the
   * ServiceTypeHealthPolicy.
   * @member {number} [totalCount] Total number of partitions of the service
   * from the health store.
   * @member {array} [unhealthyEvaluations]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of PartitionsHealthEvaluation
   *
   * @returns {object} metadata of PartitionsHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'Partitions',
      type: {
        name: 'Composite',
        className: 'PartitionsHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            type: {
              name: 'String'
            }
          },
          maxPercentUnhealthyPartitionsPerService: {
            required: false,
            serializedName: 'MaxPercentUnhealthyPartitionsPerService',
            type: {
              name: 'Number'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = PartitionsHealthEvaluation;
