/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const models = require('./index');

/**
 * Represents health evaluation for delta unhealthy cluster nodes in an upgrade
 * domain, containing health evaluations for each unhealthy node that impacted
 * current aggregated health state.
 * Can be returned during cluster upgrade when cluster aggregated health state
 * is Warning or Error.
 *
 *
 * @extends models['HealthEvaluation']
 */
class UpgradeDomainDeltaNodesCheckHealthEvaluation extends models['HealthEvaluation'] {
  /**
   * Create a UpgradeDomainDeltaNodesCheckHealthEvaluation.
   * @member {string} [upgradeDomainName] Name of the upgrade domain where
   * nodes health is currently evaluated.
   * @member {number} [baselineErrorCount] Number of upgrade domain nodes with
   * aggregated heath state Error in the health store at the beginning of the
   * cluster upgrade.
   * @member {number} [baselineTotalCount] Total number of upgrade domain nodes
   * in the health store at the beginning of the cluster upgrade.
   * @member {number} [maxPercentDeltaUnhealthyNodes] Maximum allowed
   * percentage of upgrade domain delta unhealthy nodes from the
   * ClusterUpgradeHealthPolicy.
   * @member {number} [totalCount] Total number of upgrade domain nodes in the
   * health store.
   * @member {array} [unhealthyEvaluations]
   */
  constructor() {
    super();
  }

  /**
   * Defines the metadata of UpgradeDomainDeltaNodesCheckHealthEvaluation
   *
   * @returns {object} metadata of UpgradeDomainDeltaNodesCheckHealthEvaluation
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'UpgradeDomainDeltaNodesCheck',
      type: {
        name: 'Composite',
        className: 'UpgradeDomainDeltaNodesCheckHealthEvaluation',
        modelProperties: {
          aggregatedHealthState: {
            required: false,
            serializedName: 'AggregatedHealthState',
            type: {
              name: 'String'
            }
          },
          description: {
            required: false,
            serializedName: 'Description',
            type: {
              name: 'String'
            }
          },
          kind: {
            required: true,
            serializedName: 'Kind',
            type: {
              name: 'String'
            }
          },
          upgradeDomainName: {
            required: false,
            serializedName: 'UpgradeDomainName',
            type: {
              name: 'String'
            }
          },
          baselineErrorCount: {
            required: false,
            serializedName: 'BaselineErrorCount',
            type: {
              name: 'Number'
            }
          },
          baselineTotalCount: {
            required: false,
            serializedName: 'BaselineTotalCount',
            type: {
              name: 'Number'
            }
          },
          maxPercentDeltaUnhealthyNodes: {
            required: false,
            serializedName: 'MaxPercentDeltaUnhealthyNodes',
            type: {
              name: 'Number'
            }
          },
          totalCount: {
            required: false,
            serializedName: 'TotalCount',
            type: {
              name: 'Number'
            }
          },
          unhealthyEvaluations: {
            required: false,
            serializedName: 'UnhealthyEvaluations',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'HealthEvaluationWrapperElementType',
                  type: {
                    name: 'Composite',
                    className: 'HealthEvaluationWrapper'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = UpgradeDomainDeltaNodesCheckHealthEvaluation;
